<?php

$stdout = fopen('php://stdout','w');
$stderr = fopen('php://stderr', 'w');

$tmp_path = '/tmp';

if (!can_access_file($tmp_path) || !is_dir($tmp_path)) {
	fprintf($stderr, "Couldn't access tmp dir: %s", $tmp_path);
	exit(1);
}

checkInit();

$svnroot = GFConf::get('scmsvn.localSVNRoot', '/svnroot');
$svn_path = GFConf::get('scmsvn.svnpath', '/usr/bin');

if (!can_access_file($svnroot) || !is_dir($svnroot)) {
	fprintf($stderr, "The configured value for scmsvn.localSVNRoot (%s) doesn't exist or is not readable!\n", $svnroot);
	exit(1);
}

if (!can_access_file($svn_path) || !is_dir($svn_path)) {
	fprintf($stderr, "The configured value for scmsvn.svnpath (%s) doesn't exist or is not readable!\n", $svn_path);
	exit(1);
}

//Create temporary dir
$svn_dump_dest = $tmp_path.'/gforge-repository-export';
if (!file_exists($svn_dump_dest)) {
	system('mkdir '.$svn_dump_dest, $ret);
	if ($ret) {
		fprintf($stderr, "Couldn't create destination directory at %s, aborting\n", $svn_dump_dest);
		exit(1);
	}
} else {
	fprintf($stderr, "Destination directory %s already exists, aborting\n", $svn_dump_dest);
	exit(1);
}

if ($argc >= 2) {
	//Check that all repositories are valid

	for ($i = 1; $i < $argc; $i++) {
		$repo_name = basename($argv[$i]);
		$repo_path = $svnroot.'/'. $repo_name;
		if (!can_access_file($repo_path)) {
			fprintf($stderr, "Can't access svn repository at %s, ignoring parameter\n", $svn_path);
		} else {
			$dest_path = $svn_dump_dest.'/'.$repo_name;
			export_svn($repo_path, $dest_path,$svn_path);
		}
	}

} else if ($argc == 1) {
	//Process all
	$repositories = scandir($svnroot);

	foreach($repositories as $repo) {
		if (preg_match('/^\..*/', $repo)) {
			//Ignore hidden folders
			continue;
		}
		$repo_name = basename($repo);
		$repo_path = $svnroot.'/'. $repo_name;
		$dest_path = $svn_dump_dest.'/'.$repo_name;
		if (!can_access_file($repo_path)) {
			fprintf($stderr, "Can't access svn repository at %s, ignoring\n", $svn_path);
		} else {
			fprintf($stdout, "\nProcessing %s \n", $repo);
			export_svn($repo_path, $dest_path,$svn_path);
		}
	}

} else {
	//Invalid usage
}

fprintf($stdout,"Repositories processed, you just need to tar/zip %s
and copy the tarball to the new destination\n", $svn_dump_dest);


function can_access_file($full_path) {
	return file_exists($full_path) && is_readable($full_path);
}

function export_svn($repo_path, $dest_path, $svn_path) {
	$cmd = $svn_path.'/svnadmin dump '. $repo_path .' > '.$dest_path;
	system($cmd, $ret_val);
	return $ret_val;
}

function checkInit() {
	require_once('initialize.php');
	require_once('GFConf.php');
}
