<?php
/**
 *
 * Copyright (c) 2005-2014 GForge, LLC
 *
 * @version
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 * @author Javier Adrian Ortiz de la Tabla <atabla@gforgegroup.com>
 *
 */

require_once('system/Cronjob.php');

class RobotsFileGenerator extends Cronjob {
	public function __construct() {
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}

	/**
	 * @return array
	 */
	private function getScmUrls() {
		$sql = "SELECT '/gf/project/' || unix_name ||'/'||plugin_name AS disallow FROM project_plugin, project,
		plugin WHERE project_plugin.project_id=project.project_id AND plugin.plugin_id=project_plugin.plugin_id AND plugin_name IN
		('scmcvs','scmsvn','scmgit')" ;
		$con = Propel::getConnection();
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		$disallow_lines = array();
		while ($rs->next()) {
			$disallow_lines[] = $rs->getString(1);
		}

		return $disallow_lines;
	}

	/**
	 * @return string
	 */
	private function buildRobotsLines() {
		$lines = array();

		$globalUrls = array('/gf/account','/gf/ajax','/gf/chat','/gf/download','/gf/my','/gf/stats','/gf/upload','/gf/user','/gf/admin');
		$scmUrls = $this->getScmUrls();

		$lines[]="User-agent: *";
		$lines[]="Crawl-delay: 1500";
		foreach($globalUrls as $url) {
			$lines[]="Disallow: $url";
		}
		foreach($scmUrls as $url) {
			$lines[]="Disallow: $url";
		}

		return implode("\n",$lines);
	}

	public function run() {
		$filePath = GFConf::get('system.libDir').DIRECTORY_SEPARATOR.'www'.DIRECTORY_SEPARATOR.'robots.txt';
		$content = $this->buildRobotsLines();
		$this->logCron('Writing robots.txt...');

		try {
			$file = fopen($filePath, 'a');
			ftruncate($file, 0);
			rewind($file);
			if(!empty($content)) {
				fwrite($file, $content);
			}
			flock($file, LOCK_UN);
			fclose($file);
			$this->logCron('Writing finished successfully');
		} catch (Exception $e) {
			$this->logCron('Writing robots.txt failed, error message was:'.$e->getMessage());
		}
		$this->commitLog();
	}
}

$robots = new RobotsFileGenerator();
$robots->run();

