<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version
 * @author Tim Perdue tim@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

/**
 * Stats
 */

require_once("system/Cronjob.php");
require_once('core/ActivityLog.php');
require_once('core/Filesystem.php');
require_once('core/FilesystemPeer.php');
require_once("SystemUtils.php");

class FilesystemIndexer extends Cronjob {
	const LOCK_FILE = '/tmp/gforge_index_cronjob.LCK';
	
	const TMP_XMLDOC_DIR = '/tmp/_gforge_xmldoc_index';
	
	public function __construct() {
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}

	public function run() {
		// This lock check prevents that two processes run at the same time
		$lock_handler = fopen(self::LOCK_FILE, "w+");
		if (!$lock_handler) {
			// Should not happen if we're root... we should be able to open the
			// file even if it's locked
			$this->logCron('Couldn\'t open lock file '.self::LOCK_FILE.' for writing');
			$this->commitLog();
			return;
		}
		
		if (!flock($lock_handler, LOCK_EX | LOCK_NB)) {
			// Couldn't acquire exclusive lock, means another process is running
			$this->logCron('Cronjob is already running (found exclusive lock on '.self::LOCK_FILE.'). Skipping...');
			$this->commitLog();
			return;
		}
		
		
//plain text
		Propel::getConnection()->createStatement()->executeUpdate("update filesystem set file_type='application/msword' where file_name like '%.doc' and file_type != 'application/msword'");
		Propel::getConnection()->createStatement()->executeUpdate("update filesystem set file_type='application/pdf' where file_name like '%.pdf' and file_type != 'application/pdf'");
		$crit = new Criteria();
		$crit->add(FilesystemPeer::FILE_TYPE,array('text','text/plain'),Criteria::IN);
		$crit->add(FilesystemPeer::STRINGS,'');
        /** @var Filesystem[] $fs */
		$fs = FilesystemPeer::doSelect($crit);
		foreach ($fs as $f) {
			if (file_exists($f->getFilePath())) {
				$strings = file_get_contents($f->getFilePath());
				$f->setStrings($strings);
                $this->save($f);
			} else {
				$this->logCron($f->getFilePath()." Doesn't exist on filesystem");
			}
		}

//html docs
		$crit = new Criteria();
		$crit->add(FilesystemPeer::FILE_TYPE,'text/html');
		$crit->add(FilesystemPeer::STRINGS,'');
		$fs = FilesystemPeer::doSelect($crit);
		foreach ($fs as $f) {
			if (file_exists($f->getFilePath())) {
				$strings=file_get_contents($f->getFilePath());
				$f->setStrings($strings);
				$this->save($f);
			} else {
				$this->logCron("\n".$f->getFilePath()." Doesn't exist on filesystem");
			}
		}

//msword docs
		if (file_exists(GFConf::get('system.wvTextPath').'wvText')) {
			$crit = new Criteria();
			$crit->add(FilesystemPeer::FILE_TYPE,'application/msword');
			$crit->add(FilesystemPeer::STRINGS,'');
			$fs = FilesystemPeer::doSelect($crit);
			foreach ($fs as $f) {
                $tmp_indexer = '/tmp/indexer.tmp';
                if (file_exists($tmp_indexer)) {
				    exec('rm -f '.$tmp_indexer);
                }
				if (file_exists($f->getFilePath())) {
					$output = array();
					$cmd = escapeshellcmd(GFConf::get('system.wvTextPath')."wvText ".escapeshellarg($f->getFilePath())." /tmp/indexer.tmp");
					$error = SystemUtils::execute($cmd, $output);
					
					if (!file_exists('/tmp/indexer.tmp') || $error) {
						// For some reason, we couldn't parse the file
						$strings = "-1";		// Mark as error
                        $this->logCron($f->getFilePath()."Couldn't be parsed, output was :" .implode(PHP_EOL, $output));
					} else { 
						$strings=file_get_contents("/tmp/indexer.tmp");
					}
					$f->setStrings($strings);
					$this->save($f);
				} else {
					$this->logCron($f->getFilePath()." Doesn't exist on filesystem");
				}
			}
		}
// OOXML and ODF docs
		$crit = new Criteria();

		// Word 2007 OOXML files
		$cton1 = $crit->getNewCriterion(FilesystemPeer::FILE_TYPE, 'application/vnd.openxmlformats-officedocument.%', Criteria::ILIKE);
		// OASIS ODF
		$cton2 = $crit->getNewCriterion(FilesystemPeer::FILE_TYPE, 'application/vnd.oasis.opendocument.%', Criteria::ILIKE);

		$cton1->addOr($cton2);
		$crit->add($cton1);
		$crit->add(FilesystemPeer::STRINGS,'');
		
		$fs = FilesystemPeer::doSelect($crit);
		foreach ($fs as $f) {
			if (file_exists($f->getFilePath())) {
				$strings = $this->_processXMLDoc($f->getFilePath());
				if (!$strings) {
					$strings = "-1";	// Mark as error
				}
				$f->setStrings($strings);
				$this->save($f);
			} else {
				$this->logCron($f->getFilePath()." Doesn't exist on filesystem");
			}
		}

//pdf docs
		if (file_exists(GFConf::get('system.pdftotextPath').'pdftotext')) {
			$crit = new Criteria();
			$crit->add(FilesystemPeer::FILE_TYPE,'application/pdf');
			$crit->add(FilesystemPeer::STRINGS,'');
			$fs = FilesystemPeer::doSelect($crit);
			foreach ($fs as $f) {
                if (file_exists($tmp_indexer)) {
                    exec('rm -f '.$tmp_indexer);
                }
				if (file_exists($f->getFilePath())) {
					$output = array();
					$cmd = escapeshellcmd(GFConf::get('system.pdftotextPath')."pdftotext ".escapeshellarg($f->getFilePath())." /tmp/indexer.tmp");
					$error = SystemUtils::execute($cmd, $output);
					if (!file_exists('/tmp/indexer.tmp') || $error) {
						// For some reason, we couldn't parse the file
                        $this->logCron($f->getFilePath()."Couldn't be parsed, output was :" .implode(PHP_EOL, $output));
						$strings = '-1';
					} else{
						$strings = file_get_contents("/tmp/indexer.tmp");
					}
					$f->setStrings($strings);
					$this->save($f);
				} else {
					$this->logCron($f->getFilePath()." Doesn't exist on filesystem");
				}
			}
		}

        $this->commitLog();
		
		// Unlock the file
		flock($lock_handler, LOCK_UN);
		fclose($lock_handler);
	}
	
	// Special method to process XML Doc files
	function _processXMLDoc($file) {
    if (filesize($file) == false) {
      return '';
    }

		if (SystemUtils::isRootDir(self::TMP_XMLDOC_DIR)) {
			throw new Exception("Filesystem index: pointing at root as tmp dir");
		}
		
		$output = array();
		if (is_dir(self::TMP_XMLDOC_DIR)) {
			SystemUtils::execute('rm -rf '.self::TMP_XMLDOC_DIR, $output);
		}

		$cmd = 'unzip '.escapeshellarg($file).' -d '.self::TMP_XMLDOC_DIR;
		$error = SystemUtils::execute($cmd, $output);
		if ($error) {
			return '';
		}
		
		// OOXML and ODF files are a .zip with a bunch of .xml files inside, so we must extract 
		// the strings from all of them
		$strings = '';

		$xmlFiles = array();
		SystemUtils::execute('find '.self::TMP_XMLDOC_DIR.' -name "*.xml"', $xmlFiles);
		foreach ($xmlFiles as $file) {
			// Hopefully we won't run out of memory?
			$contents = file_get_contents($file);
			if (!$contents) {
				continue;
			}

			$contents = str_replace(">", "> ", $contents);
			$contents = trim(strip_tags($contents));
			$contents = ereg_replace("[\t\n\r]+", "", $contents);
			$file_strings = trim(preg_replace("/\s+/", " ", $contents));

			if (!$file_strings) {
				continue;
			}
			$strings .= ' '.$file_strings;
		}
		
		// Purge the temp dir
		SystemUtils::execute('rm -rf '.self::TMP_XMLDOC_DIR, $output);
		
		return $strings;
	}

    /**
     * @param Filesystem $f
     */
    private function save($f) {
        try {
            $con = Propel::getConnection();
            $f->save($con, false );
        } catch (PropelException $e) {
            $message = "\n An error ocurred trying to parse " . $f->getFilePath() . ": " .
                $e->getMessage() . "\n";
            $this->logCron($message);
        }
    }
}

$cronjob = new FilesystemIndexer();
$cronjob->run();
?>
