<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

/**
 * 
 *
 * 
 */
require_once("system/Cronjob.php");
require_once('core/CronHistory.php');
require_once('core/ProjectPluginPeer.php');
require_once('core/PluginPeer.php');
require_once('plugin/ActionManager.php');

class CreateAliasCronjob extends Cronjob {
	
	const GFORGE_BEGIN_MARKER = '### GFORGEBEGIN';
	const GFORGE_END_MARKER = '### GFORGEEND';
	
	public function __construct() {
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}

	public function run() {		
		$alias_file = "/etc/aliases";
		
		$aliases_old = file($alias_file);
		if ($aliases_old === false) {
			$this->logCron("Unable to open $alias_file");
			$this->commitLog();
			return;
		}
		
		$aliases_new = array();
		
		$n = count($aliases_old);
		for ($i = 0; $i < $n; $i++) {
			$line = $aliases_old[$i];
			if (preg_match('/^'.self::GFORGE_BEGIN_MARKER.'/', $line)) {	// Found marker. Must write GForge aliases
				// Increase $i until we find the end of the markers
				while ($i < $n) {
					$i++;
					$line = $aliases_old[$i];
					if (preg_match('/^'.self::GFORGE_END_MARKER.'/', $line)) {
						break;
					}
				}
				
				// In this point, $i points to the line that has the end marker
			} else {
				$aliases_new[] = $line;
			}
		}
		
		
		// Write GForge aliases at the end of the file
		$aliases_new[] = self::GFORGE_BEGIN_MARKER."\n";
		
		// First, add the gateway alias
		$php_command = 'php -d include_path='.get_include_path();
		//Deprecated
		//$gateway_script = SystemUtils::buildFilePath(GFConf::get('system.libDir'), '/bin/mail_gateway.php');
		$forum_gateway = SystemUtils::buildFilePath(GFConf::get('system.libDir'), '/plugins/forum/bin/mail_gateway.php');
		$tracker_item_gateway = SystemUtils::buildFilePath(GFConf::get('system.libDir'), '/plugins/tracker/bin/tracker_item_gateway.php');
		//Deprecated
		//$aliases_new[] = 'gforge-gateway:	"|'.$php_command.' '.$gateway_script.'"'."\n";
		//Forum gateway alias
		$aliases_new[] = 'discuss:	"|'.$php_command.' '.$forum_gateway.'"'."\n";
		//Tracker item gateway
		$aliases_new[] = 'trackeritem: "|'.$php_command.' '.$tracker_item_gateway.'"'."\n";
		$aliases_new[] = "noreply:	/dev/null\n";  

		//get needed aliases
		$resultset = ActionManager::notify('GET_MAIL_ALIASES');		
			
		//add aliases. 
		foreach ($resultset as $result) { 
	   		$req_lists = $result->getResult();	   		
   			foreach ($req_lists as $list) { 
   				if (!isset($list['name'])) continue;
   				$name = $list['name'];
   				if (isset($list['command'])) {
   					// this address is an alias for a command
   					$aliases_new[] =  $name.':			"|'.$list['command'].'"'."\n";
   				} elseif (isset($list['alias'])) {
   					// this address is an alias for another address
   					$aliases_new[] =  $name.':	'.$list['alias']."\n";
   				}
	       	}
		}
		
		$aliases_new[] = self::GFORGE_END_MARKER."\n";

		file_put_contents($alias_file, implode('', $aliases_new));

		// Run the "newaliases" command to inform mailman to process the new aliases
		$output = array();
		$failed = SystemUtils::execute('newaliases', $output);
		if ($failed) {
			$this->logCron("Couldn't run 'newaliases' command: \n".implode('\n', $output));
		}
		
		$this->commitLog();
	}
}
$cronjob = new CreateAliasCronjob();
$cronjob->run();
?>
