<?php
require_once('system/Cronjob.php');
require_once('SystemUtils.php');
require_once('plugins/scm/lib/SCMRepositoryFactory.php');
require_once('lib/SystemUtils.php');

class IndexSCM extends Cronjob {
	private $lastClass = null;
	private $lastLineNo = 0;
	private $checkoutPath = '';
	private $index;
	private $repository;
	private $code_search_module_id;
	private $db;
	
	public function run() {
		$this->db = Propel::getConnection();
		
		////
		/*$project = ProjectPeer::retrieveByPk(4);
		$this->repository = SCMRepositoryFactory::getRepository($project);
		$this->index = array();
		$this->checkoutPath = '/tmp/gf_scmco_ENzIuq';
		$this->code_search_module_id = 1;
		$this->db->executeQuery('DELETE FROM code_search_index WHERE code_search_module_id=1');
		$this->parseSource($this->checkoutPath);*/
		////
		
		//Verify if doxygen is available
		$doxygen_bin = SystemUtils::buildFilePath(GFConf::get('system.doxygenPath'), 'doxygen');
		if (!file_exists($doxygen_bin)) {
			$this->logCron("Doxygen is not available, aborting");
			$this->commitLog();
			return;
		}

		$rs = $this->db->prepareStatement('SELECT * FROM code_search_module')->executeQuery();
		
		while ($rs->next()) {
			$this->code_search_module_id = $rs->getInt('code_search_module_id');
			$project = ProjectPeer::retrieveByPk($rs->getInt('project_id'));
			$this->repository = SCMRepositoryFactory::getRepository($project);
			
			if (!$this->repository) {
				continue;
			}
			
			// Empty the old index.
			// NOTE: we will have to remove this in the future if we want to search in more than one revision
			$stmt = $this->db->prepareStatement('DELETE FROM code_search_index WHERE code_search_module_id=?');
			$stmt->setInt(1, $this->code_search_module_id);
			$stmt->executeQuery();

			// Create a temporary dir for the checkout
			$destdir = $this->createTempDir();
			if (!is_dir($destdir) || SystemUtils::isRootDir($destdir)) {
				continue;
			}
			
			// Do the actual checkout of the code
			try {
				$this->logCron('Performing local checkout of project '.$project->getUnixName().', module '.$rs->getSTring('module').', in '.$destdir.'...');
				$this->checkoutPath = $destdir;
				$this->repository->doLocalCheckout($destdir, $rs->getString('module'));
			} catch (Exception $ex) {
				$this->logCron('Error while parsing module '.$rs->getString('module').' in project '.$project->getUnixName().': '.$ex->getMessage());
				continue;
			}
			
			$this->index = array();
			$this->parseSource($destdir);
			
			$output = array();
			SystemUtils::execute('rm -rf '.$destdir, $output);
		}
	}
	
	private function parseSource($code_path) {
		$output_path = SystemUtils::buildFilePath($code_path, 'doxygen_output');

		$config = <<<EOF
DOXYFILE_ENCODING			= UTF-8
PROJECT_NAME				= __NAME__
OUTPUT_DIRECTORY			= $output_path
CREATE_SUBDIRS				= NO
FULL_PATH_NAMES				= NO
RECURSIVE					= YES
EXTRACT_ALL					= YES
GENERATE_HTML				= NO
GENERATE_LATEX				= NO
GENERATE_RTF				= NO
GENERATE_MAN				= NO
GENERATE_XML				= YES
XML_PROGRAMLISTING			= YES
SEARCHENGINE				= NO

EOF;

		$config_file_name = 'doxygen.cfg';
		$config_file = SystemUtils::buildFilePath($code_path, $config_file_name);
		$bytes = file_put_contents($config_file, $config);

		$doxygen_bin = SystemUtils::buildFilePath(GFConf::get('system.doxygenPath'), 'doxygen');
		$cmd = $doxygen_bin.' '.$config_file_name.' 2>&1';		// Redirect stderr to stdout

		// Run doxygen against the checked out source code
		chdir($code_path);
		$output = array();
		$this->logCron('Executing doxygen...');
		$error = SystemUtils::execute($cmd, $output);
		
		// Parse each xml output file
		$this->logCron('Parsing doxygen output...');
		$path = SystemUtils::buildFilePath($output_path, '/xml');
		$dh = opendir($path);
		while ( ($filename = readdir($dh)) !== false) {
			$filename = SystemUtils::buildFilePath($path, $filename);
			if (!preg_match('/\\.xml$/i', $filename) || !is_file($filename)) {
				continue;
			}

            try {
                $this->logCron("---- Parsing $filename... -----");
                $this->parseXML($filename);
                $this->writeIndex($filename);
            } catch (\Exception $ex) {
                $this->logCron("Exception while indexing $filename: {$ex->getMessage()}");
            }
		}
		
		$this->commitLog();
	}
	
	private function parseXML($filename) {
		$doc = new DOMDocument();
		$doc->load($filename);
		$xpath = new DOMXpath($doc);
		
		$members = $xpath->query('//memberdef');		// This will get all methods, members, functions, etc
		foreach ($members as $member) {
			$this->parseMember($member, $xpath);
		}
		
		$compounds = $xpath->query("//compounddef");	// This will get the classes
		foreach ($compounds as $compound) {
			$this->parseCompound($compound, $xpath);
		}

	}
	
	private function parseMember(DOMNode $member, DOMXPath $xpath) {
		$location = $this->parseLocation($this->getSingleNode($member, 'location'));
		$definition = $this->getTextFromNode($member, 'definition');
		/*
		 * $definition can be something like this:
		 * - const Project::SECTION
		 * - Project::$permission
		 * - Project::setUnixName
		 * - int SomeClass::someVariable
		 * - int SomeClass::someMethod
		 * - void SomeClass::ChildClass::someMethod
		 * - int methodName
		 */
		
		// Remove the attribute properties (static, const, int, const int, etc)
		preg_match('/[^[:space:]]*$/', $definition, $matches);
		$definition = $matches[0];
		if (preg_match('/^(.*)::([^:]*)$/', $definition, $matches)) {			// Has a class name
			$classFullName = $matches[1];
			$classPath = explode('::', $classFullName);
			$className = $classPath[count($classPath)-1];
			$name = $matches[2];
		} else {
			$className = '';
			$classFullName = '';
			$name = $definition;
		}
		
		$comment = $this->getTextFromNode($member, 'briefdescription')."\n".$this->getTextFromNode($member, 'detaileddescription')."\n".$this->getTextFromNode($member, 'inbodydescription');
		$comment = trim($comment);
		
		$words = "$definition $name $classFullName $comment";				// Keywords to identify the element
		
		if ($member->getAttribute('kind') == 'variable') {
			$this->index[] = array('class_name' => $className, 'filename' => $location['file'], 'line' => $location['line'],
				'variable_name' => $name, 'words' => $words, 'comment' => $comment);
		} elseif ($member->getAttribute('kind') == 'function') {
			$this->index[] = array('class_name' => $className, 'filename' => $location['file'], 'line' => $location['line'],
				'method_name' => $name, 'words' => $words, 'comment' => $comment);
		}
	}
	
	private function parseCompound(DOMNode $compound, DOMXPath $xpath) {
		switch ($compound->getAttribute('kind')) {
		case 'class':
			$this->parseClass($compound, $xpath);
			break;
		}
	}
	
	private function parseClass(DOMNode $compound, DOMXPath $xpath) {
		$classFullName = $this->getTextFromNode($compound, 'compoundname');
		$classPath = explode('::', $classFullName);
		$className = $classPath[count($classPath)-1];
		
		$words = "$className $classFullName $classPath";			// Keywords to identify the class
		
		$location = $this->parseLocation($this->getSingleNode($compound, 'location'));
		$this->index[] = array('class_name' => $className, 'filename' => $location['file'], 'line' => $location['line'],
			'words' => $words);
	}
	
	private function getTextFromNode(DOMNode $parent, $nodename) {
		$node = $this->getSingleNode($parent, $nodename);
		if ($node) {
			return $node->textContent;
		} else {
			return null;
		}
	}
	
	private function getSingleNode(DOMNode $parent, $nodename) {
		$node = $parent->getElementsByTagName($nodename);
		if (!$node) return null;
		return $node->item(0);
	}
	
	private function parseLocation($location) {
		$line = $location->getAttribute('line');
		$file = $location->getAttribute('file');
		
		// Extract the checkout path from the $file
		$path = preg_quote($this->checkoutPath);
		$file = preg_replace("|^$path|", '', $file);
		$file = SystemUtils::buildFilePath('/', $file);
		return array('file' => $file, 'line' => $line);
	}
	
	
	private function createTempDir() {
		$temp_dir = (function_exists('sys_get_temp_dir') ? sys_get_temp_dir() : '/tmp');
		$name = tempnam($temp_dir, 'gf_scmco_');
		@unlink($name);
		mkdir($name);
		return $name;
	}
	
	private function writeIndex() {
		foreach ($this->index as $item) {
			$revision = $this->repository->getCheckedoutFileRevision($this->checkoutPath, $item['filename']);
			
			if (GFConf::get('system.dbtype') == 'postgres') {
				$stmt = $this->db->prepareStatement('INSERT INTO code_search_index(code_search_module_id, filename, line, revision, class_name, method_name, variable_name, comment, vectors) 
									VALUES(?,?,?,?,?,?,?,?,to_tsvector(\'default\', ?))');
				$stmt->setInt(1, $this->code_search_module_id);
				$stmt->setString(2, $item['filename']);
				$stmt->setString(3, $item['line']);
				$stmt->setString(4, $revision);
				$stmt->setString(5, isset($item['class_name']) ? $item['class_name'] : null);
				$stmt->setString(6, isset($item['method_name']) ? $item['method_name'] : null);
				$stmt->setString(7, isset($item['variable_name']) ? $item['variable_name'] : null);
				$stmt->setString(8, isset($item['comment']) ? $item['comment'] : null);
				$stmt->setString(9, $item['words']);
				$stmt->executeQuery();
			} else {
				// Oracle code here
			}
		}
		
		unset($this->index);			// Free the memory associated to the index
		$this->index = array();
	}
}

$cronjob = new IndexSCM();
$cronjob->run();

?>
