<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version
 * @author Daniel Perez danielperez.arg@gmail.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

/**
 * Notification Queue sender cronjob
 */
require_once("system/Cronjob.php");
require_once('StringUtils.php');
require_once('notification/GFNotification.php');
require_once('core/NotificationQueuePeer.php');
require_once('core/NotificationQueue.php');
require_once('core/UserPeer.php');

class SendNotificationQueue extends Cronjob {
	const LOCK_FILE = '/tmp/gforge_notification_queue.LCK';

	public function __construct() {
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}
	
	public function run() {
		// This lock check prevents that two processes try to send the same emails
		$lock_handler = fopen(self::LOCK_FILE, "w+");
		if (!$lock_handler) {
			// Should not happen if we're root... we should be able to open the
			// file even if it's locked
			$this->logCron(Language::getSessText('NotificationQueue.CouldNotOpenLock', array(self::LOCK_FILE)));
			$this->commitLog();
			return;
		}
		
		if (!flock($lock_handler, LOCK_EX | LOCK_NB)) {
			// Couldn't acquire exclusive lock, means another process is running
			$this->logCron(Language::getSessText('NotificationQueue.AlreadyRunning', array(self::LOCK_FILE)));
			$this->commitLog();
			return;
		}
		
		// get all the records from notification_queue table
		$notifications = NotificationQueuePeer::getPendingNotifications(new Criteria());
		foreach ($notifications as $notification) { // send the messages for each record
			try {
				$this->logCron('Notifying '.$notification->getUser()->getEmail());
				$notification->send();
				$notification->delete();
			} catch (Exception $ex) {
				$this->logCron($ex->__toString());
			}
		}
		
		$this->commitLog();
		
		// Unlock the file
		flock($lock_handler, LOCK_UN);
		fclose($lock_handler);
		
		
	}
}

$cronjob = new SendNotificationQueue();
$cronjob->run();
?>
