--begin;
-- **** TSEARCH2 INITIALIZATION ****

--INSERT INTO pg_ts_config VALUES('default_enUS', 'default', (SELECT setting FROM pg_settings WHERE name = 'lc_ctype'));
--INSERT INTO pg_ts_config_map SELECT 'default_enUS', tok_alias, dict_name FROM pg_ts_config_map WHERE ts_name = 'default';

CREATE TEXT SEARCH CONFIGURATION public.default ( COPY = pg_catalog.english );

CREATE TEXT SEARCH DICTIONARY default_ispell (
TEMPLATE = ispell,
DictFile = ispell_sample,
AffFile = ispell_sample,
StopWords = english
);

ALTER TEXT SEARCH CONFIGURATION public.default
ALTER MAPPING FOR asciiword WITH default_ispell, english_stem;

-- ********** Create auxiliar tables ********** 

CREATE TABLE tracker_item_idx (
	tracker_item_id integer,
	tracker_id integer,
	vectors tsvector,
	CONSTRAINT trackeritemid_fk
	FOREIGN KEY(tracker_item_id) REFERENCES tracker_item
	ON DELETE CASCADE
);

CREATE TABLE tracker_item_message_idx (
	tracker_item_message_id integer,
	tracker_item_id integer,
	vectors tsvector,
	CONSTRAINT trackeritemmessageid_fk
	FOREIGN KEY(tracker_item_message_id) REFERENCES tracker_item_message
	ON DELETE CASCADE	
);

-- CREATE TABLE doc_data_idx (
--	docid integer,
--	project_id integer,
--	vectors tsvector
--)

CREATE TABLE forum_message_idx (
	forum_message_id integer,
	forum_thread_id integer,
	vectors tsvector,
	CONSTRAINT forummessageid_fk
	FOREIGN KEY(forum_message_id) REFERENCES forum_message
	ON DELETE CASCADE	
);

--CREATE TABLE frs_file_idx (
--	frs_file_id integer,
--	frs_release_id integer,
--	vectors tsvector,
--	CONSTRAINT frsfileid_fk
--	FOREIGN KEY(frs_file_id) REFERENCES frs_file
--	ON DELETE CASCADE		
--);

CREATE TABLE frs_release_idx (
	frs_release_id integer,
	vectors tsvector,
	CONSTRAINT frsreleaseid_fk
	FOREIGN KEY(frs_release_id) REFERENCES frs_release
	ON DELETE CASCADE
);

CREATE TABLE projects_idx (
	project_id integer,
	vectors tsvector,
	CONSTRAINT projectid_fk
	FOREIGN KEY(project_id) REFERENCES project
	ON DELETE CASCADE		
);

CREATE TABLE news_idx (
	news_id integer,
	vectors tsvector,
	CONSTRAINT newsid_fk
	FOREIGN KEY(news_id) REFERENCES news
	ON DELETE CASCADE		
);

CREATE TABLE users_idx (
	user_id integer,
	vectors tsvector,
	CONSTRAINT userid_fk
	FOREIGN KEY(user_id) REFERENCES "user"
	ON DELETE CASCADE		
);

CREATE TABLE snippet_idx (
	snippet_id integer,
	vectors tsvector,
	CONSTRAINT snippetid_fk
	FOREIGN KEY(snippet_id) REFERENCES snippet
	ON DELETE CASCADE
);

-- ********** Populate with current data and create index ********** 

DELETE FROM tracker_item_idx;
INSERT INTO tracker_item_idx (tracker_item_id, tracker_id, vectors) 
SELECT tracker_item_id, tracker_id, to_tsvector('default', coalesce(details,'') ||' 
'|| coalesce(summary,'')) AS vectors
FROM tracker_item ORDER BY tracker_item_id;

CREATE INDEX tracker_item_idxFTI ON tracker_item_idx USING gist(vectors);

DELETE FROM tracker_item_message_idx;
INSERT INTO tracker_item_message_idx (tracker_item_message_id, tracker_item_id, vectors) 
SELECT tracker_item_message_id, tracker_item_id, to_tsvector('default', coalesce(body,'')) AS vectors
FROM tracker_item_message ORDER BY tracker_item_message_id;

CREATE INDEX tracker_item_message_idxFTI ON tracker_item_message_idx USING gist(vectors);

-- ********** i dont find this table (docman)
--	INSERT INTO doc_data_idx (docid, group_id, vectors)
--	SELECT docid, group_id, to_tsvector('default', coalesce(title,'') ||' 
--	'|| coalesce(description,'')) AS vectors
--	FROM doc_data ORDER BY docid;
--	
--	CREATE INDEX doc_data_idxFTI ON doc_data_idx USING gist(vectors);

DELETE FROM forum_message_idx;
INSERT INTO forum_message_idx (forum_message_id, forum_thread_id, vectors)
(SELECT fm.forum_message_id, fm.forum_thread_id, to_tsvector('default', coalesce(fm.subject,'') ||' 
'|| coalesce(fm.body,'')) AS vectors
FROM forum_message fm)
ORDER BY forum_message_id;

CREATE INDEX forum_message_idxFTI ON forum_message_idx USING gist(vectors);

--INSERT INTO frs_file_idx (frs_file_id, frs_release_id, vectors)
--SELECT frs_file_id, frs_release_id, to_tsvector('default', coalesce(file_name,'')) AS vectors
--FROM frs_file ORDER BY frs_file_id;

--CREATE INDEX frs_file_idxFTI ON frs_file_idx USING gist(vectors);

DELETE FROM frs_release_idx;
INSERT INTO frs_release_idx (frs_release_id, vectors)
SELECT frs_release_id, to_tsvector('default', coalesce(changes,'') ||' 
'|| coalesce(release_notes,'') ||' '|| coalesce(release_name,'')) AS vectors
FROM frs_release ORDER BY frs_release_id;

CREATE INDEX frs_release_idxFTI ON frs_release_idx USING gist(vectors);

DELETE FROM projects_idx;
INSERT INTO projects_idx (project_id, vectors)
SELECT project_id, to_tsvector('default', coalesce(project_name,'') ||' 
'|| coalesce(description,'') ||' '|| coalesce(unix_name,'')) AS vectors
FROM project ORDER BY project_id;

CREATE INDEX projects_idxFTI ON projects_idx USING gist(vectors);

DELETE FROM news_idx;
INSERT INTO news_idx (news_id, vectors)
SELECT news_id, to_tsvector('default', coalesce(summary,'') ||' 
'|| coalesce(details,'')) AS vectors
FROM news ORDER BY news_id;

CREATE INDEX news_idxFTI ON news_idx USING gist(vectors);

DELETE FROM users_idx;
INSERT INTO users_idx (user_id, vectors)
SELECT user_id, to_tsvector('default', coalesce(unix_name,'') ||' 
'|| coalesce(firstname,'') ||' '|| coalesce(lastname,'')) AS vectors
FROM "user" ORDER BY user_id;

CREATE INDEX users_idxFTI ON users_idx USING gist(vectors);

DELETE FROM snippet_idx;
INSERT INTO snippet_idx (snippet_id, vectors)
SELECT snippet_id, to_tsvector('default', coalesce(snippet_name,'') ||' 
'|| coalesce(snippet_description,''))  AS vectors
FROM snippet ORDER BY snippet_id;

CREATE INDEX snippet_idxFTI ON snippet_idx USING gist(vectors);

CREATE OR REPLACE FUNCTION update_vectors() RETURNS OPAQUE AS '
DECLARE
table_name TEXT;
BEGIN
	table_name := TG_ARGV[0];
	-- **** tracker_item table ****
	IF table_name = ''tracker_item'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO tracker_item_idx (tracker_item_id, tracker_id, vectors) VALUES (NEW.tracker_item_id, NEW.tracker_id, to_tsvector(''default'', coalesce(NEW.details,'''') ||'' ''|| coalesce(NEW.summary,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE tracker_item_idx SET tracker_id=NEW.tracker_id, vectors=to_tsvector(''default'', coalesce(NEW.details,'''') ||'' ''|| coalesce(NEW.summary,'''')) WHERE tracker_item_id=NEW.tracker_item_id;
		END IF;	
	-- **** tracker_item_message table ****
	ELSIF table_name = ''tracker_item_message'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO tracker_item_message_idx (tracker_item_message_id, tracker_item_id, vectors) VALUES (NEW.tracker_item_message_id, NEW.tracker_item_id, to_tsvector(''default'', coalesce(NEW.body,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE tracker_item_message_idx SET tracker_item_id=NEW.tracker_item_id, vectors=to_tsvector(''default'', coalesce(NEW.body,'''')) WHERE tracker_item_message_id=NEW.tracker_item_message_id;
		END IF;		
	-- **** doc_data table ****
--	ELSIF table_name = ''doc_data'' THEN
--		IF TG_OP = ''INSERT'' THEN
--			INSERT INTO doc_data_idx (docid, group_id, vectors) VALUES (NEW.docid, NEW.group_id, to_tsvector(''default'', coalesce(NEW.title,'''') ||'' ''|| coalesce(NEW.description,'''')));
--		ELSIF TG_OP = ''UPDATE'' THEN
--			UPDATE doc_data_idx SET group_id=NEW.group_id, vectors=to_tsvector(''default'', coalesce(NEW.title,'''') ||'' ''|| coalesce(NEW.description,'''')) WHERE docid=NEW.docid;
--		END IF;
	-- **** forum message table ****
	ELSIF table_name = ''forum_message'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO forum_message_idx (forum_message_id, forum_thread_id, vectors) VALUES (NEW.forum_message_id, NEW.forum_thread_id, to_tsvector(''default'', coalesce(NEW.subject,'''') ||'' ''|| coalesce(NEW.body,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE forum_message_idx SET vectors=to_tsvector(''default'', coalesce(NEW.subject,'''') ||'' ''|| coalesce(NEW.body,'''')) WHERE forum_message_id=NEW.forum_message_id;
		END IF;
	-- **** frs_file table ****
--	ELSIF table_name = ''frs_file'' THEN
--		IF TG_OP = ''INSERT'' THEN
--			INSERT INTO frs_file_idx (frs_file_id, frs_release_id, vectors) VALUES (NEW.frs_file_id, NEW.frs_release_id, to_tsvector(''default'', coalesce(NEW.file_name,'''')));
--		ELSIF TG_OP = ''UPDATE'' THEN
--			UPDATE frs_file_idx SET vectors=to_tsvector(''default'', coalesce(NEW.file_name,'''')), frs_release_id=NEW.frs_release_id WHERE frs_file_id=NEW.frs_file_id;
--		END IF;
	-- **** frs_release table ****
	ELSIF table_name = ''frs_release'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO frs_release_idx (frs_release_id, vectors) VALUES (NEW.frs_release_id, to_tsvector(''default'', coalesce(NEW.changes,'''') ||'' ''|| coalesce(NEW.release_notes,'''') ||'' ''|| coalesce(NEW.release_name,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE frs_release_idx SET vectors=to_tsvector(''default'', coalesce(NEW.changes,'''') ||'' ''|| coalesce(NEW.release_notes,'''') ||'' ''|| coalesce(NEW.release_name,'''')) WHERE frs_release_id=NEW.frs_release_id;
		END IF;
	ELSIF table_name = ''snippet'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO snippet_idx (snippet_id, vectors) VALUES (NEW.snippet_id, to_tsvector(''default'', coalesce(NEW.snippet_name,'''') ||'' ''|| coalesce(NEW.snippet_description,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE snippet_idx SET vectors=to_tsvector(''default'', coalesce(NEW.snippet_name,'''') ||'' ''|| coalesce(NEW.snippet_description,'''')) WHERE snippet_id=NEW.snippet_id;
		END IF;		
	-- **** projects table ****
	ELSIF table_name = ''project'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO projects_idx (project_id, vectors) VALUES (NEW.project_id, to_tsvector(''default'', coalesce(NEW.project_name,'''') ||'' ''|| coalesce(NEW.description,'''') ||'' ''|| coalesce(NEW.unix_name,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE projects_idx SET vectors=to_tsvector(''default'', coalesce(NEW.project_name,'''') ||'' ''|| coalesce(NEW.description,'''') ||'' ''|| coalesce(NEW.unix_name,'''')) WHERE project_id=NEW.project_id;
		END IF;
	-- **** news table ****
	ELSIF table_name = ''news'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO news_idx (news_id, vectors) VALUES (NEW.news_id, to_tsvector(''default'', coalesce(NEW.summary,'''') ||'' ''|| coalesce(NEW.details,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE news_idx SET vectors=to_tsvector(''default'', coalesce(NEW.summary,'''') ||'' ''|| coalesce(NEW.details,'''')) WHERE news_id=NEW.news_id;
		END IF;
	-- **** users table ****
	ELSIF table_name = ''"user"'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO users_idx (user_id, vectors) VALUES (NEW.user_id, to_tsvector(''default'', coalesce(NEW.unix_name,'''') ||'' ''|| coalesce(NEW.firstname,'''') ||'' ''|| coalesce(NEW.lastname,'''') ));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE users_idx SET vectors=to_tsvector(''default'', coalesce(NEW.unix_name,'''') ||'' ''|| coalesce(NEW.firstname,'''') ||'' ''|| coalesce(NEW.lastname,'''') ) WHERE user_id=NEW.user_id;
		END IF;
	END IF;

	RETURN NEW;
END;'
LANGUAGE 'plpgsql';

CREATE TRIGGER tracker_item_ts_update AFTER UPDATE OR INSERT ON tracker_item
FOR EACH ROW EXECUTE PROCEDURE update_vectors('tracker_item');

CREATE TRIGGER tracker_item_message_ts_update AFTER UPDATE OR INSERT ON tracker_item_message
FOR EACH ROW EXECUTE PROCEDURE update_vectors('tracker_item_message');

--	CREATE TRIGGER doc_data_ts_update AFTER UPDATE OR INSERT ON doc_data
--	FOR EACH ROW EXECUTE PROCEDURE update_vectors('doc_data');

CREATE TRIGGER forum_message_update AFTER UPDATE OR INSERT ON forum_message
FOR EACH ROW EXECUTE PROCEDURE update_vectors('forum_message');

--CREATE TRIGGER frs_file_ts_update AFTER UPDATE OR INSERT ON frs_file
--FOR EACH ROW EXECUTE PROCEDURE update_vectors('frs_file');

CREATE TRIGGER frs_release_ts_update AFTER UPDATE OR INSERT ON frs_release
FOR EACH ROW EXECUTE PROCEDURE update_vectors('frs_release');

CREATE TRIGGER snippet_ts_update AFTER UPDATE OR INSERT ON snippet
FOR EACH ROW EXECUTE PROCEDURE update_vectors('snippet');

CREATE TRIGGER projects_ts_update AFTER UPDATE OR INSERT ON project
FOR EACH ROW EXECUTE PROCEDURE update_vectors('project');

CREATE TRIGGER news_ts_update AFTER UPDATE OR INSERT ON news
FOR EACH ROW EXECUTE PROCEDURE update_vectors('news');

CREATE TRIGGER users_ts_update AFTER UPDATE OR INSERT ON "user"
FOR EACH ROW EXECUTE PROCEDURE update_vectors('"user"');
--commit;

