CREATE TABLE filesystem_idx (
        filesystem_id integer,
        vectors tsvector,
        CONSTRAINT filesystemid_fk
        FOREIGN KEY(filesystem_id) REFERENCES filesystem
        ON DELETE CASCADE
);
DELETE FROM filesystem_idx;
INSERT INTO filesystem_idx (filesystem_id, vectors)
SELECT filesystem_id, to_tsvector('default', coalesce(strings,'') ) AS vectors
FROM filesystem ORDER BY filesystem_id;
CREATE UNIQUE INDEX filesystemidx_idx ON filesystem_idx(filesystem_id);

CREATE INDEX filesystem_idxFTI ON filesystem_idx USING gist(vectors);

CREATE TABLE docman_file_idx (
        docman_file_id integer,
        vectors tsvector,
        CONSTRAINT docmanfileid_fk
        FOREIGN KEY(docman_file_id) REFERENCES docman_file
        ON DELETE CASCADE
);
CREATE UNIQUE INDEX docmanfileidx_idx ON docman_file_idx(docman_file_id);
DELETE FROM docman_file_idx;
INSERT INTO docman_file_idx (docman_file_id, vectors)
SELECT docman_file_id, to_tsvector('default', coalesce(description,'')) AS vectors
FROM docman_file ORDER BY docman_file_id;

CREATE INDEX docmanfile_idxFTI ON docman_file_idx USING gist(vectors);

CREATE TABLE docman_file_version_idx (
        docman_file_version_id integer,
        vectors tsvector,
        CONSTRAINT docmanfileid_fk
        FOREIGN KEY(docman_file_version_id) REFERENCES docman_file_version
        ON DELETE CASCADE
);
CREATE UNIQUE INDEX docmanfileversionidx_idx ON docman_file_version_idx(docman_file_version_id);
DELETE FROM docman_file_version_idx;
INSERT INTO docman_file_version_idx (docman_file_version_id, vectors)
SELECT docman_file_version_id, to_tsvector('default', coalesce(changes,'')) AS vectors
FROM docman_file_version ORDER BY docman_file_version_id;

CREATE INDEX docmanfileversion_idxFTI ON docman_file_version_idx USING gist(vectors);



CREATE OR REPLACE FUNCTION update_vectors() RETURNS TRIGGER AS '
DECLARE
table_name TEXT;
BEGIN
	table_name := TG_ARGV[0];
	-- **** tracker_item table ****
	IF table_name = ''tracker_item'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO tracker_item_idx (tracker_item_id, tracker_id, vectors) VALUES (NEW.tracker_item_id, NEW.tracker_id, to_tsvector(''default'', coalesce(NEW.details,'''') ||'' ''|| coalesce(NEW.summary,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE tracker_item_idx SET tracker_id=NEW.tracker_id, vectors=to_tsvector(''default'', coalesce(NEW.details,'''') ||'' ''|| coalesce(NEW.summary,'''')) WHERE tracker_item_id=NEW.tracker_item_id;
		END IF;	
	-- **** tracker_item_message table ****
	ELSIF table_name = ''tracker_item_message'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO tracker_item_message_idx (tracker_item_message_id, tracker_item_id, vectors) VALUES (NEW.tracker_item_message_id, NEW.tracker_item_id, to_tsvector(''default'', coalesce(NEW.body,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE tracker_item_message_idx SET tracker_item_id=NEW.tracker_item_id, vectors=to_tsvector(''default'', coalesce(NEW.body,'''')) WHERE tracker_item_message_id=NEW.tracker_item_message_id;
		END IF;		
	-- **** docman_file table ****
	ELSIF table_name = ''docman_file'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO docman_file_idx (docman_file_id, vectors) VALUES (NEW.docman_file_id, to_tsvector(''default'', coalesce(NEW.description,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE docman_file_idx SET vectors=to_tsvector(''default'', coalesce(NEW.description,'''')) WHERE docman_file_id=NEW.docman_file_id;
		END IF;		
	-- **** docman_file_version table ****
	ELSIF table_name = ''docman_file_version'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO docman_file_version_idx (docman_file_version_id, vectors) VALUES (NEW.docman_file_version_id, to_tsvector(''default'', coalesce(NEW.changes,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE docman_file_version_idx SET vectors=to_tsvector(''default'', coalesce(NEW.changes,'''')) WHERE docman_file_version_id=NEW.docman_file_version_id;
		END IF;		
	-- **** filesystem table ****
	ELSIF table_name = ''filesystem'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO filesystem_idx (filesystem_id, vectors) VALUES (NEW.filesystem_id, to_tsvector(''default'', coalesce(NEW.strings,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE filesystem_idx SET vectors=to_tsvector(''default'', coalesce(NEW.strings,'''')) WHERE filesystem_id=NEW.filesystem_id;
		END IF;		
	-- **** forum message table ****
	ELSIF table_name = ''forum_message'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO forum_message_idx (forum_message_id, forum_thread_id, vectors) VALUES (NEW.forum_message_id, NEW.forum_thread_id, to_tsvector(''default'', coalesce(NEW.subject,'''') ||'' ''|| coalesce(NEW.body,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE forum_message_idx SET vectors=to_tsvector(''default'', coalesce(NEW.subject,'''') ||'' ''|| coalesce(NEW.body,'''')) WHERE forum_message_id=NEW.forum_message_id;
		END IF;
	-- **** frs_release table ****
	ELSIF table_name = ''frs_release'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO frs_release_idx (frs_release_id, vectors) VALUES (NEW.frs_release_id, to_tsvector(''default'', coalesce(NEW.changes,'''') ||'' ''|| coalesce(NEW.release_notes,'''') ||'' ''|| coalesce(NEW.release_name,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE frs_release_idx SET vectors=to_tsvector(''default'', coalesce(NEW.changes,'''') ||'' ''|| coalesce(NEW.release_notes,'''') ||'' ''|| coalesce(NEW.release_name,'''')) WHERE frs_release_id=NEW.frs_release_id;
		END IF;
	ELSIF table_name = ''snippet'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO snippet_idx (snippet_id, vectors) VALUES (NEW.snippet_id, to_tsvector(''default'', coalesce(NEW.snippet_name,'''') ||'' ''|| coalesce(NEW.snippet_description,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE snippet_idx SET vectors=to_tsvector(''default'', coalesce(NEW.snippet_name,'''') ||'' ''|| coalesce(NEW.snippet_description,'''')) WHERE snippet_id=NEW.snippet_id;
		END IF;		
	-- **** projects table ****
	ELSIF table_name = ''project'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO projects_idx (project_id, vectors) VALUES (NEW.project_id, to_tsvector(''default'', coalesce(NEW.project_name,'''') ||'' ''|| coalesce(NEW.description,'''') ||'' ''|| coalesce(NEW.unix_name,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE projects_idx SET vectors=to_tsvector(''default'', coalesce(NEW.project_name,'''') ||'' ''|| coalesce(NEW.description,'''') ||'' ''|| coalesce(NEW.unix_name,'''')) WHERE project_id=NEW.project_id;
		END IF;
	-- **** news table ****
	ELSIF table_name = ''news'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO news_idx (news_id, vectors) VALUES (NEW.news_id, to_tsvector(''default'', coalesce(NEW.summary,'''') ||'' ''|| coalesce(NEW.details,'''')));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE news_idx SET vectors=to_tsvector(''default'', coalesce(NEW.summary,'''') ||'' ''|| coalesce(NEW.details,'''')) WHERE news_id=NEW.news_id;
		END IF;
	-- **** users table ****
	ELSIF table_name = ''"user"'' THEN
		IF TG_OP = ''INSERT'' THEN
			INSERT INTO users_idx (user_id, vectors) VALUES (NEW.user_id, to_tsvector(''default'', coalesce(NEW.unix_name,'''') ||'' ''|| coalesce(NEW.firstname,'''') ||'' ''|| coalesce(NEW.lastname,'''') ));
		ELSIF TG_OP = ''UPDATE'' THEN
			UPDATE users_idx SET vectors=to_tsvector(''default'', coalesce(NEW.unix_name,'''') ||'' ''|| coalesce(NEW.firstname,'''') ||'' ''|| coalesce(NEW.lastname,'''') ) WHERE user_id=NEW.user_id;
		END IF;
	END IF;

	RETURN NEW;
END;'
LANGUAGE 'plpgsql';

CREATE TRIGGER filesystem_ts_update AFTER UPDATE OR INSERT ON filesystem
FOR EACH ROW EXECUTE PROCEDURE update_vectors('filesystem');

CREATE TRIGGER docman_file_ts_update AFTER UPDATE OR INSERT ON docman_file
FOR EACH ROW EXECUTE PROCEDURE update_vectors('docman_file');
CREATE TRIGGER docman_file_version_ts_update AFTER UPDATE OR INSERT ON docman_file_version
FOR EACH ROW EXECUTE PROCEDURE update_vectors('docman_file_version');
