DROP FUNCTION getAllUserProjectsIds(integer);
CREATE OR REPLACE FUNCTION getAllUserProjectsIds(_user_id INTEGER) RETURNS SETOF INTEGER AS E'
DECLARE
	project_row RECORD;
	project_sql VARCHAR;
	sql VARCHAR;
BEGIN
	-- RAISE NOTICE ''getAllUserProjectsIds(%)'', _user_id;
	IF _user_id IS NULL THEN RETURN; END IF;

	project_sql := \' \';
	
	FOR project_row IN 
					SELECT DISTINCT p.root_path, LENGTH(p.root_path) AS len FROM project p, user_project up
					WHERE up.user_id IN (SELECT usergroup_id FROM usergroup_user WHERE member_user_id=_user_id UNION SELECT _user_id)
						AND up.project_id=p.project_id AND p.root_parent_project_id > 0
					ORDER BY len ASC	-- It has been reported that ordering by length increases performance, for some reason
	LOOP
		-- project_sql := project_sql || \' OR (p.root_path ~ \'\'^\' || project_row.root_path || \',\'\') \';
		project_sql := project_sql || \' OR (p.root_path LIKE \'\'\' || project_row.root_path || \',%\'\') \';
	END LOOP;
	
	-- Get all the child projects
	sql := \'SELECT DISTINCT p.project_id, p.root_path FROM project p, user_project up
					WHERE ((up.user_id=\' || _user_id::TEXT || \' OR up.user_id IN (SELECT usergroup_id FROM usergroup_user WHERE member_user_id=\' || _user_id::TEXT || \'))
					AND up.project_id=p.project_id) \' || project_sql;
	
	FOR project_row IN EXECUTE(sql) LOOP
		RETURN NEXT project_row.project_id;
	END LOOP;
END
'
LANGUAGE plpgsql
STABLE;

-- Get all the projects the user has SCM read access to
DROP FUNCTION getAllUserProjectsIdsSCM(INTEGER);
CREATE OR REPLACE FUNCTION getAllUserProjectsIdsSCM(_user_id INTEGER) RETURNS SETOF INTEGER AS E'
DECLARE
	project_row RECORD;
	project_sql VARCHAR;
	root_path VARCHAR;
	sql VARCHAR;
BEGIN
	-- RAISE NOTICE ''getAllUserProjectsIdsSCM(%)'', _user_id;
	IF _user_id IS NULL THEN RETURN; END IF;
	
	project_sql := \' \';
	
	FOR project_row IN 
					SELECT DISTINCT p.root_path, LENGTH(p.root_path) AS len FROM project p, user_project up
					WHERE up.user_id IN (SELECT usergroup_id FROM usergroup_user WHERE member_user_id=_user_id UNION SELECT _user_id)
						AND up.project_id=p.project_id AND p.root_parent_project_id > 0 AND p.root_path IS NOT NULL
					ORDER BY len ASC	-- It has been reported that ordering by length increases performance, for some reason
	LOOP
		-- project_sql := project_sql || \' OR (p.root_path ~ \'\'^\' || project_row.root_path || \',\'\') \';
		project_sql := project_sql || \' OR (p.root_path LIKE \'\'\' || project_row.root_path || \',%\'\') \';
	END LOOP;
	
	-- Get all the child projects
	sql := \'
		SELECT r.project_id, MAX(rs.value) AS max_value FROM role_setting rs, user_project_role upr, role r
		WHERE r.role_id=rs.role_id AND upr.role_id=rs.role_id AND upr.user_project_id IN (
			SELECT up.user_project_id FROM user_project up, project p
			WHERE
				up.project_id=p.project_id 
				AND (
					up.user_id=\' || _user_id::TEXT || \' 
					OR up.user_id IN (SELECT usergroup_id FROM usergroup_user WHERE member_user_id=\' || _user_id::TEXT || \' )
					\' || project_sql || \' 
				)
		) AND rs.section=\'\'scm\'\'
		GROUP BY r.project_id;
	\';
	
	-- RAISE NOTICE ''%'', sql;
	
	FOR project_row IN EXECUTE(sql) LOOP
		IF project_row.max_value >= 0 THEN
			RETURN NEXT project_row.project_id;
		END IF;
	END LOOP;
END
'
LANGUAGE plpgsql
STABLE;
