-- Utility function to make up for missing functionality in postgres < 8.2
CREATE OR REPLACE function drop_if_exists(text)
RETURNS int4 AS
$$
BEGIN
  IF (SELECT count(*) FROM pg_tables where tablename=$1) THEN
    EXECUTE 'DROP TABLE ' || $1;
    RETURN 1;
  END IF;
  RETURN 0;
END;
$$
LANGUAGE 'plpgsql' VOLATILE;


-- Table: activity_log
-- DROP TABLE activity_log;
SELECT drop_if_exists('activity_log');

CREATE TABLE activity_log
(
  activity_log_id serial NOT NULL,
  activity_date timestamp with time zone NOT NULL DEFAULT now(),
  project_id integer DEFAULT 0,
  user_id integer DEFAULT 100,
  "section" character varying(20),
  ref_id integer DEFAULT 0,
  description text NOT NULL,
  url text,
  activity_type character varying(20),
  extended_data text,
  CONSTRAINT activity_log_pkey PRIMARY KEY (activity_log_id ),
  CONSTRAINT userid_fk FOREIGN KEY (user_id)
      REFERENCES "user" (user_id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITHOUT OIDS;

CREATE INDEX activity_log_ref
  ON activity_log
  USING btree
  ("section" , ref_id );

CREATE INDEX activity_log_type
  ON activity_log
  USING btree
  (activity_type );

CREATE INDEX activity_log_project
  ON activity_log
  USING btree
  (project_id);

CREATE INDEX activity_log_date
  ON activity_log
  USING btree
  (activity_date);
