<?php

require_once('initialize.php');
require_once('core/Project.php');
//This is needed by some of the classes we are using here
require_once('www/themes/gforge5/GFTheme.php');

//We use the svn_project as the base to build the other two
$scmTemplateProject = ProjectPeer::retrieveByUnixName('svn_project');
if(!$scmTemplateProject){
	echo "Template svn project not found";
	exit(0);
}
$genericUnixName='generic_project';
$gitUnixName='git_project';

//Retrieve site admins
$c = new Criteria();
$c->add(UserPeer::STATUS, User::STATUS_ACTIVE);
/** @var SiteAdmin[] $admins  */
$admins = SiteAdminPeer::doSelectJoinUser($c);

if(!count($admins)) {
	//Shouldn't happen
	echo "No admins found!\n";
	exit(0);
}
/** @var User $siteAdmin  */
$siteAdmin = $admins[0]->getUser();

$session = Session::getSession();
$session->setUser($siteAdmin);

$project = ProjectPeer::retrieveByUnixName($gitUnixName);

if($project){
	echo "A project with the unix_name git_project already exists, skipping git project template creation\n";
	exit(0);
}

try{ 
	$gitProject = new Project();
	$gitProject->setProjectName('Basic project with GIT support');
	$gitProject->setDescription('Basic project with GIT support');
	$gitProject->setUnixName($gitUnixName);
	$gitProject->setRegisterPurpose('Basic project with GIT support');
	$gitProject->setTemplateProjectId($scmTemplateProject->getProjectId());
	$gitProject->setIsTemplate(true);
	$gitProject->save();
	//Force notification to avoid propel cache problem
	$gitProject = ProjectPeer::retrieveByUnixName($gitProject->getUnixName(), false);
	$gitProject->approve();


} catch (Exception $e){
	echo "A problem has occurred: " . $e->getMessage();
	exit(1);
}

$gitProject->disablePlugin('scmsvn');
$gitProject->enablePlugin('scmgit');
$gitProject->setTemplateProjectId(null);
$gitProject->save();

//Finally, remove Site Admin from Project
removeAdminMember($siteAdmin->getId(), $gitProject->getId());

//Generic Project

try{
	$genericProject = new Project();
	$genericProject->setProjectName('Generic Project (non-software)');
	$genericProject->setDescription('Generic Project (non-software)');
	$genericProject->setUnixName($genericUnixName);
	$genericProject->setRegisterPurpose('Generic Project (non-software)');
	$genericProject->setTemplateProjectId($scmTemplateProject->getProjectId());
	$genericProject->setIsTemplate(true);
	$genericProject->save();
	//Force save
	$genericProject = ProjectPeer::retrieveByUnixName($genericProject->getUnixName(), false);
	$genericProject->approve();
}catch (Exception $e){
	echo "A problem has occurred: " . $e->getMessage();
	exit(1);
}
$genericProject->disablePlugin('scmsvn');
$genericProject->disablePlugin('frs');
$genericProject->setTemplateProjectId(null);
$genericProject->save();

//Remove/Change trackers
$c = new Criteria();
$c->add(TrackerPeer::PROJECT_ID, $genericProject->getProjectId());
/** @var Tracker[] $trackers  */
$trackers = TrackerPeer::doSelect($c);
foreach($trackers as $tracker){
	//We will keep 'To-Do' and 'Bugs', the later will be renamed to Issues
	if($tracker->getTrackerName() == 'Bugs'){
		$tracker->setTrackerName('Issues');
		$tracker->save();
	} else if($tracker->getTrackerName() == 'To-Do'){
		continue;
	} else{
		$tracker->delete();
	}
}

//Finally, remove Site Admin
removeAdminMember($siteAdmin->getId(), $genericProject->getId());

//And close the session
$session->terminate();

exit(0);

function removeAdminMember($user_id, $project_id) {
	$c = new Criteria();
	$c->add(UserProjectPeer::USER_ID, $user_id);
	$c->add(UserProjectPeer::PROJECT_ID, $project_id);

	$up = UserProjectPeer::doSelectOne($c);
	$up->delete();
}

?>
