<?php


require_once('initialize.php');
require_once('GFConf.php');
require_once('lib/StringUtils.php');
require_once('lib/SystemUtils.php');

define ('GREEN', "\033[01;32m" );
define ('NORMAL', "\033[00m" );
define ('RED', "\033[01;31m" );

/**
 * Inserts rewrite inside the Directory /opt/gforge/wwww  section
 */
function insertRewriteRules($apacheConf, $sectionStart, $sectionEnd, $textToInsert) {

	$strUtil = new StringUtils();

	if(!is_file($apacheConf) || !is_writeable($apacheConf) || !is_readable($apacheConf)) {
		return false;
	}

	$originalContent = file($apacheConf);
	$contentStr = implode(PHP_EOL, $originalContent);

	$oldSection = $strUtil->getTextBetweenMarkers($contentStr, $sectionStart, $sectionEnd);
	//Check if changes are already there
	$linesPresent = strpos($oldSection,$textToInsert[0]);

	if($linesPresent === false) {
		$newContent = StringUtils::insertTextBetweenMarkers($sectionStart, $sectionEnd, $originalContent, $textToInsert);
      SystemUtils::writeFile($apacheConf, $newContent);
		return true;
	}

	return false;
}

function isLinePresent($content, $sectionStart, $sectionEnd, $line) {
	$strUtil = new StringUtils();
	if (is_array($content)) {
		$content = implode(PHP_EOL, $content);
	}

	$section = $strUtil->getTextBetweenMarkers($content, $sectionStart, $sectionEnd);

	$linePresent = strpos($section, $line);

	if($linePresent === false) {
		return false;
 	} else {
		return true;
	}
}

function show($text, $newLine = true) {
	if ($newLine) {
		$text = GREEN.$text .NORMAL."\n";
	}
	fwrite(STDOUT, $text);
}


function input_Y_N($text, $default='y')
{
	while (true) {
		show(RED.$text.NORMAL);
		$proceed = strtolower(trim(fgets(STDIN)));

		if (!$proceed) {
			return ($default == 'Y' || $default == 'y');
		}

		if ($proceed == 'n' || $proceed == 'no') {
			return false;
		} elseif ($proceed == 'y' || $proceed == 'yes') {
			return true;
		}
	}
}

function run($command, $ignore = false) {
	system($command, $ret);
	if ($ret != 0 && $ignore == false) {
		echo RED.'An error ocurred running the last command... aborting.'.NORMAL."\n";
		die();
	}
}

$apacheConf= GFConf::get('system.etcDir').DIRECTORY_SEPARATOR.'httpd.conf';

if(!is_file($apacheConf) || !is_readable($apacheConf)) {
	throw new Exception ('Couldn\'t read apache configuration file');
}

//Backup current apache config file
$apacheBackup = GFConf::get('system.etcDir').DIRECTORY_SEPARATOR.'httpd_'.date('Ymd-hms', time()).'.conf';
$cmd = 'cp ' . $apacheConf . ' ' . $apacheBackup;
SystemUtils::execute($cmd, $output);

$originalContent = file($apacheConf);
$sectionStart = '<Directory "/opt/gforge/www">';
$sectionEnd = '</Directory>';

$textToInsert = array(
	'RewriteEngine On',
	'RewriteBase /',
	'RewriteRule (.*\.css) /combine.php?type=css&files=$1',
	'RewriteRule (.*\.js) /combine.php?type=javascript&files=$1'
);

if(!isLinePresent($originalContent, $sectionStart, $sectionEnd, $textToInsert[0])) {

	$message = "A new set of rewrite rules are needed for the /opt/gforge/www directory in your " .
	GFConf::get('system.etcDir').DIRECTORY_SEPARATOR ."httpd.conf as follows:
	\t<Directory \"/opt/gforge/www\">
	\t\tRewriteEngine On
	\t\tRewriteBase /
	\t\tRewriteRule (.*\\.css) /combine.php?type=css&files=$1
	\t\tRewriteRule (.*\\.js) /combine.php?type=javascript&files=$1
	\t</Directory>
	Do you want the me to automatically insert the lines in the proper section?";

	$accepted = input_Y_N($message, 'n');
	if ($accepted) {
		//Insert rewrite rules for combined resources and restart apache
		insertRewriteRules($apacheConf, $sectionStart, $sectionEnd, $textToInsert);
		//We restart apache in the main upgrade_gforge script
	}
}

exit(0);

?>