<?php
require_once('initialize.php');
require_once('GFConf.php');
require_once('plugin/PluginFactory.php');
require_once('plugins/forum/lib/ForumMessagePeer.php');
require_once('plugins/docman/lib/DocmanFilePeer.php');
require_once('plugins/tracker/lib/TrackerItemPeer.php');
require_once('core/ActivityLog.php');
require_once('lib/core/PropelJson.php');

$con = Propel::getConnection();

try {

	$sql = "DELETE FROM activity_log WHERE section IN ('forumthread', 'projectchat', 'chat') AND activity_type='update'";

	$stm = $con->executeUpdate($sql);

	$c = new Criteria();
	$c->add(ForumThreadPeer::MESSAGE_COUNT, '1', Criteria::GREATER_THAN);
	/** @var Forum[] $forums  */
	$forumThreads = ForumThreadPeer::doSelect($c);

	foreach ($forumThreads as $forumThread) {
		writeActivityForForumThread($con, $forumThread);
	}

	exit(0);

} catch (Exception $e) {
	echo $e->getMessage()."\n";
	exit(1);
}


function writeActivityForForumThread($con, $forumThread) {

	$section = ForumThread::SECTION;
	$forum = $forumThread->getForum();
	$forum_id = $forum->getForumId();
	$forum_thread_id = $forumThread->getForumThreadId();

	try {
		$project = $forumThread->getForum()->getProject();
		if (!is_object($project)) {
			return;
		}
	} catch (Exception $e) {
		return;
	}
	$project_id = $project->getProjectId();
	$url = 'null';

	if (!is_null($project)) {
		$project_id = $forum->getProject()->getProjectId();

		$args = array(
			'thread_id' => $forumThread->getForumThreadId(),
			'forum_id' => $forum->getForumId(),
			'_forum_action' => 'ForumMessageBrowse'
			);
		$url = GFUrl::getUrl()->projectURL($project, 'ForumBrowse', $section, '', $args);
	}

	$sql = "INSERT INTO activity_log(project_id, section, ref_id, user_id, activity_date, description, url, activity_type, extended_data)
			SELECT
			  $project_id as project_id,
			  '$section' as section,
			  forum_thread.forum_thread_id as ref_id,
			  \"user\".user_id as user_id,
			  forum_message.post_date as activity_date,
			  \"user\".unix_name || ' updated Discussion Thread ' || forum_thread.thread_name || ' in ' || forum.forum_name as description,
			  '$url' as url,
			  'update' as activity_type,
			  null as extended_data
			FROM
			  public.forum_thread,
			  public.forum_message,
			  public.\"user\",
			  public.forum
			WHERE
			  forum_thread.forum_id = forum.forum_id AND
			  forum_message.forum_thread_id = forum_thread.forum_thread_id AND
			  forum_message.created_by = \"user\".user_id AND
			  forum_thread.message_count > 1 AND 
			  forum_thread.forum_thread_id = $forum_thread_id";

	$stm = $con->executeUpdate($sql);

	return true;
}






