<?php
/**
 * Remove rewrite directives form gforge's httpd.conf
 */

require_once('initialize.php');
require_once('GFConf.php');
require_once('lib/StringUtils.php');
require_once('lib/SystemUtils.php');

define ('GREEN', "\033[01;32m" );
define ('NORMAL', "\033[00m" );
define ('RED', "\033[01;31m" );

function commentRewriteRules($apacheConf, $sectionStart, $sectionEnd, $arrTextToComment) {

	$strUtil = new StringUtils();

	if(!is_file($apacheConf) || !is_writeable($apacheConf) || !is_readable($apacheConf)) {
		return false;
	}

	/*
	$strTextToInsert = '';

	foreach($arrTextToInsert as $line) {
		$strTextToInsert .= "\n\t\t" . "#" . $line;
	}*/

	$originalContent = file_get_contents($apacheConf);
	$section = $strUtil->getTextBetweenMarkers($originalContent, $sectionStart, $sectionEnd);

	foreach ($arrTextToComment as $line) {
		$section = str_replace($line, '#'.$line, $section);
	}

	$section = "\n\t\t".$section."\n\t";

	$newContent = $strUtil->replaceTextBetweenMarkers($originalContent, $sectionStart, $sectionEnd, $section);
	SystemUtils::writeFile($apacheConf, $newContent);

}

function isLinePresent($content, $sectionStart, $sectionEnd, $line) {
	$strUtil = new StringUtils();
	if (is_array($content)) {
		$content = implode(PHP_EOL, $content);
	}

	$section = $strUtil->getTextBetweenMarkers($content, $sectionStart, $sectionEnd);

	$linePresent = strpos($section, $line);

	if($linePresent === false) {
		return false;
 	} else {
		return true;
	}
}


function show($text, $newLine = true) {
	if ($newLine) {
		$text = GREEN.$text .NORMAL."\n";
	}
	fwrite(STDOUT, $text);
}


function input_Y_N($text, $default='y')
{
	while (true) {
		show(RED.$text.NORMAL);
		$proceed = strtolower(trim(fgets(STDIN)));

		if (!$proceed) {
			return ($default == 'Y' || $default == 'y');
		}

		if ($proceed == 'n' || $proceed == 'no') {
			return false;
		} elseif ($proceed == 'y' || $proceed == 'yes') {
			return true;
		}
	}
}

function run($command, $ignore = false) {
	system($command, $ret);
	if ($ret != 0 && $ignore == false) {
		echo RED.'An error ocurred running the last command... aborting.'.NORMAL."\n";
		die();
	}
}

$apacheConf= GFConf::get('system.etcDir').DIRECTORY_SEPARATOR.'httpd.conf';

if(!is_file($apacheConf) || !is_readable($apacheConf)) {
	throw new Exception ('Couldn\'t read apache configuration file');
}

//Backup current apache config file
$apacheBackup = GFConf::get('system.etcDir').DIRECTORY_SEPARATOR.'httpd_'.date('Ymd-hms', time()).'.conf';
$cmd = 'cp ' . $apacheConf . ' ' . $apacheBackup;
SystemUtils::execute($cmd, $output);

$originalContent = file($apacheConf);
$sectionStart = '<Directory "/opt/gforge/www">';
$sectionEnd = '</Directory>';

$textToComment = array(
	'RewriteEngine On',
	'RewriteBase /',
	'RewriteRule (^((?!\/download\/).)*\.css$) /combine.php?type=css&files=$1',
	'RewriteRule (^((?!\/download\/).)*\.js$) /combine.php?type=javascript&files=$1'
);

if(isLinePresent($originalContent, $sectionStart, $sectionEnd, $textToComment[2])) {
	$message = "Old rewrite rules for /opt/gforge directory are not needed anymore, so I need to comment the" .
	" following lines in " . GFConf::get('system.etcDir').DIRECTORY_SEPARATOR ."httpd.conf:
	\t<Directory \"/opt/gforge/www\">
	\t\tRewriteEngine On
	\t\tRewriteBase /
	\t\tRewriteRule (^((?!\\/download\\/).)*\\.css$) /combine.php?type=css&files=$1
	\t\tRewriteRule (^((?!\\/download\\/).)*\\.js$) /combine.php?type=javascript&files=$1
	\t</Directory>
	Do you want me to automatically comment the lines in the proper section?";

	$accepted = input_Y_N($message, 'n');
	if ($accepted) {
		//Insert rewrite rules for combined resources and restart apache
		commentRewriteRules($apacheConf, $sectionStart, $sectionEnd, $textToComment);
		//Run again in case there are two virual hosts on the same file
		commentRewriteRules($apacheConf, $sectionStart, $sectionEnd, $textToComment);

		//We restart apache in the main upgrade_gforge script
	} else {
		echo "Please remember to comment the lines descrived above yourself";
	}
}
