CREATE OR REPLACE FUNCTION trove_move(category_id bigint, to_id bigint)
  RETURNS integer AS
$BODY$
DECLARE
    to_trove RECORD;      -- trove category record for move destination
	to_root_id integer;   -- new value for root_trove_category_id for the category being moved
    root_id integer;      -- new value for root_trove_category_id for the children of the category being moved
    update_count integer; -- updated records count
    last_count integer;   -- temp counter
	temp RECORD;		  -- temp record    
BEGIN
	-- Validate that source and destination category are not the same
	-- and that destination category is not descendant of source category
	SELECT trove_category_id INTO temp FROM trove_children(category_id) WHERE trove_category_id = to_id;
	GET DIAGNOSTICS last_count = ROW_COUNT;
	IF last_count > 0
	THEN
		RETURN 0;
	ELSE
		-- Init update counter
		update_count := 0;
		-- Get trove category record for move destination
		SELECT parent_trove_category_id, root_trove_category_id INTO to_trove FROM trove_category WHERE trove_category_id = to_id;
		-- Get new value for root_trove_category_id for the category being moved
		IF to_trove.parent_trove_category_id = 0
		THEN
			-- If destination category depth <= 1, the id of the destination category is the root_trove_category_id to use 
			to_root_id := to_id;
		ELSE
			-- If destination category depth > 1, we inherit the root trove category from the destination category
			to_root_id := to_trove.root_trove_category_id;
		END IF;
		-- Update the record for the category we're moving
		UPDATE trove_category SET parent_trove_category_id = to_id, root_trove_category_id = to_root_id WHERE trove_category_id = category_id;
		GET DIAGNOSTICS last_count = ROW_COUNT;
		update_count := update_count + last_count;
		-- Get the new value for root_trove_category_id for the children of the category being moved
		IF to_id > 0
		THEN
			-- If destination category depth >= 1, then children inherit root category from parent
			root_id := to_root_id;
		ELSE
			-- If destination category depth is 0 (Root Cat) then we're creating a root category, so the id for the category being moved
			-- is now the root_category_id for its children
			root_id := category_id;
		END IF;
		-- Update the records for the children of the category we're moving
		UPDATE trove_category SET root_trove_category_id = root_id WHERE trove_category_id IN (SELECT trove_category_id FROM trove_children(category_id, FALSE));
		GET DIAGNOSTICS last_count = ROW_COUNT;
		update_count := update_count + last_count;    
		-- Return updated records count
		RETURN update_count;
	END IF;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
