<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version 
 * @author Daniel A. Perez	<daniel@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

/* Search class  */

require_once('SystemUtils.php');
require_once('GFConf.php');
require_once('exceptions/InvalidPluginException.php');

class GFSearch {
	
	private $words;
	
	private $project_id = 0;
	private $template = null;
	private $startdate = 0;
	private $enddate = 0;
	private $search_attach = false;
	private $project_ids = null;
	public $params;
	
	/**
	 * constructor. takes the section and ref_id (0 if none/search all), search words and an array of field to look for them
	 *
	 * @param string $section
	 * @param int $ref_id
	 * @param string $words
	 * @param array $fields
	 * @param object	template object
	 * @param int	project_id
	 * @param int whether wether we are  using exact search or not
	 */
	public function __construct($params) {
		$this->words = $params['words'];
		if (isset($params['project'])) {
			$this->project_id = $params['project']->getProjectId();
		}
		
		if (isset($params['template'])) {
			$this->template = $params['template'];
		}
		
		if (isset($params['startdate'])) {
			$this->startdate = $params['startdate'];
		}

		if (isset($params['enddate'])) {
			$this->enddate = $params['enddate'];
		}

		if (isset($params['search_attach'])) {
			$this->search_attach = $params['search_attach'];
		}

		if (isset($params['project_ids'])) {
			$this->project_ids = $params['project_ids'];
		}
		
		// Store any other parameters
		$this->params = $params;
	}

	public function getStartDate() {
		if ($this->enddate) {
			return date('Y-m-d',strtotime($this->startdate));
		} else {
			return null;
		}
	}

	public function getEndDate() {
		if ($this->startdate) {
			return date('Y-m-d',strtotime($this->enddate));
		} else {
			return null;
		}
	}

	public function getSearchAttach() {
		return $this->search_attach;
	}

	/**
	 * returns the words ready to use in the FTI query
	 *
	 * @return unknown
	 */
	public function getWordsForQuery() {
		$words = $this->words;

		// The search words come sanitized from the search script, so "a & b" becomes
		// "a &amp; b". We must undo this, otherwise tsearch2 won't work
		$words = StringUtils::unHtmlSpecialChars($words);

		// Remove single, double quotes and backslashes
		$words = str_replace('"', ' ', $words);
		$words = str_replace('\'', ' ', $words);
		$words = str_replace('\\', ' ', $words);
		
		// Remove other conflicting characters
		$words = str_replace('::', ' ', $words);		// Used in code search
		$words = str_replace('->', ' ', $words);		// Idem
		
		// Remove extra spaces
		$words = preg_replace('/[[:blank:]][[:blank:]]*/', ' ', $words);
		$words = preg_replace('/[(][[:blank:]]*/', '(', $words);
		$words = preg_replace('/[[:blank:]]*[)]/', ')', $words);
		$words = preg_replace('/[!][[:blank:]]*/', '!', $words);
		$words = trim($words);

		$tsearch_operators = array('(', ')', '!', '|', '&');
		foreach ($tsearch_operators as $operator) {
			$words = str_replace(" $operator ", $operator, $words);
		}

		// Only spaces are left. Glue them with the "and" operator
		$words = str_replace(' ', '&', $words);
		
		// Remove characters that are specific to tsearch
		/*$words = str_replace("'", '', $words);
		$words = str_replace("\\", '', $words);
		$words = str_replace("(", '', $words);
		$words = str_replace(")", '', $words);
		$words = str_replace("!", ' ', $words);
		$words = str_replace("|", ' ', $words);
		$words = str_replace("&", ' ', $words);*/

		return $words;
	}
	
	/**
	 * Returns the words just as they were typed by the user
	 */
	public function getWords() {
		return $this->words;
	}
	
	public function getSearchFields() {
		return $this->fields;
	}
	
	public function getProjectId() {
		return $this->project_id;
	}

	/**
	 * @return array|null
	 */
	public function getProjectIds() {
		return $this->project_ids;
	}

	/**
	 * @return SiteTemplate|ProjectTemplate template
	 */
	public function getTemplate() {
		return $this->template;
	}
}

?>
