<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version $Id$
 * @author Marcelo Mottalli	<mottalli@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('SystemUtils.php');
require_once('GFConf.php');
require_once('exceptions/InvalidPluginException.php');
require_once('core/Project.php');
 
class GFUrl {
	private static $instance = null;
	
	private $project = null;
	private $user = null;
	private $plugin = null;
	private $subdir = '';
	private $handler = '';
	private $action = '';
	private $plugin_exclusive_type;
	private $relative_uri = '';
	
	private $http_request;
	
	public function __construct() {
	}
	
	/**
	 * @return GFUrl
	 */
	public static function getUrl() {
		if (is_null(self::$instance)) {
			self::$instance = new GFUrl();
			self::$instance->parseRequest();
		}
		
		return self::$instance;
	}

	public function parseRequest() {
		if (isset($_SERVER) && isset($_SERVER['REQUEST_URI'])) {
			$this->parseURI($_SERVER['REQUEST_URI']);
			$this->http_request = true;
		} else {
			$this->http_request = false;
		}
	}

	public function parseURI($uri) {
		// There are several kinds of URIs:
		// /gf/project/(projectname)/(pluginname)/(subdirs)/?action=someaction&param1=xx&param2=yyy...
		// /gf/(pluginname)/(subdirs)/?action=someaction&param1=xx&param2=yyy...
		// /gf/(subdirs)/?action=someaction&param1=xx&param2=yyy...
//TODO SANITIZE URL!!!
		if (!Validator::validateLocalUri($uri)) {
			throw new PermissionDeniedException('Invalid URI');
		}

		// Start parsing the URL
		// remove leading "/" (if any)
		//$uri = preg_replace('/^\\//', '', $uri);
		$uri = preg_replace('/^\//', '', $uri);
		$uri = trim($uri, '/');

		// get uri components
		$args = explode('/', $uri);
		
		// remove the empty arguments
		$tmpargs = array();
		foreach ($args as $arg) {
			if (empty($arg)) continue;
			$tmpargs[] = $arg;
		}
		$args = $tmpargs;
			
		if (isset($args[0]) && trim($args[0]) == 'gf') {
			// Invoked via "gf" script... remove it
			array_splice($args, 0, 1);
		}
		
		if (isset($args[0])) {
			if ($args[0] == 'project') {	// Project invoked
				array_splice($args, 0, 1);
				if (!isset($args[0]) || strpos($args[0], '?') === 0) {
					//god this is getting messy
					$wwwlib = SystemUtils::buildFilePath(GFConf::get('system.libDir'), '/wwwlib/project');
					$this->handler = SystemUtils::buildFilePath($wwwlib, '/handler.php');
					
					if (isset($_GET['action']) && strlen($_GET['action']) > 0) {
						$this->action = $_GET['action'];
					} else {
						$this->action = 'ProjectBrowse';
					}
					$this->subdir = 'project/';
					return;
				}

				$this->parseProjectURI($args);
			} elseif($args[0] == 'user') {
				array_splice($args, 0, 1);
				if (!isset($args[0])) {
					//god this is getting messy
					$wwwlib = SystemUtils::buildFilePath(GFConf::get('system.libDir'), '/wwwlib/user');
					$this->handler = SystemUtils::buildFilePath($wwwlib, '/handler.php');
					$this->action = 'UserBrowse';
					return;
				}
				$this->parseUserURI($args);					
			} elseif ($this->isPlugin($args[0])) {
				$this->parsePluginURI($args);
			} elseif ($args[0] == 'admin') {	// site admin invoked
				$this->parseSubdirURI($args);
			} else {
				$this->parseSubdirURI($args);
			}
		} 
		
		// Determine the correct handler
		if (!is_null($this->plugin)) {
			$wwwlib = SystemUtils::buildFilePath(GFConf::get('system.pluginsPath'), '/'.$this->plugin->getName().'/wwwlib');
		} else if (!empty($this->plugin_exclusive_type)) {
			$wwwlib = SystemUtils::buildFilePath(GFConf::get('system.pluginsPath'), '/'.$this->plugin_exclusive_type.'/wwwlib');
		} else {
			if (!is_null($this->project)) {
				$wwwlib = SystemUtils::buildFilePath(GFConf::get('system.libDir'), '/wwwlib/project');
			} elseif (!is_null($this->user)) {
				$wwwlib = SystemUtils::buildFilePath(GFConf::get('system.libDir'), '/wwwlib/user');
			} else {
				$wwwlib = SystemUtils::buildFilePath(GFConf::get('system.libDir'), '/wwwlib');
			}
		}
		
		// Try to find the handler file. The handler is always called "handler.php",
		// and it should be somewhere inside the wwwlib path.
		// The handler is the script in charge of parsing the URL for the specific
		// plugin.
		$subdir = preg_replace('/^\\//', '', $this->subdir);		// remove first slash
		$subdir = preg_replace('/\\/$/', '', $subdir);		// remove trailing slash
		$subdirs = explode('/', $subdir);
		$this->relative_uri = '';
		while (true) {
			$subdir = implode('/', $subdirs);
			$potentialHandler = SystemUtils::buildFilePath($wwwlib, $subdir);
			$potentialHandler = SystemUtils::buildFilePath($potentialHandler, '/handler.php');
			
			if (file_exists($potentialHandler) || count($subdirs) == 0) {
				$this->handler = $potentialHandler;
				break;
			} else {
				// keep looking in the subdirectory below the one we just checked
				$last_subdir = array_pop($subdirs);
				$this->relative_uri = '/'.$last_subdir.$this->relative_uri;
			}
		};
		// Now get the action from the request parameters
		$action_get = isset($_GET['action']) ? $_GET['action'] : '';
		$action_post = isset($_POST['action']) ? $_POST['action'] : '';
		
		if ($action_get && $action_post) {
			// If both are set, POST has priority over GET
			$this->action = $action_post;
		} elseif ($action_get) {
			$this->action = $action_get;
		} elseif ($action_post) {
			$this->action = $action_post;
		} else {
			$this->action = 'index';
		}
	}
	
	/**
	 * Project URIs are usually /gf/project/(project name)/(subdir or plugin name).
	 * The subdirectories are mapped to /(gforge_root)/wwwlib/project/(subdir)/handler.php, plugins are mapped
	 * to /(plugin_path)/(plugin_name)/wwwlib/(subdir)/handler.php
	 * @todo	Parse sub-subdirectories (or an arbitrary depth tree)
	 */
	private function parseProjectURI($args) {
		if (!isset($args[0])) {
			return;
		}
		
		$unixName = $args[0];
		$project = ProjectPeer::retrieveByUnixName($unixName);
		if (is_null($project)) {
			require_once('exceptions/InvalidProjectException.php');
			throw new InvalidProjectException($unixName);
		}
		//protect access to private projects
/*		if (!$project->userCanRead()) {
			require_once('exceptions/PermissionDeniedException.php');
			throw new PermissionDeniedException();
		}*/
		$this->project = $project;
		
		// Chop off the project name
		array_splice($args, 0, 1);
		
		if (isset($args[0]) && !empty($args[0])) {	// Plugin name or subdir?
			if ($this->isPlugin($args[0])) {
				$this->parsePluginURI($args);
			} else {
				$this->parseSubdirURI($args);
			}
		}
	}
	
	/**
	 * User URIs are usually /gf/user/(user name)/(subdir or plugin name).
	 * The subdirectories are mapped to /(gforge_root)/wwwlib/user/(subdir)/handler.php, plugins are mapped
	 * to /(plugin_path)/(plugin_name)/wwwlib/(subdir)/handler.php
	 * @todo	Parse sub-subdirectories (or an arbitrary depth tree)
	 */
	private function parseUserURI($args) {
		if (!isset($args[0])) {
			return;
		}
		
		$unixName = $args[0];
		$user = UserPeer::retrieveByUnixName($unixName);
		if (is_null($user)) {
			require_once('exceptions/InvalidUserException.php');
			throw new InvalidUserException($unixName);
		}
		$this->user = $user;
		//$this->user->initializePlugins();  this will happen?
		
		// Chop off the user name
		array_splice($args, 0, 1);
		
		if (isset($args[0]) && !empty($args[0])) {	// Plugin name or subdir?
			if ($this->isPlugin($args[0])) {
				$this->parsePluginURI($args);
			} else {
				$this->parseSubdirURI($args);
			}
		}
	}	
	
	private function parsePluginURI($args) {
		$pluginName = $args[0];
		// Chop off plugin name
		array_splice($args, 0, 1);

		// Check if a site plugin with that name exists
		// (note that this effectively enables a site plugin in all the projects)
		$pf = PluginFactory::getFactory();
		$plugin = $pf->getPluginByName($pluginName);
		if ($plugin && $plugin->getType() == GFPlugin::SITE_PLUGIN) {
			$this->plugin = $plugin;
		} else {
			// Check for project plugins
			if ($this->project) {
				if (!$this->project->usesPlugin($pluginName)) {
					// check if we're referencing the exclusive_type instead of a plugin name
					// (for example, 'scm')
					// Note: this is rarely used
					if ($this->project->hasPluginType($pluginName)) {
						$this->plugin_exclusive_type = $pluginName;
					} else {
						throw new InvalidPluginException($pluginName);
					}
				} else {
					$this->plugin = $this->project->getPluginByName($pluginName);
				}
			} else {
				// check if we're referencing the exclusive_type instead of a plugin name
				// (for example, 'scm')
				// Note: this is rarely used
				if ($this->isPluginType($pluginName)) {
					$this->plugin_exclusive_type = $pluginName;
				} else {
					$pf = PluginFactory::getFactory();
					$this->plugin = $pf->getPluginByName($pluginName);
					if (is_null($this->plugin)) {
						throw new InvalidPluginException($pluginName);
					}
				}
			}
		}
		
		if (isset($args[0])) {	// Subdir included
			$this->parseSubdirURI($args);
		}
	}
	
	private function parseSubdirURI($args) {
		$this->subdir = '';
		foreach ($args as $dir) {
			// we got to the GET parameters... don't continue parsing the URL
			if (strpos($dir, '?') !== false) return;
			
			$this->subdir .= $dir.'/';
		}
	}
	
	private function isPlugin($name) {
		$pluginPath = SystemUtils::buildFilePath(GFConf::get('system.pluginsPath'), '/'.$name);
		return is_dir($pluginPath);
	}
	
	public function getHandler() {
		return $this->handler;
	}
	
	public function getAction() {
		return $this->action;
	}
	
	public function getProject() {
		return $this->project;
	}
	
	public function getUser() {
		return $this->user;
	}
	

	public function getPlugin() {
		return $this->plugin;
	}

	public function getSubdir() {
		return $this->subdir;
	}
	
	/**
	 * Keeps the current path but replaces the action and the parameters
	 */
	public function currentBaseURL($newAction = '', $params=array()) {
		$return = '/gf/';
		if ($this->project) {
			$return .= 'project/'.$this->project->getUnixName().'/';
		}
		if ($this->user) {
			$return .= 'user/'.$this->user->getUnixName().'/';
		}
		if ($this->plugin) {
			$return .= $this->plugin->getName().'/';
		} else if ($this->plugin_exclusive_type) {
			$return .= $this->plugin_exclusive_type.'/';
		}
		if ($this->subdir) {
			// note: subdir already ends with /
			$return .= $this->subdir;
		}
		if ($newAction) {
			$return .= '?action='.$newAction;
			$separator = '&amp;';
		} else {
			$separator = '?';
		}
		
		if (!empty($params)) {
			foreach ($params as $param => $value) {
				if (is_array($value)) {
					foreach ($value as $array_value) {
						$return .= $separator.$param.'[]='.urlencode($array_value);
					}
				} else {
					$return .= $separator.$param.'='.urlencode($value);
				}
				$separator = '&amp;';
			}
		}
		
		return $return;
	}
	
	/**
	 * Returns the current URL with parameters added/replaced.
	 * @param	array $newparams	Parameters to add
	 * @param	array $deletedparams	Parameters to remove
	 * @param	bool $keep_feedbac	Whether to keep the feedback message (if false, it's the same
	 *								as passing 'feedback' in $deletedparams
	 */
	public function currentUrl($newparams = array(), $deletedparams=array(), $keep_feedback=false) {
		$return = '/gf/';
		if ($this->project) {
			$return .= 'project/'.$this->project->getUnixName().'/';
		}
		if ($this->user) {
			$return .= 'user/'.$this->user->getUnixName().'/';
		}
		if ($this->plugin) {
			$return .= $this->plugin->getName().'/';
		}			
		if ($this->subdir) {
			// note: subdir already ends with /
			$return .= $this->subdir;
		}
		
		$get_params = $_GET;
		if (!$keep_feedback && array_key_exists('feedback', $get_params)) {
			unset($get_params['feedback']);
		}

		$params = array_merge($get_params, $newparams);
		$return .= '?';
		
		if (array_key_exists('action', $params)) {
			$return .= 'action='.urlencode($params['action']);
			unset($params['action']);
		} else {
			$return .= 'action=';
		}
		
		foreach ($params as $param => $value) {
			if (array_search($param, $deletedparams) !== false) continue;		// do not add deleted parameters
			if(is_array($value)) {
				foreach($value as $k => $f) {
					$return .= '&amp;'.$param.'['.urlencode($k).']='.urlencode($f);
				}
			}
			else {
				$return .= '&amp;'.$param.'='.urlencode($value);
			}
		}
		
		return $return;
	}
	
	/**
	 * @return string
	 * @todo	Make this detect we're under HTTP or HTTPS
	 */
	public function buildURL($path) {
		return $path;
/*
		static $baseURL = null;
		if (!$baseURL) {
			$baseURL = GFConf::get('system.serverName');
		}

		$slashPath = preg_match('/^\\//', $path);
		if ($slashPath) {
			return 'http://'.$baseURL.$path;
		} else  {
			return 'http://'.$baseURL.'/'.$path;
		}
*/
	}

	public function projectURL($project, $actionName='', $pluginName='', $subdir = '',$params = '') {
		if ($project instanceof Project) {
			// We need the project name to build the URL
			$project = $project->getUnixName();
		}

		$url = $this->buildURL('/gf/project/'.$project.'/');		// include 'gf' script
		if (!empty($pluginName)) {
			$url .= $pluginName.'/';
		}
		if (!empty($subdir)) {
			// remove leading / from $subdir
			$subdir = preg_replace('/^\\//', '', $subdir);

			$url .= $subdir.'/';
		}
		if (!empty($actionName)) {
			$url .= '?action='.$actionName;
			$separator = '&amp;';
		} else {
			$separator = '?';
			//$url .= '?action=index';
		}

		if (!empty($params)) {
			foreach ($params as $param => $value) {
				if (is_array($value)) {
					foreach ($value as $array_value) {
						$url .= $separator.$param.'[]='.urlencode($array_value);
					}
				} else {
					$url .= $separator.$param.'='.urlencode($value);
				}
				$separator = '&amp;';
			}
		}

		return $url;
	}

	/**
	 * Builds an url to the download page.
	 *
	 * @param $section : docman, forum, etc
	 * @param $ref_id : the id of filesystem table
	 * @param $file_id : the filesytem table PK
	 * @param $file_name : file_name_safe in filesystem table
	 */
	public static function downloadURL($section,$ref_id,$file_id,$file_name) {
		return GFUrl::getURL()->baseURL('','','download/'.$section.'/'.$ref_id.'/'.$file_id.'/'.$file_name);
	}

	public static function downloadUrlFromFS(Filesystem $fs, $open_if_available=true) {
		$str_open = '';
		if ($open_if_available) {
			if ($fs->canOpenInBrowser()) {
					$str_open = '/open';
			}
		}

		$url = self::downloadUrl($fs->getSection(), $fs->getRefId(), $fs->getPrimaryKey(), StringUtils::encodeFileUrl($fs->getFilenameSafe()) ) . $str_open;
		return $url;

	}

	public static function openUrlFromFS(Filesystem $fs) {
		$str_open = '/open';
		if (!$fs->canOpenInBrowser()) {
			return '';
		}

		return self::downloadUrl($fs->getSection(), $fs->getRefId(), $fs->getPrimaryKey(), $fs->getFilenameSafe()) . $str_open;
	}
	
	public function userURL($user, $actionName='', $pluginName='', $subdir = '', $params = '') {
		if ($user instanceof User) {
			// We need the user name to build the URL
			$user = $user->getUnixName();
		}

		$url = $this->buildURL('/gf/user/'.$user.'/');		// include 'gf' script
		if (!empty($pluginName)) {
			$url .= $pluginName.'/';
		}
		if (!empty($subdir)) {
			$url .= $subdir.'/';
		}
		if (!empty($actionName)) {
			$url .= '?action='.$actionName;
		} else {
			$url.='?';
		}
		if (!empty($params)) {
			$keys = array_keys($params);
			foreach ($keys as $key) {
				$url .= "&amp;" . $key . "=" . urldecode($params[$key]);
			}
		}	
		return $url;
	}	
	
	
	/**
	 * Builds a URL relative to the site root.
	 *
	 * @param	string $actionName	Name of the action to invoke
	 * @param	string $pluginName	Name of the plugin (may be empty)
	 * @param	string $subdir		Name of subdirectory inside the plugin that contains the action (may be empty)
	 * @param array  $params		Parameters extra. the key is the name, the value is the value
	 */
	public function baseURL($actionName='', $pluginName='', $subdir='', $params = array()) {
		$url = $this->buildURL('/gf/');
		if (!empty($pluginName)) {
			$url .= $pluginName.'/';
		}
		if (!empty($subdir)) {
			// remove first / (if any)
			$subdir = preg_replace('/^\\//','',$subdir);
			$url .= $subdir;
			if (!empty($actionName) || count($params) > 0) {
				$url .= '/';
			}
		}
		$hasSeparator = false;
		if (!empty($actionName)) {
			$url .= '?action='.$actionName;
			$hasSeparator = true;
		}
		if (!empty($params)) {
			$keys = array_keys($params);
			foreach ($keys as $key) {
				if (!$hasSeparator) {
					$separator = '?';
					$hasSeparator = true;
				} else {
					$separator = '&amp;';
				}
				$url .= $separator . $key . "=" . urlencode($params[$key]);
			}
		}
		return $url;
	}

	public static function redirect($location,$params=array()) {
		$str='';
		foreach ($params as $key=>$val) {
			// Note that the parameters on the redirect *must* be separated with "&" and not "&amp;"
			// (this is different than HTML code)
			$str .= "&$key=".urlencode($val);
		}

		// Problem: the $location variable is usually generated with currentBaseUrl() and sometimes it
		// contains separators as "&amp;" instead of "&" (for XHTML compilance). We need to replace
		// them in order to properly redirect.
		$location = str_replace('&amp;', '&', $location);

		if (strpos($location, '?') === false) {
			$redirect_str = "$location?$str";
		} else {
			// already has the ?
			$redirect_str = "$location$str";
		}
		
//		die($redirect_str);
		
		header("Location: $redirect_str");
		die();
	}
	
	/**
	 * Returns 'http://' if use_ssl=false or 'https://' otherwise
	 * @return	string
	 */
	public static function getUrlPrefix() {
		if (GFConf::get('system.use_ssl')) {
			$http = 'https://';
		} else {
			$http = 'http://';
		}

		return $http;
	}
	
	/**
	 * Given a path, builds the full URL: http(s)://this.server.name/path
	 * @param	string $path
	 * @param	string $replace_amp	Whether to replace the "&amp;" with a regular ampersand. Use if
	 *								the URL is intended to be sent through an email
	 * @return	string
	 */
	public function getFullURL($path, $replace_amp = false) {
		$http = self::getUrlPrefix();
		$host = GFConf::get('system.hostName');
		
		// remove slash (if any)
		$path = preg_replace('/^\\//', '', $path);
		if ($replace_amp) {
			$path = str_replace("&amp;", "&", $path);
		}
		return $http.$host.'/'.$path;
	}
	
	/**
	 * Get the URL used for registering and de-registering notifications
	 * @param	string $section	Section to notify (i.e. 'forum')
	 * @param	int $ref_id		ID of the item to monitor
	 * @param	string $redirect_to	Where to redirect after updating the notification
	 * @param	bool $notify	Whether to enable (true) or disable (false) notifications
	 */
	public function getMonitorURL($section, $ref_id, $redirect_to=null, $notify=true) {
		if (!$redirect_to) {
			// don't include the feedback, it will be overwritten by Monitor.php
			$redirect_to = $this->currentUrl(array(), array('feedback'));
		} else {
			//remove the original param
			preg_replace('/\&.+=.+\$/', '', urldecode($redirect_to));
		}


		return $this->baseUrl('Monitor', '', '', array('section' => $section, 'ref_id' => $ref_id,
				 'notify' => ($notify ? 1 : 0), 'redirect_to' => $redirect_to));
	}
	
	/**
	 * Returns the path relative to the current handler. For example, if the URL
	 * is /some/path/x/y/z and handler.php is under /some/path, it returns /x/y/z
	 * @return	string
	 */
	public function getRelativeURI() {
		return $this->relative_uri;
	}
	
	
	/**
	 * Returns the full url passed to the server
	 *
	 * @return string
	 */
	public function getURI() {
		return $_SERVER['REQUEST_URI'];
	}
	
	/**
	 * Small hack to handle plugin urls
	 */
	private function isPluginType($pluginName) {
		if ($pluginName == 'scm') return true;
		
		return false;
	}
}
?>
