<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
/**
 * A collection of system utilities
 */
class SystemUtils {
	const COLOR_GREEN = "\033[01;32m";
	const COLOR_NORMAL = "\033[00m";
	const COLOR_RED = "\033[01;31m";

	function isAssociativeArray($array){
		if (!is_array($array)) return false;
			foreach(array_keys($array) as $key=>$value) {
				if ($key != $value) return true;
		}
		return false;
	}

	/**
	 * Given a directory and a file name, builds the full path to the
	 * filename. This avoids having file names like /etc/gforge//local.inc.
	 *
	 * @param    string $path Directory name
	 * @param    string $filename File name to be attached to $path (note that it can also be a directory name)
	 * @return string
	 */
	static function buildFilePath($path, $filename) {
		$slashPath = preg_match("/\\/\$/", $path);
		$slashFile = preg_match("/^\\//", $filename);
		if ($slashPath && $slashFile) {
			$filename = preg_replace("/^\\/*/", "", $filename);
			return $path.$filename;
		} else if ($slashPath && !$slashFile) {
			return $path.$filename;
		} else if (!$slashPath && $slashFile) {
			return $path.$filename;
		} else if (!$slashPath && !$slashFile) {
			return $path."/".$filename;
		}

		return '';
	}

	/**
	 * sendMail() - Sends a MIME email
	 *
	 * @param string $subject
	 * @param string $body
	 * @param User|User[] $to
	 * @param bool $html
	 * @param string $from
	 * @param string $reply
	 * @param array $attachment
	 * @param array $extra_headers
	 *
	 * @return boolean    True on success, false on failure
	 */
	static function sendMail($subject,$body,$to,$html=false,$from='',$reply='', $attachment = array(), $extra_headers = array()) {

		if (empty($from)) {
			// default "from" address
			$from = self::getFromAddress();
		}
		if (empty($reply)) {
			$reply = $from;
		}

		if (is_array($to)) {
			$to_all = $to;
		} elseif ($to instanceof User) {
			$to_all = array($to->getEmail());
		} else {
			// it's a single e-mail address
			$to_all = array($to);
		}

		// Sanitize subject
		$subject = str_replace("\n", '', $subject);

	 	$sendmail = self::getSendmailPath();
		//$sendmail = '/usr/sbin/sendmail';

		if (!$sendmail) {
			return false;
		}

		$j = 0;


		foreach ($to_all as $to) {

			$uid = md5(uniqid(time()));
			$contents = '';
			//$contents .= "From: ".$from . "\r\n";		// This is included in the command with the -f parameter
			$contents .= "To: ".$to . "\r\n";
			$contents .= "Reply-To: ".$reply ."\r\n";
			$contents .= "Subject: " . "=?utf-8?B?".base64_encode($subject)."?=" . "\r\n";
			foreach($extra_headers as $eh=>$h) {
				$contents.= "$eh: $h\r\n";
			}
			
			if (count($attachment) == 0) {
				//TODO: Handle HTML
				if ($html) {
					$contents .= "Content-Type: multipart/alternative; boundary=\"".$uid."\"\r\n";
					$contents .= "--".$uid."\r\n";
					$contents .= "Content-Type: text/plain; charset=UTF-8"."\r\n";
					$contents .= "Content-Transfer-Encoding: base64";
					$contents .= "\r\n\r\n";
					$contents .= base64_encode(StringUtils::getPlainUTF8FromHtml($body))."\r\n\r\n";
					$contents .= "--".$uid."\r\n";
					$contents .= "Content-Type: text/html; charset=UTF-8"."\r\n";
					$contents .= "Content-Transfer-Encoding: base64";
					$contents .= "\r\n\r\n";
					$contents .= base64_encode($body)."\r\n\r\n";
					$contents .= "--".$uid."--"."\r\n";

				} else {
					$contents .= "Content-Type: text/plain; charset=UTF-8"."\r\n";
					$contents .= "Content-Transfer-Encoding: base64";
					$contents .= "\r\n\r\n";
					$contents .= base64_encode($body);
				}
			}  else {
				// Email with attachments
				$contents .= "MIME-Version: 1.0\r\n";
				$contents .= "Content-Type: multipart/mixed; boundary=\"".$uid."\"\r\n";
				$contents .= "\r\n";
				$contents .= "This is a multi-part message in MIME format.\r\n";
				$contents .= "--".$uid."\r\n";
				
				if ($html) {
					$contents .= "Content-Type: multipart/alternative; boundary=\"alt-".$uid."\"\r\n";
					$contents .= "--alt-".$uid."\r\n";
					$contents .= "Content-Type: text/html; charset=UTF-8"."\r\n";
					$contents .= "Content-Transfer-Encoding: base64";
					$contents .= "\r\n";
					$contents .= base64_encode($body);
					$contents .= "--alt-".$uid."\r\n";
					$contents .= "Content-Type: text/plain; charset=UTF-8"."\r\n";
					$contents .= "Content-Transfer-Encoding: base64";
					$contents .= "\r\n";
					$contents .= base64_encode(StringUtils::getPlainUTF8FromHtml($body));
					$contents .= "--alt-".$uid."\r\n";
				} else {
					$contents .= "Content-Type: text/plain; charset=UTF-8"."\r\n";
					$contents .= "Content-Transfer-Encoding: base64\r\n";
					$contents .= "\r\n";
					$contents .= base64_encode($body);
					$contents .= "\r\n\r\n";					
				}

				// Include the attachments
				for ($i=0; $i < count($attachment); $i++){
					$file = $attachment[$i];
					$name = basename($file);
					$file_size = filesize($file);
					//echo $file . " " . $file_size . "\n";
					$handle = fopen($file, "r");
					if (!$handle) {
						continue;
					}
					$file_content = fread($handle, $file_size);
					fclose($handle);
					$file_content = chunk_split(base64_encode($file_content));

					$contents .= "--".$uid."\r\n";
					$type = self::getMimeType($name);
					$contents .= "Content-Type: $type; name=\"".$name."\"\r\n";
					$contents .= "Content-Transfer-Encoding: base64\r\n";
					$contents .= "Content-Disposition: attachment; filename=\"".$name."\"\r\n\r\n";
					$contents .= $file_content."\r\n\r\n";
				}
				$contents .= "--".$uid."--";
			}

			$senderName = escapeshellarg(GFConf::get('system.name'));

			// Execute
			$tmpFile = '/tmp/out_mail.txt';
			self::writeFile($tmpFile, $contents, false);


			$cmd = '/bin/cat '.$tmpFile .' | '.$sendmail.' -f'.escapeshellarg($from).' -F'.$senderName.' -t -i > /dev/null 2>&1 &';
			$output = array();
			$error = self::execute($cmd, $output);
			self::execute("rm -f $tmpFile", $output2);

			if ($error) {
				error_log(__FUNCTION__.' command failed: '.implode(' ', $output));
				return false;
			}

			$j++;
			if (($j % 15) == 0) {
				// Pause every 15 emails. This will prevent the server from hanging.
				sleep(5);
			}
		}
		
		return true;
	}
	
	/**
	 * Returns an email address with the name of the GForge server and the host name
	 */
	public static function getFromAddress($name='noreply', $forceSimple=false) {
		if (GFConf::get('system.simpleFromEmail') || $forceSimple) {
			return $name.'@'.GFConf::get('system.hostName');
		} else {
			return '"GForge" <'.$name.'@'.GFConf::get('system.hostName').'>';
		}
	}

	public static function convertToCharset($string, $charset) {
		if (!function_exists('mb_convert_encoding')) {
			return $string;
		}
		
		$res = '=?'.$charset.'?B?';
		$res .= base64_encode(mb_convert_encoding($string, $charset, 'UTF-8'));
		$res .= '?=';
		
		return $res;
	}
	
	/**
	 * Tries to guess the system's sendmail path. Returns false if sendmail not found.
	 * @return	string
	 */
	private static function getSendmailPath() {
		static $sendmail = null;
		if (is_null($sendmail)) {

			if (GFConf::varExists('system.sendmailPath')) {
				$sendmail = GFConf::get('system.sendmailPath');
			} else {
				$sendmail = trim(`which sendmail`);
				if (!$sendmail) {
					// Default value
					$sendmail = '/usr/sbin/sendmail';
				}
			}

			if (!file_exists($sendmail)) {
				$sendmail = false;
			}
		}
		
		return $sendmail;
	}

	/**
	 * @param $subject
	 * @param $body
	 * @param bool $html
	 * @param string $from
	 * @param string $reply
	 * @throws Exception
	 */
	public static function sendUserMail($subject, $body, $html=false, $from='', $reply='') {
		$session = Session::getSession();
		if (!$session->isLoggedIn()) {
			throw new Exception('Cannot send email: not logged in');
		}
		$user = $session->getUser();
		self::sendMail($subject, $body, $user, $html, $from, $reply);
	}
	
	/**
	* isRootDir() - Checks if a directory points to the root dir
	* @param	string	Directory
	* @return bool
	*/
	public static function isRootDir($dir) {
		return !preg_match('/[^\\/]/',$dir);
	}
	
	/**
	 * Execute a command
	 * @param	string $command	Command to execute
	 * @param	array &$output	Array to store the lines of the output
	 * @param	bool $explode_output the output should be stored in an array, line by line. Could cause problems with binary files.
	 * @return	int	Result of the command
	 */
	public static function execute($command, &$output, $explode_output=true) {
		$retval = 0;
		if ($explode_output) {
			exec($command, $output, $retval);
		} else {
			$output = "";
			$ph = popen($command, 'r');
			while ($buffer = fread($ph, 4096)) {
				$output .= $buffer;
			}
			pclose($ph);
			$retval = 0;		// No way to catch errors with popen :(
		}
		return $retval;		
	}
	
	/**
	 * Checks if a given group exists in the system
	 * @param	string
	 * @return	bool
	 */
	public static function groupExists($group_name) {
		$group_name = escapeshellarg($group_name);
		return (exec('getent group '.$group_name) != '');
	}
	
	/**
	 * Checks if a given user exists in the system
	 * @param	string
	 * @return	bool
	 */
	public static function userExists($user_name) {
		$user_name = escapeshellarg($user_name);
		return (exec('getent passwd '.$user_name) != '');
	}
	
	/**
	 * Returns the users's home dir
	 * @param	mixed	User
	 * @return	string
	 */
	public static function getHomedir($user) {
		if ($user instanceof User) {
			$user = $user->getUnixName();
		}
		
		return self::buildFilePath(GFConf::get('system.homeDir'), '/'.$user);
	}

	/**
	 * Initialize a user's home directory
	 * @param    mixed    User
	 * @throws PermissionDeniedException
	 */
	public static function initializeHomedir($user) {
		if ($user instanceof User) {
			$user = $user->getUnixName();
		}
		
		$homedir = self::getHomedir($user);
		if (is_dir($homedir)) return;
		
		if (!mkdir($homedir)) {
			throw new PermissionDeniedException('Couldn\'t create user\'s home directory: '.$homedir);
		}

		chown ($homedir, $user);
		chgrp ($homedir, 'users');
	}

	/**
	 * Format file size in a human-readable way
	 *
	 * @param int $bytes
	 * @param int $round
	 * @return string human readable size
	 */
	public static function getReadableSize ($bytes,$round=0) {
		$hrb='';
		if($bytes < 1024) $hrb = $bytes.' bytes';
		if($bytes >=1024 && $bytes < 1048576) $hrb = round($bytes/1024,$round).' KB';
		if($bytes >= 1048576 && $bytes < 1073741824) $hrb = round($bytes/1048576,$round).' MB';
		if($bytes >= 1073741824) $hrb = round($bytes/1073741824,$round).' GB';
		return $hrb;
	}

	/**
	 * @param string $path
	 * @return string
	 */
	public static function getDirSize($path) {
        $cmd = "du -sb {$path};";

        // Pass-by-reference is controlled by the function decl, not the caller's usage
        // http://php.net/manual/en/language.references.pass.php
        $exitCode = self::execute($cmd, $output, false);
		return trim($output, "\x00..\x2F\x3A..\xFF");
	}

	/**
	 * @param string $string
	 * @return mixed
	 */
	public static function makeUrlsClickable($string) {
		return (preg_replace( '/(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/', '<a href="\0" target="_blank">\0</a>', $string));
	}
	
	/**
	 * Writes to a file with locking
	 * @param	string $filePath
	 * @param	mixed $content	Contents of the file. Can be a string or an array with the lines
	 * @param   bool $locked if true, this function will not try to lock the file, it will only release the lock
	 */
	public static function writeFile($filePath, $content, $locked = false) {
		if (is_array($content)) {
			//Remove new lines (if any)
			$size = count($content);
			for ($i = 0; $i < $size; $i++) {
				$content[$i] = preg_replace("/\n/", "", $content[$i]);
			}
			$content = implode("\n", $content);
		}


		$file = fopen($filePath, 'w');
		if (!$locked) {
			$locked = flock($file, LOCK_EX);
		}
		if (!empty($content)) {
			fwrite($file, $content);
		}
		if ($locked) {
			flock($file, LOCK_UN);
		}
		fclose($file);
	}


	/**
	 * Returns the GForge version as a string.
	 * @return null|string
	 */
	public static function getGForgeVersion() {
		static $version = null;
		if (!is_null($version)) {
			return $version;
		}

		$libDir = GFConf::get('system.libDir');
		$versionFile = self::buildFilePath($libDir, 'VERSION');
		if (!is_file($versionFile)) {
			error_log("Hey, I did not find the VERSION file at $libDir !");
			$version = '(unknown)';
		} else {
			$version = trim(file_get_contents($versionFile));
		}

		return $version;
	}


	/**
	 * Returns the maximum upload size in bytes
	 * @param    string $val string value representing php.ini's max_upload_size
	 * @return int|string
	 */
	public static function getMaxFileSizeInBytes($val) {
		$val = trim($val);
		$last = strtolower($val[strlen($val)-1]);
		switch($last) {
			case 'g':
				$val *= 1024;

			case 'm':
				$val *= 1024;

			case 'k':
				$val *= 1024;
		}
		return $val;
	}


	/**
	 * Returns the mime type of a file base don the /etc/mime.types file
	 * @param    string $filename of the file
	 * @param    string $mimePath alternate directory
	 * @return string
	 */
	static function getMimeType($filename, $mimePath = '/etc') {
		$fileext = substr(strrchr($filename, '.'), 1);

		if (empty($fileext)) return ('application/octet-stream');

		$regex = "/^([^#]\S+)\s+.*$fileext.*$/";
		$lines = file("$mimePath/mime.types");
		foreach($lines as $line) {
			if (substr($line, 0, 1) == '#') continue; // skip comments
				$line = rtrim($line) . " ";
			if (!preg_match($regex, $line, $matches)) continue; // no match to the extension
			return ($matches[1]);
		}
		return ('application/octet-stream'); // no match at all
	}

	/**
	 * Returns the first user with site admin privileges
	 * @param object $con The Propel database connection.  If null, one will be acquired from Propel runtime.
	 * @throws Exception
	 * @return User The first user found in the database that matches the criteria.
	 */
	static function getSiteAdminUser($con = null) {
		if (!$con) $con = Propel::getConnection();
		$siteAdmin = null;

		$sql = "select user_id from site_admin order by user_id limit 1";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		if ($rs->next()) {
			require_once('lib/core/UserPeer.php');
		    $siteAdmin = UserPeer::retrieveByPK($rs->getInt(1));
		}

		if ($siteAdmin == null) {
		    throw new Exception("ERROR: No site admin user exists in this database!");
		}

		return $siteAdmin;
	}

	/**
	 * Sets the static session user to the first site admin found in the database.
	 * @param object $con The Propel database connection.  If null, one will be acquired from Propel runtime.
	 * @return bool True if the operation was successful.
	 */
	static function useSiteAdmin($con = null) {
		$session = Session::getSession();
		$session->setUser(self::getSiteAdminUser($con));
		return true;
	}


	/**
	 * Displays text on the console
	 * @param	string $text to show on the console
	 * @param	boolean $newLine
	 */
	function showOnConsole($text, $newLine = true) {
		if ($newLine) {
			$text = SystemUtils::COLOR_GREEN.$text .SystemUtils::COLOR_NORMAL."\n";
		}
		fwrite(STDOUT, $text);
	}


	/**
	 * Asks for user input y/n, given a text and a default answer
	 * @param	string $text to show on the console
	 * @param	string $default selection ('y' or 'n')
	 * @return boolean
	 */
	public static function input_Y_N($text, $default='y') {
		while (true) {
			SystemUtils::showOnConsole(SystemUtils::COLOR_RED.$text.SystemUtils::COLOR_NORMAL);
			$proceed = strtolower(trim(fgets(STDIN)));

			if (!$proceed) {
				return ($default == 'Y' || $default == 'y');
			}

			if ($proceed == 'n' || $proceed == 'no') {
				return false;
			} elseif ($proceed == 'y' || $proceed == 'yes') {
				return true;
			}
		}

		return false;
	}

	/**
	 * Asks for user input y/n, given a text and a default answer
	 * @param       string $text to show on the console
	 * @param       string $default selection ('y' or 'n')
	 * @return string
	 */
	public static function input_string($text, $default='') {
 		SystemUtils::showOnConsole(SystemUtils::COLOR_RED.$text." [$default]".SystemUtils::COLOR_NORMAL);
 		$result = strtolower(trim(fgets(STDIN)));

		if (strlen($result) == 0)
			return $default;
		else
			return $result;
	}

	/**
	 * Generates a hash from string using the best available method
	 * @param string $string
	 * @returns string
	 */
	public static function hashString($string) {
		$salt = substr(str_replace('+','.',base64_encode(md5(mt_rand(), true))),0,16);
		$rounds = mt_rand(1000,10000);

		//We prefer SHA512
		if(CRYPT_SHA512 == 1) {
			$salt = sprintf('$6$rounds=%d$%s$', $rounds, $salt);
		} else if(CRYPT_SHA256 == 1) {
			$salt = sprintf('$5$rounds=%d%$%s$', $rounds, $salt);
		} else if (CRYPT_MD5 == 1) {
			$salt = sprintf('$1$%s',$salt);
		}

        	return crypt($string, $salt);

	}

	public static function restartApache($restart='graceful') {

		$sysv_init_path = '/etc/init.d';
		$scripts = array('httpd', 'apache2', 'cswapache2');

		foreach($scripts as $script) {
			$path = $sysv_init_path.DIRECTORY_SEPARATOR.$script;
			if (file_exists($path)) {
				return self::execute("$path  $restart", $output);
			}
		}

		$sbin_paths = array('/sbin', '/usr/sbin');
		if ($restart == 'graceful') {
			//No graceful restart on systemd :(
			$cmd = 'apachectl';
			$param = $restart;
		} else {
			$cmd = 'systemctl';
			$param = 'restart httpd.service';
		}

		foreach ($sbin_paths as $sbin_path) {
			$path = $sbin_path.DIRECTORY_SEPARATOR.$cmd;
			if (file_exists($path)) {
				return self::execute("$path $param", $output);
			}

		}

		return -1;

	}

}
