<?php
/**
 * Copyright (c) 2005-2014 GForge Group, Inc
 *
 * @version
 * @author Hector Pizarro <hpizarro@gforgegroup.com>
 * @date 2014-03-25
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('core/User.php');
require_once('lib/notification/GFNotification.php');

class TrackerSprintNotification implements  GFNotification {

    const SEND_ALL = 1;
    const SEND_GROUPS_ROLES = 2;
    const SEND_INDIVIDUAL = 3;

    /**
     * @var array int $userIds
     */
    public $userIds;
    /** @var array string An array of string that contains full paths to files */
    private $attachments = array();
    /** @var array string Array of extra headers */
    private $extraHeaders = array();
    /** @var  string */
    private $message;
    /** @var  string */
    private $subject;
    /** @var int  */
    private $ref_id;
    /** @var  int */
    private $message_id;
    /** @var  int */
    private $notification_id;
    /** @var string */
    private $from;

    /**
     * @param array $userIds
     * @param Project $project
     */
    public function __construct(&$userIds, $project) {
	$this->userIds = $userIds;
	$this->ref_id = $project->getProjectId();
    }

    /**
     * @param User $user
     * @return string
     */
    function getNotificationMessage(User $user) {
	return $this->message;

    }

    /**
     * @param User $user
     * @return string
     */
    function getNotificationSubject(User $user) {
	return $this->subject;

    }

    /**
     * @throws InvalidDataException
     * @return User[] $users
     */
    public function getUsers() {
	$users = UserPeer::retrieveByPks($this->userIds);
	if (!$users) {
	    return array();
	}
	return $users;
    }

    /**
     * @param string $message
     */
    public function setNotificationMessage($message) {
	$message .="\n\n";
	if ($this->getSection() == Project::SECTION) {
	    $project = ProjectPeer::retrieveByPk($this->getRefId());
	    $projectName = $project->getProjectName();
	    $fullUrl = GFUrl::getUrl()->getFullURL('/gf/project/'.$project->getUnixName());
	    $message .= Language::getSessText('ProjectAdmin.MassEmail.DontReply', array($projectName,$fullUrl));
	    $message .= "\n";
	}
	$this->message = $message;

    }

    /**
     * @param string $subject
     */
    public function setNotificationSubject($subject) {
	$this->subject = $subject;
    }

    /**
     * @param string $file (path to file in Gforge's filesystem)
     */
    public function addAttachment($file) {
	if (file_exists($file) && is_readable($file)) {
	    $this->attachments[] = $file;
	}
    }

    /**
     * @return null
     */
    public function getMessageId(){
	return $this->message_id;
    }

    /**
     * @return null
     */
    public function getInReplyTo(){
	return null;
    }

    /**
     * @return null
     */
    public function getRefMessage(){
	return null;
    }

    /**
     * @return array
     */
    public function getAttachments() {

	$c = new Criteria();
	$c->add(FilesystemPeer::SECTION, Notification::SECTION);
	$c->add(FilesystemPeer::REF_ID, $this->getNotificationId());

	return FilesystemPeer::doSelect($c);
    }

    /**
     * @return string
     */
    public function getSection() {
	return Project::SECTION;
    }

    /**
     * @return int
     */
    public function getRefId() {
	return $this->ref_id;
    }

    /**
     * @return bool
     */
    private function hasAttachments() {
	return count($this->attachments) > 0;
    }

    /**
     * @param integer $int
     */
    public function setMessageId($int) {
	$this->message_id = $int;
    }

    public function setNotificationId($int) {
	$this->notification_id = $int;
    }

    public function getNotificationId() {
	return $this->notification_id;
    }

    public function setFrom($from) {
	$this->from = $from;
    }
}
