<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once 'core/om/BaseNotificationQueue.php';
require_once("plugin/ActionManager.php");

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class NotificationQueue extends BaseNotificationQueue {
	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see NotificationQueue::setDeleted()
	 * @see NotificationQueue::isDeleted()
	 */
	function delete($con = null) {
		ActionManager::notify("NOTIFICATION_QUEUE_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		parent::save($con);
		ActionManager::notify("NOTIFICATION_QUEUE_SAVE", $this);
	}

	
	/**
	 * Send this queue message to the recipient
	 */
	public function send() {
		$attachments = array();
		// get the data for the msg
		$body = StringUtils::unHtmlSpecialChars($this->getBody());
		$subject = StringUtils::removeLineBreaks($this->getSubject()); // the subject cannot have a line break or the mail won�t send
		$subject = StringUtils::unHtmlSpecialChars($subject);		// Items are stored with special chars like &amp;, etc

		// We add some strings so the gateway will be able to process replies
		// Forum has his own gateway so we should skip it
		$extra_headers = array();
		if($this->getSection() != ForumMessage::SECTION && $this->getSection() != Project::SECTION && $this->getSection() != TrackerItem::SECTION) {
			$subject .= ' ('.$this->getSection().'-'.$this->getRefId().')';
		}

		if($this->getSection() == ForumMessage::SECTION || $this->getSection() == TrackerItem::SECTION) {
			//Set extra headers
			if (!is_null($this->getMessageId())) {
				$extra_headers['Message-ID'] = $this->getMessageId();
			}

			if (!is_null($this->getInReplyTo())) {
				$extra_headers['In-Reply-To'] = $this->getInReplyTo();
			}

			if (!is_null($this->getRefMessage())) {
				$extra_headers['References'] = $this->getRefMessage();
			}

		}
		
		$User = $this->getUser();
		$from_address = '';
		if ($this->getSection() == TrackerItem::SECTION) {
			//Some trackers could have an alias defined
			$TrackerItem = TrackerItemPeer::retrieveByPk($this->getRefId());
			if ($TrackerItem) {
				$from_address = $TrackerItem->getTracker()->getEmailAlias();
			}
		}

		if (!$from_address || $this->getSection() == Project::SECTION) {
			$from_address = SystemUtils::getFromAddress('noreply');
		}

		if($this->getSection() == 'forummessage') {
			$reply_address = SystemUtils::getFromAddress('discuss+'.$this->getRefId());
		} else if ($this->getSection() == 'trackeritem') {
			$reply_address = SystemUtils::getFromAddress('trackeritem+'.$this->getRefId());
		} 	else if ($this->getSection() != Project::SECTION) {
			$reply_address = SystemUtils::getFromAddress('gforge-gateway');
		} else {
			$reply_address = null;
		}

		//See if there is any attachment
		if ($ref_id = $this->getNotificationId()) {
			$c = new Criteria();
			$c->add(FilesystemPeer::REF_ID, $ref_id);
			$c->add(FilesystemPeer::SECTION, Notification::SECTION);

			/** @var FileSystem[] $files */
			$files = FilesystemPeer::doSelect($c);

			foreach ($files as $file) {
				$attachments[] = $file->getFilePath();
			}
		}

		$to_address = $User->getEmail();
		
		if (!SystemUtils::sendMail($subject, $body, $to_address, false, $from_address, $reply_address, $attachments, $extra_headers)) {
			throw new Exception('Mail COULD NOT BE SENT sent to: '.$to_address);
		}

	}

}
?>
