<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('core/om/BaseProject.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('exceptions/NotLoggedInException.php');
require_once('GFPermission.interface.php');
require_once('core/PluginPeer.php');
require_once('core/ProjectPlugin.php');
require_once('core/DeletedProjectPeer.php');
require_once('plugin/ActionManager.php');
require_once('exceptions/InvalidDataException.php');
require_once('Validator.php');
require_once('Permission.php');
require_once('html/GFWidget.php');
require_once('core/Role.php');
require_once('core/RoleSetting.php');
require_once('lib/core/TroveCategory.php');
require_once('core/UserProjectRole.php');
require_once('system/GFDaemon.php');
require_once('plugins/projectrating/lib/UserProjectRating.php');

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class Project extends BaseProject implements GFPermission, GFWidget {

	const SECTION = 'project';
	const STATUS_PENDING = 0;
	const STATUS_ACTIVE = 1;
	const STATUS_HOLDING = 3;
	const STATUS_PUBLIC = true;
	const STATUS_PRIVATE = false;
	
	const PROJECT_ROLE_ADMIN = 1;
	const PROJECT_ROLE_ADMIN_NAME = 'Admin';
	const UNIX_NAME_REGEXP = '^[a-z0-9_\\-]{3,15}$';
	
	private $permission = null;
	private $ChildProjects;
	private $ParentProject;
	private $plugins;
	/** Validation errors */
	private $invalid_data = array();
	private $all_project_subproject_ids;
	private $all_project_subprojects;
	private $all_projects_in_tree;
	
	/** Used for audit trail */
	private $old_values = array();
	
	function __construct() {
	}
	

	/**
	 * isPublic - true/false
	 * @return boolean
	 */
	function isPublic() {
		return $this->getIsPublic();
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		if (GFConf::get('system.projectRegRestricted') == false) {
			return true;
		} else if ($this->isNew() && Session::getSession()->isSiteAdmin()) {
			// we can't access the Permission object if it's a new project (no project
			// ID assigned yet)
			return true;
		} else if ($this->userIsAdmin()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	public function userIsAdmin() {
		return $this->getPermission()->isAdmin();
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		if (Session::getSession()->isSiteAdmin()) {
			return true;
		}
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		if ($this->isPublic()) {
			return true;
		}
		if ($this->userIsAdmin()) {
			return true;
		}
		if (Session::getSession()->isLoggedIn() && Session::getSession()->getUser()->isProjectMember($this)) {
			return true;
		}
		return false;
	}

	public function getProjectUrl() {
		return GFUrl::getUrl()->projectURL($this);
	}

	public function getProjectLink() {
		return "<a href=\"{$this->getProjectUrl()}\">{$this->getName()}</a>";
	}

	/**
	 * Get the Permission object for the current user
	 * @return Permission
	 * @todo	Implement this
	 */
	public function getPermission() {
		if (is_null($this->permission)) {
			$this->permission = new Permission($this);
		}
		return $this->permission;
	}
	
	/**
	 * Purge the permissions for this project. This enforces the
	 * permissions are re-read from the database. Use this when 
	 * you change the permissions.
	 */
	public function purgePermissions() {
		$this->permission = null;
	}

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see Project::setDeleted()
	 * @see Project::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete Project object: permission denied");
		}
		//delete trove
		$crit=new Criteria();
		$crit->add(TroveLinkPeer::SECTION,self::SECTION);
		$crit->add(TroveLinkPeer::REF_ID,$this->getProjectId());
		TroveLinkPeer::doDelete($crit);

		//remove us as parent of all child projects
		$cps = $this->getChildProjects();
		foreach ($cps as $cp) {
			$cp->removeParentProject();
		}
		ActionManager::notify("PROJECT_DELETE", $this);
		GFDaemon::notify("DAEMON_PROJECT_DELETE", $this->getUnixName());
		parent::delete($con);
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @throws InvalidDataException
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create Project object: permission denied");
		}  elseif ( (!$this->isNew()) && (!$this->userCanUpdate()) ) {
			throw new PermissionDeniedException("Could not update Project object: permission denied");
		}
		if (ActionManager::notify('CHECK_INTEGRITY',array()) != utf8_encode('6.0')) throw new Exception('Invalid version number detected [2]. Please contact GForge Group.');
		// Validate all the data
		$missing_data = array();
		if (is_null($this->project_name) || empty($this->project_name)) {
			$missing_data['project_name'] = Language::getSessText('Project.EmptyProjectName');
		}
		if (is_null($this->unix_name) || empty($this->unix_name)) {
			$missing_data['unix_name'] = Language::getSessText('Project.EmptyUnixName');
		}
		if (is_null($this->description) || empty($this->description)) {
			$missing_data['description'] = Language::getSessText('Project.EmptyDescription');
		}
		
		// Note that the invalid data has precedence over the missing data
		// (because invalid fields aren't set)
		$errors = array_merge($missing_data, $this->invalid_data);
		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}
		
		$modified_columns = $this->modifiedColumns;
		$createRoles = $this->isNew();
		
			
		// Wrap the process in a transaction
		$conn = Propel::getConnection(ProjectPeer::DATABASE_NAME);
		$conn->begin();
		
		$isNew = $this->isNew();
		
		// Now, save all the data
		ActionManager::notify("PROJECT_SAVE_BEFORE", $this);
		parent::save($con);
		// Now create the default role for this project, with the 
		// current user as an 	 
		// administrator. The other roles will be copied from the 
		// template project 
		// when this project is approved. We must create this role now,
		// or otherwise
		// we won't be able to link the current user to the project
		if ($createRoles) { 
			$role = new Role(); 	 
			$role->setIsNewProject();
			$role->setRoleName(self::PROJECT_ROLE_ADMIN_NAME);
			$role->setProject($this);
		
			$rs = new RoleSetting(); 	 
			$rs->setSection('projectadmin'); 	 
			$rs->setRefId(0); 	 
			$rs->setValue(self::PROJECT_ROLE_ADMIN); 	
			$role->addRoleSetting($rs); 	 
			
			$up = new UserProject(); 	 
			$up->setUser(Session::getSession()->getUser());
			$up->setProject($this);
			$up->setIsNewProject();
			
			$role->save();
			$rs->save();
			// Note: this can throw an exception if the license 
			// is invalid or if the
			// maximum user limit has been reached
			$up->save();
	
			$upr = new UserProjectRole();
			$upr->setUserProject($up);
			$upr->setRole($role);
			$upr->setIsNewProject(); 
			$upr->save();
		}


		//If it's a new project and the user is site admin, 
		//approve project
		/*
		if (Session::getSession()->isSiteAdmin() && $isNew){
			$this->approve();
		}*/

		// Update template trove linking
		if ($this->getIsTemplate()) {
			$this->addDefaultTemplateTroveLink();
		} else {
			$this->removeTemplateTroveLinks();
		}
		
		$conn->commit();
		
		$this->doAuditTrail($modified_columns);

		// Set to pending status and send email
		//$this->setStatus(Project::STATUS_PENDING);
		if ($isNew && ($this->getStatus() == self::STATUS_PENDING) ) {
			$this->sendPendingEmail();
		}
		ActionManager::notify("PROJECT_SAVE", $this);
		
	}
	
	/**
	 * Adds link between this project and default template container trove,
	 * as long as a link between this project and some descendant of default template container trove
	 * does not exists.
	 * 
	 * @return int number of affected rows
	 */
	private function addDefaultTemplateTroveLink() {
		$gf_proj_templates_trove = GFConf::get('system.troveGfProjTemplates', false);
		if ($gf_proj_templates_trove) {
			$con = Propel::getConnection();
			$sql = "INSERT INTO trove_link (section, ref_id, trove_category_id) "
					. "SELECT 'project', {$this->getProjectId()}, ".$gf_proj_templates_trove." WHERE "
					. "NOT EXISTS("
						. "SELECT trove_category_id FROM trove_link WHERE "
						. "trove_category_id IN (select trove_category_id FROM trove_children(".$gf_proj_templates_trove.")) "
						. "AND section='project' AND ref_id={$this->getProjectId()}"
					. ")";
			$stmt = $con->prepareStatement($sql);
			return $stmt->executeUpdate();
		} else {
			return 0;
		}
	}
	
	/**
	 * Removes links between this project and any trove cat under in the 
	 * template containers subtree.
	 * 
	 * @return int number of affected rows
	 */
	private function removeTemplateTroveLinks() {
		$con = Propel::getConnection();
		$sql = "DELETE FROM trove_link "
				. "WHERE section='project' AND ref_id={$this->getProjectId()} AND trove_category_id IN "
				. "(SELECT trove_category_id FROM trove_children(".GFConf::get('system.troveGfProjTemplates')."))";
		$stmt = $con->prepareStatement($sql);
		return $stmt->executeUpdate();		
	}	
	
	public function setProjectName($project_name) {
		if (strlen($project_name) > 40) {
			$this->invalid_data['project_name'] = Language::getSessText('Project.ProjectNameTooLong');
			return;
		}
		
		$this->old_values['project_name'] = $this->getProjectName();
		
		// so far so good
		parent::setProjectName($project_name);
		unset($this->invalid_data['project_name']);
	}

	public function setUnixName($unix_name) {
		$unix_name=strtolower($unix_name);
		if (empty($unix_name)) {
			$this->invalid_data['unix_name'] = Language::getSessText('Project.EmptyUnixName');
			return;
		}

		if (!ereg(Project::UNIX_NAME_REGEXP, $unix_name)) {
			$this->invalid_data['unix_name'] = Language::getSessText('Project.InvalidUnixName');
			return;
		}

		if (is_numeric($unix_name)) {
			$this->invalid_data['unix_name'] = Language::getSessText('Project.NumericUnixName');
			return;
		}
		
		if ($this->isNew() || ($this->getUnixName() != $unix_name)) {
			if (SystemUtils::groupExists($unix_name)) {
				$this->invalid_data['unix_name'] = Language::getSessText('Project.ProjectExists', array($unix_name));
				return;
			}

			// check for already existing project name
			if (is_object(ProjectPeer::retrieveByUnixName($unix_name, false))) {
				$this->invalid_data['unix_name'] = Language::getSessText('Project.ProjectExists', array($unix_name));
				return;
			}
		}
		
		// see if the project existed and was deleted
		if (is_object(DeletedProjectPeer::retrieveByPk($unix_name))) {
			$this->invalid_data['unix_name'] = Language::getSessText('Project.ProjectExists', array($unix_name));
			return;
		}

		// so far so good
		parent::setUnixName($unix_name);
		unset($this->invalid_data['unix_name']);
	}
	
	public function isInvalid($field) {
		return array_key_exists($field, $this->invalid_data);
	}

	public function sendPendingEmail() {
		$session = Session::getSession();
		$language = $session->getLanguage();
		$user = $session->getUser();
		$url = GFUrl::getUrl();
		
		$project_name = StringUtils::unHtmlSpecialChars($this->getProjectName());

		$subject = $language->getText('Project.PendingProjectSubject', array($project_name));
		$body = $language->getText('Project.PendingProjectText', array($project_name));
		SystemUtils::sendUserMail($subject, $body);
		
		// Email system administrators
		$admins = UserPeer::getSystemAdministrators();
		foreach ($admins as $admin) {
			$language = $admin->getLanguage();
			$subject = $language->getText('Project.PendingProjectSubject', array($project_name));
			$body = $language->getText('Project.PendingProjectTextAdmin', array($project_name, $user->getUnixName(), 
				$url->getFullUrl($url->baseUrl('approve-pending', '', 'admin'))));
			
			SystemUtils::sendMail($subject, $body, $admin);
		}
	}


	public function sendApprovedEmail() {
		$users = $this->getUserProjects();
		$link = GFUrl::getUrl()->projectURL($this);
		$project_name = StringUtils::unHtmlSpecialChars($this->getProjectName());
		foreach ($users as $user_project) {
			$user = $user_project->getUser();
			$lang = $user->getLanguage();
			$subject = $lang->getText('Project.ApprovedProjectSubject', array($project_name));
			$body = $lang->getText('Project.ApprovedProjectText', array($project_name, GFUrl::getFullUrl($link)));
			SystemUtils::sendMail($subject, $body, $user);
		}
	}
	
	
	/**
	 * Returns true if Project state is Active
	 *
	 * @return boolean
	 */
	function isActive () {
		if ($this->getStatus() == Project::STATUS_ACTIVE) { 
			return true;
		} else {
			return false;
		}
	}
	
	public function usesPlugin($name) {
		return array_key_exists($name, $this->getPlugins());
	}

	public function getPlugins() {
		if (!isset($this->plugins)) {
			$this->plugins = array();

			$crit = new Criteria();
			$crit->add(ProjectPluginPeer::PROJECT_ID,$this->getProjectId());
			$plugins = ProjectPluginPeer::doSelectJoinPlugin($crit);
			$pf = PluginFactory::getFactory();
			foreach ($plugins as $plugin) {
				$this->plugins[$plugin->getPlugin()->getPluginName()]=$pf->getPluginByName($plugin->getPlugin()->getPluginName());
			}
		}
		return $this->plugins;
	}

	public function getPluginByName($name) {
		$plugins = $this->getPlugins();
		if (array_key_exists($name, $plugins)) {
			return $plugins[$name];
		} else {
			return null;
		}
	}
	
	/**
	 * @param	mixed $plugin	Either the plugin name or the Plugin object
	 * @todo	If $plugin has exclusive_type, make sure to disable other plugins with the same exclusive_type
	 */
	public function enablePlugin($plugin) {
		if ($plugin instanceof Plugin) {
			$pluginName = $plugin->getPluginName();
		} else {
			$pluginName = $plugin;
			$c = new Criteria();
			$c->add(PluginPeer::PLUGIN_NAME, $pluginName);
			$plugin = PluginPeer::doSelectOne($c);
			if (is_null($plugin)) return;
		}

		if ($this->usesPlugin($pluginName)) {
			return;
		}
		
		$pp = new ProjectPlugin();
		$pp->setProject($this);
		$pp->setPlugin($plugin);
		$pp->save();
		
		// Store the GFPlugin object
		$pf = PluginFactory::getFactory();
		$gfplugin = $pf->getPluginByName($pluginName);
		$this->plugins[$pluginName] = $gfplugin;
		
		$gfplugin->enableInProject($this);
	}
	
	/**
	 * Whether this project uses a plugin of the exclusive_type
	 * @param	string $exclusive_type	Name of the exclusive_type (i.e. 'scm')
	 * @return	bool
	 */
	public function hasPluginType($exclusive_type) {
		$plugin = $this->getExclusiveTypePlugin($exclusive_type);
		return (!is_null($plugin));
	}
	
	/**
	 * Returns the plugin associated to this project whose exclusive_type is set by the
	 * parameter
	 * @param	string $exclusive_type	Exclusive type of the plugin (i.e. 'scm')
	 * @return GFPlugin
	 */
	public function getExclusiveTypePlugin($exclusive_type) {
		$c = new Criteria();
		$c->add(ProjectPluginPeer::PROJECT_ID, $this->getProjectId());
		$c->add(PluginPeer::EXCLUSIVE_TYPE, $exclusive_type);
		$p = ProjectPluginPeer::doSelectJoinPlugin($c);
		
		if (count($p) != 1) {		// should be EXACTLY 0 or 1 (that's what exclusive_type is all about)
			return null;
		}
		
		return $this->getPluginByName($p[0]->getPlugin()->getPluginName());
	}
	
	public function setPluginOrder($plugin, $order) {
		if (!($plugin instanceof Plugin)) {
			$c = new Criteria();
			$c->add(PluginPeer::PLUGIN_NAME, $plugin);
			$plugin = PluginPeer::doSelectOne($c);
			if (is_null($plugin)) return;
		}
		
		$plugin_id = $plugin->getPluginId();
		$pp = ProjectPluginPeer::retrieveByPK($this->getProjectId(), $plugin_id);
		if (!$pp) return;
		
		$pp->setPluginOrder($order);
		$pp->save();
	}
	
	/**
	 * @param	mixed $plugin	Either the plugin name or the Plugin object
	 */
	public function disablePlugin($plugin) {
		if ($plugin instanceof Plugin) {
			$pluginName = $plugin->getPluginName();
		} else {
			$pluginName = $plugin;
			$c = new Criteria();
			$c->add(PluginPeer::PLUGIN_NAME, $pluginName);
			$plugin = PluginPeer::doSelectOne($c);
			if (is_null($plugin)) return;
		}

		if (!$this->usesPlugin($pluginName)) {
			return;
		}
		
        $c = new Criteria();
        $c->add(ProjectPluginPeer::PROJECT_ID, $this->getProjectId());
        $c->add(ProjectPluginPeer::PLUGIN_ID, $plugin->getPluginId());
        $pp = ProjectPluginPeer::doSelectOne($c);

		if (is_null($pp)) {
			// weird, shouldn't happen
			return;
		}

		// this retrieves the GFPlugin object associated to the plugin
		$gfplugin = $this->getPluginByName($pluginName);
		// inform the plugin that we're going to disable it
		$gfplugin->disableFromProject();

		$pp->delete();
		
		unset($this->plugins[$pluginName]);
	}
	
	/**
	 * Returns the string representation of the status id passed as parameter
	 *
	 * @param int $status
	 */
	public static function getStatusName($status) {
		switch ($status) {
			case Project::STATUS_ACTIVE : {
				return Language::getSessText('Project.Active');
			}
			case Project::STATUS_PENDING : {
				return Language::getSessText('Project.Pending');
			}
			case Project::STATUS_HOLDING :  {
				return Language::getSessText('Project.Holding');
			}
		}
	}
	
	public function setStatus($status) {
		if ($status == self::STATUS_HOLDING) {
			if ($this->getStatus() != self::STATUS_ACTIVE) {
				throw new Exception('Can only set to holding after already active');
			}
		} elseif ($status == self::STATUS_PENDING) {
			if ($this->getStatus() == self::STATUS_ACTIVE || $this->getStatus() == self::STATUS_HOLDING) {
				throw new Exception('Can not go back to pending after already being approved or holding');
			}
		}
		
		$this->old_values['status'] = $this->getStatus();

		parent::setStatus($status);
	}

	/**
	 * Approves this project
	 */
	public function approve($ignore = false) {
    if (!$ignore && !$this->userIsAdmin()) {
			throw new PermissionDeniedException(Language::getSessText('Project.CannotApprovePermissionDenied'));
		}
		
		if ($this->getStatus() == self::STATUS_ACTIVE || $this->getStatus() == self::STATUS_HOLDING) {
			// Don't approve project more than once, it could lead to duplicate data
			return;
		}
		
		$con = Propel::getConnection(ProjectPeer::DATABASE_NAME);
		$con->begin();
		
		// Notify the plugins that they should copy the template project's data		
		$template_project = $this->getTemplateProject();

		$this->setStatus(self::STATUS_ACTIVE);
		$this->setIsPublic($template_project->getIsPublic());
		$this->save();
		

		// create the roles from the template project
		$all_settings = array();
		if (is_object($template_project)) {
			$roles = $template_project->getRoles();
			$i = 0;
			$role_mapping = array();

			//$con = Propel::getConnection(ProjectPeer::DATABASE_NAME);
			foreach ($roles as $role) {
				if (strtolower($role->getRoleName()) == strtolower(self::PROJECT_ROLE_ADMIN_NAME)) {
					// this role was created when we created the project, so we
					// don't have to create it again
					
					$c = new Criteria();
					$c->add(RolePeer::ROLE_NAME, self::PROJECT_ROLE_ADMIN_NAME);
					$c->add(RolePeer::PROJECT_ID, $this->getProjectId());
					$new_role = RolePeer::doSelectOne($c);

					// We shouldn't clone the "projectadmin" setting since it's already there
					$admin_condition = "AND section <> 'projectadmin'";

					if (!is_object($new_role)) {
						// Shouldn't happen
						$new_role = $role->copy();
						$new_role->setProjectId($this->getProjectId());
						$new_role->save();
						$admin_condition = "";
					}
				} else {
					//create the role
					$new_role = $role->copy();
					$new_role->setProjectId($this->getProjectId());
					$new_role->save();
					$admin_condition = "";
				}
				
				$new_key = $new_role->getPrimaryKey();
				$old_key = $role->getPrimaryKey();
				$role_mapping[$old_key] = $new_key;
				// insert the general settings
				$sql = "insert into role_setting (role_id,section,ref_id,value) 
						(select '$new_key',section,ref_id,value from role_setting where role_id = '$old_key' and ref_id='0' ".$admin_condition.")";
				$stmt = $con->createStatement();
				$rs = $stmt->executeQuery($sql);
				// now get the non-general settings and save the resultset in an array for later processing (less memory consuming than saving objects)
				$sql = "select * from role_setting where role_id = '$old_key' and ref_id != '0'";
				$stmt = $con->createStatement();
				$rs = $stmt->executeQuery($sql,ResultSet::FETCHMODE_ASSOC);
				while ($rs->next()) {
					$all_settings[$i]['role_id'] = $new_key;
					$all_settings[$i]['section'] = $rs->getString('section');
					$all_settings[$i]['ref_id'] = $rs->getInt('ref_id');
					$all_settings[$i]['value'] = $rs->getInt('value');
					$i++;
				}
			}
		}
		unset($roles);
		unset($settings);
		
		$new_project = $this->getPrimaryKey();
		$old_project = $template_project->getPrimaryKey();
		
		// Purge the project_plugin table (shouldn't have any entries for this project)
		$sql = "DELETE FROM project_plugin WHERE project_id=".$new_project;
		$stmt = $con->createStatement();
		$stmt->executeQuery($sql);
		
		// add the same plugins to the project
		$sql = "insert into project_plugin (project_id,plugin_id,plugin_order) (select '$new_project',plugin_id,plugin_order from project_plugin where project_id = '$old_project')";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql);
		
		// Notify the user
		$this->sendApprovedEmail();
		
		$con->commit();
		
		// RMT  This needed to be moved after the $con->commit() call,
		// or else the role id's and other records would not be available
		// to plugins running in their own xaction.
		$params = array(
					"project" => $this,
					"template_project" => $template_project,
					"role_settings" => $all_settings,
					"role_mapping" => $role_mapping
				);
		ActionManager::notify("PROJECT_APPROVE", $params);
		// Notify the daemon that the project was approved so it can perform additional initialization (SCM repositories,
		// mailing lists, etc)
		if (GFConf::get('system.useDaemon')) {
			GFDaemon::notify('DAEMON_PROJECT_APPROVE', $this->getUnixName());
		}
		
		return true;
	}
	
	/**
	 * Get the template project for this project
	 * @return	Project
	 */
	public function getTemplateProject() {
		$project_id = $this->getTemplateProjectId();
		if (!$project_id) {
			return null;
		}
		return ProjectPeer::retrieveByPk($project_id);
	}
	
	/**
	 * Returns the projects that use this project as template
	 * @return array
	 */
	public function getDependingProjects() {
		if (!$this->getIsTemplate()) {
			return array();
		}
		
		$c = new Criteria();
		$c->add(ProjectPeer::TEMPLATE_PROJECT_ID, $this->getProjectId());
		return ProjectPeer::doSelect($c);
	}

	/**
	 * Implements GFWidget::getName()
	 * @return	string
	 */
	public function getName() {
		return $this->getProjectName();
	}
	
	/**
	 * Implements GFWidget::getId()
	 * @return	int
	 */
	public function getId() {
		return $this->getProjectId();
	}
		
	/**
	 * 
	 * remove the project
	 *
	 */
	public function remove() {
		// TODO
		$this->delete();
		return true;
	}

	private function getUserList($sqlTemplate, $orderByName = false) {

		$rootPath = $this->getRootPath();
		if (!empty($rootPath)) {
			$project_ids = " up.project_id IN (".$rootPath.") ";
		} else {
			$project_ids = " up.project_id='".$this->getProjectId()."' ";
		}

		$sql = str_ireplace('$project_ids', $project_ids, $sqlTemplate);

		if ($orderByName) {
			$sql = '(' . $sql . ') ORDER BY firstname, lastname';
		}

		$con = Propel::getConnection(ProjectPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		$members = UserPeer::populateObjects($rs);

		return $members;
	}

	/**
	 * Returns the list of groups associated to this project.  Does not return users.
	 * @param bool $bolOrderByName If true, order the returned records by firstname, then lastname.  Otherwise, no specific order is enforced.
	 * @return	array	Array of users in this project
	 */
	public function getMemberGroups($bolOrderByName = false) {

		$sqlTemplate = 'SELECT u.* FROM "user" AS u WHERE u.user_id IN (
							SELECT DISTINCT up.user_id FROM user_project AS up
							WHERE $project_ids
						) AND u.is_group=true';

		return $this->getUserList($sqlTemplate, $bolOrderByName);
	}
		
	/**
	 * Retrieves a list of users belonging to this project. Returns the users in parent projects
	 * and users inside user groups  (doesn't return the group objects)
	 * @param bool $bolOrderByName If true, order the returned records by firstname, then lastname.  Otherwise, no specific order is enforced.
	 * @return	array	Array of users in this project
	 */
	public function getMembers($bolOrderByName = false) {

		$sqlTemplate = 'SELECT DISTINCT u.* FROM "user" AS u WHERE u.user_id IN (
								SELECT u2.user_id FROM user_project AS up, "user" as u2
								WHERE $project_ids AND u2.user_id=up.user_id AND
								u2.is_group=false
							UNION
								SELECT ug.member_user_id AS user_id
								FROM usergroup_user AS ug, user_project AS up
								WHERE $project_ids AND up.user_id=ug.usergroup_id
						)';
		
		return $this->getUserList($sqlTemplate, $bolOrderByName);
	}

	/**
	 * Retrieves a list of users and groups belonging to this project. Returns the users in parent projects
	 * and users inside user groups
	 * @param bool $bolOrderByName If true, order the returned records by firstname, then lastname.  Otherwise, no specific order is enforced.
	 * @return	array	Array of users in this project
	 */
	public function getMembersAndGroups($bolOrderByName = false) {

		$sqlTemplate = 'SELECT DISTINCT u.* FROM "user" AS u WHERE u.user_id IN (
								SELECT u2.user_id FROM user_project AS up, "user" as u2
								WHERE $project_ids AND u2.user_id=up.user_id
							UNION
								SELECT ug.member_user_id AS user_id
								FROM usergroup_user AS ug, user_project AS up
								WHERE $project_ids AND up.user_id=ug.usergroup_id
						)';

		return $this->getUserList($sqlTemplate, $bolOrderByName);
	}

	/**
	 * Get an array with project administrators
	 */
	public function getProjectAdministrators() {
		$projectId = $this->getProjectId();
		$sql = "SELECT u.*
FROM user_project up, user_project_role upr, \"user\" u, role_setting rs
WHERE up.user_project_id = upr.user_project_id
AND u.user_id = up.user_id
AND up.project_id = $projectId
AND upr.role_id = rs.role_id
AND rs.section = 'projectadmin'
AND rs.ref_id=0
AND rs.value='1'";
		$con = Propel::getConnection(ProjectPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql,ResultSet::FETCHMODE_NUM);

		$projectAdmins = UserPeer::populateObjects($rs);

		return $projectAdmins;
	}


	/**
	 * @return Filesystem
	 */
	public function getHomepage() {
		// Get the home page contents from the filesystem
		$c = new Criteria();
		$c->add(FilesystemPeer::SECTION, self::SECTION);
		$c->add(FilesystemPeer::REF_ID, $this->getProjectId());
		$c->add(FileSystemPeer::FILE_NAME, 'homepage');
		$homepage = FilesystemPeer::doSelectOne($c);
		
		if (is_null($homepage)) {
			$homepage = new Filesystem();
			$homepage->setPostedBy(User::UNDEFINED_USER_ID);
			$homepage->setRefId($this->getProjectId());
			$homepage->setSection(self::SECTION);
			$homepage->setFileName('homepage');
			$homepage->setFileType('text');
			$homepage->save();
			// Default text: just the news
			$homepage->setFileDataFromString('%%NEWS%%<p>%%ACTIVITY%%</p>');
		}
		
		return $homepage;
	}
	
	public function getUserPermission(User $user) {
		return new Permission($this, $user);
	}

	public function getParentProject() {
		if ($this->getParentProjectId() > 0) {
			if (isset($this->ParentProject)) {
				return $this->ParentProject;
			} else {
				$this->ParentProject = ProjectPeer::retrieveByPk($this->getParentProjectId());
				return $this->ParentProject;
			}
		} else {
			return null;
		}
	}

	public function removeParentProject() {
		//if the parent project has no other children and no parent,
		//remove the rootPath and RootParentProjectId
		$parent = $this->getParentProject();
		if ($parent->getParentProjectId() > 0 && $parent->getParentProjectId() != $parent->getProjectId()) {
			//has a parent, so don't mess with it
		} else {
			if (count($parent->getChildProjects()) > 1) { //we are still a child
				//has other children
			} else {
				$parent->setParentProjectId(0);
				$parent->setRootPath('');
				$parent->setRootParentProjectId(0);
				$parent->save();
			}
		}
		$this->setParentProjectId(0);
		
		$cps=$this->getChildProjects();
		if (count($cps)) {
			$this->setRootParentProjectId($this->getProjectId());
			$this->setRootPath($this->getProjectId());
		} else {
			$this->setRootParentProjectId(0);
			$this->setRootPath('');
		}
		$this->save();
		foreach ($cps as $cp) {
			$cp->setParentProject($this);
			$cp->save();
		}
	}

	public function setParentProject(Project $Project) {
		if ($Project->getProjectId() == $this->getProjectId()) {
			throw new InvalidDataException(array('Parent Project Cannot be Same as child project'));
		}
		$this->ParentProject=$Project;
		//the parent may need to have it's parentprojectid set
		if ($Project->getRootParentProjectId() < 1) {
			$Project->setRootParentProjectId($Project->getProjectId());
			$Project->setRootPath($Project->getProjectId());
			$Project->setParentProjectId(0);
		}
		$Project->save();
		$cps=$this->getChildProjects();
		//check for circular dependency
		$arr = explode(',',$Project->getRootPath());
		if (array_search($this->getProjectId(),$arr) !== false) {
			throw new InvalidDataException(array('Circular Dependency Detected'));
		}
		$this->setRootPath($Project->getRootPath().','.$this->getProjectId());
		$this->setRootParentProjectId($Project->getRootParentProjectId());
		$this->setParentProjectId($Project->getProjectId());
		$this->save();
		foreach ($cps as $cp) {
			$cp->setParentProject($this);
			$cp->save();
		}
	}

	/**
	 * Returns all the immediate child projects (1 level below)
	 *
	 * @return Project[]
	 */
	public function getChildProjects() {
		if (isset($this->ChildProjects)) {
			reset($this->ChildProjects);
			return $this->ChildProjects;
		} else {
			$crit=new Criteria();
			$crit->add(ProjectPeer::PARENT_PROJECT_ID,$this->getProjectId());
			$this->ChildProjects=ProjectPeer::doSelect($crit);
			return $this->ChildProjects;
		}

	}
	
	/**
	 * Returns all the child projects and their own child projects (all levels below)
	 *
	 * @return Project[]
	 */
	public function getAllSubprojects() {
		if (!isset($this->all_project_subprojects)) {
			$crit=new Criteria();
			$crit->add(ProjectPeer::ROOT_PATH, NULL, Criteria::NOT_EQUAL);
			$crit->addAnd(ProjectPeer::ROOT_PATH, $this->getRootPath().',%', Criteria::LIKE);
			$crit->add(ProjectPeer::PROJECT_ID, $this->getProjectId(), Criteria::NOT_EQUAL);
			$projects = ProjectPeer::doSelect($crit);
			if (count($projects) > 0) {
				$this->all_project_subprojects = $projects;
			} else {
				$this->all_project_subprojects = array();
			}
		}
		return $this->all_project_subprojects;
	}
	
	/**
	 * Returns the Ids of all the child projects and their own child projects (all levels below)
	 *
	 * @return int[]
	 */
	public function getAllSubprojectIds() {
		if (!isset($this->all_project_subproject_ids)) {
			$projects=$this->getAllSubprojects();
			if (count($projects) > 0) {
				$arr=array();
				foreach ($projects as $project) {
					$arr[$project->getProjectId()]=$project->getProjectId();
				}
				$this->all_project_subproject_ids=$arr;
			} else {
				$this->all_project_subproject_ids=array();
			}
		}
		return $this->all_project_subproject_ids;
	}

	/**
	 * Returns all the projects in the current Project Tree
	 *
	 * @return Project[]
	 */
	public function getAllProjectsInTree() {
		if (!isset($this->all_projects_in_tree)) {
			$crit=new Criteria();
			$crit->add(ProjectPeer::ROOT_PARENT_PROJECT_ID,$this->getRootParentProjectId());
			$projects = ProjectPeer::doSelect($crit);
			if (count($projects) > 0) {
				$this->all_projects_in_tree = $projects;
			} else {
				$this->all_projects_in_tree = array();
			}
		}
		return $this->all_projects_in_tree;
	}
	
	/**
	 * Whether a use has a certain role on the project.
	 * @return bool
	 */
	public function userHasRole(User $user, Role $role) {
		$sql = "SELECT COUNT(*) AS cnt
				FROM user_project_role AS upr, user_project AS up, project as p
				WHERE upr.role_id=".$role->getRoleId()." AND upr.user_project_id=up.user_project_id
				AND p.project_id=".$this->project_id. "
				AND (up.user_id=".$user->getUserId()." OR
					up.user_id IN (SELECT usergroup_id FROM usergroup_user WHERE member_user_id=".$user->getUserId()."))";

		$con = Propel::getConnection(ProjectPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql);
		$rs->next();

		if ($rs->getInt('cnt') == 0) {
			// This means neither the user or its groups have the corresponding role
			return false;
		} else {
			return true;
		}
	}
	
	public function setDescription($description) {
		$this->old_values['description'] = $this->getDescription();
		
		$strUtil = new StringUtils(StringUtils::BASIC_HTML);
		$description = $strUtil->cleanHtml($description);
		parent::setDescription($description);
	}

	public function setHomepageUrl($url) {
		$this->old_values['homepage_url'] = $this->getHomepageUrl();
		
		parent::setHomepageUrl($url);
	}
	
	public function setIsPublic($is_public) {
		$this->old_values['is_public'] = ($this->getIsPublic() ? 1 : 0);
		
		parent::setIsPublic($is_public);
	}
	
	private function doAuditTrail($modified_columns) {
		if (!$this->isNew()) {
			foreach ($modified_columns as $modified_column) {
				$audit = new AuditTrail();
				$audit->setUser(Session::getSession()->getUser());
				$audit->setSection('project');
				$audit->setRefId($this->getProjectId());
				
				if ($modified_column == ProjectPeer::HOMEPAGE_URL && isset($this->old_values['homepage_url'])) {
					$audit->setFieldName('homepage_url');
					$audit->setNewValue($this->getHomepageUrl());
					$audit->setOldValue($this->old_values['homepage_url']);
					$audit->save();
				} elseif ($modified_column == ProjectPeer::PROJECT_NAME && isset($this->old_values['project_name'])) {
					$audit->setFieldName('project_name');
					$audit->setNewValue($this->getProjectName());
					$audit->setOldValue($this->old_values['project_name']);
					$audit->save();
				} elseif ($modified_column == ProjectPeer::STATUS && isset($this->old_values['status'])) {
					if ($this->old_values['status'] == $this->getStatus()) {
						// For some reason Propel marks this field as changed even if it's not
						continue;
					}
					$audit->setFieldName('status');
					$audit->setNewValue($this->getStatus());
					$audit->setOldValue($this->old_values['status']);
					$audit->save();
				} elseif ($modified_column == ProjectPeer::IS_PUBLIC && isset($this->old_values['is_public'])) {
					if ($this->old_values['is_public'] == $this->getIsPublic()) {
						// For some reason Propel marks this field as changed even if it's not
						continue;
					}
					$audit->setFieldName('is_public');
					$audit->setNewValue(($this->getIsPublic() ? 1 : 0));
					$audit->setOldValue($this->old_values['is_public']);
					$audit->save();
				} elseif ($modified_column == ProjectPeer::DESCRIPTION && isset($this->old_values['description'])) {
					$audit->setFieldName('description');
					$audit->setNewValue($this->getDescription());
					$audit->setOldValue($this->old_values['description']);
					$audit->save();
				}
			}
		}
	}
	
	/**
	 * Returns an array mapping this project's roles to the other project's roles
	 * matching the names (used when working with template projects)
	 */
	public function getRoleMapping(Project $otherproject) {
		if (!$this->getIsTemplate()) {
			return array();
		}
		
		$mapping = array();
		$this_roles = $this->getRoles();
		$other_roles = $otherproject->getRoles();
		
		foreach ($this_roles as $this_role) {
			foreach ($other_roles as $other_role) {
				if ($this_role->getRoleName() == $other_role->getRoleName()) {
					$mapping[$this_role->getRoleId()] = $other_role->getRoleId();
				}
			}
		}
		
		return $mapping;
	}

	public function getEnableWorkflowScripts() {
		return (GFConf::get('tracker.enableWorkflowScripts') && parent::getEnableWorkflowScripts());
	
	}

	/**
	 * Project rating
	 * @param	int		Rating, goes from 1 (worst) to 5 (best)
	 * @param	string	Comment
	 * @param	User	User doing the rating. If null, the session user is used
	 */
	public function setProjectRating($rating, $comment, $user = null) {
		// Must be logged in to vote
		if (is_null($user)) {
			$user = Session::getSession()->getUser();
			if (!$user) {
				throw new NotLoggedInException();
			}
		}
		
		// Only project members can vote
		// NOTE: This restriction has been removed by request (see ticket #8908)
		/*if (!$user->isProjectMember($this)) {
			throw new PermissionDeniedException();
		}*/

		if ($rating < 1 || $rating > 5) {
			throw new InvalidDataException(array(Language::getSessText('ProjectRating.InvalidRating')));
		}
		
		$pr = UserProjectRatingPeer::retrieveByPk($user->getUserId(), $this->getProjectId());
		if (!$pr) {			// User hasn't rated the project before
			$pr = new UserProjectRating();
			$pr->setUser($user);
			$pr->setProject($this);
		} else {
			// User can only upvote
			if ($rating < $pr->getRating()) {
				throw new InvalidDataException(array(Language::getSessText('ProjectRating.UpvoteOnly')));
			}
		}
		
		$pr->setRating($rating);
		$pr->setComment($comment);
		$pr->save();
	}

	/**
	 * Returns list of sprints, if param is true it filters closed sprints.
	 *
	 * @param boolean $skipClosed Flag to filter sprints with 'closed' status
	 *
	 * @return array
	 */
	public function getTrackerSprints($skipClosed=false)
	{
		$criteria = null;
		if ($skipClosed) {
			$criteria = new Criteria();
			$criteria->add(
            	TrackerSprintPeer::SPRINT_STATUS,
            	TrackerSprint::STATUS_CLOSED,
            	Criteria::NOT_EQUAL
            );
		}
		return parent::getTrackerSprints($criteria);
	}

    public function getTrackerIds() {
        $trackerIds = array();
        if ($this->usesPlugin(TrackerPlugin::SECTION)) {
            $con = Propel::getConnection();
            $stmt = $con->createStatement();
            $sql = " SELECT tracker_id FROM tracker WHERE project_id = ".$this->getProjectId();
            $rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

            while ($rs->next()) {
                $trackerIds[] = $rs->getInt(1);
            }
        }
        return $trackerIds;
    }

	/**
	 * Returns assoc array where keys are sprint IDs, values are sprint labels.
	 * NOTE: The default value for $skipClosed is the opposite of the underlying getTrackerSprints method
	 *
	 * @param bool $skipClosed If true, then sprints in Closed status will not be included.  If false, then all sprints are included.
	 * @return array  Assoc array
	 */
	public function getSprintList($skipClosed=true)
	{
		$ret = array();
		$sprints = $this->getTrackerSprints($skipClosed);
		foreach($sprints as $ts) {
		  $ret[$ts->getSprintId()] = $ts->getSprintTitleText();
		}
		// Sort the list by ID, high to low - most recent sprints first.
		krsort($ret);
		return $ret;
	}


	/**
	 * Returns true if the user belongs to the project, false otherwise
	 *
	 * @param int $user_id
	 * @return bool
	 */
	public function isUserMember($user_id) {
		$sql = "SELECT count(user_project.user_id) as total
				FROM user_project, user_project_role, role_setting
				WHERE user_project.user_project_id=user_project_role.user_project_id
				AND user_project_role.role_id=role_setting.role_id
				AND user_project.project_id = " . $this->getProjectId() . "
				AND user_project.user_id = " . $user_id . "
				AND value > -1
				UNION SELECT member_user_id AS user_id FROM usergroup_user WHERE usergroup_id IN
				(
					SELECT DISTINCT user_project.user_id AS usergroup_id
					FROM user_project, user_project_role, role_setting
					WHERE user_project.user_project_id=user_project_role.user_project_id
					AND user_project_role.role_id=role_setting.role_id
					AND user_project.project_id = " . $this->getProjectId() . "
					AND user_project.user_id = " . $user_id . "
					AND value > -1
				)";

		$con = Propel::getConnection(UserPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		$rs->next();
		$total = $rs->getInt('total');
		return ($total > 0) ? true : false;
	}
}
?>
