<?php

// The parent class
require_once 'core/om/BaseSettingPeer.php';

// The object class
include_once 'core/Setting.php';

/**
 * Skeleton subclass for performing query and update operations on the 'setting' table.
 *
 *
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package    propel.generator.core
 */
class SettingPeer extends BaseSettingPeer {

	/**
	 * @param string $section
	 * @param int $ref_id
	 * @param string $chunk
	 * @return array
	 */
	public static function FindSettingsLike($section, $ref_id, $chunk) {
		$c = new Criteria();

		// Convert glob ? to pgsql _
		$chunk = str_replace('?', '_', $chunk);

		// Convert glob * to pgsql %
		$chunk = str_replace('*', '%', $chunk);

		$c->add(self::SECTION, $section)
		  ->add(self::REF_ID, $ref_id)
		  ->add(self::SETTING_NAME, $chunk, Criteria::ILIKE);

		/** @var Setting[] $results */
		$results = self::doSelect($c);

		$ret = array();

		foreach ($results as $result) {
			$ret[$result->getSettingName()] = $result->getSettingValue();
		}
		return $ret;
	}

	/**
	 * @param string $section
	 * @param int $ref_id
	 * @param string $name
	 * @param bool $default
	 * @return bool|string
	 */
	public static function GetSetting($section, $ref_id, $name, $default = FALSE) {
		$setting = self::FindSetting($section, $ref_id, $name);
		$ret = $default;
		if ($setting != null) {
			if ($setting->getSettingValue() != null) {
				$ret = $setting->getSettingValue();
			}
		}
		return $ret;
	}

	/**
	 * @param string $section
	 * @param int $ref_id
	 * @param string $name
	 * @return int
	 */
	public static function DeleteSetting($section, $ref_id, $name) {
		$c = new Criteria();

		$c->add(self::SECTION, $section)
		  ->add(self::REF_ID, $ref_id)
		  ->add(self::SETTING_NAME, $name);

		return self::doDelete($c);
	}

	/**
	 * @param string $section
	 * @param int $ref_id
	 * @param string $name
	 * @return Setting
	 */
	public static function FindSetting($section, $ref_id, $name) {
		$c = new Criteria();

		$c->add(self::SECTION, $section)
		  ->add(self::REF_ID, $ref_id)
		  ->add(self::SETTING_NAME, $name);

		return self::doSelectOne($c);
	}

	/**
	 * @param string $section
	 * @param int $ref_id
	 * @param string $name
	 * @return Setting
	 */
	private static function GetOrCreateSetting($section, $ref_id, $name) {

		$setting = self::FindSetting($section, $ref_id, $name);
		if ($setting == null) {
			$setting = new Setting();

			$setting->setSection($section);
			$setting->setRefId($ref_id);
			$setting->setSettingName($name);
		}

		return $setting;
	}

	/**
	 * @param string $section
	 * @param integer $ref_id
	 * @param string $name
	 * @param string $value
	 * @param integer|null $project_id
	 * @param bool $updateSetDate
	 * @return Setting
	 */
	public static function SaveSetting($section, $ref_id, $name, $value, $project_id=null, $updateSetDate=false) {

		$setting = self::GetOrCreateSetting($section, $ref_id, $name);
		if ($project_id) {
			$setting->setProjectId($project_id);
		} else {
			//Try to get project_id
			if ($ref_id > 0) {
				$project = PluginPeer::getRelatedProject($section, $ref_id);
			}
			if (isset($project) && is_object($project)) {
				$setting->setProjectId($project->getProjectId());
			} else {
				$setting->setProjectId(null);
			}
		}

		if ($updateSetDate || $setting->getSettingValue() !== $value) {
			$setting->setSettingValue($value);
			$setting->setSetDate('now');
			$setting->save();
		}

		return $setting;
	}

	/**
	 *
	 * @param string $section
	 * @param integer $ref_id
	 * @param array $settingKeys
	 *
	 * @return Setting[]
	 */
	public static function getSettingsFromArray($section, $ref_id, $settingKeys) {

		if (!$settingKeys || !count($settingKeys)) {
			return null;
		}

		$ec = count($settingKeys);

		for ($i = 0; $i < $ec; $i++) {
			$settingKeys[$i] = "'".$settingKeys[$i]."'";
		}

		$inString = implode(',', $settingKeys);
		try {
			$sql = "SELECT * FROM setting WHERE ref_id=$ref_id AND section='$section' AND setting_name IN ($inString)";
			$con = Propel::getConnection();
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
		} catch (PropelException $e) {
			log_error($e->getMessage());
			return null;
		}

		return self::populateObjects($rs);
	}

	public static function getSettingsFromArrayAsArray($section, $ref_id, $settingKeys) {
		$settings = self::getSettingsFromArray($section, $ref_id, $settingKeys);
		$settingsArr = array();

		foreach ($settings as $setting) {
			$settingsArr[$setting->getSettingName()] = $setting->getSettingValue();
		}

		return $settingsArr;
	}

}
