<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'core/om/BaseSiteAdmin.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class SiteAdmin extends BaseSiteAdmin implements GFPermission {

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 */
	public function userCanCreate() {
		if ($this->userIsAdmin()) {
			return true;
		}
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		if (Session::getSession()->isSiteAdmin()) {
			return true;
		}
		return false;
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 */
	public function userCanRead() {
		if ($this->userIsAdmin()) {
			return true;
		}
		return false;
	}

	
	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see SiteAdmin::setDeleted()
	 * @see SiteAdmin::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete SiteAdmin object: permission denied");
		}

		ActionManager::notify("SITE_ADMIN_DELETE", $this);
		parent::delete($con);
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create SiteAdmin object: permission denied");
		} else if (!$this->isNew() && !$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update SiteAdmin object: permission denied");
		}

		parent::save($con);
		ActionManager::notify("SITE_ADMIN_SAVE", $this);
	}

	/**
	 * THIS METHOD DOESN'T BELONG HERE!!
     * 
     * @param mixed $depth specifies how many levels down from the curent trove we want to output. Defaults to NULL for outputting all descendants.
     * @param bool $include_root TRUE to output the item for the current trove, FALSE to start output from its inmidiate descendants.
     * @param bool $include_action_checkbox output a checkbox next to the item for the current trove
     * @param bool $is_fixed_category whether the root category is configured as fixed
     * @param bool $is_enabled whether the category is enabled (fixed categories can be disabled)
	 */
	function displayTroveMap($id, $parent_id, $text, $accessLevel=0, $depth=NULL, $include_root=TRUE, $include_action_checkbox=FALSE, $is_fixed_category=FALSE, $is_enabled=TRUE) {		
		$Language = Session::getSession()->getLanguage();
		$template = new SiteAdminTemplate();		
		
        if ($include_root) {
            echo '<li style="margin-left:20px;">'.$template->image('ic/cfolder15.png',15,13).' '.$text.' ';		
            if ($id != 0) {
                echo '<a href="'.GFUrl::getUrl()->currentBaseURL('TroveEdit', array('trove_cat_id' => $id)).'">['.$Language->getText('TroveEdit.Edit').']</a> ';
            }
            echo '<a href="'.GFUrl::getUrl()->currentBaseURL('TroveAdd', array('parent_trove_cat_id' => $id)).'">['.$Language->getText('TroveAdd.Add').']</a> ';
            if ($id != 0) {
                if (!$is_fixed_category) {
                    echo '<a href="'.GFUrl::getUrl()->currentBaseURL('TroveDelete', array('trove_cat_id' => $id)).'">['.$Language->getText('TroveDelete.Delete').']</a> ';
                } else {
                    echo '<a href="#" class="trove-enabled-toggle" id="toggle-trove-'.$id.'" data-category-name="'.$text.'">['.$Language->getText($is_enabled ? 'TroveToggle.Disable' : 'TroveToggle.Enable').']</a>';
                }
            }
            if($id != 0 && $accessLevel == 99) {
                echo '<strong>['.$Language->getText('TroveEdit.AccessAdmin').']</strong>';
            }

            if ($id != 0 && $include_action_checkbox) {
                echo "<input type=\"checkbox\" id=\"trove_$id\" name=\"trove_ids[]\" value=\"$id\" style=\"margin-left: 10px; vertical-align: bottom\" data-category-name=\"$text\"></input>";
            }
            echo '</li>';
        }
        
		$c = new Criteria();
		$c->add(TroveCategoryPeer::PARENT_TROVE_CATEGORY_ID, $id);
		$c->add(TroveCategoryPeer::TROVE_CATEGORY_ID, 0, Criteria::NOT_EQUAL);
		$c->addAscendingOrderByColumn(TroveCategoryPeer::CATEGORY_NAME);
		$troves = TroveCategoryPeer::doSelect($c);
		
        if (!isset($depth) || --$depth >= 0) {
            foreach($troves as $trove) {
                echo '<ul style="margin-left:20px;">';
                $this->displayTroveMap(
                    $trove->getTroveCategoryId(), 
                    $trove->getParentTroveCategoryId(), 
                    $trove->getCategoryName(), 
                    $trove->getAccessLevel(), 
                    $depth, TRUE, $include_action_checkbox, $trove->isFixed(), $trove->isEnabled()
                );
            }
        }
        
        if ($include_root) {
            echo '</ul>';
        }
	}
	


}
?>
