<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('core/om/BaseUser.php');
require_once('plugin/ActionManager.php');
require_once('session/Session.php');
require_once('Validator.php');
require_once('UserProject.php');
require_once('core/SiteAdmin.php');
require_once('core/UsergroupUserPeer.php');
require_once('core/UserPreference.php');
require_once('core/Monitor.php');

require_once('tfa/TwoFactorAuthenticator.php');

/**
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core
 */
class User extends BaseUser {

	const STATUS_TFA_PENDING = -2;
	const STATUS_PENDING_APPROVAL = -1;
	const STATUS_PENDING = 0;
	const STATUS_ACTIVE = 1;
	const STATUS_DELETED = 2;
	const STATUS_SUSPENDED = 3;
	const SECTION = 'user';

	const UNDEFINED_USER_ID = 100;

	const STOCK_AVATARS = 2;
	const STOCK_AVATAR_EXTENSION = "jpg";
	const STOCK_AVATAR_NUMBERING = 100;

  const LOCKOUT_PREF_NAME = 'lockout';

	public $reactivate = false;
	private $changedPassword = false;
	private $password = '';
	private $prefs;
	private $is_site_admin;
	private $bypass_confirmation = false;
	/** Validation errors */
	private $invalid_data = array();
	private $all_user_projects;
	private $all_user_project_ids;

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see User::setDeleted()
	 * @see User::isDeleted()
	 */
	function delete($con = null) {
		//Only site admins can delete users, or a user can delete it's own account
		if (!(Session::getSession()->isSiteAdmin() || Session::getSession()->getUser()->getUserId() == $this->getUserId())) {
			throw new PermissionDeniedException();
		}
		//remove from projects
		$ups = $this->getUserProjects();
		foreach ($ups as $up) {
			$up->delete();
		}
		// stop all user monitoring
		$monitors = $this->getMonitors();
		foreach ($monitors as $monitor) {
			$monitor->delete();
		}

		//remove site admin entries
		$admin = SiteAdminPeer::retrieveByPk($this->getUserId());
		if (!is_null($admin)) {
			$admin->delete();
		}

		//if this is a group, remove the users from the group
		if ($this->getIsGroup() == TRUE) {
			$ugus = $this->getUsergroupUsersRelatedByUsergroupId();
			foreach ($ugus as $ugu) {
				$ugu->delete();
			}
		}

		//remove User from groups
		$UsergroupUsers = $this->getUsergroupUsersRelatedByMemberUserId();
		foreach($UsergroupUsers as $UsergroupUser_obj) {
			$UsergroupUser_obj->delete();
		}

		//delete custom avatar
		$this->deleteCustomAvatar();

		$this->setStatus(User::STATUS_DELETED);
		ActionManager::notify("USER_SAVE", $this);
		try {
			parent::save($con);
		} catch (Exception $e) {
			//low level exception, let's research trace
			error_log($e->getTraceAsString());
			throw new InvalidDataException(array('Invalid data for user'));
		}
		GFDaemon::notify('DAEMON_USER_SAVE', $this->getUnixName());
	}

	/**
	 * Deletes the user from the database.
	 * NOTE: you must be *very* sure you can do this, otherwise could delete huge amounts of data
	 */
	function deleteFromDB() {
		parent::delete();
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if (ActionManager::notify('CHECK_INTEGRITY',array()) != utf8_encode('6.0')) throw new Exception('Invalid version number detected [3]. Please contact GForge Group.');
		// Validate all the data
		$missing_data = array();
		$session = Session::getSession();

		if ($this->getIsGroup() && !$session->isSiteAdmin()) {
			// Only site admin can edit groups
			throw new PermissionDeniedException();
		}

		if (is_null($this->unix_name) || empty($this->unix_name)) {
			$missing_data['unix_name'] = Language::getSessText('User.EmptyUnixName');
		}
		if (is_null($this->email) || empty($this->email)) {
			$missing_data['email'] = Language::getSessText('User.EmptyEmail');
		}

		if (is_null($this->password_md5) || empty($this->password_md5) ||
				is_null($this->password_crypt) || empty($this->password_crypt)) {
			$missing_data['password'] = Language::getSessText('User.EmptyPassword');
		}

		if (is_null($this->firstname) || empty($this->firstname)) {
			$missing_data['firstname'] = Language::getSessText('User.EmptyFirstname');
		}

		if (is_null($this->lastname) || empty($this->lastname)) {
			$missing_data['lastname'] = Language::getSessText('User.EmptyLastname');
		}

		if (is_null($this->ccode) || empty($this->ccode)) {
			$missing_data['ccode'] = Language::getSessText('User.EmptyCountryCode');
		}

		if (is_null($this->language_id) || empty($this->language_id)) {
			// set session language as default
			$this->setLanguageId($session->getLanguage()->getLanguageId());
		}

		if (is_null($this->theme_id) || empty($this->theme_id)) {
			$this->setThemeId($session->getTheme()->getThemeId());
		}

		if (empty($this->cell)) { // no cell defined
			/*
			if (GFConf::get('system.useTFA') && !$_SESSION['catchup_tfa']) {
				// TFA requires cell, error if user is not in catchup process
				// see LoginAction.php
				$this->invalid_data['cell'] = Language::getSessText('User.BadCellNumber');
			}
			*/
		} else { // cell exists, validate
			$tfa = new TwoFactorAuthenticator();
			if (!$tfa->isValidCell($this->cell)) {
				$this->invalid_data['cell'] = Language::getSessText('User.BadCellNumber');
			} else { // cell format valid, verify is unique
			    $c = new Criteria();
			    $c->add(UserPeer::CELL, $this->cell);
			    if (!$this->isNew()) {// filter my unix name
			    	$crit1 = $c->getNewCriterion(UserPeer::UNIX_NAME, $this->unix_name, Criteria::NOT_EQUAL);
			    	$c->add($crit1);
			    }
			    $u = UserPeer::doSelectOne($c);
			    if (!empty($u)) {
			    	$this->invalid_data['cell'] = Language::getSessText("Validation.CellExists");
			    }
			}
		}

		// Note that the invalid data has precedence over the missing data
		// (because invalid fields aren't set)
		$errors = array_merge($missing_data, $this->invalid_data);

		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}
		//set to pending and send confirmation email
		if ($this->isNew()) {
			/*
			if (GFConf::get('system.useTFA')) {
				try {
					$tfa->sendToken($this->cell, 'register');
				} catch (Exception $e) {
					throw new InvalidDataException(array('cell'=>$e->getMessage()));
				}
				$this->setStatus(User::STATUS_TFA_PENDING);
			} else if ($this->bypass_confirmation) {
			*/
			if ($this->bypass_confirmation) {
				// Special functionality -- bypass the user confirmation. This is used
				// for example in LDAP when the user information is imported from the
				// directory.
				$this->setStatus(User::STATUS_ACTIVE);
				$this->setPendingTasksNotification(true);
			} elseif (GFConf::get('system.approveUsers')) {
				// If the administrator must approve the user, set it to a special state
				$this->setStatus(User::STATUS_PENDING_APPROVAL);
				$this->sendNewAccountEmail();		// emails the administrators
			} else {
				$this->setStatus(User::STATUS_PENDING);
			}

		}

		ActionManager::notify("USER_SAVE", $this);

		// Checks whether we need to update the user's password in the authorization system.
		$updateAuth = false;
		if (GFConf::get('system.useDaemon') && !$this->isNew() && $this->isColumnModified(UserPeer::PASSWORD_CRYPT)) {
			$updateAuth = true;
		}

		try {
			parent::save($con);
		} catch (Exception $e) {
			throw new InvalidDataException(array('Invalid data for user'));
		}

		if ($updateAuth) {
			// Notify the daemon to update the user's system info, if neccesary
			GFDaemon::notify('DAEMON_USER_SAVE', $this->getUnixName());
		}
	}

	public function saveAfterTFAConfirmation()
	{
		if ($this->bypass_confirmation) {
			// Special functionality -- bypass the user confirmation. This is
			// used for example in LDAP when the user information is imported
			// from the directory.
			$this->setStatus(User::STATUS_ACTIVE);
			$this->setPendingTasksNotification(true);
		} elseif (GFConf::get('system.approveUsers')) {
			// If the administrator must approve the user, set it to a
			// special state
			$this->setStatus(User::STATUS_PENDING_APPROVAL);
			$this->sendNewAccountEmail();		// emails the administrators
		} else {
			$this->setStatus(User::STATUS_PENDING);
		}
		ActionManager::notify("USER_SAVE", $this);
		try {
			parent::save($con);
		} catch (Exception $e) {
			throw new InvalidDataException(array('Invalid data for user'));
		}
		// Notify the daemon to update the user's system info, if neccesary
		GFDaemon::notify('DAEMON_USER_SAVE', $this->getUnixName());
	}

	/**
	 * Approve the user accout and send the confirmation email
	 */
	public function approve() {
		$this->setStatus(User::STATUS_PENDING);		// set to pending confirmation (don't confuse with pending approval)
		$this->sendRegistrationEmail(true);
		$this->setPendingTasksNotification(true);		// Activate the pending tasks notifications in the Monitor table
		$this->save();
		return true;
	}

	/**
	 * Send an email to the system administrators informing them of the new user
	 */
	private function sendNewAccountEmail() {
		$url = GFUrl::getUrl();
		$admins = UserPeer::getSystemAdministrators();
		foreach ($admins as $admin) {
			$language = $admin->getLanguage();
			$subject = $language->getText('User.PendingUserSubject', array($this->getFullName()));
			$body = $language->getText('User.PendingUserTextAdmin', array($this->getFullName(), $this->getUnixName(),
										$url->getFullUrl($url->baseUrl('PendingUserApproval', '', 'admin'), true)));
			SystemUtils::sendMail($subject, $body, $admin->getEmail());
		}

	}

	public function sendRegistrationEmail($approved = false) {
		$session = Session::getSession();
		$language = $this->getLanguage();
		$gfurl = GFUrl::getUrl();
		if ($approved) {
			$subject = $language->getText('UserAdd.ApprovedMessageHeader', array(GFConf::get('system.name')));
		} else {
			$subject = $language->getText('UserAdd.MessageHeader', array(GFConf::get('system.name')));
		}
		$url = $gfurl->getFullUrl( $gfurl->baseURL('verify','','account',array("confirm_hash" => $this->getConfirmHash())), true);
		$message = stripcslashes($language->getText('UserAdd.MessageBody', array($this->getUnixName(), $url, GFConf::get('system.hostName'))));
		SystemUtils::sendMail($subject, $message, $this->getEmail());
	}


	public function sendEmailEditEmail() {
		$session = Session::getSession();
		$language = $this->getLanguage();
		$gfurl = GFUrl::getUrl();

		$to = $this->getEmailNew();
		$subject = $language->getText('MyUserEmailEdit.EmailSubject', array(GFConf::get('system.name')));
		$url = $gfurl->getFullUrl($gfurl->baseURL('change_email', '', 'my', array('confirm_hash' => $this->getConfirmHash())),true);
		$message =  $language->getText('MyUserEmailEdit.EmailBody', array(GFConf::get('system.hostName'), $url, GFConf::get('system.hostName')));
		SystemUtils::sendMail($subject, stripcslashes($message), $to);
	}

	public function setUnixName($unix_name) {
		$unix_name=strtolower($unix_name);
		if (empty($unix_name)) {
			$this->invalid_data['unix_name'] = Language::getSessText('User.EmptyUnixName');
			return;
		}
		if (!ereg(GFConf::get('system.usernameregex'), $unix_name)) {
			$this->invalid_data['unix_name'] = Language::getSessText('User.InvalidUnixName');
			return;
		}

		if ($this->isNew() || ($this->getUnixName() != $unix_name)) {
			if (GFConf::get('system.usesNis')) {
				//unix machine is controlling user accounts
			} else {
				if (SystemUtils::userExists($unix_name)) {
					$this->invalid_data['unix_name'] = Language::getSessText('Validation.UserExists');
					return;
				}
			}

			//check for already existing user name
			if (is_object(UserPeer::retrieveByUnixName($unix_name))) {
				$this->invalid_data['unix_name'] = Language::getSessText('Validation.UserExists');
				return;
			}
		}

		parent::setUnixName($unix_name);
		unset($this->invalid_data['unix_name']);
	}

	public function setEmail($email) {
		$email=strtolower($email);
		if (empty($email)) {
			$this->invalid_data['email'] = Language::getSessText('User.EmptyEmail');
			return;
		}

		// check for valid email
		if (!Validator::validateEmail($email)) {
			$this->invalid_data['email'] = Language::getSessText('User.InvalidEmail');
			return;
		}

		if ($this->isNew() || ($email != $this->getEmail())) {
			// Email must be unique
			$c = new Criteria();
			$c->add(UserPeer::EMAIL, $email);
			if (is_object(UserPeer::doSelectOne($c))) {
				$this->invalid_data['email'] = Language::getSessText('Validation.EmailExists');
				return;
			}
		}

		parent::setEmail($email);
		unset($this->invalid_data['email']);
	}



	public function setEmailNew($new_email) {
		$new_email=strtolower($new_email);
		if (empty($new_email)) {
			$this->invalid_data['new_email'] = Language::getSessText('User.EmptyEmail');
			return;
		}

		// check for valid email
		if (!Validator::validateEmail($new_email)) {
			$this->invalid_data['new_email'] = Language::getSessText('User.InvalidEmail');
			return;
		}

		// Email must be unique
		$c = new Criteria();
		$c->add(UserPeer::EMAIL, $new_email);
		if (is_object(UserPeer::doSelectOne($c))) {
			$this->invalid_data['new_email'] = Language::getSessText('Validation.EmailExists');
			return;
		}

		parent::setEmailNew($new_email);
		unset($this->invalid_data['new_email']);
	}



	/**
	 * Updates the password(s)
	 */
	public function setPassword($password) {
		$passresult = Validator::validatePassword($password);
		if (!$passresult[0]) {
			// for now, return only the FIRST error found
			// (Validator::validatePassword returns an array with all the errors found)
			$this->invalid_data['password'] = $passresult[1][0];
			return;
		}

		$this->setPasswordMd5(md5($password));
		$this->setPasswordCrypt(SystemUtils::hashString($password));
		$this->password = $password;
		unset($this->invalid_data['password']);
	}

	/**
	 * Returns true if User state is Active
	 *
	 * @return boolean
	 */
	function isActive () {
		if ($this->getStatus() == self::STATUS_ACTIVE) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Indicates whether the user is a member of the project
	 *
	 * @param string	Project
	 * @return boolean	true if it's member
	 *
	 */
	public function isProjectMember($project) {
		$ids = $this->getAllProjectIds(false);			// This query is not cached
		return isset($ids[$project->getProjectId()]);
	}

	public function getUserLink() {
		if ($this->getUserid() == 100) {
			return 'Nobody';
		} else {
			return '<a href="/gf/user/'.$this->getUnixName().'/">'.$this->getFirstName(). ' '. $this->getLastName().'</a>';
		}
	}

	function getName() {
		return $this->getFirstName(). ' '. $this->getLastName();
	}

	function getId() {
		return $this->getUserId();
	}

	function getPreference($key, $default = '') {
		return UserPreferencePeer::getUserPreferenceValue($this->getUserId(), $key, $default);
	}


	function setPreference($key,$val) {
		UserPreferencePeer::setUserPref($this->getUserId(), $key, $val);
	}


	function deletePreference($key) {
		if (isset($this->prefs[$key])) {
			$this->prefs[$key]->delete();
			unset($this->prefs[$key]);
		} else {
			$c = new Criteria();
			$c->add(UserPreferencePeer::PREFERENCE_NAME, $key);
			$c->add(UserPreferencePeer::USER_ID, $this->getUserId());
			$p = UserPreferencePeer::doSelectOne($c);
			if ($p) {
				$p->delete();
			}
		}
	}

	/**
	 * Activate this user
	 */
	public function activate() {
		if ($this->getStatus()==self::STATUS_DELETED) {
			$this->reactivate=true;
		}
		$this->setStatus(self::STATUS_ACTIVE);
		$this->save();
	}

	/**
	 * Returns the user's full name
	 * @return	string
	 */
	public function getFullName() {
		return $this->getFirstName().' '.$this->getLastName();
	}



	public function usesPlugin($name) {
		return array_key_exists($name, $this->getPlugins());
	}

	public function getPlugins() {
		if (!isset($this->plugins)) {
			$this->plugins = array();

			$crit = new Criteria();
			$crit->add(UserPluginPeer::USER_ID,$this->getUserId());
			$plugins = UserPluginPeer::doSelectJoinPlugin($crit);
			$pf = PluginFactory::getFactory();
			foreach ($plugins as $plugin) {
				$this->plugins[$plugin->getPlugin()->getPluginName()]=$pf->getPluginByName($plugin->getPlugin()->getPluginName());
			}
		}
		return $this->plugins;
	}

	public function getPluginByName($name) {
		$plugins = $this->getPlugins();
		if (array_key_exists($name, $plugins)) {
			return $plugins[$name];
		} else {
			return null;
		}
	}

	/**
	 * @param	mixed $plugin	Either the plugin name or the Plugin object
	 * @todo	If $plugin has exclusive_type, make sure to disable other plugins with the same exclusive_type
	 */
	public function enablePlugin($plugin) {
		if ($plugin instanceof Plugin) {
			$pluginName = $plugin->getPluginName();
		} else {
			$pluginName = $plugin;
			$c = new Criteria();
			$c->add(PluginPeer::PLUGIN_NAME, $pluginName);
			$plugin = PluginPeer::doSelectOne($c);
			if (is_null($plugin)) return;
		}

		if ($this->usesPlugin($pluginName)) {
			return;
		}

		$pp = new UserPlugin();
		$pp->setUser($this);
		$pp->setPlugin($plugin);
		$pp->save();

		// Store the GFPlugin object
		$pf = PluginFactory::getFactory();
		$gfplugin = $pf->getPluginByName($pluginName);
		$this->plugins[$pluginName] = $gfplugin;

		$gfplugin->enableInUser(); //check this
	}

	/**
	 * @param	mixed $plugin	Either the plugin name or the Plugin object
	 */
	public function disablePlugin($plugin) {
		if ($plugin instanceof Plugin) {
			$pluginName = $plugin->getPluginName();
		} else {
			$pluginName = $plugin;
			$c = new Criteria();
			$c->add(PluginPeer::PLUGIN_NAME, $pluginName);
			$plugin = PluginPeer::doSelectOne($c);
			if (is_null($plugin)) return;
		}

		if (!$this->usesPlugin($pluginName)) {
			return;
		}

		$c = new Criteria();
		$c->add(UserPluginPeer::USER_ID,$this->getUserId());
		$c->add(UserPluginPeer::PLUGIN_ID,$plugin->getPluginId());
		$pp = UserPluginPeer::doSelectOne($c);
		if (!($pp instanceof UserPlugin )) {
			// weird, shouldn't happen
			return;
		}
		// this retrieves the GFPlugin object associated to the plugin
		$gfplugin = $this->getPluginByName($pluginName);
		// inform the plugin that we're going to disable it
		$gfplugin->disableFromUser (); //check this

		$pp->delete();

		unset($this->plugins[$pluginName]);
	}

	public function IsMonitoringSite(){
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, 'site');
		$c->add(MonitorPeer::REF_ID, '0');
		$c->add(MonitorPeer::USER_ID, $this->getPrimaryKey());
		$mon = MonitorPeer::doSelectOne($c);
		return (!is_null($mon)) ? true : false;
	}

	//Need to make a direct INSERT because Monitor->save() makes a permission check based on plugins
	//and Site Notification is not a plugin
	function setSiteMonitoring($bool){
		if($bool){
			if(!$this->IsMonitoringSite()){
				$sql = 'INSERT INTO monitor (section, ref_id, user_id) VALUES (\'site\', 0, \''.$this->getUserId().'\');';
				$prop = Propel::getConnection();
				$stmt = $prop->createStatement();
				$rs = $stmt->executeQuery($sql);
			}
		}
		if(!$bool) {
			$sql = 'DELETE FROM monitor WHERE section = \'site\' AND ref_id = 0 AND user_id = \''.$this->getUserId().'\';';
			$prop = Propel::getConnection();
			$stmt = $prop->createStatement();
			$rs = $stmt->executeQuery($sql);
		}
	}

	public function isUndefinedUser() {
		return ($this->getUserId() == self::UNDEFINED_USER_ID);
	}

	/**
	 * @return	bool
	 */
	public function isSiteAdmin() {
		if (!isset($this->is_site_admin)) {
			$sess=Session::getSession();
			if ($sess->isLoggedIn() && is_object($sess->getUser()) && $sess->getUser()->getUserId() == $this->getUserId()) {
				$this->is_site_admin = $sess->isSiteAdmin();
			} else {
				$c = new Criteria();
				$c->add(SiteAdminPeer::USER_ID, $this->user_id);
				$n = SiteAdminPeer::doCount($c);
				$this->is_site_admin = ($n > 0);
			}
		}

		return $this->is_site_admin;
	}

	/**
	 * Call before save() - If user registration is not moderated, this allows
	 * to directly create the user (no confirmation by the user required)
	 */
	public function bypassConfirmation() {
		$this->bypass_confirmation = true;
	}

	public function getEmail() {
		if ($this->getIsGroup() == TRUE) {
			$emails=array();
	        $ugus = $this->getUsergroupUsersRelatedByUsergroupId();
	        foreach ($ugus as $ugu) {
	        	$emails[]=$ugu->getUserRelatedByMemberUserId()->getEmail();
			}
			if (count($emails) < 1) {
				return 'noreply@'.GFConf::get('system.hostName');
			} else {
				return implode(',',$emails);
			}
                } else {
			return parent::getEmail();
		}
	}

	/**
	 * Gets a list of all projects you are a member of,
	 * including those via group memberships
	 * Does not yet include subprojects in hierarchies.
	 * To see the logic of this function, refer to the SQL function "getAllUserProjectsIds".
	 */
	public function getAllProjects($cached=true) {
		if (!isset($this->all_user_projects) || !$cached) {
			$sql = 'SELECT * FROM project WHERE project_id IN (SELECT * FROM getAllUserProjectsIds('.$this->getUserId().'))
					ORDER BY project_name ASC;';
			$rs = Propel::getConnection()->createStatement()->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			$this->all_user_projects = ProjectPeer::populateObjects($rs);
		}

		return $this->all_user_projects;
	}

	public function getAllProjectIds($cached=true) {
		if (!isset($this->all_user_project_ids) || !$cached) {
			$this->all_user_project_ids = array();
			$projects = $this->getAllProjects();
			foreach ($projects as $project) {
				$project_id = $project->getProjectId();
				$this->all_user_project_ids[$project_id] = $project_id;
			}
		}

		return $this->all_user_project_ids;
	}


	public function getAllAccessibleProjectsAssoc() {
		if (!isset($this->all_user_accessible_project_assoc)) {
			$this->all_user_accessible_project_assoc = array();
			$projects = $this->getAllProjects();
			foreach ($projects as $project) {
				$this->all_user_accessible_project_assoc[$project->getProjectId()] = $project->getProjectName();
			}

			$project_ids = '(' . implode(array_keys($this->all_user_accessible_project_assoc), ',') . ')';

			//Gets all public projects
			$c = new Criteria();
			$c->add(ProjectPeer::IS_PUBLIC, true);
			if (count($projects) > 0) {
				$c->add(ProjectPeer::PROJECT_ID, ProjectPeer::PROJECT_ID . ' NOT IN ' . $project_ids, CRITERIA::CUSTOM);
			}
			$public_projects = ProjectPeer::doSelect($c);

			foreach ($public_projects as $project) {
				$this->all_user_accessible_project_assoc[$project->getProjectId()] = $project->getProjectName();
			}

		}

		return $this->all_user_accessible_project_assoc;
	}

	public static function createUserGroup($group_name, $unix_group_name) {
		if (!Session::getSession()->isSiteAdmin()) {
			throw new PermissionDeniedException();
		}

		$user_group = new User();
		$user_group->setUnixName($unix_group_name);
		$user_group->setEmail( md5(mt_rand(1000000,9000000)).'@'.GFConf::get('system.hostName') );
		$password = substr(md5(mt_rand(1000000,9000000)), 0, 15);
		$user_group->setPassword($password);
		$user_group->setTimezone('US/Central');
		$user_group->setFirstname($group_name);
		$user_group->setLastname('   ');
		$user_group->setCcode('US');
		$user_group->setLanguageId(GFConf::get('system.defaultLanguageId'));
		$user_group->setThemeId(GFConf::get('system.defaultThemeId'));
		$user_group->setStatus(User::STATUS_ACTIVE);
		$user_group->setIsGroup(true);
		$user_group->bypassConfirmation();
		$user_group->save();

		return $user_group;
	}

	public function setCcode($code){
		$code =  substr($code, 0, 2);
		if(CountryCodePeer::retrieveByPk($code)){
			parent::setCcode($code);
		} else {
			$this->invalid_data['ccode'] = Session::getLanguage()->getSessText('User.InvalidCountryCode');
		}
	}

	/**
	 * Determine if the user is a member of any of the specified groups
	 * @param Array $groups As list of user ID (int) values that form the group list to be checked against
	 * @return bool false if zero of the specified groups have the user as a member, true otherwise.
	 */
	public function isMemberOfAny($groups = array()) {
		$c = new Criteria();
		$c->add(UsergroupUserPeer::USERGROUP_ID, $groups, Criteria::IN);
		$c->add(UserPeer::USER_ID, $this->getUserId());
		$c->addJoin(UsergroupUserPeer::MEMBER_USER_ID, UserPeer::USER_ID);

		$count = UserPeer::doCount($c);

		if ($count > 0) return true;
		return false;
	}

	/**
	 * Given a list of groups, returns the ones the user is a member of
	 * @param Array $groups As list of user ID (int) values that form the group list to be checked against
	 * @return array of matched groups
	 */
	public function getMemberGroups($groups = array()) {
		$c = new Criteria();
		$c->add(UsergroupUserPeer::USERGROUP_ID, $groups, Criteria::IN);
		$c->add(UsergroupUserPeer::MEMBER_USER_ID, $this->getUserId());
		$c->addJoin(UserPeer::USER_ID, UsergroupUserPeer::USERGROUP_ID);

		return UserPeer::doSelect($c);
	}


	/**
	 * Convenience version of isMemberOfAny, taking only one group ID
	 * @param int $group_id Unique identifier for the group to be checked
	 * @return bool false if the specified group has the user as a member, true otherwise.
	 */
	public function isMemberOfGroup($group_id) {
		return $this->isMemberOfAny(array($group_id));
	}

	/**
	 * Return the list of users belonging to this group
	 */
	public function getUsergroupUsers() {
		$c = new Criteria();
		$c->add(UsergroupUserPeer::USERGROUP_ID, $this->getUserId());
		$c->addJoin(UsergroupUserPeer::MEMBER_USER_ID, UserPeer::USER_ID);
		return UserPeer::doSelect($c);
	}

	public function deleteCustomAvatar() {
		$custom_avatar_path = GFConf::get('system.libDir')."/www/images/custom_avatars/".$this->getAvatarId(). "." . User::STOCK_AVATAR_EXTENSION;
		if (file_exists($custom_avatar_path)) {
			unlink($custom_avatar_path);
		}
	}


	public function setAvatarFromField($avatar_id, $files = array()) {
		function getExtension($str) {
			$i = strrpos($str,".");
			if (!$i) {
				return "";
			}

			$l = strlen($str) - $i;
		         $ext = substr($str,$i+1,$l);
		         return $ext;
		 }

		if ($files['avatar']['name'] != "") {
			//Save the avatar
			$file = $files['avatar'];
			$error = $file['error'];
			$filename=$file['name'];
			$tmp_name=$file['tmp_name'];

			if ($error > 0 && $error != 4) {
				$fileerrors = StringUtils::getFileUploadErrorString($error, $filename);
				throw new InvalidDataException($fileerrors);
			}

			$extension = getExtension($filename);
			$extension = strtolower($extension);

			if($extension=="jpg" || $extension=="jpeg" ) {
				$uploadedfile = $files['avatar']['tmp_name'];
				$src = imagecreatefromjpeg($uploadedfile);
			}
			else if($extension=="png") {
				$uploadedfile = $files['avatar']['tmp_name'];
				$src = imagecreatefrompng($uploadedfile);
			}
			else {
				$uploadedfile = $files['avatar']['tmp_name'];
				$src = imagecreatefromgif($uploadedfile);
			}

			list($width,$height)=getimagesize($uploadedfile);

			$max_width = 60;
			$max_height = 100;

			if ($width > $max_width || $height > $max_height) {
				if ($width > $height) {
					$ratio = $height / $width;
					$new_width = $max_width;
					$new_height = $new_width * $ratio;
				} else {
					$ratio = $width / $height;
					$new_height = $max_height;
					$new_width = $new_height * $ratio;
				}

				$tmp = imagecreatetruecolor($new_width, $new_height);
				imagecopyresampled($tmp,$src,0,0,0,0,$new_width,$new_height,$width,$height);

				$uploadedfile = "/tmp/resized_". $filename;
				imagejpeg($tmp,$uploadedfile,100);
			}

			$custom_avatar_path = GFConf::get('system.varDir')."/custom_avatars/".$this->getUserId().".jpg";
			$res = copy($uploadedfile, $custom_avatar_path);

			$this->setAvatarId($this->getUserId());
			$this->setUseCustomAvatar(true);

		} else {
			$this->setAvatarId($avatar_id);
			$this->setUseCustomAvatar(false);
		}
	}

	function getAvatarType() {
		if (GFConf::get('system.use_ssl') == true) {
			$url_prefix = "https://";
		} else {
			$url_prefix = "http://";
		}

		if ($this->getAvatarId() == 0) {
			return "d";
		} else {
			if ($this->getUseCustomAvatar()) {
				//Build user-uploaded avatar url
				$custom_avatar_path = GFConf::get('system.libDir')."/www/images/custom_avatars/".$this->getAvatarId(). "." . User::STOCK_AVATAR_EXTENSION;
				//Check if user-uploaded avatar exists
				if (file_exists($custom_avatar_path)) {
					return "c";
				} else {
					return "s";
				}
			}
		}
	}

	function getAvatarUrl() {
		$url_prefix = GFUrl::getUrl()->getUrlPrefix().GFConf::get('system.hostName');

		if ($this->getAvatarId() == 0) {
			$avatar_url = $url_prefix."/images/stock_avatars/default.png";
		} else {
			if ($this->getUseCustomAvatar()) {
				$custom_avatar_path = GFConf::get('system.libDir')."/www/images/custom_avatars/".$this->getAvatarId(). "." . User::STOCK_AVATAR_EXTENSION;
				if (file_exists($custom_avatar_path)) {
					$avatar_url = $url_prefix."/images/custom_avatars/".$this->getAvatarId(). "." . User::STOCK_AVATAR_EXTENSION;
				} else {
					$avatar_url = $url_prefix."/images/stock_avatars/default.png";
				}
			} else {
				//Build stock avatar url
				$avatar_url = $url_prefix."/images/stock_avatars/".$this->getAvatarId(). "." . User::STOCK_AVATAR_EXTENSION;
			}
		}
		return $avatar_url;
	}

	function getAvatarHTML($style="max-width:50px;max-height:40px;") {
		$alt = $this->getFullName();
		return '<img alt="'.$alt.'" src="'.$this->getAvatarUrl().'" '.($style ? 'style="'.$style.'"' : '').' />';
	}

	function getAvailableStockAvatars() {
		$arr_stock_avatars = array();

		for ($i=0; $i < User::STOCK_AVATARS; $i++) {
			$arr_stock_avatars[$i] = array();
			$arr_stock_avatars[$i]['id'] = User::STOCK_AVATAR_NUMBERING + $i + 1;
			$arr_stock_avatars[$i]['filename'] = $arr_stock_avatars[$i]['id'] . "." . User::STOCK_AVATAR_EXTENSION;
		}

		return $arr_stock_avatars;
	}

	public function setPendingTasksNotification($v=true) {
		try {
			if ($v) {
				$monitor = new Monitor();
				$monitor->setSection(TrackerPlugin::PENDING_TASKS_SECTION);
				$monitor->setUserId($this->getUserId());
				$monitor->save();
			} else {
				$c = new Criteria();
				$c->add(MonitorPeer::SECTION, TrackerPlugin::PENDING_TASKS_SECTION);
				$c->add(MonitorPeer::USER_ID, $this->getUserId());
				MonitorPeer::doDelete($c);
			}
		} catch (Exception $ex) {
			// Don't do anything if the entry in the Monitor table already existed
		}
	}

  public function isLocked() {
    $c = new Criteria(UserPreferencePeer::DATABASE_NAME);
    $c->add(UserPreferencePeer::PREFERENCE_NAME, User::LOCKOUT_PREF_NAME);
    $c->add(UserPreferencePeer::USER_ID, $this->getUserId());
    $p = UserPreferencePeer::doSelectOne($c);
    if (is_null($p)) {
      return false;
    } else {
      $n = intval($p->getPreferenceValue());
      $max = GFConf::get('system.maxLoginAttempts');
      return $n > $max || $n == -1;
    }
  }

  public function getLoginAttempts() {
    $c = new Criteria(UserPreferencePeer::DATABASE_NAME);
    $c->add(UserPreferencePeer::PREFERENCE_NAME, User::LOCKOUT_PREF_NAME);
    $c->add(UserPreferencePeer::USER_ID, $this->getUserId());
    $p = UserPreferencePeer::doSelectOne($c);
    if (is_null($p)) {
      return 0;
    } else {
      return intval($p->getPreferenceValue());
    }
  }

  public function setLoginAttempts($n) {
    $c = new Criteria(UserPreferencePeer::DATABASE_NAME);
    $c->add(UserPreferencePeer::PREFERENCE_NAME, User::LOCKOUT_PREF_NAME);
    $c->add(UserPreferencePeer::USER_ID, $this->getUserId());
    $p = UserPreferencePeer::doSelectOne($c);
    if (is_null($p)) {
      $p = new UserPreference();
      $p->setPreferenceName(User::LOCKOUT_PREF_NAME);
      $p->setUser($this);
      $p->setPreferenceValue($n);
    } else {
      $p->setPreferenceValue($n);
    }
    $p->save();
  }

  public function sendUnlockEmail() {
    $confirm_hash = md5(Session::getSession()->getSessionHash() . strval(time()) . strval(rand())); // First, we need to create new confirm hash
    $this->setConfirmHash($confirm_hash);
    $url = GFUrl::getFullUrl( GFUrl::getUrl()->baseURL('PasswordReset','','account',array('confirm_hash'=>$confirm_hash)), true);
    $message = stripcslashes(Language::getSessText('PasswordReset.Message',array($url,GFConf::get('system.hostName'))));
    SystemUtils::sendMail(Language::getSessText('PasswordReset.Subject',array(GFConf::get('system.hostName'))),$message,$this->getEmail());
  }

}

