<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
// The parent class
require_once 'core/om/BaseUserPeer.php';
// The object class
include_once 'core/User.php';


/** 
 * The skeleton for this class was autogenerated by Propel on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package core 
 */
class UserPeer extends BaseUserPeer {
	private static $_pkCache = array();
	private static $_unixNameUserCache = array();

	/**
	 * Retrieve a single object by pkey or NULL if not found.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key
	 * @param	bool $useCache	Whether to use caching. This will increase performance if the same object
	 *							is accessed several times
	 * @param	Connection $con	The connection to use
	 * @return	User
	 */
	public static function retrieveByPk($pk, $useCache = true, $con = null) {
		if ($useCache && array_key_exists($pk, UserPeer::$_pkCache)) {
			return UserPeer::$_pkCache[$pk];
		}
		
		$object = parent::retrieveByPk($pk, $con);
		if ($useCache) {
			UserPeer::$_pkCache[$pk] = $object;
			if (!is_null($object)) {
				UserPeer::$_unixNameUserCache[$object->getUnixName()] = $object;
			}
		}
		return $object;
	}
	
	
	/**
	 * Retrieve a single object by UnixName or NULL if not found.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $unixName	Unix name of the project
	 * @param	bool $useCache	Whether to use caching. This will increase performance if the same object
	 *							is accessed several times
	 * @param	Connection $con	The connection to use
	 * @return	User
	 */
	public static function retrieveByUnixName($unixName, $useCache = true, $con = null) {
		if ($useCache && array_key_exists($unixName, UserPeer::$_unixNameUserCache)) {
			return UserPeer::$_unixNameUserCache[$unixName];
		}				
		$c = new Criteria();
		$c->add(UserPeer::UNIX_NAME,$unixName);
		$object = UserPeer::DoSelectOne($c);		
		if ($useCache) {		 	
			UserPeer::$_unixNameUserCache[$unixName] = $object;
			if (!is_null($object)) {
				UserPeer::$_pkCache[$object->getUserID()] = $object;
			}
		}
		return $object;
	}
	
	/**
	 * Retrieve all the users by email address
	 */
	public static function retrieveByEmail($email) {
		$c = new Criteria();
		$c->add(self::EMAIL, $email)->setIgnoreCase(true);
		
		return self::doSelect($c);
	}
	
	
	/**
	 * Removes an object from the cache. This is usually done after saving.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key of the object to remove
	 */
	public static function invalidate($pk) {
		if (array_key_exists($pk, UserPeer::$_pkCache)) {
			unset(UserPeer::$_pkCache[$pk]);
		}
	}
	
	/**
	 * Get an array with system administrators
	 */
	public static function getSystemAdministrators() {
		require_once('core/SiteAdminPeer.php');
		
		$c = new Criteria();
		$c->addJoin(UserPeer::USER_ID, SiteAdminPeer::USER_ID);
		return self::doSelect($c);
	}

	/**
	 * Retrieve the list of users associated to the specified user.
	 * @param mixed $u The User object or ID of the user for whom to calculate the social graph
	 * @return array The set of Users that share one or more projects or wall postings with the named user.
	 */
	public static function getSocialGraphFor($u) {
		$user_id = 0;
		if (is_object($u)) {
			$user_id = $u->getUserId();
		} else {
			$user_id = $u;
		}

		$sql = "select distinct * from \"user\"
				where status = 1 and user_id <> $user_id
				and (
				user_id in (select distinct user_id from user_project where project_id in (select distinct project_id from user_project where user_id = $user_id))
				or
				user_id in (select distinct posted_by from wall_post where section = 'user' and ref_id = '$user_id')
				or
				user_id in (select distinct ref_id from wall_post where section = 'user' and posted_by = $user_id)
				)";

		$con = Propel::getConnection(UserPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		$ret = array();
		while ($rs->next()) {
			$user = new User;
			$user->hydrate($rs);
			$ret[] = $user;
		}
		return $ret;
	}

	/**
	 * Receives an array containing unixname values, return array of
	 * User objects.
	 *
	 * @param array      $unixNames Array with unix_name values
	 * @param Connection $con       Connection object
	 *
	 * @return array
	 */
	public static function retrieveByUnixNames($unixNames, $con=null) {
		$c = new Criteria();
		$c->add(UserPeer::UNIX_NAME, $unixNames, Criteria::IN);
		$users = UserPeer::DoSelect($c);
		return $users;
	}

	/**
	 * The returned array will contain objects of the default type or
	 * objects that inherit from the default.
	 *
	 * @throws PropelException Any exceptions caught during processing will be
	 *	       rethrown wrapped into a PropelException.
	 */
	/*
	public static function populateObjects(ResultSet $rs)
	{
		$results = array();

		// set the class once to avoid overhead in the loop
		//$cls = UserPeer::getOMClass();
		Propel::import('core.User');
		Propel::import('core.Usergroup');

		// populate the object(s)
		while($rs->next()) {

			$cls = ( $rs->getString("is_group")=="t" ? 'Usergroup' : 'User' );
			$obj = new $cls();
			$obj->hydrate($rs);
			$results[] = $obj;
		}
		return $results;
	}*/
}
?>
