<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'core/om/BaseUserUnix.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class UserUnix extends BaseUserUnix implements GFPermission {
	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 */
	public function userCanCreate() {
		if (posix_getuid() == 0 || $this->userIsAdmin()) {
			return true;
		}
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		$session=Session::getSession();
		if ($session->isSiteAdmin()) {
			return true;
		}
		
		if ($session->isLoggedIn() && $session->getUser()->getUserId() == $this->getUser()->getUserId()) {
			return true;
		}
		
		return false;
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 */
	public function userCanUpdate() {
		if(posix_getuid() == 0 || $this->userIsAdmin()){
			return true;
		}
		return false;
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 */
	public function userCanRead() {
		return $this->userIsAdmin();
	}

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see UserUnix::setDeleted()
	 * @see UserUnix::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete UserUnix object: permission denied");
		}
		ActionManager::notify("USER_UNIX_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create UserUnix object: permission denied");
		} else if (!$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update UserUnix object: permission denied");
		}
		parent::save($con);
		ActionManager::notify("USER_UNIX_SAVE", $this);
	}

	function setAuthorizedKeys($authorized_keys) {
            
		$authorized_keys = trim($authorized_keys);
		$authorized_keys = str_replace("\r\n", "\n", $authorized_keys); // Convert to Unix EOL
		$authorized_keys = str_replace("\n+", "\n", $authorized_keys); // Remove empty lines
		$authorized_keys = str_replace("\n", "###", $authorized_keys); // Convert EOL to marker

		parent::setAuthorizedKeys($authorized_keys);
	}
        
        function getAuthorizedKeys($raw=false) {	
            
            $authorized_keys = parent::getAuthorizedKeys();
            if ($raw){
                return $authorized_keys;
            }else{
                $authorized_keys = str_replace("###","\n", $authorized_keys);
                return $authorized_keys;
            }
		
	}
}
?>
