<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2010-11-10
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'core/om/BaseWallPost.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class WallPost extends BaseWallPost implements GFPermission {

	const SECTION = 'messagewall';

	/**
	 * isPublic - true/false
	 * @return boolean
	 */
	function isPublic() {
		return true;
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		if ($this->userIsAdmin()) {
			return true;
		}

		$user_session = Session::getSession()->getUser();
		$project = GFUrl::getUrl()->getProject();

		if (Session::getSession()->isLoggedIn() && $this->getSection() == Project::SECTION && is_object($project) && ($user_session->isProjectMember($project) || $project->isPublic()) ) {
			return true;
		}	
		if (Session::getSession()->isLoggedIn() && $this->getSection() == User::SECTION && $this->getPostedBy() == $user_session->getUserId()) {
			return true;
		}

		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		if (Session::getSession()->isSiteAdmin()) {
			return true;
		}
		$user_session = Session::getSession()->getUser();
		$project = GFUrl::getUrl()->getProject();

		if (Session::getSession()->isLoggedIn() && $this->getSection() == Project::SECTION && is_object($project) && $project->userIsAdmin()) {
			return true;
		}	
		/*
		//requires manual customization
		$perm = GFUrl::getUrl()->getProject()->getPermission();
		if ($perm->check(self::SECTION, SELF::ADMIN)) {
			return true;
		}
		*/
		
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		if ($this->userIsAdmin()) {
			return true;
		}

		$user_session = Session::getSession()->getUser();
 
		if (Session::getSession()->isLoggedIn() && $this->getPostedBy() == $user_session->getUserId()) {
			return true;
		}	
		
		return false;
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead($project_id="") {
		if ($this->userIsAdmin()) {
			return true;
		}

		$user_session = Session::getSession()->getUser();
		if ($project_id == "") {
			$project = GFUrl::getUrl()->getProject();
		} else {
			$project = ProjectPeer::retrieveByPk($project_id);
		}

		if ((Session::getSession()->isLoggedIn() && $this->getSection() == Project::SECTION && is_object($project) && $user_session->isProjectMember($project)) || $project->isPublic() ) {
			return true;
		}	
		if (Session::getSession()->isLoggedIn() && $this->getSection() == User::SECTION && ($this->getPostedBy() == $user_session->getUserId()) || ($this->getRefId() == $user_session->getUserId()) ) {
			return true;
		}

		return false;
	}

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see WallPost::setDeleted()
	 * @see WallPost::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete WallPost object: permission denied");
		}
		ActionManager::notify("WALL_POST_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create WallPost object: permission denied");
		} else if (!$this->isNew() && !$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update WallPost object: permission denied");
		}
		
		$errors = array();
		
		if (empty($this->comment)) {
			$errors['comment'] = Language::getSessText('WallPost.EmptyComment');
		}
		
		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}
		
		parent::save($con);
		ActionManager::notify("WALL_POST_SAVE", $this);
	}

	/**
	 * Renders a link to the messagewall post indicated in the params parameter
	 *
	 * @param array: $params
	 * @return String: link to the wall post
	 */
	function renderAssociationLink($params) {
		$arr = explode("|", $param[2]);
		$link = "";		
		$wallPost=WallPostPeer::retrieveByPk($aux[1]);
		if (is_object($wallPost)) {
			$link = GFUrl::getURL()->projectURL(GFUrl::getUrl()->getProject(), '', 'messagewall', '', array('id'=>$wallPost->getWallPostId()));
			$link = '<a href="'.GFUrl::getFullUrl($link).'">'.$wallPost->getComment().'</a>';	
			$link=array('Message Wall Post: '.$link, $params[2],	$params[3]);
		}
		return $link;
	}

}
?>
