<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2010-11-10
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'core/om/BaseWallPostComment.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class WallPostComment extends BaseWallPostComment implements GFPermission {

	/**
	 * isPublic - true/false
	 * @return boolean
	 */
	function isPublic() {
		return $this->getIsPublic();
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		if ($this->userIsAdmin()) {
			return true;
		}

		$user_session = Session::getSession()->getUser();
		$project = GFUrl::getUrl()->getProject();

		$wallpost = WallPostPeer::retrieveByPk($this->getWallPostId());
		if (!is_object($wallpost)) {
			return false;
		}
		if (Session::getSession()->isLoggedIn() && $wallpost->getSection() == "project" && is_object($project) && ($user_session->isProjectMember($project) || $project->isPublic()) ) {
			return true;
		}	
		if (Session::getSession()->isLoggedIn() && $wallpost->getSection() == "user" && $this->getPostedBy() == $user_session->getUserId()) {
			return true;
		}

		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		if (Session::getSession()->isSiteAdmin()) {
			return true;
		}
		/*
		//requires manual customization
		$perm = $this->getProject()->getPermission();
		if ($perm->check(self::SECTION, SELF::ADMIN)) {
			return true;
		}
		*/
		
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return true;
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		if ($this->userIsAdmin()) {
			return true;
		}
		if ($this->isPublic()) {
			return true;
		}
		/*
		//requires manual customization
		$perm = $this->getProject()->getPermission();
		if ($perm->check(self::SECTION, SELF::MEMBER)) {
			return true;
		}
		*/
		return false;
	}

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see WallPostComment::setDeleted()
	 * @see WallPostComment::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete WallPostComment object: permission denied");
		}
		ActionManager::notify("WALL_POST_COMMENT_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		$errors = array();
		if ($this->isNew() && !$this->userCanCreate()) {
			$errors[] = "Could not create WallPostComment object: permission denied";
		} else if (!$this->isNew() && !$this->userCanUpdate()) {
			$errors[] = "Could not update WallPostComment object: permission denied";
		}

		$errors = array();
		
		if (empty($this->comment)) {
			$errors['comment'] = Language::getSessText('WallPost.EmptyComment');
		}
		
		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}
		
		parent::save($con);
		ActionManager::notify("WALL_POST_COMMENT_SAVE", $this);
		return $errors;
	}


}
?>
