<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez ruben@gforgegroup.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once("propel/Propel.php");
require_once("lib/core/Language.php");
require_once("lib/external/jpgraph/jpgraph.php");
require_once("lib/external/jpgraph/jpgraph_pie.php");
require_once("lib/external/jpgraph/jpgraph_pie3d.php");
require_once("lib/external/jpgraph/jpgraph_canvas.php");

class GraphicPie {

	private $RecordSet;
	private $SQLQuery;
	private $Language;

	private $title;
	private $width;
	private $height;
	
	private $labelformat;
	private $legendformat;
	private $legendformat_others = false;
	private $fieldskey;
	private $fieldsval;
	
	private $auxvalue1;
	private $auxvalue2;
	
	private $total = 0;
	
	private $center = 0.40;
	private $size = 0.32;
	private $theme;
	private $data = array();
	private $dbdata = array();
	private $labels = array();
	private $legends = array();
	
	function __construct($title, $width, $height) {
		$this->title = $title;
		$this->width = $width;
		$this->height = $height;
		$this->Language = Language::getSessLang();
	}
	
	public function setSQL($query) {
		$this->SQLQuery = $query;
	}
	
	public function setSQLFields($keyname, $valname) {
		$this->fieldskey = $keyname;
		$this->fieldsval = $valname;
	}
	
	public function setCenter($center) {
		$this->center = $center;
	}

	public function setSize($size) {
		$this->size = $size;
	}

	public function setTheme($theme) {
		$this->theme = $theme;
	}

	public function showPie() {
		if ($this->SQLQuery == null || $this->fieldskey == null ||
			$this->fieldsval == null) {
			$this->data[] = 1;
			$this->legends[] = $this->Language->getText('Common.NotEnoughData');
		} else {
			$this->runSQL();
			$this->buildDbData();
			if ($this->total < 1) {
				$this->data[] = 1;
				$this->legends[] = $this->Language->getText('Common.NotEnoughData');
			} else {
				$this->buildOthers();
				$this->buildData();
			}
		}
			
		$graph = new PieGraph($this->width, $this->height);
		//$graph->SetShadow();
		$graph->setBox(false);
		$graph->setFrame(false);
		$font_prop = $this->Language->getFontProperties(FF_DEFAULT, FS_BOLD, 10);
		$graph->title->SetFont($font_prop[0], $font_prop[1], $font_prop[2]);
		$graph->title->Set($this->title);
		$font_prop = $this->Language->getFontProperties(FF_DEFAULT, FS_NORMAL, 7);
		$graph->legend->setFont($font_prop[0], $font_prop[1], $font_prop[2]);
		
		$p = new PiePlot3D($this->data);
		$p->ExplodeSlice(0);
		$p->SetCenter($this->center);
		$p->SetSize($this->size);
		$p->SetLegends($this->legends);

		if ($this->theme != null) 
			$p->SetTheme($this->theme);

		$graph->Add($p);
		$graph->Stroke();
	}

	/*
	public function setLabelFormat($label) {
		$this->labelformat = $label;
	}
	*/

	public function setLegendFormat($legend, $apply_others=false) {
		$this->legendformat = $legend;
		$this->legendformat_others = $apply_others;
	}
	
	public function setAuxValue1($value) {
		$this->auxvalue1 = $value;
	}

	public function setAuxValue2($value) {
		$this->auxvalue2 = $value;
	}

	private function runSQL() {
		$con = Propel::getConnection(TrackerPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$this->RecordSet = $stmt->executeQuery($this->SQLQuery, ResultSet::FETCHMODE_ASSOC);
	}
	
	private function buildDbData() {
		if ($this->RecordSet != null && $this->RecordSet->getRecordCount() > 0) {
			while($this->RecordSet->next()) {
				$key = $this->RecordSet->getString($this->fieldskey);
				$val = $this->RecordSet->getInt($this->fieldsval);
				$this->total += $val;
				$this->dbdata[$key] = $val;
			}
		}
	}

	private function buildOthers() {
		$mydata = $this->dbdata;
		$this->dbdata = array();
		foreach ($mydata as $key => $val) {
			if (($val / $this->total) < .05) {
				if (isset($this->dbdata['Others'])) {
					$this->dbdata['Others'] += $val;
				} else {
					$this->dbdata['Others'] = $val;
				}
			} else {
				$this->dbdata[$key] = $val;
			}
		}
	}
	
	private function buildData() {
		foreach ($this->dbdata as $key => $val) {
			$this->data[] = $val;
			if ($this->legendformat == null || ($key == 'Others' && $this->legendformat_others == false)) {
				$key = StringUtils::trimString($key, 20);	// If we allow the full key, the graph may get overwritten by the legend
				$this->legends[] = $key. ': '.$val;
			} else {
				$legend = 'error';
				eval($this->legendformat);
				$this->legends[] = $legend;
			}
		}
	}
	
	private function showEmptyImg() {
		$data = file_get_contents(dirname(__FILE__).'/EmptyPie.gif');
		header('Content-Type: image/gif');
		header('Content-Disposition: filename=EmptyPie.gif');
		header('Connection: close');
		header('Content-Length: '.strlen($data)); 
		echo $data;
	}
}


?>
