<?php

require_once 'ckeditor/ckeditor_php5.php';
require_once 'GFUrl.php';

class GFCKEditor extends CKEditor{
	
	const BASIC = 1; 
	const FULL = 2;
	private $allowedElements = array();
	private $type = GFCKEditor::BASIC;
	private $purifier_ckeditor_map = array();
	public  $config = array();
	public  $events = array();
	public  $basePath;

	/**
	 * Class constructor
	 * @access public
	 * @param integer $type OPTIONAL, default GFCKEditor::BASIC = 1
	 */
	public function __construct($type=GFCKEditor::BASIC){
		$this->type = $type;

		//Load allowed elements from html purify config
		$this->allowedElements = $this->parseConfig();

		//Load mapping (elements -> ckeditor buttons)
		$this->loadMap();

		//set base path
		$this->basePath = '/ckeditor/';

		//set DOCTYPE
		$this->config['docType']='<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">';

		//Avoid indentation and line breaks in html source formatting
		$this->events['instanceReady'] = $this->sourceFormat();
		
		//Get startup mode from user preferences, if available
		$user = Session::getSession()->getUser();
		if (is_object($user)) {
			$c = new Criteria();
			$c->add(UserPreferencePeer::USER_ID, $user->getUserId());
			$c->add(UserPreferencePeer::PREFERENCE_NAME, 'chkeditormode');
			$user_setting = UserPreferencePeer::doSelectOne($c);
			
			$startup_mode = '';
			if (is_object($user_setting)) {
				$startup_mode = $user_setting->getPreferenceValue();
			} 
			if ($startup_mode == 'source' || $startup_mode == 'wysiwyg') {
				$this->config['startupMode'] = $startup_mode;
			}
		}
		
		//Set toolbar
		if($this->type == GFCKEditor::FULL){
			$this->setFullToolbar();
		} else {
			$this->setBasicToolbar();
		}

		//Set connectors for file upload
		//$this->setConectors();
	
		parent::__construct(GFUrl::getUrl()->buildURL($this->basePath));

	}

	/**
	 * sets the connectors for file upload
	 */
	public function setConnectorURLs($ref_id){
		$ref_id = Validator::SanitizeInt($ref_id);
		$this->config['filebrowserUploadUrl'] = $this->basePath . 
			'filemanager/upload/php/upload.php?ref_id='.$ref_id;
	}
	
	/**
	 * sets the available buttons for the basic toolbar for this CKEditor instance
	 * @access private
	 */
	private function setBasicToolbar(){
		$toolbar=array('Source'=>1, 'Bold'=>0, 'Italic'=>0,'Underline'=>0,
			'Strike'=>0,'NumberedList'=>0,'BulletedList'=>0,'-'=> 1,'Link'=>0,
			'Unlink'=>0,'Anchor'=>0, '-'=>1, 'About'=>1,'Image'=>0, 'Syntaxhighlight'=>1);
		
		foreach($this->purifier_ckeditor_map as $button=>$elements){
			$button_enabled = 0;
			foreach($elements as $element=>$attributes){
				if(array_key_exists($element, $this->allowedElements) &&
					(is_null($attributes) || $this->arrayIsInArray($attributes,$this->allowedElements[$element]))){

					$button_enabled = 1;

				}
			}
			$toolbar[$button] = $button_enabled;
		}

		foreach($toolbar as $button => $enabled){
			if($enabled == 1){
				$this->config['toolbar'][][] = $button;
			}
		}
	}

	/**
	 * sets the available buttons for the full toolbar for this CKEditor instance
	 * @access private
	 */
	private function setFullToolbar(){
		$toolbar = array(
			//Document section
			array('Source'=>1,'-'=>1,'Save'=>0,'NewPage'=>1,'DocProps'=>1,
					'Preview'=>1,'Print'=>1,'-'=>0,'Templates'=>0),
			//Clipboard section
			array('Cut'=>1,'Copy'=>1,'Paste'=>1,'PasteText'=>1,
					'PasteFromword'=>1,'-'=>1,'Undo'=>1,'Redo'=>1),
			
			//Editing section 
			array('Find'=>1,'Replace'=>1,'-'=>1,'SelectAll'=>1,'-',
				'SpellChecker'=>1,'Scayt'=>1),
			
			//Basic styles section
			array('Bold'=>0,'Italic'=>0,'Underline'=>0,'Strike'=>0,
				'Subscript'=>0,'Superscript'=>0,'-'=>1,'RemoveFormat'=>1),

			//Paragraph section
			array('NumberedList'=>0,'BulletedList'=>0,'-'=>1,
				'Outdent'=>0,'Indent'=>0,'-','Blockquote'=>0,'CreateDiv'=>0,'-'=>1,
				'JustifyLeft'=>0,'JustifyCenter'=>0, 'JustifyRight'=>0, 
				'JustifyBlock'=>0,'-'=>1,'BidiLtr'=>0, 'BidiRtl'=>0),
			//Links section
			array('Link'=>0, 'Unlink'=>0, 'Anchor'=>0),
		
			//Insert section
			array('Syntaxhighlight'=>1,'Image'=>0, 'Flash'=>0, 'Table'=>0,'HorizontalRule'=>0,
				'Smiley'=>0,'SpecialChar'=>0,'PageBreak'=>0, 'Iframe'=>0),
			
			//Styles section
			
			array('Styles'=>0, 'Format'=>0, 'Font'=>0, 'FontSize'=>0),
			
			//Colors section
			array('TextColor'=>0, 'BgColor'=>0),

			//Tools section
			array('Maximize'=>1, 'ShowBlocks'=>1, 'About'=>1)
		);

		//Walk through toolbar and enabled buttons according to the mapping
		foreach($this->purifier_ckeditor_map as $button=>$elements){
			$button_enabled = 0;
			foreach($elements as $element=>$attributes){
				if(array_key_exists($element, $this->allowedElements)){
					if(is_null($attributes) || $this->arrayIsInArray($attributes, $this->allowedElements[$element])){
						$button_enabled = 1;
					}
				}
			}

			//Search for the proper button inside each section
			foreach($toolbar as $section=>$items){
				if(array_key_exists($button, $items)){
					$toolbar[$section][$button] = $button_enabled;
				}
			}

		}
		$t = 0;
		foreach($toolbar as $section => $items){
			foreach($items as $item=>$enabled){
				if($enabled == 1){
					$this->config['toolbar'][$section][] = $item;
				}
			}
			$t++;
		}

		//Insert a break after 4 sections
		for($i = 4; $i < $t; $i+=4){
			array_splice($this->config['toolbar'], $i, 0, array('/'));
		}

	}
				

	/**
	 * builds a mapping array betwen htmlpurifier and CKEditor entities
	 * @access private
	 * @return void
	 * *[style],p,span should be the default or we will end with
	 *	a silly long list of elements and attributes
	 */
	private function loadMap(){
		//These are the list of _minimal_ elements and attributes needed by 
		//a given CKEditor toolbar button/functionality. Some advanced functionality
		//might not be available once the the html is saved if the html purify rule 
		//doesn't match the advanced attribute (ie: img will need the style attribute
		//for setting the height, widht, etc)
		//Structure
	   //"Button"=>array(elements), where element is ('tag'=>array('atributes));
		$this->purifier_ckeditor_map = array(
			'Bold'=>array('strong'=>null),
			'Italic'=>array('em'=>null),
			'Underline'=>array('u'=>null),
			'Strike'=>array('strike'=>null),
			'Subscript'=>array('sub'=>null),
			'Superscript'=>array('sup'=>null),

			//Paragraph
			'BulletedList'=>array('ul'=>null,'li'=>null),
			'NumberedList'=>array('ol'=>null,'li'=>null),
			//The following buttons needs the style attribute set for every element
			'Outdent'=>array('*'=>array('style')),
			'Indent'=>array('*'=>array('style')),
			'JustifyLeft'=>array('*'=>array('style')),
			'JustifyRight'=>array('*'=>array('style')),
			'JustifyBlock'=>array('*'=>array('style')),
			'BidiLtr'=>array('*'=>array('rtl')),
			'BidiRtl'=>array('*'=>array('rtl')), //Actually this will only remove the rtl attribute
			'Blockquote'=>array('blockquote'=>null),

			//Links
			'Link'=>array('a'=>array('href')),
			'Unlink'=>array('a'=>array('href')),
			'Anchor'=>array('a'=>array('href')),

			//Insert
			'Image'=>array('img'=>array('alt','src')),
			'Syntaxhighlight'=>array('pre'=>array('class')),
			'Table'=>array('table'=>array('align','center','border','cellpadding',
				'cellspacing','summary'), 'caption'=>null,'thead'=>null,'tr'=>null,
				'td'=>array('colspan','rowspan'),'th'=>array('scope')),
			'HorizontalRule'=>array('hr'=>null),
			'PageBreak'=>array('div'=>array('style')),

			//Styles
			'Format'=>array('h1'=>array('style'),'h2'=>array('style'),
				'h3'=>array('style'),'h4'=>array('style'),'h5'=>array('style'),
				'h6'=>array('style'),'pre'=>null,'address'=>null),
			'Font'=>array('span'=>array('style')),
			'FontSize'=>array('span'=>array('style')),
			'TextColor'=>array('span'=>array('style')),
			'BGColor'=>array('span'=>array('style'))
		);

	}
	
	/**
	 * gets CKConfig from HTML Purifier config syntax
	 * @access private
	 * @return array
	 */
	private function getCKConfig(){
		$ckconfig = array();
		//Always display the Source button
		if($this->type == self::BASIC){
			
		} else{
		}
		if($this->type == GFCKEditor::BASIC){
					} else {
			$ckconfig = $this->setFullToolbar();
		}

		return $ckconfig;
	}

	/**
	 * parses config in htmlpurifier syntax
	 * @return array of allowed elements with the structure ['element']=>array of attributes
	 */
	private function parseConfig(){
		if($this->type == GFCKEditor::BASIC){
			$config = GFConf::get('system.basicHTMLWhiteList');
		} else {
			$config = GFConf::get('system.fullHTMLWhiteList');
		}
		if(!$config) throw new Exception("Couldn't read configuration for html white list");

		$items = explode(',', $config);

		$allowed_elements= array();
		foreach($items as $item){
			//Extract allowed attributes for this element
			$allowed_attributes = strstr($item, '[');
			if($allowed_attributes){
				$allowed_attributes = trim($allowed_attributes, '[]');
				$allowed_attributes = explode('|', $allowed_attributes);
			} else {
				$allowed_attributes = array();
			}
			preg_match('/[[:alpha:]\*]+(\[)?/',$item, $tmp);
			$allowed_elements[trim($tmp[0],'[]')] = $allowed_attributes;
		}
		//Attributes that applies to all elements
		if(array_key_exists('*', $allowed_elements)){
			$global_attributes = $allowed_elements['*'];
			foreach($allowed_elements as $element => $attributes){
				$allowed_elements[$element] = array_unique(array_merge($attributes,$global_attributes));
			}
		}

		return $allowed_elements;
	}

	/**
	 * gets allowed elements
	 * @return array
	 */
	public function getAllowedElements(){
		return $this->allowedElements;
	}

	/**
	 * creates an instance of GFCKEditor
	 * @return object
	 */
	public function editor($name, $value){
 		echo parent::editor($name, $value, $this->config, $this->events);
		if ($this->type == self::BASIC) {
			echo '<div id="GFCKAllowedElements" style="visibility:hidden"><p>' . $this->getAllowedElementsAsHtml() . '</p></div>';
			echo '<script type="text/javascript">
				var ckeditor = CKEDITOR.instances[\''.$name.'\'];
				ckeditor.on(\'mode\', function() {
					setUserPreference(\'chkeditormode\', this.mode);
					item = document.getElementById(\'GFCKAllowedElements\');
					if (this.mode == \'source\') {
						item.style.visibility = \'visible\';
					} else{
						item.style.visibility = \'hidden\';
					}});
				</script>';

			//echo '<script type="text/javascript"> var ckeditor = CKEDITOR.instances[\'' .$name .'\']; ckeditor.on(\'mode\', function() { setUserPreference(\'chkeditormode\', this.mode); } );</script>';
		}
		
	}

	/**
	 * replaces a textarea element with a CKEditor instance
	 * @access public
	 * @return object
	 */
	public function replace($id){
		$html = parent::replace($id, $this->config, $this->events);
		if ($this->type == self::BASIC) {
			$html .= '<div id="GFCKAllowedElements_'.$id.'" style="visibility:hidden"><p>' . $this->getAllowedElementsAsHtml() . '</p></div>';
			$html .= '<script type="text/javascript">
				var ckeditor_r = CKEDITOR.instances[\''.$id.'\'];
				ckeditor_r.on(\'mode\', function() {
					setUserPreference(\'chkeditormode\', this.mode);
					item = document.getElementById(\'GFCKAllowedElements_'.$id.'\');
					if (this.mode == \'source\') {
						item.style.visibility = \'visible\';
					} else{
						item.style.visibility = \'hidden\';
					}
				});
				</script>';

			//echo '<script type="text/javascript"> var ckeditor = CKEDITOR.instances[\'' .$name .'\']; ckeditor.on(\'mode\', function() { setUserPreference(\'chkeditormode\', this.mode); } );</script>';
		}

		return $html;
	}

	public function sourceFormat(){
		$js_snippet = "function (ev) {
			this.dataProcessor.writer.setRules( 'p',
				{
					indent: false,
					breakBeforeOpen: true,
					breakAfterOpen: true,
					breakBeforeClose: false,
					breakAfterClose: true
				});
		}";

		return $js_snippet;
	}
	/**
	 * checks if all elements of array1 are in array2
	 * @access private
	 * @param array $arr1 the child array
	 * @param array $arr2 the parent array
	 * @return boolean
	 */
	private function arrayIsInArray($arr1, $arr2){
		if(empty($arr1)){
			return true;
		}

		foreach($arr1 as $item){
			if(!in_array($item, $arr2)){
				return false;
			}
		}

		return true;
	}

	private function getAllowedElementsAsHtml(){

		$str = Language::getSessText('GFCKEditor.AllowedElements');
		$str .='<span style="color:rgb(90, 90, 137)">';

		foreach ($this->allowedElements as $element => $attr) {
				$str .= " $element";
				if(count($attr)) {
					$str .= '[' . implode('|', $attr) . ']';
				}
			$str .= ' ';
		}
		$str .= '</span>';

		return $str;
	}
		
	
}
