<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
class HTMLWidget {

	const OPTION_NONE = 100;
	const OPTION_ANY = '';

	/**
	 * 	select - build a select box from the array of objects and selected item.
	 * @param	string	$name The name of the widget.
	 * @param	array	$objects The array of objects, which must implement GFWidget, to be displayed.
	 * @param	mixed	$default_selected The id of the default  selected item or an object
	 * @param	boolean $show_none	Whether to show a 'None' row.
	 * @param	string	$text_none The text to show if you show a 'None' row.
	 * @param	boolean	$show_any Whether to show a 'Any' row.
	 * @param	string	$text_any The text to show if you show a 'Any' row.
	 * @param	array 	$override_text An array with names to display instead of $object->getName(). Should be the same size as the object array.
     * @param  boolean $disabled is this control disabled?
     * @param  array   $disabled_elements array of disabled elements ids
     *
     * @return string
	 */
	static function select($name, $objects, $selected_object = null,
	                       $default_selected = HTMLWidget::OPTION_NONE, $show_none = true,
	                       $text_none = null, $show_any = false, $text_any = null, $override_text = null, $disabled = false,
                            $disabled_elements=array()) {

		$language = Language::getSessLang();

		if ($show_none && is_null($text_none)) {
			$text_none = $language->getText('Options.None');
		}
		if ($show_any && is_null($text_any)) {
			$text_any = $language->getText('Options.Any');
		}

		if (!$selected_object) {
			$selected = $default_selected;
		} elseif (is_object($selected_object)) {
			$selected = $selected_object->getID();
		} else {
			$selected = $selected_object;
		}
		$disabledStr = $disabled ? 'disabled' : '';
		$ret = '<select name="' . $name . '" ' . $disabledStr . '>';
		if ($show_any) {
			$ret .= '
				<option value="' . HTMLWidget::OPTION_ANY . '">' . $text_any . '</option>';
		}
		if ($show_none) {
			$ret .= '
				<option value="' . HTMLWidget::OPTION_NONE . '"' . (($selected == HTMLWidget::OPTION_NONE) ? ' selected="selected"' : '' ) . '>' . $text_none . '</option>';
		}


		for ($i = 0; $i < count($objects); $i++) {
			$object = $objects[$i];
			$id = $object->getID();

			if (!is_null($override_text) && count($override_text) > $i && !is_null($override_text[$i])) {
				$text = $override_text[$i];
			} else {
				//Hack for the body user
				if($object->getName() == 'Nobody User') {
					$text = Language::getSessText('Tracker.Nobody');
				} else {
					$text = $object->getName();
				}
			}

			$ret .= '
				<option value="' . $object->getID() . '"' . (("$selected" == "$id") ? ' selected="selected" ' : '') .
                    (in_array($object->getID(), $disabled_elements) ? ' disabled="disabled"' : '' ) . '>' . $text . '</option>';
			// Note the quotes above. It solves this:
			// http://bugs.php.net/bug.php?id=6766
		}
		$ret .= '</select>';

		return $ret;
	}

	static function radio($name, $label, $value, $selectedValue = false, $extra_attributes = array()) {
		if ($value == $selectedValue) $extra_attributes['checked'] = 'checked';

		$extra = '';
		foreach ($extra_attributes as $attrName => $attrVal) {
			$extra .= "$attrName=\"$attrVal\" ";
		}

		return "<input type=\"radio\" name=\"$name\" value=\"$value\" $extra />$label";
	}

	/**
	 * 	radio - build a radiobutton list from the array of objects and selected item.
	 * 	@param	string	The name of the widget.
	 * 	@param	array	The array of objects, which must implement GFWidget, to be displayed.
	 * 	@param	object	The optional object, which must implement GFWidget, that is selected (or its ID).
	 * 	@param	string	The id of the default selected item.
	 * 	@param	boolean	Whether to show a 'None' row.
	 * 	@param	string	The text to show if you show a 'None' row.
	 * 	@param	boolean	Whether to show a 'Any' row.
	 * 	@param	string	The text to show if you show a 'Any' row.
	 */
	static function radioSet($name, $objects, $selected_object = null, $default_selected = HTMLWidget::OPTION_NONE, $show_100 = true, $text_100 = null, $show_any = false, $text_any = null) {

		$language = Language::getSessLang();

		if ($show_100 && is_null($text_100)) {
			$text_100 = $language->getText('Options.None');
		}
		if ($show_any && is_null($text_any)) {
			$text_any = $language->getText('Options.Any');
		}

		if (!$selected_object) {
			$selected = $default_selected;
		} elseif (is_object($selected_object)) {
			$selected = $selected_object->getID();
		} else {
			$selected = $selected_object;
		}

		$ret = '';
		if ($show_100) {
			$ret .= self::radio($name, $text_100, HTMLWidget::OPTION_NONE, $selected)."<br />\n";
			//$ret .= '<input type="radio" name="' . $name . '" value="' . HTMLWidget::OPTION_NONE . '"' . (($selected == HTMLWidget::OPTION_NONE) ? ' checked="checked"' : '' ) . ' />' . $text_100 . "<br />\n";
		}

		foreach ($objects as $object) {
			$id = $object->getID();
			$ret .= self::radio($name, $object->getName(), $object->getID(), $selected)."<br />\n";
			//$ret .= '<input type="radio" name="' . $name . '" value="' . $object->getID() . '"' . (("$selected" == "$id") ? ' checked="checked" ' : '' ) . ' />' . $object->getName() . "<br />\n";
			// Note the quotes above. It solves this:
			// http://bugs.php.net/bug.php?id=6766
		}

		if ($show_any) {
			$ret .= self::radio($name, $text_any, HTMLWidget::OPTION_ANY, $selected)."<br />\n";
		}

		return $ret;
	}

	/**
	 * 	checkbox - build check boxes from the array of objects and selected item.
	 * 	@param	string	The name of the widget.
	 * 	@param	array	The array of objects, which must implement GFWidget, to be displayed.
	 * 	@param	array	The optional array of id's of the selected objects.
	 * 	@param	string	The id of the default selected item.
	 * 	@param	boolean	Whether to show a 100 row.
	 * 	@param	string	The text to show if you show a 100 row.
	 * 	@param	boolean	Whether to show a 'Any' row.
	 * 	@param	string	The text to show if you show a 'Any' row.
	 */
	static function checkbox($name, $objects, $selected_objects = '', $selected_function_name = 'getId', $default_selected = 100, $show_100 = true, $text_100 = null, $show_any = false, $text_any = null) {

		$ret = '';
		$language = Language::getSessLang();

		if ($show_100 && is_null($text_100)) {
			$text_100 = $language->getText('Options.None');
		}
		if ($show_any && is_null($text_any)) {
			$text_any = $language->getText('Options.Any');
		}
		if ($show_any) {
			$ret .= '
		<input type="checkbox" name="' . $name . '" value=""/> ' . $text_any . '<br />';
		}
		if ($show_100) {
			$ret .= '
		<input type="checkbox" name="' . $name . '" value="100"';
			foreach ($selected_objects as $selected) {
				$ret .= (($selected == 100) ? ' CHECKED' : '' );
			}
			$ret .= '/>' . $text_100 . '<br />';
		}

		foreach ($objects as $object) {
			$ret .= '
		<input type="checkbox" name="' . $name . '" value="' . $object->getID() . '"';

			foreach ($selected_objects as $selected) {
				$ret .= (($selected == $object->getID()) ? ' CHECKED' : '' );
			}
			$ret .= '/>' . $object->getName() . '<br />';
		}

		return $ret;
	}

	/**
	 * 	multiselect - build multiselect box from the array of objects and selected item.
	 * 	@param	string	The name of the widget.
	 * 	@param	array	The array of objects, which must implement GFWidget, to be displayed.
	 * 	@param	array	The optional array of selected id's.
	 * 	@param	string	The id of the default selected item.
	 * 	@param	boolean	Whether to show a 100 row.
	 * 	@param	string	The text to show if you show a 100 row.
	 * 	@param	boolean	Whether to show a 'Any' row.
	 * 	@param	string	The text to show if you show a 'Any' row.
	 */
	static function multiselect($name, $objects, $selected_objects = array(), $default_selected = 100, $show_100 = true, $text_100 = null, $show_any = false, $text_any = null, $size = 10, $disabled = false, $override_text = null, $alternative_100='',
$disabled_elements = array()) {

		//	$size=;

		$language = Language::getSessLang();

		if ($show_100 && is_null($text_100)) {
			$text_100 = $language->getText('Options.None');
		}
		if ($show_any && is_null($text_any)) {
			$text_any = $language->getText('Options.Any');
		}
		if ($disabled) {
			$str_disabled = ' disabled';
		} else {
			$str_disabled = '';
		}

		$ret = '
		<select name="' . $name . '" multiple="multiple" size="' . $size . '"' . $str_disabled . '>';

		if ($show_any) {
			$ret .= '
		<option value=""> ' . $text_any . '</option>';
		}
		if ($show_100) {
			$value_100 = '100';
			if ($alternative_100 != '') {
				$value_100 = $alternative_100;
			}
			
			$ret .= '
		<option value="' . $value_100 . '"';
			foreach ($selected_objects as $selected) {
				$ret .= (($selected == $value_100) ? ' selected="selected"' : '' );
			}
			$ret .= '>' . $text_100 . '</option>';
		}

		for ($i = 0; $i < count($objects); $i++) {
			$object = $objects[$i];

			if (!is_null($override_text) && count($override_text) > $i && !is_null($override_text[$i])) {
				$text = $override_text[$i];
			} else {
				if ($object->getName() == 'Nobody User') {
					$text = Language::getSessText('Tracker.Nobody');
				} else {
					$text = $object->getName();
				}
			}

			$ret .= '
			<option value="' . $object->getID() . '"';
			if (array_search($object->getID(), $selected_objects) !== false) {
				$ret .= ' selected="selected" ';
			}

			if (array_search($object->getId(), $disabled_elements) !== false ) {
				$ret .= ' disabled="disabled" ';
			}

			$ret .= ' >'.$object->getName() .'</option>';

		}
		$ret .= '
		</select>';

		return $ret;
	}

	/**
	 * selectBoxFromUsers - build a select box from an array of users
	 * @param array  Array of users to build the select form
	 * @param string The name of the widget
	 * @param string The id of the user that is selected
	 */
	static function selectBoxFromUsers($users, $select_name, $checked_user_id) {
		$arr = array();
		foreach ($users as $u) {
			$arr[$u->getId()] = $u->getName();
		}
		asort($arr);
		return HTMLWidget::selectBoxFromAssoc($arr,$select_name, $checked_user_id,false,false);
	}

	/**
	 * 	selectBoxFromAssoc - build a select box from the associative array.
	 * 	@param	array	Associative array to build the select from.
	 * 	@param	string	The name of the widget.
	 * 	@param	string	The id of the object, which must implement GFWidget, that is selected.
	 * 	@param	boolean	Whether to swap the keys/values of the associative array
	 * 	@param	boolean	Whether to show a 'None' row.
	 * 	@param	string	The text to show if you show a 'None' row.
	 * 	@param	boolean	Whether to show a 'Any' row.
	 * 	@param	string	The text to show if you show a 'Any' row.
	 */
	static function selectBoxFromAssoc(
	$arr, $select_name, $checked_val = '', $swap = false, $show_100 = true, $text_100 = null, $show_any = false, $text_any = null) {
		if ($swap) {
			$keys = array_values($arr);
			$vals = array_keys($arr);
		} else {
			$vals = array_values($arr);
			$keys = array_keys($arr);
		}
		return self::selectBoxFromArrays($keys, $vals, $select_name, $checked_val, $show_100, $text_100, $show_any, $text_any);
	}

	/**
	 * 	selectBoxFromArrays - build a select box from the arrays.
	 * 	@param	array	keys (ids) to build the select box from.
	 * 	@param	array	values (texts) to build the select box from.
	 * 	@param	string	The name of the widget.
	 * 	@param	string	The id of the element that is selected.
	 * 	@param	boolean	Whether to show a 'None' row.
	 * 	@param	string	The text to show if you show a 'None' row.
	 * 	@param	boolean	Whether to show a 'Any' row.
	 * 	@param	string	The text to show if you show a 'Any' row.
	 */
	static function selectBoxFromArrays(
	$vals, $texts, $select_name, $checked_val = '', $show_100 = true, $text_100 = null, $show_any = false, $text_any = null) {
		$rows = count($vals);

		$language = Language::getSessLang();

		if ($show_100 && is_null($text_100)) {
			$text_100 = $language->getText('Options.None');
		}
		if ($show_any && is_null($text_any)) {
			$text_any = $language->getText('Options.Any');
		}
		$return = '
			<select name="' . $select_name . '">';
		if ($show_any) {
			$return .= '
				<option value="">' . $text_any . '</option>';
		}
		if ($show_100) {
			$return .= '
				<option value="100"' . (( $checked_val == 100 ) ? ' selected="selected"' : '' ) . '>' . $text_100 . '</option>';
		}
		for ($i = 0; $i < $rows; $i++) {
			 if (is_array($texts[$i])) {
                $text = $texts[$i]['name'];
                $attr = $texts[$i]['attr'];
            } else {
                $text = $texts[$i];
                $attr = '';
            }
			$return .= '
				<option '.$attr.' value="' . $vals[$i] . '"';
			if ($vals[$i] == $checked_val) {
				$return .= ' selected="selected"';
			}

			$return .= '>' . $text . '</option>';
		}

		$return .= '
			</select>';
		return $return;
	}

	/**
	 * 	multiSelectFromAssoc - build a multipleselect box from the associative array.
	 * 	@param	string	The name of the widget.
	 * 	@param	array	Associative array to build the select from.
	 * 	@param	string	The array of id's of the objects that are selected.
	 * 	@param	string	The id of the object that is selected if no others are selected.
	 * 	@param	boolean	Whether to show a 'None' row.
	 * 	@param	string	The text to show if you show a 'None' row.
	 * 	@param	boolean	Whether to show a 'Any' row.
	 * 	@param	string	The text to show if you show a 'Any' row.
	 * 	@param	string	The size (number of rows) of this multiselect box.
	 * 	@param	boolean	Whether to swap the keys/values of the associative array
 	 *	@param 	boolean	Mark the select as disabled.
     * @param array ids of disabled_elements
	 */
	static function multiSelectFromAssoc($name, $arr, $selected_objects = array(), $default_selected = 100, $show_100 = true, $text_100 = null, $show_any = false, $text_any = null, $size = 10, $swap = false, $disabled = false, $disabled_elements=array()) {
		if ($swap) {
			$keys = array_values($arr);
			$vals = array_keys($arr);
		} else {
			$keys = array_keys($arr);
			$vals = array_values($arr);
		}
		return self::multiSelectFromArrays($keys, $vals, $name, $selected_objects, $default_selected, $show_100, $text_100, $show_any, $text_any, $size, $disabled, $disabled_elements);
	}

	/**
	 * 	multiSelectFromAssoc - build a multipleselect box from the associative array.
	 * 	@param	array	keys (ids) to build the select box from.
	 * 	@param	array	values (texts) to build the select box from.
	 * 	@param	string	The name of the widget.
	 * 	@param	string	The array of id's of the objects that are selected.
	 * 	@param	string	The id of the object that is selected if no others are selected.
	 * 	@param	boolean	Whether to show a 'None' row.
	 * 	@param	string	The text to show if you show a 'None' row.
	 * 	@param	boolean	Whether to show a 'Any' row.
	 * 	@param	string	The text to show if you show a 'Any' row.
	 * 	@param	string	The size (number of rows) of this multiselect box.
	 *	@param 	boolean	Mark the select as disabled.
     * @param array disabled elements ids
	 */
	static function multiSelectFromArrays($keys, $vals, $name, $selected_objects = array(), $default_selected = 100, $show_100 = true, $text_100 = null, $show_any = false, $text_any = null, $size = 10, $disabled = false, $disabled_elements=array()) {

		$language = Language::getSessLang();

		if ($show_100 && is_null($text_100)) {
			$text_100 = $language->getText('Options.None');
		}
		if ($show_any && is_null($text_any)) {
			$text_any = $language->getText('Options.Any');
		}
		$disabledStr = $disabled ? 'disabled' : '';
		$ret = '
		<select name="' . $name . '" multiple="multiple" size="' . $size . '"' . $disabledStr . '>';

		if ($show_any) {
			$ret .= '
		<option value=""> ' . $text_any . '</option>';
		}
		if ($show_100) {
			$ret .= '
		<option value="100"';
			foreach ($selected_objects as $selected) {
				$ret .= (($selected == 100) ? ' selected="selected"' : '' );

			}

			$ret .= '>' . $text_100 . '<br />';
		}

		for ($i = 0; $i < count($keys); $i++) {
            if (is_array($vals[$i])) {
                $value = $vals[$i]['name'];
                $attr = $vals[$i]['attr'];
            } else {
                $value = $vals[$i];
                $attr = '';
            }
			$ret .= '
		<option '.$attr.' value="' . $keys[$i] . '"';

			foreach ($selected_objects as $selected) {
				$ret .= (($selected == $keys[$i]) ? ' selected="selected"' : '' );
			}
            if (array_search($keys[$i], $disabled_elements) !== false ) {
                $ret .= ' disabled="disabled" ';
            }
			$ret .= '>' . $value . '</option>';
		}
		$ret .= '
		</select>';

		return $ret;
	}

	/**
	 * 	multiSelectFromAssoc - build a multipleselect box from the associative array.
	 * 	@param	array	nested folders array.
	 * 	@param	array	call function with 0 value.
	 * 	@param	string	call function with 0 value.
	 * 	@param	string	call function with initially empty array.
	 * 	@param	boolean	if true -> first element in tree is root, if false -> no root element.
     *  @param  string  defines the info we want the array values to contain:
     *      'default': array values contain formated option text, 
     *      'array': array values contain an array ['name' => option's text, 'attr' => html attributes to append to html option]
	 */
	static function renderArranged($nested_folders, $folder, $spaces, $result, $root = false, $value_mode = 'default') {
		global $total_nest_count;
		$total_nest_count++;

		if (($root == true) && (sizeof($result) == 0)) {
			$result[0] = "root";
			$spaces += 10;
		}

		if (isset($nested_folders[$folder]) && is_array($nested_folders[$folder])) {

			foreach ($nested_folders[$folder] as $folder) {

                if ($value_mode == 'array') {
                    $result[$folder->getId()] = array('name' => $folder->getName(), 'attr' => '');
                } else {
                    $result[$folder->getId()] = str_repeat('&nbsp;', $spaces) . $folder->getName();
                }
				if (($spaces - 10) > 0) {
					$times = $spaces / 10;
					$result[$folder->getId()] = '';
					for ($i = 0; $i < $times - 1; $i++) {
						$result[$folder->getId()] .= '|' . str_repeat('&nbsp;', $spaces - (10 * ($times - 1)));
					}
                    if ($value_mode == 'array') {
                        $result[$folder->getId()] = array('name' => $folder->getName(), 'attr' => 'style="padding-left: '.(6 + 23 * ($i + 1)).'px"');
                    } else {
                        $result[$folder->getId()] .= '|' . str_repeat('&nbsp;', 10) . '|--' . $folder->getName();
                    }
				} elseif (($spaces - 10) == 0) {
                    if ($value_mode == 'array') {
                        $result[$folder->getId()] = array('name' => $folder->getName(), 'attr' => 'style="padding-left: 29px"');
                    } else {
                        $result[$folder->getId()] = str_repeat('&nbsp;', $spaces - 10) . '|' . str_repeat('&nbsp;', 10) . '|--' . $folder->getName();
                    }
				}
				$result = HTMLWidget::renderArranged($nested_folders, $folder->getId(), $spaces + 10, $result, $root, $value_mode);
			}
			$spaces -= 10;
		}
		return $result;
	}

	/**
	 * 	radio - build a radiobutton list from the array of objects and selected item.
	 * 	@param	string	The type of the widget.
	 */
	static function button($type = "", $text = "", $name = "", $id = "", $onclick = "", $enabled = true, $class = "button", $style = "", $dynamic = false) {
		static $buttonId = 0;

		if (strlen($type) == 0)
			$type = "button";
		if (strlen($id) == 0) {
			$buttonId++;
			if ($dynamic) {
				$id = "__button" . time() . rand() . "__";
			} else {
				$id = "__button" . $buttonId . "__";
			}
		}

		$button = "";

		if ($onclick != "" && $enabled) {
			//$onclick = 'onclick="' . $onclick . '"';
		} else {
			$onclick = "";
		}

		if ($style != "") {
			$style = ' style="' . $style . '"';
		}

		$using_ie6 = (strpos($_SERVER['HTTP_USER_AGENT'], 'MSIE 6.') !== FALSE);
		$buttondiv = "buttondiv";
		if ($using_ie6) {
			$class .= "_ie6";
			$buttondiv .= "_ie6";
		}

		if ($enabled) {
			$button = "<div class=\"$buttondiv\"$style><div class=\"$class\"><a href=\"javascript:;\" style=\"color: #FFFFFF;\" onclick=\"document.getElementById('$id').click();\">$text</a><span></span></div>";
			$button .= "<span style=\"display:none;\"><input type=\"$type\" id=\"$id\" name=\"$name\" onclick=\"$onclick\" /></span></div>";
		} else {
			$button = "<div class=\"buttondiv\"><div class=\"$class btn_disabled\"><a href=\"javascript:;\">$text</a><span></span></div></div>";
		}

		return $button;
	}

	public static function discuss($project, $description, $section, $ref_id, $forum_name, $thread_id=0, $extra_classes = '') {
		if (Session::getSession()->isLoggedIn() &&
			$project &&
			$project->usesPlugin('forum')) {

			$data_thread_id = '';
			if ($thread_id > 0) {
				$data_thread_id = 'data-thread-id="'.$thread_id.'"';
			}

			$ret = "<a href=\"javascript:;\" class=\"discuss {$extra_classes}\" data-section=\"{$section}\" {$data_thread_id} data-ref-id=\"{$ref_id}\" data-forum-name=\"${forum_name}\" onMouseOver=\"tooltip('{$description}')\" onmouseout=\"tooltipexit()\">
						<img src=\"/images/chat_icon_24.png\" alt=\"{$description}\" />
					</a>";

			return $ret;
		} else {
			return '';
		}
	}

	public static function projectChat($project, $extra_classes = '') {
		require_once('core/SettingPeer.php');

		$ret = '';
		if (Session::getSession()->isLoggedIn() &&
			$project &&
			$project->usesPlugin('forum')) {

			$section = Project::SECTION;
			$ref_id = $project->getPrimaryKey();
			$forum_name = $project->getProjectName();
			$description = $forum_name;
			$thread_id = SettingPeer::GetSetting($section, $ref_id, ForumPeer::SETTING_NAME_PROJECT_CHAT);
			$forum = null;

			if ($thread_id !== FALSE) {
				$thread = ForumThreadPeer::retrieveByPk($thread_id);
				if ($thread) {
					$forum = $thread->getForum();
				} else {
					$thread_id = '';
				}
			} else {
				$forum = ForumPeer::getOrCreateForum($section, $ref_id, $forum_name);
				$thread_id = '';
			}
			// #10329 - Don't show project chat icon if the forum is moderated.
			if ($forum && $forum->getModerationLevel() != Forum::NO_MODERATION) {
				return '';
			}

			$ret = "<a href=\"javascript:;\" class=\"discuss {$extra_classes}\" data-thread-id=\"{$thread_id}\" data-section=\"{$section}\" data-ref-id=\"{$ref_id}\" data-forum-name=\"${forum_name}\" onMouseOver=\"tooltip('{$description}')\" onmouseout=\"tooltipexit()\">
					<img src=\"/images/chat_icon_24.png\" alt=\"{$description}\" />
				</a>";
		}
		return $ret;
	}

	/**
	 * This function requires jquery-ui
	 */
	static function inputTextWithAutoComplete($inputId, $values, $atttributes=array()) {

		for ($i = 0; $i < count($values); $i++) {
			$values[$i] = '\''.$values[$i].'\'';
		}
		$values = implode(',', $values);
		$ret = '<script type="text/javascript">
			$(function() {
				var availableTags = ['.$values .

			'];

			function split( val ) {
				return val.split( /,\s*/);
			}

			function extractLast( term ) {
				return split( term ).pop();
			}

			$( "#usernames" )
				.bind( "keydown", function( event ){
					if ( event.keyCode === $.ui.keyCode.TAB &&
						$( this ).data( "ui-autocomplete" ).menu.active ) {
						event.preventDefault();
					}
				})
				.autocomplete({
					minLenght: 0,
					source: function( request, response) {
						response($.ui.autocomplete.filter(
							availableTags, extractLast( request.term)
						));
					},
				focus: function() {
					return false;
				},
				select: function( event, ui ) {
					var terms = split( this.value );
					terms.pop();
					terms.push( ui.item.value );
					terms.push( "" );
					this.value = terms.join ( ", ");
					return false;
				}
			});
		});
		</script>';
		$attr = '';
		foreach($atttributes  as $key => $value) {
			$attr .= ' ' . $key . '='.'"'.$value.'" ';
		}
		$ret .= '<input id="'.$inputId.'" '.$attr.'/>';

		return $ret;
	}

	public static function hiddenFieldSingleValue($name, $value) {
		if (is_object($value)) {
			$valueStr = $value->getID();
		} else {
			$valueStr = $value;
		}
		return '<input type="hidden" name="' . $name . '" value="' . $valueStr . '" />';
	}

	public static function hiddenFieldMultiple($name, $values, $possible_values=array()) {
		//Make sure the name has the "[]" (to allow multiple values)
		if (substr($name, -2) != '[]') $name .= '[]';
		$ret = '';
		foreach ($values as $value) {
			$valueStr = is_object($value) ? $value->getId() : $value;
			if (count($possible_values) > 0 && in_array($valueStr, $possible_values)) {
				$ret .= '<input type="hidden" name="' . $name . '" value="' . $valueStr . '" />' . PHP_EOL;
			}
		}
		return $ret;
	}
}

?>
