<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('html/TroveFilterRenderer.php');

/**
 * This class manages the Trove forms
 */
class ProjectTroveFilterRenderer extends TroveFilterRenderer {

	protected function prepareCountJoinCriteria(Criteria $c) {
		$c->add(TroveLinkPeer::REF_ID, ProjectPeer::PROJECT_ID);
		$c->addSelectColumn(ProjectPeer::COUNT);
	}
	
	protected function getCategoryResults(Criteria $c) {
		$lang = Language::getSessLang();
		$result = '';
		$theme = Session::getSession()->getTheme();

		$c->addJoin(TroveLinkPeer::REF_ID, ProjectPeer::PROJECT_ID);
		$c->add(ProjectPeer::STATUS, Project::STATUS_ACTIVE);
		$c->add(ProjectPeer::IS_PUBLIC, true);
		$rs = ProjectPeer::doSelectRS($c);		// May get lots of results, so use a recordset

		$count = 0;
		$project_result = '';

		$theader = array();
		$theader[] = $lang->getText('Project.ProjectNameName');
		$theader[] = $lang->getText('Project.DescriptionName');
		$theader[] = $lang->getText('Project.TroveCategorization');
		$project_result .= $theme->tableTop($theader);

		while ($rs->next()) {
			$project = new Project();
			$project->hydrate($rs);
			
			$trove = TroveLinkPeer::getSelectedCategoriesTree(Project::SECTION, $project->getPrimaryKey());
			if ($this->matchesFilters($trove)) {
				$count++;

				$row = array();
				$row[] = '<a href="'.GFUrl::getUrl()->projectUrl($project).'">'.$project->getProjectName().'</a>';
				$row[] = $project->getDescription();
			
				$row[] .= $this->getItemTrove($trove, false);
				$project_result .= $theme->tableRow($row);
			}
		}
		
		if ($count == 0) {
			$project_result .= $theme->tableFullRow($lang->getText('Project.TroveNoneFound'));
		}

		$project_result .= $theme->tableBottom();
		
		$result .= $lang->getText('Project.TroveFilterResultCount', array($count));
		$result .= '<br /><br />';
		$result .= $project_result;
		
		return $result;
	}
	
	protected function getCountString($category_count, $children_count) {
		return Language::getSessText('Project.TroveCount', array($category_count, $children_count));
	}
}
?>
