<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('core/Language.php');
require_once('html/Paginator.php');

class StatsRenderer {
	private $rs;
	private $project;
	private $paginator;

	/**
	 * $rs must be a ResultSet obtained by querying the activity_log table
	 */
	public function __construct(Project $project, $rs=null, $paginator=null) {
		$this->rs = $rs;
		$this->project = $project;
		$this->paginator = $paginator;
	}
	
	public function getStatsTable() {
		$Language = Language::getSessLang();
		$template = Session::getSession()->getTheme();
		$res = '';

		$theader=array();
		
		$theader[]=$Language->getText('ProjectActivity.HeaderTime');
		$theader[]=$Language->getText('ProjectActivity.HeaderActivity');
		if(is_null($this->paginator)) {
			$theader[]=$Language->getText('ProjectActivity.HeaderPerson');
		} else {
			$theader[]=$this->paginator->insertLimitOptions($Language->getText('ProjectActivity.HeaderPerson'));
		}
		$res .= $template->tableTop($theader);
	
		$j=0;
		$last_day=0;
	
		while ($this->rs->next()) {
			$j++;
			$arr = $this->rs->getRow();
			if ($last_day != date('Y-M-d',strtotime($arr['activity_date']))) {
				$res .= $template->tableFullRow(date('Y-M-d',strtotime($arr['activity_date'])));
				$last_day=date('Y-M-d',strtotime($arr['activity_date']));
			}

			$userlink = '';
			$user = UserPeer::retrieveByPk($arr['user_id']);
			if (is_object($user)) {
				$userlink = $user->getUserLink();
			}

			switch ($arr['section'])  {
			case 'forumthread'; {
				$arr['ungroup'] = false;
			}
			default: {
				$plugin_results = ActionManager::notify("PROJECT_EVENT_PULL", $arr);
				foreach ($plugin_results as $result) {
					$url = $result->getResult();
					if ($url != '') {
						break;
					}
				}
				break;
			}
			}
			$trow=array();
			if ($arr['section'] == 'forumthread') {
				$trow[]='';
			} else {
				$trow[]='&nbsp;&nbsp;&nbsp;'.date('H:i:s',strtotime($arr['activity_date']));
			}
			$trow[]=$url;
			$trow[]=$userlink;
			$res .= $template->tableRow($trow);
			unset($trow);
		}
		if ($j==0) {
			$res .= $template->tableFullRow($Language->getText('ProjectActivity.NoneFound'));
		} 
		if ($j!=0 && $this->paginator == null) {
			$url = GFUrl::getURL()->projectURL($this->project, 'Activity', '', 'reporting', array());
			$str_seeall = '<a href="' . $url . '">' . $Language->getText('ProjectActivity.SeeAll') . '</a>';
			
			$str_myactivity = '';
			if (Session::getSession()->isLoggedIn()) {
				$url = GFUrl::getURL()->baseURL('', 'my', 'activity', array('onlymine'=>'1', 'project_id'=>$this->project->getProjectId()));
				$str_myactivity = '&nbsp;&nbsp;&nbsp;&nbsp&nbsp;&nbsp;&nbsp;&nbsp<a href="' . $url . '">' . $Language->getText('ProjectActivity.MyActivity') . '</a>';
			}
			
			$res .= $template->tableFullRow($str_seeall . $str_myactivity, 'text-align: center;');
		}
		$res .= $template->tableBottom();
		return $res;
	}
}
?>
