<?php
/**
 * jQuery File Upload Plugin PHP Class 5.9.1
 * https://github.com/blueimp/jQuery-File-Upload
 *
 * Copyright 2010, Sebastian Tschan
 * https://blueimp.net
 *
 * Licensed under the MIT license:
 * http://www.opensource.org/licenses/MIT
 *
 * Customizations Copyright (c) 2005-2012 GForge, LLC
 */

require_once('StringUtils.php');
require_once('plugins/docman/lib/FastJSON.php');			//TODO: May need to move this library outside Docman

class UploadHandler
{
    protected $options;
    private $base64_img = '';
    
    function __construct($options=null, $section='', $ref_id=0) {
		$this->section = $section;
		$this->ref_id = $ref_id;
		$this->hash = null;
    	
        $this->options = array(
            'script_url' => $this->getFullUrl().'/',
            'upload_dir' => dirname($_SERVER['SCRIPT_FILENAME']).'/files/',
            'upload_url' => $this->getFullUrl().'/files/',
            'param_name' => 'files',
            // Set the following option to 'POST', if your server does not support
            // DELETE requests. This is a parameter sent to the client:
            'delete_type' => 'DELETE',
            // The php.ini settings upload_max_filesize and post_max_size
            // take precedence over the following max_file_size setting:
            'max_file_size' => null,
            'min_file_size' => 1,
            'accept_file_types' => '/.+$/i',
            'max_number_of_files' => null,
            // Set the following option to false to enable resumable uploads:
            'discard_aborted_uploads' => true,
            // Set to true to rotate images based on EXIF meta data, if available:
            'image_versions' => array(
                // Uncomment the following version to restrict the size of
                // uploaded images. You can also add additional versions with
                // their own upload directories:
                /*
                'large' => array(
                    'upload_dir' => dirname($_SERVER['SCRIPT_FILENAME']).'/files/',
                    'upload_url' => $this->getFullUrl().'/files/',
                    'max_width' => 1920,
                    'max_height' => 1200,
                    'jpeg_quality' => 95
                ),
                */
                'thumbnail' => array(
                    'upload_dir' => dirname($_SERVER['SCRIPT_FILENAME']).'/thumbnails/',
                    'upload_url' => $this->getFullUrl().'/thumbnails/',
                    'max_width' => 80,
                    'max_height' => 80
                )
            )
        );
        if ($options) {
            $this->options = array_replace_recursive($this->options, $options);
        }
    }


    protected function getFullUrl() {
      	return
    		(isset($_SERVER['HTTPS']) ? 'https://' : 'http://').
    		(isset($_SERVER['REMOTE_USER']) ? $_SERVER['REMOTE_USER'].'@' : '').
    		(isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : ($_SERVER['SERVER_NAME'].
    		(isset($_SERVER['HTTPS']) && $_SERVER['SERVER_PORT'] === 443 ||
    		$_SERVER['SERVER_PORT'] === 80 ? '' : ':'.$_SERVER['SERVER_PORT']))).
    		substr($_SERVER['SCRIPT_NAME'],0, strrpos($_SERVER['SCRIPT_NAME'], '/'));
    }

    protected function get_file_object($file_name) {
        $file_path = $this->options['upload_dir'].$file_name;
        if (is_file($file_path) && $file_name[0] !== '.') {
            $file = new stdClass();
            $file->file_name = $file_name;
            $file->file_size = filesize($file_path);
            $file->name = $file->file_name;		// Just some aliases
            $file->size = $file->file_size;
            $file->url = $this->options['upload_url'].rawurlencode($file->file_name);
            foreach($this->options['image_versions'] as $version => $options) {
                if (is_file($options['upload_dir'].$file_name)) {
                    $file->{$version.'_url'} = $options['upload_url']
                        .rawurlencode($file->file_name);
                }
            }
            //$this->set_file_delete_url($file);
            return $file;
        }
        return null;
    }
    
    protected function get_file_objects() {
        $arr_return = array();
        try {
	        $arr_return = array_values(array_filter(array_map(
	            array($this, 'get_file_object'),
	            scandir($this->options['upload_dir'])
	        )));
      	} catch(Exception $e) {
      	
      	}

        return $arr_return;
    }


    protected function has_error($uploaded_file, $file, $error) {
        if (!preg_match($this->options['accept_file_types'], $file->file_name)) {
            return 'acceptFileTypes';
        }
        if ($uploaded_file && is_uploaded_file($uploaded_file)) {
            $file_size = filesize($uploaded_file);
        } else {
            $file_size = $_SERVER['CONTENT_LENGTH'];
        }
        $max_file_size = SystemUtils::getMaxFileSizeInBytes(ini_get('upload_max_filesize'));
        if ($file_size > $max_file_size || $file->file_size > $max_file_size) {
            return 'maxFileSize';
        }
        if ($this->options['min_file_size'] &&
            $file_size < $this->options['min_file_size']) {
            return 'minFileSize';
        }
        /* This check is unneeded
        if (is_int($this->options['max_number_of_files']) && (
                count($this->get_file_objects()) >= $this->options['max_number_of_files'])
            ) {
            return 'maxNumberOfFiles';
        }
        */
        if ($error) {
            return $error;
        }
        return $error;
    }


    protected function upcount_name_callback($matches) {
        $index = isset($matches[1]) ? intval($matches[1]) + 1 : 1;
        $ext = isset($matches[2]) ? $matches[2] : '';
        return ' ('.$index.')'.$ext;
    }

    protected function upcount_name($name) {
        return preg_replace_callback(
            '/(?:(?: \(([\d]+)\))?(\.[^.]+))?$/',
            array($this, 'upcount_name_callback'),
            $name,
            1
        );
    }
    
    protected function trim_file_name($name, $type) {
        // Remove path information and dots around the filename, to prevent uploading
        // into different directories or replacing hidden system files.
        // Also remove control characters and spaces (\x00..\x20) around the filename:
        $file_name = trim(basename(stripslashes($name)), ".\x00..\x20");
        // Add missing file extension for known image types:
        if (strpos($file_name, '.') === false &&
            preg_match('/^image\/(gif|jpe?g|png)/', $type, $matches)) {
            $file_name .= '.'.$matches[1];
        }
        if ($this->options['discard_aborted_uploads']) {
            while(is_file($this->options['upload_dir'].$file_name)) {
                $file_name = $this->upcount_name($file_name);
            }
        }
        return $file_name;
    }

    protected function handle_file_upload($uploaded_file, $name, $size, $type, $error) {
        $file = new stdClass();
        $file->file_name = $this->trim_file_name($name, $type);
        //$file->file_size = intval($size);
	     $file->file_size = filesize($uploaded_file);
        
        //Get file type
	     $type = SystemUtils::getMimeType($name);
        
        $file->file_type = $type;
        $file->name = $file->file_name;		// Just some aliases
        $file->size = $file->file_size;
        $file->date = self::getUploadDate($uploaded_file);
        $error = $this->has_error($uploaded_file, $file, $error);
        if (!$error && $file->file_name) {
            $file_path = $this->options['upload_dir'].$file->file_name;
            $append_file = !$this->options['discard_aborted_uploads'] &&
                is_file($file_path) && $file->file_size > filesize($file_path);
            clearstatcache();
            if ($uploaded_file && is_uploaded_file($uploaded_file)) {
                // multipart/formdata uploads (POST method uploads)
                if ($append_file) {
                    file_put_contents(
                        $file_path,
                        fopen($uploaded_file, 'r'),
                        FILE_APPEND
                    );
                } else {
					$file = $this->GFHandleUpload($uploaded_file, $file);
                }
            } else {
                // Non-multipart uploads (PUT method support)
                file_put_contents(
                    $file_path,
                    fopen('php://input', 'r'),
                    $append_file ? FILE_APPEND : 0
                );
            }
            $file_size = is_file($file_path) ? filesize($file_path) : 0;
            if ($file_size === $file->file_size || 1==1) {

                $file->url = $this->options['upload_url'].rawurlencode($file->file_name);
                foreach($this->options['image_versions'] as $version => $options) {
	                if ($this->options['upload_dir'] !== $options['upload_dir']) {
                        $file->{$version.'_url'} = $options['upload_url'] .rawurlencode($file->file_name);
                        //BEGIN GFORGE CUSTOMIZATION
                        $file_type_icon = "/themes/gforge5/upload_widget/document_icon.png";
                        if (!(strpos($type, 'pdf') === false)) {
                            $file_type_icon = "/themes/gforge5/upload_widget/pdf_icon.png";
                        } elseif (!(strpos($type, 'mage') === false)) {
                            $file_type_icon = "/themes/gforge5/upload_widget/graphic_icon.png";
                        }
                        $file->thumbnail_url = $file_type_icon;
                        //END GFORGE CUSTOMIZATION
                    } else {
	                    clearstatcache();
	                    $file_size = filesize($file_path);
					}
                }
            } else if ($this->options['discard_aborted_uploads']) {
                unlink($file_path);
                $file->error = 'abort';
            }
        } else {
            $file->error = $error;
        }
        return $file;
    }
    
    public function get() {
        $file_name = isset($_REQUEST['file']) ?
            basename(stripslashes($_REQUEST['file'])) : null;
        if ($file_name) {
            $info = $this->get_file_object($file_name);
        } else {
            $info = $this->get_file_objects();
        }
        header('Content-type: application/json');
        echo json_encode($info);
    }
    
    public function post() {
        if (isset($_REQUEST['_method']) && $_REQUEST['_method'] === 'DELETE') {
            return $this->delete();
        }
        $upload = isset($_FILES[$this->options['param_name']]) ?
            $_FILES[$this->options['param_name']] : null;
        $info = array();
        if ($upload && is_array($upload['tmp_name'])) {
            // param_name is an array identifier like "files[]",
            // $_FILES is a multi-dimensional array:
            foreach ($upload['tmp_name'] as $index => $value) {
                $info[] = $this->handle_file_upload(
                    $upload['tmp_name'][$index],
                    isset($_SERVER['HTTP_X_FILE_NAME']) ?
                        $_SERVER['HTTP_X_FILE_NAME'] : $upload['name'][$index],
                    isset($_SERVER['HTTP_X_FILE_SIZE']) ?
                        $_SERVER['HTTP_X_FILE_SIZE'] : $upload['size'][$index],
                    isset($_SERVER['HTTP_X_FILE_TYPE']) ?
                        $_SERVER['HTTP_X_FILE_TYPE'] : $upload['type'][$index],
                    $upload['error'][$index]
                );
            }
        } elseif ($upload || isset($_SERVER['HTTP_X_FILE_NAME'])) {
            // param_name is a single object identifier like "file",
            // $_FILES is a one-dimensional array:
            $info[] = $this->handle_file_upload(
                isset($upload['tmp_name']) ? $upload['tmp_name'] : null,
                isset($_SERVER['HTTP_X_FILE_NAME']) ?
                    $_SERVER['HTTP_X_FILE_NAME'] : (isset($upload['name']) ?
                        $upload['name'] : null),
                isset($_SERVER['HTTP_X_FILE_SIZE']) ?
                    $_SERVER['HTTP_X_FILE_SIZE'] : (isset($upload['size']) ?
                        $upload['size'] : null),
                isset($_SERVER['HTTP_X_FILE_TYPE']) ?
                    $_SERVER['HTTP_X_FILE_TYPE'] : (isset($upload['type']) ?
                        $upload['type'] : null),
                isset($upload['error']) ? $upload['error'] : null
            );
        }
        header('Vary: Accept');
        //$json = json_encode($info);
        $json = FastJSON::encode($info);
        $redirect = isset($_REQUEST['redirect']) ?
            stripslashes($_REQUEST['redirect']) : null;
        if ($redirect) {
            header('Location: '.sprintf($redirect, rawurlencode($json)));
            return;
        }
        if (isset($_SERVER['HTTP_ACCEPT']) &&
            (strpos($_SERVER['HTTP_ACCEPT'], 'application/json') !== false)) {
            header('Content-type: application/json');
        } else {
            header('Content-type: text/plain');
        }
        echo $json;
    }

	//START CUSTOM CODE
    public function getTmpUploadDir() {
        return SystemUtils::buildFilePath(GFConf::get('system.varDir'), '/tmp_uploads/');
    }

	private function GFHandleUpload($uploaded_file, $file) {
        // We were requested to store the files in a tmp directory. This is usually done when we upload the files
        // for an object for which we don't have an ID yet.
        if (isset($_REQUEST['_file_upload_to_tmp']) && $_REQUEST['_file_upload_to_tmp']) {
            return $this->handleTmpUpload($uploaded_file, $file);
        }

        $params = array(
            'section' => $this->section,
            'ref_id' => $this->ref_id,
            'uploaded_file' => $uploaded_file,
            'file_name' => addslashes($file->file_name),
            // This is just to make it compatible with the $_FILES syntax
            'size' => $file->file_size,
            'type' => $file->file_type,
            'tmp_name' => $uploaded_file,
            'name' => $file->file_name,
            );

        $prs = ActionManager::notify('HANDLE_FILE_UPLOAD', $params);
        foreach ($prs as $pr) {
            // There should be only one file
            $result_file = $pr->getResult();

            // Set extra parameters for the file
            foreach ($result_file as $param => $value) {
                $file->{$param} = $value;
            }
            break;
        }

        return $file;
	}

	public function getUploadHash() {
		if ($this->hash) {
			return $this->hash;
		} elseif (isset($_REQUEST['_file_upload_form_hash'])) {		// Note that _file_upload_form_hash may come either from POST or GET
			// Uploaded file by submitting form
			$this->hash = $_REQUEST['_file_upload_form_hash'];
			self::validateHash($this->hash);
		} else {
			// Hash not set. Create a new one.
			$this->hash = $this->generateUploadHash();
		}

		return $this->hash;
	}

    /**
     * Generates a unique hash to be used for uploading files.
     */
    private function generateUploadHash() {
        $time = date('Y-m-d-H-i-s');
        $extra_hash = substr(md5(microtime()), 0, 5);
        $hash = "{$time}-{$extra_hash}";

        $suffix = (isset($this->options['suffix']) ? $this->options['suffix'] : '');
        if ($suffix) {
            $hash .= '-'.$suffix;
        }
        return $hash;
    }

    /**
     * Make sure the hash has the expected format (we only accept alphanumeric chars and dashes)
     */
    private static function validateHash($hash) {
        if (!strlen($hash) || preg_match('/[^[:alnum:]\\-]/', $hash)) {
            throw new Exception('Wrong hash format');
        }
    }

    /**
     * Moves a file uploaded by the upload widget to a tmp directory. It uses the unique hash to identify the form
     * from which the file was uploaded (note that several files can be uploaded from the same form).
     */
    function handleTmpUpload($uploaded_file, $file) {
    	$hash = $this->getUploadHash();
        self::validateHash($hash);

        $uploadDir = SystemUtils::buildFilePath(self::getTmpUploadDir(), "/$hash");
        if (!is_dir($uploadDir) && !@mkdir($uploadDir, 0770)) {
            include_once('lib/exceptions/InvalidDirectoryException.php');
            throw new InvalidDirectoryException($uploadDir);
        }

        $destName = SystemUtils::buildFilePath($uploadDir, $file->file_name);
        if (!move_uploaded_file($uploaded_file, $destName)) {
            throw new Exception("Unable to move uploaded file to $destName");
        }

        $file->file_size = filesize($destName);
        return $file;
    }

    public function handleDeletedFiles() {
        if (!isset($_REQUEST['_upload_delete_files']) || !is_array($_REQUEST['_upload_delete_files'])) {
            return;
        }
    	if ($this->section) {
    		// If the section is set, it means we are deleting the files attached to an already existing object.
    		foreach ($_REQUEST['_upload_delete_files'] as $file_id) {
    			// Note that $file_id is not *necessarily* the filesystem_id. This depends on each plugin.
    			$params = array(
    				'section' => $this->section,
    				'ref_id' => $this->ref_id,
    				'file_id' => $file_id,
    				);

    			$prs = ActionManager::notify('HANDLE_FILE_DELETE', $params);
    			if ($prs->count() == 0) {
    				// No plugin handled the request for delete. By default, delete from the filesystem.
    				$c = new Criteria();
    				$c->add(FilesystemPeer::SECTION, $this->section);
    				$c->add(FilesystemPeer::REF_ID, $this->ref_id);
    				$c->add(FilesystemPeer::FILESYSTEM_ID, $file_id);
    				$file = FilesystemPeer::doSelectOne($c);
    				if ($file) {
    					$file->delete();
    				}
    			}
    		}
    	} else {
    		// If the section is not set, we are deleting the files uploade to the tmp directory (the ones that are uploaded before
    		// the object is created)
	    	$hash = $this->getUploadHash();
	        self::validateHash($hash);
	        $uploadDir = SystemUtils::buildFilePath(self::getTmpUploadDir(), "/$hash");
	        foreach ($_REQUEST['_upload_delete_files'] as $filename) {
	        	// Safety check
	        	if (strpos($filename, '../') !== false) {
	        		throw new PermissionDeniedException();
	        	}

	        	$file = SystemUtils::buildFilePath($uploadDir, $filename);
	        	if (is_file($file)) {
	        		@unlink($file);
	        	}
	        }
    	}
    }

    /**
     * Returns the list of files uploaded in the tmp directory identified by the hash
     */
    public function getTmpUploadedFiles() {
    	$this->handleDeletedFiles();

    	$hash = $this->getUploadHash();
        self::validateHash($hash);

        $uploadDir = SystemUtils::buildFilePath(self::getTmpUploadDir(), "/$hash");
        if (!is_dir($uploadDir)) {
            return array();
        }

        $files = array();
        
        $dh = dir($uploadDir);
        while (($entry = $dh->read()) !== false) {
            if ($entry == '.' || $entry == '..') {
                continue;
            }

            $file = SystemUtils::buildFilePath($uploadDir, $entry);
            $files[] = $file;
        }

        return $files;
    }

    /**
     * Delete the tmp files uploaded by the widget
     */
    public function purgeTmpDir() {
    	$hash = $this->getUploadHash();
        self::validateHash($hash);

        $uploadDir = SystemUtils::buildFilePath(self::getTmpUploadDir(), "/$hash");
        if (!is_dir($uploadDir)) {
            return;
        }

        if (SystemUtils::isRootDir($uploadDir)) {
            // Shouldn't happen, but just in case
            throw new PermissionDeniedException();
        }

        $output = array();
        SystemUtils::execute("rm -rf $uploadDir", $output);
    }

    /**
     * Outputs JavaScript code to generate an array of uploaded files
     */
    public function loadJSUploadedFiles() {
    	if ($this->section) {
    		// If a section and a ref_id are set, load the files from the Filesystem table
    		$files = $this->getFilesystemUploadedFiles();
    	} else {
    		// Otherwise, it means we are creating a new item which doesn't have an associated ID yet and the
    		// files attached to it are being stored on a temp directory, identified by the hash
    		$files = $this->getTmpUploadedFilesInfo();
    	}

    	$varName = '_UPLOAD_UPLOADED_FILES';
    	return "$varName = ".FastJSON::convert($files).";\n";
    }

    private function getFilesystemUploadedFiles() {
    	$fs_files = FilesystemPeer::getRelatedFiles($this->section, $this->ref_id);
    	$files = array();
    	foreach ($fs_files as $file) {
    		$files[] = array(
    			'file_name' => $file->getFileName(),
    			'file_size' => $file->getFileSize(),
    			'file_type' => $file->getFileType(),
    			'uploaded_by' => $file->getUser()->getUserLink(),
    			'file_id' => $file->getPrimaryKey(),
    			'date' => $file->getUploadDate(),
    			'link' => GFUrl::downloadUrlFromFS($file),
    			'link_download' => GFUrl::downloadUrlFromFS($file, false)
    			);
    	}

    	return $files;
    }

    /**
     * Returns an array with the information of the files uploaded associated to the current upload hash.
     */
    public function getTmpUploadedFilesInfo() {
    	$fs_files = $this->getTmpUploadedFiles();

    	$files = array();

    	$user = Session::getSession()->getUser();
    	foreach ($fs_files as $file) {
    		$files[] = array(
    			'file_name' => basename($file),
    			'file_size' => filesize($file),
    			'file_type' => mime_content_type($file),
    			'date' => self::getUploadDate($file),
    			'uploaded_by' => ($user) ? $user->getFullName() : '',
    			);
    	}

    	return $files;
    }

    private static function getUploadDate($file) {
    	return date('Y-m-d H:i:s', filemtime($file));
    }

    public function getJavascriptSetup() {
    	$urlParams = array(
    			'section' => $this->section, 
    			'ref_id' => $this->ref_id,
    		);

    	$hash = $this->getUploadHash();

    	if (!$this->section) {
    		// If no object set, upload files to a tmp directory.
    		$urlParams['_file_upload_to_tmp'] = 1;
    		$urlParams['_file_upload_form_hash'] = $hash;
    	}

    	$uploadUrl = GFUrl::getUrl()->baseURL('', '', 'upload', $urlParams);

    	$loadFiles = $this->loadJSUploadedFiles();

    	$js = <<<EOF
    	<!-- Code for the file upload widget -->
		<script language="javascript" type="text/javascript">
			$(document).ready(function() {
				var upload_url = '$uploadUrl';
				if (_ENABLE_UPLOADS) {
					FileUploadWidgetInit('widget_container', 'fileupload', upload_url, []);
				}
			});

			function setUploadWidgetParameter() {
				// No parameters need to be set.
			}

			$loadFiles

			_FILE_UPLOAD_FORM_HASH = '$hash';
		</script>
EOF;
		return $js;
    }

	//END CUSTOM CODE
}
