<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Group.class,v 1.94 2005/09/06 18:13:29 danper Exp $
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-10
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('core/NotificationQueue.php');

class NotificationManager {
	private static $instance = null;

	private function __construct() {
	}
	
	/**
	 * Returns the singleton instance of the manager
	 * @return NotificationManager
	 */
	public static function getManager() {
		if (is_null(self::$instance)) {
			self::$instance = new NotificationManager;
		}
		
		return self::$instance;
	}

	/**
	 *	queue - add this notification to the queue.
	 *	@param	GFNotification
	 * 	@return	boolean	true on success
	 */
	function queue(GFNotification $GFNotification) {
		$missing_data = array();

		$Users = $GFNotification->getUsers();
		foreach ($Users as $User) {
			$Msg = $GFNotification->getNotificationMessage($User);
			$Subj = $GFNotification->getNotificationSubject($User);
			$Sect = $GFNotification->getSection();
			$Refid = $GFNotification->getRefId();

			if (empty($Subj)) {
				$missing_data['subject'] = Language::getSessText('SiteNotification.EmptySubject');
			}
			if (empty($Msg)) {
				$missing_data['message'] = Language::getSessText('SiteNotification.EmptyMessage');
			}

			if (count($missing_data) > 0) {
				throw new InvalidDataException($missing_data);
			}

			// add the message to the notification queue
			if ($User->getStatus()==User::STATUS_ACTIVE) {
				$queue = new NotificationQueue();
				$queue->setSection($Sect);
				$queue->setRefId($Refid);
				$queue->setUserId($User->getUserId());
				$queue->setSubject($Subj);
				$queue->setBody($Msg);
				if ($Sect == 'forummessage' ) {
					$queue->setMessageId($GFNotification->getMessageId());
					$queue->setRefMessage($GFNotification->getReferences());
					$queue->setInReplyTo($GFNotification->getInReplyTo());
				}
				if ($Sect == 'trackeritem') {
					$queue->setMessageId($GFNotification->getMessageId());
					$queue->setRefMessage($GFNotification->getReferences());
					$queue->setInReplyTo($GFNotification->getInReplyTo());
				}
				if ($Sect == Project::SECTION) {
					$queue->setMessageId($GFNotification->getMessageId());
					$queue->setNotificationId($GFNotification->getNotificationId());
				}
				$queue->save();
			}
		}
		return true;
	}

}

?>
