<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez ruben@gforgegroup.com
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('GFConf.php');
require_once('lib/initialize.php');
require_once('system/Cronjob.php');
require_once('plugins/build/lib/CruisecontrolDaemon.php');
require_once('plugins/build/lib/CruisecontrolPlugin.php');
require_once('plugins/build/lib/CruisecontrolPeer.php');

class CheckCruiseControl extends Cronjob {
	private $pathToCC;
	private $restarted = false;
	private $last_run = 0;
	private $last_mod = 0;

	const PID_FILE = '/tmp/gforge-cc.pid';
	const PLUGIN_NAME = 'cruisecontrol';
	
	public function __construct() {
		$this->jobname = basename(__FILE__);
		$this->pathToCC = GFConf::get('build.pathToCC');
		parent::__construct();
	}

	public function run() {
		// Get the date of creation of config.xml
		if (file_exists($this->pathToCC.'/config.xml')) {
			$this->last_mod = filemtime($this->pathToCC.'/config.xml');
		}

		if (file_exists(self::PID_FILE)) {
			$this->last_run = filemtime(self::PID_FILE);
		}

		if ($this->last_mod > $this->last_run) {
			$ccuser = GFConf::get('build.unixUser');
			$this->logCron('There were changes in the configuration');

			if (!$this->restarted) {
				$this->logCron('Will attempt to restart CruiseControl');
				try {
					if (CruisecontrolDaemon::isRunning()) {
						$this->logCron('Stoping CruiseControl..');
						CruisecontrolDaemon::stop();
					} else {
						$this->logCron('CruiseControl is not running. Starting...');
					}
					CruisecontrolDaemon::start();
					$this->logCron('CruiseControl restarted');
				} catch (Exception $e) {
					$this->logCron($e->getMessage());
					$this->commitLog();
					exit(1);
				}
			}
		} else if (!CruisecontrolDaemon::isRunning()) {
			$this->logCron('CruiseControl is not running. Starting...');
			CruisecontrolDaemon::start();
		}
		$this->commitLog();
	}
}

$cronjob = new CheckCruiseControl();
echo $cronjob->run();
?>