<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez ruben@gforgegroup.com
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('lib/initialize.php');
require_once('system/Cronjob.php');
require_once('core/ProjectPeer.php');
require_once('core/PluginPeer.php');
require_once('core/ProjectPluginPeer.php');
require_once('GFConf.php');
require_once('plugins/build/lib/CruisecontrolDaemon.php');
require_once('plugins/build/lib/BuildPlugin.php');
require_once('plugins/build/lib/CruisecontrolPeer.php');
require_once('plugins/scm/lib/ProjectScmPeer.php');
require_once('plugins/scm/lib/ScmModulePeer.php');
require_once('plugins/scm/lib/SCMRepositoryFactory.php');

class CreateCCConfig extends Cronjob {
	private $pathToCC;
	private $projectsDir;
	private $configDir;
	private $logsDir;
	private $hash;
	private $last_run = 0;
	private $last_mod = 0;

	const PLUGIN_NAME = 'build';
	
	public function __construct() {
		$this->jobname = basename(__FILE__);
		$this->pathToCC = GFConf::get('build.pathToCC');
		$this->projectsDir = GFConf::get('build.projectsDir');
		$this->configDir = GFConf::get('build.configDir');
		$this->logsDir = GFConf::get('build.logsDir');
		$this->hash = GFConf::get('build.hash');
		parent::__construct();
	}
	
	public function run() {
		// Save the date of creation of config.xml
		if (file_exists($this->pathToCC.'/config.xml')) {
			$this->last_run = filemtime($this->pathToCC.'/config.xml');
		}

		// lets get all projects that use the plugin
		$c = new Criteria();
		$c->add(PluginPeer::PLUGIN_NAME, BuildPlugin::PLUGIN_NAME);
		$project_plugins = ProjectPluginPeer::doSelectJoinPlugin($c);

		$dom = new DOMDocument('1.0', 'iso-8859-1');
		$domCControlNode = $this->doHeadConfig($dom);

		foreach ($project_plugins as $project_plugin) {
			$Project = $project_plugin->getProject(); // get the project
			$CControl = CruisecontrolPeer::retrieveByPK($Project->getProjectId());
			if ($CControl != null && is_object($CControl)) {
				$lastMod = $CControl->getLastModifiedDate(null);
				if ($lastMod > $this->last_mod) {
					$this->last_mod = $lastMod;
				}
			}
		}

		if ($this->last_mod > $this->last_run) {
			$ccuser = GFConf::get('build.unixUser');
			$this->logCron('Recreating configuration file');
			foreach ($project_plugins as $project_plugin) {
				$Project = $project_plugin->getProject(); // get the project
				$repos = SCMRepositoryFactory::getRepository($Project);
				$CControl = CruisecontrolPeer::retrieveByPK($Project->getProjectId());
				if ($repos != null && is_object($repos) && $CControl != null && is_object($CControl)) {
					if ($this->createCheckout($Project, $repos)) {
						$this->doProjectConfig($dom, $domCControlNode, $Project, $repos, $CControl);
					} else {
						$this->logCron('Checkout failed for project: ' . $Project->getProjectName());
						$this->logCron('Won\'t be added to CruiseControl');
					}
				}
			}
			$dom->save($this->pathToCC.'/config.xml');
			$this->doChOwn($this->pathToCC.'/'.$this->projectsDir, $ccuser, $ccuser);
			$this->doChOwn($this->pathToCC.'/config.xml', $ccuser, $ccuser);
		}
		
		$this->commitLog();
	}
	
	private function createCheckout($Project, $repos) {
		$ret = false;
		if ($repos->isConfigured() && $repos->isCreated()) {
			if ($repos->getType() == 'cvs') {
				$folder = $this->prepareFolder($Project, 'CVS');
				if ($folder === 1) {
					$ret = $this->doCVSCheckout($Project, $repos);
				} else if ($folder === 2) {
					$ret = true;
				}
			} else if ($repos->getType() == 'svn') {
				$folder = $this->prepareFolder($Project, '.svn');
				if ($folder === 1) {
					$ret = $this->doSVNCheckout($Project, $repos);
				} else if ($folder === 2) {
					$ret = true;
				}
			}
		}
		return $ret;
	}
	
	private function prepareFolder($Project, $token) {
		$permMask = 02775;
		$unixName = $Project->getUnixName();
		$pathToProject = $this->pathToCC.'/'.$this->projectsDir.'/'.$unixName;
		$pathToConfig = $pathToProject.'/'.$this->configDir;
		
		$ready = 0;
		// First, let's check if the folder exists. If not, we'll create them
		if (!file_exists($pathToProject)) {
			$this->logCron('Directory '. $pathToProject . ' doesn\'t exist. Will create it.');
			if (mkdir($pathToProject, $permMask, true)) {
				$this->logCron('Directory '. $pathToProject . ' sucessfully created.');
				if (!file_exists($pathToConfig)) {
					if (mkdir($pathToConfig, $permMask, true)) {
						$this->logCron('Config directory '. $pathToConfig . ' sucessfully created.');
						$ready = 1;
					} else {
						$this->logCron('Unable to create '. $pathToConfig . ' config directory.');
					}
				} else if (is_dir($pathToConfig)) {
					$ready = 1;
				} else {
					$this->logCron($pathToConfig . ' exists, but it\'s a file.');
				}

			} else {
				$this->logCron('Unable to create '. $pathToProject . ' directory.');
			}
		} else {
			// If project directory exists, make sure it's not a file
			if (is_dir($pathToProject)) {
				// Now check if there's a checkout already. If not, we'll do it later.
				if (!file_exists($pathToProject.'/'.$token)) {
					// There's no checkout yet
					if (!file_exists($pathToConfig)) {
						if (mkdir($pathToConfig, 02775, true)) {
							$this->logCron('Config directory '. $pathToConfig . ' sucessfully created.');
							$ready = 1;
						} else {
							$this->logCron('Unable to create '. $pathToConfig . ' config directory.');
						}
					} else if (is_dir($pathToConfig)) {
						$ready = 1;
					} else {
						$this->logCron($pathToConfig . ' exists, but it\'s a file.');
					}
				} else {
					// There's a checkout already
					$ready = 2;
				}
			} else {
				$this->logCron($pathToProject . ' exists, but it\'s a file.');
			}
		}
		return $ready;
	}
	
	private function doCVSCheckout($Project, $repos) {
		$pathToProject = $this->pathToCC.'/'.$this->projectsDir.'/'.$Project->getUnixName();
		$projectScm = $repos->getScmConfiguration();

		if (!$projectScm->getControlled()) {
			//@todo	This will be used when we add support for external SCM
			$this->logCron('External SCM is not supported yet.');
			return false;
		} else {
			$c = new Criteria();
			$c->add(CruisecontrolPeer::PROJECT_ID, $Project->getProjectId());
			$CControl = CruisecontrolPeer::doSelectOne($c);
			$prefsObj = $CControl->getPreferences();
			$c = new Criteria();
			$c->add(CruisecontrolModulePeer::PROJECT_ID, $Project->getProjectId());
			$moduleAux = CruisecontrolModulePeer::doSelectOne($c);
			$module = ScmModulePeer::retrieveByPk($moduleAux->getScmModuleId());
			
			$failure = array('aborted', 'failed', 'error', 'fatal');
			$root = GFConf::get('scmcvs.localCVSRoot');
			$binary = GFConf::get('scmcvs.cvsbin');
			$hostname = GFConf::get('system.hostName');
			$output = array();
			$moduleName = str_replace('/','',$module->getPath());
			chdir($pathToProject);
			$ret = SystemUtils::execute($binary.' -d '.$repos->getRoot().' co '.$moduleName, $output);
			if ($ret > 0) {
				return false;
			} else {
				return true;
			}
		}
	}
	
	private function doSVNCheckout($Project, $repos) {
		$pathToProject = $this->pathToCC.'/'.$this->projectsDir.'/'.$Project->getUnixName();
		$projectScm = $repos->getScmConfiguration();

		if (!$projectScm->getControlled()) {
			//@todo	This will be used when we add support for external SCM
			$this->logCron('External SCM is not supported yet.');
			return false;
		} else {
			$failure = array('aborted', 'failed', 'error', 'fatal');
			$root = GFConf::get('scmsvn.localSVNRoot');
			$binary = GFConf::get('scmsvn.svnpath') . '/svn';
			$hostname = GFConf::get('system.hostName');
			$output = array();
			chdir($pathToProject);
			$ret = SystemUtils::execute($binary.' co file://'.$repos->getRoot().' '.$pathToProject, $output);
			if ($ret > 0) {
				return false;
			} else {
				return true;
			}
		}
	}
	
	private function doHeadConfig($dom) {
		//<cruisecontrol>
		$domCControl = $dom->createElement('cruisecontrol');
		$domCControlNode = $dom->appendChild($domCControl);
		//<plugin>
		$domPlugin = $dom->createElement('plugin');
		$domPlugin->setAttribute('name', 'gforge');
		$domPlugin->setAttribute('classname', 'org.gforge.cruisecontrol.publishers.GForgeSOAPPublisher');
		$domPlugin->setAttribute('host', GFConf::get('system.hostName'));
		$domCControlNode->appendChild($domPlugin);
		return $domCControlNode;
	}
	
	private function doProjectConfig($dom, $domCControlNode, $Project, $repos, $CControl) {
		$scm = $repos->getType();
		$quiet_period = 30;
		
		$publisher = $CControl->getPublisher();
		$developer = $CControl->getDeveloper();
		
		$assigneeIds = '';
		$assigneesObj = $CControl->getAssignees();
		$assignees = array();
		if ($assigneesObj != null && is_array($assigneesObj) && count($assigneesObj) > 0) {
			foreach($assigneesObj as $assObj) {
				$assignees[] = $assObj->getAssignee();
			}
			$assigneeIds = implode(',', $assignees);
		}

		$prefsObj = $CControl->getPreferences();
		$prefs = array();
		if ($prefsObj != null && is_array($prefsObj) && count($prefsObj) > 0) {
			foreach($prefsObj as $prefObj) {
				$prefs[$prefObj->getName()] = $prefObj->getValue();
			}
		}
		//<project>
		$domProject = $dom->createElement('project');
		$domProject->setAttribute('name', $Project->getUnixName());
		$domProject->setAttribute('buildafterfailed', 'false');
		$domProjectNode = $domCControlNode->appendChild($domProject);
		//<listeners>
		$domListeners = $dom->createElement('listeners');
		$domListenersNode = $domProjectNode->appendChild($domListeners);
		//<currentbuildstatuslistener>
		$domCurrBuildStatus = $dom->createElement('currentbuildstatuslistener');
		$domCurrBuildStatus->setAttribute('file', $this->logsDir.'/${project.name}/status.txt');
		$domListenersNode->appendChild($domCurrBuildStatus);
		//<bootstrappers>
		$domBootStrappers = $dom->createElement('bootstrappers');
		$domBootStrappersNode = $domProjectNode->appendChild($domBootStrappers);
		//<cvsbootstrapper><svnbootstrapper>
		$domSCMBootStrapper = $dom->createElement($scm.'bootstrapper');
		$domSCMBootStrapper->setAttribute('localWorkingCopy', $this->projectsDir.'/${project.name}');
		$domBootStrappersNode->appendChild($domSCMBootStrapper);
		//<modificationset>
		$domModifSet = $dom->createElement('modificationset');
		$domModifSet->setAttribute('quietperiod', $quiet_period);
		$domModifSetNode = $domProjectNode->appendChild($domModifSet);
		//<cvs><svn>
		$domSCM = $dom->createElement($scm);
		$domSCM->setAttribute('localWorkingCopy', $this->projectsDir.'/${project.name}');
		$domModifSetNode->appendChild($domSCM);
		//<schedule>
		$domSchedule = $dom->createElement('schedule');
		$interval = (isset($prefs['build_interval']) ? ($prefs['build_interval'] * 60) : 3600);
		$domSchedule->setAttribute('interval', $interval);
		$domScheduleNode = $domProjectNode->appendChild($domSchedule);

		// HERE DO THE SCHEDULE LOGIC //
		if ($CControl->getBuilder() == Cruisecontrol::BUILDER_ANT) {
			$domBuilder = $dom->createElement('ant');
			$domBuilder->setAttribute('anthome', GFConf::get('build.antHome'));
			$domBuilder->setAttribute('usequiet', 'true');
			$domBuilder->setAttribute('uselogger', 'true');
			if (isset($prefs['ant_buildfile']) && $prefs['ant_buildfile'] != '') {
				$domBuilder->setAttribute('buildfile', $this->projectsDir.'/${project.name}/'.$prefs['ant_buildfile']);
			}
			if (isset($prefs['ant_target']) && $prefs['ant_target'] != '') {
				$domBuilder->setAttribute('target', $prefs['ant_target']);
			}
		} else if ($CControl->getBuilder() == Cruisecontrol::BUILDER_NANT) {
			$domBuilder = $dom->createElement('nant');
			$domBuilder->setAttribute('targetFramework', GFConf::get('build.targetFramework'));
			$domBuilder->setAttribute('usequiet', 'true');
			if (isset($prefs['nant_buildfile']) && $prefs['nant_buildfile'] != '') {
				$domBuilder->setAttribute('buildfile', $this->projectsDir.'/${project.name}/'.$prefs['nant_buildfile']);
			}
			if (isset($prefs['nant_target']) && $prefs['nant_target'] != '') {
				$domBuilder->setAttribute('target', $prefs['nant_target']);
			}
		} else if ($CControl->getBuilder() == Cruisecontrol::BUILDER_MAVEN) {
			$domBuilder = $dom->createElement('maven');
			$domBuilder->setAttribute('mavenscript', GFConf::get('build.mavenScript'));
			if (isset($prefs['maven_projectfile']) && $prefs['maven_projectfile'] != '') {
				$domBuilder->setAttribute('projectfile', $this->projectsDir.'/${project.name}/'.$prefs['maven_projectfile']);
			}
			if (isset($prefs['maven_goal']) && $prefs['maven_goal'] != '') {
				$domBuilder->setAttribute('goal', $prefs['maven_goal']);
			}
		} else if ($CControl->getBuilder() == Cruisecontrol::BUILDER_MAVEN2) {
			$domBuilder = $dom->createElement('maven2');
			$domBuilder->setAttribute('mvnhome', GFConf::get('build.mvnHome'));
			if (isset($prefs['maven2_pomfile']) && $prefs['maven2_pomfile'] != '') {
				$domBuilder->setAttribute('pomfile', $this->projectsDir.'/${project.name}/'.$prefs['maven2_pomfile']);
			}
			if (isset($prefs['maven2_goal']) && $prefs['maven2_goal'] != '') {
				$domBuilder->setAttribute('goal', $prefs['maven2_goal']);
			}
		} else if ($CControl->getBuilder() == Cruisecontrol::BUILDER_CUSTOM) {
			$domBuilder = $dom->createElement('exec');
			$domBuilder->setAttribute('timeout', GFConf::get('build.scriptTimeout'));
			if (isset($prefs['custom_command']) && $prefs['custom_command'] != '') {
				$domBuilder->setAttribute('command', $this->projectsDir.'/${project.name}/'.$prefs['custom_command']);
			}
			if (isset($prefs['custom_args']) && $prefs['custom_args'] != '') {
				$domBuilder->setAttribute('args', $prefs['custom_args']);
			}
			if (isset($prefs['custom_errorstr']) && $prefs['custom_errorstr'] != '') {
				$domBuilder->setAttribute('errorstr', $prefs['custom_errorstr']);
			}
		}
		if (isset($domBuilder)) {
			$domScheduleNode->appendChild($domBuilder);
		}
	
		//<log>
		$domLog = $dom->createElement('log');
		$domLogNode = $domProjectNode->appendChild($domLog);
		
		if ($CControl->getLogType() == Cruisecontrol::LOG_GZIP) {
			//<gzip>
			$domGZip = $dom->createElement('gzip');
			$domGZip->setAttribute('every', '1');
			$domGZip->setAttribute('unit', 'DAY');
			$domLogNode->appendChild($domGZip);
		} else {
			//<merge>
			$domMerge = $dom->createElement('merge');
			$domMerge->setAttribute('dir', $this->projectsDir.'/${project.name}/target/test-results');
			$domLogNode->appendChild($domMerge);
		}

		if ($publisher != null && $publisher->getTrackerId() > 0) {
			//<publishers>
			$domPublishers = $dom->createElement('publishers');
			$domPublishersNode = $domProjectNode->appendChild($domPublishers);
			//<gforge>
			$domGForge = $dom->createElement('gforge');
			$domGForge->setAttribute('userId', $developer);
			$domGForge->setAttribute('hash', $this->hash);
			$domGForge->setAttribute('projectId', $Project->getProjectId());
			$domGForge->setAttribute('trackerId', $publisher->getTrackerId());
			$domGForge->setAttribute('configDir', $this->projectsDir.'/${project.name}/'.$this->configDir);
			if ($assigneeIds != '') {
				$domGForge->setAttribute('assignees', $assigneeIds);
			}
			$domPublishersNode->appendChild($domGForge);
		}
	}
	
	private function doChOwn($path, $uid, $gid) {
		if (is_file($path)) {
			chown($path, $uid);
			chgrp($path, $gid);
		} else {
			$dir = opendir($path);
			while(($file = readdir($dir)) !== false) {
				if ($file != '.' && $file != '..') {
					if(is_dir($path.'/'.$file)) {
						$this->doChOwn($path.'/'.$file, $uid, $gid);
					} else {
						chown($path.'/'.$file, $uid);
						chgrp($path.'/'.$file, $gid);
					}
				}
			}
			closedir($dir);
		}
	}
}

$cronjob = new CreateCCConfig();
//$cronjob->run();
echo $cronjob->run();
?>