<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez ruben@gforgegroup.com
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once 'lib/core/UserPeer.php';
require_once("plugin/ActionManager.php");
require_once 'plugins/build/lib/om/BaseCruisecontrol.php';
require_once 'plugins/build/lib/CruisecontrolAssigneePeer.php';
require_once 'plugins/build/lib/CruisecontrolPublisherPeer.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");


/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.build.lib 
 */
class Cruisecontrol extends BaseCruisecontrol  implements GFPermission {
	const SECTION = 'cruisecontrol';

	private $assignees;
	private $publisher;
	private $project;
	private $developer_user;
	
	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	public function userIsAdmin() {
		$perm = $this->getProject()->getPermission();
		if (($perm->check(BuildPlugin::SECTION, 0, BuildPlugin::PERMLEVEL_ADMIN))) { // normal permission
			return true;
		}
		return false;
	}

	public function userCanRead() {
		if ($this->userIsAdmin()) {
			return true;
		}
		if ($this->isPublic()) {
			return true;
		}

		return false;
	}


	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	function delete($con = null) {
		if (!$this->userIsAdmin()) {
			throw new PermissionDeniedException("Could not delete Cruisecontrol object: permission denied");
		}
		//delete assignees
		$crit=new Criteria();
		$crit->add(CruisecontrolAssigneePeer::CRUISECONTROL_ID, $this->getPrimaryKey());
		CruisecontrolAssigneePeer::doDelete($crit);

		//delete publishers
		$crit=new Criteria();
		$crit->add(CruisecontrolPublisherPeer::CRUISECONTROL_ID, $this->getPrimaryKey());
		CruisecontrolPublisherPeer::doDelete($crit);

		ActionManager::notify("CRUISECONTROL_DELETE", $this);
		parent::delete($con);		
	}

	function save($con = null) {
		if ($this->isNew() && !$this->userIsAdmin()) {
			throw new PermissionDeniedException("Could not create Cruisecontrol object: permission denied");
		} else if (!$this->userIsAdmin()) {
			throw new PermissionDeniedException("Could not update Cruisecontrol object: permission denied");
		}
		if (isset($this->assignees) && is_array($this->assignees)) {
			foreach($this->assignees as $assignee) {
				if ($assignee instanceof CruisecontrolAssignee) {
					$assignee->save($con);
				}
			}
		}
		if (isset($this->publisher) && $this->publisher instanceof CruisecontrolPublisher) {
			$this->publisher->save($con);
		}
		$params = array(
			'is_new' => $this->isNew(),
			'item' => $this
		);
		parent::save($con);
		ActionManager::notify("CRUISECONTROL_SAVE", $params);
	}

	public function addAssignee($assignee) {
		if ($assignee instanceof CruisecontrolAssignee) {
			if (!isset($this->assignees) || !is_array($this->assignees)) {
				$this->assignees = array();
			}
			//$assignee->setProject($this->getProject());
			$this->assignees[] = $assignee;
		}
	}

	public function getAssignees() {
		if (!isset($this->assignees)) {
			$c = new Criteria();
			$c->add(CruisecontrolAssigneePeer::CRUISECONTROL_ID, $this->getPrimaryKey());
			$this->assignees = CruisecontrolAssigneePeer::doSelect($c);
		}
		return $this->assignees;
	}
	
	public function setPublisher($publisher) {
		if ($publisher instanceof CruisecontrolPublisher) {
			$this->publisher = $publisher;
		}
	}

	public function getPublisher() {
		if (!isset($this->publisher)) {
			$c = new Criteria();
			$c->add(CruisecontrolPublisherPeer::CRUISECONTROL_ID, $this->getPrimaryKey());
			$this->publisher = CruisecontrolPublisherPeer::doSelectOne($c);
		}
		return $this->publisher;
	}

	public function getProject() {
		if (!isset($this->project)) {
			$this->project = ProjectPeer::retrieveByPK($this->getProjectId());
		}
		return $this->project;
	}
	
	function isPublic() {
		return $this->getProject()->isPublic();
	}
	



}
?>
