<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-16
 *
 * CruiseControl SOAP Interface for GForge5.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('plugins/build/lib/CruisecontrolPeer.php');
require_once('plugins/build/lib/CruisecontrolBuildPeer.php');
require_once('plugins/build/lib/CruisecontrolBuildChangeUserPeer.php');
require_once('plugins/build/lib/CruisecontrolDefectPeer.php');
require_once('plugins/build/lib/LogFactory.php');
require_once('plugins/build/lib/XMLLogHelper.php');

// Used by CruiseControl
$server->register(
	'uploadBuildXML',
	array(
		'sessionHash'=>'xsd:string',
		'cruisecontrol_id'=>'xsd:int',
		'base64_contents'=>'xsd:string'
	),
	array('uploadBuildXMLResponse'=>'xsd:boolean'),
	$uri,$uri.'#uploadBuildXML','rpc','encoded'
);

// (more generic) Used by Jenkins 
$server->register(
	'uploadBuildResults',
	array(
		'sessionHash'=>'xsd:string',
		'cruisecontrol_id'=>'xsd:int',
		'event_type'=>'xsd:int',
		'was_successful'=>'xsd:boolean',
		'was_fix'=>'xsd:boolean',
		'was_necessary'=>'xsd:boolean',
		'base64_log'=>'xsd:string',
	),
	array('uploadBuildResultsResponse'=>'xsd:boolean'),
	$uri,$uri.'#uploadBuildResults','rpc','encoded'
);

function continueSession($sessionHash, $project_id, $cruisecontrol_id) {
	$CControl = CruisecontrolPeer::retrieveByPK($cruisecontrol_id);
	$session = Session::getSession();

	if ($CControl && is_object($CControl)) {
	 	$data = explode(':', $sessionHash, 2);
	 	if (count($data) == 2) {
			$aux_project_id = $data[0];
			$user_hash = base64_decode($data[1]);
	
			$seed = GFConf::get('build.hash');
			$dev_id = $CControl->getSubmitterId();
			$unique_id = ($project_id+$dev_id+$cruisecontrol_id);
			$temp_hash = base64_encode(sha1($seed.$project_id.$seed.$dev_id.$seed.$cruisecontrol_id.$seed.$unique_id.$seed, true));
			$hash = base64_encode(sha1($temp_hash.$project_id.$temp_hash.$cruisecontrol_id.$temp_hash, true));
			if ($project_id == $aux_project_id && $user_hash == $hash) {
				$user = UserPeer::retrieveByPk($dev_id);
				$session->setUser($user);
			} else {
				throw new SessionException('Invalid session string');
			}
		} else {
			throw new SessionException('Invalid session string');
		}
	} else {
		throw new SessionException('Invalid session string');
	}
	return $session;
}


function uploadBuildResults($sessionHash, $cruisecontrol_id, $event_type, $was_successful, $was_fix, $was_necessary, $base64_log) {
	$CControl = CruisecontrolPeer::retrieveByPK($cruisecontrol_id);
	if (!$CControl || !is_object($CControl)) {
		return new soap_fault ('', 'uploadBuildXML', 'Invalid Data Received', 'Wrong CruisecontrolId');
	}

	try {
		$session = continueSession($sessionHash, $CControl->getProjectId(), $cruisecontrol_id);
	} catch (SessionException $ex) {
		return new soap_fault ('', 'uploadBuildXML', $ex->getMessage(), $ex->getMessage());
	}
	
	$log = base64_decode($base64_log);
	$Project = $CControl->getProject();

	if (!$Project || !is_object($Project)) {
		return new soap_fault ('', 'uploadBuildXML', 'Invalid Project', 'Invalid Project');
	} else if (!$session->getUser()->isProjectMember($Project)) {
		return new soap_fault ('', 'uploadBuildXML', 'Permission denied', 'You need to be a member of this project');
	}

	try {
	  	$CControlBuild = new CruisecontrolBuild();
	  	$CControlBuild->setCruisecontrolId($cruisecontrol_id);
	  	$CControlBuild->setBuildDate(date('Y-m-d H:i:s'));
	  	$CControlBuild->setWasSuccessful($was_successful);
	  	$CControlBuild->setWasNecessary($was_fix);
	  	$CControlBuild->setWasFix($was_necessary);
	  	$CControlBuild->setDescription($log);
	  	$CControlBuild->setEventType($event_type);
	  	$CControlBuild->save();
	  	$build_id = $CControlBuild->getCruisecontrolBuildId();
	  	
	  	//TODO: add the cruisecontrol_build_change_user
	  	createDefect($cruisecontrol_id, $was_successful, $CControlBuild, $CControl);
	} catch (Exception $ex) {
		return new soap_fault ('', 'uploadBuildXML', $ex->getMessage(), $ex->getMessage());
	}
}

// Upload Build XML
function uploadBuildXML($sessionHash, $cruisecontrol_id, $event_type, $base64_contents) {

	$CControl = CruisecontrolPeer::retrieveByPK($cruisecontrol_id);
	if (!$CControl || !is_object($CControl)) {
		return new soap_fault ('', 'uploadBuildXML', 'Invalid Data Received', 'Wrong CruisecontrolId');
	}

	try {
		$session = continueSession($sessionHash, $CControl->getProjectId(), $cruisecontrol_id);
	} catch (SessionException $ex) {
		return new soap_fault ('', 'uploadBuildXML', $ex->getMessage(), $ex->getMessage());
	}

	$contents = substr(base64_decode($base64_contents), 10);
  	$contents = gzinflate($contents);

	$Project = $CControl->getProject();

	if (!$Project || !is_object($Project)) {
		return new soap_fault ('', 'uploadBuildXML', 'Invalid Project', 'Invalid Project');
	} else if (!$session->getUser()->isProjectMember($Project)) {
		return new soap_fault ('', 'uploadBuildXML', 'Permission denied', 'You need to be a member of this project');
	}

	$contents = substr(base64_decode($base64_contents), 10);
  	$contents = gzinflate($contents);
  	
  	try {
	  	$log = new XMLLogHelper($contents);
	  	$CControlBuild = new CruisecontrolBuild();
	  	$CControlBuild->setCruisecontrolId($cruisecontrol_id);
	  	$CControlBuild->setBuildDate($log->getBuildTimestamp());
	  	$CControlBuild->setWasSuccessful($log->isBuildSuccessful());
	  	$CControlBuild->setWasNecessary($log->wasBuildNecessary());
	  	$CControlBuild->setWasFix($log->isBuildFix());
	  	$CControlBuild->setDescription($log->getBuildInfo());
	  	$CControlBuild->setEventType($event_type);
	  	$CControlBuild->save();
	  	$build_id = $CControlBuild->getCruisecontrolBuildId();
	  	
	  	$usernames = $log->getBuildParticipants();
	  	foreach($usernames as $username => $aux) {
	  		$user = UserPeer::retrieveByUnixName($username);
	  		if ($user && is_object($user)) {
		  		$CControlBuildUser = new CruisecontrolBuildChangeUser();
		  		$CControlBuildUser->setCruisecontrolBuildId($build_id);
		  		$CControlBuildUser->setUserId($user->getUserId());
		  		$CControlBuildUser->save();
		  	}
	  	}

	    if ($event_type == CruisecontrolBuildPeer::BUILD_ENDED) {
	  	    createDefect($cruisecontrol_id, $log->isBuildSuccessful(), $CControlBuild, $CControl);
	    }

	  	return $build_id;
	} catch (Exception $ex) {
		return new soap_fault ('', 'uploadBuildXML', $ex->getMessage(), $ex->getMessage());
	}
}

function createDefect($cruisecontrol_id, $was_successful, $CControlBuild, $CControl) {
	$c=new Criteria();
	$c->add(CruisecontrolDefectPeer::CRUISECONTROL_ID, $cruisecontrol_id);
	$c->add(CruisecontrolDefectPeer::OPEN, true);
	$CCDefect = CruisecontrolDefectPeer::doSelectOne($c);

  	if (!$was_successful) {
		if (!$CCDefect || !is_object($CCDefect)) {
  			openTicket($CControl, $CControlBuild);
  		} else {
			$defect_id = $CCDefect->getCruisecontrolDefectId();
			$CControlBuild->setCruisecontrolDefectId($defect_id);
			$CControlBuild->save();
  		}
  	} else {
		if ($CCDefect && is_object($CCDefect)) {
  			closeTicket($CControl, $CControlBuild, $CCDefect);
			$defect_id = $CCDefect->getCruisecontrolDefectId();
			$CControlBuild->setCruisecontrolDefectId($defect_id);
			$CControlBuild->save();
  		}
  	}
}

function openTicket($CControl, $CControlBuild) {
	try {
		$CCPublisher = $CControl->getPublisher();
		$project_id = $CControl->getProjectId();
		$tracker_id = $CCPublisher->getTrackerId();

		$Tracker = TrackerPeer::retrieveByPk($tracker_id);
		if (!$Tracker || !is_object($Tracker) ||
			$Tracker->getProject()->getProjectId() != $project_id ||
			!$Tracker->userCanCreate()) {
			return new soap_fault('', 'uploadBuildXML',
								'Could Not Open Ticket: Permission denied',
								'Could Not Open Ticket: Permission denied');
		}

		$TrackerItem = new TrackerItem();
		$Tracker->addTrackerItem($TrackerItem);

		$priority = 3; // Medium
		$sort_ord = 0;
		$summary = 'Build failed for ' . $CControl->getBuildName() .'';
		$details  = 'Build failed for "' . $CControl->getBuildName() ."\"\n\n";
		$details .= "The error was:\n";
		$details .= $CControlBuild->getDescription() . "\n\n";
		$details .= "--------------------------------\n";
		$users = $CControlBuild->getChangeUsers();

		if (count($users) > 0) {
			$details .= "The following users made changes since the last build:\n";
			foreach($users as $key => $val) {
				$details .= "\t$key\n";
			}
		}

		$details .= "\nTime of build: " . $CControlBuild->getBuildDate();
	
		$TrackerItem->setSubmittedBy($CControl->getSubmitterId());
		$TrackerItem->setPriority($priority);
		$TrackerItem->setSummary($summary);
		$TrackerItem->setDetails($details);
		$TrackerItem->setSortOrder($sort_ord);

		$assigneesObj = $CControl->getAssignees();
		$validAssignees = array();
		if (is_array($assigneesObj) && count($assigneesObj) > 0) {
			foreach ($assigneesObj as $assignee) {
				$validAssignees[$assignee->getAssignee()] = 100;
			}
		}

		// If it's not assigned to anyone, assign to Nobody
		if (count($validAssignees) == 0) {
			$validAssignees['100'] = 0;
		}
	
		$validDependencies = array();

		$efields = array();
		$trackerEF = $Tracker->getTrackerExtraFields();
		foreach ($trackerEF as $ef) {
			if ($ef->getFieldType() == TrackerExtraField::TYPE_STATUS) {
				$elms = $ef->getTrackerExtraFieldElements();
				foreach ($elms as $elm) {
					if (strtolower($elm->getElementName()) == 'open') {
						$efields[$ef->getTrackerExtraFieldId()] = $elm->getElementId();
						break;
					}
				}
			} else if ($ef->getFieldType() == TrackerExtraField::TYPE_TEXT ||
			    $ef->getFieldType() == TrackerExtraField::TYPE_TEXTAREA) {
				$efields[$ef->getTrackerExtraFieldId()] = '';
			} else {
				$efields[$ef->getTrackerExtraFieldId()] = '0';
			}
		}
		
		$TrackerItem->addAssignees($validAssignees);
		$TrackerItem->addRelations($validDependencies);
		if (count($efields) > 0) {
			$TrackerItem->addExtraFields($efields);
		}

		$TrackerItem->save();
		// REPORT THE ACTION
		ActionManager::notify('TRACKER_ITEM_ADD_ACTION', $TrackerItem);

		$CCDefect = new CruisecontrolDefect();
		$CCDefect->setCruisecontrolId($CControl->getCruisecontrolId());
		$CCDefect->setTrackerItemId($TrackerItem->getTrackerItemId());
		$CCDefect->save();
		$defect_id = $CCDefect->getCruisecontrolDefectId();
		
		$CControlBuild->setCruisecontrolDefectId($defect_id);
		$CControlBuild->save();

		return true;

	} catch (InvalidDataException $ex) {
		return new soap_fault('', 'uploadBuildXML',
							'Could Not Open Ticket, Invalid Data: '. implode("\n", $ex->getErrors()),
							'Could Not Open Ticket, Invalid Data: '. implode("\n", $ex->getErrors()));
	} catch (Exception $ex) {
		return new soap_fault('', 'uploadBuildXML',
							'Could Not Open Ticket: '. $ex->getMessage(),
							'Could Not Open Ticket: '. $ex->getMessage());
	}
}

function closeTicket($CControl, $CCBuild, $CCDefect) {
	try {
		$tracker_item_id = $CCDefect->getTrackerItemId();
		$TrackerItem = TrackerItemPeer::retrieveByPk($tracker_item_id);

		//validate that this is a legitimate trackeritem for this tracker and user can add items
		if (!$TrackerItem || !$TrackerItem->userCanUpdate()) {
			return new soap_fault('', 'uploadBuildXML',
								'Could Not Close Ticket: Permission Denied',
								'Could Not Close Ticket: Permission Denied');
		}
		
		addMessage($CControl, $CCBuild, $CCDefect);
	
		$Tracker = $TrackerItem->getTracker();
		$TrackerItem->setLastModifiedDate(time());
		$TrackerItem->setLastModifiedBy($CControl->getSubmitterId());
	
		$efields = array();
		$trackerEF = $Tracker->getTrackerExtraFields();

		foreach ($trackerEF as $ef) {
			if ($ef->getFieldType() == TrackerExtraField::TYPE_STATUS) {
				$elms = $ef->getTrackerExtraFieldElements();
				foreach ($elms as $elm) {
					if (strtolower($elm->getElementName()) == 'closed') {
						$efields[$ef->getTrackerExtraFieldId()] = $elm->getElementId();
						break;
					}
				}
			} else if ($ef->getFieldType() == TrackerExtraField::TYPE_TEXT ||
			    $ef->getFieldType() == TrackerExtraField::TYPE_TEXTAREA) {
				$efields[$ef->getTrackerExtraFieldId()] = '';
			} else {
				$efields[$ef->getTrackerExtraFieldId()] = '0';
			}
		}
		$TrackerItem->updateExtraFields($efields);
		$TrackerItem->save();

		$CCDefect->setOpen(false);
		$CCDefect->save();

		// REPORT THE ACTION
		ActionManager::notify('TRACKER_ITEM_EDIT_ACTION',$TrackerItem);
		return true;
	
	} catch (InvalidDataException $ex) {
		return new soap_fault('', 'uploadBuildXML',
							'Could Not Close Ticket: '.implode("\n", $ex->getErrors()),
							'Could Not Close Ticket: '.implode("\n", $ex->getErrors()));
	} catch (Exception $ex) {
		return new soap_fault('', 'uploadBuildXML',
							'Could Not Close Ticket: '.$ex->getMessage(),
							'Could Not Close Ticket: '.$ex->getMessage());
	}
}

function addMessage($CControl, $CCBuild, $CCDefect) {
	try {
		$date = $CCBuild->getBuildDate();
		$tracker_item_id = $CCDefect->getTrackerItemId();
		$TrackerItem = TrackerItemPeer::retrieveByPk($tracker_item_id);
		if (!$TrackerItem || !is_object($TrackerItem)) {
			return new soap_fault('', 'uploadBuildXML',
								'Could Not Add Message: No Such Ticket',
								'Could Not Add Message: No Such Ticket');
		}
		$body = "*** Automated message from the build system ***\n\n";
        $body .= "The project can be successfully built now.\n\n";
        $body .= "This ticket is being closed.\n\n";
        $body .= "Time of build: " .$date;

		$tim = new TrackerItemMessage();
		$TrackerItem->addTrackerItemMessage($tim);
		$tim->setBody($body);
		$tim->setSubmittedBy($CControl->getSubmitterId());
		$tim->save();
		return $tim->getTrackerItemMessageId();

	} catch (Exception $ex) {
		return new soap_fault('', 'addMessage',
							'Could Not Add Message: '.$ex->getMessage(),
							'Could Not Add Message: '.$ex->getMessage());
	}
}

?>
