<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Federico Tarantini      federico@gforgegroup.com
 * @date 2006-03-13
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

include ("lib/external/jpgraph/jpgraph.php");
include ("lib/external/jpgraph/jpgraph_line.php");

$session = Session::getSession();
$Lang = $session->getLanguage();

$Project = GFUrl::getUrl()->getProject();
$project_id = $Project->getProjectId();

$con = Propel::getConnection(TrackerPeer::DATABASE_NAME);
$stmt = $con->createStatement();

// Get filter data
$config_id = Validator::sanitizeInt($_GET['config_id']);
$begin = Validator::sanitizeDate($_GET['start_date']);
$end = Validator::sanitizeDate($_GET['end_date']);
$subprojects=((isset($_REQUEST['subprojects'])) ? Validator::sanitizeInt($_REQUEST['subprojects']) : 0 );

$all_ids = array();
if ($subprojects == 1) {
	$all_ids = $Project->getAllSubprojectIds();
	$all_ids[] = $project_id;
} else {
	$all_ids[] = $project_id;
}

$sql = 'SELECT date_part(\'year\', ccb.build_date)::text || \'-\' || date_part(\'month\', ccb.build_date)::text AS month, 
		COUNT(ccb.cruisecontrol_build_id) AS builds, ccb.was_successful AS status
		FROM cruisecontrol cc, cruisecontrol_build ccb WHERE cc.project_id IN ('.implode(',',$all_ids).') ';
if ($config_id > 0) {
	$sql .= 'AND ccb.cruisecontrol_id = '.$config_id.' ';
}
$sql .= 'AND cc.cruisecontrol_id = ccb.cruisecontrol_id
		AND ccb.build_date BETWEEN \''.$begin.'\' AND \''.$end.'\'
		GROUP BY month, ccb.was_successful';

$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

$raw_data  = array();
if ($rs != null && $rs->getRecordCount() > 0) {
	while($rs->next()) {
		$raw_data[$rs->getString('status')][] = array($rs->getString('month') => $rs->getInt('builds'));
	}
}

// Create an array with all the months of the given date range
$months = array();
$i = strtotime($begin);
while($i <= strtotime($end)) {
        $date = getdate($i);
        $months[$date['year'].'-'.$date['mon']] = 0;
        $i = mktime(0, 0, 0, ($date['mon']+1), 1, $date['year']);
}

$result = array();
foreach($raw_data as $status => $value) {
	if($status == 't'){
		$st = 1;
	} else {
		$st = 0;
	}
	$result[$st] = $months;	
}
foreach($raw_data as $status => $value) {
	foreach($value as $month_builds) {
		foreach($month_builds as $month => $builds) {
			if($status == 't'){
				$st = 1;
			} else {
				$st = 0;
			}
			$result[$st][$month] = $builds;
		}
	}
}

$font = $Lang->getFontProperties(FF_FONT1, FS_BOLD, 12);
// Create the graph.
$graph = new Graph(450, 250);
$graph->SetScale('textlin');
$graph->img->SetMargin(40, 100, 15, 70);
$graph->title->Set($Lang->getText('CruiseControlBuildReport.BuildsGraphTitle'));
$graph->title->SetFont($font[0], $font[1], $font[2]);
$graph->setBox(false);
$graph->setFrame(false);

$font = $Lang->getFontProperties(FF_FONT1, FS_BOLD, 8);
$graph->xaxis->SetTitle($Lang->getText('Common.Month'));
$graph->xaxis->SetTitleMargin(39);
$graph->xaxis->title->SetFont($font[0], $font[1], $font[2]);
$graph->xaxis->SetTickLabels(array_keys($months));
$graph->xaxis->SetLabelAngle(90);

$graph->yaxis->SetTitle($Lang->getText('CruiseControlBuildReport.Builds'), 'low');
$graph->yaxis->SetTitleMargin(25);
$graph->yaxis->title->SetFont($font[0], $font[1], $font[2]);
$graph->yaxis->scale->SetGrace(0);
$graph->yaxis->SetLabelMargin(1);

$graph ->legend->Pos( 0.8,0.1,"left" ,"center");

// Set default color array
$colors = array('#66CD00', '#7AC5CD');

if (count($result) != 0) {
	foreach($result as $status => $month_builds) {
		$plots[$status] = new LinePlot(array_values($month_builds));
		if($status) {
			$legend = 'Successful';
		} else {
			$legend = 'Failed';
		}
		$plots[$status]->SetLegend($legend);
		$plots[$status]->SetFillColor($colors[$status]);
	}
	$accplot = new AccLinePlot($plots);
	$graph->Add($accplot);
} else {
        $lineplot = new LinePlot(array_values($months));
        $graph->Add($lineplot);
}

if (ob_get_level()) ob_end_clean();
$graph->Stroke();
