<?php
require_once 'plugins/calendar/lib/om/BaseCalendarEvent.php';
require_once 'plugins/calendar/lib/CalendarPeer.php';
require_once 'plugins/calendar/lib/external/bennu/lib/bennu.inc.php';

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.calendar.lib 
 */
class CalendarEvent extends BaseCalendarEvent {
	const IN_SECONDS = 1;
	const IN_HOURS   = 2;
	const IN_DAYS    = 3;
	const IN_WEEKS   = 4;
	const IN_MONTHS  = 5;

	//No repeat
	const R_NONE = 0;
	//Daily ocurrence
	const R_DAILY = 1;
	//Daoly occurrence on working days
	const R_DAILY_MF = 3;
	//Weekly occurrence
	const R_WEEKLY = 4;
	//Monthly occurrence
	const R_MONTHLY = 5;

	private $invalid_data = array();
	
	public function getDurationOfEvent($time_unit = self::IN_SECONDS){

		$duration = ($this->getToDate(null) - $this->getFromDate(null));
		switch($time_unit){
			case self::IN_SECONDS:{
				return $duration;
			}
			case self::IN_HOURS:{
				return $duration / (60 * 60);
				break;
			}
			case self::IN_DAYS:{
				return  $duration / (60 * 60 * 24);
				break;
			}
			case self::IN_WEEKS:{
				return $duration / (60 * 60 * 24 * 7);
				break;
			}
			
		}
	}

	public function durationExceedsStartingMonth(){
		
		return $this->getFromDate('%m') != $this->getToDate('%m');	
	}

	public function endsSameDay(){
		return $this->getFromDate('%d') == $this->getToDate('%d');
	}

	public function durationExceedsWeek(){
		$duration = $this->getDurationOfEvent(self::IN_WEEKS);

		return $duration > 1;
	}

	private function overlapsExistingEvents(){
		$c = new Criteria;

		//are there events  inside this event time frame?
		$calendar_id = $this->getCalendar()->getCalendarId();
		if(!$this->isNew()){
			$aux =  ' AND calendar_event_id != ' . $this->getCalendarEventId();
		} else {
			$aux = '';
		}

		$sql = 'SELECT COUNT(*) FROM calendar_event WHERE 
		((from_date <\'' . $this->getFromDate('%Y%m%d %H:%M %Z') . '\' AND  to_date >\'' . $this->getFromDate('%Y%m%d %H:%M %Z') . '\' )' .
		' OR ' .
		'(from_date <\'' . $this->getToDate('%Y%m%d %H:%M %Z') . '\' AND to_date >\'' . $this->getToDate('%Y%m%d %H:%M %Z') . '\') ' . 
		' OR '. 
		'(from_date > \'' . $this->getFromDate('%Y%m%d %H:%M %Z') . '\' AND to_date <\'' . $this->getToDate('%Y%m%d %H:%M %Z') . '\' ))' .
		' AND calendar_id = ' . $this->getCalendarId() . 
		$aux;
		//echo $sql; die();

		$stmt = Propel::getConnection(CalendarEventPeer::DATABASE_NAME)->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
		$rs->next();

		if($rs->getInt(1) > 0){
			return true;
		} else {
			return false;
		}

	}


	public function setFromDate($v){
		try{
			parent::setFromDate($v);
		} catch (PropelException $e){
			$this->invalid_data['from_date'] = Language::getSessText('UserCalendar.EventInvalidFromDate');
		}
	}

	public function setToDate($v){
		try{
			parent::setToDate($v);
		} catch (PropelException $e) {
			$this->invalid_data['to_date'] = Language::getSessText('UserCalendar.EventInvalidToDate');
		}
	}

	public function setOccurrenceTo($v){
		
		switch($this->getOccurrence()){
			case self::R_DAILY:{
				if($this->getDurationOfEvent(self::IN_DAYS) > 1){
					$this->invalid_data[] = Language::getSessText('UserCalendar.EventInvalidDailyOccurrence');
					return;
				}
				break;
			}
			case self::R_WEEKLY: {
				if($this->getDurationOfEvent(self::IN_WEEKS) > 1){
					$this->invalid_data[] = Language::getSessText('UserCalendar.EventInvalidWeeklyOccurrence');
					return;
				}
				break;
			}
			case self::R_MONTHLY: {
				if($this->getDurationOfEvent(self::IN_DAYS) > 28){
					$this->invalid_data[] = Language::getSessText('UserCalendar.EventInvalidMonthlyOccurrence');
					return;
				}
				break;
			}
		}
		
	
		try{
			parent::setOccurrenceTo($v);
		} catch (PropelException $e){
			$this->invalid_data[] = Language::getSessText('UserCalendar.EventInvalidOccurrenceTo');
		}
	}


	public function save($con=null){
		$missing_data = array();


		if($this->from_date > $this->to_date){
			$this->invalid_data['to_date'] = Language::getSessText('UserCalendar.EventDateMissmatch');
		}

		if(is_null($this->from_date) || empty($this->from_date)){
			$missing_data['from_date'] = Language::getSessText('UserCalendar.EventEmptyFromDate');
		}
		if(is_null($this->to_date) || empty($this->to_date)){
			$missing_data['to_date'] = Language::getSessText('UserCalendar.EventEmptyToDate');
		}
		if(is_null($this->subject) || empty($this->subject)){
			$missing_data['subject'] = Language::getSessText('UserCalendar.EventEmptySubject');
		}
		if($this->to_date < $this->from_date){
			$this->invalid_data['from_date'] = Language::getSessText('Usercalendar.EventDateMissmatch');
		}
		/*Disabled for now, since we need an extra field to set an event as transparent (no overlapping validation
		is performed) or opaque.
		if($this->overlapsExistingEvents()){
			$this->invalid_data['from_date'] = Language::getSessText('UserCalendar.EventOverlapsExistingEvents');
		}*/
		if($this->isNew()){
			if($this->occurrence_to && ($this->from_date > $this->occurrence_to)){
				$this->invalid_data['occurrence_to'] = Language::getSessText('UserCalendar.EventInvalidOccurrenceTo');
			}	
		
			if($this->occurrence && is_null($this->occurrence_to)){
				$this->invalid_data['occurrence_to'] = Language::getSessText('UserCalendar.EventInvalidOccurrenceTo');
			}
			if(!$this->validRecurrentEvents()){
				$this->invalid_data['occurrence_to'] = Language::getSessText('UserCalendar.MaxOccurrenceExceeded');
			}
		}


		$errors = array_merge($missing_data, $this->invalid_data);

		if(count($errors) > 0){
			throw new InvalidDataException($errors);
		}

		parent::save($con);
	}

	public function getSection(){
		return $this->getCalendar()->getSection();
	}

	public function userIsAdmin(){
		return $this->getCalendar()->userIsAdmin();
	}

	public function userCanRead(){
		if($this->getSection() == 'user'){
			if($this->userIsAdmin() || $this->getIsPublic()){
				return true;
			}
		}

		return false;
	}

	public function userCanUpdate(){
		if($this->getSection() == 'user'){
			if($this->userIsAdmin()){
				return true;
			}
		}

		return false;
	}

	public function userCanDelete(){
		if($this->getSection() == 'user'){
			return $this->userCanUpdate();
		}
	}

	public function createICalendarEvent(){
		$event = new iCalendar_event();	

		$hostname = GfConf::get('system.hostName');
		$event = new iCalendar_event();
		$event->add_property('uid', $this->getFromDate('%Y%m%dT%H%M%S%z') . $this->getCalendarEventId() . '@' . $hostname);
		$event->add_property('summary', $this->getSubject());
		$event->add_property('description', $this->getDescription());
		if($this->getIsPublic()){
			$event->add_property('class', 'PUBLIC');
		} else {
			$event->add_property('class', 'PRIVATE');
		}
		$event->add_property('dtstart', gmdate('Ymd\THis\Z',$this->getFromDate(null)), array('value'=>'DATE-TIME'));
		$event->add_property('dtend', gmdate('Ymd\THis\Z',$this->getToDate(null)), array('value'=>'DATE-TIME'));
		$event->add_property('transp', 'OPAQUE');
		$event->add_property('dtstamp', gmdate('Ymd\THis\Z'));

		return $event;
	}

	public function exportToICalendar(){
		$ical = Calendar::getICalendarObject();
		$ical->add_component($this->createICalendarEvent());
		
		return $ical->serialize();
	}

	private function validRecurrentEvents(){
		$max = GFConf::get('calendar.maxRecurringEvents');
		$is_valid = false;
		if(!$max){
			//Safe default
			$max=10;
		}

		switch($this->getOccurrence()){

			case self::R_DAILY:
				$max_date = strtotime("+$max days", $this->getFromDate(null));
				$is_valid = $this->getOccurrenceTo(null) > $max_date ? false : true;
				break;

			case self::R_WEEKLY:
				$max_date = strtotime("+$max weeks", $this->getFromDate(null));
				$is_valid = $this->getOccurrenceTo(null) > $max_date ? false : true;
				break;

			case self::R_MONTHLY:
				$max_date = strtotime("+$max months", $this->getFromDate(null));
				$is_valid = $this->getOccurrenceTo(null) > $max_date ? false : true;
				break;

			case self::R_NONE:
			default:
				//Nothing to check here
				$is_valid=true;
		}

		return $is_valid;
	}

		
			

}
