<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-03-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/docman/lib/om/BaseDocmanFolder.php';
require_once("plugin/ActionManager.php");
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");

/** 
 * The skeleton for this class was autogenerated by Propel  
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.docman.lib 
 */
class DocmanFolder extends BaseDocmanFolder implements GFPermission {
	const SECTION='docmanfolder';
	const PERMLEVEL_NOACCESS=0;
	const PERMLEVEL_READ=1;
	const PERMLEVEL_WRITE=2;
	const PERMLEVEL_ADMIN=2;
	const IS_PUBLIC=1;
	const IS_PRIVATE=0;

	function getId() {
		return $this->getDocmanFolderId();
	}

	function getName() {
		return $this->getFolderName();
	}
	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see DocmanFolder::setDeleted()
	 * @see DocmanFolder::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException();
		}
		
		$crit = new Criteria();
		$crit->add(DocmanFolderPeer::PARENT_FOLDER_ID,$this->getDocmanFolderId());
		$folders = DocmanFolderPeer::doSelect($crit);
		foreach ($folders as $folder) {
			$folder->delete();
		}
		
		// Audit trail
		$audit = new AuditTrail();
		$audit->setUser(Session::getSession()->getUser());
		$audit->setSection(Project::SECTION);
		$audit->setRefId($this->getProjectId());
		$audit->setFieldName(self::SECTION);
		$audit->setNewValue('_deleted_');
		$audit->setOldValue($this->getFolderName().' '.Language::getSessText('Docman.FolderDeleteAllContents'));
		$audit->save();		
		
		$params = array(
			'is_new' => $this->isNew(),
			'item' => $this
		);

		ActionManager::notify("DOCMAN_FOLDER_DELETE", $params);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null, $check_duplicity = true) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException();
		} else if (!$this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException();
		}

		$errors=array();
		if (strlen($this->folder_name) < 1) {
			$errors['folder_name'] = Language::getSessText('Docman.EmptyFoldername');
		}
		// first check. if they want to make the same folder name under same parent, don't allow it
		if ($this->isNew() && (count($errors) == 0) && $check_duplicity) {
			$c = new Criteria();
			$c->add(DocmanFolderPeer::FOLDER_NAME,$this->folder_name);
			$c->add(DocmanFolderPeer::PARENT_FOLDER_ID,$this->parent_folder_id);
			$c->add(DocmanFolderPeer::PROJECT_ID,$this->getProjectId());
			$already_here = DocmanFolderPeer::doSelectOne($c);
			if (is_object($already_here)) {
				$errors['folder_name_exists'] = Language::getSessText('Docman.FolderExists');
			}
		}
		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}

		if (($this->isNew()) && ($this->parent_folder_id == 0)) {
			$createrolesettings = true;
		} else {
			$createrolesettings = false;
		}

		$params = array(
			'is_new' => $this->isNew(),
			'item' => $this
		);

		// If we were only updating the document count, then don't
		// fire the update event
		$send_event = FALSE;
		if (count($this->modifiedColumns) > 1 ||
			$this->modifiedColumns[0] != DocmanFolderPeer::DOC_COUNT) {
			$send_event = TRUE;
		}

		parent::save($con);
		
		if ($send_event) {
			ActionManager::notify("DOCMAN_FOLDER_SAVE", $params);
		}

		if ($createrolesettings) {
			$con = Propel::getConnection(TrackerPeer::DATABASE_NAME);
			$sql="INSERT INTO role_setting (role_id,section,ref_id,value) 
				SELECT role_id,'".self::SECTION."','".$this->getPrimaryKey()."',
				'".self::PERMLEVEL_READ."' 
				FROM role WHERE project_id='".$this->getProject()->getProjectId()."'";
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql);
		}


	}


	/**
	 * isPublic - true/false
	 * @return boolean
	 */
	function isPublic() {
		//we are now only setting this on the root folder
		$rootFolder = $this->getRootFolder();

		if (is_object($rootFolder)) {
			return $rootFolder->getIsPublic();
		} else {
			return false;
		}
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 */
	public function userCanCreate() {
		if (!Session::getSession()->isLoggedIn()) {
			return false;
		}

		if ($this->userIsAdmin()) {
			return true;
		}

		$perm = $this->getProject()->getPermission();
		$setting = DocmanProjectSettingPeer::retrieveByPk($this->getProject()->getProjectId());
		if (is_object($setting)) {
			$allow_unprivileged_upload = $setting->getAllowUnprivilegedUpload();
		} else {
			$allow_unprivileged_upload = false;
		}

		if ($this->getTopParentFolderId() != 0) {
			if ($perm->check(DocmanFolder::SECTION, $this->getTopParentFolderId(), DocmanFolder::PERMLEVEL_WRITE)) {
				return true;
			}
		} else {
			$c = new Criteria();
			$c->add(DocmanFolderPeer::PROJECT_ID,$this->getProject()->getProjectId());
			$c->add(DocmanFolderPeer::FOLDER_NAME,'incoming'); // default master folder
			$c->add(DocmanFolderPeer::PARENT_FOLDER_ID,0);
			$incoming = DocmanFolderPeer::doSelectOne($c);
			if (is_object($incoming)) {
				if ($perm->check(DocmanFolder::SECTION, $incoming->getDocmanFolderId(), DocmanFolder::PERMLEVEL_WRITE) || $allow_unprivileged_upload) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		if (!Session::getSession()->isLoggedIn()) {
			return false;
		}

		if (Session::getSession()->getUser()->isSiteAdmin() || $this->getProject()->userIsAdmin()) {
			return true;
		}

		$perm = $this->getProject()->getPermission();
		if ($perm->check(DocmanPlugin::SECTION, 0, DocmanPlugin::ADMIN) || $perm->check(DocmanFolder::SECTION, $this->getTopParentFolderId(), DocmanFolder::PERMLEVEL_ADMIN)) {
			return true;
		}
		return false;
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 */
	public function userCanRead() {
		if ($this->isPublic()) {
			return true;
		}
		if ($this->userIsAdmin()) {
			return true;
		}
		$perm = $this->getProject()->getPermission();
		// check if the user has permissions for reading at least
		if ($perm->check(DocmanFolder::SECTION, $this->getTopParentFolderId(), DocmanFolder::PERMLEVEL_READ)) {
			return true;
		}

		return false;
	}

	/**
	 * Returns the root folder if this is not the root, otherwise returns itself
	 *
	 * @return bool
	 */
	public function getRootFolder() {
		if ($this->getTopParentFolderId() == 0) {
				$c = new Criteria();
				$c->add(DocmanFolderPeer::PROJECT_ID,$this->getProject()->getProjectId());
				$c->add(DocmanFolderPeer::FOLDER_NAME,'incoming'); // default master folder
				$c->add(DocmanFolderPeer::PARENT_FOLDER_ID,0);
				$incoming = DocmanFolderPeer::doSelectOne($c);
				if (is_object($incoming)) {
						return $incoming;
				} else {
						return null;
				}
		} else {
				return DocmanFolderPeer::retrieveByPk($this->getTopParentFolderId());
		}
	}
	
	/**
   	 * Tells if the user is monitoring this item
	 * @return bool
	 */
	public function userIsMonitoring(User $user) {
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, self::SECTION);
		$c->add(MonitorPeer::REF_ID, $this->getPrimaryKey());
		$c->add(MonitorPeer::USER_ID, $user->getUserId());
		$mon = MonitorPeer::doSelectOne($c);
		return (!is_null($mon)) ? true : false;
	}

	public function getFolderUrl() {
		$ret = GFUrl::getURL()->projectURL($this->getProject(),'','docman', $this->getFolderPath());
		return $ret;
	}

	public function getFolderPath() {

		if ($this->getFolderName() == 'incoming' &&
			$this->getParentFolder() == null) {
			return '';
		}

		$ret = '/'.$this->getFolderName();

		$parent = $this->getParentFolder();
		while ($parent != null) {
			$ret = '/'.$parent->getFolderName().$ret;
			$parent = $parent->getParentFolder();
		}
		return $ret;
	}

	public function getParentFolder() {
		if ($this->getParentFolderId() == 0) {
			return null;
		} else {
			return DocmanFolderPeer::retrieveByPk($this->getParentFolderId());
		}
	}
}
?>
