<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version $Id$
 * @author Daniel A. Perez	daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('lib/core/Filesystem.php');
require_once('lib/core/FilesystemPeer.php');
require_once('plugins/docman/lib/DocmanFile.php');
require_once('plugins/docman/lib/DocmanFileFolder.php');
require_once('plugins/docman/lib/DocmanFilePeer.php');
require_once('plugins/docman/lib/DocmanFileVersion.php');
require_once('plugins/docman/lib/DocmanFileVersionPeer.php');
require_once('plugins/docman/lib/DocmanFolder.php');
require_once('plugins/docman/lib/DocmanFolderPeer.php');
require_once('lib/exceptions/PermissionDeniedException.php');
require_once('Validator.php');
require_once('plugins/docman/lib/FastJSON.php');

$session = Session::getSession();
$Lang = $session->getLanguage();
$project = GFUrl::getUrl()->getProject();

$folder_id = isset($_REQUEST['folder_id']) ? Validator::sanitizeInt($_REQUEST['folder_id']) : 0;
$filename = isset($_POST['filename'])? Validator::sanitizeString($_POST['filename']) : "";
$description = isset($_POST['description']) ? Validator::sanitizeString($_POST['description']) : "";
$changes = isset($_POST['changes'])? Validator::sanitizeString($_POST['changes']) : "";
$message = isset($_POST['message'])? $_POST['message'] : "";
$strU = new StringUtils(StringUtils::FULL_HTML);

$where = 0;
if ($folder_id != 0) {
	$where = $folder_id;
} else {
	//Gets the default "incoming" folder
	$c = new Criteria();
	$c->add(DocmanFolderPeer::PROJECT_ID,$project->getProjectId());
	$c->add(DocmanFolderPeer::FOLDER_NAME,'incoming'); // default master folder
	$c->add(DocmanFolderPeer::PARENT_FOLDER_ID,0);
	$incoming = DocmanFolderPeer::doSelectOne($c);
	//If it doesn't exist, create it
	if (!is_object($incoming)) {
		$incoming = new DocmanFolder();
		$incoming->setFolderName('incoming');
		$incoming->setProject($project);
		$incoming->save();
	}
	
	if (is_object($incoming)) {
		$where = $incoming->getDocmanFolderId();
	}
}

if (!$session->isLoggedIn()) {
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_message"] = "Not logged in";
	echo FastJSON::convert($arr_feedback);
	exit;
}

$DocmanFolder = DocmanFolderPeer::retrieveByPK($where);
if (!is_object($DocmanFolder) || $DocmanFolder->getProjectId()!=$project->getProjectId()) {
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_message"] = "Invalid object";
	echo FastJSON::convert($arr_feedback);
	exit;
}

try {
	if ($filename == "") {
		$message = Language::getSessText('Docman.EmptyFilename');
		
		header('HTTP/1.1 401');
		$arr_feedback = array();
		$arr_feedback["error_message"] = $message;
		echo FastJSON::convert($arr_feedback);
		exit;
	}

	
	$con = Propel::getConnection(DocmanFolderPeer::DATABASE_NAME);
	$con->begin();
	$DocmanFile = new DocmanFile();
	$DocmanFile->setDocmanFolderId($where);
	$DocmanFile->setDescription($description);
	$DocmanFile->setIsLocked(false);
	$DocmanFile->setLockedBy(100);
	$DocmanFile->setFilename($filename);

	$dfv = new DocmanFileVersion();
	$DocmanFile->addDocmanFileVersion($dfv);
	$dfv->setChanges(Validator::sanitizeString($changes));
	$dfv->setFilename($filename);
	$dfv->setCreatedBy($session->getUser()->getUserId());

	$DocmanFile->save();

	//Now, assign the new file to the folder
	$docman_file_folder = new DocmanFileFolder();
	$docman_file_folder->setDocmanFolderId($where);
	$docman_file_folder->setDocmanFileId($DocmanFile->getDocmanFileId());
	$docman_file_folder->save();
	
	//Set the tag
	TagEntryPeer::addTags(DocmanFile::SECTION, $DocmanFile->getDocmanFileId(), 'incoming');			
	
	// create the file in the FS
	$newfile = new Filesystem();
	$newfile->setRefId($dfv->getPrimaryKey());
	$newfile->setSection(DocmanFileVersion::SECTION);
	$newfile->setPostedBy($session->getUser()->getUserId());
	$newfile->setFileType('text/html');
	$newfile->setFileName(trim($filename).".htm");
	$newfile->save();	
	$newfile->setFileDataFromString($strU->cleanHtml($message));

	$con->commit();

	if ($dfv->getIsApproved()) {
		$feedback = $Lang->getText('DocmanFile.AddFeedback');
	} else {
		$feedback = $Lang->getText('DocmanFile.AddQueuedFeedback');
	}

	$file_id = $DocmanFile->getDocmanFileId();
	$author = UserPeer::retrieveByPk($dfv->getCreatedBy());

	$arr_file = array();
	$arr_file['file_id'] = $file_id;
	$arr_file['folder_id'] = $where;
	$arr_file['file_name'] = $filename;
	$arr_file['file_type'] = $newfile->getFileType();
	$arr_file['file_size'] = $newfile->getFileSize();
	$arr_file['author'] = $author->getFullname();
	$arr_file['is_approved'] = $dfv->getIsApproved();
	$arr_file['file_version'] = $dfv->getVersionNumber();
	$arr_file['download_url'] = GFUrl::getURL()->getFullURL(GFUrl::getURL()->downloadURL(DocmanFileVersion::SECTION,$dfv->getDocmanFileVersionId(),$newfile->getFilesystemId(),$newfile->getFileNameSafe()));


	header('HTTP/1.1 200');
	echo FastJSON::convert($arr_file);
	exit;
} catch (InvalidDataException $ex) {
	$errors=$ex->getErrors();
	$feedback = implode('<br />',$errors);
	$arr_feedback = array();
	$arr_feedback["error_message"] = $feedback;
	echo FastJSON::convert($arr_feedback);
	header('HTTP/1.1 401');
	exit;
}

?>
