<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version $Id$
 * @author Daniel A. Perez	daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('lib/core/Filesystem.php');
require_once('lib/core/FilesystemPeer.php');
require_once('plugins/docman/lib/DocmanFile.php');
require_once('plugins/docman/lib/DocmanFilePeer.php');
require_once('plugins/docman/lib/DocmanFileVersion.php');
require_once('plugins/docman/lib/DocmanFileVersionPeer.php');
require_once('plugins/docman/lib/DocmanFolder.php');
require_once('plugins/docman/lib/DocmanFolderPeer.php');
require_once('lib/exceptions/PermissionDeniedException.php');
require_once('Validator.php');
require_once('plugins/docman/lib/FastJSON.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

$project = GFUrl::getUrl()->getProject();

$id = Validator::sanitizeInt($_REQUEST['id']);

if (!$session->isLoggedIn()) {
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_message"] = "Permission denied 1";
	echo FastJSON::encode($arr_feedback);
	exit;
}

$df = DocmanFilePeer::retrieveByPK($id);
if (!is_object($df) 
	|| $df->getDocmanFolder()->getProjectId()!=$project->getProjectId()
	|| !$df->userCanCreate()) {
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_message"] = "Permission denied 1";
	echo FastJSON::encode($arr_feedback);
	exit;
}

$errors=array();

try {
	$file_contents = $_POST['file_content'];
	$file_name = $_POST['file_name'];

	$con = Propel::getConnection(DocmanFolderPeer::DATABASE_NAME);
	$con->begin();
	$dfv = new DocmanFileVersion();
	$dfv->setDocmanFileId($df->getDocmanFileId());
	$dfv->setChanges(Validator::sanitizeString($_REQUEST['changes']));
	//$df->save();
	$dfv->setFilename($file_name);
	$dfv->save();
	// create the file in the FS
	$newfile = new Filesystem();
	$newfile->setRefId($dfv->getPrimaryKey());
	$newfile->setSection(DocmanFileVersion::SECTION);
	$newfile->setPostedBy($session->getUser()->getUserId());

	$newfile->setFileType('text/html');
	$newfile->setFileName($file_name);
	$newfile->save();
	$newfile->setFileDataFromString($file_contents);
error_log("contents: " . $file_contents);
	$con->commit();

	if ($dfv->getIsApproved()) {
		$feedback = $Lang->getText('DocmanFile.AddFeedback');
	} else {
		$feedback = $Lang->getText('DocmanFile.AddQueuedFeedback');
	}

	header('HTTP/1.1 200');
	$arr_feedback = array();
	$arr_feedback["feedback"] = $feedback;
	echo FastJSON::encode($arr_feedback);
	exit;
} catch (InvalidDataException $ex) {
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_message"] = $feedback;
	echo FastJSON::encode($arr_feedback);
	exit;
}
?>
