<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code moves a docman file to another folder
// database
//
require_once('plugins/docman/lib/DocmanFile.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');
require_once('plugins/docman/lib/FastJSON.php');

if ($_SERVER['REQUEST_METHOD'] != 'GET') {
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_message"] = "Permission denied";
	echo FastJSON::encode($arr_feedback);
	exit;
}

$current_folder_id = isset($_GET['current_folder_id']) ? intval($_GET['current_folder_id']) : 0;
$new_folder_id = isset($_GET['new_folder_id']) ? intval($_GET['new_folder_id']) : 0;
$incoming_folder_id = isset($_GET['incoming_folder']) ? intval($_GET['incoming_folder']) : 0;
$file_id = isset($_GET['file_id']) ? intval($_GET['file_id']) : 0;
$operation_type = isset($_GET['operation_type']) ? $_GET['operation_type'] : 'folder';

//If it's a move-up operation, we need to get the parent folder of the document's parent folder

if ($operation_type == 'moveup') {

	$DocmanFolder = DocmanFolderPeer::retrieveByPk($current_folder_id);
	if (is_object($DocmanFolder)) {
		$new_folder_id = $DocmanFolder->getParentFolderId();

		//If the parent-parent folder id is 0, assign the file to the incoming folder
		if ($new_folder_id == 0) {
			$new_folder_id = $incoming_folder_id;
		}
		

	} else {
		header('HTTP/1.1 401');
		$arr_feedback = array();
		$arr_feedback["error_message"] = "Permission denied";
		echo FastJSON::encode($arr_feedback);
		exit;		
	}
}

$project = GFUrl::getUrl()->getProject();
if ($new_folder_id > 0) {
	$DocmanFile = DocmanFilePeer::retrieveByPk($file_id);
	if (!is_object($DocmanFile) || !$DocmanFile->userCanUpdate()) {
		header('HTTP/1.1 401');
		$arr_feedback = array();
		$arr_feedback["error_message"] = "Permission denied";
		echo FastJSON::encode($arr_feedback);
		exit;
	} else {
		$c = new Criteria();
		$c->add(DocmanFileFolderPeer::DOCMAN_FOLDER_ID, $new_folder_id);
		$c->add(DocmanFileFolderPeer::DOCMAN_FILE_ID, $file_id);
		$DocmanFileFolder = DocmanFileFolderPeer::doSelectOne($c);
		
		if ($DocmanFile->getDocmanFolder()->getProject()->getProjectId() != $project->getProjectId() || is_object($DocmanFileFolder)) {
		
			header('HTTP/1.1 401');
			$arr_feedback = array();
			$arr_feedback["error_message"] = "Permission denied";
	
			echo FastJSON::encode($arr_feedback);
			exit;
		} else {
			$Lang = Language::getSessLang();
			
			try {
				$c = new Criteria();
				$c->add(DocmanFileFolderPeer::DOCMAN_FOLDER_ID, $current_folder_id);
				$c->add(DocmanFileFolderPeer::DOCMAN_FILE_ID, $file_id);
				$oldDocmanFileFolder = DocmanFileFolderPeer::doSelectOne($c);
				if (is_object($oldDocmanFileFolder)) {
					$oldDocmanFileFolder->delete();
				}
				
				$newDocmanFileFolder = new DocmanFileFolder();
				$newDocmanFileFolder->setDocmanFolderId($new_folder_id);
				$newDocmanFileFolder->setDocmanFileId($file_id);
				$newDocmanFileFolder->save();

				header('HTTP/1.1 200');
				$arr_feedback = array();
				$arr_feedback["folder_id"] = $newDocmanFileFolder->getDocmanFolderId();
				echo FastJSON::encode($arr_feedback);
				exit;
			} catch (InvalidDataException $ex) {
				$errors=$ex->getErrors();
				$feedback = implode('<br />', $errors);
				header('HTTP/1.1 401');
				$arr_feedback = array();
				$arr_feedback["error_message"] = $feedback;
				//$arr_feedback["error_message"] = "error";
				echo FastJSON::encode($arr_feedback);
				exit;
			}
		}
	}
		
}

?>
