<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a DocmanFile and saves it to the 
// database
//
require_once('plugins/docman/lib/DocmanFile.php');
require_once('Validator.php');
require_once('lib/StringUtils.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');
require_once('plugins/docman/lib/FastJSON.php');

if ($_SERVER['REQUEST_METHOD'] != 'GET') {
	throw new PermissionDeniedException();
}

$project = GFUrl::getUrl()->getProject();

$file_id = intval($_GET['file_id']);
$file_name = StringUtils::toValidUTF8(urldecode($_GET['file_name']));

$DocmanFile = DocmanFilePeer::retrieveByPk($file_id);
if (!is_object($DocmanFile)) {
}
if (!is_object($DocmanFile) || !$DocmanFile->userCanUpdate()) {
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_message"] = "Permission denied";
	echo FastJSON::encode($arr_feedback);
	exit;
}

$DocmanFile->setFilename($file_name);
$DocmanFile->save();

$Lang = Language::getSessLang();

//Verify that a file with the same name does not exist on any folder this file belongs to
$sql = "SELECT count(*) as total
		FROM docman_file_folder dff, docman_file_version dfv, filesystem fs
		WHERE
		dff.docman_file_id = dfv.docman_file_id AND
		dff.docman_file_id <> " . $file_id . " AND
		fs.ref_id = dfv.docman_file_version_id AND
		fs.section = 'docmanfileversion' AND
		fs.file_name_safe = '" . addslashes(StringUtils::removeInvalidChars($file_name, false)) . "' AND
		dff.docman_folder_id IN (
			SELECT dff.docman_folder_id
			FROM
			docman_file_folder dff
			WHERE
			dff.docman_file_id = " . $file_id . "
		)";

$con = Propel::getConnection(UserPeer::DATABASE_NAME);
$stmt = $con->createStatement();
$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
$rs->next();

if ($rs->getInt('total') > 0) {
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_message"] = "A file with the same name exists on a folder this file belongs to";
	echo FastJSON::encode($arr_feedback);
	exit;	
}

//Get Filesystem objects associated to this file
$c = new Criteria();
$c->add(DocmanFileVersionPeer::DOCMAN_FILE_ID, $file_id);
$c->addJoin(FilesystemPeer::REF_ID, DocmanFileVersionPeer::DOCMAN_FILE_VERSION_ID);
$c->add(FilesystemPeer::SECTION, DocmanFileVersion::SECTION);
$FilesystemObjects = FilesystemPeer::doSelect($c);

try {
	foreach($FilesystemObjects as $FilesystemObject) {
		$FilesystemObject->setFileName($file_name);
		$FilesystemObject->save();		
	}

	header('HTTP/1.1 200');
	$arr_feedback = array();
	$arr_feedback["error_message"] = "";
	echo FastJSON::encode($arr_feedback);

} catch (Exception $ex) {
	$errors=$ex->getErrors();
	$feedback = implode('<br />', $errors);
	
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_message"] = $feedback;
	echo FastJSON::encode($arr_feedback);
}

?>
