<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a DocmanFolder and saves it to the 
// database
//
require_once('plugins/docman/lib/DocmanFolder.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');
require_once('plugins/docman/lib/FastJSON.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_message"] = "Permission denied";
	echo FastJSON::encode($arr_feedback);
	exit;
}

$project = GFUrl::getUrl()->getProject();

//Get "home" folder
$c = new Criteria();
$c->add(DocmanFolderPeer::FOLDER_NAME, 'incoming');
$c->add(DocmanFolderPeer::PROJECT_ID, $project->getProjectId());
$home_folder = DocmanFolderPeer::doSelectOne($c);
if (is_object($home_folder)) {
	$home_folder_id = $home_folder->getDocmanFolderId();	
} else {
	$home_folder_id = 0;
}

$DocmanFolder = new DocmanFolder();
$DocmanFolder->setProject($project);
if ($_REQUEST['parent_folder_id'] > 0) {
	$ParentDocmanFolder = DocmanFolderPeer::retrieveByPk(intval($_REQUEST['parent_folder_id']));
	if (!is_object($ParentDocmanFolder) || !$ParentDocmanFolder->userCanCreate()) {
		header('HTTP/1.1 401');
		$arr_feedback = array();
		$arr_feedback["error_message"] = "Permission denied";
		
		echo FastJSON::encode($arr_feedback);
		exit;
	}
	if ($ParentDocmanFolder->getProject()->getProjectId() != $project->getProjectId()) {
		header('HTTP/1.1 401');
		$arr_feedback = array();
		$arr_feedback["error_message"] = "Permission denied";
		echo FastJSON::encode($arr_feedback);
		exit;
	}
	
	if ($home_folder_id == $_REQUEST['parent_folder_id']) {
		$DocmanFolder->setParentFolderId(0);
		$DocmanFolder->setTopParentFolderId(0);	
	} else {
		$DocmanFolder->setParentFolderId($_REQUEST['parent_folder_id']);
		$DocmanFolder->setTopParentFolderId($ParentDocmanFolder->getTopParentFolderId());
	}

}

$Lang = Language::getSessLang();

$is_public = ((isset($_POST) && isset($_POST['is_public'])) ? true : false);
try {
	$DocmanFolder->setFolderName( Validator::sanitizeString($_POST['folder_name']) );
	$DocmanFolder->setIsPublic( Validator::sanitizeBool($is_public) );
	$DocmanFolder->save();
	header('HTTP/1.1 200');
	$arr_feedback = array();
	$arr_feedback["folder_id"] = $DocmanFolder->getDocmanFolderId();
	$arr_feedback["parent_folder_id"] = $_POST['parent_folder_id'];
	$arr_feedback["folder_name"] = $DocmanFolder->getFolderName();
	echo FastJSON::encode($arr_feedback);
	exit;
} catch (InvalidDataException $ex) {
	$errors=$ex->getErrors();
	$feedback = implode('<br />', $errors);
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_code"] = 1;
	$arr_feedback["error_message"] = $feedback;
	echo FastJSON::encode($arr_feedback);
	exit;
} catch (PermissionDeniedException $ex) {
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_code"] = 2;
	$arr_feedback["error_message"] = "You don't have permission on this folder.";
	echo FastJSON::encode($arr_feedback);
	exit;
}

?>
