<?php

require_once('initialize.php');
require_once('MailParser.php');
require_once('session/Session.php');
require_once('core/User.php');
require_once('plugins/forum/lib/ForumThread.php');
require_once('LinkRenderer.php');



$session = Session::getSession();

$mail = new MailParser("php://stdin");

if ($mail->isAutoResponse()) {
	//Ignore auto-responses
	exit(0);
}

$from = $mail->getFrom();
$body = $mail->getBody();
$subject = $mail->getSubject();
$message_id = $mail->getMessageIDFromAddress();
$mail_message_id = $mail->getMessageId();


try {
	/** @var User[] $users  */
	$users = UserPeer::retrieveByEmail($from);



if (count($users) > 1) {
		throw new Exception(Language::getSessText('Gateway.TooManyUsers', array($from)));
	}

	if (!count($users)) {
		throw new Exception(Language::getSessText('Gateway.NoUser', array($from)));
	}

	$user = $users[0];

	$session->setUser($user);

	$message = ForumMessagePeer::retrieveByPk($message_id);

	if (is_null($message)) {
		throw new InvalidDataException(Language::getSessText('Forum.InvalidForumMessageID'));
	}

	$thread = $message->getForumThread();

	if (is_null($thread)) {
		throw new InvalidDataException(array(Language::getSessText('Forum.InvalidForumMessageID')));
	}

	if($thread->getForum()->getSection() != 'project' ) {
		throw new PermissionDeniedException('You don\'t have permissions to post in this discussion');
	}

	if (!$thread->getForum()->userCanPost($user)) {
		throw new PermissionDeniedException('You don\'t have permissions to post in this discussion');
	}


	$url = new GFUrl();
	$forum = $thread->getForum();
	$project = ProjectPeer::retrieveByPk($forum->getRefId());

	if (!$project || !$project->isActive() || !$project->usesPlugin('forum') ) {
		throw new PermissionDeniedException('You don\'t have permissions to post in this discussion');
	}

	$text = StringUtils::HTMLFromPlainString($mail->getBodyQuoteRemoved('>',$body));
	$text = LinkRenderer::renderHtmlLinks($text);



	/** @var ForumMessage $message  */
	/** @var ForumMessage $parentMessage  */
	$parentMessage = ForumMessagePeer::retrieveByPk(ForumMessagePeer::FORUM_MESSAGE_ID, $message_id);
	$message = new ForumMessage();
	$message->setForumThreadId($thread->getForumThreadId());
   $message->setParentForumMessageId($message_id);
	$message->setCreatedBy($user->getUserId());
	$message->setSubject('Re: '.$thread->getThreadName());
	$message->setBody($text);

	$message->save();

} catch (Exception $ex) {
	$subject = Language::getSessText('Gateway.ErrorSubject');
	$message = Language::getSessText('Gateway.ErrorResponse', array($ex->getMessage(), $body));

	SystemUtils::sendMail($subject, $message,$from);
}


?>
