<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
// The parent class
require_once 'plugins/forum/lib/om/BaseForumPeer.php';

// The object class
include_once 'plugins/forum/lib/Forum.php';

// Required by sendUserWallMentions method
require_once('core/UserPreferencePeer.php');
require_once('core/UserPeer.php');

// Required by getRelatedItem... functions
require_once('core/PluginPeer.php');
require_once('lib/LinkRenderer.php');

/**
 * The skeleton for this class was autogenerated by Propel on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package plugins.forum.lib
 */
class ForumPeer extends BaseForumPeer {

	const CHAT_EVENT = '_chat_event_';
	const CHAT_EVENT_CREATE = '_chat_create_';
	const CHAT_EVENT_USER_ENTER = '_chat_user_enter_';
	const CHAT_EVENT_USER_LEAVE = '_chat_user_leave_';
	const SETTING_NAME_PROJECT_CHAT = 'project-chat';

	private static $_pkCache = array();

	/**
	 * Retrieve a single object by pkey or NULL if not found.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key
	 * @param	bool $useCache	Whether to use caching. This will increase performance if the same object
	 * 							is accessed several times
	 * @param	Connection $con	The connection to use
	 * @return	Forum
	 */
	public static function retrieveByPk($pk, $useCache = true, $con = null) {
		if ($useCache && array_key_exists($pk, ForumPeer::$_pkCache)) {
			return ForumPeer::$_pkCache[$pk];
		}

		$object = parent::retrieveByPk($pk, $con);
		if ($useCache) {
			ForumPeer::$_pkCache[$pk] = $object;
		}
		return $object;
	}

	/**
	 * Removes an object from the cache. This is usually done after saving.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key of the object to remove
	 */
	public static function invalidate($pk) {
		if (array_key_exists($pk, ForumPeer::$_pkCache)) {
			unset(ForumPeer::$_pkCache[$pk]);
		}
	}

	/**
	 * Find the forums with the specified section name and ref_id
	 * @param	string	$section
	 * @param	int	$ref_id
	 * @return	Forum
	 */
	public static function retrieveForums($section, $ref_id) {
		$c = new Criteria();
		$c->add(self::SECTION, $section);
		$c->add(self::REF_ID, $ref_id);
		return self::doSelect($c);
	}

	public static function getOrCreateForum($section, $ref_id, $name = 'Discussion', $is_public = false) {

		$candidates = self::retrieveForums($section, $ref_id);
		$forum = null;
		foreach ($candidates as $candidate) {
			if (strcasecmp($candidate->getForumName(), $name) == 0) {
				$forum = $candidate;
				break;
			}
		}

		if ($forum == null) {
			//Check template project and  see if there is a Project Discussion forum already set
			/** @var Project $project  */
			if (self::userCanCreateDiscussion($section, $ref_id)) {
				if ($section == Project::SECTION) {
					$templateForum = self::getTemplateProjectDiscussion($ref_id, $name);
					if (!is_null($templateForum)) {
						$forum = self::copyNewForumFromTemplate($templateForum, $ref_id, $section);
						return $forum;
					}
				}

				$is_public = true;

				$forum = new Forum();
				$forum->setRefId($ref_id);
				$forum->setSection($section);
				$forum->setIsPublic($is_public);
				$forum->setForumName($name);
				$forum->save();

				self::assignProjectDiscussionRoles($forum);
			}
		}

		return $forum;
	}

	/**
	 * @param Forum $forum
	 */
	private static function assignProjectDiscussionRoles($forum) {
		if ($forum->getSection() == Project::SECTION) {
			$project = ProjectPeer::retrieveByPk($forum->getRefId());

			if (!$project) {
				return;
			}

			if (!$project->getIsPublic()) {
				//Only assign role settings if the project is private
				/** @var Role[] $roles  */
				$roles = $project->getRoles();

				foreach ($roles as $role) {
					$roleSetting = new RoleSetting();
					$roleSetting->setRole($role);
					$roleSetting->setRefId($forum->getForumId());
					$roleSetting->setSection(Forum::SECTION);
					$roleSetting->setValue(Forum::POST);
					$roleSetting->save();
				}
			}
		}
	}

	/**
	 * Whether the current user can create a discussion or not
	 *
	 * @param int $section
	 * @param int $ref_id
	 * @return bool
	 */
	private static function userCanCreateDiscussion($section, $ref_id) {
		$allow_creation = false;
		switch ($section) {
			case Project::SECTION :
				$project = ProjectPeer::retrieveByPk($ref_id);

				if (!$project || !is_object($project)) {
					//Invalid project
					return false;
				}

				$perm = $project->getPermission();
				$allow_creation = $project->userIsAdmin() || $perm->check(Forum::SECTION, 0, Forum::ADMIN);
				break;

			case TrackerItem::SECTION :
				$trackerItem = TrackerItemPeer::retrieveByPk($ref_id);
				$allow_creation = !is_null($trackerItem) && $trackerItem->userCanCreate();
				break;

			case WikiPage::SECTION :
				/** @var WikiPage $wikiPage  */
				$wikiPage = WikiPagePeer::retrieveByPk($ref_id);
				$allow_creation = is_object($wikiPage) && $wikiPage->userCanRead();
				break;

			case News::SECTION :
				$news = NewsPeer::retrieveByPk($ref_id);
				$allow_creation = !is_null($news) && $news->userCanCreate();
				break;

			case DocmanFolder::SECTION :
				$docmanFolder = DocmanFolderPeer::retrieveByPk($ref_id);
				$allow_creation = !is_null($docmanFolder) && $docmanFolder->userCanRead();
				break;

			case DocmanFile::SECTION :
				$docmanFile = DocmanFilePeer::retrieveByPk($ref_id);
				$allow_creation = !is_null($docmanFile) && $docmanFile->userCanRead();
				break;

			case FrsRelease::SECTION :
				$frsRelease = FrsReleasePeer::retrieveByPk($ref_id);
				$allow_creation = !is_null($frsRelease) && $frsRelease->userCanRead();
				break;
		}

		return $allow_creation;
	}

	/**
	 * @param Forum $templateForum
	 * @param int $ref_id
	 * @param string $section
	 * @return Forum
	 */
	private static function copyNewForumFromTemplate($templateForum, $ref_id, $section) {
		/** @var Forum $forum  */
		$forum = $templateForum->copy();
		$forum->setRefId($ref_id);
		$forum->setSection($section);
		$forum->setThreadCount(0);
		$forum->setMessageCount(0);
		$forum->save();

		self::copyTemplateForumRoleSettings($ref_id, $templateForum, $forum);

		return $forum;
	}

	private static function getTemplateProjectDiscussion($project_id, $name) {
		$c = new Criteria();
		$c->add(ForumPeer::SECTION, 'project');
		$c->add(ForumPeer::REF_ID, $project_id);
		$c->add(ForumPeer::FORUM_NAME, $name);
		return ForumPeer::doSelectOne($c);
	}

	/**
	 * Tries to copy permissions from the template forum for matching roles in child project
	 * @param int $project_id
	 * @param Forum $templateForum
	 * @param Forum $destForum
	 */
	private static function copyTemplateForumRoleSettings($project_id, $templateForum, $destForum) {
		//Role Settings
		//Get current project roles
		$c = new Criteria();
		$c->add(RolePeer::PROJECT_ID, $project_id);
		/** @var Role[] $roles */
		$roles = RolePeer::doSelect($c);

		//Get template Role settings
		$c = new Criteria();
		$c->add(RoleSettingPeer::SECTION, 'forum');
		$c->add(RoleSettingPeer::REF_ID, $templateForum->getId());
		$templateRoleSettings = RoleSettingPeer::doSelectJoinRole($c);

		/** @var RoleSetting[] $roleSettingsTmp */
		foreach ($roles as $role) {
			//Search role in template role setting
			foreach ($templateRoleSettings as $templateRole) {
				if ($role->getName() == $templateRole->getRole()->getRoleName()) {
					/** @var RoleSetting $roleSetting */
					$roleSetting = $templateRole->getRoleSetting()->copy();
					$roleSetting->setRoleId($role->getId());
					$roleSetting->setRefId($destForum->getId());
					$roleSetting->save();
					break;
				}
			}
		}
	}

	public static function doSelectWithLatestMessage(Criteria $c) {
		$ret = array();
		$forums = self::doSelect($c);
		foreach ($forums as $forum) {
			$item = array();
			$item['forum'] = $forum;

			// Now get the latest message from this forum
			$c2 = new Criteria();
			$c2->add(ForumThreadPeer::FORUM_ID, $forum->getForumId());
			$c2->addJoin(ForumMessagePeer::FORUM_THREAD_ID, ForumThreadPeer::FORUM_THREAD_ID);
			$c2->addDescendingOrderByColumn(ForumMessagePeer::POST_DATE);
			$c2->setOffset(0);
			$c2->setLimit(1);
			$res = ForumMessagePeer::doSelectJoinUser($c2);
			if (count($res) == 0) {
				$item['latest_message'] = null;
			} else {
				$item['latest_message'] = $res[0];
			}

			$ret[] = $item;
		}

		return $ret;
	}

	/**
	 * Get the latest threads from the forums the user is monitoring
	 */
	public static function getLatestMonitoredThreads(User $user, $section, $limit = 5) {
		// Get the forums in $section the user is monitoring
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, 'forum');
		$c->add(MonitorPeer::USER_ID, $user->getUserId());
		$c->add(self::SECTION, $section);
		$c->addJoin(MonitorPeer::REF_ID, self::FORUM_ID);
		$c->addJoin(ProjectPeer::PROJECT_ID, self::REF_ID);
		$c->addAscendingOrderByColumn(ProjectPeer::PROJECT_NAME);
		$c->addDescendingOrderByColumn(ForumPeer::MOST_RECENT_DATE);

		// Fakes a "doSelectJoinProject"
		self::addSelectColumns($c);
		ProjectPeer::addSelectColumns($c);
		$rs = self::doSelectRS($c);

		$result = array();
		$last_project = 0;
		$i = -1;
		while ($rs->next()) {
			$startcol = (self::$numColumns - self::$numLazyLoadColumns) + 1;
			$forum = new Forum();
			$forum->hydrate($rs);
			$project = new Project();
			$project->hydrate($rs, $startcol);

			if ($last_project != $project->getProjectId()) {
				$last_project = $project->getProjectId();
				$i++;
				$result[$i]['project'] = $project;
				$result[$i]['threads'] = array();
			}

			// Get the latest messages from this forum
			$latest_threads = $forum->getTopThreads($limit);
			if (count($latest_threads) == 0) {
				continue;
			}

			$result[$i]['threads'][] = array('forum' => $forum, 'latest_threads' => $latest_threads);
		}

		return $result;
	}

	/**
	 * Creates display-ready strings for short and full title of
	 * the GForge object to which the specific forum/thread is bound.
	 * @param mixed $forumOrThread A Forum or Thread object.
	 * @return array The title and full_title keys contain the constructed names.
	 */
	public static function getRelatedItemTitle($forumOrThread) {

		if ($forumOrThread instanceof Forum) {
			$forum = $forumOrThread;
			$thread = null;
		} else {
			$forum = $forumOrThread->getForum();
			$thread = $forumOrThread;
		}

		$item = null;
		try {
			if ($forum != null) {
				$title_array = PluginPeer::getRelatedItemTitle($forum->getSection(), $forum->getRefId());

				$maxlen = 25;
				if (strlen($title_array['title']) > $maxlen) {
					$title_array['title'] = StringUtils::truncateString(trim($title_array['title']), $maxlen);
				}

				return $title_array;
			}
		} catch (Exception $ex) {
			error_log($ex->getMessage());
			error_log($ex->getTraceAsString());
		}
		return null;
	}

	public static function relatedItemExists($forumOrThread) {
		if ($forumOrThread instanceof Forum) {
			$forum = $forumOrThread;
			$thread = null;
		} else {
			$forum = $forumOrThread->getForum();
			$thread = $forumOrThread;
		}

		return PluginPeer::relatedItemExistsFor($forum->getSection(), $forum->getRefId());
	}

	public static function getRelatedProject(Forum $forum) {
		return PluginPeer::getRelatedProject($forum->getSection(), $forum->getRefId());
	}

	public static function sendUserWallMentions($to_users, $from_user_id, $message, $chat_room_key = '', $bol_render_links=true) {
		if ($bol_render_links) {
			$message = LinkRenderer::renderHtmlLinks($message);
		}

		// error_log("sendUserWallMentions enter");
		$ret = 0;
		foreach ($to_users as $unixname) {
			//error_log("sendUserWallMentions for $unixname");
			if (is_numeric($unixname)) {
				// It's actually a user ID from DB
				$to_user = UserPeer::retrieveByPk($unixname);

				if ($to_user == null) {
					continue;  // No such user?, move on
				}

				$unixname = $to_user->getUnixName();

				// It's possible that the user ID and unixname could have been added to the array separately.
				// Recheck and skip the ID if this is the case
				if (in_array($unixname, $to_users))
					continue;
			} else {
				$to_user = UserPeer::retrieveByUnixName($unixname);
			}

			if ($to_user == null) {
				//error_log("sendUserWallMentions: $unixname, does not exist, skipping...");
				continue;  // No such user? typo or email address, move on
			}
			$to_user_id = $to_user->getUserId();

			try {
				if (strlen($chat_room_key) > 0) {
					// Only post to user's wall if they're not currently in chat
					if (UserPreferencePeer::getUserPreferenceValue($to_user_id, $chat_room_key, 'N') == 'Y') {
						//error_log("sendUserWallMentions: $unixname is in $chat_room_key, skipping...");
						continue;
					}
				}
				//error_log("sendUserWallMentions: calling AddUserWallPost($from_user_id, $to_user_id)");
				WallPostPeer::AddUserWallPost($from_user_id, $to_user_id, $message);
				// error_log("sendUserWallMentions: called AddUserWallPost($from_user_id, $to_user_id) OK");
				$ret++;
			} catch (Exception $ex) {
				// error_log("Exception in sendUserWallMentions: {$ex->getMessage()}");
			}
		}
		// error_log("sendUserWallMentions done, returning $ret");

		return $ret;
	}

	/**
	 * Persist the event to the DB (create an audit trail object)
	 */
	static function auditChatEvent($thread, $event_type) {
		$audit = new AuditTrail();
		$audit->setSection(Forum::SECTION);
		$audit->setRefId($thread->getForumThreadId());
		$audit->setFieldName(ForumPeer::CHAT_EVENT);
		$audit->setNewValue($event_type);
		$audit->setOldValue('');
		$audit->setUser(Session::getSession()->getUser());
		$audit->save();
	}

	/**
	 * Persist the event to the DB
	 */
	public static function logChatEvent($thread = null, $event_type = '', $user = null) {
		$session = Session::getSession();
		$Lang = $session->getLanguage();

		if (!is_object($thread) || $event_type == '') {
			return false;
		}

		//if no specific user was passed to log the event, the session user is assumed
		if (!is_object($user)) {
			$user = $session->getUser();
		}

		$str_subject = '';
		$str_message = '';
		switch ($event_type) {
			case ForumPeer::CHAT_EVENT_CREATE:
				$str_subject = $Lang->getText('Forum.ChatEventCreateSubject');
				$str_message = $Lang->getText('Forum.ChatEventCreateMessage');
				ForumPeer::auditChatEvent($thread, ForumPeer::CHAT_EVENT_CREATE);
				ActionManager::notify("CHAT_CREATE_AFTER", $thread);
				break;

			case ForumPeer::CHAT_EVENT_USER_ENTER:
				$str_subject = $Lang->getText('Forum.ChatEventUserAddSubject');
				$str_message = $Lang->getText('Forum.ChatEventUserAddMessage');
				self::DoForumActivity($thread, $user, $str_message, 'join');
				break;

			case ForumPeer::CHAT_EVENT_USER_LEAVE:
				$str_subject = $Lang->getText('Forum.ChatEventUserLeaveSubject');
				$str_message = $Lang->getText('Forum.ChatEventUserLeaveMessage');
				self::DoForumActivity($thread, $user, $str_message, 'leave');
				break;
		}
	}

	private static function DoForumActivity($thread, $user, $message, $verb) {
		$project = ForumPeer::getRelatedProject($thread->getForum());
		$project_id = 0;
		if ($project) {
			$project_id = $project->getProjectId();
		}

		$event = array(
			'project_id' => $project_id,
			'section' => $thread->getForum()->getSection(),
			'ref_id' => $thread->getForum()->getRefId(),
			'user_id' => $user->getUserId(),
			'activity_date' => time(),
			'description' => $message,
			'url' => null,
			'activity_type' => $verb,
			'extended_data' => PropelJson::ShallowJson($thread)
		);

		return ActionManager::notify('ACTIVITY_LOG_EVENT', $event);
	}

	public static function getForumCount($project_id) {
		$total = 0;
		$c = new Criteria();
		$c->add(ForumPeer::SECTION,'project');
		$c->add(ForumPeer::REF_ID,$project_id);
		$c->addSelectColumn(ForumPeer::COUNT);
		$rs = ForumPeer::doSelectRS($c);  
  		if ($rs->next()) {
   			return $rs->getInt(1);
		}
		return 0;
	}

}
