<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version $Id$
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('lib/plugin/GFProjectPlugin.php');
require_once('plugin/ActionManager.php');
require_once('SystemUtils.php');
require_once('ForumPeer.php');
require_once('Forum.php');
require_once('ForumThreadPeer.php');
require_once('lib/html/GFWidget.php');
require_once('ForumMessagePeer.php');
require_once('lib/core/ProjectPluginPeer.php');
require_once('lib/core/PluginPeer.php');
require_once('lib/core/PropelJson.php');
require_once('lib/GFExportImportUtils.php');
require_once('plugins/forum/lib/html/ForumFormRenderer.php');
require_once('lib/core/SettingPeer.php');

class ForumPlugin implements GFProjectPlugin {

//	protected $project;
	CONST NOACCESS	= Forum::NOACCESS;
	CONST READ = Forum::READ;
	CONST POST = Forum::POST;
	CONST ADMIN = Forum::ADMIN;
	CONST NOANON = Forum::NOANON;
	CONST ANON = Forum::ANON;
	CONST ISPRIVATE = Forum::ISPRIVATE;
	CONST ISPUBLIC = Forum::ISPUBLIC;
	CONST SECTION = Forum::SECTION;

	function __construct() {
	}

	/**
	 *	this is called when the first Action is called. Action manager 
	 *	gets the listeners for each plugin.
	 */
	function initialize() {
		ActionManager::registerListeners($this, $this->getListeners());
	}

	private function _buildMainTabsListener($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$project = $params['project'];
		if (!$project->usesPlugin('forum')) return null;

		$result = new PluginResult($this);
		$value = array(
				'text'	=> Language::getSessText('Forum.TabString'),
				'action'	=> GFUrl::getUrl()->projectURL($project,'','forum'),
				'id'	=> 'forum',
				'item_count' => ForumPeer::getForumCount($project->getId())
			);
		$result->setResult($value);
		return $result;
	}

	/**
	 *      getForumNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 * /
	private function getForumNavElements() {
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'Forum Title',
			'elements'=> array(
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 1'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 2'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 3')
			)
		);
		$result->setResult($arr);
		return $result;
	}

	/**
	 *      getProjectAdminNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	private function getProjectAdminNavElements($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'Forum',
			'elements'=> array(
			array('link'=>'/gf/project/'.$params['project']->getUnixName().'/forum/admin/',
				'description'=>Language::getSessText('Forum.Admin'))
			)
		);
		$result->setResult($arr);
		return $result;
	}

	/**
	 * returns the part of the html text that goes into the project public area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function _buildProjectSummary($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);

		$project = $params['project'];
		$theme = $params['theme'];
		
		//$value = $theme->infoTableTop(Language::getSessText('Project.Forums'));
	
		
		$value = Language::getSessText('Forum.TabString');
		
		$c = new Criteria();
		$c->add(ForumPeer::SECTION,Project::SECTION);
		$c->add(ForumPeer::REF_ID,$project->getPrimaryKey());
		$public_forums = ForumPeer::doSelect($c);
		foreach ($public_forums as $public_forum) {
			if ($public_forum->userCanRead()) {
				$value .= '<p><a href="'.GFUrl::getUrl()->projectUrl($project,'ForumBrowse','forum','',array('forum_id'=>$public_forum->getForumId())).'"><img src="/images/forum_icon.gif" height="16" width="16" alt="" border="0" />&nbsp;'.$public_forum->getForumName().'</a> ('.$public_forum->getMessageCount().' '.Language::getSessText('Forum.MessageCountName').')</p>';
			}
		}
		//$value .= $theme->infoTableBottom();
		
		$result->setResult($value);
		return $result;		
		
	}

	/**
	 *	Different types of plugins are possible -
	 *	GFPlugin::SITE_PLUGIN = 'site'
	 *	GFPlugin::PROJECT_PLUGIN = 'project'
	 *	GFPlugin::USER_PLUGIN = 'user'
	 */
	function getType() {
		return GFPlugin::PROJECT_PLUGIN;
	}

	/**
	 *	get the name of this plugin. example: 'tracker'
	 *	@return	string
	 */
	function getName() {
		return 'forum';
	}

	/**
	 *	get the array of action names that this plugin answers to.
	 *	@return	array
	 */
	function getListeners() {
		return array(
			'BUILD_PROJECT_TABS',
			'BUILD_PROJECT_SUMMARY',
			'GET_PROJECT_ADMIN_NAV_ELEMENTS',
			'BUILD_SEARCH_COMBO',
			'DO_SEARCH',
			'SEARCH_BY_ID_LIST',
			'SEARCH_BY_ID_ACTION',
			'PROJECT_APPROVE',
			'BUILD_MYPAGE_ELEMENTS',
			'BUILD_MYPAGE_MONITORS',
			'MONITOR_CHECK_PERMISSIONS',
			'MONITOR_GET_TEMPLATE',
			'CHECK_DOWNLOAD_PERMS',
			'MAIL_GATEWAY',
			'BUILD_MYPAGE_TABS',
			'FORUM_GET_LATEST_MONITORED_THREADS',
			'GET_ROLE_CHOICES',
			'UPDATE_ROLE_CHOICES',
			'GET_OBSERVER_ROLE_CHOICES',
			'UPDATE_OBSERVER_SETTINGS',
			'BUILD_RSS_FEED',
			'GET_DISK_USAGE',
			'PROJECT_FULLEXPORT',
			'PROJECT_FULLIMPORT',
			'PROJECT_TPLEXPORT',
			'PROJECT_TPLIMPORT',
			'SCM_COMMIT',
			'RENDER_ASSOCIATION_LINK',
			'USER_PROJECT_DELETE',
			'USERGROUP_USER_REMOVE',
			'ASSOCIATION_SECTION',
			'GET_PROJECT_SETTINGS_FORM',
			'DO_TAG_SEARCH',
			'HANDLE_FILE_UPLOAD',
			'PROJECT_EVENT_PULL',
			'ACTIVITY_LOG_MIGRATE_ROW',
			'FORUM_SAVE',
			'FORUM_DELETE',
			'FORUM_THREAD_SAVE',
			'FORUM_THREAD_VIEW',
			'FORUM_THREAD_DELETE',
			'FORUM_MESSAGE_DELETE'
	//		'ASSOCIATION_CHECK'
	//		'GET_FORUM_NAV_ELEMENTS'
			);
	}

	/**
	 *	handle action has a case statement to redirect the action to the appropriate function.
	 *	@param string
	 *	@param mixed

	 *	@return PluginResult
	 */
	function handleAction($listenerName,$params) {
		switch ($listenerName) {
			case 'BUILD_PROJECT_TABS': {
				return $this->_buildMainTabsListener($params);
				break;
			}
			case 'BUILD_PROJECT_SUMMARY': {
				return $this->_buildProjectSummary($params);
				break;
			}
			case 'GET_PROJECT_ADMIN_NAV_ELEMENTS': {
				return $this->getProjectAdminNavElements($params);
				break;
			}
			case 'BUILD_SEARCH_COMBO': {
				return $this->buildSearchOptions($params);
				break;
			}
			case 'DO_SEARCH': {
				return $this->getSearchResults($params);
				break;
			}
			case 'PROJECT_APPROVE': {
				return $this->approveActions($params);
				break;
			}
			case 'BUILD_MYPAGE_ELEMENTS': {
				return $this->buildMyPageElements($params);
				break;
			}
			case 'BUILD_MYPAGE_MONITORS': {
				return $this->buildMyPageMonitors($params);
				break;
			}
			case 'MONITOR_CHECK_PERMISSIONS': {
				return $this->monitorCheckPermissions($params);
				break;
			}
			case 'MONITOR_GET_TEMPLATE': {
				return $this->monitorGetTemplate($params);
				break;
			}
			case 'CHECK_DOWNLOAD_PERMS': {
				return $this->checkDownloadPerms($params);
				break;
			}
			case 'MAIL_GATEWAY': {
				return $this->processMailGateway($params);
				break;
			}
			case 'BUILD_MYPAGE_TABS': {
				return $this->buildMypageTabs($params);
				break;
			}
			case 'FORUM_GET_LATEST_MONITORED_THREADS': {
				return $this->getLatestMonitoredThreads($params);
				break;
			}
			case 'GET_ROLE_CHOICES': {
				return $this->getRoleChoices($params);
				break;
			}
			case 'UPDATE_ROLE_CHOICES': {
				return $this->updateRoleChoices($params);
				break;
			}
			case 'GET_OBSERVER_ROLE_CHOICES': {
				return $this->getObserverRoleChoices($params);
				break;
			}
			case 'UPDATE_OBSERVER_SETTINGS': {
				return $this->updateObserverSettings($params);
				break;
			}
			case 'BUILD_RSS_FEED': {
				return $this->buildRssFeed($params);
				break;
			}
			case 'RENDER_ASSOCIATION_LINK': {
				return $this->renderAssociationLink($params);
				break;
			}
			case 'ASSOCIATION_SECTION': {
				return $this->getAssociationSection($params);
				break;
			}
			case 'ASSOCIATION_CHECK': {
				return $this->getAssociationCheck($params);
				break;
			}
			case 'SEARCH_BY_ID_LIST': {
				return $this->searchByIDList();
				break;
			}
			case 'SEARCH_BY_ID_ACTION': {
				return $this->searchByIDAction($params);
				break;
			}
			case 'GET_DISK_USAGE': {
				return $this->getDiskUsage($params);
				break;
			}
			case 'PROJECT_FULLIMPORT':
			case 'PROJECT_FULLEXPORT': {
				return GFExportImportUtils::getSpecData($this);
				break;
			}
			case 'PROJECT_TPLIMPORT':
			case 'PROJECT_TPLEXPORT': {
				return GFExportImportUtils::getSpecData($this, 'specDataTpl.xml');
				break;
			}
			case 'SCM_COMMIT': {
				return $this->scmCommit($params);
				break;
			}
			case 'USER_PROJECT_DELETE': {
				return $this->deleteUserMonitors($params);
				break;
			}
			case 'USERGROUP_USER_REMOVE': {
				return $this->deleteUsergroupUserMonitors($params);
				break;
			}
			case 'GET_PROJECT_SETTINGS_FORM': {
				return $this->getProjectSettingsForm($params);
				break;
			}
			case 'DO_TAG_SEARCH': {
				return $this->getSearchByTagsResult($params);
				break;
			}
			case 'HANDLE_FILE_UPLOAD': {
				return $this->handleFileUpload($params);
				break;
			}
			case 'PROJECT_EVENT_PULL': {
				return $this->getProjectEvent($params);
				break;
			}
			case 'ACTIVITY_LOG_MIGRATE_ROW': {
				return $this->migrateActivityRow($params);
				break;
			}
			case 'FORUM_SAVE_BEFORE': {
				return $this->onForumSave($params);
				break;
			}
			case 'FORUM_DELETE' : {
				return $this->onForumDelete($params);
				break;
			}
			case 'FORUM_THREAD_SAVE': {
				return $this->onForumThreadSave($params);
				break;
			}
			case 'FORUM_THREAD_VIEW': {
				return $this->onForumThreadView($params);
				break;
			}
			case 'FORUM_THREAD_DELETE': {
				return $this->onForumThreadDelete($params);
				break;
			}
			case 'FORUM_MESSAGE_DELETE': {
				return $this->onForumMessageDelete($params);
				break;
			}
		}
	}

	private function onForumSave($forum) {
		$user = Session::getSession()->getUser();
		$project = ForumPeer::getRelatedProject($forum);
		$verb = $forum->isNew() ? 'create' : 'update';
		$url = GFUrl::getUrl()->projectURL($project, 'ForumBrowse', self::SECTION, '', array('forum_id' => $forum->getPrimaryKey()));
		return $this->doForumActivity($forum, $user, $project, $verb, $url);
	}

	private function onForumDelete($forum) {
		$user = Session::getSession()->getUser();
		$project = ForumPeer::getRelatedProject($forum);
		$url = GFUrl::getUrl()->projectURL($project, '', self::SECTION);
		return $this->doForumActivity($forum, $user, $project, 'delete', $url);
	}

	private function doForumActivity($forum, $user, $project, $verb, $url) {
		$user_id = 100;
		$user_name = 'Anonymous User';
		if ($user != null) {
			$user_id = $user->getUserId();
			$user_name =  $user->getUnixName();
		}

		$project_id = 0;
		if ($project != null) {
			$project_id = $project->getPrimaryKey();
		}

		$event = array(
			'project_id' => $project_id,
			'section' => Forum::SECTION,
			'ref_id' => $forum->getPrimaryKey(),
			'user_id' => $user_id,
			'activity_date' => time(),
			'description' => "{$user_name} {$verb}d Discussion {$forum->getForumName()}",
			'url' => $url,
			'activity_type' => $verb,
			'extended_data' => PropelJson::ShallowJson($forum)
		);

		$result = ActionManager::notify('ACTIVITY_LOG_EVENT', $event);

		$ret = new PluginResult($this);
		$ret->setResult(true);

		return $ret;
	}

	private function onForumThreadSave($forumthread) {
		$user = Session::getSession()->getUser();
		//returns project_id
		$project = ForumPeer::getRelatedProject($forumthread->getForum());
		$verb = $forumthread->isNew() ? 'create' : 'update';
		$args = array(
			'thread_id' => $forumthread->getPrimaryKey(),
			'forum_id' => $forumthread->getForum()->getPrimaryKey(),
			'_forum_action' => 'ForumMessageBrowse'
		);
		$url = GFUrl::getUrl()->projectURL($project, 'ForumBrowse', self::SECTION, '', $args);
		return $this->doForumThreadActivity($forumthread, $user, $project, $verb, $url);
	}

	private function onForumThreadView($forumthread) {
		$user = Session::getSession()->getUser();
		$project = ForumPeer::getRelatedProject($forumthread->getForum());
		$verb = 'view';
		$args = array(
			'thread_id' => $forumthread->getPrimaryKey(),
			'forum_id' => $forumthread->getForum()->getPrimaryKey(),
			'_forum_action' => 'ForumMessageBrowse'
		);
		$url = GFUrl::getUrl()->projectURL($project, 'ForumBrowse', self::SECTION, '', $args);
		return $this->doForumThreadActivity($forumthread, $user, $project, $verb, $url);
	}

	private function onForumThreadDelete($forumthread) {
		$user = Session::getSession()->getUser();
		$project = ForumPeer::getRelatedProject($forumthread->getForum());
		$url = GFUrl::getUrl()->projectURL($project, 'ForumBrowse', self::SECTION, '', array('forum_id' => $forumthread->getForum()->getPrimaryKey()));
		return $this->doForumThreadActivity($forumthread, $user, $project, 'delete', $url);
	}

	private function onForumMessageDelete($msg) {
		$at = new AuditTrail();
		// Change date defaults to right now
		$user = Session::getSession()->getUser();
		$at->setUser($user);
		$at->setFieldName('_deleted_');
		$at->setSection(ForumMessage::SECTION);
		$at->setRefId($msg->getPrimaryKey());
		// Old value and new value can be null
		$at->save();

		$ret = new PluginResult($this);
		$ret->setResult(true);

		return $ret;
	}

	private function doForumThreadActivity($forumthread, $user, $project, $verb, $url) {

		$user_id = 100;
		$user_name = 'Anonymous User';
		if ($user != null) {
			$user_id = $user->getUserId();
			$user_name =  $user->getUnixName();
		}

		$project_id = 0;
		if ($project instanceof Project) {// $project is object
			$project_id = $project->getPrimaryKey();
		} else if (is_numeric($project)) {// project is project ID
			$project_id = $project;
		}

		$event = array(
			'project_id' => $project_id,
			'section' => ForumThread::SECTION,
			'ref_id' => $forumthread->getPrimaryKey(),
			'user_id' => $user_id,
			'activity_date' => time(),
			'description' => "{$user_name} {$verb}d Discussion Thread {$forumthread->getThreadName()} in {$forumthread->getForum()->getForumName()}",
			'url' => $url,
			'activity_type' => $verb,
			'extended_data' => PropelJson::ShallowJson($forumthread)
		);

		$result = ActionManager::notify('ACTIVITY_LOG_EVENT', $event);

		$ret = new PluginResult($this);
		$ret->setResult(true);

		return $ret;
	}

	private function deleteUsergroupUserMonitors($usergroup_user) {
		$con = Propel::getConnection();		

		$project_ids = $usergroup_user->getProjectMonitorsToDelete();
		$user_id = $usergroup_user->getMemberUserId();

		if ($project_ids != '') {
			$stmt = $con->createStatement();
			//Delete monitor for the projects
			$sql = "DELETE FROM monitor WHERE 
							section='".ForumPlugin::SECTION."' 
							AND ref_id IN 
								(
									SELECT forum_id FROM forum, project WHERE
									forum.ref_id = project.project_id
									AND
									forum.section = 'project'
									AND
									NOT (project.is_public = true AND forum.is_public = true)
									AND
									ref_id IN (".$project_ids.")
								)
							AND user_id = ".$user_id;
			$stmt->executeQuery($sql);
		}		
	}


	private function deleteUserMonitors($params) {
		$user_id = $params['user']->getUserId();
		$project = $params['project'];

		if (is_object($project)) {
			if (!$project->isPublic()) {
				$con = Propel::getConnection();
				$stmt = $con->createStatement();
		
				$sql = "DELETE FROM monitor WHERE 
								section='".ForumPlugin::SECTION."' 
								AND ref_id IN (SELECT forum_id as ref_id FROM forum WHERE ref_id = ".$project->getProjectId().")
								AND user_id = ".$user_id;
		
				$stmt->executeQuery($sql);
			}
		}
	}

	private function getAssociationCheck($params) {
		if ($params->getSection() == ForumMessage::SECTION) {
			$fm=ForumMessagePeer::retrieveByPk($params->getRefId());
			if (!is_object($fm) || !$fm->userCanUpdate()) {
				throw new InvalidDataException(array('The forum message doesn\'t exist or you don\'t have permissions for it'));
			}
		}
		if ($params->getToSection() == ForumMessage::SECTION) {
			$fm=ForumMessagePeer::retrieveByPk($params->getToRefId());
			if (!is_object($fm) || !$fm->userCanUpdate()) {
				throw new InvalidDataException(array('The forum message doesn\'t exist or you don\'t have permissions for it'));
			}
		}
	}

	private function getAssociationSection($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$result->setResult(array(ForumMessage::SECTION,'Forum Message'));
		return $result;
	}

	private function renderAssociationLink($params) {
		$ret=array();
		foreach ($params as $param) {
			if ($param[0]==ForumMessage::SECTION) {
				$ti=ForumMessagePeer::retrieveByPk($param[1]);
				if (is_object($ti)) {
					try {
						$str_url = $ti->getForumMessageLink(true);
					} catch(Exception $e) {
						$str_url = Language::getSessText('ForumMesage.UnsupportedLink');
					}
					$ret[]=array(
						$ti->getPrimaryKey(),
						'Forum message: '.$str_url,
						$param[2],
						$param[3]
					);
				}
			} else {
				$arr = explode("|", $param[2]);
				if (isset($arr[2]) && $arr[2] == ForumMessage::SECTION && $arr[0] == WallPost::SECTION) {
					$res = Association::renderNonPluginAssociationLink($param);
					if (is_array($res)) {
						$ret[] = $res;
					}
				}
			}
		}
		$result = new PluginResult($this);
		$result->setResult($ret);
		return $result;
	}
	/**
	 *      return resultset of an array to populate the ID search select box
	 *      @return PluginResult
	 */
	function searchByIDList() {
		$pr=new PluginResult($this);
		$pr->setResult(array(ForumMessage::SECTION => 'Forum Message'));
		return $pr;
	}
	/**
	 *      return result set of URL to redirect to
	 *      @param $params array
	 *      @return PluginResult
	 */
	function searchByIDAction($params) {
		if (isset($params['searchtype']) && $params['searchtype'] == ForumMessage::SECTION) {
			$pr=new PluginResult($this);
			$ti=ForumMessagePeer::retrieveByPk($params['searchid']);
			if (!is_object($ti) || $ti->getForumThread()->getForum()->getSection() != Project::SECTION) {

			} else {
				$pr->setResult($ti->getForumMessageLink());
			}
			return $pr;
		}
	}
	/**
	 * returns the part of the html text that goes into the my page area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageElements($params) {

	}
	
	/**
	 * returns the part of the html text that goes into the my page area, for monitored items
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageMonitors($params) {	
		$con = Propel::getConnection(ForumPeer::DATABASE_NAME);
		$theme = $params['theme'];

		//get the assigned project forums for the user
		$user = Session::getSession()->getUser();
		$user_id = $user->getPrimaryKey();
		$sql = "select p.project_id,p.project_name,f.forum_id, f.forum_name from forum f, monitor m, project p
		where m.user_id = '$user_id' 
		and f.section='project' 
		and m.section='forum' 
		and m.ref_id = f.forum_id 
		and p.project_id = f.ref_id
		ORDER BY project_name,forum_name";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

		$value = '<h3>'.Language::getSessText('Forum.Monitoring').'</h3>';

		$theader = array();
		$theader[] = 'Project';
		$theader[] = Language::getSessText('My.ProjectForums');
		
		$last_project = null;
		
		if ($rs->getRecordCount()>0) {
			$theader[] = '';
			$value .= $theme->tableTop($theader);
			$trow2 = array();
			$ttype=array();
			while($rs->next()) {
				$ttype[$rs->getInt('project_id')][] = '1';
			}			
			$rs->beforeFirst();
			while($rs->next()) {
				$trow = array();
				$tmod = array();
				$forum_name = $rs->getString('forum_name');
				$project_name = $rs->getString('project_name');
				if ($last_project != $rs->getInt('project_id')) {
					$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'ForumBrowse',$this->getName(),'',array('forum_id'=>$rs->getInt('forum_id'))) . '" >' .  $project_name . '</a>';
					$tmod[]='rowspan="'.count($ttype[$rs->getInt('project_id')]).'" bgcolor="#FFFFFF" width="30%"';
				}
				$last_project = $rs->getString('project_id');
				$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'ForumBrowse',$this->getName(),'',array('forum_id'=>$rs->getInt('forum_id'))) . '" >' .  $forum_name . '</a>';
				$tmod[] = '';
				$trow[] = '<a href="'.GFUrl::getUrl()->getMonitorUrl(Forum::SECTION, $rs->getInt('forum_id'), null, false).'" >'.Language::getSessText('Forum.StopMonitoring').'</a>'; 
				$tmod[] = 'align="right"';
				$value .= $theme->tableRow($trow, $tmod);
			}
		} else {
			$value .= $theme->tableTop($theader);
			$trow = array();
			$tmod[]='colspan="2"';
			$trow[] = Language::getSessText('My.NoMonitoredProjectForums');
			$value .= $theme->tableRow($trow,$tmod);
		}
		$value .= $theme->tableBottom();
		
		$result = new PluginResult($this);
		$result->setResult($value);
		return $result;		
	}

	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return PluginResult
	 */	
	private function getSearchResults($params) {
		if (array_search($this->getName(), $params['search']) !== false) {
			$result = new PluginResult($this);
			$searcher = new GFSearch($params);
			$result->setResult($this->search($searcher));
			return $result;			
		}
	}
	
	private function getSearchByTagsResult($params) {
		if (array_search($this->getName(), $params['search']) !== false) {
			$result = new PluginResult($this);
			$searcher = new GFSearch($params);
			$result->setResult($this->searchByTags($searcher));
			return $result;			
		}
	}

	
	private function buildSearchOptions($params) {
		$result = new PluginResult($this);
		$value = array();

		if (isset($params['project'])) {
			if (!$params['project']->usesPlugin($this->getName())) {
				return;
			}
			$text = Language::getSessText('Forum.ThisProjectForums');
		} else {
			$text = Language::getSessText('Forum.Forums');
		}
		
		// only give result if we're in a project
		$value[] = array(
			'value'	=> $this->getName(),
			'display'	=> $text,
			'id'	=> 'forum',
			);
		$result->setResult($value);
		return $result;		
	}	
	
	/**
	 *	get project uri to be used as root for constructing uris
	 *	@return string
	 */
	function getProjectPluginUri() {
		return GFUrl::getUrl()->currentBaseUrl(); //projectURL($this->project,'','forum');
	}

	/**
	 *      return an sql statement for calculating disk usage
	 */
	function getDiskUsage($params) {
		$sql="select q.project_id,sum(q.usage) as usage FROM (
		--messages
		SELECT f.ref_id AS project_id,sum(length(fm.body)*2) AS usage 
			FROM forum f JOIN forum_thread USING (forum_id) JOIN forum_message fm USING (forum_thread_id) 
			WHERE f.section='project' GROUP BY project_id
		UNION
		--attachments
		SELECT f.ref_id AS project_id,sum(fs.file_size+length(fsi.vectors)) AS usage
			FROM forum f JOIN forum_thread USING (forum_id) JOIN forum_message fm USING (forum_thread_id),
			filesystem fs JOIN filesystem_idx fsi USING (filesystem_id)
			WHERE fm.forum_message_id=fs.ref_id 
			AND f.section='project'
			AND fs.section='forummessage'
			GROUP BY project_id
		) q
		GROUP BY project_id";
		$result = new PluginResult($this);
		$result->setResult($sql);
		return $result;
	}

	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return PluginResult
	 */
	function search(GFSearch $search) {
		if (!$search->getProjectId()) {
			return $this->globalSearch($search);
		}

		$project = $search->params['project'];

		if (GFConf::get('system.dbtype') == 'postgres') {
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();
			if ($search->getStartDate()) {
				$date_sql = " AND fm.post_date BETWEEN '".$search->getStartDate()."' AND '".$search->getEndDate()."' ";
			} else {
				$date_sql='';
			}
			if ($search->getSearchAttach()) {
			$attach_sql="UNION
			SELECT fm.forum_message_id, rank(filesystem_idx.vectors, q) AS rank
			FROM filesystem_idx, to_tsquery('".$words."') q, filesystem fs, forum_message fm, forum_thread ft, forum f
			WHERE filesystem_idx.vectors @@ q
			AND filesystem_idx.filesystem_id=fs.filesystem_id
			AND fs.section='forummessage'
			AND fs.ref_id=fm.forum_message_id
			AND fm.forum_thread_id=ft.forum_thread_id
			$date_sql
			AND ft.forum_id=f.forum_id
			AND f.ref_id='".$search->getProjectId()."'
			AND f.section='project' ";
			} else {
				$attach_sql='';
			}

			$sql = "SELECT forum.forum_id, forum.forum_name, forum_message.subject, forum_message.forum_message_id, 
			forum_message.created_by, forum_message.post_date, forum_thread.thread_name, forum_thread.forum_thread_id,
			forum_thread.message_count,
			sum(s.rank) AS rank 
			FROM forum_message, forum_thread, forum, (
				SELECT fm.forum_message_id, rank(forum_message_idx.vectors, q) AS rank
				FROM forum, forum_thread, forum_message fm, forum_message_idx, to_tsquery('".$words."') as q 
				WHERE 
				vectors @@ q 
				AND forum_message_idx.forum_message_id=fm.forum_message_id
				AND fm.forum_thread_id=forum_thread.forum_thread_id
				$date_sql
				AND forum_thread.forum_id=forum.forum_id
				AND forum.section = 'project' 
				AND forum.ref_id = '" . $search->getProjectId() ."' 
				$attach_sql
			) s
			WHERE s.forum_message_id=forum_message.forum_message_id
			AND forum_message.forum_thread_id=forum_thread.forum_thread_id
			AND forum_thread.forum_id=forum.forum_id
			GROUP BY forum.forum_id, forum.forum_name, forum_message.subject, forum_message.forum_message_id,
			forum_message.created_by, forum_message.post_date, forum_thread.thread_name, forum_thread.forum_thread_id,
			forum_thread.message_count
			ORDER BY rank DESC";

			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			$template = $search->getTemplate();

			$count = 0;
			$content = $template->tableTop(array(
				Language::getSessText('Forum.ThreadMessage'),
				Language::getSessText('Forum.Author'),
				Language::getSessText('ForumMessage.PostDateName')
			));
			while ($rs->next()) {
				$ftid = $rs->getInt('forum_thread_id');
				$trow = array();
				$messagesPerPage = ForumFormRenderer::THREAD_PAGE_SIZE;
				assert($messagesPerPage !=0);
				$page = $this->getForumMessagePage($rs->getInt('forum_message_id'), $rs->getInt('forum_thread_id'));
				$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project,'ForumBrowse',$this->getName(),'',
					array('forum_id'=>$rs->getInt('forum_id'),'thread_id'=>$ftid,'_forum_action'=>'ForumMessageBrowse',
						'_forum_thread_total'=>$messagesPerPage, '_forum_thread_page'=>$page, 'forum_thread_style'=>'flat')) .
					'#fm_'.$rs->getInt('forum_message_id').'">[' . $rs->getString('thread_name') . '] ' .
					$rs->getString('subject').'</a>';
				$trow[] = UserPeer::retrieveByPK($rs->getInt('created_by'))->getName();
				$trow[] = $rs->getDate('post_date');
				$content .= $template->tableRow($trow);
				$count++;
			}
			$content .= $template->tableBottom();

			$title = Language::getSessText('Forum.SearchResultsTabTitle', array($count));

			if ($count == 0) {
				$content = Language::getSessText('Search.NoResults');
			}
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}

		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content
			);
	}

	function getForumMessagePage($forum_message_id, $thread_id) {

		$messagesPerPage = ForumFormRenderer::THREAD_PAGE_SIZE;

		$con = Propel::getConnection(ForumPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$sql = "SELECT row_number FROM (SELECT row_number, thread_messages.*
			FROM (SELECT * FROM forum_message WHERE forum_thread_id=$thread_id)
			AS thread_messages CROSS JOIN (
				SELECT ARRAY(SELECT forum_message_id FROM forum_message WHERE forum_thread_id=$thread_id ORDER BY post_date ASC) as id) AS oldids
				CROSS JOIN
				generate_series(1, (SELECT COUNT(*) FROM forum_message WHERE forum_thread_id=$thread_id)) AS row_number WHERE oldids.id[row_number]=thread_messages.forum_message_id ORDER BY row_number
			)
			f WHERE forum_message_id=$forum_message_id";
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

		$row_number = 0;
		if($rs->next()) {
			$row_number = $rs->getInt('row_number');
		}

		return ceil($row_number / $messagesPerPage);

	}
	
	/**
	 *	Searches across all projects
	 *	@param	GFSearch
	 *	@return string
	 */
	function globalSearch(GFSearch $search) {
		if (GFConf::get('system.dbtype') == 'postgres') {
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();
			if ($search->getStartDate()) {
				$date_sql = " AND fm.post_date BETWEEN '".$search->getStartDate()."' AND '".$search->getEndDate()."' ";
			} else {
				$date_sql='';
			}
			$project_sql = '';
			$project_sql_attachments = '';
			$project_ids = $search->getProjectIds();
			if(!is_null($project_ids)) {
				$project_sql = ' AND forum.ref_id IN ('.implode(',', $project_ids).')';
				$project_sql_attachments = ' AND f.ref_id IN ('.implode(',', $project_ids).') ';
			}
			if ($search->getSearchAttach()) {
				$attach_sql="UNION
				SELECT fm.forum_message_id, rank(filesystem_idx.vectors, q) AS rank,
					headline(fm.subject, q) AS headline_subject, headline(ft.thread_name, q) AS headline_thread_name
				FROM filesystem_idx, to_tsquery('".$words."') q, filesystem fs, forum_message fm, forum_thread ft, forum f
				WHERE filesystem_idx.vectors @@ q
				AND filesystem_idx.filesystem_id=fs.filesystem_id
				AND fs.section='forummessage'
				AND fs.ref_id=fm.forum_message_id
				AND fm.forum_thread_id=ft.forum_thread_id
				$date_sql
				$project_sql_attachments
				AND ft.forum_id=f.forum_id
				AND f.section='project' ";
			} else {
				$attach_sql='';
			}

			$sql = "SELECT 
					project.unix_name, 						-- 1
					project.project_name, 					-- 2
					s.headline_subject,						-- 3
					forum_message.forum_message_id, 		-- 4
					forum_message.created_by, 				-- 5
					forum_message.post_date, 				-- 6
					s.headline_thread_name,					-- 7
					forum_thread.forum_thread_id,			-- 8
					sum(s.rank) AS rank, 					-- 9
					forum.*									-- 10
			FROM forum_message, forum_thread, forum, project, (
				SELECT fm.forum_message_id, rank(forum_message_idx.vectors, q) AS rank,
					headline(fm.subject, q) AS headline_subject, headline(forum_thread.thread_name, q) AS headline_thread_name
				FROM forum, forum_thread, forum_message fm, forum_message_idx, to_tsquery('".$words."') as q 
				WHERE 
				vectors @@ q 
				AND forum_message_idx.forum_message_id=fm.forum_message_id
				AND fm.forum_thread_id=forum_thread.forum_thread_id
				$date_sql
				$project_sql
				AND forum_thread.forum_id=forum.forum_id
				AND forum.section = 'project' 
				$attach_sql
			) s
			WHERE s.forum_message_id=forum_message.forum_message_id
			AND forum_message.forum_thread_id=forum_thread.forum_thread_id
			AND forum_thread.forum_id=forum.forum_id
			AND forum.ref_id=project.project_id
			GROUP BY project.unix_name, project.project_name, forum_message.subject, forum_message.forum_message_id,
			forum_message.created_by, forum_message.post_date, forum_thread.thread_name, forum_thread.forum_thread_id,
			forum.forum_id, forum.section, forum.ref_id, forum.forum_name, forum.is_public, forum.description, forum.send_all_posts_to,
			forum.thread_count, forum.message_count, forum.most_recent_date, forum.moderation_level, forum.show_associations_tab, forum.show_tags_tab,
			s.headline_subject, s.headline_thread_name
			ORDER BY rank DESC";

			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			$template = $search->getTemplate();

			$count = 0;
			$content = $template->tableTop(array(
				Language::getSessText('Project.ListNameTitle'),
				Language::getSessText('Forum.ThreadMessage'),
				Language::getSessText('Forum.Author'),
				Language::getSessText('ForumMessage.PostDateName')
			));
			$messagesPerPage = ForumFormRenderer::THREAD_PAGE_SIZE;
			assert($messagesPerPage !=0);
			while ($rs->next()) {
				$forum = new Forum();
				$forum->hydrate($rs, 10);
				if (!$forum->userCanRead()) {
					continue;
				}

				$count++;
				$project_unix_name = $rs->getString(1);
				$ftid = $rs->getInt(8);
				$fmid = $rs->getInt(4);
				$page = $this->getForumMessagePage($fmid, $ftid);

				$trow = array();
				$trow[] = '<a href="'.GFUrl::getUrl()->projectUrl($project_unix_name).'">'.$rs->getString(2).'</a>';
				$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project_unix_name,'ForumBrowse',$this->getName(),'',
					array('forum_id'=>$rs->getInt(10),'thread_id'=>$ftid,'_forum_action'=>'ForumMessageBrowse',
						'_forum_thread_total'=>$messagesPerPage, '_forum_thread_page'=>$page, 'forum_thread_style'=>'flat')
				) . '#fm_'.$fmid.'">[' . $rs->getString(7) . '] ' . $rs->getString(3).'</a>';
				$trow[] = UserPeer::retrieveByPK($rs->getInt(5))->getName();		// Created by
				$trow[] = $rs->getDate(6);		// Post dates
				$content .= $template->tableRow($trow);
			}

			if ($count == 0) {
				$content .= $template->tableFullRow(Language::getSessText('Search.NoResults'));
			}
			$content .= $template->tableBottom();
			
			$title = Language::getSessText('Forum.SearchResultsTabTitle', array($count));

		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}

		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content
			);
	}


	public function searchByTags(GFSearch $search){

		$words = $search->getWordsForQuery();
		$project_id = $search->getProjectId();
		$project_ids = $search->getProjectIds();
		$template = $search->getTemplate();

		//Clean key words from tsearch operators
		$words = preg_replace('/,+/',',', $words);
		$tsearch_operators = array('(', ')', '!', '|','&');
		$replacements = array('','','','',',');
		$words = str_replace($tsearch_operators, $replacements, $words);
		$words_arr = explode(',', $words);

		$wc = count($words_arr);
		if($wc == 0){
			return '';
		}
		for($i = 0; $i < $wc; $i++){
			$words_arr[$i] = "'" . $words_arr[$i] . "'";
		}
		$words = implode(',', $words_arr);

		if($project_id){
			$forum_prj = " AND forum.ref_id = $project_id ";
			$tracker_prj = " AND tracker.project_id=$project_id";
		} else if (!is_null($project_ids)) {
			$forum_prj = ' AND forum.ref_id IN ('.implode(',', $project_ids).') ';
		} else {
			$forum_prj = ' ';
		}


		$forum_perms_sql = "SELECT distinct forum_thread.forum_id AS forum_id 
		FROM forum_message INNER JOIN forum_thread 
		ON forum_message.forum_thread_id = forum_thread.forum_thread_id
		INNER JOIN forum ON forum_thread.forum_id = forum.forum_id
		WHERE forum_message.forum_message_id IN
		(SELECT tag_entry.ref_id FROM tag_entry WHERE section='forummessage'
		AND tag_name IN ($words)
		GROUP BY tag_entry.ref_id
		HAVING COUNT(tag_entry.ref_id) = $wc) $forum_prj";
		$con = Propel::getConnection();
		$stmt = $con->createStatement();

		$rs = $stmt->executeQuery($forum_perms_sql, ResultSet::FETCHMODE_NUM);
		$Lang = Session::getSession()->getLanguage();

		$columns = array(
			$Lang->getText('Forum.ThreadMessage'),
			$Lang->getText('Forum.Author'),
			$Lang->getText('ForumMessage.PostDateName')
		);

		if(!$project_id){
			array_unshift($columns,$Lang->getText('Project.ListNameTitle'));
		}

		$content = $template->tableTop($columns);

		$forum_aux = '';
		if($rs->getRecordCount() > 0) {
			while($rs->next()){
				$forums[] = $rs->getInt(1);
			}
			$frs = ForumPeer::retrieveByPks($forums);
			$forums = array();

			foreach ($frs as $fr){
				$project = ProjectPeer::retrieveByPk($fr->getRefId());
				if($project->usesPlugin('forum') && $fr->getShowTagsTab() && $fr->userCanRead()){
					$forums[] = $fr->getForumId();
				}
			}
			if(count($forums) == 0){
				//no forums
				$content .= $template->tableFullRow(Language::getSessText('Search.NoResults'));
				$content .= $template->tableBottom();
				return array(
					'title'=>Language::getSessText('Forum.SearchResultsTabTitle', array(0)),
					'count'=>0,
					'content'=> $content
				);

			} else {
				$forum_aux = ' AND forum_thread.forum_id IN (' . implode(',', $forums) . ')';
			}
		} else {
			$content .= $template->tableFullRow(Language::getSessText('Search.NoResults'));
			$content .= $template->tableBottom();
			return array(
					'title'=>Language::getSessText('Forum.SearchResultsTabTitle', array(0)),
					'count'=>0,
					'content'=>$content
			);
		}
		
		$forum_message_sql = "SELECT 'forummessage' AS section, forum_message.forum_message_id AS
		ref_id, forum_message.subject AS description, forum_message.created_by AS author,
		forum_message.post_date AS post_date, project.unix_name AS prj_unix_name,
		project.project_name AS prj_name, 
		forum_thread.forum_thread_id AS forum_thread_id,
		forum.forum_id AS forum_id,
		forum.forum_name AS forum_name
		FROM forum_message INNER JOIN forum_thread 
		ON forum_message.forum_thread_id = forum_thread.forum_thread_id
		INNER JOIN forum ON forum_thread.forum_id = forum.forum_id
		INNER JOIN project ON forum.ref_id = project.project_id
		WHERE forum_message.forum_message_id IN
		(SELECT tag_entry.ref_id FROM tag_entry WHERE section='forummessage'
		AND tag_name IN ($words)
		GROUP BY tag_entry.ref_id
		HAVING COUNT(tag_entry.ref_id) = $wc) $forum_aux $forum_prj";

		$stmt = $con->createStatement();

		$rs = $stmt->executeQuery($forum_message_sql, ResultSet::FETCHMODE_ASSOC);
		$count = 0;

		if($rs->getRecordCount() > 0){
			while($rs->next()){
				$trow=array();
				if(!$project_id){
					$trow[] = '<a href="' . GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name')) . '">'. $rs->getString('prj_name') . '</a>';
				}

				$trow[] = '<a href="'. GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name'), 'ForumBrowse','forum','', array('forum_id'=>$rs->getInt('forum_id'),'thread_id'=>$rs->getInt('forum_thread_id'),'_forum_action'=>'ForumMessageBrowse')) .'">' . $rs->getString('description') . '</a>';
				$trow[] = UserPeer::retrieveByPK($rs->getInt('author'))->getName();
				$trow[] = $rs->getDate('post_date');

				$content .= $template->tableRow($trow);
				$count++;
			}
		}


		$title = $Lang->getText('Forum.SearchResultsTabTitle', array($count));
		if ($count == 0) {
				$content .= $template->tableFullRow($Lang->getText('Search.NoResults'));
		}

		$content .= $template->tableBottom();

		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content
		);


	}


	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return resultset
	 */
	private function checkDownloadPerms($params) {
		$section = $params['section'];
		$ref_id = $params['ref_id'];
		$result = new PluginResult($this);
		$result->setResult(false); // false by default, looking for true
		// if it's for another section, exit
		if ($section!=ForumMessage::SECTION) {
			$result->setResult(false);
		} else {
			$msg = ForumMessagePeer::retrieveByPK($ref_id);
			if (is_object($msg)) {
				$forum = $msg->getForumThread()->getForum();
				if ($forum->userCanRead()) {
					$result->setResult(true);
				}
			}
		}
		return $result;
	}
	
	/**
	 * returns an array of the fields where we want the search to be done upon
	 *
	 */
	private function getSearchFields() {
		return array(ForumMessagePeer::SUBJECT,ForumMessagePeer::BODY);
	}	
	
	/**
	 * 	The addMember listener - this hook will fire
	 *	whenever a member is added to a project.
	 *	@param	User	User object
	 *	@return	boolean
	 */
	function addMemberListener($params) {
		return false; // XXX
	}

	/**
	 * 	The removeMember listener - this hook will fire
	 *	whenever a member is removed from a project.
	 *	@param	User	User object
	 *	@return	boolean
	 */
	function removeMemberListener($params) {
		return false; // XXX
	}

	/**
	 * 	The approveProject listener - this hook will fire
	 *	when a project is approved.
	 *	@param	Project	the project object
	 *	@return	boolean
	 */
	function approveProjectListener($params) {
		return false; // XXX
	}

	/**
	 * 	The deleteProject listener - this hook will fire
	 *	when a project is deleted.
	 *	@return	boolean
	 */
	function deleteProjectListener() {
		return false; // XXX
	}

	/**
	 * 	The checkPermission - this listener will fire
	 *	when permissions are being checked for this project.
	 *	@param	Array	An associative array containing 
	 *	'section_name' a name like 'tracker' or 'tracker_item'
	 *	'section_id'=>id (the id of the section, for example a specific tracker)
	 *	'subsection_id'=>id (the id of the subsection, for example an tracker_item id)
	 *	'perm_level'=>int (the value, such a 1 for read, 2 for write, that is being requested)
	 *	@return	boolean
	 * /
	function checkPermission($params);
*/

	function enableInProject(Project $project) {
		return;
	}

	function disableFromProject() {
		// unregister the actions
		ActionManager::unregisterListeners($this);
	}
	
	/**
	 * returns the possible role settings available for this plugin
	 *
	 * @return array	the roles
	 */
	function getAvailableRoleSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Forum.NoAccess'); //descriptions
		$return[1][] = ForumPlugin::NOACCESS; // values
		$return[0][] = Language::getSessText('Forum.Read');
		$return[1][] = ForumPlugin::READ;
		$return[0][] = Language::getSessText('Forum.Post');
		$return[1][] = ForumPlugin::POST;
		$return[0][] = Language::getSessText('Forum.Admin');
		$return[1][] = ForumPlugin::ADMIN;		
		
		return $return;
	}

	/**
	 * returns the possible site-wide admin role settings for this plugin
	 *
	 * @return array	the roles
	 */	
	function getAdminRoleSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Forum.None'); //descriptions
		$return[1][] = ForumPlugin::NOACCESS; // values
		$return[0][] = Language::getSessText('Forum.Admin');
		$return[1][] = ForumPlugin::ADMIN;
		return $return;
	}

	/**
	 * 	The getRoleChoices listener - this hook will fire
	 *	when a new role is added or one is going to be updated.
	 * 	It returns an array of role settings or null
	 *	@return	array
	 */
	function getRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$roles = array();
		$c = new Criteria();
		$c->add(ForumPeer::SECTION, 'project');
		$c->add(ForumPeer::REF_ID, $params['project']->getPrimaryKey());
		//add special case, project site-wide forum admin
		$roles[0]['ref_id'] = 0;
		$roles[0]['section'] = Language::getSessText('Forum.SiteWideAdmin');
		$roles[0]['description'] = '<span id="forumAdminDescription">'. Language::getSessText('Forum.SiteWideAdmin.Description').'</span>';
		$roles[0]['subsection'] = '-';
		$roles[0]['settings'][0] = 'forum'; // the section
		$roles[0]['settings'][] = $this->getAdminRoleSettings();
		//retrieve the forums for the project
		$forums = ForumPeer::doSelect($c);
		foreach ($forums as $forum) {
			$key = $forum->getPrimaryKey();
			$roles[$key]['ref_id'] = $key;
			$roles[$key]['section'] = 'Forum'; // the displayed text in "section" field
			$roles[$key]['description'] = Language::getSessText('Forum.ForumWideAdmin.Description');
			$roles[$key]['subsection'] = $forum->getForumName(); // the displayed text in the "subsection" field
			$roles[$key]['settings'][0] = $this->getName(); // the section for the DB, one for all the settings
			$roles[$key]['settings'][] = $this->getAvailableRoleSettings();
		}
		
		$result = new PluginResult($this);
		$result->setResult($roles);
		return $result;
	}

	/**
	 * returns the settings for the forum messages posting
	 *
	 * @return array
	 */
	function getPostingObserverSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Forum.NoAnon'); //descriptions
		$return[1][] = ForumPlugin::NOANON; // values
		$return[0][] = Language::getSessText('Forum.Anon');
		$return[1][] = ForumPlugin::ANON;
		return $return;		
	}

	/**
	 * returns the settings for the forum status
	 *
	 * @return array
	 */
	function getStatusObserverSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Forum.Private'); //descriptions
		$return[1][] = ForumPlugin::ISPRIVATE; // values
		$return[0][] = Language::getSessText('Forum.Public');
		$return[1][] = ForumPlugin::ISPUBLIC;
		return $return;		
	}	
	
	/**
	 * 	The getObserverRoleChoices listener - this hook will fire
	 *	when the observer role is modified.
	 * 	It returns an array of role settings or null
	 *	@return	array or null
	 */
	function getObserverRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		
		$choices = array();
		$c = new Criteria();
		$c->add(ForumPeer::SECTION,'project');
		$c->add(ForumPeer::REF_ID,$params['project']->getPrimaryKey());
		//retrieve the forums for the project
		$forums = ForumPeer::doSelect($c);
		$i = 0;
		foreach ($forums as $forum) {
			$key = $forum->getPrimaryKey();
			$choices[$i]['ref_id'] = $key;
			$choices[$i]['section'] = Language::getSessText('Forum.Privileges');
			$choices[$i]['description'] = Language::getSessText('Forum.ForumWideAdmin.Description');
			$choices[$i]['subsection'] = $forum->getForumName();
			$choices[$i]['settings'][0] = 'forumpublic';
			$choices[$i]['settings'][] = $this->getStatusObserverSettings();
			$i++;
		}
		
		$result = new PluginResult($this);
		$result->setResult($choices);
		return $result;
	}
	
	/**
	 * Retrieves the concerning data and updates its observer role settings
	 *
	 * @param array	$data
	 */
	function updateObserverSettings($params) {
		$project = $params['project'];
		$data = $params['data'];
		
		$onedata = current($data);
		while ($onedata != null) {
			$section = key($data);
			switch ($section) {
				case 'forumpublic' : {
					$oneitem = current($onedata);
					while ($oneitem != null) {
						$ref_id = key($onedata);
						$value = $oneitem;
						//retrieve the forum and update it
						$c = new Criteria();
						$c->add(ForumPeer::FORUM_ID,$ref_id);
						try {
							$forum = ForumPeer::doSelectOne($c);
						} catch (Exception $e) {
							// something wrong happened
							echo $e->getMessage();
							next($onedata);
							$oneitem = current($onedata);
							continue;	
						}
						$forum->setIsPublic($value);
						$forum->save();
						next($onedata);
						$oneitem = current($onedata);					
					}		
					break;
				}
				default : { 
					//skip it
					continue;
				}
			}
			next($data);
			$onedata = current($data);
		}
	}
	
	/**
	 * Retrieve the corresponding setting (value) for the ref_id passed as argument, using the appropriate section
	 *
	 * @param int $ref_id
	 * @param string $section
	 * @return setting
	 */
	function getObserverSetting($ref_id,$section) {
		switch ($section) {
			case 'forumpublic' : {
				$c = new Criteria();
				$c->add(ForumPeer::FORUM_ID,$ref_id);
				try {
					$forum = ForumPeer::doSelectOne($c);
				} catch (Exception $e) {
					echo $e->getMessage();
					return '';
				}
				return $forum->getIsPublic();
				break;
			}
			default : {
				return '';
			}
		}
	}
	
	/**
	 * actions to perform on project approval
	 *
	 * @param $params array
	 * @return PluginResult
	 */
	private function approveActions($params) {
		$template_project = $params['template_project'];
		$project = $params['project'];
		$con = Propel::getConnection(UserPeer::DATABASE_NAME);
		if ( (is_object($project)) && (is_object($template_project))) {
			// get the forums for the template project
			$c = new Criteria();
			$c->add(ForumPeer::SECTION,'project');
			$c->add(ForumPeer::REF_ID,$template_project->getProjectId());
			$template_forums = ForumPeer::doSelect($c);
			foreach ($template_forums as $template_forum) {
				// create new ones for the new project
				$new_forum = $template_forum->copy();
				$new_forum->setRefId($project->getProjectId());
				$new_forum->setThreadCount(0);
				$new_forum->setMessageCount(0);
				$new_forum->save();
				// search the role_settings and let's see if for this item we have one set
				foreach ($params['role_settings'] as $role_setting) {
					if (strstr($role_setting['section'],$this->getName()) && $role_setting['ref_id']==$template_forum->getPrimaryKey()) {
						$role_id = $role_setting['role_id'];
						$section = $role_setting['section'];
						$ref_id = $new_forum->getPrimaryKey();
						$value = $role_setting['value'];
						RoleSettingPeer::setRoleSetting($role_id, $section, $ref_id, $value);
					}
				}

			}

			if ($project->getIsTemplate() == FALSE) {
				$section = Project::SECTION;
				$refid = $project->getProjectId();

				$forum = ForumPeer::getOrCreateForum($section, $refid, $project->getProjectName());
				$thread = ForumThreadPeer::getNamedThread($forum, Session::getSession()->getUser());

				try {
					SettingPeer::SaveSetting(Project::SECTION, $project->getProjectId(), ForumPeer::SETTING_NAME_PROJECT_CHAT, $thread->getPrimaryKey(), $project->getProjectId());
				} catch (Exception $ex) {
					error_log("Could not save project chat thread setting in project approve: {$ex->getMessage()}");
				}

				// [#10162] Make sure that the project chat forum has role settings
				// During project approval, these aren't written to the database yet,
				// so we need to manually create them from the provided param list
				$section = Forum::SECTION;
				$refid = $forum->getPrimaryKey();
				$value = self::POST;
				$role_ids = array();
				foreach ($params['role_settings'] as $role_setting) {
					$current_value = RoleSettingPeer::getRoleSetting($role_id, $section, $ref_id);
					if ($current_value < $value) {
						RoleSettingPeer::setRoleSetting($role_id, $section, $ref_id, $value);
					}
				}
			}
		}
	}

	/**
	 * Check if the user can monitor certain item
	 */	
	private function monitorCheckPermissions($params) {
		$lang = Session::getSession()->getLanguage();

		$section = $params['section'];
		$ref_id = $params['ref_id'];
		
		if ($section != 'forum') return;		// only deal with forum monitor
		$pr = new PluginResult($this);
		
		$forum = ForumPeer::retrieveByPk($ref_id);
		if (is_null($forum) || !$forum->userCanRead()) {
			$pr->setResult(false);
		} else {
			$pr->setResult(true);
		}

		return $pr;
	}

	private function monitorGetTemplate($params){
		$section = $params['section'];
		$ref_id = $params['ref_id'];
		$project = $params['project'];
		$Lang = $params['lang'];

		if($section != self::SECTION){
			return;
		}

		include_once('plugins/forum/lib/html/ForumTemplate.php');
		$result['template'] = new ForumTemplate();

		$result['params'] = array(
			'project'=>$project,
			'forum'=>ForumPeer::retrieveByPk($ref_id),
			'title'=>$Lang->getText('Monitor.UsersMonitoringPlugin'),
		);

		$pr = new PluginResult($this);
		$pr->setResult($result);

		return $pr;
	}

	
	/**
	 * Process an incoming email
	 */
	private function processMailGateway($params) {
		if ($params['section'] != ForumMessage::SECTION) return;
		
		$message_id = $params['ref_id'];
		$user = $params['from_user'];
		$subject = $params['subject'];
		$body = $params['body'];
		
		$text = StringUtils::getTextBetweenMarkers($body, ForumMessageNotification::FORUM_MAIL_MARKER, ForumMessageNotification::FORUM_MAIL_MARKER);
		// Last line in $message may be "> " because the user hit "reply". Remove that.
		$text = preg_replace('/> ?$/m', '', $text);
		
		if (empty($text)) return;		// No message to post
		
		$orig_message = ForumMessagePeer::retrieveByPk($message_id);
		if (is_null($orig_message)) {
			throw new Exception(Language::getSessText('ForumMessage.InvalidId'), array($message_id));
		}
		
		$forum_name = $orig_message->getForumThread()->getForum()->getForumName();
		// Probably subject starts with "[forum name] " or "Re: [forum name] "
		$subject = str_replace('['.$forum_name.'] ', '', $subject);
		
		$body = Validator::sanitizeString($text);
		// Emulates newlines as if they were entered with fckeditor
		$body = nl2br($body);
		
		$new_message = new ForumMessage();
		$new_message->setForumThreadId($orig_message->getForumThreadId());
		$new_message->setCreatedBy($user->getUserId());
		$new_message->setSubject(Validator::sanitizeString($subject));
		$new_message->setBody($body);
		$new_message->setParentForumMessageId($orig_message->getForumMessageId());
		$new_message->save();
		
		return;		
	}
	
	private function buildMyPageTabs($params) {
		$lang = Session::getSession()->getLanguage();

		$result = array(
				'text' => $lang->getText('Forum.MyLatestMessagesTab'),
				'id' => 'latest_forum_messages',
				'action' => GFUrl::getUrl()->baseUrl('LatestForumMessages', '', 'my')
			);
		
		$pr = new PluginResult($this);
		$pr->setResult($result);
		
		return $pr;
	}

	private function getLatestMonitoredThreads($params) {
		$lang = Session::getSession()->getLanguage();
		$user = Session::getSession()->getUser();
		$theme = Session::getSession()->getTheme();
		$url = GFUrl::getUrl();
		
		$output = '';
		
		$latest_messages_data = ForumPeer::getLatestMonitoredThreads($user, 'project');
	
		$theader = array();
		$theader[] = $lang->getText('Project.Project');
		$theader[] = $lang->getText('Forum.Forum');
		$theader[] = $lang->getText('Forum.LatestThreads');
		$theader[] = $lang->getText('ForumThread.MostRecentDateName');
		$output .= $theme->tableTop($theader);

		if (count($latest_messages_data) > 0) {

			foreach ($latest_messages_data as $data) {
				$project = $data['project'];
				$rowcount = 0;
				
				foreach ($data['threads'] as $latest_threads_data) {
					$rowcount += count($latest_threads_data['latest_threads']);
				}
				
				$trow = array();
				$tmod = array();
				
				$trow[] = '<a href="'.$url->projectUrl($project).'">'.$project->getProjectName().'</a>';
				$tmod[] = 'rowspan="'.$rowcount.'" bgcolor="#FFFFFF"';
				
				foreach ($data['threads'] as $latest_threads_data) {
					$forum = $latest_threads_data['forum'];
					$latest_threads = $latest_threads_data['latest_threads'];
					
					$trow[] = '<a href="'.$url->projectUrl($project, 'ForumBrowse', 'forum', '', array('forum_id' => $forum->getForumId())).'">'.$forum->getForumName().'</a>';
					$tmod[] = 'rowspan="'.count($latest_threads).'" bgcolor="#FFFFFF"';
					
					foreach ($latest_threads as $thread_data) {
						$thread = $thread_data['thread'];
						$message = $thread_data['latest_message'];
						$user = $thread_data['user'];
						
						$trow[] = '<a href="'.$url->projectUrl($project, 'ForumBrowse', '', 'forum', array('_forum_action' => 'ForumMessageBrowse', 'thread_id' => $thread->getForumThreadId())).'">'.$thread->getThreadName().'</a>';
						$tmod[] = '';
						
						if ($user->isUndefinedUser()) {
							// undefined user?
							$user_desc = $message->getMailSender();
						} else {
							$user_desc = '<a href="'.$url->userUrl($user).'">'.$user->getFullName().'</a>';	// retrieves the created_by field
						}					
						
						$trow[] = $lang->getText('Forum.LatestPostDescription', array($thread->getMostRecentDate(), $user_desc));
						$tmod[] = '';
						
						$output .= $theme->tableRow($trow, $tmod);
						$trow = array();
						$tmod = array();
					}
				}
			}
		} else {
			$output .= $theme->tableFullRow(Language::getSessText('My.NoMonitoredProjectForums'));
		}
		$output .= $theme->tableBottom();

		$result = new PluginResult($this);
		$result->setResult($output);
		return $result;
	}

	private function updateRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		
		$section = self::SECTION;

		$project = $params['project'];
		$data = $params['data'];
		$role = $params['role'];
		
		if (!isset($data[$section])) {
			return;
		}
		
		foreach ($data[$section] as $ref_id => $value) {
			$c = new Criteria();
			$c->add(RoleSettingPeer::ROLE_ID, $role->getPrimaryKey());
			$c->add(RoleSettingPeer::SECTION, $section);
			$c->add(RoleSettingPeer::REF_ID, $ref_id);
			$rolesetting = RoleSettingPeer::doSelectOne($c);
			if (!is_object($rolesetting)) {
				// Enter here if we have a role setting which we didn't have previously (e.g.: activate an inactive plugin)
				$rolesetting = new RoleSetting();
				$rolesetting->setSection($section);
				$rolesetting->setRefId($ref_id);
				$rolesetting->setRoleId($role->getPrimaryKey());
			}
			
			// Being a project admin overrides any other value
			if (isset($data['projectadmin']) && $data['projectadmin'][0] >= Project::PROJECT_ROLE_ADMIN) {
				$value = self::ADMIN;
			}			
			
			$rolesetting->setValue($value);
			$rolesetting->save();						
		}	
		
		return;
	}

	private function buildRssFeed($params) {
		$section = $params['section'];
		$forum = ForumPeer::retrieveByPk($params['ref_id']);
		$project = $params['project'];
		$url = GFUrl::getURL();

		$result = new PluginResult($this);

		if (!is_object($forum) || !$forum->isPublic()) {
			$result->setResult(false);
			return $result;
		}

		$c = new Criteria();
		$c->addJoin(ForumMessagePeer::FORUM_THREAD_ID, ForumThreadPeer::FORUM_THREAD_ID);
		$c->add(ForumThreadPeer::FORUM_ID, $forum->getForumId());
		$c->addDescendingOrderByColumn(ForumMessagePeer::POST_DATE);
		$c->setLimit(20);
		$items = ForumMessagePeer::doSelectJoinForumThread($c);

		$channelName = 'Gforge Advanced Server - '.$project->getName().' - '.$forum->getForumName();

		$rssarr = array();
		$rssarr['channel']['name'] = $channelName;
		$rssarr['channel']['url'] = $url->getFullURL($url->baseURL());
		$rssarr['channel']['description'] = 'Gforge Advanced Server RSS';

		$i = 0;
		foreach ($items as $item) {
			$rssarr['items'][$i]['channel_name'] = $channelName;
			$rssarr['items'][$i]['title'] = $item->getSubject();
			$rssarr['items'][$i]['link'] = '<![CDATA['.$url->getFullURL($url->projectURL($project, 'ForumBrowse', $section, '', array('forum_id' => $forum->getForumId(), '_forum_action' => 'ForumMessageBrowse', 'thread_id' => $item->getForumThreadId()))).']]>';
			$rssarr['items'][$i]['author'] = $item->getUser()->getName().' &lt;'.$item->getUser()->getEmail().'&gt;';
			$rssarr['items'][$i]['guid'] = '<![CDATA['.$url->getFullURL($url->projectURL($project, 'ForumBrowse', $section, '', array('forum_id' => $forum->getForumId(), '_forum_action' => 'ForumMessageBrowse', 'thread_id' => $item->getForumThreadId()))).']]>';
			$rssarr['items'][$i]['description'] = StringUtils::stripHtmlTags($item->getBody());

			$i++;
		}

		if (array_key_exists('channel', $rssarr) && array_key_exists('items', $rssarr)) {
			$result->setResult($rssarr);
		} else {
			$result->setResult(false);
		}
		return $result;
	}
	
	/**
	 * Make the association (if necessary) between the SCM commit and the forum message
	 */
	private function scmCommit($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) return null;

		$scmcommit = $params['commit'];
		$log = $scmcommit->getMessageLog();
		
		// $params['associations'] has all the associations
		
		foreach ($params['associations'] as $association) {
			list($section, $ref_id) = $association;
			if ($section != ForumMessage::SECTION) {
				continue;
			}
			
			$forum_message_id = $ref_id;
	
			// see if the item belongs to the project
			$message = ForumMessagePeer::retrieveByPk($forum_message_id);
			if (is_null($message)) continue;
			
			$forum = $message->getForumThread()->getForum();
			if ($forum->getSection() != Project::SECTION || $forum->getRefId() != $project->getProjectId()) {
				// The referenced message is not part of this project
				continue;
			}
			
			// Message belongs to the project. Add entry to the association table.
			try {
				$association = new Association();
				$association->setSection(ScmCommit::SECTION);
				$association->setRefId($scmcommit->getPrimaryKey());
				$association->setToSection(ForumMessage::SECTION);
				$association->setToRefId($forum_message_id);
				$association->setComment($scmcommit->getMessageLog());
				$association->save();
			} catch (Exception $ex) {
				// This means the association is duplicated. Ignore the exception.
			}
		}
		
		return null;
	}

	function getProjectSettingsForm($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) {
			return;
		}
		
		$Lang = Session::getSession()->getLanguage();
		$theme = Session::getSession()->getTheme();

		$form = array();
		$form['title'] = Language::getSessText('Forum.ProjectSettingsTitle');
		$contents = '';
		
		$crit = new Criteria();
		$crit->add(ForumPeer::SECTION, 'project');
		$crit->add(ForumPeer::REF_ID, $project->getProjectId());
		$crit->addAscendingOrderByColumn(ForumPeer::FORUM_NAME);
		$arr = ForumPeer::doSelect($crit);
		
		$theader = array();
	
		$theader[] = $Lang->getText('Forum.ForumNameName');
		$theader[] = $Lang->getText('Forum.DescriptionName');
		$theader[] = $Lang->getText('Forum.ProjectSettingsSettings');

		$contents .= $theme->tableTop($theader);
		$url = GFUrl::getUrl();

		if (count($arr) < 1) {
			$contents .= $theme->tableFullRow('<span class="nonefound">'.$Lang->getText('Forum.NoneFound').'</span>');
		} else {
			foreach ($arr as $item) {
				$trow = array();
				$trow[]=$item->getForumName();
				$trow[]=$item->getDescription();				
				$trow[]= '<a href="'.$url->projectUrl($project, 'ForumEdit', $this->getName(), 'admin', array('id' => $item->getForumId(), 'redirect_to' => $url->currentUrl())).'">'.$Lang->getText('Forum.ProjectSettingsSettings').'</a>';
		
				$contents .= $theme->tableRow($trow, array(), false);
			}
		}
	
		$contents .= $theme->tableBottom();
		
		$form['contents'] = $contents;

		$result = new PluginResult($this);
		$result->setResult($form);
		return $result;
	}


	function handleFileUpload($params) {
		if ($params['section'] != ForumMessage::SECTION && $params['section'] != ForumThread::SECTION) {
			return;
		}
		if ($params['section'] == ForumMessage::SECTION) {
			$forum_message_id = $params['ref_id'];
			$forum_message = ForumMessagePeer::retrieveByPk($forum_message_id);
			if (!$forum_message) {
				return;
			}

			if (!$forum_message->userCanUpdate()) {
				throw new PermissionDeniedException();
			}

			$forum_message->attachFile($params);
		} else {
			$forum_thread_id = $params['ref_id'];
			$forum_thread = ForumThreadPeer::retrieveByPk($forum_thread_id);
			if (!$forum_thread) {
				return;
			}

			$user = Session::getSession()->getUser();

			$con = Propel::getConnection(ForumMessagePeer::DATABASE_NAME);
			$con->begin();

			$m = new ForumMessage(null, false);
			$m->setBody("n/a");
			$m->setCreatedBy($user->getUserId());
			$m->setForumThread($forum_thread);
			$m->setPostDate(time());
			$m->setSubject($user->getUnixName());
			$m->setPostDate(time());
			//We will create our own notification for file uploads here
			$m->save(null, false);

			$params['ref_id'] = $m->getForumMessageId();
			$filesystem = $m->attachFile($params);

			$url = GFUrl::downloadUrlFromFS($filesystem);
			$str_url_target = '';
			if ($filesystem->canOpenInBrowser()) {
				$str_url_target = ' target="_blank"';
			}
			
			$decoded_url = urldecode($url);
			$str_rendered_link = "";
			$str_rendered_link = LinkRenderer::renderGFASImageUrl($decoded_url);

			if ($str_rendered_link == "") {
				$str_rendered_link = '<img src="/images/docman_doc.gif" width="16" height="16" alt="" /><a href="'.$url.'"'.$str_url_target.'>'.$filesystem->getFileName().'</a>';	
			}

			$str_body = "<p>User " . $user->getUnixName() . " uploaded the file:";
			$str_body .=  '<br>'.$str_rendered_link.'</p>';

			$m->setBody($str_body);
			$m->save();
			$con->commit();
			$appendString = "\nDownload url: " . GFUrl::getUrl()->getFullURL($url) . "\n";
			$notification = new ForumMessageNotification($m, $appendString);
			NotificationManager::getManager()->queue($notification);

		}
	}


	function getProjectEvent($params) {
		if ($params['section'] != 'forumthread') {
			return;
		}

		if (isset($params['ungroup']) && $params['ungroup']) {

			$body = '<a href="'.$params['url'] . '">' . $params['description']."</a>";

		} else {
			$forumThread = ForumThreadPeer::retrieveByPk($params['ref_id']);

			if(!$forumThread) {
				return;
			}

			$project = ProjectPeer::retrieveByPk($params['project_id']);
			if ($forumThread->getThreadName() == 'Discussion') {
				$name = 'Project\'s';
			} else {
				$name = $forumThread->getThreadName();
			}
			$body = '<a href="'.
				GFUrl::getUrl()->projectURL($project, 'ForumBrowse', 'forum', '',
				array('_forum_action'=>'ForumMessageBrowse',  'thread_id'=>$params['ref_id'])) .
				'">' . Language::getSessText('ProjectActivity.DiscussionActivityDescription',
				array($params['posters_count'], $params['messages_count'], $name) );
		}

		$pr = new PluginResult($this);
		$pr->setResult($body);		// Mark the request as handled
		return $pr;
	}


	function migrateActivityRow($params) {
		if ($params['section'] != ForumPlugin::SECTION) {
			return;
		}

		$project = ProjectPeer::retrieveByPk($params['project_id']);
		$activity_desc = Language::getSessText('ProjectActivity.ForumPostDescription', array($params['description1']));
		$url = GFUrl::getURL()->projectURL($project, 'ForumBrowse', 'forum', '', array('_forum_action'=>'MessageReply','message_id'=>$params['subref_id']));

		$event = array(
			'project_id' => $params['project_id'],
			'section' => $params['section'],
			'ref_id' => $params['ref_id'],
			'user_id' => $params['user_id'],
			'activity_date' => $params['activity_date'],
			'description' => $activity_desc,
			'url' => $url,
			'activity_type' => $params['action'],
			'extended_data' => json_encode($params)
		);

		$result = ActionManager::notify('ACTIVITY_LOG_EVENT', $event);

		foreach ($result as $item) {
			if ($item->isError()) {
				return $item;
			}
		}

		$pr = new PluginResult($this);
		$pr->setResult($params['id']);		// Mark the request as handled
		return $pr;
	}

	private function getActivityDescription($params, $ti) {
		$action = $params[4];
		$refid = $params[2];
		$desc1 = $params[5];
		$desc2 = $params[6];
		$user = $params[9];

		switch ($action) {
			case 'create':
				return "$user opened #{$refid} in {$ti->getTracker()->getTrackerName()}: $desc1";

			case 'delete':
				return "$user deleted Tracker Item '{$desc1}' from {$ti->getTracker()->getTrackerName()}";

			default: // includes 'statuschange'
				return "$user updated #{$refid} to {$desc2}";
		}
	}
}


?>
