<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugins/forum/lib/ForumThreadPeer.php');
require_once('plugins/forum/lib/ForumThread.php');
require_once('plugins/forum/lib/Forum.php');
require_once('plugins/forum/lib/html/ForumFormRenderer.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('external/fckeditor/fckeditor.php');

$kept_variables = array('action', 'forum_id', 'thread_id');

$session = Session::getSession();
$Lang = $session->getLanguage();

require_once('plugins/forum/lib/html/ForumTemplate.php');
$theme = new ForumTemplate();

$Project = GFUrl::getUrl()->getProject();

if (isset($_GET['message_id'])) {
	$title=$Lang->getText('Forum.ViewMessage');
}elseif (isset($_GET['thread_id'])) {
	$title=$Lang->getText('Forum.ViewThread');
}elseif (isset($_GET['forum_id'])) {
	$title=$Lang->getText('Forum.ViewThreads');
}
if (isset($_GET['forum_id'])) {
	$forum_id = Validator::sanitizeInt($_GET['forum_id']);
	$forum = ForumPeer::retrieveByPk(Validator::sanitizeInt($_GET['forum_id']));
} elseif (isset($_GET['message_id'])) {
	$fm=ForumMessagePeer::retrieveByPk(Validator::sanitizeInt($_GET['message_id']));
	if (is_object($fm)) {
		$forum = $fm->getForumThread()->getForum();
	} else {
		$forum = null;
	}
} elseif (isset($_GET['thread_id'])) {
	$ft = ForumThreadPeer::retrieveByPk(Validator::sanitizeInt($_GET['thread_id']));
	if (is_object($ft)) {
		$forum = $ft->getForum();
	} else {
		$forum = null;
	}
} else {
	$forum = null;
}

if (is_null($forum)) {
	$body = $Lang->getText('Forum.InvalidForumID');
} elseif (!$forum->userCanRead()) {
	throw new PermissionDeniedException();
} else {
	$renderer = new ForumFormRenderer('project', $Project->getProjectId(), $theme, $Lang, $kept_variables);
	$body = $renderer->handleBrowse($forum);

	if ($renderer->hasErrors()) {
		$feedback = "*".implode('<br />*', $renderer->getErrors());
	}
}

$params = array(
		'project' => $Project,
		'action' => isset($action) ? $action : '',
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => isset($title) ? $title : $Lang->getText('Forum.TabString'),
		'enableUploads' => !is_null($forum) ? $renderer->mustEnableUploads() : false,
		);

if ($forum) {
	$params['forum'] = $forum;
}

$theme->header($params);
echo $body;
$theme->footer($params);
?>
