<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/frs/lib/om/BaseFrsRelease.php';
require_once("plugin/ActionManager.php");
require_once('lib/core/Filesystem.php');
require_once('lib/core/TroveCategory.php');
require_once('lib/core/TroveLinkPeer.php');
require_once('plugins/frs/lib/FrsPackageNotification.php');

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.frs.lib 
 */
class FrsRelease extends BaseFrsRelease implements GFWidget {

	const SECTION='frsrelease';
	const STATUS_ACTIVE=1;
	const STATUS_HIDDEN=2;
	var $send_notice;

	function setReleaseName($filename) {
		parent::setReleaseName(StringUtils::removeInvalidChars($filename));
	}
	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see FrsRelease::setDeleted()
	 * @see FrsRelease::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userIsAdmin()) {
			throw new PermissionDeniedException();
		}
		//delete files
		$crit=new Criteria();
		$crit->add(FilesystemPeer::SECTION,self::SECTION);
		$crit->add(FilesystemPeer::REF_ID,$this->getFrsReleaseId());
		$files = FilesystemPeer::doSelect($crit);
		foreach ($files as $file) {
			$file->delete();
		}
		//delete trove
		$crit=new Criteria();
		$crit->add(TroveLinkPeer::SECTION,self::SECTION);
		$crit->add(TroveLinkPeer::REF_ID,$this->getFrsReleaseId());
		TroveLinkPeer::doDelete($crit);
		
		//delete monitors
		$crit=new Criteria();
		$crit->add(MonitorPeer::SECTION, self::SECTION);
		$crit->add(MonitorPeer::REF_ID, $this->getFrsReleaseId());
		MonitorPeer::doDelete($crit);
		
		// Audit trail
		$audit = new AuditTrail();
		$audit->setUser(Session::getSession()->getUser());
		$audit->setSection(Project::SECTION);
		$audit->setRefId($this->getFrsPackage()->getProjectId());
		$audit->setFieldName(self::SECTION);
		$audit->setNewValue('_deleted_');
		$audit->setOldValue($this->getReleaseName().' '.Language::getSessText('Frs.AuditDeleteAllContents'));
		$audit->save();

		$params = array(
			'is_new' => FALSE,
			'item' => $this
		);

		ActionManager::notify("FRS_RELEASE_DELETE", $params);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if (!$this->userIsAdmin()) {
			throw new PermissionDeniedException();
		}
		$errors=array();
		if (strlen($this->release_name) < 4) {
			$errors['release_name'] = Language::getSessText('FrsRelease.EmptyReleaseName');
		} else {
			// Check for duplicated release name
			$c = new Criteria();
			$c->add(FrsReleasePeer::FRS_PACKAGE_ID, $this->frs_package_id);
			$c->add(FrsReleasePeer::RELEASE_NAME, $this->release_name, Criteria::EQUAL);
			if (!$this->isNew()) {
				$c->add(FrsReleasePeer::FRS_RELEASE_ID, $this->frs_release_id, Criteria::NOT_EQUAL);
			}
			$match_name = FrsReleasePeer::doSelect($c);
			if (count($match_name) > 0) {
				$errors['package_name'] = Language::getSessText('FrsRelease.DuplicatedName');
			}

		}
		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}
		
		$is_new = $this->isNew();

		$con = Propel::getConnection(FrsPackagePeer::DATABASE_NAME);
		$con->begin();
		ActionManager::notify("FRS_RELEASE_SAVE_BEFORE", $this);
		parent::save($con);

		$params = array(
			'is_new' => $is_new,
			'item' => $this
		);

		ActionManager::notify("FRS_RELEASE_SAVE", $params);
		
		// if we're submitting a new release, notify the users monitoring the
		// corresponding package
		if (isset($this->send_notice)) {
			$notification = new FrsPackageNotification($this->getFrsPackage(), $this);
			NotificationManager::queue($notification);
		}
		
		$con->commit();
	}

	public function setIsReleased($released) {
		if (!$this->getIsReleased()) {
			if ($released) {
				parent::setIsReleased($released);
				$this->send_notice=true;
			}
		}
	}

	// These functions come from GFWidget
	public function getID() {
		return $this->getFrsReleaseId();
	}
	public function getName() {
		return $this->getReleaseName();
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		if ($this->userIsAdmin()) {
			return true;
		}
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	public function userIsAdmin() {
		return $this->getFrsPackage()->userIsAdmin();
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		if ($this->userIsAdmin()) {
			return true;
		}
		
		$package = $this->getFrsPackage();
		if ( $package->userCanRead() ) {
			return true;
		}
		return false;
	}

	public function getFrsReleaseUrl() {
		return GFUrl::getURL()->projectURL($this->getFrsPackage()->getProject(),'FrsReleaseView','frs','',array('release_id'=>$this->getFrsReleaseId()));
	}

	public function getFrsReleaseLink($full=false) {
		$url = $this->getFrsReleaseUrl();
		if ($full) {
			return "<a href=\"$url\">{$this->getReleaseName()}</a>";
		} else {
			return $url;
		}
	}
	
	public function releaseFile($filepath, $filename='', $type='', $size=0) {
		if (!$this->userIsAdmin()) {
			return new PermissionDeniedException();
		}

		if (!$filename) {
			$filename = basename($filepath);
		}

		if (!$size) {
			$size = filesize($filepath);
		}

		if (!$type) {
			$type =  SystemUtils::getMimeType($filename);
		}
		
		$fs = new Filesystem();
		$fs->setSection(self::SECTION);
		$fs->setRefId($this->getFrsReleaseId());

		$fs->setFileName($filename);
		$fs->setFileType($type);
		$fs->setFileSize($size);
		$fs->setPostedBy(Session::getSession()->getUser()->getUserId());
		$fs->save();
		$fs->setFileDataFromPath($filepath, false);

		return $fs;
	}
	
	/**
	 * Returns the changelog for this release (the list of closed tasks assigned
	 * to the release)
	 */
	public function getChangeLog() {
		$c = new Criteria();
		
		// Get the list of closed items assigned to this release
		$c->addJoin(TrackerExtraFieldDataPeer::TRACKER_EXTRA_FIELD_ID, TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID);
		$c->addJoin(TrackerExtraFieldDataPeer::TRACKER_ITEM_ID, TrackerItemPeer::TRACKER_ITEM_ID);

		$c->add(TrackerExtraFieldDataPeer::FIELD_DATA, $this->getPrimaryKey());
		$c->add(TrackerExtraFieldPeer::FIELD_TYPE, TrackerExtraField::TYPE_RELEASE);
		$c->add(TrackerItemPeer::STATUS_ID, Tracker::STATUS_CLOSED);
		
		//$c->addAscendingOrderByColumn(TrackerItemPeer::TRACKER_ID);
		$c->addAscendingOrderByColumn(TrackerItemPeer::TRACKER_ITEM_ID);
		
		return TrackerItemPeer::doSelect($c);
	}

	public function deleteFile($filesystem_id) {
		$file = FilesystemPeer::retrieveByPk($filesystem_id);
		if (!$file) {
			return;
		}

		if ($file->getSection() != self::SECTION || $file->getRefId() != $this->getPrimaryKey()) {
			throw new PermissionDeniedException();
		} else if (!$this->userCanUpdate()) {
			throw new PermissionDeniedException();
		}

		$project = $this->getFrsPackage()->getProject();

		// Create the audit trail
		$audit = new AuditTrail();
		$audit->setUser(Session::getSession()->getUser());
		$audit->setSection(Project::SECTION);
		$audit->setRefId($project->getProjectId());
		$audit->setFieldName('frsfile');
		$audit->setNewValue('_deleted_');
		$audit->setOldValue($file->getFileName());
		$audit->save();

		$file->delete();
	}
}
?>
