<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code creates a new FrsRelease and saves it to the 
// database
//
require_once('plugins/frs/lib/FrsRelease.php');
require_once('lib/core/Filesystem.php');
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');
require_once('lib/core/TroveLink.php');
require_once('lib/html/UploadHandler.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}
$sess=Session::getSession();
$Project = GFUrl::getURL()->getProject();
$FrsPackage = FrsPackagePeer::retrieveByPk($_REQUEST['frs_package_id']);
if (!is_object($FrsPackage) || !$FrsPackage->userIsAdmin()
	|| $FrsPackage->getProject()->getProjectId() != $Project->getProjectId()) {
	throw new PermissionDeniedException();
}

$FrsRelease = new FrsRelease();
$FrsRelease->setFrsPackage($FrsPackage);

$Lang = Language::getSessLang();

$errors = array();

try {
	//$FrsRelease->setFrsPackageId( Validator::sanitizeInt($_POST['frs_package_id']) );
	//names are sanitized internally
	$FrsRelease->setReleaseName( $_POST['release_name'] );
	$FrsRelease->setReleaseNotes( Validator::sanitizeString($_POST['release_notes']) );
	$FrsRelease->setChanges( Validator::sanitizeString($_POST['changes']) );
	$FrsRelease->setStatusId( FrsRelease::STATUS_ACTIVE );
	if (array_key_exists('preformatted',$_POST)) {
		$preformatted = 1;
	} else {
		$preformatted = 0;
	}	
	$FrsRelease->setPreformatted( Validator::sanitizeInt($preformatted) );
	$FrsRelease->setReleaseDate( Validator::sanitizeDate($_POST['release_date']) );
	$FrsRelease->setReleasedBy( $sess->getUser()->getUserId() );
	if (isset($_REQUEST['is_released'])) {
		$FrsRelease->setIsReleased(true);
	}
	$FrsRelease->save();

	//
	//      handle the files
	//

	$isUploadEnable = GFConf::get('system.ftp_upload');
	if ($isUploadEnable){
		$prUnixName = $Project->getUnixName();
		$path = GFConf::get('system.ftp_upload_dir');
		$fullpath = $path.$prUnixName;
		if (isset($_POST['files'])) {
			$filename_id = $_POST['files'];
			$frs_files = $_POST['frs_files'];
			$filename = $frs_files[$filename_id];

			if ($filename !== 'None'){
				$size=filesize($fullpath."/".$filename);
				$type=mime_content_type($fullpath."/".$filename);	
			
				$fs = new Filesystem();
				$fs->setSection(FrsRelease::SECTION);
				$fs->setRefId($FrsRelease->getFrsReleaseId());

				$fs->setFileName( $filename );
				$fs->setFileType( $type );
				$fs->setFileSize( $size );
				$fs->setPostedBy( $sess->getUser()->getUserId() );
				$fs->save();
				$fs->setFileDataFromFTP($fullpath."/".$filename);
				// Note that setFileDataFromFTP deletes the original file.
			}
		}
	}

	// Now handle the files uploaded using the upload widget.
	$uploadHandler = new UploadHandler();
	$files = $uploadHandler->getTmpUploadedFiles();
	foreach ($files as $file) {
		$FrsRelease->releaseFile($file);
	}
	$uploadHandler->purgeTmpDir();

	for ($i=0; $i<5; $i++) {
		$error=$_FILES['addfile']['error'][$i];
		$filename=$_FILES['addfile']['name'][$i];

		if ($error > 0 && $error != 4) {
			$fileerrors = StringUtils::getFileUploadErrorString($error, $filename);
			throw new InvalidDataException($fileerrors);
			continue;
		}

		$tmp_name=$_FILES['addfile']['tmp_name'][$i];
		if (!is_uploaded_file($tmp_name)) {
			continue;
		}
		$size=$_FILES['addfile']['size'][$i];
		$type=$_FILES['addfile']['type'][$i];
		
		$FrsRelease->releaseFile($tmp_name, $filename, $type, $size);
	}

	//trove
	$troveLink = new TroveLink();
	$troveLink->setSection(FrsRelease::SECTION);
	$troveLink->setRefId($FrsRelease->getFrsReleaseId());
	$troveLink->setTroveCategoryId($_POST['trove_devstatus']);
	try {
		$troveLink->save();		
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());		
	}
	
	GFUrl::redirect(GFUrl::getUrl()->currentBaseURL('FrsReleaseBrowse'), array('feedback' => $Lang->getText('FrsRelease.AddFeedback'),'frs_package_id'=>$FrsPackage->getFrsPackageId()));

} catch (InvalidDataException $ex) {
	$errors = array_merge($errors, $ex->getErrors());		// attach object errors to validation errors
	$feedback = '* '.implode('<br />* ', $errors);
	//GFUrl::redirect(GFUrl::getUrl()->currentBaseURL('FrsReleaseAdd'), array('feedback' => $feedback,'frs_package_id'=>$FrsPackage->getFrsPackageId()));
	include('FrsReleaseAdd.php');

}
?>
