<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code lists FrsPackage
//
require_once('plugins/frs/lib/FrsPackagePeer.php');
require_once('plugins/frs/lib/FrsReleasePeer.php');
require_once('lib/core/FilesystemPeer.php');
require_once('plugins/frs/lib/FrsPackage.php');
require_once('plugins/frs/lib/FrsPlugin.php');
require_once('lib/core/TroveLink.php');
require_once('html/Paginator.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

require_once('plugins/frs/lib/html/FrsTemplate.php');
$theme = new FrsTemplate();

$Project = GFUrl::getUrl()->getProject();

$params = array(
		'project' => $Project,
		'action' => isset($action) ? $action : '',
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('FrsPackage.BrowsePageTitle')
		);
$theme->header($params);

//make paginator code
$total=0;
$default_sort_by='package_name';
$columns = array (
'package_name'=>Language::getSessText('FrsPackage.PackageNameName'),
'release_name'=>Language::getSessText('FrsRelease.LatestRelease'),
'category_name'=>Language::getSessText('FrsRelease.TroveCategory'),
'file_name'=>Language::getSessText('FrsRelease.Files'),
'file_size'=>Language::getSessText('FrsRelease.FileSize'),
'download_count'=>Language::getSessText('FrsRelease.Downloads'),
6 => Language::getSessText('FrsRelease.ChangelogTitle'),		// No sorting by changelog
);

//list of packages that you can access
$c = new Criteria();
$c->add(FrsPackagePeer::PROJECT_ID,$Project->getPrimaryKey());
$c->addAscendingOrderByColumn(FrsPackagePeer::FRS_PACKAGE_ID);
$packages = FrsPackagePeer::doSelect($c);
$readable=array();
foreach ($packages as $package) {
	if ($package->userCanRead()) {
		$readable[]=$package->getFrsPackageId();
	}
}

if (count($readable) > 0) {
	$publicsql="AND frsp.frs_package_id IN (".implode(',',$readable).")";
} else {
	$publicsql="AND frsp.is_public='true'";
}
//
//	COMPLEX SQL!
//
$sqltmp="SELECT frsp.frs_package_id,frsp.package_name,frsr.frs_release_id,
frsr.release_name,frsr.release_date,tc.category_name,fs.filesystem_id,fs.file_name,fs.file_name_safe,
fs.file_size,fs.download_count 
FROM frs_package frsp JOIN frs_release frsr ON frsp.frs_package_id=frsr.frs_package_id 
LEFT JOIN trove_link tl ON tl.ref_id=frsr.frs_release_id AND tl.section='frsrelease'
LEFT JOIN trove_category tc ON tl.trove_category_id=tc.trove_category_id
JOIN filesystem fs ON fs.section='frsrelease' AND fs.ref_id=frsr.frs_release_id 
AND frsr.release_date=(SELECT max(release_date) from frs_release JOIN filesystem fs1 
ON fs1.section='".FrsRelease::SECTION."' AND fs1.ref_id=frs_release_id where frs_package_id=frsp.frs_package_id
AND is_released=true
AND frsp.status_id='".FrsPackage::STATUS_ACTIVE."'
AND frs_release.status_id='".FrsRelease::STATUS_ACTIVE."'
$publicsql
)
AND frsr.is_released=true
AND frsr.status_id='".FrsRelease::STATUS_ACTIVE."'
AND frsp.status_id='".FrsPackage::STATUS_ACTIVE."'
$publicsql
AND frsp.project_id='".$Project->getProjectId()."' ";

//get count of matching items for paginator
$sqlCount = "SELECT count(*) AS count FROM ($sqltmp) foo";

$con=Propel::getConnection(FrsPackagePeer::DATABASE_NAME);
$stmtCount = $con->createStatement();
$rsCount = $stmtCount->executeQuery($sqlCount);
if ($rsCount->next()) {
	$total = $rsCount->getInt('count');
}

$paginator = new Paginator('br_pkgrls', -1, $total, $default_sort_by, $columns);
$sqlAux = $paginator->getSQL();

//get the items to put into paginator
$sql="$sqltmp $sqlAux";

$stmt = $con->createStatement();
$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

// Auxiliary structures to properly render the table grouping packages and releases
$pkgreleases = array();
$releaserows = array();
$rows = array();
foreach ($rs as $row) {
	$rows[] = $row;	
	$frs_package_id = $row['frs_package_id'];
	$frs_release_id = $row['frs_release_id'];
	
	if (!isset($pkgreleases[$frs_package_id])) $pkgreleases[$frs_package_id] = array();
	if (!isset($releaserows[$frs_release_id])) $releaserows[$frs_release_id] = array();
	
	$pkgreleases[$frs_package_id][] = $frs_release_id;
	$releaserows[$frs_release_id][] = $row;
}

// Remove duplicates
$pkgreleases = array_map('array_unique', $pkgreleases);

$sort = $paginator->getSortingOptions();
$sort_by = $sort['sort_by'];
$sort_order = $sort['order'];


$theader = $paginator->getSortableTableHeaders($theme);

echo $theme->tableTop($theader);

if (count($releaserows) < 1) {
	// No releases found -- just show the empty packages
	$c = new Criteria();
	$c->add(FrsPackagePeer::PROJECT_ID, $Project->getProjectId());
	$packages = FrsPackagePeer::doSelect($c);
	
	$trows = array();
	$tmods = array();
	
	foreach ($packages as $package) {
		if (!$package->userCanRead()) {
			// User doesn't have access to the package
		} else {
			$url = GFUrl::getUrl()->currentBaseUrl('FrsReleaseBrowse', array('frs_package_id' => $package->getFrsPackageId()));
			$trow = array();
			$tmod = array();
			$trow[] = '<a href="'.$url.'">'.$package->getPackageName().'</a>';
			$tmod[] = '';
			$trow[] = $Lang->getText('FrsRelease.NoReleasesForPackage');
			$tmod[] = 'colspan="6"';
			$trows[] = $trow;
			$tmods[] = $tmod;
		}
	}
	
	if (count($trows) == 0) {
		echo $theme->tableFullRow($Lang->getText('FrsPackage.NoneFound'));
	} else {
		for ($i = 0; $i < count($trows); $i++) {
			echo $theme->tableRow($trows[$i], $tmods[$i]);
		}
	}
} else {
	
	foreach ($pkgreleases as $frs_package_id => $frs_release_ids) {
		$package_row = true;
		
		// How many rows this package expands
		$package_rowspan = 0;
		foreach ($frs_release_ids as $frs_release_id) {
			$package_rowspan += count($releaserows[$frs_release_id]);
		}
		
		foreach ($frs_release_ids as $frs_release_id) {
			$release_row = true;
			foreach ($releaserows[$frs_release_id] as $row) {
				$trow = array();
				$tmod = array();
				
				if ($package_row) {
					$package_url = GFUrl::getUrl()->currentBaseUrl('FrsReleaseBrowse', array('frs_package_id' => $frs_package_id));
					$trow[] = '<a href="'.$package_url.'"><img src="/images/package_icon.gif" height="16" width="16" border="0" />&nbsp;'.$row['package_name'].'</a>';
					// For the default sorting (by package name) we use rowspans to group the packages and releases
					$tmod[] = 'rowspan="'.$package_rowspan.'" bgcolor="#ffffff"';
				}
				
				if ($release_row) {
					$release_url = GFUrl::getUrl()->currentBaseUrl('FrsReleaseView', array('release_id' => $frs_release_id));
					$trow[] = '<a href="'.$release_url.'">'.$row['release_name'].'</a>';
					$tmod[] = 'rowspan="'.count($releaserows[$frs_release_id]).'" bgcolor="#ffffff"';
				}
				
				// Category
				$trow[] = ((!empty($row['category_name'])) ? $row['category_name'] : 'Not Categorized' );
				$tmod[] = '';

				// Download link
				// If the package requires login and the user is not logged in
				// we show a link to login page, otherwise we show the download link

				$Package_obj = FrsPackagePeer::retrieveByPk($row['frs_package_id']);
				$str_url_target =  '';
				if (($Package_obj->getRequireLogin() || !$Package_obj->isPublic()) && !$session->isLoggedIn()) {
					$url = GFUrl::getURL()->baseURL('Login', '', 'account', array('redirect' => $_SERVER['REQUEST_URI']));
				} else {
					$fs = FilesystemPeer::retrieveByPk($row['filesystem_id']);
					if (is_object($fs)) {
						$url = GFUrl::downloadUrlFromFS($fs);
						
						if ($fs->canOpenInBrowser()) {
							$str_url_target = ' target="_blank"';
						}
					} else {
						$url = '';
					}
					//$url = GFUrl::getURL()->downloadURL(FrsRelease::SECTION, $row['frs_release_id'], $row['filesystem_id'], $row['file_name_safe']);
				}

				// File name
				$trow[] = '<a href="'.$url.'"'.$str_url_target.'>'.$row['file_name'].'</a>';
				$tmod[] = '';

				// Readable file size
				$trow[] = SystemUtils::getReadableSize($row['file_size'], 2);
				$tmod[] = '';
				
				// Download count
				$trow[] = $row['download_count'];
				$tmod[] = '';
				
				// Changelog link
				$trow[] = '<a href="'.GFUrl::getUrl()->currentBaseUrl('FrsReleaseChangelog', array('release_id' => $row['frs_release_id'])).'">'.Language::getSessText('FrsRelease.Changelog').'</a>';
				$tmod[] = '';
				
				echo $theme->tableRow($trow, $tmod);

				
				$release_row = false;
				$package_row = false;
			}
		}
	}
}

$perm = $Project->getPermission(); 
$button_enabled = $perm->check(FrsPlugin::SECTION, 0, FrsPlugin::WRITE);
echo $theme->tableBottom(array(GFUrl::getURL()->projectUrl($Project,'','frs','admin',array())=>$Lang->getText('FrsPackage.ManagePackages')),array($button_enabled));

if(isset($paginator) && is_object($paginator)) {
	echo '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
}

$theme->footer($params);
?>
