<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/mailman/lib/om/BaseMailman.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.mailman.lib 
 */
class Mailman extends BaseMailman implements GFPermission {
	const SECTION = 'mailman';

	const STATUS_REQUESTED 			= 1;
	const STATUS_CREATED 			= 2;
	const STATUS_DELETE_REQUEST		= 0;
	
	/** Validation errors */
	private $invalid_data = array();


	/**
	 * isPublic - true/false
	 * @return boolean
	 */
	function isPublic() {
		return $this->getIsPublic();
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		if ($this->userIsAdmin()) {
			return true;
		}
		if ($this->isPublic()) {
			return true;
		}
		/*
		//requires manual customization
		$perm = $this->getProject()->getPermission();
		if ($perm->check(self::SECTION, SELF::CREATE)) {
			return true;
		}
		*/
		return false;
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 *
	 *
	 * The current implemented rule is the OPTION 1. The two options were:
	 * -------------------------------OPTION 1-------------------------------
	 * When L is private, only the members of P can see it.
	 * When L is public, the set of users that can see it is the same that the
	 *	set of users that can see P. (P public would allow ALL users, P private
	 *	would allow only members of P)
	 *
	 * if (!$this->isPublic())
	 *	return (current user is member of $this->getProject())
	 * else
	 *	return $this->getProject()->userCanRead()
	 *
	 * -------------------------------OPTION 2-------------------------------
	 * When L is private, only the members of P can see it.
	 * When L is public, all the users can access the list.
	 * 
	 * if (!this->isPublic())
	 *	return (current user is member of $this->getProject())
	 * else
	 *	return true
	 */

	public function userCanRead() {
		if ($this->userIsAdmin()) {
			return true;
		}

		if ($this->isPublic() == false) {
			return (Session::getSession()->isLoggedIn() && Session::getSession()->getUser()->isProjectMember($this->getProject()));
		}

		return $this->getProject()->userCanRead();
	}

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see Mailman::setDeleted()
	 * @see Mailman::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete Mailman object: permission denied");
		}
		ActionManager::notify("MAILMAN_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create Mailman object: permission denied");
		} else if (!$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update Mailman object: permission denied");
		}
		
		$missing_data = array();
		if (is_null($this->list_name) || empty($this->list_name)) {
			$missing_data['list_name'] = Language::getSessText('MailmanAddNew.ErrorNoListName');
		}
		$errors = array_merge($missing_data, $this->invalid_data);
		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}

		parent::save($con);
		ActionManager::notify("MAILMAN_SAVE", $this);
	}

	/**
	 * Sets the list name, and does validation check. $list_name should be of type <project_unix_name>-<list_name>
	 *
	 * @param string $list_name
	 */
	public function setListName($list_name) {

		$project = GFUrl::getUrl()->getProject();
		if (!is_object($project)) { // when approving project, we don't have the project in the url
			$project = $this->getProject();
		}		
		if((strlen($list_name) - strlen($project->getUnixName()) - 1) < 4) {
			$this->invalid_data['list_name'] = Language::getSessText('MailmanAddNew.ErrorNoListName');
			return;
		}
		
		if(!Validator::validateEmail($list_name.'@'.GFConf::get('mailman.listsHost'))) {
			$this->invalid_data['list_name'] = Language::getSessText('MailmanAddNew.ErrorInvalidListName', array($list_name));
			return;
		}
		
		$crit = new Criteria();
		$crit->add(MailmanPeer::LIST_NAME, $list_name);
		$arr = MailmanPeer::doSelect($crit);
		if (count($arr)>0) { 
			$this->invalid_data['list_name'] = Language::getSessText('MailmanAddNew.ErrorListAlreadyExists');
			return;
		}

	
		parent::setListName($list_name);
		unset($this->invalid_data['list_name']);
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	public function userIsAdmin() {
		if(posix_getuid() === 0){
			//root user, the daemon has called
			return true;
		}
		return $this->getProject()->userIsAdmin();
	}
	
	function getListinfoURL() {
		$serverPath = '/mailman/listinfo/'.$this->getListName();

		if (GFConf::get('system.hostName') == GFconf::get('mailman.listsHost')) {
			return GFUrl::getUrl()->buildUrl($serverPath);
		} else {
			return 'http://'.GFconf::get('mailman.listsHost').$serverPath;
		}
	}
	
	function getAdminURL() {
		$serverPath = '/mailman/admin/'.$this->getListName();

		if (GFConf::get('system.hostName') == GFconf::get('mailman.listsHost')) {
			return GFUrl::getUrl()->buildUrl($serverPath);
		} else {
			return 'http://'.GFconf::get('mailman.listsHost').$serverPath;
		}
	}
}
?>
