<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code lists News
//
require_once('plugins/news/lib/NewsPeer.php');
require_once('plugins/news/lib/News.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

require_once('plugins/news/lib/html/NewsTemplate.php');
$theme = new NewsTemplate();

$Project = GFUrl::getUrl()->getProject();

$blog_post = new News();
$blog_post->setProjectId($Project->getProjectId());
if (is_object($blog_post) && !$blog_post->userCanRead()) {
	throw new PermissionDeniedException("You don't have permission to access the Blog: permission denied");
}

$columns = array(
	NewsPeer::POST_DATE => $Lang->getText('News.PostDateName'), 
	$Lang->getText('News.SummaryName'));

//Get params
$tag = isset($_REQUEST['tag']) ? Validator::sanitizeString($_REQUEST['tag']) : "";
$year = isset($_REQUEST['year']) ? Validator::sanitizeInt($_REQUEST['year']) : "";
$month = isset($_REQUEST['month']) ? Validator::sanitizeInt($_REQUEST['month']) : "";
$news_id = isset($_REQUEST['id']) ? Validator::sanitizeInt($_REQUEST['id']) : 0;
$thread_id = isset($_REQUEST['thread_id']) ? Validator::sanitizeInt($_REQUEST['thread_id']) : 0;
$message_id = isset($_REQUEST['message_id']) ? Validator::sanitizeInt($_REQUEST['message_id']) : 0;
$forum_action = isset($_REQUEST['_forum_action']) ? Validator::sanitizeString($_REQUEST['_forum_action']) : "";

if (!( ( ($month >=1 && $month <= 12) && ($year > 0 && $year <= 9999) ) || ($month == "" && $year == "") ) ) {
	throw new Exception("Invalid date.");
}

$con = Propel::getConnection(NewsPeer::DATABASE_NAME);

// Retrieve the objects array
$crit = new Criteria();
$crit->add(NewsPeer::PROJECT_ID, $Project->getProjectId());
if ($tag != "") {
	$crit->addJoin(NewsPeer::NEWS_ID, TagEntryPeer::REF_ID, Criteria::JOIN);
	$crit->add(TagEntryPeer::SECTION, News::SECTION);
	$crit->add(TagEntryPeer::TAG_NAME, $tag);
}
if ($year != 0 && $month != 0) {
	$start_date = $year."-".$month."-01" . " 00:00:00-05";
	$last_day_month = date('t',strtotime($month."/1/".$year));
	$end_date = $year."-".$month."-".$last_day_month . " 23:59:59-05";
	
	$crit->add(NewsPeer::POST_DATE, $start_date, Criteria::GREATER_EQUAL);
	$crit->addAnd(NewsPeer::POST_DATE, $end_date, Criteria::LESS_EQUAL);
}
if ($news_id != 0) {
	$crit->add(NewsPeer::NEWS_ID, $news_id);
}

$total = NewsPeer::doCount($crit);

$paginator = new Paginator('news', -1, $total, NewsPeer::POST_DATE, $columns);
$paginator->addPageClauses($crit);
$arr = NewsPeer::doSelect($crit);

/*$theader = array();
$theader[] = $Lang->getText('News.PostDateName');
$theader[] = $Lang->getText('News.SummaryName');*/
//$theader[] = $Lang->getText('News.DetailsName');

$theader = $paginator->getSortableTableHeaders($theme);
//Get tags
$str_sql = "SELECT tag_name, count(*) as quantity FROM tag_entry t, news n WHERE t.ref_id=n.news_id AND n.project_id =" . $Project->getProjectId() . " AND section='" . News::SECTION . "' GROUP BY tag_name";
$stmtCount = $con->createStatement();
$rs = $stmtCount->executeQuery($str_sql);

$arr_tagnames = array();
foreach ($rs as $row) {
	$arr_tagnames[ $row['tag_name'] ] = $row['quantity'];
}

//Set min and max font sizes
$min_font_size = 10; // min font size in pixels
$max_font_size = 18; // max font size in pixels

//Get min and max tag values
$min_tag_value = (sizeof(array_values($arr_tagnames)) > 0) ? min(array_values($arr_tagnames)) : 0;
$max_tag_value = (sizeof(array_values($arr_tagnames)) > 0) ? max(array_values($arr_tagnames)) : 0;

//Get the range of values
$spread = $max_tag_value - $min_tag_value;
if ($spread == 0) {
	$spread = 1;
}

//Set the font-size increment
$step = ($max_font_size - $min_font_size) / ($spread);

$arr_tags = array();     
//Loop through the tags array
foreach ($arr_tagnames as $key => $value) {
	$size = round($min_font_size + (($value - $min_tag_value) * $step));
	$arr_tags[ $key ] = $size;
}

//Get Archives
$str_sql = "SELECT date_part('month', post_date) as smonth, date_part('year', post_date) as syear 
						FROM news
						WHERE project_id = " . $Project->getProjectId() . "
						GROUP BY syear, smonth
						ORDER BY syear, smonth DESC";

$stmtArchives = $con->createStatement();
$rs_archives = $stmtCount->executeQuery($str_sql);

$arr_archives = array();
$i=0;
foreach ($rs_archives as $row) {
	$arr_archives[$i]['month'] =  $row['smonth'];
	$arr_archives[$i]['year'] =  $row['syear'];
	$arr_archives[$i]['desc'] =  date("F", mktime(0, 0, 0, $row['smonth'])) . " " . $row['syear'];
	$i++;
}

if ($Project->getPermission()->isAdmin()) {
	$button_enabled = true;
} else {
	$button_enabled = false;
}

//If we are showing a post with the threads:
if ($news_id != 0) {
	$News = NewsPeer::retrieveByPk($news_id);
	if (!is_object($News) || !$News->userCanRead()) {
		throw new PermissionDeniedException();
	}
	
	// Generate the forum page. This is done BEFORE outputting any text before this code could redirect.
	include_once('plugins/news/lib/html/NewsCommentsRenderer.php');
	$kept_vars = array();
	$renderer = new NewsCommentsRenderer(News::SECTION, NewsPlugin::getFormPaths(), $News->getPrimaryKey(), $theme, $Lang, $kept_vars, $forum_action);
	
	// Try to create a forum for this news on the fly
	$c = new Criteria();
	$c->add(ForumPeer::SECTION, News::SECTION);
	$c->add(ForumPeer::REF_ID, $News->getPrimaryKey());
	$forum = ForumPeer::doSelectOne($c);
	if (is_null($forum)) {
		$forum = new Forum();
		$forum->setSection(News::SECTION);
		$forum->setRefId($News->getPrimaryKey());
		$forum->setForumName($News->getSummary());
		// We do this because we may be logged in as any user, so we must skip
		// permission checks
		$forum->forceSave();
		$forum->save();
	}

	//Process action
	$form = "";
	if ($forum_action != "") {
		$form = $renderer->handleBrowse($forum, $News->getSummary(), $thread_id, $message_id, $forum_action);
	}

	//Get the thread messages
	$forum_messages = "";
	if ($message_id != 0) {
		$forum_messages = $renderer->handleBrowse($forum, $News->getSummary(), $thread_id, $message_id, 'ForumMessageBrowse');
	}
	
	//Get the threads
	$forum_page = $renderer->handleBrowse($forum, $News->getSummary(), $thread_id, $message_id, '', $forum_messages);
	
	if ($renderer->hasErrors()) {
		$feedback = "*".implode('<br />*', $renderer->getErrors());
	}
	
} else {
	$forum_page = "";
}

$feedback = isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : '');

$params = array(
		'project' => $Project,
		'action' => isset($action) ? $action : '',
		'feedback' => $feedback,
		'title' => $Lang->getText('News.BrowsePageTitle')
		);

$theme->header($params);

require_once("forms/NewsBrowse-tpl.php");

$str_paginator = '';
if(isset($paginator) && is_object($paginator)) {
	$str_paginator = '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
}

?>
<?php
$theme->footer($params);
?>
