<?php
require_once 'plugins/pastebin/lib/om/BasePastebin.php';

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.pastebin.lib 
 */
class Pastebin extends BasePastebin {
	const SECTION = 'pastebin';

	const EXPOSURE_PUBLIC = 0;
	const EXPOSURE_PRIVATE = 1;
	const EXPOSURE_PROJECT = 2;

	const EXPIRATION_NEVER = 0;
	const EXPIRATION_10_MINUTES = 1;
	const EXPIRATION_ONE_HOUR = 2;
	const EXPIRATION_ONE_DAY = 3;
	const EXPIRATION_ONE_WEEK = 4;
	const EXPIRATION_TWO_WEEKS = 5;
	const EXPIRATION_ONE_MONTH = 6;

	/**
	 * Saves the pastebin object, enforcing permission
	 * @var object db conneection, optional
	 */
	function save($con = null) {
		if (!$this->userCanWrite()) {
			throw new PermissionDeniedException("Could not create save pastebin object: permission denied");
		}
		parent::save($con);
	}

	/**
	 * Retrieves the url for the pastebin object
	 * @var object params for the url
	 * @return string
	 */
	public function getPasteUrl($params=array()) {
		return GFUrl::getFullURL(GFUrl::getUrl()->baseURL('', Pastebin::SECTION, PastebinPeer::getPasteHash($this), $params));
	}

	/**
	 * Validates permissions on this object for reading
	 * @return boolean
	 */
	public function userCanRead() {
		//If exposure is public, anyone can read
		if ($this->getExposureType() == Pastebin::EXPOSURE_PUBLIC) {
			return true;
		}

		//If it's not public and the user is not logged in, then he cannot read
		if (!Session::getSession()->isLoggedIn()) {
			return false;
		}

		//If the user is site admin, no more checks required
		if (Session::getSession()->getUser()->isSiteAdmin()) {
			return true;
		}

		//If the exposure is private, we already know the user is logged in, therefore he can read
		if ($this->getExposureType() == Pastebin::EXPOSURE_PRIVATE) {
			return true;
		}

		//If the exposure is project, then we need to check if the project is public, or if the user is a member
		if ($this->getExposureType() == Pastebin::EXPOSURE_PROJECT) {
			$project = ProjectPeer::retrieveByPk( $this->getRefId() );
			if (!is_object($project)) {
				return false;
			}
			$user = Session::getSession()->getUser();
			if ($project->isPublic() || $user->isProjectMember($project)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Validates permissions on this object for writing
	 * @return boolean
	 */
	public function userCanWrite() {
		return $this->userCanRead();
	}

	/**
	 * Validates permissions on this object for updating
	 * @return boolean
	 */
	public function userCanUpdate() {
		$user = Session::getSession()->getUser();
		if ($this->userCanRead() && is_object($user) && ($user->getUserId() == $this->getPostedBy() || $user->isSiteAdmin()) ) {
			return true;
		} else {
			return false;
		}
	}


	/**
	 * Validates that the paste is not expired
	 * @return boolean
	 */
	public function isExpired() {
		if ($this->getExpiration() == Pastebin::EXPIRATION_NEVER) {
			return false;
		}

		if ($this->getExpirationTimestamp() > time()) {
			return false;
		}

		return true;
	}

	/**
	 * Returns an array with the offsets for each expiration, in seconds
	 * @return array
	 */
	public function getExpirationTimestamp() {
		$offset = 0;
		switch($this->getExpiration()) {
			case Pastebin::EXPIRATION_10_MINUTES:
				$offset = 600; //60 * 10
				break;

			case Pastebin::EXPIRATION_ONE_HOUR:
				$offset = 3600; //60 * 60
				break;

			case Pastebin::EXPIRATION_ONE_DAY:
				$offset = 86400; //60 * 60 * 24
				break;

			case Pastebin::EXPIRATION_ONE_WEEK:
				$offset = 604800; // 60 * 60 * 24 * 7
				break;

			case Pastebin::EXPIRATION_TWO_WEEKS:
				$offset = 1209600; // 60 * 60 * 24 * 7 * 2
				break;

			case Pastebin::EXPIRATION_ONE_MONTH:
				$offset = 2419200; // 60 * 60 * 24 * 7 * 4
				break;
		}

		return (strtotime($this->getPostDate()) + $offset);
	}


	public function getLanguage() {
		$language = parent::getLanguage();
		if (trim($language) == '100') {
			return 'none';
		} else {
			return $language;
		}
	}
}

