<?php
// The parent class
require_once 'plugins/pastebin/lib/om/BasePastebinPeer.php';

// The object class
include_once 'plugins/pastebin/lib/Pastebin.php';

/** 
 * The skeleton for this class was autogenerated by Propel  *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package plugins.pastebin.lib 
 */
class PastebinPeer extends BasePastebinPeer {

	/**
	 * Gets an array of the latest paste objects
	 * @return array of Pastebin
	 */
	public static function getLatestPastes($user=null) {
		$exposure_public = Pastebin::EXPOSURE_PUBLIC;
		$exposure_private = Pastebin::EXPOSURE_PRIVATE;
		$exposure_project = Pastebin::EXPOSURE_PROJECT;
		$section = Project::SECTION;

		$sql = "SELECT * FROM pastebin WHERE ";

		if (is_object($user)) {
			$project_ids = implode( array_keys($user->getAllAccessibleProjectsAssoc()) , ',' );
			if ($project_ids != '') {
				$project_ids =  ' AND ref_id IN ('.$project_ids.')';
			}
			$sql .= "(exposure_type = '{$exposure_public}' OR (exposure_type = '{$exposure_private}') OR (exposure_type = '{$exposure_project}' AND section = '{$section}'{$project_ids}))";
		} else {
			$sql .= "(exposure_type = '{$exposure_public}')";
		}

		//Check expiration
		$expiration_offsets = PastebinPeer::getExpirationOffsets();
		$arr_sql_blocks = array();
		foreach($expiration_offsets as $expiration_type => $expiration_offset) {
			$arr_sql_blocks[] = "(expiration = " . $expiration_type . " AND (post_date + interval '" . $expiration_offset ." seconds') > now())";
		}
		$sql_expiration = implode( $arr_sql_blocks , ' OR ' );

		$sql .= " AND ((expiration = 0) OR " . $sql_expiration . ") ORDER BY post_date DESC LIMIT 10";

		$con = Propel::getConnection(UserPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		try {
			return PastebinPeer::populateObjects($rs);
		} catch(Exception $e) {
			return array();
		}
	}

	/**
	 * Gets url hash of a given paste
	 *
	 * @param Pastebin
	 * @return string that represents the hash
	 */
	public static function getPasteHash($pastebin) {
		return base_convert($pastebin->getPastebinId(), 10, 32);
	}

	/**
	 * Given a hash, returns the ID of the paste
	 *
	 * @param string paste hash
	 * @return int ID of the paste
	 */
	public static function convertPasteHash($pastebin_hash) {
		return base_convert($pastebin_hash, 32, 10);
	}

	/**
	 * Returns an array with the language codes for the syntax highlighter
	 *
	 * @return array of language codes
	 */
	public static function getLanguages() {
		return array(
			"c" => "C",
			"cpp" => "C++",
			"csharp" => "C#",
			"css" => "CSS",
			"flex" => "Flex",
			"html" => "HTML",
			"java" => "Java",
			"javaScript" => "Javascript",
			"javascript_dom" => "JavaScript with DOM",
			"perl" => "Perl",
			"php" => "PHP",
			"python" => "Python",
			"ruby" => "Ruby",
			"sql" => "SQL",
			"xml" => "XML"
		);
	}

	/**
	 *
	 * Returns an array with the expiration options
	 *
	 * @return array of expiration options
	 */
	public static function getExpirations() {
		$session = Session::getSession();
		$Lang = $session->getLanguage();

		return array(
			Pastebin::EXPIRATION_NEVER => $Lang->getText('Pastebin.ExpirationNever'),
			Pastebin::EXPIRATION_10_MINUTES => $Lang->getText('Pastebin.ExpirationTenMin'),
			Pastebin::EXPIRATION_ONE_HOUR => $Lang->getText('Pastebin.ExpirationOneHour'),
			Pastebin::EXPIRATION_ONE_DAY => $Lang->getText('Pastebin.ExpirationOneDay'),
			Pastebin::EXPIRATION_ONE_WEEK => $Lang->getText('Pastebin.ExpirationOneWeek'),
			Pastebin::EXPIRATION_TWO_WEEKS => $Lang->getText('Pastebin.ExpirationTwoWeeks'),
			Pastebin::EXPIRATION_ONE_MONTH => $Lang->getText('Pastebin.ExpirationOneMonth')
		);
	}

	/**
	 * Returns an array with the exposure options
	 *
	 * @return array of exposure options
	 */
	public static function getExposures() {
		$session = Session::getSession();
		$Lang = $session->getLanguage();

		return array(
			Pastebin::EXPOSURE_PUBLIC => $Lang->getText('Pastebin.ExposurePublic'),
			Pastebin::EXPOSURE_PRIVATE=> $Lang->getText('Pastebin.ExposureRegistered'),
			Pastebin::EXPOSURE_PROJECT => $Lang->getText('Pastebin.ExposureProject')
		);
	}

	/**
	 * Returns an array with the syntax highlighting themes
	 *
	 * @return array of theme options
	 */
	public static function getThemes()	{
		return array(
			"acid"=>"acid", "berries-dark"=>"berries-dark", "berries-light"=>"berries-light", "bipolar"=>"bipolar", "blacknblue"=>"blacknblue", "bright"=>"bright", "contrast"=>"contrast", "darkblue"=>"darkblue",
			"darkness"=>"darkness", "desert"=>"desert", "dull"=>"dull", "easter"=>"easter", "emacs"=>"emacs", "golden"=>"golden", "greenlcd"=>"greenlcd", "ide-anjuta"=>"ide-anjuta", "ide-codewarrior"=>"ide-codewarrior",
			"ide-devcpp"=>"ide-devcpp", "ide-eclipse"=>"ide-eclipse", "ide-kdev"=>"ide-kdev", "ide-msvcpp"=>"ide-msvcpp", "kwrite"=>"kwrite", "matlab"=>"matlab", "navy"=>"navy", "nedit"=>"nedit", "neon"=>"neon",
			"night"=>"night", "pablo"=>"pablo", "peachpuff"=>"peachpuff", "print"=>"print", "rand01"=>"rand01", "random"=>"random", "the"=>"the", "typical"=>"typical", "typical"=>"typical", "vim"=>"vim", "vim-dark"=>"vim-dark",
			"whatis"=>"whatis", "whitengrey"=>"whitengrey", "zellner"=>"zellner");
	}


	public static function getExpirationOffsets() {
		$arr_expiration = array();
		$arr_expiration[Pastebin::EXPIRATION_10_MINUTES] = 600;
		$arr_expiration[Pastebin::EXPIRATION_ONE_HOUR] = 3600;
		$arr_expiration[Pastebin::EXPIRATION_ONE_DAY] = 86400;
		$arr_expiration[Pastebin::EXPIRATION_ONE_WEEK] = 604800;
		$arr_expiration[Pastebin::EXPIRATION_TWO_WEEKS] = 1209600;
		$arr_expiration[Pastebin::EXPIRATION_ONE_MONTH] = 2419200;
		return $arr_expiration;
	}


	public static function renderFirstNLines($str,$numlines) {
		$lines = explode("\n", $str);
		$first_n = array_slice($lines, 0, $numlines);
		$res = '<ol>';
		$i = 1;
		foreach ($first_n as $line) {
			if ($i == count($first_n) && trim($line) == '') {
				continue;
			}
			$res .= '<li>' . $line . '</li>';
			$i++;
		}
		$res .= '</ol>';
		return $res;
	}
}
