<?php
/**
 * Copyright (c) 2005-2013 GForge, LLC
 *
 * @version   $Id$
 * @author Mario A. Papayannis	<mpapayannis@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugins/pastebin/lib/Pastebin.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');

$session = Session::getSession();
$user = $session->getUser();

if ($_SERVER['REQUEST_METHOD'] != 'POST' || !is_object($user)) {
	throw new PermissionDeniedException();
}

$Lang = Language::getSessLang();
$pastebin = new Pastebin();
$errors = array();

$pastebin->setTitle( Validator::sanitizeString($_POST['title']) );
$pastebin->setContent( Validator::sanitizeString($_POST['content']) );
$pastebin->setPostedBy( $user->getUserId() );
$pastebin->setExposureType( Validator::sanitizeString($_POST['exposure']) );
$pastebin->setExpiration( Validator::sanitizeString($_POST['expiration']) );
$pastebin->setLanguage( Validator::sanitizeString($_POST['language']) );
$project_id = Validator::sanitizeString($_POST['project']);
if ($project_id > 0) {
	$pastebin->setSection(Project::SECTION);
	$pastebin->setRefId($project_id);
	$project = ProjectPeer::retrieveByPk($project_id);
	if (!is_object($project) && $pastebin->getExposureType() == Pastebin::EXPOSURE_PROJECT) {
		$errors[] = $Lang->getText('Pastebin.ErrorProjectRequired');
	}
} else {
	$pastebin->setSection('site');
	if ($pastebin->getExposureType() == Pastebin::EXPOSURE_PROJECT) {
		$errors[] = $Lang->getText('Pastebin.ErrorProjectRequired');
	}
}

if ($pastebin->getTitle() == '') {
	$errors[] = $Lang->getText('Pastebin.ErrorTitleRequired');
}

if ($pastebin->getContent() == '') {
	$errors[] = $Lang->getText('Pastebin.ErrorContentRequired');
}

//Share to users
$share_users = $_REQUEST['share_users'];
$arr_share_users = explode(',',$share_users);
$arr_users = array();
if (count($arr_share_users) > 0 && trim($share_users) != '') {
	foreach($arr_share_users as $unix_name) {
		$user_share = UserPeer::retrieveByUnixName($unix_name);
		if (!is_object($user_share)) {
			$errors[] = $Lang->getText('Pastebin.ShareUserInvalid', array($unix_name));
		} else {
			if ($user_share->getIsGroup()) {
				$group_users = $user_share->getUsergroupUsers();
				foreach($group_users as $group_user) {
					$arr_users[] = $group_user->getUnixName();
				}
			} else {
				$arr_users[] = $user_share->getUnixName();
			}
		}
	}
}

//Get tags
$tags = $_POST['tags'];


if (count($errors) == 0) {
	try {
		$pastebin->save();

		//Get list of unix names to notify
		$message = ' <a href="'.$pastebin->getPasteUrl().'">'.$pastebin->getPasteUrl().'</a>';
		ForumPeer::sendUserWallMentions($arr_users, $user->getUserId(), $message, '', true);

		//Save tags
		TagEntryPeer::addTags(pastebin::SECTION, $pastebin->getPastebinId(), Validator::sanitizeString($tags));
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());
	}

}

if (count($errors) > 0) {
	$feedback = '* '.implode('<br />* ', $errors);
	require_once('Pastebin.php');
} else {
	$params = array('feedback' => $Lang->getText('Pastebin.Added'));
	GFUrl::getUrl()->redirect( $pastebin->getPasteUrl($params) );
}
?>
