<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('html/ProjectTemplate.php');
require_once('html/HTMLWidget.php');
require_once('plugins/projectrating/lib/UserProjectRating.php');
require_once('plugins/projectrating/lib/html/RatingWidget.php');

$project = GFUrl::getUrl()->getProject();
$lang = Session::getSession()->getLanguage();
$template = new ProjectTemplate();
$session = Session::getSession();

$params = array(
	'project' => $project,
	'title' => $lang->getText('ProjectRating.ProjectComments'),
	'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
	'extra_head' => RatingWidget::getExtraHeaders(),
);
$template->header($params);

$c = new Criteria();
$c->add(UserProjectRatingPeer::PROJECT_ID, $project->getProjectId());
$c->addDescendingOrderByColumn(UserProjectRatingPeer::RATING_DATE);
$ratings = UserProjectRatingPeer::doSelectJoinUser($c);


$columns = array(
	$lang->getText('User.UserTitle'),
	$lang->getText('ProjectRating.CommentsTitle'),
);

if ($project->userIsAdmin()) {
	// Project admin can delete comments
	$columns[] = $lang->getText('submit.delete');
}

echo $theme->tableTop($columns);

foreach ($ratings as $rating) {
	$user = $rating->getUser();
	
	$columns = array();
	$mod = array();
	
	$userColumn = $lang->getText('User.UserTitle').': '.$user->getUserLink().'<br />';
	$userColumn .= $lang->getText('ProjectRating.RatingDateTitle').': '.$rating->getRatingDate();
	
	$ratingColumn = RatingWidget::getWidget('rating'.$user->getUserId(), $rating->getRating(), true, Language::getSessText('ProjectRating.RatingTitle').':');
	$ratingColumn .= nl2br($rating->getComment());
	
	$columns[] = $userColumn;
	$mod[] = 'width="20%"';
	$columns[] = $ratingColumn;
	$mod[] = '';

	if ($project->userIsAdmin()) {
		// Project admin can delete comments
		$url = GFUrl::getUrl()->currentBaseUrl('UserProjectRatingDelete', array('user_id' => $user->getUserId()));
		$deleteColumn = '<form method="post" action="'.$url.'">';
		$deleteColumn .= HtmlWidget::button('submit', $lang->getText('submit.delete'));
		$deleteColumn .= '</form>';
		$columns[] = $deleteColumn;
		$mod[] = '';
	}
	
	echo $theme->tableRow($columns, $mod);
}

echo $theme->tableBottom(array(GFUrl::getUrl()->projectUrl($project) => $lang->getText('Common.Back')));


$template->footer();
?>
