<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2007-08-02
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/scm/lib/om/BaseScmCommitFilter.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.scm.lib 
 */
class ScmCommitFilter extends BaseScmCommitFilter implements GFPermission {


	const PRE_COMMIT = 1;
	const POST_COMMIT = 2;


	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		if ($this->userIsAdmin()) {
			return true;
		}
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		if (Session::getSession()->isSiteAdmin()) {
			return true;
		}

		return false;
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		return true;
	}

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @throws PermissionDeniedException
	 * @return void
	 * @see ScmCommitFilter::setDeleted()
	 * @see ScmCommitFilter::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete ScmCommitFilter object: permission denied");
		}
		ActionManager::notify("SCM_COMMIT_FILTER_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @throws InvalidDataException
	 * @throws PermissionDeniedException
	 * @return void
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create ScmCommitFilter object: permission denied");
		} else if (!$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update ScmCommitFilter object: permission denied");
		}
		
		$errors = array();
		
		if (is_null($this->filter_description) || strlen($this->filter_description) < 5) {
			$errors['filter_description'] = Language::getSessText('ScmCommitFilter.FilterDescriptionTooShort');
		}
		if (is_null($this->filter_string) || strlen($this->filter_string) < 5) {
			$errors['filter_string'] = Language::getSessText('ScmCommitFilter.FilterStringTooShort');
		}
		
		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}
		
		parent::save($con);
		ActionManager::notify("SCM_COMMIT_FILTER_SAVE", $this);
		if (GFConf::get('system.useDaemon')) {
			GFDaemon::notify('DAEMON_SCM_COMMIT_FILTER_SAVE', $this->getScmCommitFilterId());
		}
	}

	public function getHRFilterType() {

		$filterTypeStr = '';
		$plugin_name = $this->getPlugin()->getPluginName();

		switch ($this->getFilterType()) {
			case self::PRE_COMMIT: {
				if ($plugin_name == 'scmcvs') {
					$filterTypeStr = 'commitinfo';
				} else if ($plugin_name == 'scmsvn') {
					$filterTypeStr = 'Pre-commit';
				} else if ($plugin_name == 'scmgit') {
					$filterTypeStr = 'Pre-receive';
				} else {
					$filterTypeStr = 'N/A';
				}
				break;
			}
			case self::POST_COMMIT: {
				if ($plugin_name == 'scmcvs'){
					$filterTypeStr = 'verifymsg';
				} else if ($plugin_name == 'scmsvn') {
					$filterTypeStr = 'Post-commit';
				} else if ($plugin_name == 'scmgit') {
					$filterTypeStr = 'Post-receive';
				} else {
					$filterTypeStr = 'N/A';
				}
				break;
			}

		}
		return $filterTypeStr;
	}
}
?>