<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('plugin/GFProjectPlugin.php');
require_once("plugin/ActionManager.php");
require_once('core/Plugin.php');
require_once('core/ProjectPlugin.php');
require_once('core/Role.php');
require_once('core/PropelJson.php');
require_once('exceptions/InvalidPluginException.php');
require_once('plugins/scm/lib/ProjectScm.php');
require_once('lib/GFExportImportUtils.php');
require_once('plugins/scm/lib/RepositoryBrowser.php');
require_once('plugins/scm/lib/SCMRepositoryFactory.php');
require_once('lib/system/GFDaemon.php');

abstract class ScmPlugin implements GFProjectPlugin {

	const NOACCESS = -1;
	const READ = 0;
	const WRITE = 1;
	const ISPRIVATE = 0;
	const ISPUBLIC = 1;
	const SECTION = 'scm';
	const CODE_SEARCH_ELEMENT_CLASS = 1;
	const CODE_SEARCH_ELEMENT_METHOD = 2;
	const CODE_SEARCH_ELEMENT_VARIABLE = 3;
	const CODE_SEARCH_ELEMENT_COMMENT = 4;

	/**
	 * Project this plugin is assigned to
	 */
	protected $project;

	function __construct() {
		
	}

	function initialize() {
		ActionManager::registerListeners($this, $this->getListeners());
	}

	/**
	 * 	Different types of plugins are possible -
	 * 	GFPlugin::SITE_PLUGIN = 'site'
	 * 	GFPlugin::PROJECT_PLUGIN = 'project'
	 * 	GFPlugin::USER_PLUGIN = 'user'
	 */
	function getType() {
		return GFPlugin::PROJECT_PLUGIN;
	}

	/**
	 * 	get the array of action names that this plugin answers to.
	 * 	@return	array
	 */
	function getListeners() {
		return array(
			'BUILD_PROJECT_TABS',
			'GET_PROJECT_ADMIN_NAV_ELEMENTS',
			'GET_PROJECT_REPORTING_NAV_ELEMENTS',
			'GET_SCM_NAV_ELEMENTS',
			//		'BUILD_PROJECT_SUMMARY',
			'PROJECT_APPROVE',
			'DAEMON_SCM_ENABLE',
			'DAEMON_PROJECT_APPROVE',
			'DAEMON_UP_DELETE',
			'DAEMON_UP_SAVE',
			'DAEMON_UPDATE_ROLE_CHOICES',
			'DAEMON_UPDATE_OBSERVER',
			'DAEMON_MODULE_SAVE',
			'DAEMON_MODULE_USER_SAVE',
			'DAEMON_ACCESS_TYPE_UPDATE',
			'DAEMON_SSH_KEY_CHANGED',
			'DAEMON_USER_SAVE',
			'DAEMON_USERGROUP_DELETE',
			'DAEMON_COMMIT_FILTER_PROJECT_SAVE',
			'DAEMON_SCM_COMMIT_FILTER_SAVE',
			'DAEMON_SCM_EMAIL_COMMIT',
			'DAEMON_PROJECT_DELETE',
			'BUILD_MYPAGE_ELEMENTS',
			'GET_ROLE_CHOICES',
			'UPDATE_ROLE_CHOICES',
			'GET_OBSERVER_ROLE_CHOICES',
			'UPDATE_OBSERVER_SETTINGS',
			'GET_DISK_USAGE',
			'GET_DISK_QUOTA_USAGE',
			'PROJECT_FULLEXPORT',
			'PROJECT_FULLIMPORT',
			'PROJECT_TPLEXPORT',
			'PROJECT_TPLIMPORT',
			'ASSOCIATION_SECTION',
			'ASSOCIATION_CHECK',
			'RENDER_ASSOCIATION_LINK',
			'GET_ASSOCIATED_ITEM_INFO',
			'BUILD_SEARCH_COMBO',
			'DO_SEARCH',
			'DO_TAG_SEARCH',
			'PROJECT_EVENT_PULL',
			'ACTIVITY_LOG_MIGRATE_ROW',
			'SCM_COMMIT'
		);
	}

	/**
	 * 	handle action has a case statement to redirect the action to the appropriate function.
	 * 	@param string
	 * 	@param mixed
	 * 	@return PluginResult
	 */
	function handleAction($listenername, $params) {
		switch ($listenername) {
			case 'BUILD_PROJECT_TABS': {
					return $this->buildMainTabsListener($params);
					break;
				}
			case 'GET_PROJECT_ADMIN_NAV_ELEMENTS': {
					return $this->getProjectAdminNavElements($params);
					break;
				}
			case 'GET_PROJECT_REPORTING_NAV_ELEMENTS': {
					return $this->getProjectReportingNavElements($params);
					break;
				}

			case 'GET_SCM_NAV_ELEMENTS': {
					return $this->getScmNavElements($params);
					break;
				}
			case 'BUILD_PROJECT_SUMMARY': {
					return $this->buildProjectSummary($params);
					break;
				}
			case 'PROJECT_APPROVE': {
					return $this->approveProjectListener($params);
					break;
				}
			case 'DAEMON_SCM_ENABLE':
			case 'DAEMON_COMMIT_FILTER_PROJECT_SAVE':
			case 'DAEMON_ACCESS_TYPE_UPDATE':
			case 'DAEMON_MODULE_USER_SAVE':
			case 'DAEMON_MODULE_SAVE':
			case 'DAEMON_PLUGIN_ENABLE':
			case 'DAEMON_UP_SAVE':
			case 'DAEMON_UP_DELETE':
			case 'DAEMON_UPDATE_ROLE_CHOICES':
			case 'DAEMON_UPDATE_OBSERVER':
			case 'DAEMON_PROJECT_APPROVE':
			case 'DAEMON_SCM_EMAIL_COMMIT': {
					return $this->daemonSetupRepository($params);
					break;
				}
			case 'DAEMON_PROJECT_DELETE': {
					return $this->daemonProjectDelete($params);
					break;
			}
			case 'DAEMON_SCM_COMMIT_FILTER_SAVE': {
					return $this->daemonScmCommitFilterSave($params);
					break;
				}
			case 'DAEMON_USER_SAVE': {
					return $this->daemonUserSave($params);
					break;
				}
			case 'DAEMON_SSH_KEY_CHANGED': {
					return $this->daemonSshKeyChanged($params);
					break;
				}
			case 'DAEMON_USERGROUP_DELETE': {
					return $this->daemonUserGroupDelete($params);
					break;
			}
			case 'BUILD_MYPAGE_ELEMENTS': {
					return $this->buildMyPageElements($params);
					break;
				}
			case 'GET_ROLE_CHOICES': {
					return $this->getRoleChoices($params);
					break;
				}
			case 'UPDATE_ROLE_CHOICES': {
					return $this->updateRoleChoices($params);
					break;
				}
			case 'GET_OBSERVER_ROLE_CHOICES': {
					return $this->getObserverRoleChoices($params);
					break;
				}
			case 'UPDATE_OBSERVER_SETTINGS': {
					return $this->updateObserverSettings($params);
					break;
				}
			case 'GET_DISK_USAGE': {
					return $this->getDiskUsage($params);
					break;
				}
			case 'GET_DISK_QUOTA_USAGE': {
					return $this->getDiskQuotaUsage($params);
					break;
				}
			case 'PROJECT_FULLIMPORT':
			case 'PROJECT_FULLEXPORT': {
					return GFExportImportUtils::getSpecData($this, '/scm/conf/specData.xml');
					break;
				}
			case 'PROJECT_TPLIMPORT':
			case 'PROJECT_TPLEXPORT': {
					return GFExportImportUtils::getSpecData($this, '/scm/conf/specDataTpl.xml');
					break;
				}
			case 'ASSOCIATION_SECTION': {
					return $this->getAssociationSection($params);
					break;
				}
			case 'ASSOCIATION_CHECK': {
					return $this->getAssociationCheck($params);
					break;
				}
			case 'RENDER_ASSOCIATION_LINK': {
					return $this->renderAssociationLink($params);
					break;
				}
			case 'GET_ASSOCIATED_ITEM_INFO': {
					return $this->getAssociatedItemInfo($params);
					break;
				}
			case 'BUILD_SEARCH_COMBO': {
					return $this->buildSearchOptions($params);
					break;
				}
			case 'DO_SEARCH': {
					return $this->getSearchResults($params);
					break;
				}
			case 'DO_TAG_SEARCH': {
					return $this->getSearchByTagsResult($params);
					break;
				}
			case 'PROJECT_EVENT_PULL': {
					return $this->getProjectEvent($params);
					break;
				}
			case 'ACTIVITY_LOG_MIGRATE_ROW': {
					return $this->migrateActivityRow($params);
					break;
				}
			case 'SCM_COMMIT': {
					return $this->onScmCommit($params);
					break;
				}
		}
	}

	private function onScmCommit($params) {
		try {
			$project = $params['project'];
			if ($project->usesPlugin($this->getName())) {
				$commit = $params['commit'];
				return $this->doCommitActivity(
						$commit,
						$commit->getUser(),
						$commit->getProject(),
						'create',
						$commit->getScmCommitUrl(),
						$commit->getMessageLog());
			}
		} catch (Exception $ex) {
			error_log("Error in {$this->getName()}->onScmCommit(): {$ex->getMessage()}");
		}
	}

	private function doCommitActivity($commit, $user, $project, $verb, $url, $message = '') {

		$user_id = 100;
		$user_name = 'Anonymous User';
		if ($user != null) {
			$user_id = $user->getUserId();
			$user_name =  $user->getUnixName();
		}

		$event = array(
			'project_id' => $project->getPrimaryKey(),
			'section' => ScmCommit::SECTION,
			'ref_id' => $commit->getPrimaryKey(),
			'user_id' => $user_id,
			'activity_date' => time(),
			'description' => "{$user_name} {$verb}d a commit in {$project->getProjectName()}: {$message}",
			'url' => $url,
			'activity_type' => $verb,
			'extended_data' => PropelJson::ShallowJson($commit)
		);

		$result = ActionManager::notify('ACTIVITY_LOG_EVENT', $event);

		$ret = new PluginResult($this);
		$ret->setResult(true);

		return $ret;
	}

	/**
	 * returns the part of the html text that goes into the my page area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageElements($params) {
		
	}

	private function getProjectReportingNavElements($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$arr = array(
			'title' => 'SCM',
			'elements' => array(
				array('link' => '/gf/project/' . $params['project']->getUnixName() . '/' . $this->getName() . '/?action=ScmCommitsReport',
					'description' => Language::getSessText('ScmCommitsReport.CommitsReport'))
			)
		);
		$result->setResult($arr);
		return $result;
	}

	protected abstract function buildMainTabsListener($params);

	/**
	 *      getScmNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	protected abstract function getScmNavElements($params);

	/**
	 * 	The addMember listener - this hook will fire
	 * 	whenever a member is added to a project.
	 * 	@param	User	User object
	 * 	@return	boolean
	 */
	function addMemberListener($params) {

	}

	/**
	 * 	The removeMember listener - this hook will fire
	 * 	whenever a member is removed from a project.
	 * 	@param	User	User object
	 * 	@return	boolean
	 */
	function removeMemberListener($params) {

	}

	/**
	 * 	get the name of this plugin. example: 'tracker'
	 * 	@return	string
	 */
	public function getName() {
		return 'scm';
	}

	/**
	 * 	The approveProject listener - this hook will fire
	 * 	when a project is approved.
	 * 	@param	Project	the project object
	 * 	@return	boolean
	 */
	function approveProjectListener($params) {
		$project = $params['project'];
		$template_project = $params['template_project'];
		$con = Propel::getConnection(UserPeer::DATABASE_NAME);
		// If SCM was enabled on the template project, copy all
		// the information
		if ($template_project->usesPlugin($this->getName())) {
			$tpscm = ProjectScmPeer::retrieveByPk($template_project->getPrimaryKey());
			if (is_null($tpscm))
				return;  // SCM not configured in the project

			$pscm = $tpscm->copy();

			// Check if the SCM server ID is needed and present on the configuration
			$plugin_name = $this->getName();
			if (GFConf::get('system.splitScmServers') && $pscm->getControlled() && GFConf::varExists($plugin_name . '.defaultScmServerId')) {
				$scm_server_id = GFConf::get($plugin_name . '.defaultScmServerId');
				$pscm->setScmServerId($scm_server_id);
			}


			$project_id = $project->getId();
			$pscm->setProjectId($project_id);
			$pscm->save();
			// search the role_settings and let's see if for this item we have one set
			foreach ($params['role_settings'] as $role_setting) {
				if (strstr($role_setting['section'], $this->getName())) {
					$role_id = $role_setting['role_id'];
					$section = $role_setting['section'];
					$ref_id = 0;
					$value = $role_setting['value'];
					$sql = "insert into role_setting (role_id,section,ref_id,value) values ('$role_id','$section','$ref_id','$value')";
					$stmt = $con->createStatement();
					$rs = $stmt->executeQuery($sql);
				}
			}
		}
	}

	/**
	 * This function will be called inside the daemon process after the project is approved
	 */
	function daemonSetupRepository($params) {
		GFDaemon::logDebugMessage("ScmPlugin ({$this->getName()}): Processing daemon message");
		$unix_name = $params;
		$project = ProjectPeer::retrieveByUnixName($unix_name, false);
		if (!$project) {
			return;
		}

		if (!$project->usesPlugin($this->getName())) {
			return;
		}

		$repo = SCMRepositoryFactory::getRepository($project);
		if (!$repo) {
			return;
		}

		// Create the repository and all the files associated to it
		GFDaemon::logDebugMessage("ScmPlugin ({$this->getName()}): Setting up repository");
		$repo->setupRepository();
	}

	function daemonProjectDelete($params) {
		GFDaemon::logDebugMessage("ScmPlugin ({$this->getName()}): Processing daemon message");
		$unix_name = $params;
		$project = ProjectPeer::retrieveByUnixName($unix_name, false);
		if (!$project) {
			return;
		}

		if (!$project->usesPlugin($this->getName())) {
			return;
		}

		$repo = SCMRepositoryFactory::getRepository($project);
		if (!$repo) {
			return;
		}

		// Create the repository and all the files associated to it
		GFDaemon::logDebugMessage("ScmPlugin ({$this->getName()}): Setting up repository");
		$repo->removeProjectGroups();

	}

	/**
	 * This function will be called inside the daemon process when the user information is updated. If the password changes, we need to update the
	 * system config files.
	 */
	function daemonUserSave($params) {
		$unix_name = $params;
		$user = UserPeer::retrieveByUnixName($unix_name);
		if (!$user) {
			return;
		}
		$projects = $user->getAllProjects(false);


		foreach ($projects as $project) {
			// Update the project's SCM information. While this may seem overkill (we only need to update one user's information), it's the
			// safest way to do it.
			$this->daemonSetupRepository($project->getUnixName());
		}

	}

	function daemonUserGroupDelete($params) {
		$name = $this->getName();

		//Avoid running the whole thing for anything other than these three plugins
		if ($name != 'scmgit' && $name != 'scmsvn' && $name != 'scmcvs') {
			return;
		}

		//I don't think this is really needed, but just in case
		$projects = array_unique(unserialize($params));


		$con = Propel::getConnection();
		$sql = 'SELECT unix_name FROM project WHERE project_id IN ('.implode(',', $projects).')';
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		while ($rs->next()) {
			$this->daemonSetupRepository($rs->getString(1));
		}
	}

	/**
	 * 	The deleteProject listener - this hook will fire
	 * 	when a project is deleted.
	 * 	@return	boolean
	 */
	function deleteProjectListener() {

	}

	/**
	 * 	The checkPermission - this listener will fire
	 * 	when permissions are being checked for this project.
	 * 	@param	Array	An associative array containing 
	 * 	'section_name' a name like 'tracker' or 'tracker_item'
	 * 	'section_id'=>id (the id of the section, for example a specific tracker)
	 * 	'subsection_id'=>id (the id of the subsection, for example an tracker_item id)
	 * 	'perm_level'=>int (the value, such a 1 for read, 2 for write, that is being requested)
	 * 	@return	boolean
	 * /
	  function checkPermission($params);
	 */
	function enableInProject(Project $project) {
		// The plugin was enabled in the project -- create an initial SCM configuration if needed
		$c = new Criteria();
		$c->add(ProjectScmPeer::PROJECT_ID, $project->getProjectId());
		$projectscm = ProjectScmPeer::doSelectOne($c);
		if (is_object($projectscm)) {
			// project_scm already exists. Check if the type of SCM has changed. In that case,
			// update the SCM server (only applies to split SCM installations).
			$typename = 'scm' . strtolower($projectscm->getScmServer()->getScmType());

			if ($typename != $this->getName()) {
				$varName = $this->getName() . '.defaultScmServerId';
				if (GFConf::varExists($varName)) {
					// The SCM type has changed, update the ProjectScm
					$projectscm->setScmServerId(GFConf::get($varName));
					$projectscm->save();
				}
			}
			return;
		}

		$projectscm = $this->createDefaultScmConfig($project);
		$projectscm->save();

		return;
	}

	function disableFromProject() {
		// unregister the actions
		ActionManager::unregisterListeners($this);
	}

	// Plugin-specific methods

	/**
	 * returns the possible role settings available for this plugin
	 *
	 * @return array	the roles
	 */
	function getAvailableRoleSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Scm.NoAccess'); //descriptions
		$return[1][] = ScmPlugin::NOACCESS; // values
		$return[0][] = Language::getSessText('Scm.Read');
		$return[1][] = ScmPlugin::READ;
		$return[0][] = Language::getSessText('Scm.Write');
		$return[1][] = ScmPlugin::WRITE;
		return $return;
	}

	/**
	 * 	The getRoleChoices listener - this hook will fire
	 * 	when a new role is added or one is going to be updated.
	 * 	It returns an array of role settings
	 * 	@return	array
	 */
	function getRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$roles = array();
		$roles[0]['ref_id'] = 0;
		$roles[0]['section'] = Language::getSessText('Scm.Scm', array($this->getname()));
		$roles[0]['description'] = Language::getSessText('Scm.Roles.Description');
		$roles[0]['subsection'] = '-';
		$roles[0]['settings'][0] = self::SECTION;
		$roles[0]['settings'][] = $this->getAvailableRoleSettings();

		$result = new PluginResult($this);
		$result->setResult($roles);
		return $result;
	}

	/**
	 * returns the settings for the forum status
	 *
	 * @return array
	 */
	function getStatusObserverSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Scm.Private'); //descriptions
		$return[1][] = ScmPlugin::ISPRIVATE; // values
		$return[0][] = Language::getSessText('Scm.Public');
		$return[1][] = ScmPlugin::ISPUBLIC;
		return $return;
	}

	/**
	 * 	The getObserverRoleChoices listener - this hook will fire
	 * 	when the observer role is modified.
	 * 	It returns an array of role settings or null
	 * 	@return	array or null
	 */
	function getObserverRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$choices = array();
		$choices[0]['ref_id'] = $params['project']->getProjectId(); // special case, we need to know somehow later it's the project id in project_scm table
		$choices[0]['section'] = Language::getSessText('Scm.Privileges');
		$choices[0]['description'] = Language::getSessText('Scm.Roles.Description');
		$choices[0]['subsection'] = '-';
		$choices[0]['settings'][0] = self::SECTION;
		$choices[0]['settings'][] = $this->getStatusObserverSettings();

		$result = new PluginResult($this);
		$result->setResult($choices);
		return $result;
	}

	/**
	 * Retrieves the concerning data and updates its observer role settings
	 *
	 * @param array	$data
	 */
	function updateObserverSettings($params) {
		$project = $params['project'];
		$data = $params['data'];

		$onedata = current($data);
		while ($onedata != null) {
			$section = key($data);
			switch ($section) {
				case self::SECTION : {
						$oneitem = current($onedata);
						while ($oneitem != null) {
							$ref_id = key($onedata);
							$value = $oneitem;
							//retrieve the scm and update it
							$c = new Criteria();
							$c->add(ProjectScmPeer::PROJECT_ID, $ref_id);
							try {
								$projectscm = ProjectScmPeer::doSelectOne($c);
							} catch (Exception $e) {
								// something wrong happened
								echo $e->getMessage();
								next($onedata);
								$oneitem = current($onedata);
								continue;
							}

							if (!is_object($projectscm)) {
								// For some reason the object wasn't created... create it now
								$projectscm = $this->createDefaultScmConfig($project);
							}

							$projectscm->setEnableAnonread($oneitem);
							$projectscm->save();
							next($onedata);
							$oneitem = current($onedata);
						}
						break;
					}
				default : {
						//skip it
						continue;
					}
			}
			next($data);
			$onedata = current($data);
		}
	}

	/**
	 * Retrieve the corresponding setting (value) for the ref_id passed as argument, using the appropriate section
	 *
	 * @param int $ref_id
	 * @param string $section
	 * @return setting
	 */
	function getObserverSetting($ref_id, $section) {
		switch ($section) {
			case self::SECTION : {
					$c = new Criteria();
					$c->add(ProjectScmPeer::PROJECT_ID, $ref_id);
					try {
						$projectscm = ProjectScmPeer::doSelectOne($c);
					} catch (Exception $e) {
						echo $e->getMessage();
						return '';
					}

					if (is_null($projectscm)) {
						// This means the project_scm object still wasn't created. Allow
						// SCM access by default
						return true;
					} else {
						return $projectscm->getEnableAnonread();
					}

					break;
				}
			default : {
					return '';
				}
		}
	}

	private function updateRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$section = self::SECTION;

		$project = $params['project'];
		$data = $params['data'];
		$role = $params['role'];

		if (!isset($data[$section])) {
			return;
		}

		foreach ($data[$section] as $ref_id => $value) {
			$c = new Criteria();
			$c->add(RoleSettingPeer::ROLE_ID, $role->getPrimaryKey());
			$c->add(RoleSettingPeer::SECTION, $section);
			$c->add(RoleSettingPeer::REF_ID, $ref_id);
			$rolesetting = RoleSettingPeer::doSelectOne($c);
			if (!is_object($rolesetting)) {
				// Enter here if we have a role setting which we didn't have previously (e.g.: activate an inactive plugin)
				$rolesetting = new RoleSetting();
				$rolesetting->setSection($section);
				$rolesetting->setRefId($ref_id);
				$rolesetting->setRoleId($role->getPrimaryKey());
			}

			// Being a project admin overrides any other value
			if (isset($data['projectadmin']) && $data['projectadmin'][0] >= Project::PROJECT_ROLE_ADMIN) {
				$value = self::WRITE;
			}

			$rolesetting->setValue($value);
			$rolesetting->save();
		}

		return;
	}

	/**
	 * Creates a default SCM configuration object for the current project. 
	 * Usually the configuration object is created when the project is based on a template 
	 * project with SCM enabled, but when SCM is disabled on the template project and SCM is
	 * enabled on the project, this object is not created.
	 * @return	ProjectScm	The ProjectScm object. This object is NOT SAVED to the database.
	 */
	protected function createDefaultScmConfig(Project $project) {
		$projectscm = new ProjectScm();
		$projectscm->setProject($project);
		$projectscm->setEnableAnonRead(true);
		$projectscm->setControlled(true);

		// We're on a problem here... we must assign the SCM server to the object.
		$c = new Criteria();
		$c->add(ScmServerPeer::SCM_TYPE, strtoupper($this->getName()));
		$server = ScmServerPeer::doSelectOne($c);  // Assign the first one

		if (!is_object($server)) {
			// Problem -- no SCM server for this SCM type. We assign any server.
			$server = ScmServerPeer::doSelectOne(new Criteria());
		}

		$projectscm->setScmServer($server);

		return $projectscm;
	}

	/**
	 *      return an sql statement for calculating disk usage
	 */
	function getDiskUsage($params) {
		$con = Propel::getConnection(ProjectPeer::DATABASE_NAME);
		$IterateSql = "SELECT pp.project_id, pr.unix_name, pl.plugin_name
					FROM project pr, project_plugin pp, project_scm ps, plugin pl
					WHERE pr.project_id=pp.project_id AND pp.project_id=ps.project_id
					AND ps.controlled=true AND pp.plugin_id=pl.plugin_id AND pl.exclusive_type='scm'";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($IterateSql, ResultSet::FETCHMODE_ASSOC);

		if ($rs->getRecordCount() > 0) {
			while ($rs->next()) {
				$scm_type = $rs->getString('plugin_name');
				if ($scm_type == 'scmcvs') {
					$scmdir = GFConf::get('scmcvs.localCVSRoot');
				} else if ($scm_type == 'scmsvn') {
					$scmdir = GFConf::get('scmsvn.localSVNRoot');
				} else {
					continue;
				}
				$filesize = 0;
				$project_id = $rs->getInt('project_id');
				$project_name = $rs->getString('unix_name');
				$repository = $scmdir . '/' . $project_name;
				if (file_exists($repository)) {
					$filesize += SystemUtils::getDirSize($repository);
				}
				$stmt->executeQuery("UPDATE project_scm SET bytes_used={$filesize} WHERE project_id={$project_id}");
			}
		}

		$sql = "SELECT project_scm.project_id AS project_id, SUM(project_scm.bytes_used) AS usage FROM project_scm, project_plugin, plugin
				WHERE plugin.plugin_name='" . $this->getName() . "' AND project_plugin.plugin_id=plugin.plugin_id AND project_scm.project_id=project_plugin.project_id
				GROUP BY project_scm.project_id ORDER BY project_scm.project_id";
		$result = new PluginResult($this);
		$result->setResult($sql);
		return $result;
	}

	protected function getDiskQuotaUsage($params) {
		//Check if there is a folder for the project in git, svn or cvs directories
		$project_name = $params['project_unix_name'];
		switch ($this->getName()) {
			case 'scmcvs' :
				$scmdir = GFConf::get('scmcvs.localCVSRoot');
				break;
			case 'scmsvn' :
				$scmdir = GFConf::get('scmsvn.localSVNRoot');
				break;
			case 'scmgit' :
				$scmdir = GFConf::get('scmgit.localGitRoot');
				break;
			default :
				return false;
		}
		$repository = $scmdir . '/' . $project_name;
		$filesize = 0;
		if (file_exists($repository)) {
			$filesize += SystemUtils::getDirSize($repository);
		}
		$resArray = array(
			'plugin_name' => $this->getName(),
			'usage'   => array($repository => $filesize)
		);
		$result = new PluginResult($this);
		$result->setResult($resArray);
		return $result;
	}

	private function getAssociationCheck($params) {
		if ($params->getSection() == ScmCommit::SECTION) {
			$sc = ScmCommitPeer::retrieveByPk($params->getRefId());
			if (!is_object($sc) || !$sc->userCanRead()) {
				throw new InvalidDataException(array('The scm commit doesn\'t exist or you don\'t have permissions for it'));
			}
		}
		if ($params->getToSection() == ScmCommit::SECTION) {
			$sc = ScmCommitPeer::retrieveByPk($params->getToRefId());
			if (!is_object($sc) || !$sc->userCanRead()) {
				throw new InvalidDataException(array('The scm commit doesn\'t exist or you don\'t have permissions for it'));
			}
		}
	}

	private function getAssociationSection($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$result->setResult(array(ScmCommit::SECTION, Language::getSessText('ScmCommit.ScmCommit')));
		return $result;
	}

	private function renderAssociationLink($params) {
		$ret = array();
		foreach ($params as $param) {
			if ($param[0] == ScmCommit::SECTION) {
				$sc = ScmCommitPeer::retrieveByPk($param[1]);
				if (is_object($sc) && $sc->getProject()->usesPlugin($this->getName())) {
					$ret[] = array(
						$sc->getPrimaryKey(),
						Language::getSessText('ScmCommit.ScmCommit') . ': ' . $sc->getScmCommitLink(false),
						$param[2],
						$param[3]
					);
				}
			} else {
				$arr = explode("|", $param[2]);
				if (isset($arr[2]) && $arr[2] == ScmCommit::SECTION && $arr[0] == WallPost::SECTION) {
					$res = Association::renderNonPluginAssociationLink($param);
					if (is_array($res)) {
						$ret[] = $res;
					}
				}
			}
		}
		$result = new PluginResult($this);
		$result->setResult($ret);
		return $result;
	}

	/**
	 * 	Returns additional info for the associated item
	 * 		 
	 * 	@param array $params needed params to build the associated item info
	 * 	@return array with associated item info (Description, URL)
	 */
	private function getAssociatedItemInfo($params) {
		$ret = array();
		foreach ($params as $param) {
			if ($param[0] == ScmCommit::SECTION) {
				$sc = ScmCommitPeer::retrieveByPk($param[1]);
				if (is_object($sc) && $sc->getProject()->usesPlugin($this->getName())) {
					$ret[] = array(
						$param[2],
						$param[3],
						$param[4],
						$param[5],
						$param[6],
						Language::getSessText('ScmCommit.ScmCommit') . ': #' . $sc->getScmCommitId(),
						$sc->getScmCommitUrl(false));
				}
			}
		}
		$result = new PluginResult($this);
		$result->setResult($ret);
		return $result;
	}

	private function buildSearchOptions($params) {
		$result = new PluginResult($this);
		$value = array();

		if (isset($params['project'])) {
			if (!$params['project']->usesPlugin($this->getName())) {
				return;
			}
			$text = Language::getSessText('Scm.Search.ThisProjectSource');
		} else {
			// Hack so it will only show once on the search combo
			// (all the SCM plugins inherit from this class, without this the combo will show
			// as many duplicated entries as SCM plugins)
			static $shown = false;
			if ($shown) {
				return;
			}
			$shown = true;

			$text = Language::getSessText('Scm.Search.SourceCode');
		}

		$value[] = array(
			'value' => $this->getName(),
			'display' => $text,
			'id' => 'scm',
		);
		$result->setResult($value);
		return $result;
	}

	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return PluginResult
	 */
	private function getSearchResults($params) {
		if (array_search($this->getName(), $params['search']) !== false) {
			$result = new PluginResult($this);
			$searcher = new GFSearch($params);
			$result->setResult($this->search($searcher));
			return $result;
		}
	}

	private function getSearchByTagsResult($params) {
		$result = new PluginResult($this);
		$search = new GFSearch($params);

		// Hack so it will only show the tagged commits once
		// (all the SCM plugins inherit from this class, without this the table will show
		// as many duplicated entries as SCM plugins)

		static $shown = false;

		if ($shown) {
			return;
		} else {

			$result->setResult($this->searchByTags($search));
			$shown = true;

			return $result;
		}
	}

	/**
	 * 	take a search object and handle the search internally.
	 * 	@param	GFSearch
	 * 	@return PluginResult
	 */
	function search(GFSearch $search) {
		if (!$search->getProjectId()) {
			return $this->globalSearch($search);
		}

		$project = $search->params['project'];

		$template = $search->getTemplate();
		$con = Propel::getConnection();
		$words = $search->getWordsForQuery();

		$repos = SCMRepositoryFactory::getRepository($project);
		$reposBrowser = new RepositoryBrowser($repos);

		if (GFConf::get('system.dbtype') == 'postgres') {
			$sql = 'SELECT csm.module, csi.filename, HEADLINE(csi.filename, q) AS headline_filename, csi.line, csi.revision, 
						HEADLINE(csi.class_name, q) AS class_name, HEADLINE(csi.method_name, q) AS method_name,
						HEADLINE(csi.variable_name, q) AS variable_name, HEADLINE(csi.comment, q) AS comment
					FROM code_search_module csm, code_search_index csi, to_tsquery(?) q
					WHERE csi.code_search_module_id=csm.code_search_module_id AND
					csm.project_id=? AND csi.vectors @@ q';
		}

		$stmt = $con->prepareStatement($sql);
		$stmt->setString(1, $words);
		$stmt->setInt(2, $search->getProjectId());

		$rs = $stmt->executeQuery();

		$cols = array();
		$cols[] = Language::getSessText('Scm.Search.Module');
		$cols[] = Language::getSessText('Scm.Search.Revision');
		$cols[] = Language::getSessText('Scm.Search.Filename');
		$cols[] = Language::getSessText('Scm.Search.Line');
		$cols[] = Language::getSessText('Scm.Search.MatchType');
		$cols[] = Language::getSessText('Scm.Search.Element');
		$cols[] = Language::getSessText('Scm.Search.Comment');
		$content = $template->tableTop($cols);

		$count = 0;
		while ($rs->next()) {
			$count++;

			$match = $this->parseCodeSearchMatch($rs);
			$comment = ($rs->getString('comment') ? $rs->getString('comment') : '-');

			if ($repos->getType() == 'git') {
				// In git we don't need the name of the module (branch) because it's implicit in the revision #
				$fullpath = $rs->getString('filename');
			} else {
				$fullpath = SystemUtils::buildFilePath('/' . $rs->getString('module'), $rs->getString('filename'));
			}
			$browseURL = $reposBrowser->getBrowsePathUrl($fullpath, $rs->getString('revision'));

			$cols = array();
			$cols[] = $rs->getString('module');
			$cols[] = $rs->getString('revision');
			$cols[] = '<a href="' . $browseURL . '">' . $rs->getString('headline_filename') . '</a>';
			$cols[] = $rs->getInt('line');
			$cols[] = $match['element_type'];
			$cols[] = $match['full_name'];
			$cols[] = $comment;
			$content .= $template->tableRow($cols);
		}
		$content .= $template->tableBottom();

		$title = Language::getSessText('Scm.SearchResultsTabTitle', array($count));

		if ($count == 0) {
			$content = Language::getSessText('Search.NoResults');
		}

		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content
		);
	}

	function globalSearch(GFSearch $search) {
		$template = $search->getTemplate();
		$con = Propel::getConnection();
		$words = $search->getWordsForQuery();

		$project_sql = '';
		$project_ids = $search->getProjectIds();

		if (!is_null($project_ids)) {
			$project_sql = ' AND project.project_id IN ('.implode(',', $project_ids).')';
		}

		if (GFConf::get('system.dbtype') == 'postgres') {
			// If searching for something like "Class.Method" we want the period to be replaced by an ampersand
			//UPDATE: removed, this breaks searching ".php", "aaa..bbb"
			// $words = str_replace('.', '&', $words);

			$sql = "SELECT
						csm.module,  										-- 1
						csi.filename, 										-- 2
						HEADLINE(csi.filename, q) AS headline_filename,		-- 3 
						csi.line, 											-- 4
						csi.revision, 										-- 5
						HEADLINE(csi.class_name, q) AS class_name,			-- 6 
						HEADLINE(csi.method_name, q) AS method_name,		-- 7
						HEADLINE(csi.variable_name, q) AS variable_name,	-- 8
						HEADLINE(csi.comment, q) AS comment,				-- 9
						project.*											-- 10
					FROM code_search_module csm, code_search_index csi, project, to_tsquery(?) q
					WHERE csi.code_search_module_id=csm.code_search_module_id $project_sql
					AND csm.project_id=project.project_id AND csi.vectors @@ q
					ORDER BY project.project_name ASC";
		}
		$stmt = $con->prepareStatement($sql);
		$stmt->setString(1, $words);

		$rs = $stmt->executeQuery(ResultSet::FETCHMODE_NUM);

		$cols = array();
		$cols[] = Language::getSessText('Project.ListNameTitle');
		$cols[] = Language::getSessText('Scm.Search.Module');
		$cols[] = Language::getSessText('Scm.Search.Revision');
		$cols[] = Language::getSessText('Scm.Search.Filename');
		$cols[] = Language::getSessText('Scm.Search.Line');
		$cols[] = Language::getSessText('Scm.Search.MatchType');
		$cols[] = Language::getSessText('Scm.Search.Element');
		$cols[] = Language::getSessText('Scm.Search.Comment');
		$content = $template->tableTop($cols);

		$count = 0;
		while ($rs->next()) {
			$count++;

			$project = new Project();
			$project->hydrate($rs, 10);
			if (!$project->userCanRead()) {
				continue;
			}

			// Build the URL for browsing the file
			$repos = SCMRepositoryFactory::getRepository($project);
			$reposBrowser = new RepositoryBrowser($repos);


			if ($repos->getType() == 'git') {
				// In git we don't need the name of the module (branch) because it's implicit in the revision #
				$fullpath = $rs->getString(2);
			} else {
				$fullpath = SystemUtils::buildFilePath('/' . $rs->getString(1), $rs->getString(2));
			}

			$browseURL = $reposBrowser->getBrowsePathUrl($fullpath, $rs->getString(5));

			$match = $this->parseCodeSearchMatch($rs);
			$comment = ($rs->getString(9) ? $rs->getString(9) : '-');

			$cols = array();
			$cols[] = $project->getProjectLink();
			$cols[] = $rs->getString(1);
			$cols[] = $rs->getString(5);
			$cols[] = '<a href="' . $browseURL . '">' . $rs->getString(3) . '</a>';
			$cols[] = $rs->getInt(4);
			$cols[] = $match['element_type'];
			$cols[] = $match['full_name'];
			$cols[] = $comment;
			$content .= $template->tableRow($cols);
		}
		if ($count == 0) {
			$content .= $template->tableFullRow(Language::getSessText('Search.NoResults'));
		}
		$content .= $template->tableBottom();

		$title = Language::getSessText('Scm.SearchResultsTabTitle', array($count));



		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content
		);
	}

	/**
	 * Given a row in code_search_index, finds out what kind of match it is (class name, method name, variable name,
	 * comment)
	 */
	private function parseCodeSearchMatch($rs) {
		/*
		  const CODE_SEARCH_ELEMENT_CLASS = 1;
		  const CODE_SEARCH_ELEMENT_METHOD = 2;
		  const CODE_SEARCH_ELEMENT_VARIABLE = 3;
		  const CODE_SEARCH_ELEMENT_COMMENT = 4;
		 */

		if ($rs->getFetchMode() == ResultSet::FETCHMODE_ASSOC) {
			$class_name = $rs->getString('class_name');
			$method_name = $rs->getString('method_name');
			$variable_name = $rs->getString('variable_name');
			$comment = $rs->getString('comment');
		} else {
			$class_name = $rs->getString(6);
			$method_name = $rs->getString(7);
			$variable_name = $rs->getString(8);
			$comment = $rs->getString(9);
		}

		$fullname = '';

		if ($variable_name) {
			$kind = self::CODE_SEARCH_ELEMENT_VARIABLE;
			$full_name = $variable_name;
			if ($method_name)
				$full_name = $method_name . '::' . $full_name;
			if ($class_name)
				$full_name = $class_name . '::' . $full_name;
			$element_type = Language::getSessText('Scm.Search.VariableTitle');
		} elseif ($method_name) {
			$kind = self::CODE_SEARCH_ELEMENT_METHOD;
			$full_name = $method_name;
			if ($class_name)
				$full_name = $class_name . '::' . $full_name;
			$element_type = Language::getSessText('Scm.Search.MethodTitle');
		} elseif ($class_name) {
			$kind = self::CODE_SEARCH_ELEMENT_CLASS;
			$full_name = $class_name;
			$element_type = Language::getSessText('Scm.Search.ClassTitle');
		} else {
			$kind = self::CODE_SEARCH_ELEMENT_COMMENT;
			$full_name = '';
			$element_type = Language::getSessText('Scm.Search.CommentTitle');
		}

		return array('full_name' => $full_name, 'element_type' => $element_type);
	}

	public function searchByTags(GFSearch $search) {

		$words = $search->getWordsForQuery();
		$project_id = $search->getProjectId();
		$template = $search->getTemplate();

		//Clean key words from tsearch operators
		$words = preg_replace('/,+/', ',', $words);
		$tsearch_operators = array('(', ')', '!', '|', '&');
		$replacements = array('', '', '', '', ',');
		$words = str_replace($tsearch_operators, $replacements, $words);
		$words_arr = explode(',', $words);

		$wc = count($words_arr);
		if ($wc == 0) {
			return '';
		}
		for ($i = 0; $i < $wc; $i++) {
			$words_arr[$i] = "'" . $words_arr[$i] . "'";
		}
		$words = implode(',', $words_arr);

		if ($project_id) {
			$scm_prj_aux = " AND scm_commit.project_id=$project_id";
		} else {
			$scm_prj_aux = ' ';
		}


		$scm_perms_sql = "SELECT DISTINCT scm_commit.project_id AS project_id
		FROM scm_commit WHERE scm_commit.scm_commit_id IN
		(SELECT tag_entry.ref_id FROM tag_entry WHERE section='scmcommit'
		AND tag_name IN ($words) 
		GROUP BY tag_entry.ref_id
		HAVING COUNT(ref_id) =  $wc) $scm_prj_aux";

		$con = Propel::getConnection();
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($scm_perms_sql, ResultSet::FETCHMODE_NUM);


		$scm_aux = '';
		//Get project_scm and check permissions
		if ($rs->getRecordCount() > 0) {
			$prj_ids = array();
			while ($rs->next()) {
				$prj_ids[] = $rs->getInt(1);
			}

			$scm_prjs = ProjectScmPeer::retrieveByPks($prj_ids);

			//if user can read, add the scm to the tracker_aux sentences
			$scms = array();
			foreach ($scm_prjs as $scm_prj) {
				if ($scm_prj->userCanRead()) {
					$scms[] = $scm_prj->getProjectId();
				}
			}
			if (count($scms) == 0) {
				return null;
			} else {
				$scm_aux = ' AND scm_commit.project_id IN (' . implode(',', $scms) . ')';
			}
		}

		$scm_sql = "SELECT 'scmcommit' AS section, scm_commit.scm_commit_id AS ref_id,
		scm_commit.message_log AS description, project.unix_name AS prj_unix_name,
		project.project_name AS prj_name, plugin.plugin_name AS scm_name
		FROM scm_commit INNER JOIN project ON scm_commit.project_id = project.project_id
		INNER JOIN project_plugin ON project.project_id=project_plugin.project_id
		INNER JOIN plugin ON project_plugin.plugin_id = plugin.plugin_id
		WHERE scm_commit.scm_commit_id IN
		(SELECT tag_entry.ref_id FROM tag_entry WHERE section='scmcommit'
		AND tag_name IN ($words) 
		GROUP BY tag_entry.ref_id
		HAVING COUNT(ref_id) =  $wc) AND plugin.exclusive_type='scm' $scm_prj_aux $scm_aux ";

		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($scm_sql, ResultSet::FETCHMODE_ASSOC);


		$ret = null;
		if ($rs->getRecordCount() > 0) {
			$Lang = Session::getSession()->getLanguage();
			while ($rs->next()) {
				$trow = array();

				if (!$project_id) {
					$trow[] = '<a href="' . GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name')) . '">' . $rs->getString('prj_name') . '</a>';
				}
				$trow[] = $rs->getInt('ref_id');
				$trow[] = $Lang->getText('Tag.ScmCommitSection');
				$trow[] = '-';
				$trow[] = '<a href="' . GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name'), 'ScmCommitDetail', $rs->getString('scm_name'), '', array('scm_commit_id' => $rs->getInt('ref_id'))) . '">' . $rs->getString('description') . "</>";
				$ret .= $template->tableRow($trow);
			}
		}

		return $ret;
	}

	function daemonSshKeyChanged($params) {
		if (preg_match('/'.GFConf::get('system.usernameregex').'/',$params)) {
			$unixName = Validator::sanitizeString($params);
			GFDaemon::logDebugMessage("User $unixName: updating ssh keys");
			SystemConfigManager::updateSshKey($unixName);
		}
	}

	/**
	 * Updates scm commit filter on every related project
	 * @param int $params The scm commit filter id
	 */
	private function daemonScmCommitFilterSave($params) {
		$scmCommitFilterId = Validator::sanitizeInt($params);

		$c = new Criteria();
		$c->add(ScmCommitFilterProjectPeer::SCM_COMMIT_FILTER_ID, $scmCommitFilterId);
		/** @var ScmCommitFilterProject[] $projectFilters  */
		$projectFilters = ScmCommitFilterProjectPeer::doSelectJoinProject($c);

		foreach ($projectFilters as $pf) {

			$project = $pf->getProject();
			if (!$project->usesPlugin($this->getName())) {
				return;
			}


			$repo = SCMRepositoryFactory::getRepository($project);
			if (!$repo) {
				return;
			}
			$repo->setupRepository();
		}
	}

	function getProjectEvent($params) {
		if ($params['section'] != 'scmcommit') {
			return;
		}



		$project = ProjectPeer::retrieveByPk($params['project_id']);
		$description = $params['description'];
		// CVS commits for initial imports may be huge, so we trim it
		$description = StringUtils::trimString($description, 256);
		$activity_desc = $params['description']; //Language::getSessText('ProjectActivity.CommitDescription', array($description));

		$scmplugin = $project->getExclusiveTypePlugin('scm');
		if (is_null($scmplugin)) {
			return;
		}
		$body = '<a href="' . GFUrl::getURL()->projectURL($project, 'ScmCommitDetail', $scmplugin->getName(), '', array('scm_commit_id' => $params['ref_id'])) . '">' . $activity_desc . '</a>';

		$pr = new PluginResult($this);
		$pr->setResult($body);  // Mark the request as handled
		return $pr;
	}

	function migrateActivityRow($params) {
		if ($params['section'] == ScmPlugin::SECTION) {

			$project = ProjectPeer::retrieveByPk($params['project_id']);
			$description = $params['description1'];
			// CVS commits for initial imports may be huge, so we trim it
			$description = StringUtils::mbtrimString($description, 256);
			$activity_desc = Language::getSessText('ProjectActivity.CommitDescription', array($description));

			if (!is_object($project)) {
				$pr = new PluginResult($this);
				$pr->setResult('Project #' . $params['project_id'] . ' does not exist, not migrated');
				return $pr;
			}

			$scmplugin = $project->getExclusiveTypePlugin('scm');
			if (is_null($scmplugin)) {
				return;
			}
			$url = GFUrl::getURL()->projectURL($project, 'ScmCommitDetail', $scmplugin->getName(), '', array('scm_commit_id' => $params['ref_id']));

			$event = array(
				'project_id' => $params['project_id'],
				'section' => $params['section'],
				'ref_id' => $params['ref_id'],
				'user_id' => $params['user_id'],
				'activity_date' => $params['activity_date'],
				'description' => $activity_desc,
				'url' => $url,
				'activity_type' => $params['action'],
				'extended_data' => json_encode($params)
			);

			$result = ActionManager::notify('ACTIVITY_LOG_EVENT', $event);

			foreach ($result as $item) {
				if ($item->isError()) {
					return $item;
				}
			}

			$pr = new PluginResult($this);
			$pr->setResult($params['id']);  // Mark the request as handled
			return $pr;
		}
	}

}

?>
