<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Federico Tarantini      federico@gforgegroup.com
 * @date 2006-03-13
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once ("lib/external/jpgraph/jpgraph.php");
require_once ("lib/external/jpgraph/jpgraph_line.php");
require_once ("lib/external/jpgraph/jpgraph_canvas.php");

$session = Session::getSession();
$Lang = $session->getLanguage();

$Project = GFUrl::getUrl()->getProject();
$project_id = $Project->getProjectId();

$con = Propel::getConnection(TrackerPeer::DATABASE_NAME);
$stmt = $con->createStatement();

// Get filter data
//$user = (isset($_GET['user_id']) ? Validator::sanitizeInt($_GET['user_id']) : 0);
$begin = Validator::sanitizeDate($_GET['start_date']);
$end = Validator::sanitizeDate($_GET['end_date']);
$subprojects=((isset($_REQUEST['subprojects'])) ? Validator::sanitizeInt($_REQUEST['subprojects']) : 0 );

$all_ids = array();
if ($subprojects == 1) {
	$all_ids = $Project->getAllSubprojectIds();
	$all_ids[] = $project_id;
} else {
	$all_ids[] = $project_id;
}

/*
if ($user > 0) {
        $sqluser = "AND sc.user_id = '$user'";
} else {
        $sqluser = '';
}
*/

/*$sql = 'SELECT date_part(\'year\', sc.commit_date)::text || \'-\' || date_part(\'month\', sc.commit_date)::text AS month, u.unix_name, COUNT(sc.user_id) AS commits 
	FROM scm_commit sc, "user" u 
	WHERE sc.project_id IN ('.implode(',',$all_ids).') '; 
$sql .= 'AND sc.commit_date BETWEEN \''.$begin.'\' AND \''.$end.'\' 
	 AND sc.user_id = u.user_id 
	 GROUP BY u.unix_name, month';*/
	 
$sql = "SELECT 
			DATE_PART('year', sc.commit_date)::TEXT || '-' || DATE_PART('month', sc.commit_date)::TEXT AS month, 
				u.unix_name, COUNT(sc.user_id) AS commits 
		FROM scm_commit sc, \"user\" u 
		WHERE
			sc.project_id IN (".implode(',',$all_ids).") 
			AND sc.commit_date BETWEEN '$begin' AND '$end'
			AND sc.user_id = u.user_id
		GROUP BY u.unix_name, month";
$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

$raw_data  = array();
if ($rs != null && $rs->getRecordCount() > 0) {
	while($rs->next()) {
		$raw_data[] = array($rs->getString('month') => $rs->getInt('commits'));
	}
}

// Create an array with all the months of the given date range
$months = array();
$i = strtotime($begin);
while($i <= strtotime($end)) {
	$date = getdate($i);
	$months[$date['year'].'-'.sprintf("%02u",$date['mon'])] = 0;
	$i = mktime(0, 0, 0, ($date['mon']+1), 1, $date['year']);
}

// Create the array for passing to jpgraph
$result = $months;
foreach($raw_data as $key => $value) {
	foreach($value as $month => $commits) {
		if (isset($result[$month])) {
			$result[$month] += $commits;
		}
	}
}
/*
foreach($raw_data as $key => $value) {
	foreach ($value as $unix_name => $month_data) {
		foreach($month_data as $month => $commits) {
			$result[$unix_name][$month] = $commits;
		}
	}
}
*/

$font = $Lang->getFontProperties(FF_DEFAULT, FS_BOLD, 10);

// Need at least two months to plot
if (count($months) < 2) {
	// Not enough months -- just show a message and quit
	$graph = new CanvasGraph(450, 250);
	$graph->SetMargin(0,0,0,0);
	$graph->initFrame();
	$graph->setMarginColor("white");
 
	$text = new Text("Not enough data to plot - Needs at least two months", 200, 10);
	$text->align("center", "middle");
	$text->SetFont(FF_DEFAULT,FS_NORMAL, 7);
	$text->paragraphAlign("center");
	$text->stroke($graph->img);
	$graph->stroke();
	exit();
}
// Create the graph.
$graph = new Graph(450, 250);
$graph->SetScale('textlin');
$graph->img->SetMargin(40, 100, 15, 70);
$graph->title->Set($Lang->getText('ScmCommitsReport.CommitsOverTime'));
$graph->title->SetFont($font[0], $font[1], $font[2]);
$graph->setBox(false);
$graph->setFrame(false);

$font = $Lang->getFontProperties(FF_DEFAULT, FS_NORMAL, 7);
$graph->xaxis->SetTitle($Lang->getText('Common.Month'));
$graph->xaxis->SetTitleMargin(39);
$graph->xaxis->title->SetFont($font[0], $font[1], $font[2]);
$graph->xaxis->SetTickLabels(array_keys($months));
$graph->xaxis->SetLabelAngle(90);

$graph->yaxis->SetTitle($Lang->getText('ScmCommitsReport.Commits'), 'low');
$graph->yaxis->SetTitleMargin(25);
$graph->yaxis->title->SetFont($font[0], $font[1], $font[2]);
$graph->yaxis->scale->SetGrace(0);
$graph->yaxis->SetLabelMargin(1);

$graph ->legend->Pos( 0.8,0.1,"left" ,"center");

if (count($result) != 0) {
	$lineplot = new LinePlot(array_values($result));
	$lineplot->SetFillColor('#ADD8E6');
	$graph->Add($lineplot);
} else {
	$lineplot = new LinePlot(array_values($months));
	$graph->Add($lineplot);
}

if (ob_get_level()) ob_end_clean();
$graph->Stroke();
