<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version $Id$
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('system/Cronjob.php');
require_once('core/CronHistory.php');
require_once('core/Project.php');
require_once('plugins/scm/lib/ProjectScm.php');
require_once('GFConf.php');

class CreatePserverConf extends Cronjob {
	private $pserverParametersFile;

	public function __construct() {
		$this->pserverParametersFile = SystemUtils::buildFilePath(GFConf::get('system.varDir'), 'cvspserver.params');
		parent::__construct();
	}


	public function run($useSSH = false) {
		$cvsroot = GFConf::get('scmcvs.localCVSRoot');
		if (!is_dir($cvsroot)) {
			echo 'CVSROOT dir '.$cvsroot.' doesn\'t exist'."\n";
			exit(1);
		}
	
		// Create the string to be passed as a parameter to the "cvs pserver" script
		$allow_root_params = '';
		
		$dh = opendir($cvsroot);
		while ($dir = readdir($dh)) {
			$repository = SystemUtils::buildFilePath($cvsroot, $dir);
			
			if (preg_match('/^\\./', $dir) || $dir == 'cvs-locks' || !is_dir($repository)) {
				continue;
			}
			
			$project = ProjectPeer::retrieveByUnixName($dir);
			if (!$project) {
				continue;
			}
			$project_scm = ProjectScmPeer::retrieveByPk($project->getProjectId());
			if (!$project_scm) {
				continue;
			}
			
			if (!($project->getIsPublic() && $project_scm->getEnableAnonread())) {
				continue;
			}
			
			$allow_root_params .= '--allow-root='.$repository.' ';
		}
		closedir($dh);
		
		// Now create the pserver parameters file
		$cvspserver_parameters_contents = GFConf::get('scmcvs.cvsbin').' -f \\'."\n";
		$cvspserver_parameters_contents .= " ".$allow_root_params." pserver";

		if (!file_put_contents($this->pserverParametersFile, $cvspserver_parameters_contents)) {
			$this->logCron('Couldn\'t write to ' . $this->pserverParametersFile);
		}
		
		$this->commitLog();
	}
}

$cronjob = new CreatePserverConf();
$cronjob->run();
?>
