<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: cleanup.php 489 2006-02-26 20:04:55Z tperdue $
 * @author	Marcelo Mottalli <mottalli@gforgegroup.com>
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once("system/Cronjob.php");
require_once("system/SystemConfigManager.php");
require_once('system/SystemConfigManagerSolaris.php');
require_once('core/CronHistory.php');
require_once('plugins/scmcvs/lib/CVSRepository.php');
require_once('plugins/scm/lib/ProjectScm.php');
require_once('plugins/scm/lib/SCMRepositoryFactory.php');
require_once('core/Project.php');
require_once('core/ProjectPlugin.php');
require_once('core/Plugin.php');
require_once('core/UserUnix.php');
require_once('core/UnixShell.php');
require_once('core/DeletedProjectPeer.php');

class UserGroupCronjob extends Cronjob {
	const DEBUG = false;		// Set this to true for testing purposes, system files won't get overwritten
	
	private $configman;

	public function __construct() {
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}

	public function run() {
		// If the permissions are handled by an external NSS like nss-pgsql, don't update the system files
		if (GFConf::get('system.useNSS')) {
			$this->logCron('NSS active, skipping user creation...');
			return;
		}
		
		
		if (strtolower(php_uname('s')) == 'sunos') {
		    $this->configman = new SystemConfigManagerSolaris(self::DEBUG);
		} else {
		    $this->configman = new SystemConfigManager(self::DEBUG);
		}
		
		$this->configman->purgeGForgeGroups();
		$this->configman->purgeGForgeUsers();
		
		/*
		$svnSSH = GFConf::get('scmsvn.useSSH');
		
		if (GFConf::varExists('scmgit.useSSH')) {
			$gitSSH = GFConf::get('scmgit.useSSH');
		} else {
			$gitSSH = false;
		}
		
		// first, get all the CVS and SVN configurations
		$c = new Criteria();
		$c->addJoin(PluginPeer::PLUGIN_ID, ProjectPluginPeer::PLUGIN_ID);
		$c->addJoin(ProjectPluginPeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
		$criterion = $c->getNewCriterion(PluginPeer::PLUGIN_NAME, 'scmcvs');
		if ($svnSSH) {
			$criterion->addOr($c->getNewCriterion(PluginPeer::PLUGIN_NAME, 'scmsvn'));
		}
		if ($gitSSH) {
			$criterion->addOr($c->getNewCriterion(PluginPeer::PLUGIN_NAME, 'scmgit'));
		}
		
		$c->add($criterion);
		// Process only active projects
		$c->addJoin(ProjectScmPeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
		$c->add(ProjectPeer::STATUS, Project::STATUS_ACTIVE);
		
		$project_scms = ProjectScmPeer::doSelectJoinProject($c);		
*/
		$c = new Criteria();
		$c->add(ProjectScmPeer::USE_SSH, true);
		$c->addJoin(ProjectScmPeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
		$c->add(ProjectPeer::STATUS, Project::STATUS_ACTIVE);
		$project_scms = ProjectScmPeer::doSelectJoinProject($c);			
		foreach ($project_scms as $project_scm) {
			$project = $project_scm->getProject();
			echo $project->getUnixName();

			if(!($project->usesPlugin('scmcvs') ||
				$project->usesPlugin('scmsvn') || 
				$project->usesPlugin('scmgit'))
				){
				continue;
			}
			
			if (GFConf::get('system.splitScmServers')) {
				// If we're splitting the SCM servers, check that only the users and groups
				// for this server's projects are created
				$scm_server_id = $project_scm->getScmServerId();
				
				// Check that the repository's server ID is this server ID (if set)
				// 4 cases: both CVS and SVN server ID are set, only CVS, only SVN or none
				if (GFConf::varExists('scmcvs.scmServerId') && GFConf::varExists('scmsvn.scmServerId')) {
					if ($scm_server_id != GFConf::get('scmcvs.scmServerId') && $scm_server_id != GFConf::get('scmsvn.scmServerId')) {
						continue;
					}
				} elseif (GFConf::varExists('scmcvs.scmServerId')) {
					if ($scm_server_id != GFConf::get('scmcvs.scmServerId')) {
						continue;
					}
				} elseif (GFConf::varExists('scmsvn.scmServerId')) {
					if ($scm_server_id != GFConf::get('scmsvn.scmServerId')) {
						continue;
					}
				} else {
					// Neither scmsvn.scmServerId nor scmcvs.scmServerId are set. Assume
					// the project is hosted on this server
				}
			}
			
			// Create the entries for the group in /etc/group and the needed entries
			// for the users in /etc/passwd
			$this->createProjectUnixGroup($project_scm);
			$this->assignUsersToGroup($project_scm);
		}

		$this->configman->writeChanges();
		
	}

	private function createProjectUnixGroup(ProjectScm $project_scm) {
		$project = $project_scm->getProject();
		if ($this->configman->groupExists($project->getUnixName())) {
			// Group already exists... do nothing
			return;
		}
		
		$group_name = $project->getUnixName();
		$group_id = $project_scm->getUnixGroupId();
		$this->configman->createGForgeGroup($group_name, $group_id);
	}
	
	private function assignUsersToGroup(ProjectScm $project_scm) {
		$project = $project_scm->getProject();
		$users = $project->getMembers();
		foreach ($users as $user) {
			$up = $project->getUserPermission($user);
			if ($up->check('scm', 0, ScmPlugin::READ)) {
				// Only add the user if it has at least read permissions
				
				if (!GFConf::get('system.usesNis') && !$this->configman->userExists($user->getUnixName())) {
					// see if the UNIX account is configured for this user
					// IMPORTANT: Note that the entry is automatically created
					// if it isn't found
					$user_unix = UserUnixPeer::retrieveByPkOrCreate($user->getUserId());
					if (is_null($user_unix)) {
						// shouldn't happen!
						continue;
					}
					
					$user_shell = $user_unix->getUnixShell();
					
					$uid = $user_unix->getUnixUid();
					$gid = $user_unix->getUnixGid();
					if (!$gid) $gid = $uid;
					$unix_password = $user->getPasswordCrypt();
					$home = SystemUtils::buildFilePath(GFConf::get('system.homeDir'), '/'.$user->getUnixName());
					$shell = $user_shell->getPath();
					$gcos = $user->getUnixName();
					
					if (!$this->configman->createGForgeUser($user->getUnixName(), $uid, $unix_password, $gid, $home, $shell, $gcos)) {
						$this->logCron('Couldn\'t create user '.$user->getUnixName());
					}
				}
				
				$this->configman->addUserToGroup($user->getUnixName(), $project->getUnixName());
			}
		}
		
		// Must make sure to add the apache user to the group if it's private
		// in order to browse it
		if (!$project->isPublic() || !$project_scm->getEnableAnonread()) {
			$apache_user = GFConf::get('system.apacheUser');
			$this->configman->addUsertoGroup($apache_user, $project->getUnixName());
		}
		
		// Must add the "anonymous" user to the group so we can make pserver
		// checkouts
		if ($project_scm->getEnableAnonread()) {
			if (SystemUtils::userExists('anonymous')) {
				$this->configman->addUserToGroup('anonymous', $project->getUnixName());
			} else {
				$this->logCron('IMPORTANT: UNIX user "anonymous" doesn\'t exist, anonymous CVS/SVN checkout for project "'.$project->getUnixName().'" will be disabled until the user is created');
			}
		}
		
		$this->commitLog();
	}
}

$cronjob = new UserGroupCronjob();
$cronjob->run();
?>
