<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('GFUrl.php');
require_once('plugins/scmcvs/lib/html/ScmCVSTemplate.php');
require_once('plugins/scmcvs/lib/CVSRepository.php');


$project = GFUrl::getUrl()->getProject();
$cvsplugin = $project->getPluginByName('scmcvs');
if (is_null($cvsplugin)) {
	throw new InvalidPluginException('scmcvs');
}

$session = Session::getSession();
$lang = $session->getLanguage();
$unix_name = 'developername';
if ($session->isLoggedIn()) {
	$user = $session->getUser();
	if (is_object($user)) {
		$unix_name = $user->getUnixname();
	}
}

$template = new ScmCVSTemplate();
$params = array(
		'project' => $project,
		'title' => $lang->getText('Scmcvs.AccessInfoTitle')
	);
$template->header($params);

// See if SCM is configured for this project
$repos = new CVSRepository($project);
if ($repos->isConfigured()) {
	$projectScm = $repos->getScmConfiguration();

	if (!$projectScm->getControlled()) {
		//@todo	This will be used when we add support for external SCM
		//die("UNSUPPORTED");
	}
	
	$root = GFConf::get('scmcvs.localCVSRoot');
	if (GFConf::get('system.splitScmServers')) {
		$hostname = $projectScm->getScmServer()->getHostname();
	} else {
		$hostname = GFConf::get('system.hostName');
	}
	
	// full CVSROOT is something like /cvsroot/project_name
	$fullRoot = SystemUtils::buildFilePath($root, '/'.$project->getUnixName());
	
	$custom_access_text = $projectScm->getAccessText();
	if (!empty($custom_access_text)) {
?>
<p>
	<h3><?php echo $lang->getText('Scmcvs.AccessTextTitle') ?></h3>
</p>
<p>
	<?php echo $custom_access_text ?>
</p>
<?php
	}

	if ($projectScm->getControlled()) {		// Only display the access info if access is controlled
		if ($project->getIsPublic() && $projectScm->getEnableAnonread()) {
?>
<p>
	<h3><?php echo $lang->getText('Scmcvs.AnonymousAccess'); ?></h3>
</p>
<p>
	<?php echo $lang->getText('Scmcvs.AnonymousAccessDescription'); ?>
</p>
<br /><br />
<p>
	<?php
		echo '<tt>cvs -d :pserver:anonymous@'.$hostname.':'.$fullRoot.' login</tt><br />'."\n";
		echo '<tt>cvs -d :pserver:anonymous@'.$hostname.':'.$fullRoot.' checkout <i>modulename</i></tt><br />'."\n";
	?>
</p>
<?php 	} /*if ($projectScm->getEnableAnonread()) */ ?>
<p>
	<h3><?php echo $lang->getText('Scmcvs.DeveloperAccess'); ?></h3>
</p>
<p>
	<?php echo $lang->getText('Scmcvs.DeveloperAccessDescription'); ?>
</p>
<br /> <br />
<p>
	<tt>export CVS_RSH=ssh</tt><br />
	<?php 
		echo '<tt>cvs -d :ext:<i>'.$unix_name.'</i>@'.$hostname.':'.$fullRoot.' checkout <i>modulename</i></tt><br />'."\n";
	?>
</p>

<?php
	} /* $projectScm->getControlled() */
} else {		// SCM is not configured for this project
?>
	<a href="<?php echo GFUrl::getUrl()->projectURL($project, '', 'scmcvs', 'admin'); ?>"><?php echo $lang->getText('Scm.Admin'); ?></a>
	
	<p><h3><?php echo $lang->getText('Scm.AdminTitle'); ?></h3></p>
	
	<p><?php echo $lang->getText('Scm.NotConfigured') ?></p>

<?php
}
$template->footer($params);
?>
