<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code shows a form for editing ProjectScm
//
require_once('plugins/scm/lib/ProjectScmPeer.php');
require_once('plugins/scm/lib/ProjectScm.php');
require_once('plugins/scm/lib/ScmPreferencePeer.php');
require_once('plugins/scm/lib/ScmPreference.php');
require_once('plugins/scm/lib/ScmServer.php');
require_once('plugins/scmcvs/lib/CVSRepository.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('GFUrl.php');
require_once('html/HTMLWidget.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

$project = GFUrl::getUrl()->getProject();
$cvsplugin = $project->getPluginByName('scmcvs');
if (is_null($cvsplugin)) {
	throw new InvalidPluginException('scmcvs');
}

require_once('plugins/scmcvs/lib/html/ScmCVSAdminTemplate.php');
$theme = new ScmcvsAdminTemplate();

$repos = new CVSRepository($project);
$ProjectScm = $repos->getScmConfiguration();
// $ProjectScm can be null if SCM is not configured for this project
if ( (!is_object($ProjectScm) && !$project->getPermission()->isAdmin()) || (is_object($ProjectScm) && !$ProjectScm->userCanUpdate()) ) {
	throw new PermissionDeniedException();
}

$params = array(
		'project'	=> $project,
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('ProjectScm.EditPageTitle'),
		);
$theme->header($params);

// Create the SCM preferences on the fly if needed
$scm_pref = ScmPreferencePeer::retrieveByPk($project->getProjectId());
if (is_null($scm_pref)){
	$scm_pref = new ScmPreference();
	$scm_pref->setProjectId($project->getProjectId());
	$scm_pref->setRequireTrackerItemId(false);
	$scm_pref->save();
}


?>

<form action="<?php echo GFUrl::getUrl()->currentBaseURL('ProjectScmEditAction') ?>" method="POST">
	<table border="0">
		<tr>
			<td> <input type="checkbox" name="require_tracker_item_id" <?php if ($scm_pref->getRequireTrackerItemId() == true) echo 'checked="checked"'; ?> /> <strong><?php echo $Lang->getText('ProjectScm.RequireTrackerItemId');?></strong><br /></td>
		</tr>
		<tr>
			<td><input type="checkbox" name="mail_commits" <?php if ($scm_pref->getMailCommits() == true) echo 'checked="checked"'; ?> /> <strong><?php echo $Lang->getText('ProjectScm.MailCommits');?></strong><br /></td>
		</tr>
		<tr>
			<td>
				<br />
				<strong><?php echo $Lang->getText('ProjectScm.AccessTextName'); ?>:</strong><br />
				<?php 
				$CKEditor = new GFCKEditor(GFCKEditor::FULL);
				$CKEditor->config['width'] = 700;
				$CKEditor->config['height'] = 300;
				$accessText= !is_null($ProjectScm) ? $ProjectScm->getAccessText() : '';
				$CKEditor->editor('access_text', $accessText);
				$CKEditor->setConnectorURLs($project->getProjectId());
				?>
			</td>
		</tr>
		<tr>
			<td>
				<br />
				<strong><?php echo $Lang->getText('ProjectScm.BrowseTextName'); ?>:</strong><br />
				<?php
				$CKEditor = new GFCKEditor(GFCKEditor::FULL);
				$CKEditor->config['width'] = 700;
				$CKEditor->config['height'] = 300;
				$browseText= !is_null($ProjectScm) ? $ProjectScm->getBrowseText() : '';
				$CKEditor->editor('browse_text', $browseText);
				$CKEditor->setConnectorURLs($project->getProjectId());
				?>
			</td>
		</tr>
		<?php if (!is_null($ProjectScm) && !$ProjectScm->getControlled()) : ?>
		<tr>
			<td>
				<br />
				<strong><?php echo $Lang->getText('ProjectScm.ExternalBrowseUrlName'); ?>:</strong> 
			<input type="text" name="external_browse_url" value="<?php echo htmlspecialchars($ProjectScm->getExternalBrowseUrl()) ?>" size="40">
			</td>
		</tr>
		<?php endif; ?>
		<?php if (!is_null($ProjectScm)) : ?>
		<tr>
			<td><strong><?php echo $Lang->getText('ProjectScm.ControlledTitle'); ?>:</strong><br />
				<?php
					if ($ProjectScm->getControlled()) {
						// Repository controlled by GForge
						echo $Lang->getText('ProjectScm.ControlledDescription', array(GFUrl::getUrl()->currentBaseUrl('ScmSetControlled', array('controlled' => 0))));
					} else {
						// Repository NOT controlled by GForge
						echo $Lang->getText('ProjectScm.NotControlledDescription', array(GFUrl::getUrl()->currentBaseUrl('ScmSetControlled', array('controlled' => 1))));
					}
				?>
			</td>
		</tr>
		<?php endif; ?>
		<tr>
		    <td align="center"><?php echo HTMLWidget::button("submit","" .   $Lang->getText('ProjectScmEditForm.submit') . " ","submit","","")?></td>
		</tr>
	</table>
</form>
<?php
$theme->footer($params);
?>
