<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: aclcheck.php 816 2006-04-18 17:36:20Z marcelo $
 * @author Marcelo Mottalli	marcelo@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('initialize.php');
require_once('GFConf.php');
require_once('SystemUtils.php');
require_once('core/Project.php');
require_once('plugins/scm/lib/ScmPreference.php');
require_once('plugins/scmgit/lib/GitReceive.php');

// Get the project from the current working directory
$cwd = getcwd();
$path = explode('/', $cwd);
$project_name = $path[count($path)-1];
$project = ProjectPeer::retrieveByUnixName($project_name);
if (!$project) {
	echo 'Invalid project name: '.$project_name."\n";
}

$repository = new GITRepository($project);

// Get the committing user
if ($repository->getAccessType() == GITRepository::ACCESS_TYPE_SSH) {
	// Retrieve the user name from the environment
	$user_arr = posix_getpwuid(posix_geteuid());
	$user_name = $user_arr['name'];
} else {
	// Committing through HTTP
	$user_name = (isset($_ENV["REMOTE_USER"]) ? $_ENV["REMOTE_USER"] : "");
	//For some reason, this seems to work on gforge.com (git 1.7.4) but it doesn't
	//on local Centos6 with official git packages (git 1.7.1), so if $user_name has not been
	// set, we use getenv()
	if (!$user_name) {
		$user_name = getenv("REMOTE_USER");
	}
} 

if (!$user_name) {
	//This shouldn't happen, git should always provide an username
	echo "Couldn't retrieve user name from the environment, the commit information will not be stored in the database\n";
	exit(1);
}

if ($user_name == 'root') {
	exit(0);
}

// Start the session as the user that's committing
$user = UserPeer::retrieveByUnixName($user_name);
if (is_null($user)) {
	echo 'Invalid user name: '.$user_name."\n";
	exit(1);
}

$gitReceive = new GitReceive($project, $user, GitReceive::POST_SCRIPT);
echo "* Starting Gforge post-receive script.\n\n";

echo "\tCreating commit associations, please wait...\n";
$gitReceive->parseIds();
$gitReceive->parseAssociations();


// Now, fire up the mail notifications (if configured), we don't ignore merges this time
$preferences = $repository->getScmPreferences();
if (!is_null($preferences) && $preferences->getMailCommits() && $project->usesPlugin('mailman')) {
	$gitReceive->sendCommitEmail();
}

echo "\nAll updates were successfully processed.\n\n";

exit(0);
?>

