<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

/**
 * 
 *
 * Leonel Corso <leonelcorso@yahoo.com.ar>
 *
 * @version   $Id$
 */
require_once('lib/initialize.php');
require_once('system/Cronjob.php');
require_once('core/ProjectPeer.php');
require_once('core/PluginPeer.php');
require_once('core/DeletedProjectPeer.php');
require_once('core/ProjectPluginPeer.php');
require_once('plugins/scm/lib/ProjectScmPeer.php');
require_once('plugins/scmsvn/lib/SVNRepository.php');
require_once('GFConf.php');
require_once('plugins/scmsvn/lib/exceptions/SVNException.php');

class CreateSVNRepositoriesCronjob extends Cronjob {
	private $svndir_prefix,$plugins_path,$repositoryPath;
	const PLUGIN_NAME = "scmsvn";
	
	public function __construct() {
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}
	
	public function run() {
		$this->cleanup();
		
		$this->svndir_prefix = GFConf::get('scmsvn.localSVNRoot');
		$this->plugins_path = GFConf::get('system.pluginsPath');

		// Loop through and create/update each repository for every project that uses SCMSVN plugin

		
		if (is_dir($this->svndir_prefix)) {
			$this->updateSvnRepositories();
		} else {
			if (is_file($this->svndir_prefix)) {
				$this->logCron($this->svndir_prefix." exists but is a file");
				$this->commitLog();
				exit();
			} else {
				if (mkdir($this->svndir_prefix)) {
					//need to update group permissions using chmod
					$this->updateSvnRepositories();
				} else {
					$this->logCron("Unable to create ".$this->svndir_prefix." directory");
					$this->commitLog();
					exit();
				}
			}
		} 
		
		$this->commitLog();
	}

	/**
	 * Change the repository's permissions in order to enable anonymous access
	 */
	public function enableAnonAccess() {
		SystemUtils::execute('chmod -R o+r '.$this->repositoryPath, $output);
		// Anonymous can't write
		SystemUtils::execute('chmod -R o-w '.$this->repositoryPath, $output);
		SystemUtils::execute('chmod o+x '.$this->repositoryPath, $output);
		// Allow access to directories
		SystemUtils::execute('find '.$this->repositoryPath.' -type d -exec chmod o+x {} \;', $output);
	}
	
	/**
	 * Change the repository's permissions in order to disable anonymous access
	 */
	public function disableAnonAccess() {
		SystemUtils::execute('chmod -R o-r '.$this->repositoryPath, $output);
		SystemUtils::execute('chmod -R o-w '.$this->repositoryPath, $output);
		SystemUtils::execute('chmod o-x '.$this->repositoryPath, $output);
		// Disable access to directories
		SystemUtils::execute('find '.$this->repositoryPath.' -type d -exec chmod o-x {} \;', $output);
	}

	public function updateSvnRepositories() {
		// let's get all projects that use the plugin
		$c = new Criteria();
		$c->add(PluginPeer::PLUGIN_NAME, self::PLUGIN_NAME);
		$project_plugins = ProjectPluginPeer::doSelectJoinPlugin($c);

		foreach ($project_plugins as $project_plugin) {
			
			
			$project = $project_plugin->getProject(); // get the project
			$svnplugin = $project->getPluginByName(self::PLUGIN_NAME);
			
			$repos = new SVNRepository($project);
			// SVN not configured for this project
			if (is_null($svnplugin) || !$repos->isConfigured()) {
				continue;
			}
			try{
				if($project->isActive()){
					$repos->setupRepository();
				}
			} catch (SVNException $e){
				$this->logCron($e->getMessage());
			}
		}
		
	}
	
	public function cleanup() {
		$c = new Criteria();	
		$c->add(DeletedProjectPeer::ISDELETED, 0);	
		$deletedProjects = DeletedProjectPeer::doSelect($c);
		$svnroot = GFConf::get('scmsvn.localSVNRoot');

		$deletedRoot = SystemUtils::buildFilePath($svnroot, '/.deleted');
		//The trailing slash in the dir name is due to the way is_dir checks
		//directories, useful for unit testing
		if (!is_dir($deletedRoot)) {
			mkdir($deletedRoot);
		}

		$output = array();
		
		foreach ($deletedProjects as $deletedProject){
			$deletedUnixName = $deletedProject->getUnixGroupName();

			$reposPath = SystemUtils::buildFilePath($svnroot, $deletedUnixName);
			if (!is_dir($reposPath)) {
				// Repository doesn't exist
				continue;
			}

			$deletedPath = SystemUtils::buildFilePath($deletedRoot, '/'.$deletedUnixName);
			SystemUtils::execute('mv -f '. $reposPath . ' ' . $deletedPath, $output);
			$deletedProject->delete();
		}	

		SystemUtils::execute('chown -h -R root:root '.$deletedRoot, $output);
		SystemUtils::execute('chmod -R o-rwx '.$deletedRoot, $output);
	}
}


if(!defined("TEST_RUN")){
	$cronjob = new CreateSVNRepositoriesCronjob();
	$cronjob->run();
};
?>
