<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: ProjectScmEdit.php 947 2006-05-06 20:50:52Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code shows a form for editing ProjectScm
//
require_once('plugins/scm/lib/ProjectScmPeer.php');
require_once('plugins/scm/lib/ProjectScm.php');
require_once('plugins/scm/lib/ScmServer.php');
require_once('plugins/scmvss/lib/VSSRepository.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('GFUrl.php');
require_once('html/HTMLWidget.php');
require_once('plugins/scm/lib/ScmPreferencePeer.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

$project = GFUrl::getUrl()->getProject();
$vssplugin = $project->getPluginByName('scmvss');
if (is_null($vssplugin)) {
	throw new InvalidPluginException('scmvss');
}

require_once('plugins/scmvss/lib/html/ScmVSSAdminTemplate.php');
$theme = new ScmVSSAdminTemplate();

if(!isset($ProjectScm)){
	$repos = new VSSRepository($project);
	$ProjectScm = $repos->getScmConfiguration();
}

// $ProjectScm can be null if SCM is not configured for this project
if ( (!is_object($ProjectScm) && !$project->getPermission()->isAdmin()) || (is_object($ProjectScm) && !$ProjectScm->userCanUpdate()) ) {
	throw new PermissionDeniedException();
}

$params = array(
		'project'	=> $project,
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'action' => isset($action) ? $action : '',
		'title' => $Lang->getText('ProjectScm.EditPageTitle'),
		'selectedbottomtab'	=> 'scmvss'
		);
$theme->header($params);

// show the available VSS servers
$c = new Criteria();
$c->add(ScmServerPeer::SCM_TYPE, 'VSS');
$servers = ScmServerPeer::doSelect($c);
if (!is_null($ProjectScm)) {
	$selected = $ProjectScm->getScmServerId();
} else {
	$selected = null;
}
if (count($servers) == 0) {
	//no SCM servers available
	$scm_server = $Lang->getText('Scm.NoServerAvailable');
} elseif (count($servers) == 1) {
	$server = $servers[0];
	$scm_server  = $server->getHostname()."\n";
	$scm_server .= '<input type="hidden" name="scm_server_id" value="'.$server->getScmServerId().'">';
} elseif (count($servers) > 1) {
	$scm_server = HTMLWidget::select('scm_server_id', $servers, $selected, null, false);
}

?>
<form action="<?php echo GFUrl::getUrl()->currentBaseURL('ProjectScmEditAction') ?>" method="POST">
	<table border="0">
		<!--tr>
			<td><strong><?php echo $Lang->getText('Scmvss.Hostname');?></strong><span class="required">*</span></td>
			<td><?php echo $scm_server;?></td>
		</tr-->
		<tr>
			<td><strong><?php echo $Lang->getText('Scmvss.Hostname');?></strong>:<span class="required">*</span><br />
			<input type="text" name="hostname" value="<?php echo (!is_null($ProjectScm)) ? htmlspecialchars($ProjectScm->getHostname()) : '';?>" /></td>
		</tr>
		<tr>
			<td><strong><?php echo $Lang->getText('Scmvss.RootDir');?>:</strong> (i.e. \project_name\Database.ini)<br />
			<input type="text" name="root_dir" value="<?php echo (!is_null($ProjectScm)) ? htmlspecialchars($ProjectScm->getRootDirectory()) : '';?>" /></td>
		</tr>
		<tr>
			<td><strong><?php echo $Lang->getText('ProjectScm.AccessTextName'); ?>:</strong><br />
		    	<div><input type="hidden" id="access_text" name="access_text" value="<?php if (!is_null($ProjectScm)) echo htmlspecialchars($ProjectScm->getAccessText()); ?>" /><input type="hidden" id="access_text___Config" value="" /><iframe id="access_text___Frame" src="/fckeditor/editor/fckeditor.html?InstanceName=access_text&amp;Toolbar=GForge" width="700" height="300" frameborder="no" scrolling="no"></iframe></div>	
			</td>
		</tr>
		<tr>
			<td><strong><?php echo $Lang->getText('ProjectScm.BrowseTextName'); ?>:</strong><br />
		    	<div><input type="hidden" id="browse_text" name="browse_text" value="<?php if (!is_null($ProjectScm)) echo htmlspecialchars($ProjectScm->getBrowseText()); ?>" /><input type="hidden" id="browse_text___Config" value="" /><iframe id="browse_text___Frame" src="/fckeditor/editor/fckeditor.html?InstanceName=browse_text&amp;Toolbar=GForge" width="700" height="300" frameborder="no" scrolling="no"></iframe></div>	
			</td>
		</tr>
		<tr>
			<td><strong><?php echo $Lang->getText('ProjectScm.ExternalBrowseUrlName');?>:</strong><br />
			<input type="text" name="external_browse_url" size="40" value="<?php echo (!is_null($ProjectScm)) ? htmlspecialchars($ProjectScm->getExternalBrowseUrl()) : '';?>" /></td>
		</tr>
		<tr>
			<td align="center"><?php echo HTMLWidget::button("submit","" .   $Lang->getText('ProjectScmEditForm.submit') . " ","submit","","")?></td>
		</tr>
	</table>
</form>

<?php
$theme->footer($params);
?>
