<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code creates a new Snippet and saves it to the 
// database
//
require_once('plugins/snippet/lib/Snippet.php');
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');
require_once('core/Filesystem.php');
require_once('html/TroveRenderer.php');

Validator::validatePOSTAction();

$Snippet = new Snippet();

$Lang = Language::getSessLang();
$errors = array();

try {
	//$Snippet->setCreatedBy( Validator::sanitizeInt($_POST['created_by']) );
	$invalid_arr = array();
	$name = Validator::sanitizeString($_POST['snippet_name']);
	if (strlen($name)==0) {
		$invalid_arr[] = 'Empty Name';
	}
	if (!isset($_POST['trove_programminglanguage'])) {
		$invalid_arr[] = Language::getSessText('Snippet.InvalidProg');
	}	
	if (!isset($_POST['trove_topic'])) {
		$invalid_arr[] = Language::getSessText('Snippet.InvalidTrove');
	}
	if (strlen($_POST['snippet_description'])==0) {
		$invalid_arr[] = Language::getSessText('Snippet.InvalidDesc');
	}
	$Snippet->setSnippetName( $name );
	$Snippet->setSnippetDescription( Validator::sanitizeString($_POST['snippet_description']) );
	$Snippet->setSection('snippet');
	$Snippet->setRefId(0);
	$Snippet->setCreatedBy($session->getUser()->getUserId());
	if (count($invalid_arr)>0) {
		throw  new InvalidDataException($invalid_arr);
	}
	$con = Propel::getConnection(SnippetPeer::DATABASE_NAME);
	$con->begin();
	$Snippet->save();
	
	//
	//      handle the files
	//
	for ($i=0; $i<5; $i++) {
		$error=$_FILES['addfile']['error'][$i];
		$filename=$_FILES['addfile']['name'][$i];

		if ($error > 0 && $error != 4) {
			$fileerrors = StringUtils::getFileUploadErrorString($error, $filename);
			throw new InvalidDataException($fileerrors);
			continue;
		}
		

		$tmp_name=$_FILES['addfile']['tmp_name'][$i];
		if (!is_uploaded_file($tmp_name)) {
			continue;
		}
		$size=$_FILES['addfile']['size'][$i];
		$type=$_FILES['addfile']['type'][$i];

		try {
			$fs = new Filesystem();
			$fs->setSection(Snippet::SECTION);
			$fs->setRefId($Snippet->getSnippetId());
	
			$fs->setFileName( $filename );
			//$fs->setFileNameSafe();
			$fs->setFileType( $type );
			$fs->setFileSize( $size );
			$fs->setPostedBy( $session->getUser()->getUserId() );
			$fs->save();
			$fs->setFileDataFromPath( $tmp_name );
		} catch (Exception $e) {
			// rollback if we couldnt create a file (write permissions)
			$con->rollback();
			$err = array($e->getMessage());
			throw new InvalidDataException($err);
		}
	}
	$con->commit();
	
	// save trove information
	$renderer = new TroveRenderer();
	$renderer->setSection('snippet');
	$renderer->setRefId($Snippet->getSnippetId());
	$renderer->savePOSTData();
	
	GFUrl::redirect(GFUrl::getUrl()->currentBaseURL('SnippetBrowse'), array('id'=>$Snippet->getSnippetId(),'feedback' => $Lang->getText('Snippet.AddFeedback')));
} catch (InvalidDataException $ex) {
	$errors = array_merge($errors, $ex->getErrors());		// attach object errors to validation errors
	$feedback = '* '.implode('<br />* ', $errors);
	include('SnippetAdd.php');

}
?>
