<?php

require_once('initialize.php');
require_once('MailParser.php');
require_once('session/Session.php');
require_once('core/User.php');
require_once('plugins/tracker/lib/TrackerItem.php');


$session = Session::getSession();

$mail = new MailParser("php://stdin");

if ($mail->isAutoResponse()) {
	//Ignore autoresponse emails
	exit(0);
}
$from = $mail->getFrom();
$body = $mail->getBody();
$subject = $mail->getSubject();
$tracker_item_id = $mail->getMessageIDFromAddress();
$mail_message_id = $mail->getMessageId();
$users = UserPeer::retrieveByEmail($from);


try {
	if (count($users) > 1) {
		throw new Exception(Language::getSessText('Gateway.TooManyUsers', array($from)));
	}

	if (!count($users)) {
		throw new Exception(Language::getSessText('Gateway.NoUser', array($from)));
	}

	$user = $users[0];
	$session->setUser($user);

	$trackerItem = TrackerItemPeer::retrieveByPk($tracker_item_id);

	if (is_null($trackerItem)) {
		throw new InvalidDataException(Language::getSessText('TrackerItem.InvalidId'));
	}

	$project = $trackerItem->getTracker()->getProject();
	if (!$project || !$project->isActive() || !$project->usesPlugin('tracker')) {
		throw new PermissionDeniedException('You don\'t have permissions to access this project');
	}

	if (!$trackerItem->userCanUpdate()) {
		throw new  PermissionDeniedException('You don\'t have permissions to update this tracker item');
	}

	$clean_body = $mail->getBodyQuoteRemoved('>',$body);

    if ($mail->getBodyContentType() == MailParser::CONTENT_TYPE_HTML) {
	    $str = new StringUtils(StringUtils::BASIC_HTML);
        $text = $str->cleanHtml($clean_body);
	} else {
        $text = StringUtils::HTMLFromPlainString($clean_body);
	}

	$tim = new TrackerItemMessage();
	$tim->setBody($text);
	$tim->setSubmittedBy($user->getUserId());
	$tim->setTrackerItem($trackerItem);
	$tim->save();

  //Update waiting for based on user
  $trackerItem->updateWaitingFor($user);
  $trackerItem->save();

	if (GFConf::get('system.useDaemon')) {
		$attachments = $mail->getAttachments();

		foreach ($attachments as $attachment) {
			$tim->getTrackerItem()->attachFileFromEmail($attachment,$user->getUserId());
		}
	}
} catch (Exception $ex) {
	echo($ex->getMessage());
	$subject = Language::getSessText('Gateway.ErrorSubject');
	$message = Language::getSessText('Gateway.ErrorResponse', array($ex->getMessage(), $body));

    if (GFConf::get('system.debugIncomingEmail')) {
        $mail->dumpRawEmail();
    }

	SystemUtils::sendMail($subject, $message,$from);
}

