<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/tracker/lib/om/BaseTracker.php';
require_once("plugins/tracker/lib/TrackerExtraField.php");
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");
require_once("SystemUtils.php");
require_once('lib/core/Monitor.php');
require_once('lib/core/ProjectPeer.php');
require_once('lib/core/UserPeer.php');
require_once("plugins/tracker/lib/TrackerEfeTransitionTrigger.php");
require_once("lib/exceptions/InvalidDataException.php");


/**
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Wed Nov 16 20:42:33 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @property User[] historicalAssignees
 * @package plugins.tracker.lib
 */
class Tracker extends BaseTracker implements GFPermission
{

	const STATUS_OPEN = 1;
	const STATUS_CLOSED = 0;

	const DATATYPE_TRACKER = 1;
	const DATATYPE_TASK = 2;
	const DATATYPE_SUPPORT = 3;

	const PERMLEVEL_NONE = 0;
	const PERMLEVEL_MEMBER = 1; // can read/edit all within the related group
	const PERMLEVEL_READ = 5; //can view all, unless restrict browse
	const PERMLEVEL_TECH = 10; //can view/edit all
	const PERMLEVEL_TECH_ADMIN = 20; //can read/edit all + be assignee
	const PERMLEVEL_ADMIN = 99; //can read/edit all + NOT assignee

	const SECTION = 'tracker';
	const EMAIL_ALIAS = 'email_alias';

	const PRIORITY_VERYHIGH = 1;
	const PRIORITY_HIGH = 2;
	const PRIORITY_MEDIUM = 3;
	const PRIORITY_LOW = 4;
	const PRIORITY_VERYLOW = 5;

	const TRACKER_FIELD_PRIORITY = 'priority';
	const TRACKER_FIELD_OPEN_DATE = 'open_date';
	const TRACKER_FIELD_CLOSE_DATE = 'close_date';
	const TRACKER_FIELD_DETAILS = 'details';
	const TRACKER_FIELD_WAITING_FOR = 'waiting_for';
	const TRACKER_FIELD_SPRINT_ID = 'sprint_id';

	const MONITORING_SETTING_NAME = 'montracker';
	const DEFAULT_NAG_PERIOD = 24;

	//Auto-close feature
	const AUTO_CLOSE_SETTING = 'auto_close';
	const AUTO_CLOSE_STATUS = 'auto_close_status';
	const AUTO_CLOSE_DEFAULT = 48;

	private $technicians;
	private $historicalAssignees;
	private $intrinsic_fields = null;

	/**
	 * getID
	 * @return integer
	 */
	function getID()
	{
		return $this->getPrimaryKey();
	}

	/**
	 * getName
	 * @return string
	 */
	function getName()
	{
		return $this->getTrackerName();
	}

	/**
	 * isPublic - true/false
	 * @return boolean
	 */
	function isPublic()
	{
		return $this->getIsPublic() && $this->getProject()->getIsPublic();
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate()
	{
		if ($this->userIsAdmin()) {
			return true;
		}

		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	public function userIsAdmin()
	{
		$perm = $this->getProject()->getPermission();
		if (($perm->check(Tracker::SECTION, 0, Tracker::PERMLEVEL_ADMIN))) { // normal permission
			return true;
		}
		if (($perm->check(Tracker::SECTION, $this->getTrackerId(), Tracker::PERMLEVEL_TECH_ADMIN))) { // normal permission
			return true;
		}
		return false;
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate()
	{
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete()
	{
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead()
	{
		if ($this->isPublic()) {
			return true;
		}
		if ($this->userIsAdmin()) {
			return true;
		}
		$perm = $this->getProject()->getPermission();

		if ($perm->check(Tracker::SECTION, $this->getTrackerId(), Tracker::PERMLEVEL_READ)) {
			return true;
		}

		if ($perm->check(Tracker::SECTION, $this->getTrackerId(), Tracker::PERMLEVEL_MEMBER)) { // lowest permission
			// If read is false above and member is true, then they have ONLY member permission.
			//Get current groups and users
			/** @var User[] $groups */
			$groups = $this->getCustomerGroups();
			$u = Session::getSession()->getUser();
			if ($u->isMemberOfAny($groups)) {
				return true;
			}
		}
		return false;
	}


	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @throws PermissionDeniedException
	 * @return void
	 * @see Tracker::setDeleted()
	 * @see Tracker::isDeleted()
	 */
	function delete($con = null)
	{
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete Tracker object: permission denied");
		}
		//delete monitors
		$crit = new Criteria();
		$crit->add(MonitorPeer::SECTION, self::SECTION);
		$crit->add(MonitorPeer::REF_ID, $this->getTrackerId());
		MonitorPeer::doDelete($crit);

		// Create the project-level audit trail (see comments on TrackerItem class)
		$audit = new AuditTrail();
		$audit->setSection(Project::SECTION);
		$audit->setRefId($this->getProject()->getProjectId());
		$audit->setFieldName(self::SECTION);
		$audit->setNewValue('_deleted_');        // mark the tracker item as deleted
		$audit->setOldValue($this->getTrackerName());
		$audit->setUser(Session::getSession()->getUser());
		$audit->save();

		ActionManager::notify("TRACKER_DELETE", $this);
		parent::delete($con);
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @throws InvalidDataException
	 * @throws PermissionDeniedException
	 * @return void
	 */
	function save($con = null)
	{
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create Tracker object: permission denied");
		} else if (!$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update Tracker object: permission denied");
		}
		$errors = array();
		if (strlen($this->tracker_name) < 4) {
			$errors['tracker_name'] = Language::getSessText('Tracker.EmptyTrackerName');
		}
		if (strlen($this->description) < 4) {
			$errors['description'] = Language::getSessText('Tracker.EmptyDescription');
		}
		if ($this->email_all_updates) {
			$invalid_emails = Validator::validateEmailList($this->email_address);
			if (count($invalid_emails) > 0 || empty($this->email_address)) {
				$invalid_str = implode(', ', $invalid_emails);
				$errors['email_address'] = Language::getSessText('Tracker.InvalidEmail', array($invalid_str));
			}
		} else {
			if (!empty($this->email_address)) {
				$invalid_emails = Validator::validateEmailList($this->email_address);
				if (count($invalid_emails) > 0) {
					$invalid_str = implode(', ', $invalid_emails);
					$errors['email_address'] = Language::getSessText('Tracker.InvalidEmail', array($invalid_str));
				}
			}
		}
		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}
		$new = $this->isNew();

		parent::save($con);

		if ($new) {
			// Default Permission Level should be PERMLEVEL_TECH so that this
			// works with the MS Plugin
			$con = Propel::getConnection(TrackerPeer::DATABASE_NAME);
			$sql = "INSERT INTO role_setting (role_id,section,ref_id,value)
				SELECT role_id,'" . self::SECTION . "','" . $this->getPrimaryKey() . "',
				'" . self::PERMLEVEL_TECH . "'
				FROM role WHERE project_id='" . $this->getProject()->getProjectId() . "'";
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql);
		}

		$this->setNew($new);  // fake the flag back for event purposes
		ActionManager::notify("TRACKER_SAVE", $this);
		$this->setNew(FALSE);

	}

	/**
	 * @param Tracker $tracker_from
	 */
	function cloneFields(Tracker $tracker_from)
	{
		$efs = $tracker_from->getTrackerExtraFields();

		$extra_field_map = array();
		$extra_field_element_map = array();

		// Get a mapping between the projects' roles
		$role_mapping = array();
		$from_roles = $tracker_from->getProject()->getRoles();
		$to_roles = $this->getProject()->getRoles();

		foreach ($from_roles as $from_role) {
			// Find the matching role in this project (if any)
			foreach ($to_roles as $to_role) {
				if ($from_role->getRoleName() == $to_role->getRoleName()) {
					if (!in_array($to_role->getRoleId(), $role_mapping)) {
						$role_mapping[$from_role->getRoleId()] = $to_role->getRoleId();
					}
				}
			}
		}

		// clone the extra fields
		foreach ($efs as $ef) {
			//$cloned_ef = $ef->copy();
			// If we call $ef->copy(), the alias may get screwed since it tries to assign the alias
			// of the original item, and it may have random numbers on it
			$cloned_ef = new TrackerExtraField();
			$cloned_ef->setTracker($this);
			$cloned_ef->setFieldName($ef->getFieldName());    // Note that this also sets the alias
			$cloned_ef->setAlias($ef->getAlias());    // Note that we override the alias
			$cloned_ef->setFieldType($ef->getFieldType());
			$cloned_ef->setAttribute1($ef->getAttribute1());
			$cloned_ef->setAttribute2($ef->getAttribute2());
			$cloned_ef->setIsRequired($ef->getIsRequired());
			$cloned_ef->setFieldOrder($ef->getFieldOrder());
			$cloned_ef->setUseInQuickBrowse($ef->getUseInQuickBrowse());
			$cloned_ef->setShowOnSubmit($ef->getShowOnSubmit());
			// Only clone auto_assign if we're under the same project
			if ($this->getProjectId() != $tracker_from->getProjectId()) {
				$cloned_ef->setUseAutoAssign(false);
			} else {
				$cloned_ef->setUseAutoAssign($ef->getUseAutoAssign());
			}
			$cloned_ef->setUseWorkflow($ef->getUseWorkflow());
			$cloned_ef->setUseCommitRule($ef->getUseCommitRule());
			$cloned_ef->setDefaultValue($ef->getDefaultValue());
			$cloned_ef->save();

			$extra_field_map[$ef->getTrackerExtraFieldId()] = $cloned_ef->getTrackerExtraFieldId();

			// Now clone the field's elements
			$elems = $ef->getTrackerExtraFieldElements();
			foreach ($elems as $elem) {
				$cloned_elem = $elem->copy();
				$cloned_elem->setTrackerExtraField($cloned_ef);
				if ($this->getProjectId() != $tracker_from->getProjectId()) {
					$cloned_elem->setAutoAssignTo(User::UNDEFINED_USER_ID);
				}
				$cloned_elem->save();

				$extra_field_element_map[$elem->getElementId()] = $cloned_elem->getElementId();
			}
		}

		// Clone the workflow fields
		$c = new Criteria();
		$c->addJoin(TrackerExtraFieldElementTransitionPeer::ELEMENT_ID, TrackerExtraFieldElementPeer::ELEMENT_ID);
		$c->addJoin(TrackerExtraFieldElementPeer::TRACKER_EXTRA_FIELD_ID, TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID);
		$c->add(TrackerExtraFieldPeer::TRACKER_ID, $tracker_from->getTrackerId());
		$transitions = TrackerExtraFieldElementTransitionPeer::doSelect($c);
		foreach ($transitions as $transition) {
			$cloned_transition = new TrackerExtraFieldElementTransition();
			$cloned_transition->setElementId($extra_field_element_map[$transition->getElementId()]);
			$cloned_transition->setToElementId($extra_field_element_map[$transition->getToElementId()]);
			$cloned_transition->setNotificationMessage($transition->getNotificationMessage());
			$cloned_transition->save();

			// Roles
			$c = new Criteria();
			$c->add(TrackerExtraFieldElementTransitionRolePeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID, $transition->getTrackerExtraFieldElementTransitionId());
			$roles = TrackerExtraFieldElementTransitionRolePeer::doSelect($c);
			foreach ($roles as $role) {
				if (!array_key_exists($role->getRoleId(), $role_mapping)) {
					continue;
				}

				$cloned_transition_role = new TrackerExtraFieldElementTransitionRole();
				$cloned_transition_role->setTrackerExtraFieldElementTransitionId($cloned_transition->getTrackerExtraFieldElementTransitionId());
				$cloned_transition_role->setRoleId($role_mapping[$role->getRoleId()]);
				$cloned_transition_role->save();
			}


			// Required fields
			$required_fields = $transition->getTrackerExtraFieldElementTransitionFields();
			foreach ($required_fields as $required_field) {
				$cloned_required_field = new TrackerExtraFieldElementTransitionField();
				$cloned_required_field->setTrackerExtraFieldElementTransitionId($cloned_transition->getTrackerExtraFieldElementTransitionId());
				$cloned_required_field->setRequiredTrackerExtraFieldId($extra_field_map[$required_field->getRequiredTrackerExtraFieldId()]);
				$cloned_required_field->save();
			}

			// Notifications
			$c = new Criteria();
			$c->add(TrackerExtraFieldElementTransitionNotifyPeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID, $transition->getTrackerExtraFieldElementTransitionId());
			$notifications = TrackerExtraFieldElementTransitionNotifyPeer::doSelect($c);
			foreach ($notifications as $notification) {
				if (!array_key_exists($notification->getRoleId(), $role_mapping)) {
					continue;
				}
				$cloned_notification = new TrackerExtraFieldElementTransitionNotify();
				$cloned_notification->setTrackerExtraFieldElementTransitionId($cloned_transition->getTrackerExtraFieldElementTransitionId());
				$cloned_notification->setRoleId($role_mapping[$notification->getRoleId()]);
				$cloned_notification->save();
			}

			// Reassigns
			$c = new Criteria();
			$c->add(TrackerEfeTransitionReassignPeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID, $transition->getTrackerExtraFieldElementTransitionId());
			$reassigns = TrackerEfeTransitionReassignPeer::doSelect($c);
			foreach ($reassigns as $reassign) {
				if (!array_key_exists($reassign->getAssignToRoleId(), $role_mapping)) {
					continue;
				}
				$cloned_reassign = new TrackerEfeTransitionReassign();
				$cloned_reassign->setTrackerExtraFieldElementTransitionId($cloned_transition->getTrackerExtraFieldElementTransitionId());
				$cloned_reassign->setAssignToRoleId($role_mapping[$reassign->getAssignToRoleId()]);
				$cloned_reassign->save();
			}

			// Transition triggers
			$c = new Criteria();
			$c->add(TrackerEfeTransitionTriggerPeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID, $transition->getTrackerExtraFieldElementTransitionId());
			$triggers = TrackerEfeTransitionTriggerPeer::doSelect($c);
			foreach ($triggers as $trigger) {
				$cloned_trigger = new TrackerEfeTransitionTrigger();
				$cloned_trigger->setTrackerExtraFieldElementTransitionId($cloned_transition->getTrackerExtraFieldElementTransitionId());
				$cloned_trigger->setChangeTrackerExtraFieldId($extra_field_map[$trigger->getChangeTrackerExtraFieldId()]);
				$cloned_trigger->setToElementId($extra_field_element_map[$trigger->getToElementId()]);
				$cloned_trigger->save();
			}

		}


		// Clone the commit triggers
		$c = new Criteria();
		$c->add(TrackerCommitTriggerPeer::TRACKER_ID, $tracker_from->getTrackerId());
		$triggers = TrackerCommitTriggerPeer::doSelect($c);
		foreach ($triggers as $trigger) {
			if (!array_key_exists($trigger->getChangeTrackerExtraFieldId(), $extra_field_map) || !array_key_exists($trigger->getToElementId(), $extra_field_element_map)) {
				continue;
			}
			$cloned_trigger = new TrackerCommitTrigger();
			$cloned_trigger->setTracker($this);
			$cloned_trigger->setChangeTrackerExtraFieldId($extra_field_map[$trigger->getChangeTrackerExtraFieldId()]);
			$cloned_trigger->setToElementId($extra_field_element_map[$trigger->getToElementId()]);
			$cloned_trigger->save();
		}

		// Clone the commit reassign triggers
		$c = new Criteria();
		$c->add(TrackerCommitReassignTriggerPeer::TRACKER_ID, $tracker_from->getTrackerId());
		$triggers = TrackerCommitReassignTriggerPeer::doSelect($c);
		foreach ($triggers as $trigger) {
			// See if there's a matching role on the destination project
			if (!array_key_exists($trigger->getRole()->getRoleId(), $role_mapping)) {
				continue;
			}

			$cloned_trigger = new TrackerCommitReassignTrigger();
			$cloned_trigger->setTracker($this);
			$cloned_trigger->setAssignToRoleId($role_mapping[$trigger->getRole()->getRoleId()]);
			$cloned_trigger->save();
		}

	}

	function buildDefaultFields()
	{
		$efs = $this->getTrackerExtraFields();
		$efarr = array();
		foreach ($efs as $ef) {
			$efarr[$ef->getAlias()] = $ef;
		}

		if (!isset($efarr['status'])) {
			$status = new TrackerExtraField();
			$status->setTracker($this);
			$status->setFieldName('Status');
			$status->setAlias('status');
			$status->setFieldType(TrackerExtraField::TYPE_STATUS);
			//	$status->setAttribute1('4');
			//	$status->setAttribute2('4');g
			$status->setIsRequired(false);
			$status->setUseInQuickBrowse(true);
			$status->setShowOnSubmit(false);

			$efe = new TrackerExtraFieldElement();
			$status->addTrackerExtraFieldElement($efe);
			$efe->setElementName('Open');
			$efe->setStatusId(Tracker::STATUS_OPEN);
			$efe->setDefaultSelected(true);
			$efe->setSortOrder(1);

			$efe = new TrackerExtraFieldElement();
			$status->addTrackerExtraFieldElement($efe);
			$efe->setElementName('Closed');
			$efe->setStatusId(Tracker::STATUS_CLOSED);
			$efe->setDefaultSelected(false);
			$efe->setSortOrder(2);

			$status->save();
		}

		if ($this->getDatatype() == Tracker::DATATYPE_SUPPORT) {
			// @TODO add params/logic for the member group field
			$members = new TrackerExtraField();
			$members->setTracker($this);
			$members->setFieldName(Language::getSessText('TrackerExtraField.SupportGroup'));
			$members->setAlias('SupportGroup');
			$members->setFieldType(TrackerExtraField::TYPE_MEMBERS);
			$members->setAttribute1(TrackerExtraField::MEMBER_GROUPS);
			//Not required since we will auto-assign the group
			$members->setIsRequired(false);
			$members->setUseInQuickBrowse(true);
			//Don't show on submit since we will auto-assign
			$members->setShowOnSubmit(false);
			$members->save();

			// @TODO Create a role and associate to this members field for support tracker type
		}
	}

	/**
	 * returns arranged where each element is an array, cannot
	 * be directly used by trackeritem->addExtraFIelds() or updateExtraFields
	 * pass $arranged=true if you want it to be arranged for those functions
	 */
	function getTrackerDefaultSelected($criteria, $arranged = false)
	{
		$fields = $this->getTrackerExtraFields($criteria);
		$selected = array();
		foreach ($fields as $field) {

			$elements = $field->getTrackerExtraFieldElements();
			foreach ($elements as $element) {
				if ($element->getDefaultSelected()) {
					if ($arranged) {
						if ($field->getFieldType() == TrackerExtraField::TYPE_CHECKBOX ||
							$field->getFieldType() == TrackerExtraField::TYPE_MULTISELECT
						) {
							//need to be arrays
							$selected[$field->getTrackerExtraFieldId()][] = $element->getId();
						} else {
							$selected[$field->getTrackerExtraFieldId()] = $element->getId();
						}
					} else {
						$selected[$field->getTrackerExtraFieldId()][] = $element->getId();
					}
				}
			}
			//
			//	may not have an element selected for default
			//
			if (!isset($selected[$field->getTrackerExtraFieldId()])) {
				if ($field->getFieldType() == TrackerExtraField::TYPE_DATE
					|| $field->getFieldType() == TrackerExtraField::TYPE_TEXTAREA
				) {
					if ($arranged) {
						$selected[$field->getTrackerExtraFieldId()] = '';
					} else {
						$selected[$field->getTrackerExtraFieldId()][] = '';
					}
				} elseif ($field->getFieldType() == TrackerExtraField::TYPE_NUMERIC
					|| $field->getFieldType() == TrackerExtraField::TYPE_TEXT
				) {
					if ($arranged) {
						$selected[$field->getTrackerExtraFieldId()] = $field->getDefaultValue();
					} else {
						$selected[$field->getTrackerExtraFieldId()][] = $field->getDefaultValue();
					}

				} else {
					if ($arranged) {
						if ($field->getFieldType() == TrackerExtraField::TYPE_CHECKBOX ||
							$field->getFieldType() == TrackerExtraField::TYPE_MULTISELECT
						) {
							//need to be arrays
							$selected[$field->getTrackerExtraFieldId()][] = 100;
						} else {
							$selected[$field->getTrackerExtraFieldId()] = 100;
						}
					} else {
						$selected[$field->getTrackerExtraFieldId()][] = 100;
					}
				}
			}
		}
		$selected['PRIORITY'] = 3;
		$selected['ASSIGNEDTO'] = array(100);
		return $selected;
	}

	/**
	 *    getStatuses - return an associative array with the statuses
	 *
	 * @return array
	 */
	static function getStatuses()
	{
		$language = Session::getSession()->getLanguage(); // get the language object for this session
		//set the various statuses
		$open = $language->getText('tracker.status.open');
		$closed = $language->getText('tracker.status.closed');
		$statuses = array(Tracker::STATUS_OPEN => $open, Tracker::STATUS_CLOSED => $closed);
		return $statuses;
	}


	/**
	 * getStatusNames - return an array with the status names
	 *
	 * @param bool $lowercase
	 * @return array
	 */
	public function getStatusNames($lowercase = false)
	{
		$status_names = array();

		$c = new Criteria();
		$c->add(TrackerExtraFieldPeer::TRACKER_ID, $this->getTrackerId());
		$c->add(TrackerExtraFieldPeer::FIELD_TYPE, TrackerExtraField::TYPE_STATUS);
		$c->addJoin(TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID, TrackerExtraFieldElementPeer::TRACKER_EXTRA_FIELD_ID);
		$c->addAscendingOrderByColumn(TrackerExtraFieldElementPeer::SORT_ORDER);

		$tefes = TrackerExtraFieldElementPeer::doSelect($c);

		foreach ($tefes as $tefe) {
			$status_names[] = ($lowercase) ? strtolower($tefe->getElementName()) : $tefe->getElementName();
		}

		return $status_names;
	}

	/**
	 *    getStatusAliases - return an array with the status aliases
	 *
	 * @return array
	 */
	public function getStatusAliases()
	{
		$status_aliases = array();

		$c = new Criteria();
		$c->add(TrackerExtraFieldPeer::TRACKER_ID, $this->getTrackerId());
		$c->add(TrackerExtraFieldPeer::FIELD_TYPE, TrackerExtraField::TYPE_STATUS);
		$c->addJoin(TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID, TrackerExtraFieldElementPeer::TRACKER_EXTRA_FIELD_ID);
		$c->addAscendingOrderByColumn(TrackerExtraFieldElementPeer::SORT_ORDER);

		$tefes = TrackerExtraFieldElementPeer::doSelect($c);

		foreach ($tefes as $tefe) {
			if ($tefe->getAlias() != '') {
				$status_aliases[] = strtolower($tefe->getAlias());
			}
		}

		return $status_aliases;
	}

	/**
	 *    getStatusByName - returns a status object based on the status name
	 *
	 * @param $status
	 * @return object
	 */
	public function getStatusByName($status)
	{
		$c = new Criteria();
		$c->add(TrackerExtraFieldPeer::TRACKER_ID, $this->getTrackerId());
		$c->add(TrackerExtraFieldPeer::FIELD_TYPE, TrackerExtraField::TYPE_STATUS);
		$c->addJoin(TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID, TrackerExtraFieldElementPeer::TRACKER_EXTRA_FIELD_ID);
		$c->add(TrackerExtraFieldElementPeer::ELEMENT_NAME, $status)->setIgnoreCase(true);

		$tefe = TrackerExtraFieldElementPeer::doSelectOne($c);

		return $tefe;
	}


	/**
	 * getStatusByAlias - return a status object based on the status alias
	 *
	 * @param $status
	 * @return object
	 */
	public function getStatusByAlias($status)
	{
		$c = new Criteria();
		$c->add(TrackerExtraFieldPeer::TRACKER_ID, $this->getTrackerId());
		$c->add(TrackerExtraFieldPeer::FIELD_TYPE, TrackerExtraField::TYPE_STATUS);
		$c->addJoin(TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID, TrackerExtraFieldElementPeer::TRACKER_EXTRA_FIELD_ID);
		$c->add(TrackerExtraFieldElementPeer::ALIAS, $status)->setIgnoreCase(true);

		$tefe = TrackerExtraFieldElementPeer::doSelectOne($c);

		return $tefe;
	}


	/**
	 *    getPriorities - return an associative array with the priorities
	 *
	 * @return array
	 */
	static function getPriorities()
	{
		$language = Session::getSession()->getLanguage(); // get the language object for this session
		//set the various priorities
		$veryhigh = $language->getText('tracker.priority.veryhigh');
		$high = $language->getText('tracker.priority.high');
		$medium = $language->getText('tracker.priority.medium');
		$low = $language->getText('tracker.priority.low');
		$verylow = $language->getText('tracker.priority.verylow');
		$priorities = array(Tracker::PRIORITY_VERYHIGH => $veryhigh,
			Tracker::PRIORITY_HIGH => $high,
			Tracker::PRIORITY_MEDIUM => $medium,
			Tracker::PRIORITY_LOW => $low,
			Tracker::PRIORITY_VERYLOW => $verylow); // this is an associative array with the priorities and their string representation
		return $priorities;
	}

	/**
	 *    getStatusName - construct a string with the status of the task.
	 *
	 * @param    int    StatusId
	 * @return string    Status name
	 */
	function getStatusName($StatusId)
	{

		$id = $StatusId; // get the status_id

		$statuses = $this->getStatuses();
		if (array_key_exists($StatusId, $statuses)) {
			return $statuses[$StatusId];
		} else {
			return "--"; // this really shouldn't happen
		}
	}

	function getTechnicians($historicalAssignees = false)
	{

		$nobody = User::UNDEFINED_USER_ID;
		$section = self::SECTION;
		$technician_roles = array(
			self::PERMLEVEL_TECH,
			self::PERMLEVEL_TECH_ADMIN
		);
		$role_list = implode(',', $technician_roles);

		if ($historicalAssignees) {
			$addedCriteria = " UNION
				(SELECT assignee FROM tracker_item_assignee INNER JOIN tracker_item ON tracker_item.tracker_item_id =
				tracker_item_assignee.tracker_item_id WHERE tracker_id ={$this->getTrackerId()} AND assignee <> $nobody)";
		} else {
			$addedCriteria = '';
		}

		$sql = "SELECT * FROM \"user\" WHERE user_id IN
					(
						SELECT user_project.user_id
						FROM user_project, user_project_role, role_setting
						WHERE user_project.user_project_id=user_project_role.user_project_id
						AND user_project_role.role_id=role_setting.role_id
						AND role_setting.ref_id={$this->getTrackerId()}
						AND role_setting.section='$section'
						AND value IN ($role_list)
						UNION SELECT member_user_id AS user_id FROM usergroup_user WHERE usergroup_id IN
						(
							SELECT user_project.user_id AS usergroup_id
							FROM user_project, user_project_role, role_setting
							WHERE user_project.user_project_id=user_project_role.user_project_id
							AND user_project_role.role_id=role_setting.role_id
							AND role_setting.ref_id={$this->getTrackerId()}
							AND role_setting.section='$section'
							AND value IN ($role_list)
						)
						$addedCriteria

					)
					AND is_group = 'f'
					ORDER BY firstname ASC, lastname ASC;
				";

		$con = Propel::getConnection(UserPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		$this->technicians = UserPeer::populateObjects($rs);

		return $this->technicians;
	}

	/**
	 * Retrieve the list of users that have any access to this tracker list.
	 * @return User[] A list of Users that have any access to the current tracker.
	 */
	function getTrackerUsers()
	{
		$section = self::SECTION;
		$min_role = self::PERMLEVEL_MEMBER;

		$sql = "SELECT * FROM \"user\" WHERE user_id IN
					(
						SELECT user_project.user_id
						FROM user_project, user_project_role, role_setting
						WHERE user_project.user_project_id=user_project_role.user_project_id
						AND user_project_role.role_id=role_setting.role_id
						AND role_setting.ref_id={$this->getTrackerId()}
						AND role_setting.section='$section'
						AND value > $min_role
						UNION SELECT member_user_id AS user_id FROM usergroup_user WHERE usergroup_id IN
						(
							SELECT user_project.user_id AS usergroup_id
							FROM user_project, user_project_role, role_setting
							WHERE user_project.user_project_id=user_project_role.user_project_id
							AND user_project_role.role_id=role_setting.role_id
							AND role_setting.ref_id={$this->getTrackerId()}
							AND role_setting.section='$section'
							AND value > $min_role
						)
					)
					AND is_group = 'f'
					ORDER BY firstname ASC, lastname ASC;
					";

		$con = Propel::getConnection(UserPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		return UserPeer::populateObjects($rs);
	}

	/**
	 * Tells if the user is monitoring this tracker
	 * @param User $user
	 * @return bool
	 */
	public function userIsMonitoring(User $user)
	{
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, self::SECTION);
		$c->add(MonitorPeer::REF_ID, $this->getPrimaryKey());
		$c->add(MonitorPeer::USER_ID, $user->getUserId());
		$mon = MonitorPeer::doSelectOne($c);
		return (!is_null($mon)) ? true : false;
	}

	public function getTrackerUrl()
	{
		return GFUrl::getUrl()->projectURL($this->getProject(), 'TrackerItemBrowse', self::SECTION, '', array('tracker_id' => $this->getTrackerId()));
	}

	public function getTrackerLink()
	{
		$url = $this->getTrackerUrl();
		return "<a href=\"$url\">{$this->getTrackerName()}</a>";
	}

	public function isTemplateTracker()
	{
		return $this->getProject()->getIsTemplate();
	}

	/**
	 * If this tracker is in a template project, return all the trackers with the same
	 * name that reference this project as a template project
	 * @return array
	 */
	public function getDependingTrackers()
	{
		$this_project = $this->getProject();

		if (!$this_project->getIsTemplate()) {
			return array();
		}

		$c = new Criteria();
		$c->add(ProjectPeer::TEMPLATE_PROJECT_ID, $this_project->getProjectId());
		$c->addJoin(TrackerPeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
		$c->add(TrackerPeer::TRACKER_NAME, $this->getTrackerName());
		return TrackerPeer::doSelect($c);
	}

	/**
	 * Given an extra field and its value as a strings, tries to normalize it depending on the
	 * type of the extra field (string, date, int, multiselect, etc)
	 *
	 * @param TrackerExtraField $ef
	 * @param string $value
	 * @return array|int|string
	 * @throws InvalidDataException
	 */
	public function normalizeExtraFieldDataFromString(TrackerExtraField $ef, $value)
	{
		$type = $ef->getFieldType();
		// Check for single-value data
		$data = '';

		switch ($type) {
			case TrackerExtraField::TYPE_TEXT:
			case TrackerExtraField::TYPE_TEXTAREA:
				return Validator::sanitizeString($value);
				break;
			case TrackerExtraField::TYPE_DATE:
				return Validator::sanitizeDate($value);
				break;
			case TrackerExtraField::TYPE_NUMERIC:
				return Validator::sanitizeInt($value);
				break;
			case TrackerExtraField::TYPE_SELECT:
			case TrackerExtraField::TYPE_RADIO:
			case TrackerExtraField::TYPE_STATUS:
				if (is_numeric($value)) {
					$element = TrackerExtraFieldElementPeer::retrieveByPk(intval($value));
				} else {
					$c = new Criteria();
					$c->add(TrackerExtraFieldElementPeer::TRACKER_EXTRA_FIELD_ID, $ef->getTrackerExtraFieldId());
					$c->add(TrackerExtraFieldElementPeer::ELEMENT_NAME, Validator::sanitizeString($value), Criteria::ILIKE);
					$element = TrackerExtraFieldElementPeer::doSelectOne($c);
				}

				if ($value && !$element) {
					throw new InvalidDataException(array("Invalid value for field '" . $ef->getFieldName() . "': '" . $value . "'."));
				} elseif (!$value) {
					return 100;
				} else {
					return $element->getPrimaryKey();
				}
				break;
			case TrackerExtraField::TYPE_CHECKBOX:            // Multiple-value data
			case TrackerExtraField::TYPE_MULTISELECT:
				$matches = array();
				// Split the values enclosed in double quotes or commas
				// This will properly split strings like 'option 1, option 2, "foo", "foo bar", "foo, bar", option 3, option 4'
				$values = preg_split('/,|"([^"]*)"/', $value, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
				$data = array();
				$errors = array();
				foreach ($values as $value) {
					$value = trim($value);
					if (!$value) continue;

					$c = new Criteria();
					$c->add(TrackerExtraFieldElementPeer::TRACKER_EXTRA_FIELD_ID, $ef->getTrackerExtraFieldId());
					$c->add(TrackerExtraFieldElementPeer::ELEMENT_NAME, Validator::sanitizeString($value), Criteria::ILIKE);
					$element = TrackerExtraFieldElementPeer::doSelectOne($c);
					if ($value && !$element) {
						$errors[] = "Invalid value for field '" . $ef->getFieldName() . "': '" . $value . "'.";
					} elseif ($element) {
						$data[] = $element->getPrimaryKey();
					}
				}

				if (!empty($errors)) {
					throw new InvalidDataException($errors);
				}

				return $data;
				break;
			case TrackerExtraField::TYPE_RELEASE:        // Unsupported yet
			case TrackerExtraField::TYPE_MEMBERS:        // Unsupported yet
			default:
				return false;
				break;
		}

	}

	/**
	 * @param bool $stripHtml
	 * @return string
	 */
	public function getSubmitInstructions($stripHtml = true)
	{
		if ($stripHtml) {
			return strip_tags(parent::getSubmitInstructions());
		}

		return parent::getSubmitInstructions();
	}

	/**
	 * @param bool $stripHtml
	 * @return string
	 */
	public function getBrowseInstructions($stripHtml = true)
	{
		if ($stripHtml) {
			return strip_tags(parent::getBrowseInstructions());
		}

		return parent::getBrowseInstructions();
	}

	/**
	 * @param string $value
	 */
	public function setSubmitInstructions($value)
	{
		$strU = new StringUtils();
		$value = $strU->cleanHtml($value);

		parent::setSubmitInstructions($value);
	}

	/**
	 * @param string $value
	 */
	public function setBrowseInstructions($value)
	{
		$strU = new StringUtils();
		$value = $strU->cleanHtml($value);

		parent::setBrowseInstructions($value);
	}

	/**
	 * @return string
	 */
	public function getEmailAlias()
	{
		$setting = SettingPeer::FindSetting(Tracker::SECTION, $this->getTrackerId(), Tracker::EMAIL_ALIAS);
		if (!is_object($setting)) {
			return '';
		} else {
			return $setting->getSettingValue();
		}
	}


	/**
	 * @param $value
	 */
	public function setEmailAlias($value)
	{
		SettingPeer::SaveSetting(Tracker::SECTION, $this->getTrackerId(), TRACKER::EMAIL_ALIAS, $value);
	}


	public function unsetEmailAlias()
	{
		$setting = SettingPeer::FindSetting(Tracker::SECTION, $this->getTrackerId(), TRACKER::EMAIL_ALIAS);
		if ($setting) {
			$setting->delete();
		}
	}

	/**
	 * @param string $email_alias
	 * @return bool
	 */
	public function isEmailAliasUnique($email_alias)
	{

		$c = new Criteria();
		$c->add(SettingPeer::SECTION, Tracker::SECTION);
		$c->add(SettingPeer::SETTING_NAME, Tracker::EMAIL_ALIAS);
		$c->add(SettingPeer::SETTING_VALUE, $email_alias);
		//Exclude this tracker
		$c->add(SettingPeer::REF_ID, $this->getTrackerId(), Criteria::NOT_EQUAL);
		$setting = SettingPeer::doSelectOne($c);

		if (!is_null($setting)) {
			return false;
		}

		$c = new Criteria();
		$c->add(MailmanPeer::LIST_NAME, $email_alias);
		$mailman_list = MailmanPeer::doSelectOne($c);

		if (!is_null($mailman_list)) {
			return false;
		}

		// RMT - Also check project unix names and user names, since they can also be used as mailman aliases
		$u = UserPeer::retrieveByUnixName($email_alias);
		if (!is_null($u)) {
			return false;
		}

		$p = ProjectPeer::retrieveByUnixName($email_alias);
		if (!is_null($p)) {
			return false;
		}

		return true;
	}


	/**
	 * @return array
	 */
	public function getIntrinsicFieldsPropertiesArray()
	{
		return array(
			Tracker::TRACKER_FIELD_PRIORITY => array(
				'field_desc' => Language::getSessText('TrackerItem.PriorityName'),
				'field_type' => TrackerExtraField::TYPE_SELECT),
			Tracker::TRACKER_FIELD_OPEN_DATE => array(
				'field_desc' => Language::getSessText('TrackerItem.OpenDateName'),
				'field_type' => TrackerExtraField::TYPE_DATE),
			Tracker::TRACKER_FIELD_CLOSE_DATE => array(
				'field_desc' => Language::getSessText('TrackerItem.CloseDateName'),
				'field_type' => TrackerExtraField::TYPE_DATE),
			Tracker::TRACKER_FIELD_DETAILS => array(
				'field_desc' => Language::getSessText('TrackerItem.DetailsName'),
				'field_type' => TrackerExtraField::TYPE_TEXT),
			Tracker::TRACKER_FIELD_WAITING_FOR => array(
				'field_desc' => Language::getSessText('TrackerItem.WaitingForName'),
				'field_type' => TrackerExtraField::TYPE_TEXT),
			Tracker::TRACKER_FIELD_SPRINT_ID => array(
				'field_desc' => Language::getSessText('TrackerSprint.Name'),
				'field_type' => TrackerExtraField::TYPE_SELECT)
		);
	}

	public function flushIntrinsicFieldSettingsCache()
	{
		$this->intrinsic_fields = null;
	}

	/**
	 * @return array
	 */
	public function getIntrinsicFieldsSettings()
	{

		if ($this->intrinsic_fields == null) {
			$intrinsic_fields = $this->getIntrinsicFieldsPropertiesArray();

			$settings = SettingPeer::getSettingsFromArray(Tracker::SECTION, $this->getTrackerId(), array_keys($intrinsic_fields));
			/** @var Setting[] $settings_array */
			$settings_array = array();

			foreach ($settings as $setting) {
				$settings_array[$setting->getSettingName()] = $setting->getSettingValue();
			}

			foreach ($intrinsic_fields as $field => $property) {
				if (isset($settings_array[$field]) && !is_null($settings_array[$field])) {
					$json = $settings_array[$field];
				} else {
					$json = null;
				}

				$intrinsic_fields[$field] = TrackerPeer::createVerifyFieldSettings(json_decode($json), $this->getTrackerId(),
					$field, $property['field_type'], $property['field_desc'], $this->getProjectId());
			}

			$this->intrinsic_fields = $intrinsic_fields;
		}
		return $this->intrinsic_fields;
	}

	/**
	 * @param integer $from_tracker_id
	 */
	public function portIntrinsicFieldsSettings($from_tracker_id)
	{
		$intrinsic_fields = $this->getIntrinsicFieldsPropertiesArray();

		$settings = SettingPeer::getSettingsFromArray(Tracker::SECTION, $from_tracker_id, array_keys($intrinsic_fields));
		$settings_array = array();

		foreach ($settings as $setting) {
			$settings_array[$setting->getSettingName()] = $setting->getSettingValue();
		}

		foreach ($intrinsic_fields as $field => $property) {
			if (isset($settings_array[$field]) && !is_null($settings_array[$field])) {
				$json = $settings_array[$field];
			} else {
				$json = null;
			}
			$intrinsic_fields[$field] = TrackerPeer::createVerifyFieldSettings(json_decode($json), $this->getTrackerId(),
				$field, $property['field_type'], $property['field_desc'], $this->getProjectId());
		}
	}


	/**
	 * @param int $element_id
	 * @param bool $is_customer
	 * @return bool|int|string
	 */
	public function getStatusNagtime($element_id, $is_customer = false)
	{
		if ($is_customer) {
			$section = 'cus_';
		} else {
			$section = 'sup_';
		}

		$res = SettingPeer::GetSetting(Tracker::SECTION, $this->getTrackerId(), $section . 'nagtime_' . $element_id, -1);

		if ($res == -1) {
			$def_nagtime = Tracker::DEFAULT_NAG_PERIOD;
			if (GFConf::get('tracker.trackerStatusNagTime')) {
				$def_nagtime = Validator::sanitizeInt(GFConf::get('tracker.trackerStatusNagTime'));
			}
			return $def_nagtime;
		} else {
			return $res;
		}
	}

	/**
	 * @param int $element_id
	 * @param string $value
	 * @param bool $is_customer
	 */
	public function setStatusNagtime($element_id, $value, $is_customer = false)
	{
		if ($is_customer) {
			$section = 'cus_';
		} else {
			$section = 'sup_';
		}

		SettingPeer::SaveSetting(Tracker::SECTION, $this->getTrackerId(), $section . 'nagtime_' . $element_id, $value, $this->getProjectId());
	}

	/**
	 * @param bool $is_customer
	 * @return bool|string
	 */
	public function getStatusNag($is_customer = false)
	{
		if ($is_customer) {
			$section = 'cus_';
		} else {
			$section = 'sup_';
		}

		return SettingPeer::GetSetting(Tracker::SECTION, $this->getTrackerId(), $section . 'nagalerts', false);
	}


	/**
	 * @param bool $enabled
	 * @param bool $is_customer
	 */
	public function setStatusNag($enabled = false, $is_customer = false)
	{
		$enabled = Validator::sanitizeBool($enabled);

		if ($is_customer) {
			$section = 'cus_';
		} else {
			$section = 'sup_';
		}

		SettingPeer::SaveSetting(Tracker::SECTION, $this->getTrackerId(), $section . 'nagalerts', $enabled, $this->getProjectId());
	}

	/**
	 * @param bool $is_customer
	 * @return array
	 */
	public function getStatusNagSettings($is_customer = false)
	{
		if ($is_customer) {
			$section = 'cus';
		} else {
			$section = 'sup';
		}

		$c = new Criteria();
		$c->add(SettingPeer::SECTION, Tracker::SECTION);
		$c->add(SettingPeer::REF_ID, $this->getTrackerId());
		$tracker_status_settings = SettingPeer::doSelect($c);

		$arr_status_settings = array();
		foreach ($tracker_status_settings as $tracker_status_setting) {
			$arr_element_id = explode("_", $tracker_status_setting->getSettingName());
			if (sizeof($arr_element_id) == 3 && $arr_element_id[0] == $section && $arr_element_id[1] == 'nagtime') {
				$element_id = $arr_element_id[2];
				$arr_status_settings[$element_id] = $tracker_status_setting->getSettingValue();
			}
		}

		$tracker_statuses = $this->getTrackerStatuses(Tracker::STATUS_OPEN);

		$arr_status = array();
		foreach ($tracker_statuses as $tracker_status) {
			$arr_status[$tracker_status->getElementId()] = array();
			$arr_status[$tracker_status->getElementId()]['element_name'] = $tracker_status->getElementName();
			$arr_status[$tracker_status->getElementId()]['element_id'] = $tracker_status->getElementId();
			if (array_key_exists($tracker_status->getElementId(), $arr_status_settings)) {
				$arr_status[$tracker_status->getElementId()]['nagtime'] = $arr_status_settings[$tracker_status->getElementId()];
			} else {
				$def_nagtime = 24;
				if (GFConf::get('tracker.trackerStatusNagTime')) {
					$def_nagtime = Validator::sanitizeInt(GFConf::get('tracker.trackerStatusNagTime'));
				}

				$arr_status[$tracker_status->getElementId()]['nagtime'] = $def_nagtime;;
			}
		}
		return $arr_status;
	}

	/**
	 * @param bool $is_customer
	 * @return bool|string
	 */

	public function getStatusNagText($is_customer = false)
	{
		if ($is_customer) {
			$setting_name = 'tracker_cus_nag_text';
			$default_text = Language::getSessText('Tracker.CustomerStatusNagDefaultBody');
		} else {
			$setting_name = 'tracker_nag_text';
			$default_text = Language::getSessText('Tracker.SupportStatusNagDefaultBody');
		}

		return SettingPeer::GetSetting(Tracker::SECTION, $this->getTrackerId(), $setting_name, $default_text);
	}


	/**
	 * @param $text
	 * @param bool $is_customer
	 */
	public function setStatusNagText($text, $is_customer = false)
	{
		$text = Validator::sanitizeString($text);

		if ($is_customer) {
			$setting_name = 'tracker_cus_nag_text';
		} else {
			$setting_name = 'tracker_nag_text';
		}

		SettingPeer::SaveSetting(Tracker::SECTION, $this->getTrackerId(), $setting_name, $text, $this->getProjectId());
	}

	/**
	 * Returns the available tracker statuses
	 *
	 * @param int $type if -1: return both open and close statuses, if Tracker::STATUS_OPEN: return only open statuses, if Tracker::STATUS_CLOSED: return only close statuses
	 * @param bool $require_alias
	 * @internal param string $value
	 * @return array of statuses
	 */
	public function getTrackerStatuses($type = -1, $require_alias = false)
	{
		$c = new Criteria();
		$c->add(TrackerExtraFieldPeer::TRACKER_ID, $this->getTrackerId());
		$c->add(TrackerExtraFieldPeer::FIELD_TYPE, TrackerExtraField::TYPE_STATUS);
		$c->addJoin(TrackerExtraFieldElementPeer::TRACKER_EXTRA_FIELD_ID, TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID);
		if ($type == Tracker::STATUS_OPEN) {
			$c->add(TrackerExtraFieldElementPeer::STATUS_ID, Tracker::STATUS_OPEN);
		} else if ($type == Tracker::STATUS_CLOSED) {
			$c->add(TrackerExtraFieldElementPeer::STATUS_ID, Tracker::STATUS_CLOSED);
		}
		if ($require_alias) {
			$c->add(TrackerExtraFieldElementPeer::ALIAS, null, Criteria::NOT_EQUAL);
			$c->add(TrackerExtraFieldElementPeer::ALIAS, '', Criteria::NOT_EQUAL);
		}

		return TrackerExtraFieldElementPeer::doSelect($c);
	}

	/**
	 * @param integer $status_element_id
	 * @return TrackerItem[]
	 */
	public function getTrackerItemsByStatus($status_element_id)
	{
		$sql = "SELECT ti.*
				FROM tracker_item ti, tracker_extra_field_data tefd
				WHERE
					ti.tracker_id = " . $this->getTrackerId() . " AND
					ti.status_id  = " . Tracker::STATUS_OPEN . " AND
					tefd.field_data = '" . $status_element_id . "' AND
					tefd.tracker_item_id = ti.tracker_item_id";

		$con = Propel::getConnection(UserPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		return TrackerItemPeer::populateObjects($rs);
	}

	/**
	 * @param $value
	 */
	public function setAutoCloseSetting($value)
	{
		SettingPeer::SaveSetting(Tracker::SECTION, $this->getTrackerId(), Tracker::AUTO_CLOSE_SETTING, $value, $this->getProjectId());
	}

	/**
	 * @return bool|string
	 */
	public function getAutoCloseSeting()
	{
		return SettingPeer::GetSetting(Tracker::SECTION, $this->getTrackerId(), Tracker::AUTO_CLOSE_SETTING, Tracker::AUTO_CLOSE_DEFAULT);
	}

	/**
	 * @param integer $status_id
	 */
	public function setAutoCloseStatus($status_id)
	{
		SettingPeer::SaveSetting(Tracker::SECTION, $this->getTrackerId(), Tracker::AUTO_CLOSE_STATUS, $status_id, $this->getProjectId());
	}

	public function getAutoCloseStatus()
	{
		return SettingPeer::GetSetting(Tracker::SECTION, $this->getTrackerId(), Tracker::AUTO_CLOSE_STATUS, 0);
	}


	/**
	 * @return array|null
	 */
	public function getCustomerGroups()
	{
		$user = Session::getSession()->getUser();
		if (!is_object($user)) {
			return null;
		}

		$group_ids = array();

		//Get project roles
		$roles = $this->getProject()->getRoles();
		foreach ($roles as $role) {
			if ($this->getProject()->userHasRole($user, $role)) {
				//Session user is a member of this role
				//Fetch the related member field, if any
				$member_field = $role->getRelatedMemberField();

				if (is_object($member_field)) {
					//Get the group IDs
					$tefels = $member_field->getTrackerExtraFieldElements();
					foreach ($tefels as $tef) {
						$group_ids[$tef->getElementId()] = $tef->getElementId();
					}
				}
			}
		}
		return array_keys($group_ids);
	}

	/**
	 * @return bool
	 */
	public function userHasMemberPermission()
	{
		$perm = $this->getProject()->getPermission();
		// If the user has member but not member + 1, they're ONLY a member
		// We hide member fields for folks with only the members permission
		if ($perm->check(Tracker::SECTION, $this->getTrackerId(), Tracker::PERMLEVEL_MEMBER) &&
			!$perm->check(Tracker::SECTION, $this->getTrackerId(), Tracker::PERMLEVEL_MEMBER + 1)
		) {
			return true;
		}

		return false;
	}

	/**
	 * @return bool
	 */
	public function getCloseDateIsEnabled()
	{

		/** @var Setting $setting */
		$settings = $this->getIntrinsicFieldsSettings();
		return $settings[Tracker::TRACKER_FIELD_CLOSE_DATE]['is_enabled'];

	}

	/**
	 * Returns a collection of TrackerExtraField that are required but don't have a default value set
	 * @return TrackerExtraField[]
	 */
	public function getTrackerExtraFieldsRequiredWithNoDefaultSet()
	{
		/** @var TrackerExtraField[] $tef */
		$tef = $this->getTrackerExtraFields();
		$result = array();

		foreach ($tef  as $ef) {
			if ($ef->getIsRequired() && !$ef->hasDefaultValue()) {
				$result[] = $ef;
			}
		}

		return $result;
	}

	/**
	 * Returns tracker default email alias. Alias format is:
	 *
	 *     PROJECT_NAME + "_" + TRACKER_NAME [ + counter]
	 *
	 * So depending on availability a new tracker named 'trackerA' from project
	 * 'projectZ' would have the first name available on this list:
	 *
	 *     trackerA_projectZ
	 *     trackerA_projectZ_1
	 *     trackerA_projectZ_2
	 *     ...
	 *
	 * @return string
	 */

	/**
	 * Returns tracker default email alias. Alias format is:
	 *
	 *     PROJECT_NAME + "-" + CLEAN_TRACKER_NAME
	 *
	 * Where 'CLEAN_TRACKER_NAME' uses the tracker name, transforms it to UTF8
	 * from HTML and cleans all non alphanumeric chars.
	 *
	 * @return string
	 */
	public function getDefaultEmailAlias()
	{
		$strU = new StringUtils();
		//use a non html version of the tracker name, change into '-' any
		//non alphanumeric character
		$str = preg_replace(
			'/[^a-z0-9]/i',
			'-',
			$strU->unHtmlSpecialChars($this->getTrackerName())
        );
		// collapse multiple consecutive '-' into a single char, initial and
		//ending '-' are removed if present
		$cleanTrackerName = preg_replace(
            array('/-{1,}/', '/^-/', '/-$/'),
            array('-', '', ''),
		    $str
        );
        if (empty($cleanTrackerName)) {//filters returned empty, give up
            return '';
        }
        $projectName = $this->getProject()->getUnixName();
		return "{$projectName}-{$cleanTrackerName}";
	}
}
