<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2007-09-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/tracker/lib/om/BaseTrackerCommitReassignTrigger.php';
require_once("plugin/ActionManager.php");

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.tracker.lib 
 */
class TrackerCommitReassignTrigger extends BaseTrackerCommitReassignTrigger {
	
	const ACTION_CREATE = 1;
	const ACTION_UPDATE = 2;
	const ACTION_DELETE = 3;

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see TrackerCommitReassignTrigger::setDeleted()
	 * @see TrackerCommitReassignTrigger::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->getTracker()->userIsAdmin()) {
			throw new PermissionDeniedException();
		}

		$this->updateTemplate(self::ACTION_DELETE);
		
		ActionManager::notify("TRACKER_COMMIT_REASSIGN_TRIGGER_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if (!$this->getTracker()->userIsAdmin()) {
			throw new PermissionDeniedException();
		}
		
		$isnew = $this->isNew();

		parent::save($con);
		ActionManager::notify("TRACKER_COMMIT_REASSIGN_TRIGGER_SAVE", $this);
		
		if ($isnew) {
			$this->updateTemplate(self::ACTION_CREATE);
		} else {
			// This shouldn't be triggered!
			$this->updateTemplate(self::ACTION_UPDATE);
		}
	}
	
	private function updateTemplate($action) {
		// Replicate the data only if it's a template tracker and it's a
		// site admin
		if (!$this->getTracker()->isTemplateTracker() || !Session::getSession()->isSiteAdmin()) {
			return;
		}
		

		$dependingTrackers = $this->getTracker()->getDependingTrackers();
		foreach ($dependingTrackers as $dependingTracker) {
			if ($dependingTracker->getProject()->getProjectId() == $this->getTracker()->getProject()->getProjectId()) {
				// Shouldn't happen, means that the project.template_project_id=project.project_id
				// but we avoid infinite loop anyways
				continue;
			}
			
			$role_mapping = $this->getTracker()->getProject()->getRoleMapping($dependingTracker->getProject());
			
			if (array_key_exists($this->getAssignToRoleId(), $role_mapping)) {
				// Found matching role
				
				$c = new Criteria();
				$c->add(TrackerCommitReassignTriggerPeer::TRACKER_ID, $dependingTracker->getTrackerId());
				$c->add(TrackerCommitReassignTriggerPeer::ASSIGN_TO_ROLE_ID, $role_mapping[$this->getAssignToRoleId()]);
				$existing_trigger = TrackerCommitReassignTriggerPeer::doSelectOne($c);
				
				switch ($action) {
				case self::ACTION_CREATE:
					if (!$existing_trigger) {
						$trigger = new TrackerCommitReassignTrigger();
						$trigger->setTrackerId($dependingTracker->getTrackerId());
						$trigger->setAssignToRoleId($role_mapping[$this->getAssignToRoleId()]);
						$trigger->save();
					}
					continue;
				case self::ACTION_UPDATE:
					//unimplemented
					continue;
				case self::ACTION_DELETE:
					if ($existing_trigger) {
						$existing_trigger->delete();
					}
					continue;
				}
			}
		}
	}

}
?>
