<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
class TrackerItemQuery {

	const FIELD_ASSIGNEE = '1';
	const FIELD_MODDATE = '3';
	const FIELD_EXTRAFIELD = '4';
	const FIELD_SORTCOL = '5';
	const FIELD_SORTORD = '6';
	const FIELD_OPENDATE = '7';
	const FIELD_CLOSEDATE = '8';
	const FIELD_LIST = '9';
	const FIELD_PRIORITY = '10';
	const FIELD_TRACKERSPRINT = '11';
	const FIELD_WAITINGFOR = '12';
	const FIELD_SPRINT = '13';
	const DATE_RANGE_DELIMITER = '|-|';
	const CURRENT_USER = -1;
	const ANY_USER = -2;
	const STATUS_ANY_OPEN = -3;
	const STATUS_ANY_CLOSED = -4;

	protected $tracker_item_id;
	protected $summary;
	protected $submittedBy;
	protected $assignee;
	protected $extra_fields;
	protected $moddaterange;
	protected $opendaterange;
	protected $closedaterange;
	protected $sortord;
	protected $sortcol;
	protected $TrackerItems;
	protected $priority;
	protected $tracker_sprint;
	protected $field_list;
	protected $Tracker;
	protected $showcols;
	protected $isTaskTracker;
	protected $parentId;
	protected $getOnlyParents = false;
	protected $intrinsic_fields;
	protected $waitingFor;

	function __construct(Tracker $Tracker) {
		$this->Tracker = $Tracker;
		$this->intrinsic_fields = $this->Tracker->getIntrinsicFieldsSettings(); //Get intrinsic fields settings
		$this->setupShowColumns();
		$this->isTaskTracker = ($Tracker->getDatatype() == Tracker::DATATYPE_TASK);
		$this->parentId = 0;
	}

	function getTrackerItemId() {
		return $this->tracker_item_id;
	}

	function getSummary() {
		return $this->summary;
	}

	function getSubmittedBy() {
		return $this->submittedBy;
	}

	function getAssignedTo() {
//echo "Assignee:".$this->assignee;
		return $this->assignee;
	}

	function getExtraFields() {
		return $this->extra_fields;
	}

	function getModDateRange() {
		if ($this->moddaterange)
			return $this->moddaterange;
	}

	function getOpenDateRange() {
		if ($this->opendaterange)
			return $this->opendaterange;
	}

	function getCloseDateRange() {
		if ($this->closedaterange)
			return $this->closedaterange;
	}

	function getWaitingFor() {
		return $this->waitingFor;
	}

	function getSortOrd() {
//echo "Sort Ord".$this->sortord;
		if (isset($this->sortord) && $this->sortord == 'DESC') {
			return 'DESC';
		} else {
			return 'ASC';
		}
	}

	function getSortCol() {
//echo "Sort Col".$this->sortcol;
		if (isset($this->sortcol)) {
			$arr = $this->getAvailableSortColumns();
			if (isset($arr[$this->sortcol])) {
				return $this->sortcol;
			} else {
				return 'tracker_item_id';
			}
		} else {
			return 'tracker_item_id';
		}
	}

	function getpriority() {
//echo "priority".$this->priority;
		return $this->priority;
	}

  function getTrackerSprint() {
    return $this->tracker_sprint;
  }

	function getWhereClause() {
		//this may have been called already
		if (isset($this->TrackerItems) && is_array($this->TrackerItems)) {
			return $this->TrackerItems;
		}

		if (isset($this->summary) && strlen($this->summary) > 0) {
			$summary_str = ' AND summary ILIKE \'%' . $this->summary . '%\' ';
		} else {
			$summary_str = '';
		}

		//if assigned to selected, and more to where clause
		if (is_array($this->assignee)) {
			$assignees = implode(',', $this->assignee);
		} else {
			$assignees = $this->assignee;
		}

		//default values
		$assigned_str = '';
		$assigned_table_str = ', tracker_item_assignee tia';

		if ($assignees == '') {
			$assigned_str = '';
			$assigned_table_str = '';
			// [#9232] Allow "any user" as criteria
		} elseif ($assignees == TrackerItemQuery::ANY_USER || stripos($assignees, TrackerItemQuery::ANY_USER) !== FALSE) {
			$assigned_str = "AND tia.tracker_item_id=ti.tracker_item_id AND tia.assignee <> 100";
		} else {
			$assigned_str = "AND tia.tracker_item_id=ti.tracker_item_id AND tia.assignee IN ($assignees)";
		}

		if (isset($this->submittedBy) && !empty($this->submittedBy)) {
			$submitted_by_str = "AND ti.submitted_by IN (SELECT user_id FROM users_idx WHERE users_idx.vectors @@ to_tsquery('" . $this->submittedBy . "'))";
		} else {
			$submitted_by_str = '';
		}

		if (isset($this->priority) && $this->priority > 0) {
			if (is_array($this->priority)) {
				if (count($this->priority) > 0) {
					$priority_str = " and ti.priority IN ('" . implode("','", $this->priority) . "') ";
				} else {
					$priority_str = '';
				}
			} else {
				$priority_str = " and ti.priority='" . $this->priority . "' ";
			}
		} else {
			$priority_str = '';
		}

		$waitingfor_str = '';
		if (isset($this->waitingFor)) {
			$valid_values = array(TrackerItem::WAITINGFOR_ASSIGNEE, TrackerItem::WAITINGFOR_SUBMITTER, 100);
			if (in_array(intval($this->waitingFor), $valid_values)) {
				$criteria = intval($this->waitingFor) == 100 ? ' is null' : '= ' . intval($this->waitingFor);
				$waitingfor_str = "  AND ti.waiting_for $criteria ";
			}
		}

		if (isset($this->tracker_sprint) && $this->tracker_sprint > 0) {
			if (is_array($this->tracker_sprint)) {
				if (count($this->tracker_sprint) > 0) {
					$arr_sprint_values = array();
					$arr_sprint_str = array();
					foreach($this->tracker_sprint as $value) {
						if ($value == 100) {
							$arr_sprint_str[] = " ti.sprint_id is null ";
						} else {
							$arr_sprint_values[] = $value;
						}
					}
					if (count($arr_sprint_values) > 0) {
						$arr_sprint_str[] = " ti.sprint_id IN (" . implode(",", $this->tracker_sprint) . ") ";
					}
					$sprint_str = ' and (' . implode(' or ', $arr_sprint_str) . ') ';
				} else {
					$sprint_str = '';
				}
			} else {
				if ($this->tracker_sprint == '100') {
					$sprint_str = " and ti.sprint_id is null ";
				} else {
					$sprint_str = " and ti.sprint_id ='" . $this->tracker_sprint . "' ";
				}
			}
		} else {
			$sprint_str = '';
		}

		$ef_table_str = '';
		$ef_where_str = '';
		if (isset($this->extra_fields) && is_array($this->extra_fields)) {
			$keys = array_keys($this->extra_fields);
			$vals = array_values($this->extra_fields);
			for ($i = 0; $i < count($keys); $i++) {
				if (empty($vals[$i])) {
					continue;
				}
				$tef = TrackerExtraFieldPeer::retrieveByPk($keys[$i]);
				if (!is_object($tef) || $tef->getTrackerId() != $this->Tracker->getTrackerId()) {
					continue;
				}
//date types have to be cast and use BETWEEN
				switch ($tef->getFieldType()) {
					case TrackerExtraField::TYPE_DATE:
						if (is_array($vals[$i]) && count($vals[$i]) > 0) {
							$betweenClause = $this->getDateRangeClause("aefd$i.field_data::date", $vals[$i][0]);
							if (strlen($betweenClause) > 0) {
								$ef_table_str.=", tracker_extra_field_data aefd$i ";
								$ef_where_str.=" AND aefd$i.tracker_extra_field_id='" . $keys[$i] . "'
								AND aefd$i.field_data != ''
								$betweenClause
								AND aefd$i.tracker_item_id=ti.tracker_item_id ";
							}
						}
						break;

					case TrackerExtraField::TYPE_NUMERIC:
					case TrackerExtraField::TYPE_SERIAL:
						$ef_table_str.=", tracker_extra_field_data aefd$i ";
						$ef_where_str.=" AND aefd$i.tracker_extra_field_id='" . $keys[$i] . "'
							AND aefd$i.field_data != '' AND aefd$i.field_data::float BETWEEN " . $vals[$i][0] . " AND " . $vals[$i][1] . "
							AND aefd$i.tracker_item_id=ti.tracker_item_id ";
						break;

					case TrackerExtraField::TYPE_STATUS:
						if (is_array($vals[$i])) {
							$statuses = $vals[$i];
						} else {
							$statuses = explode(',', $vals[$i]);
						}

						$status_vals = array();
						$status_ids = array();

						if (count($statuses) > 0) {
							// Sort the status criteria into two bins:
							// One for specific status values (Fixed, Invalid, etc.), and
							// the other for the more global status id's (any open, any closed)
							foreach ($statuses as $status) {
								switch ($status) {
								case TrackerItemQuery::STATUS_ANY_OPEN:
										$status_ids[] = Tracker::STATUS_OPEN;
									break;

								case TrackerItemQuery::STATUS_ANY_CLOSED:
										$status_ids[] = Tracker::STATUS_CLOSED;
									break;

								default:
										$status_vals[] = $status;
									break;
							}
						}

							// The same basic SQL is used for any combination of status values and id's
							$ef_table_str.=", tracker_extra_field_data aefd$i, tracker_extra_field_element aefe$i ";
							$ef_where_str.=" AND aefd$i.tracker_extra_field_id=" . $keys[$i] . "
							AND aefd$i.field_data::float = aefe$i.element_id
							AND aefd$i.tracker_item_id=ti.tracker_item_id";

							// First case is the special case where *both* values and id's are selected.
							// Doesn't make much sense, but the results are wrong and confusing if this
							// is not handled appropriately
							if (count($status_ids) > 0 && count($status_vals) > 0) {
								$ef_where_str .= " AND (aefe$i.status_id IN (".implode(',', $status_ids).')'.
												     " OR aefe$i.element_id IN (".implode(',', $status_vals).'))';

							// This case is if just (any open), (any closed) or both are chosen
							// (but no status values)
							} elseif (count($status_ids) > 0) {
								$ef_where_str .= " AND aefe$i.status_id IN (".implode(',', $status_ids).')';

							// This case just for status values (no id's)
							} elseif (count($status_vals) > 0) {
								$ef_where_str .= " AND aefe$i.element_id IN (".implode(',', $status_vals).')';
							}
						}
						break;

					// includes TrackerExtraField::TYPE_MEMBERS
					default:
						if (is_array($vals[$i]) && !empty($vals[$i])) {
							$vals[$i] = implode("','", $vals[$i]);
						}
						$ef_table_str.=", tracker_extra_field_data aefd$i ";
						$ef_where_str.=" AND aefd$i.tracker_extra_field_id='" . $keys[$i] . "'
							AND aefd$i.field_data IN ('" . $vals[$i] . "')
							AND aefd$i.tracker_item_id=ti.tracker_item_id ";
						break;
				}
			}
		}

		//add constraint of range of modified dates
		$moddatesql = $this->getDateRangeClause('ti.last_modified_date', $this->moddaterange);
		//add constraint of range of open dates
		$opendatesql = $this->getDateRangeClause('ti.open_date', $this->opendaterange);
		//add constraint of range of close dates
		$closedatesql = $this->getDateRangeClause('ti.close_date', $this->closedaterange);

		$perm = $this->Tracker->getProject()->getPermission();

		// Let's see if they're group-limited
		$restrict_str = '';
		if ($perm->check('tracker', $this->Tracker->getTrackerId(), Tracker::PERMLEVEL_MEMBER) &&
			!$perm->check('tracker', $this->Tracker->getTrackerId(), Tracker::PERMLEVEL_MEMBER + 1)) {
			// Group restricted user - limit visibility by groups the user belongs to
			$user_id = Session::getSession()->getUser()->getUserId();
			$subquery = $this->getGroupMemberQuery($this->Tracker->getTrackerId(), $user_id);
			$restrict_str .= " AND (ti.tracker_item_id IN (\n $subquery \n) OR submitted_by=$user_id)";
		} else {
			// Not group-limited?  Then see if we need to apply limits for restricted browsing
			if ($this->Tracker->getRestrictBrowse() && !$this->Tracker->isPublic()) {
				if (!$perm->check('tracker', $this->Tracker->getTrackerId(), Tracker::PERMLEVEL_TECH)) {
					if (!Session::getSession()->isLoggedIn()) {
						Throw new PermissionDeniedException();
					} else {
						$restrict_str = " AND submitted_by='" . Session::getSession()->getUser()->getUserId() . "'";
					}
				}
			}
		}

		// Constraint for child items (only when building a tracker tree)
		if ($this->showTree()) {
			if ($this->parentId) {
				$child_str = ' AND ti.parent_id=' . $this->parentId . ' ';
			} else {
				$child_str = ' AND ti.parent_id=0 ';
			}
		} else {
			$child_str = '';
		}

		$qstr = "$ef_table_str $assigned_table_str
			WHERE
			ti.tracker_id='" . $this->Tracker->getTrackerID() . "'
			$opendatesql $moddatesql $closedatesql $priority_str $waitingfor_str $sprint_str $summary_str
			$submitted_by_str $assigned_str $ef_where_str $restrict_str $child_str";
		if ($this->getTrackerItemId() > 0) {
			$qstr = " WHERE
			ti.tracker_id='" . $this->Tracker->getTrackerID() . "'
			AND ti.tracker_item_id='" . $this->getTrackerItemId() . "'
			$restrict_str ";
		}

		if ($this->getOnlyParents) {
			$qstr = $qstr . ' AND ti.has_subitems=true';
		}

		return $qstr;
	}

	function getGroupMemberQuery($tracker_id, $user_id) {

		// Work around stupid limitation of PHP string embedding
		// Can't embed a class constant into a double-quoted string
		// like you can with a var.
		$section = Tracker::SECTION;
		$no_access = Tracker::PERMLEVEL_NONE;

		$sql = "SELECT ti.tracker_item_id
			FROM
			  public.role_setting rs,
			  public.user_project_role upr,
			  public.user_project up,
			  public.usergroup_user ug,
			  public.tracker_item ti,
			  public.tracker_extra_field_data tefd
			WHERE
			  rs.related_id = tefd.tracker_extra_field_id AND
			  upr.role_id = rs.role_id AND
			  up.user_project_id = upr.user_project_id AND
			  up.user_id = ug.usergroup_id AND
			  ti.tracker_id = rs.ref_id AND
			  tefd.tracker_item_id = ti.tracker_item_id AND
			  tefd.field_data = ug.usergroup_id AND
			  ti.tracker_id = $tracker_id AND
			  ug.member_user_id = $user_id AND
			  rs.section = '$section' AND
			  rs.value > $no_access";

		return $sql;
	}

	function getDateRangeClause($fieldname, $daterange) {
		if (isset($daterange)) {
			list($begin, $end) = explode(TrackerItemQuery::DATE_RANGE_DELIMITER, $daterange);
			$begin = strtotime($begin);
			$end = strtotime($end);

			if ($begin !== FALSE || $end !== FALSE) {
				$begin = date('c', $begin);
				$end = date('c', $end);
				return " AND $fieldname BETWEEN '$begin' AND '$end' ";
			}
		}
		return '';
	}

	function getTrackerItemCount() {

		$sql = "SELECT count(*) FROM tracker_item ti " . $this->getWhereClause();
//echo $sql;

		$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		try {
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
		} catch(Exception $e) {
			return 0;
		}
		while ($rs->next()) {
			return $rs->getInt(1);
		}
	}

	function getTrackerItems($limit = 0, $offset = 0) {
		//
		// add LIMIT if necessary
		//
		
		$limit = Validator::sanitizeInt($limit);
		$offset = Validator::sanitizeInt($offset);

		$limitStr = "";
		if ($limit > 0 && $offset > 0) {
//PGSQL POSTGRESQL specific
			$limitStr = "LIMIT $limit OFFSET $offset";
		} else if ($limit > 0 && $offset == 0) {
			$limitStr = "LIMIT $limit";
		} else if ($limit == 0 && $offset > 0) {
			$limitStr = "LIMIT ALL, $offset";
		}

		//
		//  now run the query using the criteria chosen above
		//
		//$sql="SELECT * FROM tracker_item WHERE tracker_item_id IN (
		
		$sort_ord = $this->getSortOrd();
		//If the sort column is 'priority' we need to invert the sort order 
		if ($this->getSortCol() == 'priority') {
			if ($this->getSortOrd() == 'ASC') {
				$sort_ord = 'DESC';
			} else {
				$sort_ord = 'ASC';
			}
		}
		
		$sql = "SELECT * FROM tracker_item
				WHERE tracker_item.tracker_item_id IN (
					SELECT ti.tracker_item_id as ti_id 
					FROM tracker_item ti {$this->getWhereClause()}
				)
				ORDER BY tracker_item.{$this->getSortCol()} {$sort_ord}, tracker_item.tracker_item_id ASC
				$limitStr";
		//$sqlcount="SELECT count(*) FROM tracker_item ti $qstr";
//echo $sql;

		$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		try {
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			$this->TrackerItems = TrackerItemPeer::populateObjects($rs);
		} catch(Exception $e) {
			return array();
		}
		return $this->TrackerItems;
	}

	function getAvailableColumns() {
		$Lang = Session::getSession()->getLanguage();
		$cols = array();
		$cols['priority'] = $Lang->getText('TrackerItem.PriorityName');
		$cols['tracker_sprint'] = $Lang->getText('TrackerItem.Sprint');
		$cols['assigned_to'] = $Lang->getText('TrackerItem.AssignedTo');
		$cols['submitted_by'] = $Lang->getText('TrackerItem.SubmittedByName');
		$cols['open_date'] = $Lang->getText('TrackerItem.OpenDateName');
		$cols['close_date'] = $Lang->getText('TrackerItem.CloseDateName');
		$cols['last_modified_date'] = $Lang->getText('TrackerItem.LastModifiedDateName');
		$efs = $this->Tracker->getExtraFields();
		foreach ($efs as $ef) {
			$cols[$ef->getTrackerExtraFieldId()] = $ef->getFieldName();
		}
		return $cols;
	}

    public static function getStandardSortColumns() {
		$Lang = Session::getSession()->getLanguage();
		$cols = array(
            'tracker_item_id' => $Lang->getText('TrackerItem.TrackerItemIdName'),
            'summary' => $Lang->getText('TrackerItem.SummaryName'),
            'priority' => $Lang->getText('TrackerItem.PriorityName'),
			'tracker_sprint' => $Lang->getText('TrackerItem.Sprint'),
            'submitted_by' => $Lang->getText('TrackerItem.SubmittedByName'),
            'open_date' => $Lang->getText('TrackerItem.OpenDateName'),
            'close_date' => $Lang->getText('TrackerItem.CloseDateName'),
            'last_modified_date' => $Lang->getText('TrackerItem.LastModifiedDateName'),
        );
        return $cols;
    }

	function getAvailableSortColumns() {
		$cols = self::getStandardSortColumns();
		if (! $this->intrinsic_fields[Tracker::TRACKER_FIELD_PRIORITY]['is_enabled']) {
            unset($cols['priority']);
        }
        if (! $this->intrinsic_fields[Tracker::TRACKER_FIELD_OPEN_DATE]['is_enabled']) {
            unset($cols['open_date']);
        }
        if (! $this->intrinsic_fields[Tracker::TRACKER_FIELD_CLOSE_DATE]['is_enabled']) {
		    unset($cols['close_date']);
        }
		return $cols;
	}

	function getAvailableSortOrd() {
		$Lang = Session::getSession()->getLanguage();
		$sort = array();
		$sort['ASC'] = $Lang->getText('Tracker.SortAsc');
		$sort['DESC'] = $Lang->getText('Tracker.SortDesc');
		return $sort;
	}

	function setupShowColumns() {
		$cols = array();
		$Lang = Session::getSession()->getLanguage();
        if ($this->intrinsic_fields[Tracker::TRACKER_FIELD_PRIORITY]['is_enabled']) {
            $cols['priority'] = $Lang->getText('TrackerItem.PriorityName');
        }
		$cols['assigned_to'] = $Lang->getText('TrackerItem.AssignedTo');
		$cols['submitted_by'] = $Lang->getText('TrackerItem.SubmittedByName');
        if ($this->intrinsic_fields[Tracker::TRACKER_FIELD_SPRINT_ID]['is_enabled']) {
        $cols['tracker_sprint'] = $Lang->getText('TrackerSprint.Name');
        }
        if ($this->intrinsic_fields[Tracker::TRACKER_FIELD_OPEN_DATE]['is_enabled']) {
		    $cols['open_date'] = $Lang->getText('TrackerItem.OpenDateName');
        }
        if ($this->intrinsic_fields[Tracker::TRACKER_FIELD_CLOSE_DATE]['is_enabled']) {
		    $cols['close_date'] = $Lang->getText('TrackerItem.CloseDateName');
        }
        if ($this->intrinsic_fields[Tracker::TRACKER_FIELD_WAITING_FOR]['is_enabled']) {
	        $cols['waiting_for'] = $Lang->getText('TrackerItem.WaitingForName');
        }
		$cols['last_modified_date'] = $Lang->getText('TrackerItem.LastModifiedDateName');
		$efs = $this->Tracker->getTrackerExtraFields();
		foreach ($efs as $ef) {
			if ($ef->getUseInQuickBrowse()) {
				$cols[$ef->getTrackerExtraFieldId()] = $ef->getFieldName();
			}
		}
		$this->showcols = $cols;
	}

	function showColumn($column) {
		return isset($this->showcols[$column]);
	}

	public function getSubitemsQuery(TrackerItem $item) {
		$tq = clone $this;
		$tq->parentId = $item->getTrackerItemId();
		$tq->TrackerItems = null;

		return $tq;
	}

	function setGetOnlyParents($v) {
		$this->getOnlyParents = $v;
	}

	/**
	 * Returns true if the query must be displayed as a tree 
	 * (currently, only Quick Queries without parameters are shown as a tree)
	 */
	public function showTree() {
		return false;
	}

}
