<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('plugins/tracker/lib/TrackerItemQuery.php');
require_once('plugins/tracker/lib/TrackerQueryDisplayColumnPeer.php');

class TrackerItemSavedQuery extends TrackerItemQuery {
	
	protected $showcols;
	protected $sort_order;
	protected $tqid;

	function __construct(Tracker $Tracker, TrackerQuery $TrackerQuery, User $currentUser = null) {
		parent::__construct($Tracker);
		$this->setupItems($TrackerQuery, $currentUser);
	}

	function getTrackerQueryId(){
		return $this->tqid;
	}

	function setupShowColumns() {
		$cols=array();
		$sorted_cols=array();

		$tqid = isset($_GET['tracker_query_id']) ? $_GET['tracker_query_id'] : null;
		$Lang=Session::getSession()->getLanguage();
		$cols['priority']=$Lang->getText('TrackerItem.PriorityName');
		$sorted_cols['priority'] = 0;
		$cols['assigned_to']=$Lang->getText('TrackerItem.AssignedTo');
		$sorted_cols['assigned_to'] = 1;
		$cols['submitted_by']=$Lang->getText('TrackerItem.SubmittedByName');
		$sorted_cols['submitted_by'] = 2;
		
		$c = new Criteria();
		$c->add(TrackerQueryDisplayColumnPeer::TRACKER_QUERY_ID, $tqid);
		$c->addAscendingOrderByColumn(TrackerQueryDisplayColumnPeer::DISPLAY_FIELD_ORDER);
		$display_efs = TrackerQueryDisplayColumnPeer::doSelect($c);
		$efs_ids = array();
		foreach($display_efs as $def){
			$efs_ids[] = $def->getDisplayFieldId();
			$sorted_cols[$def->getDisplayFieldId()] = $def->getDisplayFieldOrder() + 3;
		}

		$c = new Criteria();
		$c->add(TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID, $efs_ids, Criteria::IN);
		$efs = TrackerExtraFieldPeer::doSelect($c);
		
		
		foreach ($efs as $ef) {
				$cols[$ef->getTrackerExtraFieldId()]=$ef->getFieldName();
		}

		$offset = count($cols);
		$cols['tracker_sprint']=$Lang->getText('TrackerSprint.Name');
		$sorted_cols['tracker_sprint']=$offset++;
		$cols['open_date']=$Lang->getText('TrackerItem.OpenDateName');
		$sorted_cols['open_date'] = $offset++;
		$cols['close_date']=$Lang->getText('TrackerItem.CloseDateName');
		$sorted_cols['close_date'] = $offset++;
		$cols['waiting_for']=$Lang->getText('TrackerItem.WaitingForName');
		$sorted_cols['waiting_for']=$offset++;
		$cols['last_modified_date']=$Lang->getText('TrackerItem.LastModifiedDateName');
		$sorted_cols['last_modified_date']=$offset++;

		$this->showcols=$cols;
		$this->sort_order = $sorted_cols;
	}


	private function setupItems(TrackerQuery $TrackerQuery, User $currentUser) {
		$fields = $TrackerQuery->getTrackerQueryFields();
		foreach ($fields as $field) {
			switch ($field->getQueryFieldType()) {
				case self::FIELD_ASSIGNEE: {
					if ($field->getQueryFieldValues() != 0) {
                                            // RMT [#9231] - Allow "current user" filter in canned queries
                                            $values = str_ireplace(TrackerItemQuery::CURRENT_USER, $currentUser->getUserId(), $field->getQueryFieldValues());
                                            //error_log("TrackerItemSavedQuery->setupItems, assignee value = ($values)");
                                            $this->assignee=explode(',', $values);
					} else {
                                            $this->assignee=array();
					}
					break;
				}
				case self::FIELD_PRIORITY: {
					if ($field->getQueryFieldValues() > 0) {
						$this->priority=explode(',',$field->getQueryFieldValues());
					} else {
						$this->priority= array();
					}
					break;
				}
				case self::FIELD_SPRINT: {
					if ($field->getQueryFieldValues() != '') {
						$this->tracker_sprint=explode(',',$field->getQueryFieldValues());
					} else {
						$this->tracker_sprint= array();
					}
					break;
				}
				case self::FIELD_MODDATE: {
					$this->moddaterange=$field->getQueryFieldValues();
					break;
				}
				case self::FIELD_CLOSEDATE: {
					$this->closedaterange=$field->getQueryFieldValues();
					break;
				}
				case self::FIELD_OPENDATE: {
					$this->opendaterange=$field->getQueryFieldValues();
					break;
				}
				case self::FIELD_SORTCOL: {
					$this->sortcol=$field->getQueryFieldValues();
					break;
				}
				case self::FIELD_SORTORD: {
					$this->sortord=$field->getQueryFieldValues();
					break;
				}
				case self::FIELD_WAITINGFOR: {
					$this->waitingFor=$field->getQueryFieldValues();
					break;
				}
				case self::FIELD_EXTRAFIELD: {
					/*if (strpos($field->getQueryFieldValues(),'*DATE*') !== false) {
						$this->extra_fields[$field->getQueryFieldId()]=
							explode('*DATE*',$field->getQueryFieldValues());
					} elseif (strpos($field->getQueryFieldValues(),'*NUMERIC*') !== false) {
						$this->extra_fields[$field->getQueryFieldId()]=
							explode('*NUMERIC*',$field->getQueryFieldValues());
					} else {*/
						$this->extra_fields[$field->getQueryFieldId()]=
							explode(',',$field->getQueryFieldValues());
					//}
					break;
				}
			}
		}
	//	return $this->getTrackerItems();
	}

	function showColumn($column) {
		return isset($this->showcols[$column]);
	}

	function getSortOrder($field_id) {
		if(array_key_exists($field_id, $this->sort_order)){
			return $this->sort_order[$field_id];
		}

		return 0;
	}

	function sortByDisplayOrder(TrackerExtraField $a, TrackerExtraField $b){
		if(array_key_exists($a->getId(), $this->sort_order) &&
		array_key_exists($b->getId(), $this->sort_order)){
			if($this->sort_order[$a->getId()] == $this->sort_order[$b->getId()]){
				return 0;
			}	
			return $this->sort_order[$a->getId()] < $this->sort_order[$b->getId()] ? -1 : 1;
		}

		return 0;

	}


}

?>
