<?php
/**
 * Copyright (c) 2005-2014 GForge, LLC
 *
 * @version   $Id: Group.class,v 1.94 2005/09/06 18:13:29 danper Exp $
 * @author Mario A. Papayannis <mpapayannis@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('notification/GFNotification.php');
require_once("plugin/ActionManager.php");
require_once('core/ProjectPeer.php');
require_once('TrackerItemPeer.php');
require_once('TrackerItemAssigneePeer.php');
require_once('TrackerItemMessagePeer.php');
require_once('StringUtils.php');

class TrackerItemSupportNotification implements GFNotification {

	private $TrackerItem;
	private $users;
	private $pending_hours_to_close;

	function __construct(TrackerItem $TrackerItem, $users, $pending_hours_to_close, $send_to_submitter) {
		$this->TrackerItem = $TrackerItem;
		$this->users = $users;
		$this->pending_hours_to_close = $pending_hours_to_close;
		$this->send_to_submitter = $send_to_submitter;
	}


	private function processEmailBody($str_body) {
		$arr_keys = array('%URL%', '%CUSTOMER_GROUP%', '%LAST_MODIFIED_DATE%', '%SITE_NAME%', '[hours]');

		foreach($arr_keys as $key) {
			$str_replacement = '';
			switch($key) {
				case "%URL%":
					$str_replacement = $this->TrackerItem->getTrackerItemUrl(true);
					break;

				case "%CUSTOMER_GROUP%":
					$str_replacement = $this->TrackerItem->getCustomerGroup();
					break;

				case "%LAST_MODIFIED_DATE%":
					$str_replacement = $this->TrackerItem->getLastModifiedDate();
					break;

				case "%SITE_NAME%":
					$str_replacement = GFConf::get('system.name');
					break;
				case "[hours]":
					$str_replacement = $this->pending_hours_to_close;
					break;
			}

			$str_body = str_replace($key, $str_replacement, $str_body);
		}

		return htmlspecialchars_decode($str_body);
	}


	/**
	 *	getNotificationMessage - construct a string of this message content.
	 *
	 *	@param	User
	 *	@return string
	 */
	function getNotificationMessage(User $User) {
		if ($this->send_to_submitter) {
			$body = $this->TrackerItem->getTracker()->getStatusNagText(true);
		} else {
			$body = $this->TrackerItem->getTracker()->getStatusNagText();
		}

		$body = $this->processEmailBody($body);

		return $body;
	}


	/**
	 *	getNotificationSubject - construct a string of this message subject.
	 *
	 *	@param	User
	 *	@return string
	 */
	function getNotificationSubject(User $User) {
		if ($this->send_to_submitter) {
			$subject = Language::getSessText('Tracker.CustomerStatusNagSubject', array($this->TrackerItem->getTrackerItemId()));
		} else {
			$subject = Language::getSessText('Tracker.StatusNagSubject', array($this->TrackerItem->getTrackerItemId()));
		}
		return $subject;
	}

	/**
	 *	getUsers - get an array of user objects
	 *	@return array
	 */
	public function getUsers() {
		return $this->users;
	}

	/**
	 *	getSection - retrieves the section name of the tracker item
	 *	@return string
	 */
	function getSection() {
		return TrackerItem::SECTION;
	}

	/**
	 *	getRefId - retrieves the id of the tracker item that the notification belongs to
	 *	@return int
	 */
	function getRefId() {
		return $this->TrackerItem->getTrackerItemId();
	}

	/**
	 *	getMessageId
	 *	@return string
	 */
	function getMessageId() {
		return '<'.$this->getSection().'-'.$this->getRefId().'@'.GFConf::get('system.hostName').'>';
	}

	/**
	 *	getInReplyTo
	 *	@return string
	 */
	function getInReplyTo() {
		$inReplyTo = $this->TrackerItem->getTrackerItemId();
		return '<'.$this->getSection().'-'.$inReplyTo.'@'.GFConf::get('system.hostName').'>';
	}

	/**
	 *	getReferences
	 *	@return string
	 */
	public function getReferences() {
		return $this->getInReplyTo();
	}

}

?>
