<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
// The parent class
require_once 'plugins/tracker/lib/om/BaseTrackerPeer.php';

// The object class
include_once 'plugins/tracker/lib/Tracker.php';

/**
 * The skeleton for this class was autogenerated by Propel on:
 *
 * [Wed Nov 16 20:42:33 2005]
 *
 *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package plugins.tracker.lib
 */

class TrackerPeer extends BaseTrackerPeer
{

    private static $_pkCache = array();

    /**
     * Retrieve a single object by pkey or NULL if not found.
     * This method was added by propel_customizer.php.
     *
     * @param	mixed $pk	The primary key
     * @param	bool $useCache	Whether to use caching. This will increase performance if the same object
     * 							is accessed several times
     * @param	Connection $con	The connection to use
     * @return	Tracker
     */
    public static function retrieveByPk($pk, $useCache = true, $con = null)
    {
        if ($useCache && array_key_exists($pk, TrackerPeer::$_pkCache))
        {
            return TrackerPeer::$_pkCache[$pk];
        }

        $object = parent::retrieveByPk($pk, $con);
        if ($useCache)
        {
            TrackerPeer::$_pkCache[$pk] = $object;
        }
        return $object;
    }

    /**
     * Removes an object from the cache. This is usually done after saving.
     * This method was added by propel_customizer.php.
     *
     * @param	mixed $pk	The primary key of the object to remove
     */
    public static function invalidate($pk)
    {
        if (array_key_exists($pk, TrackerPeer::$_pkCache))
        {
            unset(TrackerPeer::$_pkCache[$pk]);
        }
    }

    /**
     * Verifies the structure of the field settings object, and creates/completes it
     * by setting default values
     *
     * @param	array $settings	The intrinsic field settings
     */
    public static function createVerifyFieldSettings($settings, $ref_id, $field_name, $field_type, $field_desc, $project_id, $enabled = true) {

        $settings = (array) $settings;
	    $save = false;


        if (!is_array($settings))
        {
            $settings = array();
	        $save = true;
        }

        if (!array_key_exists("field_name", $settings))
        {
            $settings["field_name"] = $field_desc;
	        $save = true;
        }

        if (!array_key_exists("is_required", $settings))
        {
            $settings["is_required"] = false;
	        $save = true;
        }

        if (!array_key_exists("is_enabled", $settings))
        {
            $settings["is_enabled"] = true;
	        $save = true;
        }

        if (!array_key_exists("field_order", $settings))
        {
            $settings["field_order"] = 0;
	        $save = true;
        }

        if (!array_key_exists("on_browse", $settings))
        {
            $settings["on_browse"] = false;
	        $save = true;

        }

        if (!array_key_exists("on_submit", $settings))
        {
            $settings["on_submit"] = false;
	        $save = true;
        }

        if (!array_key_exists("field_type", $settings))
        {
            $settings["field_type"] = $field_type;
	        $save = true;
        }

	    if ($save) {
            SettingPeer::SaveSetting(Tracker::SECTION, $ref_id, $field_name, json_encode($settings), $project_id);
	    }

        return $settings;
    }

    public static function getTrackerCount($project_id)
    {
        $total = 0;
        $c = new Criteria();
        $c->add(TrackerPeer::PROJECT_ID, $project_id);
	    $Trackers = TrackerPeer::doSelect($c);

	    foreach($Trackers as $Tracker) {
		    if ($Tracker->userCanRead()) {
				$total++;
		    }
	    }
        return $total;
    }

    /**
     * Adds a User Stories tracker to the specified project.
     * @param mixed $proj The ID or unix_name for the project to which the User Stories Tracker should be added.
     * @return boolean TRUE if all went well, FALSE otherwise (e.g., invalid ID provided, tracker already exists, etc).
     */
    public static function addUserStoriesTracker($proj)
    {

        if (is_int($proj))
        {
            $project = ProjectPeer::retrieveByPk($proj);
        } else
        {
            $project = ProjectPeer::retrieveByUnixName($proj);
        }

        if ($project == null)
        {
            error_log("addUserStoriesTracker($proj) does not exist, exit!");
            return FALSE;
        }

        $user = UserPeer::retrieveByPk(100);  // Nobody user

        if ($user == null)
        {
            error_log("addUserStoriesTracker($proj) Nobody user does not exist, exit!");
            return FALSE;
        }

        try {
            $con = Propel::getConnection();

            $con->begin();

            $field_order = 0;
            $name = 'User Stories';

            $t = new Tracker();
            $t->setTrackerName($name);
            $t->setDatatype(Tracker::DATATYPE_TASK);
            $t->setDescription($name);
            $t->setProject($project);
            $t->setDuePeriod(0);
            $t->setEmailAllUpdates(false);
            $t->setIsPublic(false);
            $t->setRestrictBrowse(false);
            $t->setSendAbbreviatedEmail(false);
            $t->setShowDependenciesTab(true);
            $t->setShowDuplicatesTab(true);
            $t->setShowAssociationsTab(true);
            $t->setShowTimeTrackingTab(true);
            $t->setShowTagsTab(true);


            // Status extra field
			$status = new TrackerExtraField();
            $t->addTrackerExtraField($status);
			$status->setFieldName('Status');
			$status->setAlias('status');
			$status->setFieldType(TrackerExtraField::TYPE_STATUS);
            $status->setFieldOrder($field_order++);
			$status->setIsRequired(true);
			$status->setUseInQuickBrowse(true);
			$status->setShowOnSubmit(false);

            $statuses = array(
                'Proposed' => Tracker::STATUS_OPEN,
                'Accepted' => Tracker::STATUS_OPEN,
                'Implemented' => Tracker::STATUS_CLOSED,
                'Rejected' => Tracker::STATUS_CLOSED
            );

            $order = 0;
            foreach ($statuses as $status_name => $status_id) {
    			$efe = new TrackerExtraFieldElement();
                $status->addTrackerExtraFieldElement($efe);
    			$efe->setElementName($status_name);
    			$efe->setStatusId($status_id);
    			$efe->setDefaultSelected($order == 0);
    			$efe->setSortOrder($order++);
            }


            // Story points extra field
            $story_pts = new TrackerExtraField();
            $t->addTrackerExtraField($story_pts);
            $story_pts->setFieldName('Story Points');
            $story_pts->setAlias('story_pts');
            $story_pts->setFieldType(TrackerExtraField::TYPE_SELECT);
            $story_pts->setFieldOrder($field_order++);
            $story_pts->setIsRequired(true);
            $story_pts->setUseInQuickBrowse(true);
            $story_pts->setShowOnSubmit(true);

            $points = array(
                'X-Small (1)' => 1,
                'Small (2)' => 2,
                'Medium (4)' => 4,
                'Large (8)' => 8, 
                'X-Large (16+)' => 16
            );
            $order = 0;
            foreach ($points as $pts_name => $point) {
                $efe = new TrackerExtraFieldElement();
                $story_pts->addTrackerExtraFieldElement($efe);
                $efe->setElementName($pts_name);
                $efe->setAlias($point);
                $efe->setDefaultSelected($order == 0);
                $efe->setSortOrder($order++);
            }


            // As a... extra field
            $as_a = new TrackerExtraField();
            $t->addTrackerExtraField($as_a);
            $as_a->setFieldName('As A...');
            $as_a->setAlias('as_a');
            $as_a->setFieldType(TrackerExtraField::TYPE_SELECT);
            $as_a->setFieldOrder($field_order++);
            $as_a->setIsRequired(true);
            $as_a->setUseInQuickBrowse(true);
            $as_a->setShowOnSubmit(true);

            $as_list = array(
                'User',
                'Manager',
                'Customer'
            );
            $order = 0;
            foreach ($as_list as $as_name) {
                $efe = new TrackerExtraFieldElement();
                $as_a->addTrackerExtraFieldElement($efe);
                $efe->setElementName($as_name);
                $efe->setDefaultSelected($order == 0);
                $efe->setSortOrder($order++);
            }


            // I Want To... extra field
            $i_want_to = new TrackerExtraField();
            $t->addTrackerExtraField($i_want_to);
            $i_want_to->setTracker($t);
            $i_want_to->setFieldName('I Want To...');
            $i_want_to->setFieldOrder($field_order++);
            $i_want_to->setAlias('i_want_to');
            $i_want_to->setFieldType(TrackerExtraField::TYPE_TEXTAREA);
            $i_want_to->setIsRequired(true);
            $i_want_to->setShowOnSubmit(true);


            // So I Can... extra field
            $so_i_can = new TrackerExtraField();
            $t->addTrackerExtraField($so_i_can);
            $so_i_can->setTracker($t);
            $so_i_can->setFieldName('So I Can...');
            $so_i_can->setFieldOrder($field_order++);
            $so_i_can->setAlias('so_i_can');
            $so_i_can->setFieldType(TrackerExtraField::TYPE_TEXTAREA);
            $so_i_can->setIsRequired(true);
            $so_i_can->setShowOnSubmit(true);

            $t->save($con);

            $con->commit();
        } catch (\Exception $ex) {
            error_log("addUserStoriesTracker($proj) failed with error: {$ex->getMessage()}");
            error_log($ex->getTraceAsString());
            $con->rollback();
            return false;
        }

        return TRUE;
    }

    /**
     * Retrieves, parses and return data required in the 'Workflow Rules'
     * trackers page:
     *     /tracker/?action=TrackerExtraFieldElementTransitionBrowse
     *  
     * Response has the following structure:
     *
     * array(
     *     rows  => array(
     *                 DB result, (Result from DB)
     *                 to_value,  (string, element name)
     *                 roles,     (Array with role names)
     *                 fields,    (Array with field names)
     *                 notifs     (Array with role names)
     *                 )
     *     nTEF  => array(array(tracker extra field id=>int counter)... )
     *     nElem => array(array(element id=>int counter)... )
     * )
     * 
     * @param  int   $trackerId Tracker id
     * @return array            Data required in 'Workflow Rules' page
     * 
     * @author Hector Pizarro <hpizarro@gforgegroup.com>
     */
    public function getWorkflowRulesData($trackerId)
    {
        // Array with method response
        $resp = array('rows'=>array(), 'nTEF'=>array(), 'nElem'=>array());
        // Exec a SQL query to use the DB sort engine.
        // Response is ordered by 'Field', 'From Value', 'To Value'
        $sql=<<<SQL
            SELECT
                tef.tracker_extra_field_id,
                tef.field_name,
                tefe.element_id,
                tefe.element_name,
                tefe.alias,
                tefet.tracker_extra_field_element_transition_id,
                tefet.to_element_id
            FROM tracker_extra_field AS TEF,
              tracker_extra_field_element AS TEFE,
              tracker_extra_field_element_transition AS TEFET,
              tracker_extra_field_element AS TEFE2
            WHERE 
              TEF.tracker_id='{$trackerId}'
              AND TEF.use_workflow='t'
              AND TEF.tracker_extra_field_id=TEFE.tracker_extra_field_id
              AND TEFE.element_id=TEFET.element_id
              AND TEFET.to_element_id=TEFE2.element_id
            ORDER BY TEF.field_order,TEFE.sort_order, TEFE2.sort_order
SQL;
        $con = Propel::getConnection();
        $stmt = $con->createStatement();
        $rs = $stmt->executeQuery($sql);
        $to_elem_ids = array();// to_element_id as keys to assure uniqueness
        // For each row request 'Roles that make change',
        // 'Required fields for change' and 'Notify on change'
        while ($rs->next()) {
            $row = $rs->getRow();
            $tefId = $row['tracker_extra_field_id'];
            $elemId = $row['element_id'];
            $resp['nTEF'][$tefId] = (!isset($resp['nTEF'][$tefId])) ? 1 :
                                                $resp['nTEF'][$tefId] + 1;
            $resp['nElem'][$elemId] = (!isset($resp['nElem'][$elemId])) ? 1 :
                                                $resp['nElem'][$elemId] + 1;
            //store all to_element_id, we will add 'to_value' when loop ends
            $to_elem_ids[$row['to_element_id']] = 1;
            // Request all Roles that may make this change for
            // tracker_extra_field_element_transition_id
            $row['roles'] = array();
            $cr = new Criteria();
            $cr->add(TrackerExtraFieldElementTransitionRolePeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID,
               $row['tracker_extra_field_element_transition_id']);
            $Roles = TrackerExtraFieldElementTransitionRolePeer::doSelectJoinRole($cr);
            foreach($Roles as $tefetr) {
                $row['roles'][] = $tefetr->getRole()->getRoleName();
            }

            // Request all Required fields for this change for
            // tracker_extra_field_element_transition_id
            $row['fields'] = array();
            $cr = new Criteria();
            $cr->add(TrackerExtraFieldElementTransitionFieldPeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID,
               $row['tracker_extra_field_element_transition_id']);
            $Fields = TrackerExtraFieldElementTransitionFieldPeer::doSelectJoinTrackerExtraField($cr);
            foreach($Fields as $tefetf) {
                $row['fields'][] = $tefetf->getTrackerExtraField()->getFieldName();
            }

            // Request all Notify on Change (Roles notified) for
            // tracker_extra_field_element_transition_id
            $row['notifs'] = array();
            $cr = new Criteria();
            $cr->add(TrackerExtraFieldElementTransitionNotifyPeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID,
               $row['tracker_extra_field_element_transition_id']);
            $Fields = TrackerExtraFieldElementTransitionNotifyPeer::doSelectJoinRole($cr);
            foreach($Fields as $tefetf) {
                $row['notifs'][] = $tefetf->getRole()->getRoleName();
            }

            //storing all on response
            $resp['rows'][] = $row;
        }

        //add 'to_value' for each data row using their to_element_id
        if (!empty($to_elem_ids)) {
            $criteria = new Criteria();
            $criteria->add(TrackerExtraFieldElementPeer::ELEMENT_ID,
                                    array_keys($to_elem_ids), Criteria::IN);
            $objs = TrackerExtraFieldElementPeer::doSelect($criteria);
            $elems = array();// to_element_id is key, name is value
            foreach ($objs as $o) {
                $elems[$o->getElementId()] = $o->getElementName();
            }
            $totalRows = count($resp['rows']);
            for ($i=0;$i<$totalRows;$i++) {
                $elem_id = $resp['rows'][$i]['to_element_id'];
                $resp['rows'][$i]['to_value'] = $elems[$elem_id];
            }
        }

        return $resp;
    }
}