<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");
require_once("SystemUtils.php");
require_once('lib/core/Monitor.php');
require_once('lib/core/ProjectPeer.php');
require_once('lib/core/UserPeer.php');
include_once('plugins/tracker/lib/TrackerSprintPeer.php');
include_once('plugins/tracker/lib/om/BaseTrackerSprint.php');
require_once("lib/exceptions/InvalidDataException.php");


/**
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Wed Nov 16 20:42:33 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @property User[] historicalAssignees
 * @package plugins.tracker.lib
 */
class TrackerSprint extends BaseTrackerSprint implements GFPermission {

	const SECTION = 'trackersprint';

	const TRACKER_SPRINT_NONE = 100;

	const PERMLEVEL_NONE = 0;
	const PERMLEVEL_MEMBER = 1; // can read/edit all within the related group
	const PERMLEVEL_READ = 5; //can view all, unless restrict browse
	const PERMLEVEL_TECH = 10; //can view/edit all
	const PERMLEVEL_TECH_ADMIN = 20; //can read/edit all + be assignee
	const PERMLEVEL_ADMIN = 99; //can read/edit all + NOT assignee

	const STATUS_NOT_STARTED = 0;
	const STATUS_IN_PROGRESS = 1;
	const STATUS_CLOSED      = 2;

	var $oldValues = array();

	static function getStatuses() {
		$lang = Language::getSessLang();

		return array(
			TrackerSprint::STATUS_NOT_STARTED => $lang->getText('TrackerSprint.StatusNotStarted'),
			TrackerSprint::STATUS_IN_PROGRESS => $lang->getText('TrackerSprint.StatusInProgress'),
			TrackerSprint::STATUS_CLOSED => $lang->getText('TrackerSprint.StatusClosed')
		);
	}

	function setSprintStatus($status) {
		if (!array_key_exists('sprint_status', $this->oldValues) &&
			$status != $this->getSprintStatus()) {
			$this->oldValues['sprint_status'] = $this->getSprintStatus();
		}
		return parent::setSprintStatus($status);
	}

	function getSprintStatusName() {
		$statuses = TrackerSprint::getStatuses();
		return $statuses[$this->getSprintStatus()];
	}

	function getTrackerItemsAtIndex($limit = 0, $offset = 0) {
		$limitStr = "";
		if ($limit > 0 && $offset > 0) {
			$limitStr = "LIMIT $limit OFFSET $offset";
		} else if ($limit > 0 && $offset == 0) {
			$limitStr = "LIMIT $limit";
		} else if ($limit == 0 && $offset > 0) {
			$limitStr = "LIMIT ALL, $offset";
		}

		$sql = "SELECT * FROM tracker_item
		WHERE tracker_item.sprint_id = {$this->getSprintId()} (
		$limitStr";

		$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		try {
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			return TrackerItemPeer::populateObjects($rs);
		} catch(Exception $e) {
			return array();
		}
	}

	function getPercentComplete() {
		$all = $this->getTrackerItemCount();
		$open = $this->getTrackerItemCount(Tracker::STATUS_OPEN);

		if ($all == 0) {
			return 0;
		}

		$pct = (1 - ($open / $all));
		return round($pct * 100);
	}

	function getTrackerItems($status = null) {
		$c = new Criteria();
		if ($status != null) {
			$c->add(TrackerItemPeer::STATUS_ID, $status);
		}

		$c->add(TrackerItemPeer::SPRINT_ID, $this->getSprintId());

		return TrackerItemPeer::doSelect($c);
	}


	function getTrackerItemCount($status = null) {
		$c = new Criteria();
		if (!is_null($status)) {
			$c->add(TrackerItemPeer::STATUS_ID, $status);
		}
		$c->add(TrackerItemPeer::SPRINT_ID, $this->getSprintId());
		return TrackerItemPeer::doCount($c);
	}

	function getID() {
		return $this->getSprintId();
	}

	function getName() {
		return $this->getSprintName();
	}

	function getSprintTitleText($lang = null) {
		return self::TitleForSprint($this->getSprintProgression(), $this->getSprintName());
	}

	public static function TitleForSprint($sprint_progression, $sprint_name = '', $lang = null) {
		if (! $lang) {
			$lang = Language::getSessLang();
		}

		// RMT - Only include the sprint name if it's
		$title = "{$lang->getText('TrackerSprint.Name')} {$sprint_progression}";
		if ($sprint_name != '' &&
			$sprint_name != $sprint_progression &&
			$sprint_name != $title) {
			$title = "{$title}: {$sprint_name}";
		}

		return $title;
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		$is_new = $this->isNew();
		if ($is_new && $this->getSprintProgression() != '') {
			$newSprintNumber = $this->getNextSprintNumber($this->getProject());
			$this->setSprintProgression($newSprintNumber);
		}
		parent::save($con);
		ActionManager::notify("TRACKER_SPRINT_SAVE", array(
			'object' => $this,
			'is_new' => $is_new
		));
	}
  
	/**
	 * Deletes sprint
	 * 
	 * @param $con
	 */
	function delete($con = null) {
		ActionManager::notify("TRACKER_SPRINT_DELETE", $this);
		parent::delete($con);		
	}
	
	/**
	 * Returns url for accessing TrackerSprintView for this sprint
	 * 
	 * @param bool $full
	 * @return string
	 */
	public function getSprintUrl($full = false) {
		$link = GFUrl::getUrl()->projectURL($this->getProject(), 'TrackerSprintView', 'tracker', '', array('sprint_id' => $this->getID()));
		if ($full) {
			$link = GFUrl::getFullUrl($link);
		}
		return $link;
	}

	/**
	 * isPublic - true/false
	 * @return boolean
	 */
	function isPublic() {
		return $this->getIsPublic() && $this->getProject()->getIsPublic();
	}

	/**
	 * Get the IsPublic column value.
	 *  
	 * @return boolean	  
	 */
	public function getIsPublic()
	{
		return true;
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		if ($this->userIsAdmin()) {
			return true;
		}

		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	public function userIsAdmin() {
		if ($this->getProject()->userIsAdmin() ) {
			return true;
		}
		$perm = $this->getProject()->getPermission();
		if ( ($perm->check(Tracker::SECTION, 0, Tracker::PERMLEVEL_ADMIN))) {
			return true;
		}
		if ( ($perm->check(TrackerSprint::SECTION, $this->getProject()->getProjectId(), TrackerSprint::PERMLEVEL_ADMIN))) { // normal permission
			return true;
		}
		return false;
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		if ($this->isPublic()) {
			return true;
		}		
		if ($this->userIsAdmin()) {
			return true;
		}
		$perm = $this->getProject()->getPermission();
		$trackerIds = $this->getTrackerIds();

		if (!count($trackerIds)) {
			return true;
		}

		if ( ($perm->checkAllHigherThan(Tracker::SECTION, $trackerIds,
			TrackerSprint::PERMLEVEL_MEMBER))) {
			return true;
		}

		return false;
	}


	public function setProjectId($v) {
		if ($this->project_id !== $v) {
			$this->project_id = $v;
			$this->modifiedColumns[] = TrackerSprintPeer::PROJECT_ID;
		}
		if ($this->getProject() !== null && $this->getProject()->getProjectId() !== $v) {
			$this->setProject(null);
		}
	}

	public function setSprintName($v) {
		if($v !== null) {
			$this->sprint_name = $v;
			$this->modifiedColumns[] = TrackerSprintPeer::SPRINT_NAME;
		}
	}

	public function getNextSprintNumber($project) {
		if (is_object($project)) {
			$project = $project->getProjectId();
		}

		$max = TrackerSprintPeer::getMaxTrackerSprintNumber($project);

		return ($max + 1);
	}

	public function getTrackerIds() {
		$con = Propel::getConnection();
		$stmt = $con->createStatement();
		$sql = "SELECT DISTINCT tracker_id FROM tracker_item WHERE sprint_id=".$this->getSprintId();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		$tIds = array();
		while ($rs->next()) {
			$tIds[] = $rs->getInt(1);
		}

		return $tIds;
	}
}
