<?php
require_once 'plugins/tracker/lib/om/BaseTrackerTransitionScript.php';

require_once('plugins/tracker/lib/Tracker.php');

/**
 * Class for catching execution errors
 */
class TrackerTransitionScriptException extends Exception {
	public $errorCode;
	public $message;
	
	public function __construct($script, $errorCode, $message) {
		$this->script = $script;
		$this->errorCode = $errorCode;
		$this->message = $message;

		parent::__construct($message);
	}
}

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.tracker.lib 
 */
class TrackerTransitionScript extends BaseTrackerTransitionScript {
	const SCRIPT_TYPE_LOCAL = 1;			// Script stored somewhere in the server
	const SCRIPT_TYPE_UPLOADED = 2;			// Script uploaded and managed by the Filesystem class
	
	const SECTION = 'tracker_trans_script';
	
	private $errors = array();
	private $uploadedFile = null;
	
	public function userIsAdmin() {
		if (Session::getSession()->isSiteAdmin()) {
			return true;
		}
		
		// Site-wide tracker admin can manage scripts
		if ($this->project_id) {
			$perm = $this->getProject()->getPermission();
			return $perm->check(Tracker::SECTION, 0, Tracker::PERMLEVEL_ADMIN);
		}
		
		return false;
	}
	
	public function userCanCreate() {
		return $this->userIsAdmin();
	}
	
	public function userCanEdit() {
		return $this->userIsAdmin();
	}
	
	public function userCanDelete() {
		return $this->userIsAdmin();
	}
	
	public function getScriptType() {
		return ($this->script_path ? self::SCRIPT_TYPE_LOCAL : self::SCRIPT_TYPE_UPLOADED);
	}
	
	public function getScriptName() {
		if ($this->getScriptType() == self::SCRIPT_TYPE_LOCAL) {
			return $this->script_path;
		} else {
			$fs = $this->getFilesystem();
			return $fs->getFilename();
		}
	}
	
	public function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException();
		} else if (!$this->isNew() && !$this->userCanEdit()) {
			throw new PermissionDeniedException();
		}
		
		if (!$this->script_path && !$this->script_filesystem_id && !$this->uploadedFile) {
			// At least one of the two must be selected
			$this->errors['script_type'] = Language::getSessText('Tracker.WorkflowScriptNoType');
		} else {
			unset($this->errors['script_type']);
		}
		
		if (empty($this->description)) {
			$this->errors['script_type'] = Language::getSessText('Tracker.WorkflowScriptEmptyDescription');
		}
		
		if (!empty($this->uploadedFile)) {
			if ($this->uploadedFile['error'] !== UPLOAD_ERR_OK) {
				$this->errors['uploaded_file'] = Language::getSessText('Tracker.WorkflowScriptErrorUpload');
			} else {
				unset($this->errors['uploaded_file']);
			}
		}

		if (count($this->errors) > 0) {
			throw new InvalidDataException($this->errors);
		}
		parent::save($con);
		
		// Save the uploaded script (if any)
		if ($this->uploadedFile) {
			$fs = new Filesystem();
			$fs->setSection(self::SECTION);
			$fs->setRefId($this->getPrimaryKey());
			$fs->setPostedBy(Session::getSession()->getUser()->getUserId());
			$fs->setFileDataFromUploadedFile($this->uploadedFile);
			$fs->save();
			
			$this->setScriptFilesystemId($fs->getFilesystemId());
			parent::save($con);
		} else if ($this->script_filesystem_id && $this->script_path) {
			// If this happens, it means the path was set and it was pointing to an uploaded script.
			// Delete the corresponding Filesystem entry
			$fs = $this->getFilesystem();
			$this->setFilesystem(null);
			parent::save($con);
			$fs->delete();
		}
	}
	
	public function setScriptPath($path) {
		// Validation
		if (!$path) {
			$this->errors['script_path'] = Language::getSessText('Tracker.WorkflowScriptEmptyPath');
		} elseif (strpos($path, '/') !== 0) {
			$this->errors['script_path'] = Language::getSessText('Tracker.WorkflowScriptMustBeAbsolute');
		} elseif (!is_file($path)) {
			$this->errors['script_path'] = Language::getSessText('Tracker.WorkflowScriptNotValidFile');
		} else {
			// Everything OK
			unset($this->errors['script_path']);
		}

		parent::setScriptPath($path);
	}
	
	/**
	 * Sets an uploaded file (from $_FILES) as the script
	 */
	public function setUploadedScript($uploadedFile) {
		parent::setScriptPath(null);

		if ($this->script_filesystem_id) {
			$this->getFilesystem()->delete();
		}
		
		$this->uploadedFile = $uploadedFile;
	}
	
	public function getFullScriptPath() {
		if ($this->getScriptType() == self::SCRIPT_TYPE_LOCAL) {
			return $this->script_path;
		} else {
			$fs = $this->getFilesystem();
			return $fs->getFilePath();
		}
	}
	
	public function executeScript(TrackerExtraFieldElementTransition $trans) {
		$path = $this->getFullScriptPath();
		@chmod($path, 0750);				// Set the script as executable
		if (!is_executable($path)) {		// Error
			throw new Exception("Could not make $path executable");
		}
		
		$sess = Session::getSession();
		$user_id = ($sess->isLoggedIn() ? $sess->getUser()->getUserId() : 0);
		
		$cmd = $this->getFullScriptPath()." ".$trans->getPrimaryKey()." ".$user_id." 2>&1";
		$output = array();
		$error = SystemUtils::execute($cmd, $output);
		
		if ($error) {
			throw new TrackerTransitionScriptException($this, $error, implode("\n", $output));
		}
		
		return $output;
	}
}
