<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code creates a new TrackerItem and saves it to the 
// database
//
require_once('plugins/tracker/lib/Tracker.php');
require_once('plugins/tracker/lib/TrackerQuery.php');
require_once('lib/core/UserPreference.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('exceptions/NotLoggedInException.php');
require_once('core/Language.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

//force login
$sess = Session::getSession();
if (!$sess->isLoggedIn()) {
	throw new NotLoggedInException();
}

$errors = array();
$language = Language::getSessLang();

// The query being added as a tickler
$tracker_query_id=Validator::sanitizeInt($_REQUEST['tracker_query_id']);

// The tracker that is currently being viewed - may not be the same as
// the one referred to by the query above!
// This is where the user will go when we're done
$tracker_id=Validator::sanitizeInt($_REQUEST['tracker_id']);

if ($tracker_query_id > 0) {
    $preference_name = "ti-tq-{$tracker_id}-{$tracker_query_id}";

    //validate that this is a legitimate tracker for this project and user can add items
    $TrackerQuery = TrackerQueryPeer::retrieveByPk($tracker_query_id);
    if (!$TrackerQuery || !is_object($TrackerQuery) ||
        (! $TrackerQuery->getIsShared() && $TrackerQuery->getUserId() != $sess->getUser()->getUserId())) {
        // Not shared, not theirs, skip it
        $errors[] = $language->getText('Tracker.AddTicklerPermission');
    }

    if (count($errors) == 0) {
        try {
            UserPreferencePeer::setUserPref($sess->getUser()->getUserId(), $preference_name, $tracker_query_id);
        } catch (\Exception $ex) {
            error_log("Exception while adding tickler {$TrackerQuery->getQueryName()} for {$sess->getUser()->getUnixname()}: {$ex->getMessage()}");
            $errors[] = $language->getText('Tracker.AddTicklerError');
        }
    }

    if (count($errors) > 0) {
        $feedback = '* '.implode('<br />* ', $errors);
    } else {
        $feedback = $language->getText('Tracker.AddTicklerSuccess', array($TrackerQuery->getQueryName()));
    }
} else {
    $feedback = $language->getText('Tracker.AddTicklerEmpty');
}

$url = GFUrl::getUrl()->currentBaseURL('TrackerItemBrowse', array('feedback' => $feedback, 'tracker_id'=>$tracker_id));
GFUrl::getUrl()->redirect($url);
