<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code shows a form for TrackerItem
//
require_once('plugins/tracker/lib/TrackerItemPeer.php');
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('plugins/tracker/lib/html/TrackerExtraFieldRenderer.php');
require_once('GFUrl.php');
require_once('lib/exceptions/NotLoggedInException.php');
require_once('html/GFCKEditor.php');
require_once('lib/html/UploadHandler.php');

$session = Session::getSession();
if (!$session->isLoggedIn()) {
	throw new NotLoggedInException();
}
$Lang = $session->getLanguage();
$StringUtils = new StringUtils();

require_once('plugins/tracker/lib/html/TrackerTemplate.php');
$theme = new TrackerTemplate();

$Project = GFUrl::getUrl()->getProject();
$tracker_id=$_REQUEST['tracker_id'];
$Tracker = TrackerPeer::retrieveByPk($tracker_id);
//check if user can create
$TrackerItem = new TrackerItem();
$Tracker->addTrackerItem($TrackerItem);
if (!$Tracker || !is_object($Tracker) || $Tracker->getProject()->getProjectId() != $Project->getProjectId() || !$TrackerItem->userCanCreate()) {
        throw new PermissionDeniedException();
}

if (array_key_exists('sprint_id', $_REQUEST)) {
  $sprint_id = $_REQUEST['sprint_id'];
  $TrackerSprint = TrackerSprintPeer::retrieveByPk($sprint_id);
  if (!is_object($TrackerSprint)) {
    throw new PermissionDeniedException();
  }
} else {
  $TrackerSprint = null;
}

//If it's a support tracker, then check the user's group membership
if ($Tracker->getDatatype() == Tracker::DATATYPE_SUPPORT) {
	$user = $session->getUser();
	$project_groups = $Project->getMemberGroups();
	$project_group_ids = array();
	foreach($project_groups as $project_group) {
		$project_group_ids[] = $project_group->getUserId();
	}
	$member_groups = $user->getMemberGroups($project_group_ids);
	$groups_count = sizeof($member_groups);

	if ($groups_count == 1) {
		$member_group = $member_groups[0];
		$autoassign_group = $member_group->getUserId();
	}

	//Get members group field id
	$c = new Criteria();
	$c->add(TrackerExtraFieldPeer::TRACKER_ID, $Tracker->getTrackerId());
	$c->add(TrackerExtraFieldPeer::FIELD_TYPE, TrackerExtraField::TYPE_MEMBERS);
	$tef_members = TrackerExtraFieldPeer::doSelectOne($c);
}

//Get intrinsic fields settings
$intrinsic_fields = $TrackerItem->getTracker()->getIntrinsicFieldsSettings();

$params = array(
		'project' => $Project,
		'tracker' => $Tracker,
		'action' => isset($action) ? $action : '',
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('TrackerItem.AddNewPageTitle'),
		'enableUploads' => true,
		);

$setting_name =  Tracker::MONITORING_SETTING_NAME . '_' . $session->getUser()->getUserId();
$monitor_setting = SettingPeer::GetSetting(Tracker::SECTION, $Tracker->getTrackerId(), $setting_name, 'false');

$priority=((isset($_POST['priority'])) ? htmlspecialchars($_POST['priority']) : '' );
$summary=((isset($_POST['summary'])) ? $StringUtils->cleanHTML($_POST['summary']) : '' );
$details=((isset($_POST['details'])) ? $StringUtils->cleanHTML($_POST['details']) : '' );
$monitor=((isset($_POST['summary']) && !isset($_POST['monitor'])) ? false : $monitor_setting);

$tefr = new TrackerExtraFieldRenderer($Tracker);

//@TODO we will probably have to change this to adjust to the user's locale
if (!isset($_POST['start_date'])) {
	$start_date = date('M d Y');
} else {
	$start_date = $_POST['start_date'];
}

$next_week = (time() + (60*60*24*7));
$end_date = date('M d Y', $next_week);
if (isset($_POST['end_date'])) {
	$end_date = $_POST['end_date'];
}

// This will let us upload files even if we don't have the tracker item ID
$uploadHandler = new UploadHandler();
$file_upload_form_hash = $uploadHandler->getUploadHash();

include('forms/TrackerItemAdd-tpl.php');

?>
