<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code creates a new TrackerItem and saves it to the 
// database
//
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('exceptions/MissingDataException.php');
require_once('exceptions/PermissionDeniedException.php');

$project = GFUrl::getUrl()->getProject();
$lang = Session::getSession()->getLanguage();
$user = Session::getSession()->getUser();

if (!isset($_GET['tracker_item_id'])) {
	throw new MissingDataException('Missing tracker item ID');
}

$tracker_item = TrackerItemPeer::retrieveByPk($_GET['tracker_item_id']);
if (is_null($tracker_item)) {
	throw new PermissionDeniedException();
}
$tracker = $tracker_item->getTracker();
if (!$tracker->userIsAdmin()) {
	throw new PermissionDeniedException();
}

if (isset($_POST['to_tracker_id'])) {
	$tracker_to = TrackerPeer::retrieveByPk($_POST['to_tracker_id']);
	if (is_null($tracker_to)) {
		throw new PermissionDeniedException();
	}
	
	try {
		$tracker_item->moveToTracker($tracker_to);
	} catch (InvalidDataException $ex) {
		// If we don't invalidate this item on the cache, the TrackerItemMove page would
		// retrieve the item with the incorrect tracker id
		TrackerItemPeer::invalidate($_GET['tracker_item_id']);
		$feedback = '* '.implode('<br />* ', $ex->getErrors());
		include('TrackerItemMove.php');
		exit;
	}

	ActionManager::notify('TRACKER_ITEM_MOVE', array($tracker_item, $tracker, $tracker_to));

	$feedback = $lang->getText('TrackerItem.MoveItem.Moved', array($tracker_item->getTrackerItemId(), $tracker_to->getTrackerName()));
	GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseUrl('TrackerItemBrowse', array('tracker_id' => $tracker->getTrackerId(), 'feedback' => $feedback)));
} else {
	$feedback = $lang->getText('TrackerItem.MoveItem.SelectTracker');
	include('TrackerItemMove.php');
}
?>
