<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code lists TrackerQuery
//
require_once('plugins/tracker/lib/TrackerQueryPeer.php');
require_once('plugins/tracker/lib/TrackerQuery.php');
require_once('lib/exceptions/NotLoggedInException.php');

$session = Session::getSession();

//must be logged in
if (!$session->isLoggedIn()) {
	throw new NotLoggedInException();
}
$Lang = $session->getLanguage();

require_once('plugins/tracker/lib/html/TrackerTemplate.php');
$theme = new TrackerTemplate();

$Project = GFUrl::getUrl()->getProject();
$Tracker = TrackerPeer::retrieveByPk($_GET['tracker_id']);

$params = array(
		'project' => $Project,
		'tracker' => $Tracker,
		'action' => isset($action) ? $action : '',
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('TrackerQuery.BrowsePageTitle')
		);
		
$theme->header($params);

function sortQueries(TrackerQuery $a, TrackerQuery $b) {
    $aname = $a->getQueryName();
    $bname = $b->getQueryName();
    if ($aname == $bname) {
        return 0;
    }
    return ($aname < $bname) ? -1 : 1;
}

// Retrieve the objects array
$crit = new Criteria();
$cton1=$crit->getNewCriterion(TrackerQueryPeer::USER_ID, $session->getUser()->getID());
$cton2=$crit->getNewCriterion(TrackerQueryPeer::IS_SHARED, true);
$cton1->addOr($cton2);

$crit->add($cton1);
/** @var TrackerQuery[] $arr  */
$arr = $Tracker->getTrackerQuerys($crit);
usort($arr, 'sortQueries');


	$theader = array();

	$theader[] = $Lang->getText('TrackerQuery.QueryNameName');
	$theader[] = $Lang->getText('Edit');
	$theader[] = $Lang->getText('Delete');
	$theader[] = $Lang->getText('TrackerQuery.Run');
	$theader[] = $Lang->getText('TrackerQuery.Export');
//	$theader[] = $Lang->getText('TrackerQuery.Graph');

	echo $theme->tableTop($theader);

if (count($arr) < 1) {
	echo $theme->tableFullRow($Lang->getText('TrackerQuery.NoneFound'));
} else {
	foreach ($arr as $item) {
                $name = $item->getQueryName();
                if ($item->getUserId() != $session->getUser()->getID()) {
					// 100 = Special "Nobody" user
                    if ($item->getUserId() != TrackerExtraFieldElement::ELEMENT_NONE) {
                        $unixname = $item->getUser()->getUnixName();
                        $userUrl = GFUrl::getUrl()->userURL($unixname);
                        $name .= " (shared by <a href='$userUrl' title='User profile for $unixname'>$unixname</a>)";
                    }
                }
		$trow = array();
		$trow[]=$name;
		if($Tracker->userIsAdmin() || $item->getUserId() == Session::getSession()->getUser()->getUserId()) {
			$trow[]='<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerQueryEdit').'&amp;tracker_id='.$Tracker->getTrackerId().'&amp;tracker_query_id='.$item->getTrackerQueryID() .'">'. $Lang->getText('Edit').'</a>';
			$trow[]='<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerQueryDelete').'&amp;tracker_id='.$Tracker->getTrackerId().'&amp;tracker_query_id='.$item->getTrackerQueryID() .'">'. $Lang->getText('Delete').'</a>';
		} else {
			$trow[] ='';
			$trow[] = '';
		}
		$trow[]='<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerItemBrowse').'&amp;tracker_id='.$Tracker->getTrackerId().'&amp;tracker_query_id='.$item->getTrackerQueryID() .'">'. $Lang->getText('TrackerQuery.Run').'</a>';
		$temprow  = '<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerQueryCSV').'&amp;tracker_id='.$Tracker->getTrackerId().'&amp;tracker_query_id='.$item->getTrackerQueryID() .'">'. $Lang->getText('TrackerQuery.ExportCSV').'</a>  ';
		$temprow .= '<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerQueryExcel').'&amp;tracker_id='.$Tracker->getTrackerId().'&amp;tracker_query_id='.$item->getTrackerQueryID() .'">'. $Lang->getText('TrackerQuery.ExportExcel').'</a>  ';
		$temprow .= '<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerQueryXML').'&amp;tracker_id='.$Tracker->getTrackerId().'&amp;tracker_query_id='.$item->getTrackerQueryID() .'">'. $Lang->getText('TrackerQuery.ExportXML').'</a>  ';
		//$temprow .= '<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerQueryPDF').'&amp;tracker_id='.$Tracker->getTrackerId().'&amp;tracker_query_id='.$item->getTrackerQueryID() .'">'. $Lang->getText('TrackerQuery.ExportPDF').'</a>  ';
		$trow[]= $temprow;
	//	$trow[]='<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerQueryPie').'&amp;tracker_id='.$Tracker->getTrackerId().'&amp;tracker_query_id='.$item->getTrackerQueryID() .'">'. $Lang->getText('TrackerQuery.PieGraph').'</a>  <a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerQueryBar').'&amp;tracker_id='.$Tracker->getTrackerId().'&amp;tracker_query_id='.$item->getTrackerQueryID() .'">'. $Lang->getText('TrackerQuery.BarGraph').'</a>';

		echo $theme->tableRow($trow);
	}

}
echo $theme->tableBottom(array(GFUrl::getUrl()->currentBaseUrl('TrackerQueryAdd').'&amp;tracker_id='.$Tracker->getTrackerId()=>$Lang->getText('TrackerQuery.AddNew')));

$theme->footer($params);
?>
