<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code lists TrackerItem
//
require_once('plugins/tracker/lib/TrackerItemPeer.php');
require_once('plugins/tracker/lib/TrackerItemSavedQuery.php');
require_once('plugins/tracker/lib/TrackerItemQuickQuery.php');
require_once('plugins/tracker/lib/TrackerQuery.php');
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('StringUtils.php');
require_once('exceptions/PermissionDeniedException.php');

$session = Session::getSession();
$user = $session->getUser();
$Lang = $session->getLanguage();

$Project = GFUrl::getUrl()->getProject();
$Tracker = TrackerPeer::retrieveByPk($_REQUEST['tracker_id']);
$pref_query = null;

if (!$Tracker) {
	throw new InvalidDataException('Invalid tracker id');
}

if ($session->isLoggedIn()) {
	//See if there is a quick query stored, in case there is no tracker query id passed through url
	$preference_name = 'quickquery'.$Tracker->getTrackerId();
	$pref_query = $user->getPreference($preference_name);
}


if (isset($_GET['tracker_query_id'])) {
	if (!$session->isLoggedIn()) {
		throw new PermissionDeniedException();
	}
	$tsq = TrackerQueryPeer::retrieveByPk($_GET['tracker_query_id']);
	if (!$tsq->getIsShared() && $tsq->getUserId() != $session->getUser()->getUserId()) {
		throw new PermissionDeniedException();
	}
	$tq = new TrackerItemSavedQuery($Tracker, $tsq, $session->getUser());
} else {
	$tq = new TrackerItemQuickQuery($Tracker);
}

$separator = ',';

if (is_a($tq,'TrackerItemSavedQuery')) {
	$filename = StringUtils::removeInvalidChars(str_replace(' ', '_',$tsq->getQueryName()));
} else {
	$filename = 'quickquery';
}


$totalcount=$tq->getTrackerItemCount();
/**
 * @var TrackerItem[] $arr
 */
$arr=$tq->getTrackerItems();

$header = array();
$header[] = 'TrackerItemID';
$header[] = $Lang->getText('TrackerItem.SummaryName');
if ($tq->showColumn('priority'))
	$header[] = $Lang->getText('TrackerItem.PriorityName');
if ($tq->showColumn('assigned_to'))
	$header[] = $Lang->getText('TrackerItem.AssignedTo');
if ($tq->showColumn('submitted_by'))
	$header[] = $Lang->getText('TrackerItem.SubmittedByName');

$efs=$Tracker->getTrackerExtraFields();
usort($efs, array($tq,'sortByDisplayOrder'));

foreach ($efs as $ef) {
	$efcache[$ef->getTrackerExtraFieldId()]=$ef;
	if ($tq->showColumn($ef->getId())) {
		$header[] = $ef->getFieldName();
	}
}


if ($tq->showColumn('open_date'))
	$header[] = $Lang->getText('TrackerItem.OpenDateName');
if ($tq->showColumn('close_date'))
	$header[] = $Lang->getText('TrackerItem.CloseDateName');
if ($tq->showColumn('last_modified_date'))
	$header[] = $Lang->getText('TrackerItem.LastModifiedDateName');

//Follow-ups
$header[] = 'Follow-ups';
header("Content-Type: application/force-download\n");
header("Expires: 0");
header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
header("content-disposition: attachment;filename=$filename.csv");

echo implode($separator, StringUtils::cleanForCSV($header, $separator)) . "\r\n";

for ($i=0; $i<count($arr); $i++) {
	$row = array();
	$row[]= $arr[$i]->getTrackerItemId();
	$row[]= $arr[$i]->getSummary();
	if ($tq->showColumn('priority'))
		$row[]=$arr[$i]->getPriority();
	if ($tq->showColumn('assigned_to')) {
		$asses = $arr[$i]->getTrackerItemAssignees();
		$assignees = array();
		foreach ($asses as $ass) {
			$assignees[] = trim($ass->getUser()->getUnixName());
		}
		$row[]=implode(':', $assignees);
		unset($assignees);
	}
	if ($tq->showColumn('submitted_by'))
		$row[]= trim(UserPeer::retrieveByPk($arr[$i]->getSubmittedBy())->getUnixName());

	// Begin Extra Fields //
	$colarray=array();
	$efdatas=$arr[$i]->getTrackerExtraFieldDatas();
	foreach($efdatas as $efdata) {
		//Members type needs an special treatment
		$field_type = $efcache[$efdata->getTrackerExtraFieldId()]->getFieldType();
		if($field_type == TrackerExtraField::TYPE_MEMBERS){
			$member = UserPeer::retrieveByPk($efdata->getFieldData());
      if (is_object($member)) {
        $colarray[$efdata->getTrackerExtraFieldId()][] = $member->getUnixName();
      } else {
        $colarray[$efdata->getTrackerExtraFieldId()][] = "None";
      }
		} else {
			$colarray[$efdata->getTrackerExtraFieldId()][]=$efcache[$efdata->getTrackerExtraFieldId()]->translateFieldDataToName($efdata->getFieldData());
		}

	}
	reset($efs);
	foreach ($efs as $ef) {
		if ($tq->showColumn($ef->getId())) {
			if (isset($colarray[$ef->getTrackerExtraFieldId()]) && count($colarray[$ef->getTrackerExtraFieldId()]) > 1) {
				$row[] = implode(':',$colarray[$ef->getTrackerExtraFieldId()]);
			} else {
				if (isset($colarray[$ef->getTrackerExtraFieldId()])) {
					if ($ef->getFieldType() == TrackerExtraField::TYPE_DATE) {
						$row[] = date('Y-m-d', strtotime($colarray[$ef->getTrackerExtraFieldId()][0]));
					} else {
						$row[] = $colarray[$ef->getTrackerExtraFieldId()][0];
					}
				} else {
					$row[] = 'None';
				}
			}
		}
	}
	// End Extra Fields //

	if ($tq->showColumn('open_date'))
		$row[]=$arr[$i]->getOpenDate('Y-m-d');
	if ($tq->showColumn('close_date'))
		$row[]=$arr[$i]->getCloseDate('Y-m-d');
	if ($tq->showColumn('last_modified_date'))
		$row[]=$arr[$i]->getLastModifiedDate('Y-m-d');

	//Get Messages
	/**
	 * @var TrackerITemMessage[] $tims
	 */

	$tims = $arr[$i]->getTrackerItemMessages();

	$messages = array();
	foreach($tims as $tim) {
		$tmp = " --- Posted By " . $tim->getUser()->getUnixName() . " on ".
			$tim->getAdddate() .": ";
		$tmp .= $tim->getBody(). '   '. PHP_EOL;
		$messages[] = $tmp;
	}
	$row[] .= implode(PHP_EOL, $messages);
	echo implode($separator, StringUtils::cleanForCSV($row, $separator)) . "\r\n";
	unset($row);
}


?>
