<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code shows a form for editing TrackerQuery
//
require_once('plugins/tracker/lib/TrackerQueryPeer.php');
require_once('plugins/tracker/lib/TrackerQuery.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('plugins/tracker/lib/html/TrackerExtraFieldRenderer.php');
require_once('plugins/tracker/lib/TrackerItemSavedQuery.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

require_once('plugins/tracker/lib/html/TrackerTemplate.php');
$theme = new TrackerTemplate();

//must be logged in
if (!$session->isLoggedIn()) {
      throw new NotLoggedInException();
}

$Project = GFUrl::getUrl()->getProject();
$Tracker = TrackerPeer::retrieveByPk($_REQUEST['tracker_id']);
$tefr = new TrackerExtraFieldRenderer($Tracker);

$crit = new Criteria();
// [#9237] - Allow project admins to edit any shared query
// Only filter by user if the user is NOT a project admin
if ($Tracker->userIsAdmin() == FALSE) {
    $crit->add(TrackerQueryPeer::USER_ID, $session->getUser()->getID());
}
$crit->add(TrackerQueryPeer::TRACKER_QUERY_ID,$_REQUEST['tracker_query_id']);
$arr = $Tracker->getTrackerQuerys($crit);
$obj=$arr[0];
if (!is_object($obj)) {
      throw new PermissionDeniedException();
}
$tq=new TrackerItemSavedQuery($Tracker,$obj,$session->getUser());

//Tracker extra fields available to display selection
$tefs = $Tracker->getTrackerExtraFields();

$tefs_arr = array();
foreach($tefs as $tef){
	$tefs_arr[$tef->getId()] = $tef->getName();
}

$crit = new Criteria();
$crit->add(TrackerQueryDisplayColumnPeer::TRACKER_QUERY_ID, $_REQUEST['tracker_query_id']);
$crit->addAscendingOrderByColumn(TrackerQueryDisplayColumnPeer::DISPLAY_FIELD_ORDER);
$tefs_disp_cols = TrackerQueryDisplayColumnPeer::doSelect($crit);
//$tefs_disp_cols = array(); /*Read from DB, ordered by sort order */
//foreach($tefs_disp_arr as $tef_disp){
//	$tefs_disp_cols[$tef_disp->getDisplayFieldId()] = $tefs_arr[$tef_disp->getDisplayFieldId()];
//}


$params = array(
		'project' => $Project,
		'tracker' => $Tracker,
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'action' => isset($action) ? $action : '',
		'title' => $Lang->getText('TrackerQuery.EditPageTitle'),
		'extra_head' => array('<script language="javascript" type="text/javascript" src="/selectbox.js"></script>')
	);

//$tracker_id = ( (isset($_POST) && isset($_POST['tracker_id'])) ? htmlspecialchars($_POST['tracker_id']) : $obj->getTrackerId() );
$is_shared = ( (isset($_POST) && isset($_POST['is_shared'])) ? Validator::sanitizeBool($_POST['is_shared']) : $obj->getIsShared() );
$query_name = ( (isset($_POST) && isset($_POST['query_name'])) ? htmlspecialchars($_POST['query_name']) : $obj->getQueryName() );
$sortcol = ( (isset($_POST) && isset($_POST['sortcol'])) ? $_POST['sortcol'] : $tq->getSortCol() );
$sortord = ( (isset($_POST) && isset($_POST['sortord'])) ? $_POST['sortord'] : $tq->getSortOrd() );
//$is_selected = ( (isset($_POST) && isset($_POST['is_selected'])) ? htmlspecialchars($_POST['is_selected']) : $obj->getIsSelected() );

// Default values for the date fields

list($moddatestart, $moddateend) = GetDateFieldValues($tq->getModDateRange(), 'moddatestart', 'moddateend');

list($opendatestart, $opendateend) = GetDateFieldValues($tq->getOpenDateRange(), 'opendatestart', 'opendateend');

list($closedatestart, $closedateend) = GetDateFieldValues($tq->getCloseDateRange(), 'closedatestart', 'closedateend');

$efs_selected=array();
if (isset($_POST['efs_display'])){
	$efs_tmp=$_POST['efs_display'];
	foreach($efs_tmp as $ef){
		$efs_selected[$ef] = $tefs_arr[$ef];
	}
} else if(!empty($tefs_disp_cols)){
	//Fields are in the DB
	foreach($tefs_disp_cols as $tefdc){
		$efs_selected[$tefdc->getDisplayFieldId()] =  $tefs_arr[$tefdc->getDisplayFieldId()];
		unset($tefs_arr[$tefdc->getDisplayFieldId()]);
	}
}

function GetDateFieldValues($dbValue, $postedStartField = null, $postedEndField = null, $defaultValue = '') {
    
    // Lowest priority = default value
    $start = $defaultValue;
    $end = $defaultValue;
    
    // Second-lowest priority = current db value
    if (stripos($dbValue, TrackerItemQuery::DATE_RANGE_DELIMITER)) {
        list($start, $end) = explode(TrackerItemQuery::DATE_RANGE_DELIMITER, $dbValue);
    }
    
    // Highest-priority = posted value from user
    if (isset($postedStartField) && array_key_exists($postedStartField, $_POST)) {
        $start = htmlspecialchars($_POST[$postedStartField]);
    }
    if (isset($postedEndField) && array_key_exists($postedEndField, $_POST)) {
        $end = htmlspecialchars($_POST[$postedEndField]);
    }
    
    return array($start, $end);
}

include ('forms/TrackerQueryEdit-tpl.php');
