<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code creates a new Tracker and saves it to the
// database
//
require_once('plugins/tracker/lib/Tracker.php');
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$session = Session::getSession();
$Lang = $session->getLanguage();

$Project = GFUrl::getUrl()->getProject();
$Tracker = new Tracker();

$errors = array();
$language = Language::getSessLang();
$stringUtils = new StringUtils();

if (isset($_POST['clone_tracker']) && !$_POST['clone_tracker_id']) {
	//User chose to clone tracker but didn't select any tracker
	$errors[] = $language->getText('Tracker.SelectCloneTracker');
}


if (count($errors) == 0) {
	$Tracker->setProject( $Project );
	$Tracker->setTrackerName( Validator::sanitizeString($_POST['tracker_name']) );
	$Tracker->setDescription( Validator::sanitizeString($_POST['description']) );
	$Tracker->setIsPublic( isset($_POST['is_public']) ? true : false );
	$Tracker->setRestrictBrowse( isset($_POST['restrict_browse']) ? true : false );
	$Tracker->setEmailAllUpdates( isset($_POST['email_all_updates']) ? true : false );
	$Tracker->setEmailAddress( Validator::sanitizeString($_POST['email_address']) );
	$duedays=isset($_POST['due_period']) ? Validator::sanitizeInt($_POST['due_period']) : 15;
	$Tracker->setDuePeriod( $duedays*60*60*24 );
	//The following fields are cleaned by html purify
	$Tracker->setSubmitInstructions($_POST['submit_instructions']);
	$Tracker->setBrowseInstructions($_POST['browse_instructions']);
	$Tracker->setAutoresponseInstructions( $stringUtils->stripHTMLTags($_POST['autoresponse_instructions']) );

	$Tracker->setShowDependenciesTab(isset($_POST['show_dependencies_tab']) ? true : false);
	$Tracker->setShowDuplicatesTab(isset($_POST['show_duplicates_tab']) ? true : false);
	$Tracker->setShowAssociationsTab(isset($_POST['show_associations_tab']) ? true : false);
	$Tracker->setShowTimeTrackingTab(isset($_POST['show_time_tracking_tab']) ? true : false);
	$Tracker->setShowTagsTab(isset($_POST['show_tags_tab']) ? true : false);

	$Tracker->setDatatype( isset($_POST['datatype']) ? Validator::sanitizeInt($_POST['datatype']) : Tracker::DATATYPE_TRACKER );

	$isEmailAliasPost = true;//flag if emailAlias was POSTed
	$email_alias = empty($_POST['email_alias']) ? '' : trim($_POST['email_alias']);
	if (! empty($email_alias)) {
		$email_alias  = Validator::sanitizeString($_POST['email_alias']);
	} else {
		$email_alias = $Tracker->getDefaultEmailAlias();
		$isEmailAliasPost = false;
	}
	if (isset($_POST['clone_tracker'])) {
		$clone_tracker = TrackerPeer::retrieveByPk($_POST['clone_tracker_id']);
	} else {
		$clone_tracker = null;
	}

	try {
		if ($email_alias) {
			$msg = '';
			if (!Validator::validateEmail($email_alias.'@'.GFConf::get('system.hostName'))) {
				$msg = $language->getText('Tracker.InvalidEmail');
			}
			if (!Validator::validateEmailAlias($email_alias)) {
				$msg = $language->getText('Tracker.reservedEmailAlias', array($email_alias, $Tracker->getProject()->getUnixName()));
			}
			if (!$Tracker->isEmailAliasUnique($email_alias)) {
				$msg = $language->getText('Tracker.EmailAddressFromError', array($email_alias, $Tracker->getProject()->getUnixName() ));
			}
			if (!empty($msg)) {//error detected
				if ($isEmailAliasPost) {//user POSTed email alias, return error
					throw new InvalidDataException(array($msg));
				} else {// created default alias failed, just set empty
					$email_alias = '';
				}
			}
		}

		$Tracker->save();

		if (!is_null($clone_tracker)) {
			// copy the fields from some other tracker
			$Tracker->cloneFields($clone_tracker);
		} else {
			// if we're not cloning the tracker, create some default fields
			$Tracker->buildDefaultFields($support);
		}



		try {
			require_once('plugins/tracker/lib/TrackerQueryPeer.php');
			TrackerQueryPeer::addStandardQueries($Tracker->getTrackerId());
		} catch (Exception $ex) {
			$msg = "Could not create standard queries for new Tracker {$Tracker->getTrackerName()}: {$ex->getMessage()}";
			throw new InvalidDataException(array($msg));
		}

		//Set notification email settings
		if ($email_alias) {
			$Tracker->setEmailAlias($email_alias);
		}

		GFUrl::getUrl()->redirect(GFUrl::getFullURL(GFUrl::getUrl()->projectURL($Tracker->getProject(), '', 'tracker','',array('feedback' => $language->getText('Tracker.AddFeedback')))));
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());		// attach object errors to validation errors
		$feedback = '* '.implode('<br />* ', $errors);
		include('TrackerAdd.php');
	}
} else {
	$feedback = '* '.implode('<br />* ', $errors);
	include('TrackerAdd.php');
}

?>
