<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a Tracker and saves it to the 
// database
//
require_once('plugins/tracker/lib/Tracker.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');
require_once('StringUtils.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}
/** @var Tracker $Tracker */
$Tracker = TrackerPeer::retrieveByPk($_REQUEST['tracker_id']);
if (!is_object($Tracker) || !$Tracker->userIsAdmin()) {
	throw new PermissionDeniedException();
}

$errors = array();
$language = Language::getSessLang();
$stringUtils = new StringUtils();

//
// TODO Validate the values passed by $_POST. Store the errors in $error like
// $errors["field_name"] = error_description
//
if (count($errors) == 0) {
	$Tracker->setTrackerName( Validator::sanitizeString($_POST['tracker_name']) );
	$Tracker->setDescription( Validator::sanitizeString($_POST['description']) );
	$Tracker->setIsPublic( isset($_POST['is_public']) ? true : false );
	$Tracker->setRestrictBrowse( isset($_POST['restrict_browse']) ? true : false );
	$Tracker->setEmailAllUpdates( isset($_POST['email_all_updates']) ? true : false );
	$Tracker->setEmailAddress( Validator::sanitizeString($_POST['email_address']) );
	$duedays=isset($_POST['due_period']) ? Validator::sanitizeInt($_POST['due_period']) : 15;
	$Tracker->setDuePeriod( $duedays*60*60*24 );
	$Tracker->setSubmitInstructions( $stringUtils->cleanHtml($_POST['submit_instructions']) );
	$Tracker->setBrowseInstructions( $stringUtils->cleanHtml($_POST['browse_instructions']) );
	$Tracker->setAutoresponseInstructions( $stringUtils->stripHTMLTags($_POST['autoresponse_instructions']) );
	
	$Tracker->setShowDependenciesTab( isset($_POST['show_dependencies_tab']) ? true : false );
	$Tracker->setShowDuplicatesTab( isset($_POST['show_duplicates_tab']) ? true : false );
	$Tracker->setShowAssociationsTab( isset($_POST['show_associations_tab']) ? true : false );
	$Tracker->setShowTimeTrackingTab( isset($_POST['show_time_tracking_tab']) ? true : false );
	$Tracker->setShowTagsTab( isset($_POST['show_tags_tab']) ? true : false );
	$Tracker->setSendAbbreviatedEmail( isset($_POST['send_abbreviated_email']) ? true : false );

	$email_alias  = isset($_POST['email_alias']) ? Validator::sanitizeString($_POST['email_alias']) : $Tracker->getEmailAlias();

	$arr_status_nagtime = (isset($_POST['arr_status_nagtime']) && is_array($_POST['arr_status_nagtime'])) ? $_POST['arr_status_nagtime'] : array();

	try {
		if ($email_alias) {
			if (!Validator::validateEmail($email_alias.'@'.GFConf::get('system.hostName'))) {
				$msg = $language->getText('Tracker.InvalidEmail');
				throw new InvalidDataException(array($msg));
			}
			if (!Validator::validateEmailAlias($email_alias)) {
				$msg = $language->getText('Tracker.reservedEmailAlias', array($email_alias, $Tracker->getProject()->getUnixName()));
				throw new InvalidDataException(array($msg));
			}
			if (!$Tracker->isEmailAliasUnique($email_alias)) {
				$msg = $language->getText('Tracker.EmailAddressFromError', array($email_alias, $Tracker->getProject()->getUnixName() ));
				throw new InvalidDataException(array($msg));
			}

			if (count($Tracker->getTrackerExtraFieldsRequiredWithNoDefaultSet())) {
				$msg = $language->getText('Tracker.TrackerExtraFieldsRequiredWithNoDefault');
				throw new InvalidDataException(array($msg));
			}
		}

		$Tracker->save();

		$arr_status_nagtime = (isset($_POST['arr_status_nagtime']) && is_array($_POST['arr_status_nagtime'])) ? $_POST['arr_status_nagtime'] : array();
		$arr_customer_status_nagtime = (isset($_POST['arr_customer_status_nagtime']) && is_array($_POST['arr_customer_status_nagtime'])) ? $_POST['arr_customer_status_nagtime'] : array();
	
		$enable_nag_alerts = isset($_POST['enable_nag_alerts']) ? true : false;
		$enable_customer_nag_alerts = isset($_POST['enable_customer_nag_alerts']) ? true : false;
		$Tracker->setStatusNag($enable_nag_alerts);
		$Tracker->setStatusNag($enable_customer_nag_alerts, true);
		$Tracker->setStatusNagText($_POST['tracker_status_nag_text']);
		$Tracker->setStatusNagText($_POST['tracker_customer_status_nag_text'], true);

		$Tracker->setAutoCloseStatus(Validator::sanitizeInt($_POST['auto_close_status']));
		$Tracker->setAutoCloseSetting(Validator::sanitizeInt($_POST['auto_close_interval']));

		//Process nagtime array
		foreach($arr_status_nagtime as $element_id => $status_nagtime) {
			$status_nagtime = Validator::sanitizeInt($status_nagtime);
			if ($status_nagtime == 0) {
				$status_nagtime = Tracker::DEFAULT_NAG_PERIOD;
			}

			//Update setting for this nagtime value
			$Tracker->setStatusNagtime($element_id, $status_nagtime);
		}

		//Process customer nagtime array
		foreach($arr_customer_status_nagtime as $element_id => $status_nagtime) {
			$status_nagtime = Validator::sanitizeInt($status_nagtime);
			if ($status_nagtime == 0) {
				$status_nagtime = Tracker::DEFAULT_NAG_PERIOD;
			}
			
			//Update setting for this nagtime value
			$Tracker->setStatusNagtime($element_id, $status_nagtime, true);
		}
		
		if (isset($_REQUEST['redirect_to'])) {
			$redirect_to = StringUtils::stripHtmlTags($_REQUEST['redirect_to']);
		} else {
			$redirect_to = GFUrl::getFullURL(GFUrl::getUrl()->projectURL($Tracker->getProject(), '', 'tracker','',array()));
		}
		
      //Set notification email settings
		if ($email_alias) {
			$Tracker->setEmailAlias($email_alias);
		} else {
			$Tracker->unsetEmailAlias();
		}

		
		GFUrl::getUrl()->redirect($redirect_to, array('feedback' => $language->getText('Tracker.EditFeedback')));
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());
		$feedback = '* '.implode('<br />* ', $errors);
		include('TrackerEdit.php');
	}
} else {
	$feedback = implode('<br />', $errors);
	include('TrackerEdit.php');
}

?>
