<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a TrackerExtraField and saves it to the 
// database
//
require_once('plugins/tracker/lib/TrackerExtraField.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$Tracker = TrackerPeer::retrieveByPk($_REQUEST['tracker_id']);
if (!is_object($Tracker) || !$Tracker->userIsAdmin()) {
	throw new PermissionDeniedException();
}

$tefs= $Tracker->getTrackerExtraFields();

foreach ($tefs as $tef) {
	$tef->setFieldOrder( Validator::sanitizeInt($_REQUEST['field_order'][$tef->getTrackerExtraFieldId()]) );
	if ($_REQUEST['auto_assign_to'] == $tef->getTrackerExtraFieldId()) {
		$tef->setUseAutoAssign(true);
	} else {
		$tef->setUseAutoAssign(false);
	}
	if (isset($_REQUEST['use_workflow']) && isset($_REQUEST['use_workflow'][$tef->getTrackerExtraFieldId()])) {
		$tef->setUseWorkflow(true);
	} else {
		$tef->setUseWorkflow(false);
	}
	if (isset($_REQUEST['use_commit_rule']) && isset($_REQUEST['use_commit_rule'][$tef->getTrackerExtraFieldId()])) {
		$tef->setUseCommitRule(true);
	} else {
		$tef->setUseCommitRule(false);
	}
	if (isset($_REQUEST['is_required'][$tef->getTrackerExtraFieldId()]) || isset($_REQUEST['show_on_submit'][$tef->getTrackerExtraFieldId()]) || ($_REQUEST['auto_assign_to'] == $tef->getTrackerExtraFieldId())) {
		$tef->setShowOnSubmit(true);
	} else {
		$tef->setShowOnSubmit(false);
	}
	if (isset($_REQUEST['show_on_browse'][$tef->getTrackerExtraFieldId()])) {
		$tef->setUseInQuickBrowse(true);
	} else {
		$tef->setUseInQuickBrowse(false);
	}
	if (isset($_REQUEST['is_required'][$tef->getTrackerExtraFieldId()])) {
		$tef->setIsRequired(true);
	} else {
		$tef->setIsRequired(false);
	}
	try {	
		$tef->save();
	} catch (InvalidDataException $ex) {
		//print_r($ex->getErrors());
		//exit;
		//$errors = array_merge($errors, $ex->getErrors());               // attach object errors to validation errors
		$feedback = implode('<br />', $ex->getErrors());
		GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerExtraFieldBrowse'), array('feedback'=>$feedback,'tracker_id'=>$Tracker->getTrackerId()));
		exit;
	}
}

//Process intrinsic fields

$intrinsic_fields = $Tracker->getIntrinsicFieldsSettings();
foreach ($intrinsic_fields as $name => $field) {
	$json = SettingPeer::GetSetting(Tracker::SECTION, $Tracker->getTrackerId(), $name);
	$setting = json_decode($json);
	$required  = isset($_REQUEST['is_required_' . $name]);
	$enabled   = isset($_REQUEST['is_enabled_' . $name]);
	$submit    = isset($_REQUEST['show_on_submit_' . $name]);
	$browse    = isset($_REQUEST['show_on_browse_' . $name]);
	$order     = isset($_REQUEST['field_order_' . $name]) ?
	Validator::sanitizeInt($_REQUEST['field_order_']) : 0;

	$setting->is_required = $required;
	$setting->is_enabled = $enabled;
	$setting->on_submit = $submit;
	$setting->on_browse = $browse;
	$setting->field_order = $order;

	$newJson = json_encode($setting);
	SettingPeer::SaveSetting(Tracker::SECTION, $Tracker->getTrackerId(), $name, $newJson, $Tracker->getProjectId());
	$Tracker->flushIntrinsicFieldSettingsCache();
}

$feedback=$language->getText('TrackerExtraField.EditFeedback');
GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerExtraFieldBrowse'), array('feedback'=>$feedback,'tracker_id'=>$Tracker->getTrackerId()));
?>
