<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugins/tracker/lib/TrackerExtraField.php');
require_once('plugins/tracker/lib/TrackerExtraFieldElement.php');
require_once('plugins/tracker/lib/TrackerExtraFieldElementTransition.php');
require_once('exceptions/MissingDataException.php');

$session = Session::getSession();
$Lang = $session->getLanguage();
$Project = GFUrl::getUrl()->getProject();


if (!isset($_GET['tracker_extra_field_id'])) {
	throw new MissingDataException();
}

$extra_field = TrackerExtraFieldPeer::retrieveByPk($_GET['tracker_extra_field_id']);
$tracker = $extra_field->getTracker();
if (!$tracker->userIsAdmin()) {
	throw new PermissionDeniedException();
}

require_once('plugins/tracker/lib/html/TrackerAdminTemplate.php');
$theme = new TrackerAdminTemplate();

$theme->addBreadCrumb('<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldElementBrowse', array('tracker_extra_field_id' => $_REQUEST['tracker_extra_field_id'])).'">'.$extra_field->getFieldName().'</a>');
$theme->addBreadCrumb('<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldBrowse').'&amp;tracker_id='.$tracker->getTrackerID() .'">'. $Lang->getText('TrackerExtraField.Edit') .'</a>');

$params = array(
		'project' => $Project,
		'tracker' => $tracker,
		'action' => isset($action) ? $action : '',
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('WorkflowMatrix.TransitionMatrix')
		);
$theme->header($params);




$element_map = array();

// Get all the elements for this field
$c = new Criteria();
$c->add(TrackerExtraFieldElementPeer::TRACKER_EXTRA_FIELD_ID, $extra_field->getTrackerExtraFieldId());
$c->addAscendingOrderByColumn(TrackerExtraFieldElementPeer::SORT_ORDER);
$available_elements = TrackerExtraFieldElementPeer::doSelect($c);

// Create the initial matrix
$n = count($available_elements);
for ($i = 0; $i < $n; $i++) {
	$element_from = $available_elements[$i]->getElementId();
	
	// This is just used for quick access to the objects
	$element_map[$element_from] = $available_elements[$i];
	
	for ($j = 0; $j < $n; $j++) {
		$element_to = $available_elements[$j]->getElementId();
		$transition_matrix[$element_from][$element_to] = 0;
	}
	
	// Get the available transitions for this element
	$c = new Criteria();
	$c->add(TrackerExtraFieldElementTransitionPeer::ELEMENT_ID, $element_from);
	$available_transitions = TrackerExtraFieldElementTransitionPeer::doSelect($c);
	foreach ($available_transitions as $available_transition) {
		$available_to = $available_transition->getToElementId();
		if (!array_key_exists($available_to, $transition_matrix[$element_from])) {
			// Shouldn't happen -- means that the to_element_id doesn't exist for this tracker extra field
			continue;
		}
		$transition_matrix[$element_from][$available_to] = $available_transition->getTrackerExtraFieldElementTransitionId();
	}
}

// Now display the matrix
// (note: there's an HTML hack to properly display the table header. It should actually call the $theme
//  methods, but there's no way to construct this table layout using those methods)
?>
<div class="infobox">
<?php echo $Lang->getText('WorkflowMatrix.info'); ?>
</div>
<form method="post" action="<?php echo GFUrl::getUrl()->currentBaseUrl('WorkflowMatrixAction', array('tracker_extra_field_id' => $_GET['tracker_extra_field_id'])); ?>">
<?php

$theader = array();
$theader[] = $Lang->getText('WorkflowMatrix.FromState');
for ($i = 0; $i < $n; $i++) {
	$theader[] = $Lang->getText('WorkflowMatrix.ToState');
}
echo $theme->tableTop($theader);

$trow = array();
$trow[] = '&nbsp;';
for ($i = 0; $i < $n; $i++) {
	$trow[] = $available_elements[$i]->getElementName();
}
echo $theme->tableRow($trow);


for ($i = 0; $i < $n; $i++) {
	$trow = array();
	$trow[] = $available_elements[$i]->getElementName();
	$element_from = $available_elements[$i]->getElementId();
	for ($j = 0; $j < $n; $j++) {
		$element_to = $available_elements[$j]->getElementId();
		$has_transition = ($transition_matrix[$element_from][$element_to] != 0);
		if ($i == $j) {
			$trow[] = '&nbsp;';
		} elseif ($has_transition) {
			$transition_id = $transition_matrix[$element_from][$element_to];
			$text = '<input type="checkbox" name="matrix['.$element_from.']['.$element_to.']" value="1" checked="checked" /> ';
                        //Searchs the transition by transition_id
                        $c = new Criteria();
                        $c->add(TrackerExtraFieldElementTransitionPeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID, $transition_id);
                        $transitions = TrackerExtraFieldElementTransitionPeer::doSelect($c);
                        
                        $roles = false;
						$tooltipDescription = '';
                        foreach ($transitions as $transition) {
                            //Looking for the tooltip infomation                            
                            if ($transition->getTrackerExtraFieldElementTransitionRoles()) {
								$change_field = '';
	                            $c = new Criteria();
	                            $c->add(TrackerEfeTransitionTriggerPeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID, $transition_id);
	                            $trigger = TrackerEfeTransitionTriggerPeer::doSelectOne($c);
	                            if (is_object($trigger)) {
		                            $extra_field_id = $trigger->getChangeTrackerExtraFieldId();
		                            $to_element_id = $trigger->getToElementId();

		                            $extraField = TrackerExtraFieldPeer::retrieveByPk($extra_field_id);
		                            $extraFieldElement = TrackerExtraFieldElementPeer::retrieveByPk($to_element_id);
		                            if (is_object($extraField) && is_object($extraFieldElement)) {
			                            $change_field = $Lang->getSessText('TrackerExtraFieldElementTransition.ChangeFieldNameTooltip', array($extraField->getFieldName(), $extraFieldElement->getElementName()));
		                            }
	                            }

                                $roles = true;
	                            $arrRoles = array();
                                foreach ($transition->getTrackerExtraFieldElementTransitionRoles() as $rol){
	                                $arrRoles[] = $rol->getRole()->getName();
                                }
	                            $tooltipDescription .= (sizeof($arrRoles) > 0) ? "Roles: " . implode(',', $arrRoles) : '';

	                            $arrRequiredFields = array();
                                foreach ($transition->getTrackerExtraFieldElementTransitionFields() as $fields){
	                                $arrRequiredFields[] = $fields->getTrackerExtraField()->getName();
                                }
	                            $tooltipDescription .= (sizeof($arrRequiredFields) > 0) ? "</br>Required fields: " . implode(',', $arrRequiredFields) : '';

                                if (count($transition->getTrackerExtraFieldElementTransitionNotifys()) > 0){
	                                $arrNotifies = array();
                                    foreach ($transition->getTrackerExtraFieldElementTransitionNotifys() as $notifies){
                                        $c = new Criteria();
                                        $c->add(BaseRolePeer::ROLE_ID, $notifies->getRefId());
                                        $rolesSelect = BaseRolePeer::doSelectOne($c);
	                                    $arrNotifies[] = $rolesSelect->getName();
                                    }
	                                $tooltipDescription .= (sizeof($arrNotifies) > 0) ? "</br>Notifies on change: " . implode(',', $arrNotifies) : '';
                                }
                                if ($transition->getNotificationMessage()){
                                     $tooltipDescription .= "</br>Notification message: " . $transition->getNotificationMessage();
                                }
                                if (count($transition->getTrackerEfeTransitionReassignsJoinRole()) > 0){
	                                $arrReassignTo = array();
                                    foreach ($transition->getTrackerEfeTransitionReassignsJoinRole() as $reassign){
	                                    $arrReassignTo = $reassign->getRole()->getName();
                                    }
	                                $tooltipDescription .= "</br>Reassign to: " . $arrReassignTo;
                                }

	                            if ($change_field != '') {
		                            $tooltipDescription .= '<br>' . $change_field;
	                            }
                            }

                        }

						//Preprocesses tooltip string
						$tooltipDescription = StringUtils::generateTooltip($tooltipDescription);

                        //If user defined some rules it shows a blue ball and a info resume tooltip
                        if ($roles){
                            $text .= "<span onMouseOver=\"tooltip('{$tooltipDescription}')\" onmouseout=\"tooltipexit()\"><a href=\"".GFUrl::getUrl()->CurrentBaseUrl('TrackerExtraFieldElementTransitionEdit', array('tracker_extra_field_element_transition_id' => $transition_id, 'element_id' => $element_from, 'return_to' => 'matrix')).'">'.$Lang->getText('WorkflowMatrix.Details')."</a><img class=\"workflowBallTooltip\" src=\"/images/blue-ball.gif\"/></span>";
                        } else {
                        //If the user didn't define some rules it shows a grey ball tooltip with a "No transitions rules defined" note
                            $text .= "<span  onMouseOver=\"tooltip('No transitions rules defined')\" onmouseout=\"tooltipexit()\"><a href=\"".GFUrl::getUrl()->CurrentBaseUrl('TrackerExtraFieldElementTransitionEdit', array('tracker_extra_field_element_transition_id' => $transition_id, 'element_id' => $element_from, 'return_to' => 'matrix')).'">'.$Lang->getText('WorkflowMatrix.Details')."</a><img class=\"workflowBallTooltip\" src=\"/images/grey-ball.gif\"/></span>";
                        }
			$trow[] = $text;
		} else {
			$trow[] = '<input type="checkbox" name="matrix['.$element_from.']['.$element_to.']" value="1" />';
		}
	}
	echo $theme->tableRow($trow);
}

echo $theme->tableBottom();

?>
<?php echo HTMLWidget::button("submit","" .   $Lang->getText('Common.Submit')  . " ","submit","","")?>
</form>
<?php
$theme->footer($params);
?>
