<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code lists Tracker
//
require_once('plugins/tracker/lib/TrackerPeer.php');
require_once('plugins/tracker/lib/Tracker.php');
require_once('plugins/tracker/lib/html/TrackerTemplate.php');
require_once('html/Paginator.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

require_once('lib/html/ProjectTemplate.php');
$theme = new TrackerTemplate();

$Project = GFUrl::getUrl()->getProject();

$perm = $Project->getPermission();
if ($perm->check(Tracker::SECTION, 0, Tracker::PERMLEVEL_ADMIN)) {
	$isAdmin = true;
} else {
	$isAdmin = false;
}

$params = array(
		'selectedbottomtab' => 'tracker',
		'project' => $Project,
		'action' => isset($action) ? $action : '',
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('Tracker.BrowsePageTitle')
		);
$theme->addBreadCrumb('<a href="'.GFUrl::getUrl()->projectUrl($params['project'],'','tracker').'">'.$Lang->getText('Tracker.TabString').'</a>');
$theme->header($params);

//make paginator code
$total=0;
$default_sort_by=TrackerPeer::DATATYPE;

$columns = array(
TrackerPeer::DATATYPE=>$Lang->getText('Tracker.DatatypeName'),
TrackerPeer::TRACKER_NAME=>$Lang->getText('Tracker.TrackerNameName'),
TrackerPeer::DESCRIPTION=>$Lang->getText('Tracker.DescriptionName'),
TrackerPeer::ITEM_TOTAL=>$Lang->getText('Tracker.ItemTotalName'),
TrackerPeer::OPEN_COUNT=>$Lang->getText('Tracker.OpenCountName'),
);

$columns[] = $Lang->getText('Edit');
$columns[] = $Lang->getText('Delete');

// Retrieve the objects array
$crit = new Criteria();
$crit->add(TrackerPeer::PROJECT_ID, $Project->getProjectId());
//$crit->addAscendingOrderByColumn(TrackerPeer::DATATYPE);

$total = TrackerPeer::doCount($crit);

$paginator = new Paginator('br_trackers', -1, $total, $default_sort_by, $columns);

$paginator->addPageClauses($crit);

$arr = TrackerPeer::doSelect($crit);

$sort = $paginator->getSortingOptions();
$sort_by = $sort['sort_by'];
$sort_order = $sort['order'];

$theader = $paginator->getSortableTableHeaders($theme);

echo $theme->tableTop($theader);

if (count($arr) < 1) {
	echo $theme->tableFullRow($Lang->getText('Tracker.NoneFound'));
} else {
	//if sort column is tracker type custom reordering should be applied.
	// Default behavior fails because trackertype is numeric, not the label
	// shown in the column.
	if ($sort_by == $default_sort_by) {
		$arSort = array();
		foreach ($arr as $key => $value) {
			$dataTypeName = '---';
			switch($value->getDatatype()) {
				case Tracker::DATATYPE_TRACKER :
					$dataTypeName = $Lang->getText('Tracker.Issues');
					break;
				case Tracker::DATATYPE_TASK :
					$dataTypeName = $Lang->getText('Tracker.Tasks');
					break;
				case Tracker::DATATYPE_SUPPORT :
					$dataTypeName = $Lang->getText('Tracker.Support');
					break;
			}
			$arSort[$key] = strtolower($dataTypeName . '_' . $value->getTrackerName());
		}
		if ($sort_order == 'asc') {
			asort($arSort);
		} else {
			arsort($arSort);
		}
		$arSort = array_keys($arSort);//keys are the new order
		$dataArray = array();// store data with new order
		foreach ($arSort as $value) {
			$dataArray[] = $arr[$value];
		}
		$arr = $dataArray;
	}
	// end of custom sort
	reset($arr);
	$ttype=array();
	foreach ($arr as $item) {
		if ($item->userCanRead()) {
			$ttype[$item->getDataType()][]=$item;
		}
	}
	$bol_accessible = false;
	reset($arr);
	$lasttype='-1';
	/** @var Tracker $item */
	foreach ($arr as $item) {
		if ($item->userCanRead()) {
			$bol_accessible = true;
			$trow = array();
			$tmod = array();
			//if using non-standard sort, the grouping will not work
			if ($item->getDataType() != $lasttype || $default_sort_by != $sort_by) {
				$lasttype=$item->getDataType();
				if ($default_sort_by != $sort_by) {
					$tmod[]='rowspan="1"';
				} else {
					$tmod[]='rowspan="'.count($ttype[$item->getDataType()]).'" bgcolor="#FFFFFF"';
				}
				switch($item->getDatatype()) {
					case Tracker::DATATYPE_TRACKER :
						$trow[] = $Lang->getText('Tracker.Issues');
						$icon = 'tracker';
						break;

					case Tracker::DATATYPE_TASK :
						$trow[] = $Lang->getText('Tracker.Tasks');
						$icon = 'task';
						break;

					case Tracker::DATATYPE_SUPPORT :
						$trow[] = $Lang->getText('Tracker.Support');
						$icon = 'tracker';
						break;
				}
			}
			$trow[]='<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerItemBrowse').'&amp;tracker_id='.$item->getTrackerID() .'"><img src="/images/'.$icon.'_icon.gif" height="16" width="16" border="0" />&nbsp;'. $item->getTrackerName().'</a>';
			$tmod[]='';
			$trow[]=$item->getDescription();
			$tmod[]='';
			$trow[]=$item->getItemTotal();
			$tmod[]='';
			$trow[]=$item->getOpenCount();
			$tmod[]='';
			if ($item->userIsAdmin()) {
				$trow[]='<a href="'. GFUrl::getUrl()->projectURL($Project, 'TrackerEdit', 'tracker','admin',array('tracker_id'=>$item->getTrackerId())) . '">'. $Lang->getText('Edit') .'</a>';
				$tmod[]='';
				$trow[]='<a href="'. GFUrl::getUrl()->projectURL($Project, 'TrackerDelete', 'tracker','admin',array('tracker_id'=>$item->getTrackerId())) . '">'. $Lang->getText('Delete') .'</a>';
				$tmod[]='';
			} else {
				$trow[]='';
				$tmod[]='';
				$trow[]='';
				$tmod[]='';
			}
			echo $theme->tableRow($trow,$tmod, false);
		}
	}
	if ($bol_accessible == false) {
		echo $theme->tableFullRow($Lang->getText('Tracker.NoneFound'));
	}	
}

if ($isAdmin) {
	$arr_buttons = array(GFUrl::getUrl()->projectURL($Project, 'TrackerAdd', 'tracker','admin',array()) => $Lang->getText('Tracker.AddNew'));
} else {
	$arr_buttons = array();
}

echo $theme->tableBottom($arr_buttons);

if(isset($paginator) && is_object($paginator)) {
	echo '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
}


$theme->footer($params);
?>
