<?php

require_once('plugins/calendar/lib/external/activecalendar/activecalendar.php');
require_once('plugins/calendar/lib/external/activecalendar/activecalendarweek.php');
require_once('plugins/calendar/lib/CalendarEvent.php');
require_once('plugins/calendar/lib/om/BaseCalendarEvent.php');
require_once('lib/GFUrl.php');
require_once('plugins/tracker/lib/TrackerItem.php');

class UsercalendarRenderer {

	const MVIEW_MAX_EVENTS=2;
	const DAY_V = 1;
	const WEEK_V = 2;
	const MONTH_V = 3;

	//Event type
	const E_START = 1; //Event starts
	const E_END = 2; //Event ends
	const T_START = 3; //Task starts
	const T_END = 4;	//Task ends
	const E_START_END = 5; //Task starts and ends in the same day


	public $year;
	public $month;
	public $day;
	private $calendar;

	/**
	* the class constructor
	*
	*/
	function __construct($start_date, $ref_id){
		$this->year = strftime('%Y', $start_date);
		$this->month = strftime('%m', $start_date);
		$this->day = strftime('%d', $start_date);

		//Get the user calendar
		$c = new Criteria();
		$c->add(CalendarPeer::REF_ID, $ref_id);
		$c->add(CalendarPeer::SECTION, 'user');
		$this->calendar = CalendarPeer::doSelectOne($c);
		if(is_null($this->calendar) || !is_object($this->calendar)){
			throw new InvalidDataException(array("No calendar created for user"));
		}


	}

	public function getCalendar(){
		return $this->calendar;
	}


	/**
	 * builds a matrix of events according to the view type
	 *
	 * @param integer $v_type the view type
	 * @param integer $amount the amount of weeks to display (only for weeks views)
	 * 
	 * @returns array
	 */
	private function buildEventsMatrix($v_type, $amount=1){
	
		//Calculate date boundaries
		if($v_type == self::MONTH_V){
			$starting_day = 1;
			$max_days = CalendarPeer::getMonthDays($this->year, $this->month);
		} else if ($v_type == self::WEEK_V){
			$starting_day = $this->day;
			$max_days = 7 * $amount;
		} else if ($v_type == self::DAY_V){
			$starting_day = $this->day;
			$max_days = 1;
		}

		$start_date = strtotime($this->month . "/$starting_day/". $this->year . ' 00:00');
		$end_date = strtotime("+$max_days days", $start_date);

		//Get tasks

		$matrix = array();
		if($this->calendar->userIsAdmin()){
			$tasks = $this->calendar->getAllAssignedTIInTimeFrame($start_date, $end_date);

			//The assigned tracker items go first before any event
			foreach($tasks as $t){
				if($t['t'] == 'ts'){
					$key = $t['ti']->getOpenDate('%m%d');
					$matrix[$key][] = $this->generateEventContent($t['ti'], self::T_START, $v_type);
				}
				if($t['t'] == 'te'){
					$key = $t['ti']->getCloseDate('%m%d');
					$matrix[$key][] = $this->generateEventContent($t['ti'], self::T_END, $v_type);
				}
			}
		}
	
		//Get events
		$starting_events = $this->calendar->getStartingEventsInTimeFrame($start_date, $end_date);
		$ending_events = $this->calendar->getEndingEventsInTimeFrame($start_date, $end_date);

		foreach($starting_events as $se){
			if($se->userCanRead()){
				$key = $se->getFromDate('%m%d');
				if(!$se->endsSameDay()){
					$matrix[$key][] = $this->generateEventContent($se, self::E_START, $v_type);
				} else {
					$matrix[$key][] = $this->generateEventContent($se, self::E_START_END, $v_type);
				}
			}
		}

		foreach($ending_events as $ee){
			if(!$ee->endsSameDay() && $ee->userCanRead()){
				$key = $ee->getToDate('%m%d');
				//We only need events that ends on a different day
				$matrix[$key][] = $this->generateEventContent($ee, self::E_END, $v_type);
			}
		}
		//Sort events
		foreach($matrix as &$container){
			usort($container , array($this, 'cmp'));
		}
		return $matrix;
	}



	/**
	 * display a month view of a calendar
	 *
	 * @returns string the html output
	 */
	public function renderMonthView(){
		$lang = Session::getSession()->getLanguage();
		$theme = Session::getSession()->getTheme();
		$cal = new activeCalendar($this->year, $this->month, $this->day);
		$cal->enableDayLinks($this->getDayUrl('monthview'));
		$cal->setFirstWeekDay($this->calendar->getFirstDayOfWeek() - 1);
		$cal->enableMonthNav($this->getMonthUrl());
		$container = $this->buildEventsMatrix(self::MONTH_V);

		foreach($container as $events){
			//Loop trough the container, then for each day print up to MVIEW_MAX_EVENTS
			for($i = 0; $i < self::MVIEW_MAX_EVENTS ;$i++){
				if(isset($events[$i])){
					$cal->setEventContent($events[$i]['year'], $events[$i]['month'], $events[$i]['day'], $events[$i]['content'],false,false);
				}
			}
			//if the current day has more, then print a link to display those events
			$event_counter = count($events);
			if($event_counter > self::MVIEW_MAX_EVENTS){
				$popup_id = 'events_'.$events[$i]['day'];
				$popup_width = 120;
				$popup_height = $event_counter * 25;
				//If there is more events, print a link to display a popup
				$events_remaining = $event_counter - self::MVIEW_MAX_EVENTS;

				//Print the rest of the events inside the span
				$popup = '<div id="' .$popup_id. '" class="e_popup">';
				//$popup .= '<a href="javascript:void(0)" onclick="showFloatingPopup(\''. $popup_id . '\',0,0)">X</a><br />';
				for($i = self::MVIEW_MAX_EVENTS; $i < $event_counter; $i++){
					if(isset($events[$i])){
						$popup .= '<table class="eventcontent"><tbody><tr>';
						$popup .= '<td>' . $events[$i]['content'] . '</td>';
						$popup .= '</tr><tbody></table>';
						if(strlen($events[$i]['content']) > 13)
							$popup_height += 15;
					}  
				}
				$popup .='</div>';
				$cal->setEventContent($events[self::MVIEW_MAX_EVENTS]['year'], $events[self::MVIEW_MAX_EVENTS]['month'], $events[self::MVIEW_MAX_EVENTS]['day'], 
					'<span class="moreEHeader">'. $lang->getText("UserCalendar.More", array($events_remaining)).'</span>'.$popup);
									
			}

		}
		return $cal->showMonth() .  $theme->tableBottom();
	}



	/**
	 * display a week view 
	 *
	 *@param integer $weeks weeks to show, 1 to 4
	 */
	public function renderWeekView($weeks=1){
		if($weeks < 1 || $weeks > 4){
			$weeks = 1;
		}
		$lang = Session::getSession()->getLanguage();
		$cal = new activeCalendarWeek($this->year, $this->month, $this->day);
		$cal->enableDayLinks($this->getDayUrl('weekview'));
		$cal->setFirstWeekDay($this->calendar->getFirstDayOfWeek());

		//Get events
		$container = $this->buildEventsMatrix(self::WEEK_V, $weeks);

		$start_date = strtotime($this->month.'/'.$this->day.'/'.$this->year);

		foreach($container as $events){
			foreach($events as $event){
				$cal->setEventContent($event['year'], $event['month'], $event['day'], $event['content'],false,false, $start_date);
			}
		}
		return $cal->showWeeks($weeks);
	}
	
	public function renderDayView(){
		$lang = Session::getSession()->getLanguage();
		$theme = Session::getSession()->getTheme();
		$container = $this->buildEventsMatrix(self::DAY_V);
		$day_name = strftime("%A", strtotime($this->month . '/'. $this->day . '/' . $this->year));
		$month_name = strftime("%B", strtotime($this->month . '/'. $this->day . '/' . $this->year));


		$title = $lang->getText('UserCalendar.'.$day_name) . ', ' . $lang->getText('UserCalendar.'.$month_name) . ' ' . $this->day . ', ' . $this->year;
		$return = '<span class="monthname">' . $title . '</span>';
		$theader = array($lang->getText('UserCalendar.EventSubject'), $lang->getText('UserCalendar.EventDescription'));
		$return .= $theme->tableTop($theader);
		
		$event_counter = 0;
		foreach($container as $events){
			foreach($events as $event){
				if(strtotime($event['month'] . '/' . $event['day'] . '/' . $event['year']) ==
				strtotime($this->month . '/'. $this->day . '/' . $this->year)){	
					$trow = array();
					$trow[] = $event['content'];
					$trow[] = $event['description'];
					$event_counter++;
					$return .= $theme->tableRow($trow);
				}
			}
		} 
		if(!$event_counter){
			$return .= $theme->tableFullRow($lang->getText('UserCalendar.NoEventsFound'));
		}
		$return .= '</table>';
		
		
		return $return;
	}

	/**
	 * builds an url for a given day
	 *
	 *@returns string
	 */
	public function getDayUrl($return_to='monthview'){
		$my='';
		$user = GFUrl::getUrl()->getUser();
		$components = GFUrl::getUrl()->getURI();
		$arr = explode('/',$components);
		if(in_array('my', $arr)){
			$my='/my';
		}
		return '/gf/user/'.$user->getUnixName().'/usercalendar'.$my.'/?action=dayview&return_to='.$return_to;
	}

	
	public function getMonthUrl(){
		$my='';
		$user = GFUrl::getUrl()->getUser();
		$components = GFUrl::getUrl()->getURI();
		$arr = explode('/',$components);
		if(in_array('my', $arr)){
			$my='my';
		}
		return '/gf/user/'.$user->getUnixName().'/usercalendar/'.$my.'/?action=monthview';
	}

	private function generateTooltip($event){
		if(!is_object($event)){
			return;
		}
		if(($event instanceOf CalendarEvent)){
			$tooltip = "From: " . $event->getFromDate("%Y-%m-%d %H:%M") . "<br />";
			$tooltip .= "To: " . $event->getToDate("%Y-%m-%d %H:%M") . "<br /><br />";
			$tooltip .= htmlspecialchars($event->getSubject()) . '<br />';
			$tooltip .= StringUtils::trimString(htmlspecialchars($event->getDescription()), 300);
		} else if ( ($event instanceOf TrackerItem) ){
			$tooltip = "From: " . $event->getOpenDate("%Y-%m-%d") .  "<br />";
			$tooltip .= "To: " . $event->getCloseDate("%Y-%m-%d") . "<br />";
			$tooltip .= StringUtils::trimString(htmlspecialchars($event->getDetails()), 300);
		}
		// Replace single quotes with \'
		$tooltip = preg_replace('/([^\\\\])\'/', '$1\\\'', $tooltip);
		$tooltip = preg_replace('/^\'/', '\\\'', $tooltip);

		
		return $tooltip; 
	}

	private function generateEventContent($event, $e_type,$view, $start_date = null){

		if($event instanceOf CalendarEvent){
			$type = 'event';
		} else if ($event instanceOf TrackerItem) {
			$type = 'task';
		} else {
			return;
		}

		if(is_null($start_date)){
			$start_date = strtotime($this->month . '/'  . $this->day . '/' . $this->year);
		}

		$tooltip = $this->generateTooltip($event);

		if($type == 'event'){
			if($e_type == self::E_START || $e_type == self::E_START_END){
				$event_data = array(
					'sort_index'=>$event->getFromDate(null),
					'year'=>$event->getFromDate("%Y"),
					'month'=>$event->getFromDate("%m"),
					'day'=>$event->getFromDate("%d"),
				);
				if($e_type == self::E_START_END){
					$event_data['image'] = '';
				} else {
					$event_data['image'] = '<img src="/images/event_start.gif" alt="s"/>';
				}
			} else if($e_type == self::E_END){
				$event_data = array(
					'sort_index'=>$event->getToDate(null),
					'year'=>$event->getToDate("%Y"),
					'month'=>$event->getToDate("%m"),
					'day'=>$event->getToDate("%d"),
					'image'=>'<img src="/images/event_end.gif" alt="e"/>',
				);
			}

			$event_data['subject'] = $event->getSubject();
			$event_data['description'] = $event->getDescription();

		} else {
			if($e_type == self::T_START){
				$event_data = array(
					'sort_index'=>$event->getOpenDate(null),
					'year'=>$event->getOpenDate("%Y"),
					'month'=>$event->getOpenDate("%m"),
					'day'=>$event->getOpenDate("%d"),
					'image'=>'<img src="/images/task_icon.gif" alt="ts" />',
				);
			} else if($e_type == self::T_END){
				$event_data = array(
					'sort_index'=>$event->getCloseDate(null),
					'year'=>$event->getCloseDate("%Y"),
					'month'=>$event->getCloseDate("%m"),
					'day'=>$event->getCloseDate("%d"),
					'image'=>'<img src="/images/task_icon.gif" alt="ts" />',
				);
			}
			$event_data['subject'] = $event->getSummary();
			$event_data['description'] = $event->getDetails();
		}


		if($view == self::WEEK_V){
			$redirect_to = 'weekview';
		} else if($view == self::DAY_V){
			$redirect_to = 'dayview';
		} else {
			$redirect_to = 'monthview';
		}


		if($view == self::MONTH_V ){
			//Month view
			$subject = StringUtils::trimString($event_data['subject'],25);
		} else if($view == self::WEEK_V || $view == self::DAY_V){
			//Calendar events
			if($type == 'event'){
				if(!$event->endsSameDay()){
					if($e_type == self::E_START){
						$subject = $event->getFromDate("%H:%M") . '<br />';
					} else if($e_type == self::E_END){
						$subject = $event->getToDate("%H:%M") . '<br />';
					}
					$subject .= $event_data['subject'];
				} else {
					$subject = $event->getFromDate("%H:%M") . ' - ' . $event->getToDate("%H:%M") . '<br />' . $event_data['subject'];
				}
				
			} else {
			//Tasks
				$subject = $event_data['subject'];
			}
		}

		if($type == 'event'){
			$url = GFUrl::getUrl()->currentBaseUrl('UsercalendarEventView', array('event_id'=>$event->getCalendarEventId(), 
			'redirect_to'=>$redirect_to, 'start_date'=>$start_date));
		} else {
			$url = GFUrl::getUrl()->projectURL($event->getTracker()->getProject(), 'TrackerItemEdit', 'tracker','', array('tracker_item_id'=>$event->getTrackerItemId()));
		}

		$event_data['content'] = $event_data['image'] . '<a href="'. $url . '" onmouseover="tooltip(\''. $tooltip .
		'\')" onmouseout="tooltipexit()">'. $subject . '</a>';

		return $event_data;

	}

	private function setDayNames(){
		$lang = Session::getSession()->getLanguage();
		$days = array('UserCalendar.Monday', 'UserCalendar.Tuesday', 
		'UserCalendar.Wednesday', 'UserCalendar.Thursday', 'UserCalendar.Friday',
		'UserCalendar.Saturday', 'UserCalendar.Friday');
		$dayNames = array();
		foreach($days as $day){
			$dayNames[] = StringUtils::trimString($lang->getText($day),3);
		}

		return $dayNames;
	}

	private function setMonthNames(){
		$lang = Session::getSession()->getLanguage();
		$months = array('UserCalendar.January', 'UserCalendar.February', 
		'UserCalendar.March', 'UserCalendar.April','UserCalendar.May', 
		'UserCalendar.June', 'UserCalendar.July', 'UserCalendar.August', 
		'UserCalendar.September','UserCalendar.October', 
		'UserCalendar.November', 'UserCalendar.December');
		$monthNames = array();
		foreach($months as $month){
			$monthNames[] = $lang->getText($month);
		}

		return $monthNames;
	}

	static function buildMinutesOptions($selected_minute=null){
		$minutes = array();
		if(is_null($selected_minute)){
			$selected_minute = strftime("%M");
		}
		
		for($i = 0; $i < 60; $i++){
			$minutes[] = $i < 10 ? '0' . $i : "$i";
		}
	
		$options = '';
		foreach($minutes as $minute){
			$options .= "<option " . (($minute == $selected_minute) ? 'selected="selected"':'') . " value=\"$minute\">$minute</option>";
		}

		return $options;
	}

	static function buildHoursOptions($selected_hour=null){
		$hours = array();
		if(is_null($selected_hour)){
			$selected_hour = strftime("%H");
		}

		for($i = 0; $i < 24; $i++){
			$hours[] = $i < 10 ? '0' . $i : "$i";
		}

		$options = '';
		foreach($hours as $hour){
			$options .= "<option " . (($hour == $selected_hour) ? 'selected="selected"' : '') . " value=\"$hour\">$hour</option>";
		}
		
		return $options;
	}

	static function buildEventOccurrenceOptions($selected=0){
		$lang = Session::getSession()->getLanguage();

		$occurrences = array(
		0 => $lang->getText('UserCalendar.EventOccurrenceNone'),
		CalendarEvent::R_DAILY => $lang->getText('UserCalendar.EventOccurrenceDaily'),
		//CalendarEvent::R_DAILY_MF => $lang->getText('UserCalendar.EventOccurrenceDailyMF'),
		CalendarEvent::R_WEEKLY => $lang->getText('UserCalendar.EventOccurrenceWeekly'),
		CalendarEvent::R_MONTHLY => $lang->getText('UserCalendar.EventOccurrenceMonthly')
		);
		$options = '';
		foreach ($occurrences as $code => $text){
			$options .= "<option " . (($code == $selected) ? 'selected="selected"' : '' ) . ' value ="' . $code . '">' . $text . '</option>';
		}

		return $options;
	}


	private  function cmp($a,$b){
		if($a['sort_index'] < $b['sort_index']){
			return -1;
		} else if($a['sort_index'] == $b['sort_index']){
			return 0; 
		} else {
			return 1;
		}
	}

}
?>
			

