<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Daniel Perez daniel@gforgroup.com
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once ('plugins/wiki/lib/om/BaseWikiPage.php');
require_once("plugin/ActionManager.php");
require_once ('plugins/wiki/lib/WikiVersionPeer.php');
require_once ('lib/core/Filesystem.php');
require_once ('lib/core/FilesystemPeer.php');
require_once('plugins/wiki/lib/WikiPlugin.php');
require_once('plugins/wiki/lib/WikiPageNotification.php');

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.wiki.lib 
 */
class WikiPage extends BaseWikiPage {
	
	private $monitoring_users;
	protected $latest_version;
	private $latest_version_number;
	protected $version;
	protected $previous_version; // previous existing version from latest, or null if there�s not
	CONST FLAGS_LOCKED = 1;
	CONST FLAGS_UNLOCKED = 0;
	const SECTION = 'wikipage';

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null,$force=false) {
		//force save for the increased hit counts
		if ($force) {
			parent::save($con);
			return;
		}
		$notification = new WikiPageNotification($this);
		$notification->setNew($this->isNew());
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create WikiPage object: permission denied");
		} else if (!$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update WikiPage object: permission denied");
		}

		$params = array(
			'is_new' => $this->isNew(),
			'item' => $this
		);

		parent::save($con);
		ActionManager::notify("WIKI_PAGE_SAVE", $params);

		// Notify there was a change in the page
		// For now, we only send project notifications
		if ($this->getSection() == 'project') {
			NotificationManager::getManager()->queue($notification);
		}
	}
	
	/**
	 * Inits the latest version object
	 *
	 */
	private function initLatestVersion() {
		$this->latest_version = null;

		$con = Propel::getConnection(WikiPagePeer::DATABASE_NAME);
		$sql = 'select * from wiki_version where wiki_page_id=\'' . $this->getPrimaryKey() . '\' and page_version = (select max(page_version) from wiki_version where wiki_page_id=\'' . $this->getPrimaryKey() . '\')'; // get highest version
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
		if ($rs->next()) {
			$this->latest_version = new WikiVersion();
			$this->latest_version->hydrate($rs);
		}
	}
	
	/**
	 * Inits the previous version object
	 *
	 */
	private function initPreviousVersion() {
		$this->previous_version = null; // by default it�s null, so we don�t have a previous version
		// get all the versions previous to the latest one and see which is first
		$c = new Criteria();
		$c->add(WikiVersionPeer::WIKI_PAGE_ID,$this->getPrimaryKey());
		$c->addDescendingOrderByColumn(WikiVersionPeer::PAGE_VERSION);
		$versions = WikiVersionPeer::doSelect($c);
		if (count($versions)>1) { // more than 2 versions
			$this->previous_version = $versions[1];
		}
	}	
	
	/**
	 * Inits the version object of the version passed as parameter
	 *
	 * @param int version -> page_version
	 * 
	 */
	private function initVersion($version) {
		$c = new Criteria();
		$c->add(WikiVersionPeer::PAGE_VERSION,$version);
		$arr=$this->getWikiVersions($c);
		if (count($arr) == 1) {
			$this->version = $arr[0];
		} else {
			$this->version = null;
		}
	}	
	
	/**
	 * Fetchs the latest version contents and returns it
	 *
	 * @return string	contents
	 */
	public function getLatestContents() {
		if (!is_object($this->latest_version)) { // fetch the latest version if not set
			$this->initLatestVersion();
		}
		if (is_object($this->latest_version)) {
			$filesystem_id = $this->latest_version->getPrimaryKey();
			$c = new Criteria();
			$c->add(FilesystemPeer::REF_ID,$filesystem_id);
			$c->add(FilesystemPeer::SECTION,WikiPlugin::SECTION);
			$fso = FilesystemPeer::doSelectOne($c);
			if (is_object($fso)) {
				return $fso->getFileData();
			}
		}
		return null;
	}
	
	/**
	 * Fetchs the version contents 
	 *
	 * @param int $version (page_version)
	 * @return string	contents
	 */
	public function getContents($version) {
		if ($version!=null) {
			$ver = $this->getVersion($version);
			if (is_object($ver)) {
				$filesystem_id = $this->version->getPrimaryKey();
				$c = new Criteria();
				$c->add(FilesystemPeer::REF_ID,$filesystem_id);
				$c->add(FilesystemPeer::SECTION,WikiPlugin::SECTION);
				$fso = FilesystemPeer::doSelectOne($c);
				if (is_object($fso)) {
					return $fso->getFileData(false);
				}
			} else {
				return null;
			}
		} else {
			return null;
		}
	}

	/**
	 * ugly workaround - bad coding resulted in getLatestVersion overwriting the parent
	 * function and the var that stores the int with an object
	 */
	public function getLatestVersionNumber(){
		if (!$this->latest_version_number) {
			$c=new Criteria();
			$c->add(WikiPagePeer::WIKI_PAGE_ID,$this->getPrimaryKey());
			$obj=WikiPagePeer::doSelectOne($c);
			$this->latest_version_number=$obj->latest_version;
		}
		return $this->latest_version_number;
	}

	/**
	 * Returns the version object corresponding to the latest version of the page
	 * DANIEL'S MESS - this overrides the base function which returned an int
	 * USE getLatestVersionNumber() to get the int
	 * @return version object
	 */
	public function getLatestVersion(){
		if (!is_object($this->latest_version)) { // fetch the latest version if not set
			$this->initLatestVersion();
		}
		return $this->latest_version;
	}

	/**
	 * Returns the version object corresponding to the previous version of the page or null
	 *
	 * @return version object
	 */
	public function getPreviousVersion(){
		if (!is_object($this->previous_version)) { // fetch the previous version if not set
			$this->initPreviousVersion();
		}
		return $this->previous_version;
	}	
	
	
	/**
	 * returns the version (if exists) from the version passed as a parameter
	 *
	 * @param int $version (page_version)
	 * @return version object if exists, null otherwise
	 */
	public function getVersion($version) {
		if ( (!is_object($this->version)) || ($this->version->getPageVersion()!=$version) ) { // fetch the version if not set
			$this->initVersion($version);
		}
		return $this->version;		
	}
	
	/**
	 * returns the type of file from the filesystem object.
	 * we have 2 types, text/html which is fckeditor edited pages (new) and text only, 
	 * which are pages ported from old gforge4 wiki
	 *
	 */
	public function getType() {
		$ver = $this->getLatestVersion(); // retrieve for latest version, don�t think we�ll need to edit old versions
		if (is_object($ver)) {
			$filesystem_id = $this->latest_version->getPrimaryKey();
			$c = new Criteria();
			$c->add(FilesystemPeer::REF_ID,$filesystem_id);
			$c->add(FilesystemPeer::SECTION,WikiPlugin::SECTION);
			$fso = FilesystemPeer::doSelectOne($c);
			return $fso->getFileType();
		} else {
			return null;
		}
	}
	
	/**
	 * increases the hit count by one to the version 
	 *
	 * @param int $version (page_version)
	 */
	public function increaseHitCount($version) {
		$this->setPageViews($this->getPageViews()+1);
		$this->save(null,true);
		/*
		$ver = $this->getVersion($version);
		if ($ver instanceof WikiVersion) {
			$ver->setHits($ver->getHits()+1);
			$ver->save(null,true);
		}*/
	}

	/**
	 * Loads and returns the project to which this wiki belongs, if appropriate.
	 * If this is a user wiki, returns null.
	 * @return Project $proj The project, if this wiki is associated to a project.
	 */
	public function getRelatedProject() {
		$ret = null;
		if ($this->getSection()== Project::SECTION) {
			$ret = ProjectPeer::retrieveByPk($this->getRefId());
		}
		return $ret;
	}

	public function checkProjectPermission($level) {
		$project = $this->getRelatedProject();
		if (is_object($project)) {
			$perm =  $project->getPermission();
		} else {
			return false;
		}
		if ($perm->check(WikiPlugin::SECTION, 0, $level)) {
			return true;
		}
		return false;
	}
	
	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		if ($this->getSection()==User::SECTION) {
			return $this->userIsAdmin();
		}

		return $this->checkProjectPermission(WikiPlugin::EDIT);
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	public function userIsAdmin() {
		//requires manual customization
		if ($this->getSection()==User::SECTION) {
			$sess=Session::getSession();
			if ($sess->isLoggedIn() && $this->getRefId() == $sess->getUser()->getUserId()) {
				return true;
			}
		}

		return $this->checkProjectPermission(WikiPlugin::ADMIN);
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userCanCreate();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		if ($this->getSection() == User::SECTION ||
			$this->isPublic()) {
			return true;
		}
		return $this->checkProjectPermission(WikiPlugin::READ);
	}
	
	public function isPublic() {
		$project = $this->getRelatedProject();
		if (is_object($project)) {
			if ($project->isPublic()) {
				// See if the wiki is public too
				$setting = WikiProjectSettingPeer::retrieveByPk($project->getProjectId());
				if (!$setting) {
					// Not defined on database, default to true
					return true;
				}
				return ($setting->getIsPublic() ? true : false);
			}
		}
		if ($this->getSection()==User::SECTION) {
			return true;
		}
		return false;
	}

	public function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete DocmanFile object: permission denied");
		}
		// get the versions and delete them
		$versions = $this->getWikiVersions();
		foreach ($versions as $version) {
			$version->delete();
		}
		ActionManager::notify('WIKI_PAGE_DELETE', $this);
		parent::delete($con);
	}

	public function getWikiPageUrl() {
		$project = $this->getRelatedProject();
		if (is_object($project)) {
			return GFUrl::getURL()->projectURL($project, '', 'wiki', '', array('pagename'=>$this->getPageName()));
		}
		return null;
	}

	public function getWikiPageLink($full = FALSE) {
		$link = $this->getWikiPageUrl();
		if ($link == null) return '';
		if ($full) {
			return '<a href="'.$link.'">'.$this->getPageName().'</a>';
		} else {
			return $link;
		}
	}
	
	public function getAllVersions() {
		$c = new Criteria();
		$c->add(WikiVersionPeer::WIKI_PAGE_ID, $this->getPrimaryKey());
		$c->addDescendingOrderByColumn(WikiVersionPeer::WIKI_VERSION_ID);
		$versions = WikiVersionPeer::doSelect($c);
		return $versions;
	}

	public function getVersionsCount() {
		$c = new Criteria();
		$c->add(WikiVersionPeer::WIKI_PAGE_ID, $this->getPrimaryKey());
		$c->addDescendingOrderByColumn(WikiVersionPeer::WIKI_VERSION_ID);
		return WikiVersionPeer::doCount($c);
	}
	
	public function userIsMonitoring(User $user) {
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, self::SECTION);
		$c->add(MonitorPeer::REF_ID, $this->getPrimaryKey());
		$c->add(MonitorPeer::USER_ID, $user->getUserId());
		$mon = MonitorPeer::doSelectOne($c);
		return (!is_null($mon)) ? true : false;
	}
	
	public function getMonitoringUsers() {
		$project_id = $this->getRefId();

		if (!$this->monitoring_users) {
			$con = Propel::getConnection(WikiPagePeer::DATABASE_NAME);
			$sql = 'SELECT * FROM "user" WHERE user_id IN (
						SELECT user_id FROM monitor WHERE section=\''.self::SECTION.'\' AND ref_id='.$this->getPrimaryKey().'
						UNION
						SELECT user_id FROM monitor WHERE section=\''.WikiPlugin::SECTION.'\' AND ref_id='.$project_id.'
					)';
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			$this->monitoring_users = UserPeer::populateObjects($rs);
		}
		
		return $this->monitoring_users;
	}

	public function attachFile($file) {
		$fs = new Filesystem();
		$fs->setSection(Wiki::WIKI_FILES_SECTION);
		$fs->setRefId($this->getPrimaryKey());
		$fs->setPostedBy(Session::getSession()->getUser()->getUserId());

		if (is_string($file)) {
			// $file is specified by a FS path
			if (!is_file($file)) {
				return false;
			}

			$fs->setFileDataFromLocalFile($file);
		} else if (is_array($file)) {
			// $file comes from $_FILES
			$fs->setFileDataFromUploadedFile($file);
		}

		return $fs;
	}

	public function deleteAttachment($file_id) {
		$fs = FilesystemPeer::retrieveByPK($file_id);
		if (!is_object($fs)
			|| $fs->getSection() != Wiki::WIKI_FILES_SECTION
			|| $fs->getRefId() != $this->getPrimaryKey()) {
			throw new PermissionDeniedException();
		}
		if (!$this->userCanUpdate()) {
			throw new PermissionDeniedException();
		}
		
		$fs->delete();
	}
	
	public function getFilesCount() {
		$crit=new Criteria();
		$crit->add(FilesystemPeer::SECTION,Wiki::WIKI_FILES_SECTION);
		$crit->add(FilesystemPeer::REF_ID,$this->getWikiPageId());
		return FilesystemPeer::doCount($crit);
	}
	
	public function getCommentsCount() {
		$c = new Criteria();
		$c->add(ForumPeer::SECTION, WikiPage::SECTION);
		$c->add(ForumPeer::REF_ID, $this->getWikiPageId());
		$c->addJoin(ForumPeer::FORUM_ID, ForumThreadPeer::FORUM_ID);
		$c->addJoin(ForumMessagePeer::FORUM_THREAD_ID, ForumThreadPeer::FORUM_THREAD_ID);
		return ForumMessagePeer::doCount($c);
	}
}

