<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-16
 *
 * SOAP plugin for GForge5.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('plugins/xmlcompatibility/lib/soap/artifact/ArtifactTypes.php');
require_once('plugins/tracker/lib/TrackerItemPeer.php');
require_once('plugins/tracker/lib/TrackerItemAssigneePeer.php');
require_once('lib/core/UserPeer.php');

//
//	getArtifacts
//

$server->register(
	'getArtifacts',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int',
		'status'=>'xsd:int',
		'limit'=>'xsd:int',
		'offset'=>'xsd:int',
		'query_id'=>'xsd:int'),
	array('getArtifactsResponse'=>'tns:ArrayOfArtifact'),
	$uri,$uri.'#getArtifacts','rpc','encoded');

//
//	addArtifact
//
$server->register(
	'addArtifact',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int',
		'status_id'=>'xsd:int',
		'priority'=>'xsd:int',
		'assignees'=>'tns:ArrayOfArtifactAssignee',
		'summary'=>'xsd:string',
		'details'=>'xsd:string',
		'parent_id'=>'xsd:int',
		'sort_ord'=>'xsd:int',
		'open_date'=>'xsd:int',
		'close_date'=>'xsd:int',
		'extra_fields'=>'tns:ArrayOfArtifactExtraFieldsData',
		'dependencies'=>'tns:ArrayOfArtifactDependency'
	),
	array('addArtifactResponse'=>'xsd:int'),
	$uri,$uri.'#addArtifact','rpc','encoded'
);

//
//	updateArtifact
//
$server->register(
	'updateArtifact',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int',
		'artifact_id'=>'xsd:int',
		'status_id'=>'xsd:int',
		'priority'=>'xsd:int',
		'assignees'=>'tns:ArrayOfArtifactAssignee',
		'summary'=>'xsd:string',
		'details'=>'xsd:string',
		'new_artifact_type_id'=>'xsd:int',
		'parent_id'=>'xsd:int',
		'sort_ord'=>'xsd:int',
		'open_date'=>'xsd:int',
		'close_date'=>'xsd:int',
		'extra_fields_data'=>'tns:ArrayOfArtifactExtraFieldsData',
		'dependencies'=>'tns:ArrayOfArtifactDependency'
	),
	array('addArtifactResponse'=>'xsd:int'),
	$uri,$uri.'#updateArtifact','rpc','encoded'
);

//
//	Artifact Delete
//
$server->register(
	'artifactDelete',
	array('session_ser'=>'xsd:string','group_id'=>'xsd:int','group_artifact_id'=>'xsd:int','artifact_id'=>'xsd:int'),
	array('artifactDeleteResponse'=>'xsd:boolean'),
	$uri,$uri.'#artifactDeleteResponse','rpc','encoded'
);


//
//	getArtifacts
//
function getArtifacts($session_ser, $group_id, $group_artifact_id, $status, $limit=0, $offset=0, $query_id=null) {
	try {
		$session = continueSession($session_ser);

		if ($query_id != null && $query_id < 0) {
			// getArtifact() hack
			$artifact_id = -1 * $query_id;
			$c = new Criteria();
			$c->add(TrackerItemPeer::TRACKER_ITEM_ID, $artifact_id);
			$trackerItems = TrackerItemPeer::doSelect($c);
			if (count($trackerItems) > 0) {
				$Item = $trackerItems[0];
				$Tracker = $Item->getTracker();
				if (!$Tracker->userCanRead()) {
					return new soap_fault('', 'getArtifacts', 'Could Get Artifacts: Permission Denied',
										'Could Not Get Artifacts: Permission Denied');
				}
			}
		}
		else if ($query_id != null && $query_id > 0) {
			$Tracker = TrackerPeer::retrieveByPk($group_artifact_id);
			if (!$Tracker || !is_object($Tracker)) {
				return new soap_fault('', 'getArtifacts', 'Could Not Get Artifacts: No Such Artifact Group',
									'Could Not Get Artifacts: No Such Artifact Group');
			}
			if ($Tracker->getProject()->getProjectId() != $group_id) {
				return new soap_fault('', 'getArtifacts', 'Could Get Artifacts: Permission Denied',
									'Could Not Get Artifacts: Permission Denied');
			}
			$c = new Criteria();
			$c->add(TrackerQueryPeer::USER_ID, $session->getUser()->getID());
			$c->add(TrackerQueryPeer::TRACKER_QUERY_ID, $query_id);
			$c->setLimit(1);
			$TrackerQueries = $Tracker->getTrackerQuerys($c);
			if (is_array($TrackerQueries) && count($TrackerQueries) > 0) {
				$trackerItemQuery = new TrackerItemSavedQuery($Tracker, $TrackerQueries[0]);
				if (!$trackerItemQuery || !is_object($trackerItemQuery)) {
				return new soap_fault('', 'getArtifacts', 'Could Not Get Artifacts: No Such View',
									'Could Not Get Artifacts: No Such View');
				}
			}
			$trackerItems = $trackerItemQuery->getTrackerItems($limit, $offset);
		}
		else {
			// Retrieve the Tracker Items
			$c = new Criteria();
			if ($limit > 0) {
				$c->setLimit($limit);
			} else {
				$c->setLimit(100);
			}

			if ($offset > 0) {
				$c->setOffset($offset);
			}

			$c->add(TrackerItemPeer::TRACKER_ID, $group_artifact_id);
			// Do we really need this?
			$c->addJoin(TrackerPeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
			$c->addJoin(TrackerPeer::TRACKER_ID, TrackerItemPeer::TRACKER_ID);
			$c->add(ProjectPeer::PROJECT_ID, $group_id);
			$c->addDescendingOrderByColumn(TrackerItemPeer::TRACKER_ID);
			//
			if (!empty($status)) {
				$c->add(TrackerItemPeer::STATUS_ID, $status);
			}
			$trackerItems = TrackerItemPeer::doSelect($c);
		}
	
		if ($trackerItems == null) {
			$trackerItems = array();
		}
		return trackerItemsToSoap($trackerItems);
	} catch (Exception $ex) {
		return new soap_fault ('', 'getArtifacts',
							'Could Not Get Artifacts: ' . $ex->getMessage(),
							'Could Not Get Artifacts: ' . $ex->getMessage());
	}
}

//
//	addArtifact
//
function addArtifact($session_ser, $group_id, $group_artifact_id, $status_id, $priority, 
				$assignees, $summary, $details, $parent_id, $sort_ord, $open_date,
				$close_date, $extra_fields, $dependencies = 0) {
	try {
		$session = continueSession($session_ser);
		$Tracker = TrackerPeer::retrieveByPk($group_artifact_id);
		if (!$Tracker || !is_object($Tracker) || $Tracker->getProject()->getProjectId() != $group_id) {
			return new soap_fault('', 'addArtifact',
								'Could Not Add Artifact: Permission denied',
								'Could Not Add Artifact: Permission denied');
		}
		$TrackerItem = new TrackerItem();
		$Tracker->addTrackerItem($TrackerItem);
		if (!$TrackerItem->userCanCreate()) {
			return new soap_fault('', 'addArtifact',
								'Could Not Add Artifact: Permission denied',
								'Could Not Add Artifact: Permission denied');
		}
	
		$TrackerItem->setSubmittedBy($session->getUser()->getUserId());
		$TrackerItem->setPriority(Validator::sanitizeInt($priority));
		//$TrackerItem->setStatusId(Validator::sanitizeInt($status_id));
		$TrackerItem->setSummary(Validator::sanitizeString($summary));
		$TrackerItem->setDetails(Validator::sanitizeString($details));
		$TrackerItem->setParentId(Validator::sanitizeInt($parent_id));
		$TrackerItem->setSortOrder(Validator::sanitizeInt($sort_ord));
		if ($Tracker->getDatatype()==Tracker::DATATYPE_TASK) {
			$TrackerItem->setOpenDate(Validator::sanitizeInt($open_date));
			$TrackerItem->setCloseDate(Validator::sanitizeInt($close_date));
		}
		
		$validAssignees = array();
		if (isset($assignees) && is_array($assignees) && count($assignees) > 0) {
			foreach ($assignees as $assignee) {
				$validAssignees[$assignee['user_id']] = $assignee['percent_effort'];
			}
		}

		// If it's not assigned to anyone, assign to Nobody
		if (count($validAssignees) == 0) {
			$validAssignees['100'] = 0;
		}
	
		$validDependencies = array();
		if (isset($dependencies) && is_array($dependencies) && count($dependencies) > 0) {
			foreach ($dependencies as $dependency) {
				if (is_object(TrackerItemPeer::retrieveByPk($dependency['related_artifact_id']))) {
					$validDependencies[$dependency['related_artifact_id']] = $dependency['link_type'];
				}
			}
		}

		$efields = array();
		$efTypeMap = array();
		$trackerEF = $Tracker->getTrackerExtraFields();
		foreach ($trackerEF as $ef) {
			$efTypeMap[$ef->getTrackerExtraFieldId()] = $ef->getFieldType();
			if ($ef->getFieldType() == TrackerExtraField::TYPE_TEXT ||
			    $ef->getFieldType() == TrackerExtraField::TYPE_TEXTAREA) {
				$efields[$ef->getTrackerExtraFieldId()] = '';
			} else {
				$efields[$ef->getTrackerExtraFieldId()] = '0';
			}
		}
		
		if (isset($extra_fields) && is_array($extra_fields) && count($extra_fields) > 0) {
			foreach ($extra_fields as $extra_field) {
				if (isset($extra_field['extra_field_id']) && isset($extra_field['field_data'])) {
					if (isset($efTypeMap[$extra_field['extra_field_id']]) &&
						($efTypeMap[$extra_field['extra_field_id']] == TrackerExtraField::TYPE_MULTISELECT ||
						$efTypeMap[$extra_field['extra_field_id']] == TrackerExtraField::TYPE_MEMBERS ||
						$efTypeMap[$extra_field['extra_field_id']] == TrackerExtraField::TYPE_CHECKBOX)) {
						if (strpos($extra_field['field_data'], ',') !== false) {
							$efields[$extra_field['extra_field_id']] = explode(',', $extra_field['field_data']);
						} else {
							$efields[$extra_field['extra_field_id']] = array($extra_field['field_data']);
						}
					} else {
						$efields[$extra_field['extra_field_id']] = $extra_field['field_data'];
					}
				}
			}
		}

		$TrackerItem->addAssignees($validAssignees);
		$TrackerItem->addRelations($validDependencies);
		if (count($efields) > 0) {
			$TrackerItem->addExtraFields($efields);
		}

		$TrackerItem->save();
		// REPORT THE ACTION
		ActionManager::notify('TRACKER_ITEM_ADD_ACTION', $TrackerItem);
		return $TrackerItem->getTrackerItemId();

	} catch (InvalidDataException $ex) {
		return new soap_fault('', 'addArtifact',
							'Could Not Add Artifact, Invalid Data: '. implode("\n", $ex->getErrors()),
							'Could Not Add Artifact, Invalid Data: '. implode("\n", $ex->getErrors()));
	} catch (Exception $ex) {
		return new soap_fault('', 'addArtifact',
							'Could Not Add Artifact: '. $ex->getMessage(),
							'Could Not Add Artifact: '. $ex->getMessage());
	}
}


//
//	updateArtifact
//
function updateArtifact($session_ser, $group_id, $group_artifact_id, $artifact_id, $status_id,
	$priority, $assignees, $summary, $details, $new_artifact_type_id, 
	$parent_id, $sort_ord, $open_date, $close_date, $extra_fields, $dependencies = 0) {
	try {
		$session = continueSession($session_ser);
	
		$TrackerItem = TrackerItemPeer::retrieveByPk($artifact_id);
		if (!$TrackerItem || !is_object($TrackerItem)) {
			return new soap_fault('', 'updateArtifact',
								'Could Not Update Artifact: No Such Item',
								'Could Not Update Artifact: No Such Item');
		}
	
		$Tracker=$TrackerItem->getTracker();
		//validate that this is a legitimate trackeritem for this tracker and user can add items
		if (!$TrackerItem->userCanUpdate() || $Tracker->getProject()->getProjectId() != $group_id ||
			$Tracker->getTrackerId() != $group_artifact_id) {
			return new soap_fault('', 'updateArtifact',
								'Could Not Update Artifact: Permission Denied',
								'Could Not Update Artifact: Permission Denied');
		}
	
		// Change the TrackerId/ArtifactTypeId
		// TODO: FINISH THIS
		if (isset($new_artifact_type_id) && $new_artifact_type_id != 0
			&& $new_artifact_type_id != $Tracker->getTrackerId()) {
			$TrackerItem->setTrackerId($new_artifact_type_id);
			$assignees = array(100);
		}
		
		$TrackerItem->setPriority(Validator::sanitizeInt($priority));
		//$TrackerItem->setStatusId(Validator::sanitizeInt($status_id));
		$TrackerItem->setSummary(Validator::sanitizeString($summary));
		$TrackerItem->setDetails(Validator::sanitizeString($details));
		$TrackerItem->setParentId(Validator::sanitizeInt($parent_id));
		$TrackerItem->setSortOrder(Validator::sanitizeInt($sort_ord));
		if ($Tracker->getDatatype()==Tracker::DATATYPE_TASK) {
			$TrackerItem->setOpenDate(Validator::sanitizeInt($open_date));
			$TrackerItem->setCloseDate(Validator::sanitizeInt($close_date));
		}
		$TrackerItem->setLastModifiedDate(time());
		$TrackerItem->setLastModifiedBy($session->getUser()->getUserId());
	
		$validAssignees = array();
		if (isset($assignees) && is_array($assignees) && count($assignees) > 0) {
			foreach ($assignees as $assignee) {
				$validAssignees[$assignee['user_id']] = $assignee['percent_effort'];
			}
		}
		// If it's not assigned to anyone, assign to Nobody
		if (count($validAssignees) == 0) {
			$validAssignees[100] = 0;
		}
	
		$validDependencies = array();
		if (isset($dependencies) && is_array($dependencies) && count($dependencies) > 0) {
			foreach ($dependencies as $dependency) {
				if (is_object(TrackerItemPeer::retrieveByPk($dependency['related_artifact_id']))) {
					$validDependencies[$dependency['related_artifact_id']] = $dependency['link_type'];
				}
			}
		}

		$efields = array();
		$efTypeMap = array();
		$trackerEF = $Tracker->getTrackerExtraFields();
		foreach ($trackerEF as $ef) {
			$efTypeMap[$ef->getTrackerExtraFieldId()] = $ef->getFieldType();
		}
		if (isset($extra_fields) && is_array($extra_fields) && count($extra_fields) > 0) {
			foreach ($extra_fields as $extra_field) {
				if (isset($extra_field['extra_field_id']) && isset($extra_field['field_data'])) {
					if (isset($efTypeMap[$extra_field['extra_field_id']]) &&
						($efTypeMap[$extra_field['extra_field_id']] == TrackerExtraField::TYPE_MULTISELECT ||
						$efTypeMap[$extra_field['extra_field_id']] == TrackerExtraField::TYPE_MEMBERS ||
						$efTypeMap[$extra_field['extra_field_id']] == TrackerExtraField::TYPE_CHECKBOX)) {
						if (strpos($extra_field['field_data'], ',') !== false) {
							$efields[$extra_field['extra_field_id']] = explode(',', $extra_field['field_data']);
						} else {
							$efields[$extra_field['extra_field_id']] = array($extra_field['field_data']);
						}
					} else {
						$efields[$extra_field['extra_field_id']] = $extra_field['field_data'];
					}
				}
			}
		}


		$TrackerItem->updateAssignees($validAssignees);
		$TrackerItem->updateRelations($validDependencies);

		if (count($efields) > 0) {
			$TrackerItem->updateExtraFields($efields);
		}

		$TrackerItem->save();
		// REPORT THE ACTION
		ActionManager::notify('TRACKER_ITEM_EDIT_ACTION',$TrackerItem);
		return $TrackerItem->getTrackerItemId();
	
	} catch (InvalidDataException $ex) {
		return new soap_fault('', 'updateArtifact',
							'Could Not Update Artifact: '.implode("\n", $ex->getErrors()),
							'Could Not Update Artifact: '.implode("\n", $ex->getErrors()));
	} catch (Exception $ex) {
		return new soap_fault('', 'updateArtifact',
							'Could Not Update Artifact: '.$ex->getMessage(),
							'Could Not Update Artifact: '.$ex->getMessage());
	}
}

//
//	Artifact Delete
//
function artifactDelete($session_ser, $group_id, $group_artifact_id, $artifact_id) {
	try {
		$session = continueSession($session_ser);
		
		$TrackerItem = TrackerItemPeer::retrieveByPk($artifact_id);
		if (!$TrackerItem || !is_object($TrackerItem)) {
			return new soap_fault('', 'artifactDelete',
								'Could Not Delete Artifact: No Such Item',
								'Could Not Delete Artifact: No Such Item');
		}
		$Tracker=$TrackerItem->getTracker();
	
		if (!$TrackerItem->userCanDelete() || $Tracker->getProject()->getProjectId() != $group_id ||
			$Tracker->getTrackerId() != $group_artifact_id) {
			return new soap_fault('', 'artifactDelete',
								'Could Not Delete Artifact: Permission Denied',
								'Could Not Delete Artifact: Permission Denied');
		}
		$TrackerItem->delete();
		// REPORT THE ACTION?
		return true;

	} catch (Exception $ex) {
		return new soap_fault('', 'updateArtifact',
							'Could Not Delete Artifact: '.$ex->getMessage(),
							'Could Not Delete Artifact: '.$ex->getMessage());
	}
}

function getArtifact($session_ser, $group_id, $group_artifact_id, $artifact_id) {
	//TODO?
}

function trackerItemsToSoap($trackerItems) {
	$return = array();
	foreach ($trackerItems as $trackerItem) {
		$extra_fields = array();
		$dependencies = array();

		$extraFieldDatas = $trackerItem->getArrangedExtraFieldData();
		$dependenciesObj = $trackerItem->getTrackerItemRelationsRelatedByTrackerItemId();
		
		$efdatas = array();
		foreach ($extraFieldDatas as $id => $extraFieldData) {
			;
			if (is_array($extraFieldData)) {
				$extraFieldData = implode(',', $extraFieldData);
			}
			$extra_fields[] = array(
								'extra_field_id'=>$id,
								'field_data'=>$extraFieldData
							);
		}

		foreach ($dependenciesObj as $dependency) {
			$dependencies[] = array(
								'related_artifact_id'=>$dependency->getRelatedTrackerItemId(),
								'link_type'=>$dependency->getLinkType()
							);
		}

		$assignees = array();
		$c = new Criteria();
		$c->add(TrackerItemAssigneePeer::TRACKER_ITEM_ID, $trackerItem->getTrackerItemId());
		$assigneesObj = TrackerItemAssigneePeer::doSelect($c);
		
		if (count($assigneesObj) == 0) {
			$assignees[] = array('user_id'=>100, 'percent_effort'=>0);
		}
		foreach ($assigneesObj as $assignee) {
			$assignees[] = array('user_id'=>$assignee->getAssignee(), 'percent_effort'=>$assignee->getPercentUtilization());
		}

		$return[]=array(
			'artifact_id'=>$trackerItem->getTrackerItemId(),
			'group_artifact_id'=>$trackerItem->getTrackerId(),
			'status_id'=>$trackerItem->getStatusId(),
			'priority'=>$trackerItem->getPriority(),
			'submitted_by'=>$trackerItem->getSubmittedBy(),
			'assignees'=>$assignees,
			'open_date'=>$trackerItem->getOpenDate(null),
			'close_date'=>$trackerItem->getCloseDate(null),
			'summary'=>$trackerItem->getSummary(),
			'details'=>$trackerItem->getDetails(),
			'sort_order'=>$trackerItem->getSortOrder(),
			'parent_id'=>$trackerItem->getParentId(),
			'extra_fields'=>$extra_fields,
			'dependencies'=>$dependencies,
			'has_subitems'=>$trackerItem->hasSubitems(),
			'subitems_count'=>$trackerItem->getSubitemsCount()
		);
	}
	return $return;
}

?>
