<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-16
 *
 * SOAP plugin for GForge5.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('plugins/xmlcompatibility/lib/soap/artifact/ArtifactViewTypes.php');
require_once('plugins/tracker/lib/TrackerPeer.php');
require_once('plugins/tracker/lib/TrackerQueryPeer.php');
require_once('plugins/tracker/lib/TrackerItemSavedQuery.php');

//
// artifactGetViews
//
$server->register(
	'artifactGetViews',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int'
	),
	array('artifactGetViewsResponse'=>'tns:ArrayOfArtifactQuery'),
	$uri,
	$uri.'#artifactGetViews','rpc','encoded'
);

//
// artifactCreateView
//
$server->register(
	'artifactCreateView',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int',
		'name'=>'xsd:string',
		'priority'=>'tns:ArrayOfint',
		'assignee'=>'tns:ArrayOfUserID',
		'moddaterange'=>'xsd:string',
		'sort_col'=>'xsd:string',
		'sort_ord'=>'xsd:string',
		'extra_fields'=>'tns:ArrayOfArtifactExtraFieldsData',
		'opendaterange'=>'xsd:string',
		'closedaterange'=>'xsd:string'
	),
	array('artifactCreateViewResponse'=>'xsd:int'),
	$uri,
	$uri.'#artifactCreateView','rpc','encoded'
);


//
// artifactUpdateView
//
$server->register(
	'artifactUpdateView',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int',
		'query_id'=>'xsd:int',
		'name'=>'xsd:string',
		'priority'=>'tns:ArrayOfint',
		'assignee'=>'tns:ArrayOfUserID',
		'moddaterange'=>'xsd:string',
		'sort_col'=>'xsd:string',
		'sort_ord'=>'xsd:string',
		'extra_fields'=>'tns:ArrayOfArtifactExtraFieldsData',
		'opendaterange'=>'xsd:string',
		'closedaterange'=>'xsd:string'
	),
	array('artifactUpdateViewResponse'=>'xsd:int'),
	$uri,
	$uri.'#artifactUpdateView','rpc','encoded'
);

//
// artifactDeleteView
//
$server->register(
	'artifactDeleteView',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int',
		'artifact_query_id' => 'xsd:int'
	),
	array('artifactDeleteViewResponse'=>'xsd:boolean'),
	$uri,
	$uri.'#artifactDeleteView','rpc','encoded'
);

function artifactGetViews($session_ser, $group_id, $group_artifact_id) {
	try {
		$session = continueSession($session_ser);
		$user = $session->getUser();
				
		$Tracker = TrackerPeer::retrieveByPk($group_artifact_id);
		if (!$Tracker || !is_object($Tracker) || $Tracker->getProject()->getProjectId() != $group_id) {
			return new soap_fault('', 'artifactGetViews',
								'Could Not Get Artifact Views: Permission denied',
								'Could Not Get Artifact Views: Permission denied');
		}
		$c = new Criteria();
		$c->add(TrackerQueryPeer::TRACKER_ID, $group_artificat_id);
		$Queries = $Tracker->getTrackerQuerys($c);
	
		$QueriesFiltered = array();
		foreach($Queries as $Query) {
			if ($Query->getUserId() == 100 || $Query->getUserId() == $user->getUserId() || $Query->getIsShared()) {
				$QueriesFiltered[] = $Query;
			}
		}
	
		if ($Queries == null) {
			$Queries = array();
		}
		return trackerQueriesToSoap($QueriesFiltered, $user);
	} catch (InvalidDataException $ex) {
		return new soap_fault('', 'artifactGetViews',
							'Could Not Get Views, Invalid Data: '. implode(" / ", $ex->getErrors()),
							'Could Not Get Views, Invalid Data: '. $ex->getErrors());
	} catch (Exception $ex) {
		return new soap_fault('', 'artifactGetViews',
							'Could Not Get Views: '.$ex->getMessage(),
							'Could Not Get Views: '.$ex->getMessage());
	}
}

function artifactCreateView($session_ser, $group_id, $group_artifact_id, $name, $priority, $assignees, $moddaterange='',
					$sort_col, $sort_ord, $extra_fields = null, $opendaterange='', $closedaterange='') {
	try {
		$session = continueSession($session_ser);
		
		$Tracker = TrackerPeer::retrieveByPk($group_artifact_id);
		if (!$Tracker || !is_object($Tracker)) {
			return new soap_fault('', 'artifactCreateView',
								'Could Not Create View: No Such Artifact Group',
								'Could Not Create View: No Such Artifact Group');
		}
		if ($Tracker->getProject()->getProjectId() != $group_id) {
			return new soap_fault('', 'artifactCreateView',
								'Could Not Create View: Permission Denied',
								'Could Not Create View: Permission Denied');
		}
		//Convert Date Ranges to the right format
		$moddaterange = convertDate($moddaterange);
		$opendaterange = convertDate($opendaterange);
		$closedaterange = convertDate($closedaterange);
		
		//Convert Extra Fields to the right format
		$efields = array();
		foreach($extra_fields as $extra_field) {
			if (isset($extra_field['extra_field_id']) && isset($extra_field['field_data'])) {
				$efields[$extra_field['extra_field_id']] = explode(',', $extra_field['field_data']);
			}
		}
		
		$TrackerQuery = new TrackerQuery();
		$TrackerQuery->setTracker($Tracker);
		$TrackerQuery->setUser($session->getUser());
		$TrackerQuery->setQueryName(Validator::sanitizeString($name));

		$assignees=((isset($assignees)) ? $assignees : array());
		$efields=((isset($efields)) ? $efields : array());
		$priority=((isset($priority)) ? $priority : array());

		$TrackerQuery->setQueryFields($assignees, $efields, $moddaterange, $opendaterange,
									$closedaterange, $sort_ord, $sort_col, array(), $priority);
		$TrackerQuery->save();
		return $TrackerQuery->getTrackerQueryId();

	} catch (InvalidDataException $ex) {
		return new soap_fault('', 'artifactCreateView',
							'Could Not Add View, Invalid Data: '. implode(" / ", $ex->getErrors()),
							'Could Not Add View, Invalid Data: '. $ex->getErrors());
	} catch (PropelException $ex) {
		return new soap_fault ('','artifactCreateView', $ex->getCause(), $ex->getCause());
	} catch (Exception $ex) {
		return new soap_fault ('','artifactCreateView', $ex->getMessage(), $ex->getMessage());
	}
}

function artifactUpdateView($session_ser, $group_id, $group_artifact_id, $query_id, $name, $priority, $assignees,
					$moddaterange= '', $sort_col, $sort_ord, $extra_fields = null, $opendaterange='', $closedaterange='') {
	try {		
		$session = continueSession($session_ser);
		
		$Tracker = TrackerPeer::retrieveByPk($group_artifact_id);
		if (!$Tracker || !is_object($Tracker)) {
			return new soap_fault('', 'artifactCreateView', 'Could Not Update View: No Such Artifact Group',
								'Could Not Update View: No Such Artifact Group');
		}
		if ($Tracker->getProject()->getProjectId() != $group_id) {
			return new soap_fault('', 'artifactCreateView', 'Could Not Update View: Permission Denied',
								'Could Not Update View: Permission Denied');
		}
	
		$c = new Criteria();
		$c->add(TrackerQueryPeer::USER_ID, $session->getUser()->getID());
		$c->add(TrackerQueryPeer::TRACKER_QUERY_ID, $query_id);
		$c->setLimit(1);
		$TrackerQueries = $Tracker->getTrackerQuerys($c);
		if ($TrackerQueries && is_array($TrackerQueries) && count($TrackerQueries) > 0) {
			$TrackerQuery = $TrackerQueries[0];
			if (!$TrackerQuery || !is_object($TrackerQuery)) {
			return new soap_fault('', 'artifactUpdateView', 'Could Not Update View: No Such View',
								'Could Not Update View: No Such View');
			}
		}
		else {
			return new soap_fault('', 'artifactUpdateView', 'Could Not Update View: No Such View or Permission Denied',
								'Could Not Update View: No Such View or Permission Denied');
		}
	
		//Convert Date Ranges to the right format
		$moddaterange = convertDate($moddaterange);
		$opendaterange = convertDate($opendaterange);
		$closedaterange = convertDate($closedaterange);
		
		//Convert Extra Fields to the right format
		$efields = array();
		foreach($extra_fields as $extra_field) {
			if (isset($extra_field['extra_field_id']) && isset($extra_field['field_data'])) {
				$efields[$extra_field['extra_field_id']] = explode(',', $extra_field['field_data']);
			}
		}
		
		$TrackerQuery->setQueryName(Validator::sanitizeString($name));
		$TrackerQuery->setQueryFields($assignees, $efields, $moddaterange, $opendaterange,
									$closedaterange, $sort_ord, $sort_col, array(), $priority);
		$TrackerQuery->save();
		return $TrackerQuery->getTrackerQueryId();
	} catch (Exception $ex) {
		return new soap_fault ('','artifactUpdateView', $ex->getMessage(), $ex->getMessage());
	}
}

function artifactDeleteView($session_ser, $group_id, $group_artifact_id, $query_id) {
	try {
		$session = continueSession($session_ser);

		$Tracker = TrackerPeer::retrieveByPk($group_artifact_id);
		if (!$Tracker || !is_object($Tracker)) {
			return new soap_fault('', 'artifactDeleteView', 'Could Not Delete View: No Such Artifact Group',
								'Could Not Delete View: No Such Artifact Group');
		}
		if ($Tracker->getProject()->getProjectId() != $group_id) {
			return new soap_fault('', 'artifactDeleteView', 'Could Not Delete View: Permission Denied',
								'Could Not Delete View: Permission Denied');
		}
	
		$c = new Criteria();
		$c->add(TrackerQueryPeer::USER_ID, $session->getUser()->getID());
		$c->add(TrackerQueryPeer::TRACKER_QUERY_ID, $query_id);
		$c->setLimit(1);
		$TrackerQueries = $Tracker->getTrackerQuerys($c);
		if (is_array($TrackerQueries) && count($TrackerQueries) > 0) {
			$TrackerQuery = $TrackerQueries[0];
			if (!$TrackerQuery || !is_object($TrackerQuery)) {
			return new soap_fault('', 'artifactDeleteView', 'Could Not Delete View: No Such View',
								'Could Not Delete View: No Such View');
			}
		}
		else {
			return new soap_fault('', 'artifactDeleteView', 'Could Not Delete View: No Such View or Permission Denied',
								'Could Not Delete View: No Such View or Permission Denied');
		}
		$TrackerQuery->delete();
		return true;

	} catch (Exception $ex) {
		return new soap_fault ('','artifactDeleteView', $ex->getMessage(), $ex->getMessage());
	}
	return true;
}

function trackerQueriesToSoap($Queries, $user) {
	$result = array();
	foreach($Queries as $Query) {
		$itemQuery = new TrackerItemSavedQuery($Query->getTracker(), $Query, $user);
		$queryExtraFields = $itemQuery->getExtraFields();

		$extra_fields = array();
		if (is_array($queryExtraFields)) {
			foreach ($queryExtraFields as $id => $values) {
				if (!is_array($values)) $values = array($values);
				$extra_fields[] = array("extra_field_id" => $id, "values" => $values);
			}
		}
		
		$assignee = $itemQuery->getAssignedTo();
		if (!is_array($assignee)) {
			$assignee = array($assignee);
		}
		$result[] = array(
						"artifact_query_id"	=> $Query->getTrackerQueryId(),
						"name" => $Query->getQueryName(),
						"fields"=> array(
									"sortcol"	=> $itemQuery->getSortCol(),
									"sortord"	=> $itemQuery->getSortOrd(),
									"moddaterange"=> convertDate($itemQuery->getModDateRange(), 1),
									"assignee"	=> $assignee,
									"priority"	=> $itemQuery->getPriority(),
									"extra_fields"	=> $extra_fields,
									"opendaterange"	=> convertDate($itemQuery->getOpenDateRange(), 1),
									"closedaterange"=> convertDate($itemQuery->getCloseDateRange(), 1)
									),
						);
	}
	return $result;
}

function convertDate($daterange, $mode = 0) {
	if ($mode == 0) {
		$new_dates = array();
		if (strlen(trim($daterange)) > 20) {
			$dates = explode(' ', trim($daterange));
			foreach ($dates as $date) {
				list($year, $month, $day) = explode('-', $date);
				$new_dates[] = date('M j Y', mktime(0, 0, 0, $month, $day, $year));
			}
		}
		if (count($new_dates) == 0) return "";
		return implode(' ', $new_dates);
	} else {
		if (strlen($daterange) > 17) {
			list($month1, $day1, $year1, $month2, $day2, $year2) = explode(' ', $daterange);
			$partone = date('Y-m-d', strtotime("$day1 $month1 $year1"));
			$parttwo = date('Y-m-d', strtotime("$day2 $month2 $year2"));
			$new_dates = $partone . ' ' . $parttwo;
		} else {
			$new_dates = '';
		}
		return $new_dates;
	}
}

?>
