<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-16
 *
 * SOAP plugin for GForge5.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */


require_once('plugins/xmlcompatibility/lib/soap/common/UserTypes.php');
require_once('plugins/xmlcompatibility/lib/soap/common/Group.php');
require_once('lib/core/UserPeer.php');

//getUsers (id array)
$server->register(
    'getUsers',
    array('session_ser'=>'string','user_ids'=>'tns:ArrayOfint'),
    array('userResponse'=>'tns:ArrayOfUser'),
    $uri,
    $uri.'#getUsers','rpc','encoded'
);

//getUsersByName (unix_name array)
$server->register(
    'getUsersByName',
    array('session_ser'=>'string','user_ids'=>'tns:ArrayOfstring'),
    array('userResponse'=>'tns:ArrayOfUser'),
    $uri,
    $uri.'#getUsersByName','rpc','encoded'
);

//getGroups (id array)
$server->register(
    'userGetGroups',
    array('session_ser'=>'string','user_id'=>'xsd:int'),
    array('groupResponse'=>'tns:ArrayOfGroup'),
    $uri,
    $uri.'#userGetGroups','rpc','encoded'
);


function getUsers($session_ser, $user_ids) {
	$session = continueSession($session_ser);
	$users = array();
	
	for ($i=0; $i<count($user_ids); $i++) {
		$userId = $user_ids[$i];
		$user = UserPeer::retrieveByPk($userId);
		if ($user != null) {
			$users[] = $user;
		}
	}
	if ((count($user_ids) > 0) && (count($users) == 0)) {
		return new soap_fault('3001', 'user', 'Could Not Get Users By Id', 'Could Not Get Users By Id');
	}
	return usersToSoap($users);
}

function getUsersByName($session_ser, $user_names) {
	$session = continueSession($session_ser);
	$users = array();

	if (!is_array($user_names)) {
		$user_names = array($user_names);
	}
	
	for ($i=0; $i<count($user_names); $i++) {
		$userName = $user_names[$i];
		$user = UserPeer::retrieveByUnixName($userName);
		if ($user != null) {
			$users[] = $user;
		}
	}
	if (count($users) == 0) {
		return new soap_fault('3002', 'user', 'Could Not Get Users By Name', 'Could Not Get Users By Name');
	}
	return usersToSoap($users);
}

function userGetGroups($session_ser, $user_id) {
	$session = continueSession($session_ser);
	$user = UserPeer::retrieveByPk($user_id);

	if ($user == null) {
		return new soap_fault ('3003','user','Could Not Get Users Groups','Could Not Get Users Groups');
	}
	return projectsToSoap($user->getAllProjects());
}


/*
	Converts an array of Group objects to soap data
*/
function usersToSoap($users) {
	$return = array();

	foreach ($users as $user) {
		$return[] = array(
			'user_id'=>$user->getId(), 
			'user_name'=>$user->getUnixName(),
			'title'=>$user->getTitle(),
			'firstname'=>$user->getFirstName(),
			'lastname'=>$user->getLastName(),
			'address'=>$user->getAddress(),
			'address2'=>$user->getAddress2(),
			'phone'=>$user->getPhone(),
			'fax'=>$user->getFax(),
			'status'=>$user->getStatus(),
			'timezone'=>$user->getTimeZone(),
			'country_code'=>$user->getCcode(),
			'add_date'=>$user->getCreateDate(null),
			'language_id'=>$user->getLanguageId()
		);
	}
	return $return;
}

?>
