<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-16
 *
 * SOAP plugin for GForge5.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('plugins/xmlcompatibility/lib/soap/scm/ScmTypes.php');
require_once('plugins/tracker/lib/TrackerItemScmCommitPeer.php');
require_once('plugins/scm/lib/ScmCommitPeer.php');
require_once('plugins/scm/lib/ScmCommitFilePeer.php');

//
// artifactGetViews
//
$server->register(
	'getScmCommits',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int',
		'artifact_id'=>'xsd:int'
	),
	array('getScmCommitsResponse'=>'tns:ArrayOfScmCommit'),
	$uri,
	$uri.'#getScmCommits','rpc','encoded'
);


function getScmCommits($session_ser, $group_id, $group_artifact_id, $artifact_id) {
	$session = continueSession($session_ser);

	$TrackerItem = TrackerItemPeer::retrieveByPk($artifact_id);
	if (!$TrackerItem || !is_object($TrackerItem)) {
		return new soap_fault('', 'getScmCommits', 'Could Not Get Scm Commit: No Such Artifact',
							'Could Not Get Scm Commit: No Such Artifact');
	}
	$Tracker=$TrackerItem->getTracker();
	//validate that this is a legitimate trackeritem for this tracker and user can add items
	if ($Tracker->getProject()->getProjectId() != $group_id || $Tracker->getTrackerId() != $group_artifact_id) {
		return new soap_fault('', 'getScmCommits', 'Could Not Get Scm Commit: Permission Denied',
							'Could Not Get Scm Commit: Permission Denied');
	}

	$c = new Criteria();
	$c->addJoin(TrackerItemScmCommitPeer::SCM_COMMIT_ID, ScmCommitPeer::SCM_COMMIT_ID);
	$c->add(TrackerItemScmCommitPeer::TRACKER_ITEM_ID, $artifact_id);	

	$ScmCommits = ScmCommitPeer::doSelect($c);
	
	return scmCommitsToSoap($ScmCommits);
}


function scmCommitsToSoap($ScmCommits) {
	$result = array();
	foreach($ScmCommits as $item) {
		$c = new Criteria();
		$c->add(ScmCommitFilePeer::SCM_COMMIT_ID, $item->getScmCommitId());
		$ScmCommitFiles = ScmCommitFilePeer::doSelect($c);

		$files = scmCommitFilesToSoap($ScmCommitFiles);

		$result[] = array(
						"commit_id"		=> $item->getScmCommitId(),
						"commit_date"	=> $item->getCommitDate(null),
						"message_log" => $item->getMessageLog(),
						"files" => $files,
						"user_id"=> $item->getUserId()
					);
	}
	return $result;
}

function scmCommitFilesToSoap($ScmCommitFiles) {
	$result = array();
	foreach($ScmCommitFiles as $item) {
		$result[] = array(
						"modified_file" => $item->getModifiedFile(),
						"prev_version" => $item->getPrevVersion(),
						"new_version"=> $item->getNewVersion()
					);
	}
	return $result;
}
?>
