<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling DiskUsage

require_once('core/DiskUsage.php');
require_once('core/DiskUsagePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'DiskUsage',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'project_id' => array('name'=>'project_id', 'type' => 'xsd:int'),
		'check_date' => array('name'=>'check_date', 'type' => 'xsd:string'),
		'bytes_used' => array('name'=>'bytes_used', 'type' => 'xsd:int'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfDiskUsage',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:DiskUsage[]')),
	'tns:DiskUsage'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getDiskUsages',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_unix_name' => 'xsd:string',
		'check_date' => 'xsd:string',
	), 
	array('getDiskUsages' => 'tns:ArrayOfDiskUsage'),
	$uri, 
	$uri.'#getDiskUsages',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of DiskUsage
 * @param	$project_unix_name	Filter by project_unix_name
 * @param	$check_date	Filter by check_date
 */
function getDiskUsages($sessionHash, $project_unix_name, $check_date) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);
	
	$PrjObj = ProjectPeer::retrieveByUnixName($project_unix_name);
	if (is_null($PrjObj) || !$PrjObj->userCanRead()) {
		return new soap_fault('', 'getDiskUsages','Permission Denied - Project Doesn\'t Exist or You Cannot View This Record','Permission Denied - Project Doesn\'t Exist or You Cannot View This Record');
	}

	$crit = new Criteria();
	$crit->add(DiskUsagePeer::PROJECT_ID, $PrjObj->getProjectId());

	if (!is_null($check_date) && $check_date != '') {
		$crit->add(DiskUsagePeer::CHECK_DATE, $check_date);
	}
	
	$objs = DiskUsagePeer::doSelect($crit);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = DiskUsageToSOAP($obj);
	}
	return $res;
}



function DiskUsageToSOAP(&$obj) {
	$ret = array();
	$ret['project_id'] = $obj->getProjectId();
	$ret['check_date'] = $obj->getCheckDate();
	$ret['bytes_used'] = $obj->getBytesUsed();
	return $ret;
}
?>
