<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling Theme

require_once('core/Theme.php');
require_once('core/ThemePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'Theme',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'theme_id' => array('name'=>'theme_id', 'type' => 'xsd:int'),
		'dirname' => array('name'=>'dirname', 'type' => 'xsd:string'),
		'fullname' => array('name'=>'fullname', 'type' => 'xsd:string'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfTheme',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:Theme[]')),
	'tns:Theme'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getThemes',
	array(
		'sessionHash'	=> 'xsd:string'
	), 
	array('getThemes' => 'tns:ArrayOfTheme'),
	$uri, 
	$uri.'#getThemes',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve the list of Themes
 */
function getThemes($sessionHash) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$c = new Criteria();
	$objs = ThemePeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = ThemeToSOAP($obj);
	}
	return $res;
}

$server->register(
	'getThemeArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'theme_id' => 'tns:ArrayOfInt',
	), 
	array('getTheme' => 'tns:ArrayOfTheme'),
	$uri, 
	$uri.'#getTheme',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of Theme
 * @param	$theme_id	Retrieve the Theme array identified by the array of theme_id
 */
function getThemeArray($sessionHash, $theme_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$ret = array();

	$crit=new Criteria();
        $crit->add(ThemePeer::THEME_ID,$theme_idArr,Criteria::IN);
	$objs = ThemePeer::doSelect($crit);
	foreach ($objs as $obj) {
		$ret[] = ThemeToSOAP($obj);
	}
	return $ret;
}

$server->register(
	'getTheme',
	array(
		'sessionHash'	=> 'xsd:string',
		'theme_id' => 'xsd:int',
	), 
	array('getTheme' => 'tns:Theme'),
	$uri, 
	$uri.'#getTheme',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single Theme
 * @param	$theme_id	Retrieve the Theme identified by this theme_id
 */
function getTheme($sessionHash, $theme_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = ThemePeer::retrieveByPK($theme_id);
	if (is_null($obj)) {
		return null;
	}
	return ThemeToSOAP($obj);
}


function ThemeToSOAP(&$obj) {
	$ret = array();
	$ret['theme_id'] = $obj->getThemeId();
	$ret['dirname'] = $obj->getDirname();
	$ret['fullname'] = $obj->getFullname();
	return $ret;
}
?>