<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling DocmanFile

require_once('plugins/docman/lib/DocmanFile.php');
require_once('plugins/docman/lib/DocmanFilePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'DocmanFile',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'docman_file_id' => array('name'=>'docman_file_id', 'type' => 'xsd:int'),
		'docman_folder_id' => array('name'=>'docman_folder_id', 'type' => 'xsd:int'),
		'description' => array('name'=>'description', 'type' => 'xsd:string'),
		'is_locked' => array('name'=>'is_locked', 'type' => 'xsd:boolean'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfDocmanFile',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:DocmanFile[]')),
	'tns:DocmanFile'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getDocmanFiles',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_folder_id' => 'xsd:int'
	), 
	array('getDocmanFiles' => 'tns:ArrayOfDocmanFile'),
	$uri, 
	$uri.'#getDocmanFiles',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of DocmanFile
 * @param	$docman_folder_id	Filter by docman_folder_id
 */
function getDocmanFiles($sessionHash, $docman_folder_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$c = new Criteria();
	
	if (!is_null($docman_folder_id)) {
		$c->add(DocmanFilePeer::DOCMAN_FOLDER_ID, $docman_folder_id);
	}
	
	$objs = DocmanFilePeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$res[] = DocmanFileToSOAP($obj);
		}
	}
	
	return $res;
	
}

$server->register(
	'getDocmanFileArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_file_id' => 'tns:ArrayOfInt',
	), 
	array('getDocmanFile' => 'tns:ArrayOfDocmanFile'),
	$uri, 
	$uri.'#getDocmanFile',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of DocmanFile
 * @param	$docman_file_id	Retrieve the DocmanFile array identified by the array of docman_file_id
 */
function getDocmanFileArray($sessionHash, $docman_file_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$ret = array();

	$crit=new Criteria();
        $crit->add(DocmanFilePeer::DOCMAN_FILE_ID,$docman_file_idArr,Criteria::IN);
	$objs = DocmanFilePeer::doSelect($crit);
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$ret[] = DocmanFileToSOAP($obj);
		}
	}
	return $ret;
}

$server->register(
	'getDocmanFile',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_file_id' => 'xsd:int',
	), 
	array('getDocmanFile' => 'tns:DocmanFile'),
	$uri, 
	$uri.'#getDocmanFile',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single DocmanFile
 * @param	$docman_file_id	Retrieve the DocmanFile identified by this docman_file_id
 */
function getDocmanFile($sessionHash, $docman_file_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = DocmanFilePeer::retrieveByPK($docman_file_id);
	if (is_null($obj)) {
		return null;
	}
	if (!$obj->userCanRead()) {
		return new soap_fault('', 'getDocmanFile','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return DocmanFileToSOAP($obj);
}


$server->register(
	'deleteDocmanFile',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_file_id' => 'xsd:int',
	), 
	array('deleteDocmanFile' => 'xsd:int'),
	$uri, 
	$uri.'#deleteDocmanFile',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single DocmanFile
 * @param	$docman_file_id	Delete the DocmanFile identified by this docman_file_id
 */
function deleteDocmanFile($sessionHash, $docman_file_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = DocmanFilePeer::retrieveByPK($docman_file_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteDocmanFile','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
		
		//Delete entries in docman docman file folder
		$c = new Criteria();
		$c->add(DocmanFileFolderPeer::DOCMAN_FILE_ID, $docman_file_id);
		DocmanFileFolderPeer::doDelete($c);
	} catch (Exception $e) {
		return new soap_fault('', 'deleteDocmanFile','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}


$server->register(
	'updateDocmanFile',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_file_id' => 'xsd:int',
		'description' => 'xsd:string',
		'is_locked' => 'xsd:boolean',
	), 
	array('updateDocmanFile' => 'xsd:int'),
	$uri, 
	$uri.'#updateDocmanFile',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update DocmanFile
 * @param	$docman_folder_id
 * @param	$description
 * @param	$is_locked
 */
function updateDocmanFile($sessionHash, $docman_file_id, $description, $is_locked) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = DocmanFilePeer::retrieveByPK($docman_file_id);
	if (is_null($obj)) {
		return new soap_fault('', 'updateDocmanFile','Couldn\'t update DocmanFile: invalid docman_file_id '.$docman_file_id);
	}
	
	if (!is_null($description)) {
		$obj->setDescription( Validator::sanitizeString($description) );
	}
	if (!is_null($is_locked)) {
		$obj->setIsLocked( Validator::sanitizeBool($is_locked) );
	}
	
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateDocmanFile','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}



$server->register(
	'addDocmanFile',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_folder_id' => 'xsd:int',
		'description' => 'xsd:string',
		'is_locked' => 'xsd:boolean',
	), 
	array('addDocmanFile' => 'xsd:int'),
	$uri, 
	$uri.'#getDocmanFiles',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a DocmanFile
 * @param	$docman_file_id
 * @param	$docman_folder_id
 * @param	$description
 * @param	$is_locked
 */
function addDocmanFile($sessionHash, $docman_folder_id, $description, $is_locked) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$docFolder = DocmanFolderPeer::retrieveByPK($docman_folder_id);
	if (is_null($docFolder) || (!$docFolder->isPublic() && !$docFolder->userCanUpdate())) {
		return new soap_fault('', 'addDocmanFile','Permission Denied - Cannot add This Record', 'Permission Denied - Cannot add This Record');
	}

	$is_locked =  Validator::sanitizeBool($is_locked);

	$obj = new DocmanFile();
	
	$obj->setDocmanFolderId( Validator::sanitizeInt($docman_folder_id) );
	$obj->setDescription( Validator::sanitizeString($description) );
	$obj->setIsLocked( $is_locked );
	if ($is_locked) {
		$user_lock = Session::getSession()->getUser()->getUserId();
	} else {
		$user_lock = User::UNDEFINED_USER_ID;
	}
	$obj->setLockedBy($user_lock);

	try {
		$obj->save();

		//Create an entry en the docman file folder
		$docman_file_folder = new DocmanFileFolder();
		$docman_file_folder->setDocmanFolderId( $docman_folder_id );
		$docman_file_folder->setDocmanFileId( $obj->getDocmanFileId() );
		$docman_file_folder->save();

		return $obj->getDocmanFileId();
	} catch (Exception $e) {
		return new soap_fault('', 'addDocmanFile','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function DocmanFileToSOAP(&$obj) {
	$ret = array();
	$ret['docman_file_id'] = $obj->getDocmanFileId();
	$ret['docman_folder_id'] = $obj->getDocmanFolderId();
	$ret['description'] = $obj->getDescription();
	$ret['is_locked'] = $obj->getIsLocked();
	return $ret;
}


$server->register(
	'assignDocmanFileToDocmanFolder',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_file_id' => 'xsd:int',
		'docman_folder_id' => 'xsd:int'
	), 
	array('assignDocmanFileToDocmanFolder' => 'xsd:int'),
	$uri, 
	$uri.'#assignDocmanFileToDocmanFolder',
	'rpc',
	'encoded'
);

/**
 * SOAP method to assign a DocmanFile to a DocmanFolder
 * @param	$docman_file_id
 * @param	$docman_folder_id
 */
function assignDocmanFileToDocmanFolder($sessionHash, $docman_file_id, $docman_folder_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$docFolder = DocmanFolderPeer::retrieveByPK($docman_folder_id);
	if (is_null($docFolder) || (!$docFolder->isPublic() && !$docFolder->userCanUpdate())) {
		return new soap_fault('', 'assignDocmanFileToDocmanFolder','Permission Denied - Cannot add This Record', 'Permission Denied - Cannot add This Record');
	}

	$docFile = DocmanFilePeer::retrieveByPK($docman_file_id);
	if (is_null($docFile) || (!$docFile->isPublic() && !$docFile->userCanUpdate())) {
		return new soap_fault('', 'assignDocmanFileToDocmanFolder','Permission Denied - Cannot add This Record', 'Permission Denied - Cannot add This Record');
	}

	$obj = new DocmanFileFolder();
	
	$obj->setDocmanFolderId( Validator::sanitizeInt($docman_folder_id) );
	$obj->setDocmanFileId( Validator::sanitizeInt($docman_file_id) );
	try {
		$obj->save();
		return $obj->getDocmanFileFolderId();
	} catch (Exception $e) {
		return new soap_fault('', 'assignDocmanFileToDocmanFolder','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}


$server->register(
	'removeDocmanFileFromDocmanFolder',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_file_id' => 'xsd:int',
		'docman_folder_id' => 'xsd:int'
	), 
	array('removeDocmanFileFromDocmanFolder' => 'xsd:int'),
	$uri, 
	$uri.'#removeDocmanFileFromDocmanFolder',
	'rpc',
	'encoded'
);


/**
 * SOAP method to remove a DocmanFile to a DocmanFolder
 * @param	$docman_file_id
 * @param	$docman_folder_id
 */
function removeDocmanFileFromDocmanFolder($sessionHash, $docman_file_id, $docman_folder_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$docFolder = DocmanFolderPeer::retrieveByPK($docman_folder_id);
	if (is_null($docFolder) || (!$docFolder->isPublic() && !$docFolder->userCanUpdate())) {
		return new soap_fault('', 'removeDocmanFileFromDocmanFolder','Permission Denied - Cannot add This Record', 'Permission Denied - Cannot add This Record');
	}

	$docFile = DocmanFilePeer::retrieveByPK($docman_file_id);
	if (is_null($docFile) || (!$docFile->isPublic() && !$docFile->userCanUpdate())) {
		return new soap_fault('', 'removeDocmanFileFromDocmanFolder','Permission Denied - Cannot add This Record', 'Permission Denied - Cannot add This Record');
	}

	$obj = new DocmanFileFolder();
	
	$obj->setDocmanFolderId( Validator::sanitizeInt($docman_folder_id) );
	$obj->setDocmanFileId( Validator::sanitizeInt($docman_file_id) );
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'removeDocmanFileFromDocmanFolder','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}
?>
