<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling ForumMessage

require_once('plugins/forum/lib/ForumMessage.php');
require_once('plugins/forum/lib/ForumMessagePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'ForumMessage',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'forum_message_id' => array('name'=>'forum_message_id', 'type' => 'xsd:int', 'nillable' => 'false'),
		'forum_thread_id' => array('name'=>'forum_thread_id', 'type' => 'xsd:int', 'nillable' => 'false'),
		'created_by' => array('name'=>'created_by', 'type' => 'xsd:int', 'nillable' => 'false'),
		'subject' => array('name'=>'subject', 'type' => 'xsd:string', 'nillable' => 'false'),
		'body' => array('name'=>'body', 'type' => 'xsd:string', 'nillable' => 'false'),
		'post_date' => array('name'=>'post_date', 'type' => 'xsd:string', 'nillable' => 'false'),
		'parent_forum_message_id' => array('name'=>'parent_forum_message_id', 'type' => 'xsd:int'),
		'mail_msg_id' => array('name'=>'mail_msg_id', 'type' => 'xsd:string', 'nillable' => 'true'),
		'mail_sender' => array('name'=>'mail_sender', 'type' => 'xsd:string', 'nillable' => 'true'),
		'is_approved' => array('name'=>'is_approved', 'type' => 'xsd:int', 'nillable' => 'false'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfForumMessage',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:ForumMessage[]')),
	'tns:ForumMessage'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getForumMessages',
	array(
		'sessionHash'	=> 'xsd:string',
		'forum_thread_id' => 'xsd:int',
		'created_by' => 'xsd:int',
		'subject' => 'xsd:string',
		'body' => 'xsd:string',
		'post_date_min' => 'xsd:string',
		'post_date_max' => 'xsd:string',
		'parent_forum_message_id' => 'xsd:int',
		'mail_msg_id' => 'xsd:string',
		'mail_sender' => 'xsd:string',
		'is_approved' => 'xsd:int',
	), 
	array('getForumMessages' => 'tns:ArrayOfForumMessage'),
	$uri, 
	$uri.'#getForumMessages',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of ForumMessage
 * @param	$forum_message_id	Filter by forum_message_id
 * @param	$forum_thread_id	Filter by forum_thread_id
 * @param	$created_by	Filter by created_by
 * @param	$subject	Filter by subject
 * @param	$body	Filter by body
 * @param	$post_date	Filter by post_date
 * @param	$parent_forum_message_id	Filter by parent_forum_message_id
 * @param	$mail_msg_id	Filter by mail_msg_id
 * @param	$mail_sender	Filter by mail_sender
 * @param	$is_approved	Filter by is_approved
 */
function getForumMessages($sessionHash, $forum_thread_id, $created_by, $subject, $body, $post_date_min, $post_date_max, $parent_forum_message_id, $mail_msg_id, $mail_sender, $is_approved) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$crit = new Criteria();
	
	if (!is_null($forum_thread_id) && $forum_thread_id>0) {
		$crit->add(ForumMessagePeer::FORUM_THREAD_ID, $forum_thread_id);
	}
	if (!is_null($created_by) && $created_by > 0) {
		$crit->add(ForumMessagePeer::CREATED_BY, $created_by);
	}
	if (!is_null($subject) && $subject != '') {
		$crit->add(ForumMessagePeer::SUBJECT, "%$subject%", Criteria::ILIKE );
	}
	if (!is_null($body) && $body != '') {
		$crit->add(ForumMessagePeer::BODY, "%$body%", Criteria::ILIKE);
	}
	if (!is_null($post_date_min) && $post_date_min != '') {
		$crit->add(ForumMessagePeer::POST_DATE, $post_date_min, Criteria::GREATER_EQUAL );
	}
	if (!is_null($post_date_max) && $post_date_max != '') {
		$crit->add(ForumMessagePeer::POST_DATE, $post_date_min, Criteria::LESS_EQUAL );
	}
	if (!is_null($parent_forum_message_id) && $parent_forum_message_id > 0) {
		$crit->add(ForumMessagePeer::PARENT_FORUM_MESSAGE_ID, $parent_forum_message_id);
	}
	if (!is_null($mail_msg_id) && $mail_msg_id != '') {
		$crit->add(ForumMessagePeer::MAIL_MSG_ID, $mail_msg_id);
	}
	if (!is_null($mail_sender) && $mail_sender != '') {
		$crit->add(ForumMessagePeer::MAIL_SENDER, $mail_sender);
	}
	if (!is_null($is_approved) && ($is_approved==0 || $is_approved==1)) {
		$crit->add(ForumMessagePeer::IS_APPROVED, $is_approved==1 ? true:false);
	}
	
	$objs = ForumMessagePeer::doSelect($crit);
	$res = array();
	foreach ($objs as $obj) {
		if ($obj->getForumThread()->getForum()->userCanRead()) {
			$res[] = ForumMessageToSOAP($obj);
		}
	}
	return $res;
}

$server->register(
	'getForumMessageArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'forum_message_id' => 'ArrayOfxsd:int',
	), 
	array('getForumMessage' => 'tns:ArrayOfForumMessage'),
	$uri, 
	$uri.'#getForumMessage',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of ForumMessage
 * @param	$forum_message_id	Retrieve the ForumMessage array identified by the array of forum_message_id
 */
function getForumMessageArray($sessionHash, $forum_message_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$ret = array();

	$crit=new Criteria();
        $crit->add(ForumMessagePeer::FORUM_MESSAGE_ID,$forum_message_idArr,Criteria::IN);
	$objs = ForumMessagePeer::doSelect($crit);
	foreach ($objs as $obj) {
		if ($obj->getForumThread()->getForum()->userCanRead()) {
			$ret[] = ForumMessageToSOAP($obj);
		}
	}
	return $ret;
}

$server->register(
	'getForumMessage',
	array(
		'sessionHash'	=> 'xsd:string',
		'forum_message_id' => 'xsd:int',
	), 
	array('getForumMessage' => 'tns:ForumMessage'),
	$uri, 
	$uri.'#getForumMessage',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single ForumMessage
 * @param	$forum_message_id	Retrieve the ForumMessage identified by this forum_message_id
 */
function getForumMessage($sessionHash, $forum_message_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = ForumMessagePeer::retrieveByPK($forum_message_id);
	if (is_null($obj)) {
		return null;
	}
	if (!$obj->getForumThread()->getForum()->userCanRead()) {
		return new soap_fault('', 'getForumMessage','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return ForumMessageToSOAP($obj);
}

$server->register(
	'updateForumMessage',
	array(
		'sessionHash'	=> 'xsd:string',
		'forum_message_id' => 'xsd:int',
		'subject' => 'xsd:string',
		'body' => 'xsd:string',
		'is_approved' => 'xsd:boolean',
	), 
	array('updateForumMessage' => 'xsd:int'),
	$uri, 
	$uri.'#updateForumMessage',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update ForumMessage
 * @param	$forum_message_id	 ForumMessage to update
 * @param	$forum_thread_id
 * @param	$created_by
 * @param	$subject
 * @param	$body
 * @param	$post_date
 * @param	$parent_forum_message_id
 * @param	$mail_msg_id
 * @param	$mail_sender
 * @param	$is_approved
 */
function updateForumMessage($sessionHash, $forum_message_id, $subject, $body, $is_approved) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = ForumMessagePeer::retrieveByPK($forum_message_id);
	if (is_null($obj)) {
		throw new soap_fault('', 'updateForumMessage', 'Couldn\'t update ForumMessage: invalid forum_message_id '.$forum_message_id);
	}
	
	if(!$obj->userCanUpdate()) {
		return new soap_fault('', 'updateForumMessage', 'Couldn\'t update ForumMessage: You dont have update permissions ');		
	}
	
	if (!is_null($subject)) {
		$obj->setSubject( Validator::sanitizeString($subject) );
	}
	if (!is_null($body)) {
		$obj->setBody( Validator::sanitizeString($body) );
	}
	if (!is_null($is_approved)) {
		$obj->setIsApproved( Validator::sanitizeBool($is_approved) );
	}
	
	try {		
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateForumMessage','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}



$server->register(
	'addForumMessage',
	array(
		'sessionHash'	=> 'xsd:string',
		'forum_thread_id' => 'xsd:int',
		'subject' => 'xsd:string',
		'body' => 'xsd:string',
		'parent_forum_message_id' => 'xsd:int',
		'mail_msg_id' => 'xsd:string',
		'mail_sender' => 'xsd:string',
		'is_approved' => 'xsd:boolean',
	), 
	array('addForumMessage' => 'xsd:int'),
	$uri, 
	$uri.'#addForumMessage',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a ForumMessage
 * @param	$forum_thread_id
 * @param	$created_by
 * @param	$subject
 * @param	$body
 * @param	$post_date
 * @param	$parent_forum_message_id
 * @param	$mail_msg_id
 * @param	$mail_sender
 * @param	$is_approved
 */
function addForumMessage($sessionHash, $forum_thread_id, $subject, $body, $parent_forum_message_id, $mail_msg_id, $mail_sender, $is_approved) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);
	
	$forumThread = ForumThreadPeer::retrieveByPk($forum_thread_id);
	if (!is_object($forumThread)) {
		return new soap_fault('', 'addForumMessage','Permission Denied - Invalid Forum Thread');		
	}
	
	if (!$forumThread->getForum()->userCanPost(Session::getSession()->getUser())) {
		return new soap_fault('', 'addForumMessage','Permission Denied - You dont have posting permissions in this Forum');
	}
	
	$obj = new ForumMessage();
	
	$obj->setForumThreadId( Validator::sanitizeInt($forum_thread_id) );
	$obj->setCreatedBy( Session::getSession()->getUser()->getUserId() );
	$obj->setSubject( Validator::sanitizeString($subject) );
	$obj->setBody( Validator::sanitizeString($body) );
	
	$parentMsg = ForumMessagePeer::retrieveByPk($parent_forum_message_id);
	if(is_object($parentMsg)) {
		$obj->setParentForumMessageId( Validator::sanitizeInt($parent_forum_message_id) );
	}
	
	$obj->setMailMsgId( Validator::sanitizeString($mail_msg_id) );
	$obj->setMailSender( Validator::sanitizeString($mail_sender) );
	$obj->setIsApproved( Validator::sanitizeBool($is_approved) );
	try {
		$obj->save();
		return $obj->getForumMessageId();
	} catch (Exception $e) {
		return new soap_fault('', 'addForumMessage','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}


$server->register(
	'deleteForumMessage',
	array(
		'sessionHash'	=> 'xsd:string',
		'forum_message_id' => 'xsd:int',
	), 
	array('deleteForumMessage' => 'xsd:int'),
	$uri, 
	$uri.'#deleteForumMessage',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single ForumMessage
 * @param	$forum_message_id
 */
function deleteForumMessage($sessionHash, $forum_message_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = ForumMessagePeer::retrieveByPK($forum_message_id);
	if (is_null($obj) || !$obj->userCanDelete()) {
		return new soap_fault('', 'deleteForumMessage','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteForumMessage','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}

function ForumMessageToSOAP(&$obj) {
	$ret = array();
	$ret['forum_message_id'] = $obj->getForumMessageId();
	$ret['forum_thread_id'] = $obj->getForumThreadId();
	$ret['created_by'] = $obj->getCreatedBy();
	$ret['subject'] = $obj->getSubject();
	$ret['body'] = $obj->getBody();
	$ret['post_date'] = $obj->getPostDate();
	$ret['parent_forum_message_id'] = $obj->getParentForumMessageId();
	$ret['mail_msg_id'] = $obj->getMailMsgId();
	$ret['mail_sender'] = $obj->getMailSender();
	$ret['is_approved'] = $obj->getIsApproved();
	return $ret;
}
?>