<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling TrackerExtraFieldElement

require_once('plugins/tracker/lib/TrackerExtraFieldElement.php');
require_once('plugins/tracker/lib/TrackerExtraFieldElementPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'TrackerExtraFieldElement',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'element_id' => array('name'=>'element_id', 'type' => 'xsd:int'),
		'tracker_extra_field_id' => array('name'=>'tracker_extra_field_id', 'type' => 'xsd:int'),
		'element_name' => array('name'=>'element_name', 'type' => 'xsd:string'),
		'status_id' => array('name'=>'status_id', 'type' => 'xsd:int'),
		'default_selected' => array('name'=>'default_selected', 'type' => 'xsd:boolean'),
		'sort_order' => array('name'=>'sort_order', 'type' => 'xsd:int'),
		'auto_assign_to' => array('name'=>'auto_assign_to', 'type' => 'xsd:int'),
		'can_commit' => array('name'=>'can_commit', 'type' => 'xsd:boolean'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfTrackerExtraFieldElement',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:TrackerExtraFieldElement[]')),
	'tns:TrackerExtraFieldElement'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getTrackerExtraFieldElements',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_extra_field_id' => 'xsd:int',
		'element_name' => 'xsd:string',
		'status_id' => 'xsd:int',
		'auto_assign_to' => 'xsd:int',
	), 
	array('getTrackerExtraFieldElements' => 'tns:ArrayOfTrackerExtraFieldElement'),
	$uri, 
	$uri.'#getTrackerExtraFieldElements',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of TrackerExtraFieldElement
 * @param	$tracker_extra_field_id	Filter by tracker_extra_field_id
 * @param	$element_name	Filter by element_name
 * @param	$status_id	Filter by status_id
 * @param	$auto_assign_to	Filter by auto_assign_to
 */
function getTrackerExtraFieldElements($sessionHash, $tracker_extra_field_id, $element_name, $status_id, $auto_assign_to) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$c = new Criteria();
	
	if ($tracker_extra_field_id >= 0) {
		$c->add(TrackerExtraFieldElementPeer::TRACKER_EXTRA_FIELD_ID, $tracker_extra_field_id);
	}
	if ($element_name != '') {
		$c->add(TrackerExtraFieldElementPeer::ELEMENT_NAME, $element_name);
	}
	if ($status_id >= 0) {
		$c->add(TrackerExtraFieldElementPeer::STATUS_ID, $status_id);
	}
	if ($auto_assign_to >= 0) {
		$c->add(TrackerExtraFieldElementPeer::AUTO_ASSIGN_TO, $auto_assign_to);
	}
	
	$objs = TrackerExtraFieldElementPeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		if (checkTrackerExtraFieldPermission($obj->getTrackerExtraFieldId())) {
			$res[] = TrackerExtraFieldElementToSOAP($obj);
		}
	}
	
	return $res;
	
}

$server->register(
	'getTrackerExtraFieldElementArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_extra_field_id' => 'xsd:int',
		'element_id' => 'tns:ArrayOfInt',
	), 
	array('getTrackerExtraFieldElement' => 'tns:ArrayOfTrackerExtraFieldElement'),
	$uri, 
	$uri.'#getTrackerExtraFieldElement',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of TrackerExtraFieldElement
 * @param	$tracker_extra_field_id
 * @param	$element_id	Retrieve the TrackerExtraFieldElement array identified by the array of element_id
 */
function getTrackerExtraFieldElementArray($sessionHash, $tracker_extra_field_id, $element_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerExtraFieldPermission($tracker_extra_field_id)) {
		return new soap_fault('', 'getTrackerExtraFieldElementArray','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$ret = array();

	$crit=new Criteria();
    $crit->add(TrackerExtraFieldElementPeer::ELEMENT_ID,$element_idArr,Criteria::IN);
    $crit->add(TrackerExtraFieldElementPeer::TRACKER_EXTRA_FIELD_ID,$tracker_extra_field_id);
	$objs = TrackerExtraFieldElementPeer::doSelect($crit);
	foreach ($objs as $obj) {
		$ret[] = TrackerExtraFieldElementToSOAP($obj);
	}
	return $ret;
}

$server->register(
	'getTrackerExtraFieldElement',
	array(
		'sessionHash'	=> 'xsd:string',
		'element_id' => 'xsd:int'
	), 
	array('getTrackerExtraFieldElement' => 'tns:TrackerExtraFieldElement'),
	$uri, 
	$uri.'#getTrackerExtraFieldElement',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single TrackerExtraFieldElement
 * @param	$element_id	Retrieve the TrackerExtraFieldElement identified by this element_id
 */
function getTrackerExtraFieldElement($sessionHash, $element_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerExtraFieldElementPeer::retrieveByPK($element_id);
	if (is_null($obj)) {
		return null;
	}

	if (!checkTrackerExtraFieldPermission($obj->getTrackerExtraFieldId())) {
		return new soap_fault('', 'getTrackerExtraFieldElement','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return TrackerExtraFieldElementToSOAP($obj);
}


$server->register(
	'deleteTrackerExtraFieldElement',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_extra_field_id' => 'xsd:int',
		'element_id' => 'xsd:int',
	), 
	array('deleteTrackerExtraFieldElement' => 'xsd:int'),
	$uri, 
	$uri.'#deleteTrackerExtraFieldElement',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single TrackerExtraFieldElement
 * @param	$tracker_extra_field_id
 * @param	$element_id	Delete the TrackerExtraFieldElement identified by this element_id
 */
function deleteTrackerExtraFieldElement($sessionHash, $tracker_extra_field_id, $element_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerExtraFieldPermission($tracker_extra_field_id, 'update')) {
		return new soap_fault('', 'deleteTrackerExtraFieldElement','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}

	$obj = TrackerExtraFieldElementPeer::retrieveByPK($element_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteTrackerExtraFieldElement','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
		return 1;
	} catch (Exception $e) {
		return new soap_fault('', 'deleteTrackerExtraFieldElement','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
}


$server->register(
	'updateTrackerExtraFieldElement',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_extra_field_id' => 'xsd:int',
		'element_id' => 'xsd:int',
		'element_name' => 'xsd:string',
		'status_id' => 'xsd:int',
		'default_selected' => 'xsd:boolean',
		'sort_order' => 'xsd:int',
		'auto_assign_to' => 'xsd:int',
		'can_commit' => 'xsd:boolean',
	), 
	array('updateTrackerExtraFieldElement' => 'xsd:int'),
	$uri, 
	$uri.'#updateTrackerExtraFieldElement',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update TrackerExtraFieldElement
 * @param	$element_id	 TrackerExtraFieldElement to update
 * @param	$tracker_extra_field_id
 * @param	$element_name
 * @param	$status_id
 * @param	$default_selected
 * @param	$sort_order
 * @param	$auto_assign_to
 * @param	$can_commit
 */
function updateTrackerExtraFieldElement($sessionHash, $element_id, $tracker_extra_field_id, $element_name, $status_id, $default_selected, $sort_order, $auto_assign_to, $can_commit) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerExtraFieldPermission($tracker_extra_field_id, 'update')) {
		return new soap_fault('', 'updateTrackerExtraFieldElement','Couldn\'t update TrackerExtraFieldElement: invalid element_id '.$element_id);
	}

	$obj = TrackerExtraFieldElementPeer::retrieveByPK($element_id);
	if (is_null($obj)) {
		return new soap_fault('', 'updateTrackerExtraFieldElement','Couldn\'t update TrackerExtraFieldElement: invalid element_id '.$element_id);
	}
	
	$auto_assign_to = Validator::sanitizeInt($auto_assign_to);
	if ($auto_assign_to == 0) {
		$auto_assign_to = User::UNDEFINED_USER_ID;
	}
	
	if (!is_null($element_name)) {
		$obj->setElementName( Validator::sanitizeString($element_name) );
	}
	if (!is_null($status_id)) {
		$obj->setStatusId( Validator::sanitizeInt($status_id) );
	}
	if (!is_null($default_selected)) {
		$obj->setDefaultSelected( Validator::sanitizeString($default_selected) );
	}
	if (!is_null($sort_order)) {
		$obj->setSortOrder( Validator::sanitizeInt($sort_order) );
	}
	if (!is_null($auto_assign_to)) {
		$obj->setAutoAssignTo( $auto_assign_to );
	}
	if (!is_null($can_commit)) {
		$obj->setCanCommit( Validator::sanitizeBool($can_commit) );
	}
	
	try {
		$obj->save();
		return 1;
	} catch (Exception $e) {
		return new soap_fault('', 'updateTrackerExtraFieldElement','Permission Denied - Cannot update This Record',$e->getMessage());
	}
}



$server->register(
	'addTrackerExtraFieldElement',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_extra_field_id' => 'xsd:int',
		'element_name' => 'xsd:string',
		'status_id' => 'xsd:int',
		'default_selected' => 'xsd:boolean',
		'sort_order' => 'xsd:int',
		'auto_assign_to' => 'xsd:int',
		'can_commit' => 'xsd:boolean',
	), 
	array('addTrackerExtraFieldElement' => 'xsd:int'),
	$uri, 
	$uri.'#addTrackerExtraFieldElement',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a TrackerExtraFieldElement
 * @param	$tracker_extra_field_id
 * @param	$element_name
 * @param	$status_id
 * @param	$default_selected
 * @param	$sort_order
 * @param	$auto_assign_to
 * @param	$can_commit
 */
function addTrackerExtraFieldElement($sessionHash, $tracker_extra_field_id, $element_name, $status_id, $default_selected, $sort_order, $auto_assign_to, $can_commit) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerExtraFieldPermission($tracker_extra_field_id, 'update')) {
		return new soap_fault('', 'addTrackerExtraFieldElement','Permission Denied - Cannot add This Record',$e->getMessage());
	}
	$auto_assign_to = Validator::sanitizeInt($auto_assign_to);
	if ($auto_assign_to == 0) {
		$auto_assign_to = User::UNDEFINED_USER_ID;
	}
	
	$obj = new TrackerExtraFieldElement();
	
	$obj->setTrackerExtraFieldId( Validator::sanitizeInt($tracker_extra_field_id) );
	$obj->setElementName( Validator::sanitizeString($element_name) );
	$obj->setStatusId( Validator::sanitizeInt($status_id) );
	$obj->setDefaultSelected( Validator::sanitizeString($default_selected) );
	$obj->setSortOrder( Validator::sanitizeInt($sort_order) );
	$obj->setAutoAssignTo( $auto_assign_to );
	$obj->setCanCommit( Validator::sanitizeBool($can_commit) );
	try {
		$obj->save();
		return $obj->getElementId();
	} catch (Exception $e) {
		return new soap_fault('', 'addTrackerExtraFieldElement','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function TrackerExtraFieldElementToSOAP(&$obj) {
	$ret = array();
	$ret['element_id'] = $obj->getElementId();
	$ret['tracker_extra_field_id'] = $obj->getTrackerExtraFieldId();
	$ret['element_name'] = $obj->getElementName();
	$ret['status_id'] = $obj->getStatusId();
	$ret['default_selected'] = $obj->getDefaultSelected();
	$ret['sort_order'] = $obj->getSortOrder();
	$ret['auto_assign_to'] = $obj->getAutoAssignTo();
	$ret['can_commit'] = $obj->getCanCommit();
	return $ret;
}
?>
