<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling TrackerItemRelation

require_once('plugins/tracker/lib/TrackerItemRelation.php');
require_once('plugins/tracker/lib/TrackerItemRelationPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'TrackerItemRelation',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'tracker_item_id' => array('name'=>'tracker_item_id', 'type' => 'xsd:int'),
		'related_tracker_item_id' => array('name'=>'related_tracker_item_id', 'type' => 'xsd:int'),
		'link_type' => array('name'=>'link_type', 'type' => 'xsd:string'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfTrackerItemRelation',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:TrackerItemRelation[]')),
	'tns:TrackerItemRelation'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getTrackerItemRelations',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'related_tracker_item_id' => 'xsd:int',
		'link_type' => 'xsd:string',
	), 
	array('getTrackerItemRelations' => 'tns:ArrayOfTrackerItemRelation'),
	$uri, 
	$uri.'#getTrackerItemRelations',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of TrackerItemRelation
 * @param	$tracker_item_id	Filter by tracker_item_id
 * @param	$related_tracker_item_id	Filter by related_tracker_item_id
 * @param	$link_type	Filter by link_type
 */
function getTrackerItemRelations($sessionHash, $tracker_item_id, $related_tracker_item_id, $link_type) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerItemPermission($tracker_item_id)) {
		return new soap_fault('', 'getTrackerItemRelations','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$c = new Criteria();
	$c->add(TrackerItemRelationPeer::TRACKER_ITEM_ID, $tracker_item_id);

	if (!is_null($related_tracker_item_id)) {
		$c->add(TrackerItemRelationPeer::RELATED_TRACKER_ITEM_ID, $related_tracker_item_id);
	}
	if (!is_null($link_type)) {
		$c->add(TrackerItemRelationPeer::LINK_TYPE, $link_type);
	}
	
	$objs = TrackerItemRelationPeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = TrackerItemRelationToSOAP($obj);
	}
	
	return $res;
	
}

$server->register(
	'deleteTrackerItemRelation',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'related_tracker_item_id' => 'xsd:int'
	), 
	array('deleteTrackerItemRelation' => 'xsd:int'),
	$uri, 
	$uri.'#deleteTrackerItemRelation',
	'rpc',
	'encoded'
);


/**
 * SOAP method to delete a TrackerItemRelation
 * @param	$tracker_item_id	Filter by tracker_item_id
 * @param	$related_tracker_item_id	Filter by related_tracker_item_id
 */
 
function deleteTrackerItemRelation($sessionHash, $tracker_item_id, $related_tracker_item_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerItemPermission($tracker_item_id, 'update')) {
		return new soap_fault('', 'deleteTrackerItemRelation','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot Delete This Record');
	}

	$c = new Criteria();
	$c->add(TrackerItemRelationPeer::TRACKER_ITEM_ID, $tracker_item_id);
	$c->add(TrackerItemRelationPeer::RELATED_TRACKER_ITEM_ID, $related_tracker_item_id);

	$obj = TrackerItemRelationPeer::doSelectOne($c);

	if (is_null($obj)) {
		return new soap_fault('', 'deleteTrackerItemRelation','Permission Denied - Cannot Delete This Record', 'Permission Denied - Cannot Delete This Record');
	}
	
	try {
		$obj->delete();
		return 1;
	} catch (Exception $e) {
		return new soap_fault('', 'deleteTrackerItemRelation','Permission Denied - Cannot Delete This Record', $e->getMessage());
	}
}


$server->register(
	'addTrackerItemRelation',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'related_tracker_item_id' => 'xsd:int',
		'link_type' => 'xsd:string',
	), 
	array('addTrackerItemRelation' => 'xsd:int'),
	$uri, 
	$uri.'#getTrackerItemRelation',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a TrackerItemRelation
 * @param	$tracker_item_id
 * @param	$related_tracker_item_id
 * @param	$link_type
 */
function addTrackerItemRelation($sessionHash, $tracker_item_id, $related_tracker_item_id, $link_type) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = new TrackerItemRelation();
	
	$obj->setTrackerItemId( Validator::sanitizeInt($tracker_item_id) );
	$obj->setRelatedTrackerItemId( Validator::sanitizeInt($related_tracker_item_id) );
	$obj->setLinkType( Validator::sanitizeString($link_type) );
	try {
		$obj->save();
		return 1;
	} catch (Exception $e) {
		return new soap_fault('', 'addTrackerItemRelation','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function TrackerItemRelationToSOAP(&$obj) {
	$ret = array();
	$ret['tracker_item_id'] = $obj->getTrackerItemId();
	$ret['related_tracker_item_id'] = $obj->getRelatedTrackerItemId();
	$ret['link_type'] = $obj->getLinkType();
	return $ret;
}
?>