<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling TrackerItemTime

require_once('plugins/tracker/lib/TrackerItemTime.php');
require_once('plugins/tracker/lib/TrackerItemTimePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'TrackerItemTime',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'tracker_item_id' => array('name'=>'tracker_item_id', 'type' => 'xsd:int'),
		'tracker_item_time_id' => array('name'=>'tracker_item_time_id', 'type' => 'xsd:int'),
		'tracker_time_code_id' => array('name'=>'tracker_time_code_id', 'type' => 'xsd:int'),
		'date_entry' => array('name'=>'date_entry', 'type' => 'xsd:string'),
		'user_id' => array('name'=>'user_id', 'type' => 'xsd:int'),
		'hours' => array('name'=>'hours', 'type' => 'xsd:double')
	)
);


$server->wsdl->addComplexType(
	'ArrayOfTrackerItemTime',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:TrackerItemTime[]')),
	'tns:TrackerItemTime'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getTrackerItemTimes',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'tracker_time_code_id' => 'xsd:int',
		'date_entry' => 'xsd:string',
		'user_id' => 'xsd:int',
		'hours' => 'xsd:double'
	), 
	array('getTrackerItemTimes' => 'tns:ArrayOfTrackerItemTime'),
	$uri, 
	$uri.'#getTrackerItemTimes',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of TrackerItemTime
 * @param	$tracker_item_id	Filter by tracker_item_id
 * @param	$tracker_time_code_id	Filter by tracker_time_code_id
 * @param	$date_entry	Filter by date_entry
 * @param	$user_id	Filter by user_id
 * @param	$hours	Filter by hours
 */
function getTrackerItemTimes($sessionHash, $tracker_item_id, $tracker_time_code_id, $date_entry, $user_id, $hours) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerItemPermission($tracker_item_id)) {
		return new soap_fault('', 'getTrackerItemTimes','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$c = new Criteria();
	$c->add(TrackerItemTimePeer::TRACKER_ITEM_ID, $tracker_item_id);

	if ($tracker_time_code_id > 0) {
		$c->add(TrackerItemTimePeer::TRACKER_TIME_CODE_ID, $tracker_time_code_id);
	}
	if ($date_entry != '') {
		$c->add(TrackerItemTimePeer::DATE_ENTRY, $date_entry);
	}
	if ($user_id > 0) {
		$c->add(TrackerItemTimePeer::USER_ID, $user_id);
	}
	if ($hours > 0) {
		$c->add(TrackerItemTimePeer::HOURS, $hours);
	}

	$objs = TrackerItemTimePeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = TrackerItemTimeToSOAP($obj);
	}
	
	return $res;
}

$server->register(
	'getTrackerItemTimeArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'tracker_item_time_id' => 'tns:ArrayOfInt',
	), 
	array('getTrackerItemTime' => 'tns:ArrayOfTrackerItemTime'),
	$uri, 
	$uri.'#getTrackerItemTime',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of TrackerItemTime
 * @param	$tracker_item_id	Filter by tracker_item_id
 * @param	$tracker_item_time_id	Retrieve the TrackerItemTime array identified by the array of tracker_item_time_id
 */
function getTrackerItemTimeArray($sessionHash, $tracker_item_id, $tracker_item_time_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerItemPermission($tracker_item_id)) {
		return new soap_fault('', 'getTrackerItemTimeArray','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$ret = array();

	$crit=new Criteria();
	$crit->add(TrackerItemTimePeer::TRACKER_ITEM_TIME_ID,$tracker_item_time_idArr,Criteria::IN);
	$objs = TrackerItemTimePeer::doSelect($crit);
	foreach ($objs as $obj) {
		$ret[] = TrackerItemTimeToSOAP($obj);
	}
	return $ret;
}

$server->register(
	'getTrackerItemTime',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'tracker_item_time_id' => 'xsd:int',
	), 
	array('getTrackerItemTime' => 'tns:TrackerItemTime'),
	$uri, 
	$uri.'#getTrackerItemTime',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single TrackerItemTime
 * @param	$tracker_item_id	Filter by tracker_item_id
 * @param	$tracker_item_time_id	Retrieve the TrackerItemTime identified by this tracker_item_time_id
 */
function getTrackerItemTime($sessionHash, $tracker_item_id, $tracker_item_time_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerItemPermission($tracker_item_id)) {
		return new soap_fault('', 'getTrackerItemTime','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$obj = TrackerItemTimePeer::retrieveByPK($tracker_item_time_id);
	if (is_null($obj)) {
		return null;
	}
	return TrackerItemTimeToSOAP($obj);
}


$server->register(
	'deleteTrackerItemTime',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'tracker_item_time_id' => 'xsd:int',
	), 
	array('deleteTrackerItemTime' => 'xsd:int'),
	$uri, 
	$uri.'#deleteTrackerItemTime',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single TrackerItemTime
 * @param	$tracker_item_time_id	Delete the TrackerItemTime identified by this tracker_item_time_id
 */
function deleteTrackerItemTime($sessionHash, $tracker_item_id, $tracker_item_time_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerItemPermission($tracker_item_id, 'update')) {
		return new soap_fault('', 'deleteTrackerItemTime','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$obj = TrackerItemTimePeer::retrieveByPK($tracker_item_time_id);
	if (is_null($obj) || $obj->getTrackerItemId() != $tracker_item_id) {
		return new soap_fault('', 'deleteTrackerItemTime','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
		return 1;
	} catch (Exception $e) {
		return new soap_fault('', 'deleteTrackerItemTime','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
}


$server->register(
	'updateTrackerItemTime',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'tracker_item_time_id' => 'xsd:int',
		'date_entry' => 'xsd:string',
		'user_id' => 'xsd:int',
		'hours' => 'xsd:double',
		'tracker_time_code_id' => 'xsd:int',
	), 
	array('updateTrackerItemTime' => 'xsd:int'),
	$uri, 
	$uri.'#updateTrackerItemTime',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update TrackerItemTime
 * @param	$tracker_item_id
 * @param	$tracker_item_time_id	 TrackerItemTime to update
 * @param	$date_entry
 * @param	$user_id
 * @param	$hours
 * @param	$tracker_time_code_id
 */
function updateTrackerItemTime($sessionHash, $tracker_item_id, $tracker_item_time_id, $date_entry, $user_id, $hours, $tracker_time_code_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerItemPermission($tracker_item_id, 'update')) {
		return new soap_fault('', 'updateTrackerItemTime','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$obj = TrackerItemTimePeer::retrieveByPK($tracker_item_time_id);
	if (is_null($obj) || $obj->getTrackerItemId() != $tracker_item_id) {
		return new soap_fault('', 'updateTrackerItemTime','Couldn\'t update TrackerItemTime: invalid tracker_item_time_id '.$tracker_item_time_id);
	}
	
	if (!is_null($date_entry)) {
		$obj->setDateEntry( Validator::sanitizeDate($date_entry) );
	}
	if (!is_null($user_id)) {
		$obj->setUserId( Validator::sanitizeInt($user_id) );
	}
	if (!is_null($tracker_item_id)) {
		$obj->setTrackerItemId( Validator::sanitizeInt($tracker_item_id) );
	}
	if (!is_null($hours)) {
		$obj->setHours( Validator::sanitizeString($hours) );
	}
	if (!is_null($tracker_time_code_id)) {
		$obj->setTrackerTimeCodeId( Validator::sanitizeInt($tracker_time_code_id) );
	}
	
	try {
		$obj->save();
		return 1;
	} catch (Exception $e) {
		return new soap_fault('', 'updateTrackerItemTime','Permission Denied - Cannot update This Record',$e->getMessage());
	}
}



$server->register(
	'addTrackerItemTime',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'tracker_time_code_id' => 'xsd:int',
		'date_entry' => 'xsd:string',
		'user_id' => 'xsd:int',
		'hours' => 'xsd:double'
	), 
	array('addTrackerItemTime' => 'xsd:int'),
	$uri, 
	$uri.'#addTrackerItemTime',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a TrackerItemTime
 * @param	$tracker_item_id
 * @param	$tracker_time_code_id
 * @param	$date_entry
 * @param	$user_id
 * @param	$tracker_item_id
 * @param	$hours
  */
function addTrackerItemTime($sessionHash, $tracker_item_id, $tracker_time_code_id, $date_entry, $user_id, $hours) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerItemPermission($tracker_item_id, 'update')) {
		return new soap_fault('', 'updateTrackerItemTime','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$obj = new TrackerItemTime();
	
	$obj->setDateEntry( Validator::sanitizeDate($date_entry) );
	$obj->setUserId( Validator::sanitizeInt($user_id) );
	$obj->setTrackerItemId( Validator::sanitizeInt($tracker_item_id) );
	$obj->setHours( Validator::sanitizeString($hours) );
	$obj->setTrackerTimeCodeId( Validator::sanitizeInt($tracker_time_code_id) );
	try {
		$obj->save();
		return $obj->getTrackerItemTimeId();
	} catch (Exception $e) {
		return new soap_fault('', 'addTrackerItemTime','Permission Denied - Cannot add This Record',$e->getMessage());
	}
	return 1;
}

function TrackerItemTimeToSOAP(&$obj) {
	$ret = array();
	$ret['tracker_item_id'] = $obj->getTrackerItemId();
	$ret['tracker_item_time_id'] = $obj->getTrackerItemTimeId();
	$ret['tracker_time_code_id'] = $obj->getTrackerTimeCodeId();
	$ret['date_entry'] = $obj->getDateEntry();
	$ret['user_id'] = $obj->getUserId();
	$ret['hours'] = $obj->getHours();
	return $ret;
}
?>
