<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling TrackerQuery

require_once('plugins/tracker/lib/TrackerQuery.php');
require_once('plugins/tracker/lib/TrackerQueryPeer.php');
require_once('plugins/tracker/lib/TrackerItemSavedQuery.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'TrackerQuery',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'tracker_query_id' => array('name'=>'tracker_query_id', 'type' => 'xsd:int'),
		'tracker_id' => array('name'=>'tracker_id', 'type' => 'xsd:int'),
		'user_id' => array('name'=>'user_id', 'type' => 'xsd:int'),
		'query_name' => array('name'=>'query_name', 'type' => 'xsd:string'),
		'is_shared' => array('name'=>'is_shared', 'type' => 'xsd:boolean'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfTrackerQuery',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:TrackerQuery[]')),
	'tns:TrackerQuery'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getTrackerQuerys',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
		'user_id' => 'xsd:int',
		'query_name' => 'xsd:string',
		'is_shared' => 'xsd:boolean'
	), 
	array('getTrackerQuerys' => 'tns:ArrayOfTrackerQuery'),
	$uri, 
	$uri.'#getTrackerQuerys',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of TrackerQuery
 * @param	$tracker_id	Filter by tracker_id
 * @param	$user_id	Filter by user_id
 * @param	$query_name	Filter by query_name
 * @param	$is_shared	Filter by is_shared
 */
function getTrackerQuerys($sessionHash, $tracker_id = 0, $user_id = 0, $query_name = '', $is_shared = null) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$c = new Criteria();
	
	if ($tracker_id >= 0) {
		$c->add(TrackerQueryPeer::TRACKER_ID, $tracker_id);
	}
	if ($user_id >= 0) {
		$c->add(TrackerQueryPeer::USER_ID, $user_id);
	}
	if ($query_name != "") {
		$c->add(TrackerQueryPeer::QUERY_NAME, $query_name);
	}
	if (!is_null($is_shared)) {
		$c->add(TrackerQueryPeer::IS_SHARED, $is_shared);
	}
	
	$objs = TrackerQueryPeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		if (checkQueryPermission($obj)) {
			$res[] = TrackerQueryToSOAP($obj);
		}
	}
	return $res;
}

$server->register(
	'getTrackerQueryArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_query_id' => 'tns:ArrayOfInt'
	), 
	array('getTrackerQuery' => 'tns:ArrayOfTrackerQuery'),
	$uri, 
	$uri.'#getTrackerQuery',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of TrackerQuery
 * @param	$tracker_query_id	Retrieve the TrackerQuery array identified by the array of tracker_query_id
 */
function getTrackerQueryArray($sessionHash, $tracker_query_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$ret = array();

	$crit=new Criteria();
	$crit->add(TrackerQueryPeer::TRACKER_QUERY_ID,$tracker_query_idArr,Criteria::IN);
	$objs = TrackerQueryPeer::doSelect($crit);
	foreach ($objs as $obj) {
		if (checkQueryPermission($obj)) {
			$ret[] = TrackerQueryToSOAP($obj);
		}
	}
	return $ret;
}

$server->register(
	'getTrackerQuery',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_query_id' => 'xsd:int'
	), 
	array('getTrackerQuery' => 'tns:TrackerQuery'),
	$uri, 
	$uri.'#getTrackerQuery',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single TrackerQuery
 * @param	$tracker_id	Filter by tracker_id
 * @param	$tracker_query_id	Retrieve the TrackerQuery identified by this tracker_query_id
 */
function getTrackerQuery($sessionHash, $tracker_query_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerQueryPeer::retrieveByPK($tracker_query_id);
	if (is_null($obj)) {
		return null;
	}
	if (!checkQueryPermission($obj)) {
		return new soap_fault('', 'getTrackerQuery','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return TrackerQueryToSOAP($obj);
}


$server->register(
	'getTrackerItemsByQueryId',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_query_id' => 'xsd:int',
		'limit' => 'xsd:int',
		'offset' => 'xsd:int'
	), 
	array('getTrackerItemsByQueryId' => 'tns:ArrayOfTrackerItem'),
	$uri, 
	$uri.'#getTrackerItemsByQueryId',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve TrackerItems by Query
 * @param	$tracker_query_id	Retrieve the TrackerQuery identified by this tracker_query_id
 * @param	$limit	Max items to retrieve (100 maximum)
 * @param	$offset	Where to start retrieving
 */
function getTrackerItemsByQueryId($sessionHash, $tracker_query_id, $limit=100, $offset=0) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerQueryPeer::retrieveByPK($tracker_query_id);
	if (is_null($obj) || !checkQueryPermission($obj)) {
		return new soap_fault('', 'getTrackerItemsByQueryId','Permission Denied - Cannot View These Records','Permission Denied - Cannot View This Record');
	}
	$Tracker = TrackerPeer::retrieveByPK($obj->getTrackerId());
	if (is_null($Tracker) || !$Tracker->userCanRead()) {
		return new soap_fault('', 'getTrackerItemsByQueryId','Permission Denied - Cannot View These Records','Permission Denied - Cannot View This Record');
	}
	$tq = new TrackerItemSavedQuery($Tracker, $obj);

	$ret = array();
	// TODO: should we still limit it to 100?
	$limit = ($limit > 100 || $limit==0) ? 100 : $limit;
	$objs = $tq->getTrackerItems($limit, $offset);
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$ret[] = TrackerItemToSOAP($obj);
		}
	}
	return $ret;
}


$server->register(
	'getTrackerItemsFullByQueryId',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_query_id' => 'xsd:int',
		'limit' => 'xsd:int',
		'offset' => 'xsd:int'
	), 
	array('getTrackerItemsFullByQueryId' => 'tns:ArrayOfTrackerItemFull'),
	$uri, 
	$uri.'#getTrackerItemsFullByQueryId',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve TrackerItemsFull by Query
 * @param	$tracker_query_id	Retrieve the TrackerQuery identified by this tracker_query_id
 * @param	$limit	Max items to retrieve (100 maximum)
 * @param	$offset	Where to start retrieving
 */
function getTrackerItemsFullByQueryId($sessionHash, $tracker_query_id, $limit=100, $offset=0) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerQueryPeer::retrieveByPK($tracker_query_id);
	if (is_null($obj) || !checkQueryPermission($obj)) {
		return new soap_fault('', 'getTrackerItemsByQueryId','Permission Denied - Cannot View This Records','Permission Denied - Cannot View This Record');
	}
	$Tracker = TrackerPeer::retrieveByPK($obj->getTrackerId());
	if (is_null($Tracker) || !$Tracker->userCanRead()) {
		return new soap_fault('', 'getTrackerItemsByQueryId','Permission Denied - Cannot View This Records','Permission Denied - Cannot View This Record');
	}
	$tq = new TrackerItemSavedQuery($Tracker, $obj);

	$ret = array();
	// TODO: should we still limit it to 100?
	$limit = ($limit > 100) ? 100 : $limit;
	$objs = $tq->getTrackerItems($limit, $offset);
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$ret[] = TrackerItemToSOAP($obj, true);
		}
	}
	return $ret;
}


$server->register(
	'deleteTrackerQuery',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_query_id' => 'xsd:int'
	), 
	array('deleteTrackerQuery' => 'xsd:int'),
	$uri, 
	$uri.'#deleteTrackerQuery',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single TrackerQuery
 * @param	$tracker_query_id	Delete the TrackerQuery identified by this tracker_query_id
 */
function deleteTrackerQuery($sessionHash, $tracker_query_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerQueryPeer::retrieveByPK($tracker_query_id);
	if (is_null($obj) || !checkQueryPermission($obj)) {
		return new soap_fault('', 'deleteTrackerQuery','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteTrackerQuery','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}


$server->register(
	'updateTrackerQuery',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_query_id' => 'xsd:int',
		'query_name' => 'xsd:string',
		'is_shared' => 'xsd:boolean',
	), 
	array('updateTrackerQuery' => 'xsd:int'),
	$uri, 
	$uri.'#updateTrackerQuery',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update TrackerQuery
 * @param	$tracker_query_id	 TrackerQuery to update
 * @param	$query_name
 * @param	$is_shared
 */
function updateTrackerQuery($sessionHash, $tracker_query_id, $query_name, $is_shared) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerQueryPeer::retrieveByPK($tracker_query_id);
	if (is_null($obj) || !checkQueryPermission($obj)) {
		return new soap_fault('', 'updateTrackerQuery','Couldn\'t update TrackerQuery: invalid tracker_query_id '.$tracker_query_id);
	}
	if ($query_name != "") {
		$obj->setQueryName( Validator::sanitizeString($query_name) );
	}
	if (!is_null($is_shared)) {
		$obj->setIsShared( Validator::sanitizeBool($is_shared) );
	}
	
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateTrackerQuery','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}


$server->register(
	'addTrackerQuery',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
		'user_id' => 'xsd:int',
		'query_name' => 'xsd:string',
		'is_shared' => 'xsd:boolean',
	), 
	array('addTrackerQuery' => 'xsd:int'),
	$uri, 
	$uri.'#addTrackerQuery',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a TrackerQuery
 * @param	$tracker_id
 * @param	$user_id
 * @param	$query_name
 * @param	$is_shared
 */
function addTrackerQuery($sessionHash, $tracker_id, $user_id, $query_name, $is_shared) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerPermission($tracker_id)) {
		return new soap_fault('', 'addTrackerQuery','Permission Denied - Cannot add This Record', 'Permission Denied - Cannot add This Record');
	}

	$obj = new TrackerQuery();
	$obj->setTrackerId( Validator::sanitizeInt($tracker_id) );
	$obj->setUserId( Validator::sanitizeInt($user_id) );
	$obj->setQueryName( Validator::sanitizeString($query_name) );
	$obj->setIsShared( Validator::sanitizeBool($is_shared) );
	try {
		$obj->save();
		return $obj->getTrackerQueryId();
	} catch (Exception $e) {
		return new soap_fault('', 'addTrackerQuery','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function checkQueryPermission($var) {
	if (is_object($var)) {
		$obj = $var;
	} else {
		$obj = TrackerQueryPeer::retrieveByPK($var);
	}
	if (is_null($obj)) {
		return false;
	} else if ($obj->getUserId() == Session::getSession()->getUser()->getUserId() 
		|| Session::getSession()->isSiteAdmin()) {
		return true;
	} else {
		return false;
	}
}

function TrackerQueryToSOAP(&$obj) {
	$ret = array();
	$ret['tracker_query_id'] = $obj->getTrackerQueryId();
	$ret['tracker_id'] = $obj->getTrackerId();
	$ret['user_id'] = $obj->getUserId();
	$ret['query_name'] = $obj->getQueryName();
	$ret['is_shared'] = $obj->getIsShared();
	return $ret;
}
?>