<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling Tracker

require_once('plugins/tracker/lib/Tracker.php');
require_once('plugins/tracker/lib/TrackerPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'Tracker',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'tracker_id' => array('name'=>'tracker_id', 'type' => 'xsd:int'),
		'project_id' => array('name'=>'project_id', 'type' => 'xsd:int'),
		'tracker_name' => array('name'=>'tracker_name', 'type' => 'xsd:string'),
		'description' => array('name'=>'description', 'type' => 'xsd:string'),
		'is_public' => array('name'=>'is_public', 'type' => 'xsd:boolean'),
		'restrict_browse' => array('name'=>'restrict_browse', 'type' => 'xsd:boolean'),
		'email_all_updates' => array('name'=>'email_all_updates', 'type' => 'xsd:boolean'),
		'email_address' => array('name'=>'email_address', 'type' => 'xsd:string'),
		'due_period' => array('name'=>'due_period', 'type' => 'xsd:int'),
		'submit_instructions' => array('name'=>'submit_instructions', 'type' => 'xsd:string'),
		'browse_instructions' => array('name'=>'browse_instructions', 'type' => 'xsd:string'),
		'datatype' => array('name'=>'datatype', 'type' => 'xsd:int'),
		'item_total' => array('name'=>'item_total', 'type' => 'xsd:int'),
		'open_count' => array('name'=>'open_count', 'type' => 'xsd:int'),
		'show_dependencies_tab' => array('name'=>'show_dependencies_tab', 'type' => 'xsd:boolean'),
		'show_duplicates_tab' => array('name'=>'show_duplicates_tab', 'type' => 'xsd:boolean'),
		'show_associations_tab' => array('name'=>'show_associations_tab', 'type' => 'xsd:boolean'),
		'show_time_tracking_tab' => array('name'=>'show_time_tracking_tab', 'type' => 'xsd:boolean'),
		'show_tags_tab' => array('name'=>'show_tags_tab', 'type' => 'xsd:boolean'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfTracker',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:Tracker[]')),
	'tns:Tracker'
);


$server->wsdl->addComplexType(
	'TrackerFull',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'tracker_id' => array('name'=>'tracker_id', 'type' => 'xsd:int'),
		'project_id' => array('name'=>'project_id', 'type' => 'xsd:int'),
		'tracker_name' => array('name'=>'tracker_name', 'type' => 'xsd:string'),
		'description' => array('name'=>'description', 'type' => 'xsd:string'),
		'is_public' => array('name'=>'is_public', 'type' => 'xsd:boolean'),
		'restrict_browse' => array('name'=>'restrict_browse', 'type' => 'xsd:boolean'),
		'email_all_updates' => array('name'=>'email_all_updates', 'type' => 'xsd:boolean'),
		'email_address' => array('name'=>'email_address', 'type' => 'xsd:string'),
		'due_period' => array('name'=>'due_period', 'type' => 'xsd:int'),
		'submit_instructions' => array('name'=>'submit_instructions', 'type' => 'xsd:string'),
		'browse_instructions' => array('name'=>'browse_instructions', 'type' => 'xsd:string'),
		'datatype' => array('name'=>'datatype', 'type' => 'xsd:int'),
		'item_total' => array('name'=>'item_total', 'type' => 'xsd:int'),
		'open_count' => array('name'=>'open_count', 'type' => 'xsd:int'),
		'extra_fields' => array('name'=>'extra_fields', 'type' => 'tns:ArrayOfTrackerExtraField'),
		'extra_field_elements' => array('name'=>'extra_field_elements', 'type' => 'tns:ArrayOfTrackerExtraFieldElement'),
		'techinicians' => array('name'=>'techinicians', 'type' => 'tns:ArrayOfUser')
	)
);


$server->wsdl->addComplexType(
	'ArrayOfTrackerFull',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:TrackerFull[]')),
	'tns:TrackerFull'
);
/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getTrackers',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int',
		'is_public' => 'xsd:boolean',
		'datatype' => 'xsd:int'
	), 
	array('getTrackers' => 'tns:ArrayOfTracker'),
	$uri, 
	$uri.'#getTrackers',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of Tracker
 * @param	$project_id	Filter by project_id
 * @param	$is_public	Filter by is_public
 * @param	$datatype	Filter by datatype
 */
function getTrackers($sessionHash, $project_id, $is_public, $datatype) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$c = new Criteria();
	
	if ($project_id >= 0) {
		$c->add(TrackerPeer::PROJECT_ID, $project_id);
	}
	if (!is_null($is_public)) {
		$c->add(TrackerPeer::IS_PUBLIC, $is_public);
	}
	if ($datatype >= 0) {
		$c->add(TrackerPeer::DATATYPE, $datatype);
	}
	
	$objs = TrackerPeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$res[] = TrackerToSOAP($obj);
		}
	}
	
	return $res;
	
}

$server->register(
	'getTrackerArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'tns:ArrayOfInt',
	), 
	array('getTracker' => 'tns:ArrayOfTracker'),
	$uri, 
	$uri.'#getTracker',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of Tracker
 * @param	$tracker_id	Retrieve the Tracker array identified by the array of tracker_id
 */
function getTrackerArray($sessionHash, $tracker_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$ret = array();

	$crit=new Criteria();
    $crit->add(TrackerPeer::TRACKER_ID,$tracker_idArr,Criteria::IN);
	$objs = TrackerPeer::doSelect($crit);
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$ret[] = TrackerToSOAP($obj);
		}
	}
	return $ret;
}

$server->register(
	'getTracker',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
	), 
	array('getTracker' => 'tns:Tracker'),
	$uri, 
	$uri.'#getTracker',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single Tracker
 * @param	$tracker_id	Retrieve the Tracker identified by this tracker_id
 */
function getTracker($sessionHash, $tracker_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerPeer::retrieveByPK($tracker_id);
	if (is_null($obj)) {
		return null;
	}
	if (!$obj->userCanRead()) {
		return new soap_fault('', 'getTracker','Permission Denied - Cannot View This Record: Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return TrackerToSOAP($obj);
}


$server->register(
	'getTrackerFull',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
	), 
	array('getTrackerFull' => 'tns:TrackerFull'),
	$uri, 
	$uri.'#getTrackerFull',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single Tracker
 * @param	$tracker_id	Retrieve the Tracker identified by this tracker_id
 */
function getTrackerFull($sessionHash, $tracker_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerPeer::retrieveByPK($tracker_id);
	if (is_null($obj)) {
		return null;
	}
	if (!$obj->userCanRead()) {
		return new soap_fault('', 'getTracker','Permission Denied - Cannot View This Record: Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return TrackerToSOAP($obj, true);
}


$server->register(
	'deleteTracker',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
	), 
	array('deleteTracker' => 'xsd:int'),
	$uri, 
	$uri.'#deleteTracker',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single Tracker
 * @param	$tracker_id	Delete the Tracker identified by this tracker_id
 */
function deleteTracker($sessionHash, $tracker_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerPeer::retrieveByPK($tracker_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteTracker','Permission Denied - Cannot Delete This Record: Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteTracker','Permission Denied - Cannot Delete This Record: ' . $e->getMessage(),$e->getMessage());
	}
	return 1;
}


$server->register(
	'updateTracker',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
		'project_id' => 'xsd:int',
		'tracker_name' => 'xsd:string',
		'description' => 'xsd:string',
		'is_public' => 'xsd:boolean',
		'restrict_browse' => 'xsd:boolean',
		'email_all_updates' => 'xsd:boolean',
		'email_address' => 'xsd:string',
		'due_period' => 'xsd:int',
		'submit_instructions' => 'xsd:string',
		'browse_instructions' => 'xsd:string',
		'show_dependencies_tab' => 'xsd:boolean',
		'show_duplicates_tab' => 'xsd:boolean',
		'show_associations_tab' => 'xsd:boolean',
		'show_time_tracking_tab' => 'xsd:boolean',
		'show_tags_tab' => 'xsd:boolean',
	), 
	array('updateTracker' => 'xsd:int'),
	$uri, 
	$uri.'#updateTracker',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update Tracker
 * @param	$tracker_id	 Tracker to update
 * @param	$project_id
 * @param	$tracker_name
 * @param	$description
 * @param	$is_public
 * @param	$restrict_browse
 * @param	$email_all_updates
 * @param	$email_address
 * @param	$due_period
 * @param	$submit_instructions
 * @param	$browse_instructions
 */
function updateTracker($sessionHash, $tracker_id, $project_id, $tracker_name, $description, $is_public, $restrict_browse, $email_all_updates, $email_address, $due_period, $submit_instructions, $browse_instructions, $show_dependencies_tab, $show_duplicates_tab, $show_associations_tab, $show_time_tracking_tab, $show_tags_tab) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$objAux = ProjectPeer::retrieveByPK($project_id);
	if (is_null($objAux) || !$objAux->getPermission()->isAdmin()) {
		return new soap_fault('', 'updateTracker', 'Permission Denied - Cannot add This Record: Permission Denied', 'Permission Denied');
	}

	$obj = TrackerPeer::retrieveByPK($tracker_id);
	if (is_null($obj)) {
		return new soap_fault('', 'updateTracker', 'Couldn\'t update Tracker: invalid tracker_id '.$tracker_id);
	}
	
	if (!is_null($project_id)) {
		$obj->setProjectId( Validator::sanitizeInt($project_id) );
	}
	if (!is_null($tracker_name)) {
		$obj->setTrackerName( Validator::sanitizeString($tracker_name) );
	}
	if (!is_null($description)) {
		$obj->setDescription( Validator::sanitizeString($description) );
	}
	if (!is_null($is_public)) {
		$obj->setIsPublic( Validator::sanitizeBool($is_public) );
	}
	if (!is_null($restrict_browse)) {
		$obj->setRestrictBrowse( Validator::sanitizeBool($restrict_browse) );
	}
	if (!is_null($email_all_updates)) {
		$obj->setEmailAllUpdates( Validator::sanitizeBool($email_all_updates) );
	}
	if (!is_null($email_address)) {
		$obj->setEmailAddress( Validator::sanitizeString($email_address) );
	}
	if (!is_null($due_period)) {
		$obj->setDuePeriod( Validator::sanitizeInt($due_period) );
	}
	if (!is_null($submit_instructions)) {
		$obj->setSubmitInstructions( Validator::sanitizeString($submit_instructions) );
	}
	if (!is_null($browse_instructions)) {
		$obj->setBrowseInstructions( Validator::sanitizeString($browse_instructions) );
	}
	if (!is_null($show_dependencies_tab)) {
		$obj->setShowDependenciesTab(Validator::sanitizeBool($show_dependencies_tab));
	}
	if (!is_null($show_duplicates_tab)) {
		$obj->setShowDuplicatesTab(Validator::sanitizeBool($show_duplicates_tab));
	}
	if (!is_null($show_associations_tab)) {
		$obj->setShowAssociationsTab(Validator::sanitizeBool($show_associations_tab));
	}
	if (!is_null($show_time_tracking_tab)) {
		$obj->setShowTimeTrackingTab(Validator::sanitizeBool($show_time_tracking_tab));
	}
	if (!is_null($show_tags_tab)) {
		$obj->setShowTagsTab(Validator::sanitizeBool($show_tags_tab));
	}
	
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateTracker','Permission Denied - Cannot update This Record: ' . $e->getMessage(),$e->getMessage());
	}
	return 1;
}



$server->register(
	'addTracker',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int',
		'tracker_name' => 'xsd:string',
		'description' => 'xsd:string',
		'is_public' => 'xsd:boolean',
		'restrict_browse' => 'xsd:boolean',
		'email_all_updates' => 'xsd:boolean',
		'email_address' => 'xsd:string',
		'due_period' => 'xsd:int',
		'submit_instructions' => 'xsd:string',
		'browse_instructions' => 'xsd:string',
		'datatype' => 'xsd:int',
		'show_dependencies_tab' => 'xsd:boolean',
		'show_duplicates_tab' => 'xsd:boolean',
		'show_associations_tab' => 'xsd:boolean',
		'show_time_tracking_tab' => 'xsd:boolean',
		'show_tags_tab' => 'xsd:boolean',
	),
	array('addTracker' => 'xsd:int'),
	$uri, 
	$uri.'#addTracker',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a Tracker
 * @param	$project_id
 * @param	$tracker_name
 * @param	$description
 * @param	$is_public
 * @param	$restrict_browse
 * @param	$email_all_updates
 * @param	$email_address
 * @param	$due_period
 * @param	$submit_instructions
 * @param	$browse_instructions
 * @param	$datatype
 */
function addTracker($sessionHash, $project_id, $tracker_name, $description, $is_public, $restrict_browse, $email_all_updates, $email_address, $due_period, $submit_instructions, $browse_instructions, $datatype, $show_dependencies_tab, $show_duplicates_tab, $show_associations_tab, $show_time_tracking_tab, $show_tags_tab) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$objAux = ProjectPeer::retrieveByPK($project_id);
	if (is_null($objAux) || !$objAux->getPermission()->isAdmin()) {
		return new soap_fault('', 'addTracker', 'Permission Denied - Cannot add This Record: Permission Denied', 'Permission Denied');
	}

	$obj = new Tracker();
	
	$obj->setProjectId( Validator::sanitizeInt($project_id) );
	$obj->setTrackerName( Validator::sanitizeString($tracker_name) );
	$obj->setDescription( Validator::sanitizeString($description) );
	$obj->setIsPublic( Validator::sanitizeBool($is_public) );
	$obj->setRestrictBrowse( Validator::sanitizeBool($restrict_browse) );
	$obj->setEmailAllUpdates( Validator::sanitizeBool($email_all_updates) );
	$obj->setEmailAddress( Validator::sanitizeString($email_address) );
	$obj->setDuePeriod( Validator::sanitizeInt($due_period) );
	$obj->setSubmitInstructions( Validator::sanitizeString($submit_instructions) );
	$obj->setBrowseInstructions( Validator::sanitizeString($browse_instructions) );
	$obj->setDatatype( Validator::sanitizeInt($datatype) );
	$obj->setShowDependenciesTab(Validator::sanitizeBool($show_dependencies_tab));
	$obj->setShowDuplicatesTab(Validator::sanitizeBool($show_duplicates_tab));
	$obj->setShowAssociationsTab(Validator::sanitizeBool($show_associations_tab));
	$obj->setShowTimeTrackingTab(Validator::sanitizeBool($show_time_tracking_tab));
	$obj->setShowTagsTab(Validator::sanitizeBool($show_tags_tab));
	try {
		$obj->save();
		
		//Creates the default fields
		$obj->buildDefaultFields();

		//Adds the standard queries
    require_once('plugins/tracker/lib/TrackerQueryPeer.php');
    TrackerQueryPeer::addStandardQueries($obj->getTrackerId());
		
		return $obj->getTrackerId();
	} catch (Exception $e) {
		return new soap_fault('', 'addTracker','Permission Denied - Cannot add This Record: ' . $e->getMessage(),$e->getMessage());
	}
}



$server->register(
	'getTrackerTechnicians',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
	), 
	array('getTrackerTechnicians' => 'tns:ArrayOfUser'),
	$uri, 
	$uri.'#getTrackerTechnicians',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single Tracker
 * @param	$tracker_id	Retrieve the Tracker identified by this tracker_id
 */
function getTrackerTechnicians($sessionHash, $tracker_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerPeer::retrieveByPK($tracker_id);
	if (is_null($obj)) {
		return null;
	}
	if (!$obj->userCanRead()) {
		return new soap_fault('', 'getTracker','Permission Denied - Cannot View This Record: Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	$ret = array();
	foreach ($obj->getTechnicians() as $TrackerTechnician) {
		$ret[] = UserToSOAP($TrackerTechnician);
	}
	return $ret;
}

function checkTrackerPermission($tracker_id, $type='') {
	$obj = TrackerPeer::retrieveByPK($tracker_id);
	if (is_null($obj)) {
		return false;
	} else if ($type=='delete') {
		return $obj->userCanDelete();
	} else if ($type=='create') {
		return $obj->userCanCreate();
	} else if ($type=='update') {
		return $obj->userCanUpdate();
	} else {
		return $obj->userCanRead();
	}
}

function TrackerToSOAP(&$obj, $full=false) {
	$ret = array();
	$ret['tracker_id'] = $obj->getTrackerId();
	$ret['project_id'] = $obj->getProjectId();
	$ret['tracker_name'] = $obj->getTrackerName();
	$ret['description'] = $obj->getDescription();
	$ret['is_public'] = $obj->getIsPublic();
	$ret['restrict_browse'] = $obj->getRestrictBrowse();
	$ret['email_all_updates'] = $obj->getEmailAllUpdates();
	$ret['email_address'] = $obj->getEmailAddress();
	$ret['due_period'] = $obj->getDuePeriod();
	$ret['submit_instructions'] = $obj->getSubmitInstructions();
	$ret['browse_instructions'] = $obj->getBrowseInstructions();
	$ret['datatype'] = $obj->getDatatype();
	$ret['item_total'] = $obj->getItemTotal();
	$ret['open_count'] = $obj->getOpenCount();
	$ret['show_dependencies_tab'] = $obj->getShowDependenciesTab();
	$ret['show_duplicates_tab'] = $obj->getShowDuplicatesTab();
	$ret['show_associations_tab'] = $obj->getShowAssociationsTab();
	$ret['show_time_tracking_tab'] = $obj->getShowTimeTrackingTab();
	$ret['show_tags_tab'] = $obj->getShowTagsTab();

	if ($full) {
		$ret['extra_fields'] = array();
		$ret['extra_field_elements'] = array();
		foreach ($obj->getTrackerExtraFields() as $TrackerExtraField) {
			$ret['extra_fields'][] = TrackerExtraFieldToSOAP($TrackerExtraField);
			foreach ($TrackerExtraField->getTrackerExtraFieldElements() as $TrackerExtraFieldElement) {
				$ret['extra_field_elements'][] = TrackerExtraFieldElementToSOAP($TrackerExtraFieldElement);
			}
		}
		foreach ($obj->getTechnicians() as $TrackerTechnician) {
			$ret['techinicians'][] = UserToSOAP($TrackerTechnician);
		}
	}
	return $ret;
}
?>
