<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling TrackerTimeCode

require_once('plugins/tracker/lib/TrackerTimeCode.php');
require_once('plugins/tracker/lib/TrackerTimeCodePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'TrackerTimeCode',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'tracker_time_code_id' => array('name'=>'tracker_time_code_id', 'type' => 'xsd:int'),
		'time_name' => array('name'=>'time_name', 'type' => 'xsd:string'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfTrackerTimeCode',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:TrackerTimeCode[]')),
	'tns:TrackerTimeCode'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getTrackerTimeCodes',
	array(
		'sessionHash'	=> 'xsd:string',
	), 
	array('getTrackerTimeCodes' => 'tns:ArrayOfTrackerTimeCode'),
	$uri, 
	$uri.'#getTrackerTimeCodes',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of TrackerTimeCode
 * @param	$tracker_time_code_id	Filter by tracker_time_code_id
 * @param	$time_name	Filter by time_name
 */
function getTrackerTimeCodes($sessionHash) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$c = new Criteria();
	$objs = TrackerTimeCodePeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = TrackerTimeCodeToSOAP($obj);
	}
	
	return $res;
	
}

$server->register(
	'getTrackerTimeCodeArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_time_code_id' => 'tns:ArrayOfInt',
	), 
	array('getTrackerTimeCode' => 'tns:ArrayOfTrackerTimeCode'),
	$uri, 
	$uri.'#getTrackerTimeCode',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of TrackerTimeCode
 * @param	$tracker_time_code_id	Retrieve the TrackerTimeCode array identified by the array of tracker_time_code_id
 */
function getTrackerTimeCodeArray($sessionHash, $tracker_time_code_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$ret = array();

	$crit=new Criteria();
        $crit->add(TrackerTimeCodePeer::TRACKER_TIME_CODE_ID,$tracker_time_code_idArr,Criteria::IN);
	$objs = TrackerTimeCodePeer::doSelect($crit);
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$ret[] = TrackerTimeCodeToSOAP($obj);
		}
	}
	return $ret;
}

$server->register(
	'getTrackerTimeCode',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_time_code_id' => 'xsd:int',
	), 
	array('getTrackerTimeCode' => 'tns:TrackerTimeCode'),
	$uri, 
	$uri.'#getTrackerTimeCode',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single TrackerTimeCode
 * @param	$tracker_time_code_id	Retrieve the TrackerTimeCode identified by this tracker_time_code_id
 */
function getTrackerTimeCode($sessionHash, $tracker_time_code_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerTimeCodePeer::retrieveByPK($tracker_time_code_id);
	if (is_null($obj)) {
		return null;
	}
	/*if (!$obj->userCanRead()) {
		return new soap_fault('', 'getTrackerTimeCode','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}*/
	return TrackerTimeCodeToSOAP($obj);
}


$server->register(
	'deleteTrackerTimeCode',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_time_code_id' => 'xsd:int',
	), 
	array('deleteTrackerTimeCode' => 'xsd:int'),
	$uri, 
	$uri.'#deleteTrackerTimeCode',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single TrackerTimeCode
 * @param	$tracker_time_code_id	Delete the TrackerTimeCode identified by this tracker_time_code_id
 */
function deleteTrackerTimeCode($sessionHash, $tracker_time_code_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerTimeCodePeer::retrieveByPK($tracker_time_code_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteTrackerTimeCode','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteTrackerTimeCode','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}


$server->register(
	'updateTrackerTimeCode',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_time_code_id' => 'xsd:int',
		'time_name' => 'xsd:string',
	), 
	array('updateTrackerTimeCode' => 'xsd:int'),
	$uri, 
	$uri.'#updateTrackerTimeCode',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update TrackerTimeCode
 * @param	$tracker_time_code_id	 TrackerTimeCode to update
 * @param	$time_name
 */
function updateTrackerTimeCode($sessionHash, $tracker_time_code_id, $time_name) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerTimeCodePeer::retrieveByPK($tracker_time_code_id);
	if (is_null($obj)) {
		return new soap_fault('', 'updateTrackerTimeCode','Couldn\'t update TrackerTimeCode: invalid tracker_time_code_id '.$tracker_time_code_id);
	}
	
	if (!is_null($time_name)) {
		$obj->setTimeName( Validator::sanitizeString($time_name) );
	}
	
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateTrackerTimeCode','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}



$server->register(
	'addTrackerTimeCode',
	array(
		'sessionHash'	=> 'xsd:string',
		'time_name' => 'xsd:string',
	), 
	array('addTrackerTimeCode' => 'xsd:int'),
	$uri, 
	$uri.'#addTrackerTimeCode',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a TrackerTimeCode
 * @param	$time_name
 */
function addTrackerTimeCode($sessionHash, $time_name) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = new TrackerTimeCode();
	$obj->setTimeName( Validator::sanitizeString($time_name) );
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'addTrackerTimeCode','Permission Denied - Cannot add This Record',$e->getMessage());
	}
	return 1;
}

function TrackerTimeCodeToSOAP(&$obj) {
	$ret = array();
	$ret['tracker_time_code_id'] = $obj->getTrackerTimeCodeId();
	$ret['time_name'] = $obj->getTimeName();
	return $ret;
}
?>
