// Debugging only!  Creates a lot of clutter in the console.
var log = false;

var xhr = null;
var tabChanging = false;
var buttons_block = false;
var spinner = null;

$(document).ready(function() {
    var tabs = $('.tab-container:first').tabs();
    tabs.tabs({
        cache: true,
        heightStyle: "auto",
        activate: function(event, ui) {
            var sendie = $(ui.newPanel).find('.sendie').first();
            if (sendie !== null) {
                adjustFooter(ui.newPanel);
                $(sendie).focus();
            }
        },
        beforeActivate: function(event, ui) {
            $('a', ui.newTab).removeClass('mentioned');
            $('a', ui.newTab).removeClass('new-messages');
        }
    });


    initTemplates();

    initEmoticonList();

    bindEvents();
    setTimeout(updateChats, 1000);  // first poll

    spinner_opts = {
        lines: 16, // The number of lines to draw
        length: 10, // The length of each line
        width: 5, // The line thickness
        radius: 20, // The radius of the inner circle
        color: '#000', // #rgb or #rrggbb
        speed: 1, // Rounds per second
        trail: 79, // Afterglow percentage
        shadow: false, // Whether to render a shadow
        hwaccel: false, // Whether to use hardware acceleration
        className: 'spinner', // The CSS class to assign to the spinner
        zIndex: 2e9, // The z-index (defaults to 2000000000)
        top: 220, // Top position relative to parent in px
        left: 380 // Left position relative to parent in px
    };

    spinner_target = document.getElementsByTagName("body")[0];
    spinner = new Spinner(spinner_opts).spin(spinner_target);
});

function initTemplates() {
    $( "#tabTemplate" ).template( "tab" );
    $( "#panelTemplate" ).template( "tab.panel" );
    $( "#userListItemTemplate" ).template( "user" );
    $( "#chatMessageTemplate" ).template( "msg" );
    $( "#chatMessageDeletedTemplate" ).template( "msg.deleted" );
    $( "#chatDateDelimiterTemplate" ).template( "dateline" );
    $( "#chatLinkTemplate" ).template( "link" );
    $( "#chatThreadTemplate" ).template( "thread" );
    $( "#chatScrollbackTemplate" ).template( "scrollback" );
    $( "#chatNoMoreScrollbackTemplate" ).template( "scrollback.none" );
    $( "#chatActionSoundTemplate" ).template( "action.sound" );
    $( "#chatPlaySoundTemplate" ).template( "play.sound" );
    $( "#chatActionEmoticonTemplate" ).template( "emoticon.button" );
}

function toggleBlockButtons(setting) {
    if (setting) {
        buttons_block = true;
        jQuery('.tools-container').fadeTo(500, 0.2);
    } else {
        buttons_block = false;
        jQuery('.tools-container').fadeTo(500, 1);
    }
}

function initEmoticonList() {
    if (smilies) {
        var container = $('#emoticon-list ul li');
        for (var pattern in smilies) {
            var data = {
                text: pattern,
                title: smilies[pattern][1],
                emoticon: emotify(pattern)
            };
            $.tmpl('emoticon.button', data).appendTo(container);
        }
    }
}

function bindEvents() {
    $( document ).on( "click", "span.ui-icon-close",
        function() {
            leaveChat(this);
        });

    $( document ).on( "click", '.chat-send',
        function() {
            submitChat($(this).parents('.chatpanel'));
        });

    $( document ).on( 'keydown', '.sendie',
        function(event) {
            if (event.which == 13) {
                if (!event.shiftKey && !event.CtrlKey) {
                    event.preventDefault();
                    submitChat($(this).parents('.chatpanel'));
                }
            }
        });

    $( document ).on( 'click', '.play-sound',
        function(event) {
            showSoundsPanel(this);
        });


    $( document ).on( 'click', '.sound-toggle',
        function(event) {
            if (!buttons_block) {
                toggleBlockButtons(true);
                toggleSoundSetting(this);
            }
        });

    $( document ).on( 'click', '.activity-toggle',
        function(event) {
            if (!buttons_block) {
                buttons_block = true;
                toggleBlockButtons(true);
                toggleActivitySetting(this);
            }
        });

    $( document ).on( 'click', 'a[data-sound-path]',
        function(event) {
            sendPlaySoundItem(this);
        });

    $( document ).on( 'click', 'a[data-emoticon]',
        function(event) {
            var panel = $('.tab-container > .ui-tabs-panel[aria-hidden=false]')[0];
            if (panel) addEmoticon(this, panel);
        });

    $(document).on('mouseenter', '.hot-hover',
        function() {
            $(this).addClass('hover');
        });

    $(document).on('mouseleave', '.hot-hover',
        function() {
            $(this).removeClass('hover');
        });

    $(document).on('mouseenter', '.line.message[@data-message-id] .message-header',
        function() {
            $('.delete-button img', this).fadeIn();
        });

    $(document).on('mouseleave', '.line.message[@data-message-id] .message-header',
        function() {
            $('.delete-button img', this).fadeOut();
        });

    $(document).on('click', '.delete-button', function(e) {
        var msg_id = $(this).attr('data-message-id');
        deleteMessage(msg_id);
    });

    $(document).on('click', '.discuss', function(e) {
        //Hack: passing this bogus function prevents the discuss
        //method from calling the openChatWindow function, which
        //is * not * needed when in chat context
        function _onSuccess() {
        }

        discuss(e.target, _onSuccess);
        return false;
    });

    $( document ).on( 'click', '.help-toggle',
        function(event) {
            showHelp();
        });

    $(window).resize(
        function(event) {
            adjustAllFooters();
        });

    $( document ).on( 'click', '.upload-toggle',
        function(event) {
            showUpload();
        });


    $(document ).on( 'click', '.open-multiline', function(event){
            event.preventDefault();
            var message_id = $(this).closest('.multiline').attr('data-message-id');
            openMultilinePaste(message_id);
        });

}

function tabId(thread_id) {
    var id = 'chat-tab-'+thread_id.toString();
    return id;
}

function selectorForTab(thread_id) {
    var tabSelector = '#'+tabId(thread_id);
    return tabSelector;
}

function getTab(thread_id) {
    return $(selectorForTab(thread_id));
}

function getTabLink(thread_id) {
    var href = window.location.href + selectorForTab(thread_id);
    var ret = $('a[href="'+href+'"]')[0];
    if (ret == undefined || ret == null) {
        ret = $('a[href="'+selectorForTab(thread_id)+'"]')[0];
    }
    return ret;
}


function checkAddTab(thread_id, url, title, title_full, callback) {
    var existingTab = getTab(thread_id);
    if ($(existingTab).length == 1) return false;

    var tabs = $('.tab-container').tabs();

    var newTabPanelId = tabId(thread_id);
    var newTabSelector = selectorForTab(thread_id);
    if (log) console.log("Adding tab panel id for thread " + thread_id + " from " + url);

    var newTab = $.tmpl( 'tab', { href : newTabSelector, title : title, title_full : title_full });
    $('.tab-container > ul.ui-tabs-nav').append(newTab);

    var newTabIndex = tabs.find('.ui-tabs-nav li').length - 1;

    var newPanel = $.tmpl( 'tab.panel', { id : newTabPanelId });
    tabs.append(newPanel);

    $(tabs).tabs( 'refresh' );

    if (log) console.log("New tab panel id for thread " + thread_id + " is " + newTabPanelId);

    if (log) console.log($(newTabSelector));

    $.ajax({
        url: url,
        async: false,
        success: function(data) {
            $(newPanel).html(data);
            $('.userlist-container', $(newPanel)).tabs();
            $('.auto-complete', $(newPanel)).each(function(index, elem) {
                addAutoComplete(elem);
            });
            adjustFooter($(newPanel));
            if (callback != undefined && typeof(callback) == 'function') {
                callback($(newPanel));
            }
            $( tabs ).tabs( "option", "active", newTabIndex );
            getRoleSettings(thread_id);
        }
    });

    return true;
}

function checkAddScrollback(elem) {
    var existing = $(elem).find('.scrollback');
    if (existing == null || $(existing).length == 0) {
        var before = $(elem).find('.line')[0];
        if (before) {
            $.tmpl( "scrollback", {} ).insertBefore($(before)).fadeIn();
        }
    }
}

function getLastMessageId(elem) {
    var lastMsgId = getLastIdForType(elem, 'message');
    var lastActivityId = getLastIdForType(elem, 'project');

    var ret = lastMsgId;
    if (lastActivityId > 0) {
        ret += ","+lastActivityId;
    }
    return ret;
}

function getLastIdForType(elem, type) {
    var last = $('.line.'+type, elem).last();
    while (last !== null) {
        var ret = $(last).attr('data-message-id');
        if (ret === undefined || ret === null) return 0;
        if (ret > 0) return ret;
        last = $(last).prev();
    }
    return 0;
}

function getThreadParams() {
    var params = "";
    $(document).find('.chatpanel').each( function(index, elem) {
        var thread_id = $('.threadId', elem).val();
        var lastMsgId = getLastMessageId(elem);
        if (thread_id != "") {
            params = params + "&" + encodeURIComponent(thread_id) + "=" + encodeURIComponent(lastMsgId);
        }
    });
    return params;
}

function deleteMessage(msg_id) {

    var url = '/gf/chat/?action=ChatMessageDelete&message_id='+msg_id;

    if (log) console.log("deleteMessage, making request for message "+msg_id);
    xhr = $.ajax({
        async: true,
        type: "GET",
        url: url,
        dataType: "json",
        cache: false,
        timeout: 2500,
        success: function() {
            if (log) console.log("deleteMessage, success handler");
        },
        statusCode: {
            301: function() { showDeleteError("The content is not available."); },
            302: function() { showDeleteError("The content has been moved."); },
            401: function() { showDeleteError("Your session has expired."); },
            403: function() { showDeleteError("You are not allowed to delete the message."); },
            404: function() { showDeleteError("The message could not be found."); },
            500: function() { showDeleteError("An error occurred."); }
        },
        error: function(jqXHR, textStatus, errorThrown) {
            var msg = "Could not delete the requested message";
            if (textStatus == 'error') {
                msg += '. An error occurred';
            }
            if (textStatus == 'timeout') {
                msg += '. The request timed out';
            }
            if (textStatus == 'abort') {
                msg += '. The request was cancelled';
            }
            if (textStatus == 'parseerror') {
                msg += '. The response was not valid';
            }
            if ('' != errorThrown) {
                msg += ': ' + errorThrown;
            }
            if (log) console.log('deleteMessage from '+url+' error, message is '+msg);
        },
        complete: function() {
            if (log) console.log("deleteMessage, complete handler");
            if (spinner) spinner.stop();
            xhr = null;
        }
    });
}

function updateChats() {

    if (tabChanging) {
        if (log) console.log("updateChats, tabChanging is true, exit");
        return;
    }

    if (xhr) {
        if (log) console.log("updateChats, xhr is running, exit");
        return;
    }

    var url = '/gf/chat/?action=ChatMessage'+getThreadParams();

    if (log) console.log("updateChats, making poll request");
    xhr = $.ajax({
        async: true,
        type: "GET",
        url: url,
        dataType: "json",
        cache: false,
        timeout: 30000,
        success: function(data) {
            if (log) console.log("updateChats, success handler");
            
            processThreads(data);           // try...catch in the function
            checkDisappearedThreads(data);
            processDeletedMessages(data);   // try...catch in the function
            processWallMessages(data);      // try...catch in the function

        },
        statusCode: {
            301: function() { closeOnError("The content is not available."); },
            302: function() { closeOnError("The content has been moved."); },
            401: function() { closeOnError("Your session has expired."); },
            403: function() { closeOnError("You are not authorized for this Discussion."); },
            404: function() { closeOnError("The Discussion does not exist."); },
            500: function() { closeOnError("An error occurred."); }
        },
        error: function(jqXHR, textStatus, errorThrown) {
            var msg = "Could not retrieve chat data";
            if (textStatus == 'error') {
                msg += '. An error occurred';
            }
            if (textStatus == 'timeout') {
                msg += '. The request timed out';
            }
            if (textStatus == 'abort') {
                msg += '. The request was cancelled';
            }
            if (textStatus == 'parseerror') {
                msg += '. The response was not valid';
            }
            if ('' != errorThrown) {
                msg += ': ' + errorThrown;
            }
            if (log) console.log('updateChat from '+url+' error, message is '+msg);
        },
        complete: function() {
            if (log) console.log("updateChats, complete handler");
            if (spinner) spinner.stop();
            xhr = null;

            adjustAllFooters();

            // If someone else is closing a tab, they will call updateChats() when they're done.
            if (tabChanging == false) {
                if (log) console.log("updateChats, calling updateChats to continue poll");
                setTimeout(updateChats, 500);
            }
        }
    });
}

function showDeleteError(msg) {
    tabChanging = true;
    alert("Sorry, could not delete the message you requested: "+msg);
    tabChanging = false;
}

function showSubmitError(msg) {
    tabChanging = true;
    alert("Sorry, could not post your Discussion message: "+msg);
    tabChanging = false;
}

function closeOnError(msg) {
    tabChanging = true;
    alert("Sorry, can't show you the Discussion(s) you requested: "+msg+"\nClick OK to continue.  This window will close and you can try the discuss link again, if appropriate.");
    window.close();
}



function openMultilinePaste(id) {
    window.open("/gf/chat/popups/?action=ViewMultilineMessage&message_id="+id,"_blank","modal=yes,status=no,menubar=no,location=no,toolbar=no,scrollbars=yes,resizable=yes,top=500,left=500,width=800,height=500");
}



function renderMultilineMessage(message) {
    message = message.replace('<pre>', '');
    message = message.replace('</pre>', '');

    var rendered_message = '',
        arr_message = message.split("\n");

    if (arr_message.length > 1) {
        rendered_message = '<ol>'
        for(var i=0; i < arr_message.length; i++) {
            rendered_message += '<li>' +  arr_message[i] + '</li>';
        }
        rendered_message += '</ol>';
        return '<pre>' + rendered_message + '</pre>';
    } else {
        return '<pre>' + message + '</pre>';
	}
}

        
function processThreads(data) {
    try {
        if (data && data.threads) {
            for(var i = 0, len = data.threads.length; i < len; ++i) {
                var thread = data.threads[i];
                var thread_id = thread.thread_id;

                checkAddTab(thread.thread_id, thread.tab_url, thread.title, thread.title_full);
                var selector = "#chat-tab-"+thread_id.toString();
                var elem = $(selector)[0];

                if (elem) {
                    $('.temp-message', elem).fadeOut(function() {
                        $(this).remove();
                    });

                    updateChatUI(thread, elem);

                    updateUserList(thread.users, elem, thread_id);

                } else {
                    if (log) console.log("Did not find element matching selector "+selector);
                }
            }
        }
    } catch(err) {
        if (log) console.log(err);
    }
}


function processDeletedMessages(data) {
    try {
        // 2013-03-12 [#10161] Enable message deletion through chat
        if (data && data.deleted) {
            for (var i = 0; i < data.deleted.length; i++) {
                var deleted = data.deleted[i];
                $('.line[data-message-id='+deleted+']').each(function(index, elem) {
                    var newItem = $.tmpl( "msg.deleted" );
                    $(elem).fadeOut(function() {
                        $(newItem).insertAfter( elem );
                        $(newItem).fadeIn();
                    });
                });
            }
        }
    } catch(err) {
        if (log) console.log(err);
    }
}


function processWallMessages(data) {
    try {
        if (data && data.wall) {
            if (log) console.log("There are "+data.wall.length+" wall messages to post");
            var activeTabId = $('.tab-container ul.ui-tabs-nav > li.ui-tabs-active').first().attr('aria-controls');
            if (log) console.log("Active tab id is "+activeTabId);
            var activeElem = $("#"+activeTabId)[0];
            if (log) console.log(activeElem);
            if (activeElem) {
                var whoami = $(activeElem).find('.whoami').val();
                var chatArea = $(activeElem).find('.chat-area')[0];
                var linklist = $(activeElem).find('.linklist ul')[0];
                for (var i = 0; i < data.wall.length; i++) {
                    if (log) console.log("Adding wall message "+i+" to chat panel");
                    var newItem = addChatMessage(data.wall[i], whoami, activeElem);
                    if (newItem != null) {
                        newItem.appendTo($(chatArea));
                        updateLinkList(newItem, linklist);
                    }
                }
                scrollBottom(elem, false);
            }
        }    
    } catch(err) {
        if (log) console.log(err);
    }
}


function addChatMessage(message, whoami, elem) {
    var msgclass = message.message_type; // default value


    // If we've already rendered this message, then never mind
    if (msgclass == 'message' || msgclass == 'project') {
        var duplicate_selector = "div[data-message-id=" + message.message_id + "]";
        if ($(duplicate_selector).length > 0) {
            return null;
        }
    }

    //Check if it's a multiline message
    if (message.body.indexOf("<pre>") == 0) {
        msgclass += ' multiline';
        message.body = renderMultilineMessage(message.body);
    }

    // Use a time component to mention flagging and sounds,
    // so that historical messages don't trigger these UI features
    if (minutesAgo(message.date) < 1) {
        if (message.body.indexOf("@" + whoami) > -1) {
            mentioned = true;
            msgclass += ' mentioned-me';
        }

        if (message.body.indexOf("data-sound-path") > -1) {
            var url = $(message.body).attr('data-sound-path');
            if (url && url != '') {
                msgclass += ' user-action';
                playChatSound(url, elem);
            }
        }
    }

    checkDateDelimiter(elem, message.date);
    var dateString = getTimeString(new Date(message.date), true);

    var msg_data = {
        message_id: message.message_id,
        dateString: dateString,
        unix_name: message.user.unix_name,
        avatar: message.user.avatar,
        msg_type: msgclass,
        msg: emotify(message.body)
    };

    var newItem = $.tmpl("msg", msg_data);
    $('a', newItem).each(function(index, elem) {
        $(elem).attr('target', '_blank');
    });

    return newItem;
}

function updateChatUI(data, elem) {
    if (data == undefined || data == null) {
        if (log) console.log('updateChatUI got an empty data message, exit');
        return false;
    }

    var whoami = $(elem).find('.whoami').val();
    var mentioned = false;
    var chatArea = $(elem).find('.chat-area')[0];
    var linklist = $(elem).find('.linklist ul')[0];
    var threadlist = $(elem).find('.threadlist ul')[0];
    var previousMsgId = getLastMessageId(elem);
    if (data.messages !== null && data.messages.length > 0) {
        for (var i = 0; i < data.messages.length; i++) {
            var newItem = addChatMessage(data.messages[i], whoami, elem);

            if (newItem != null) {
                newItem.appendTo($(chatArea));
                updateLinkList(newItem, linklist);
                if (data.messages[0].body.indexOf("@" + whoami) > -1) {
                    mentioned = true;
                }
            }
        }

        var arr_aux = $(chatArea).children('.message');
        var obj_aux = arr_aux[arr_aux.length - 1];
        var obj_ellipsis = $(obj_aux).find('.chat-autolink-description')[0];
        $(obj_ellipsis).ellipsis(true, false, '(read more)', ' (less)');

        //Scroll to bottom for any type of new messages (including activity)
        checkAddScrollback(elem);
        var forceScroll = true;
        if (previousMsgId > 0) {
            forceScroll = false;
        }
        scrollBottom(elem, forceScroll);

        var panel = $(elem).closest('.ui-tabs-panel'); // actually, elem should already be the panel, but we'll make sure
        var selector = "a[href='#"+panel.attr('id')+"']";
        var tablink = $(document).find(selector);
        var isActiveTab = ($(tablink).parent().attr('aria-selected') === 'true');

        // If this is not the active tab, then we make some UI
        // tweaks to the tab header link to alert the user
        if (! isActiveTab) {
            // Was this user mentioned?  Then bold the tab header
            if (mentioned) {
                $(tablink).addClass('mentioned');
                playChatSound('/themes/gforge5/sounds/chat/mention.mp3', elem);
            } else {
                // Not mentioned, but new messages?  Then italicize the tab header
                $(tablink).addClass('new-messages');
            }

            // New messages in inactive tab, whether mentioned or not, we bounce the tab header
            $(tablink).effect("bounce", {
                times: 3
            }, 500);

        }
        // Mentions, active or not, always get the ding
        if (mentioned) {
            playChatSound('/themes/gforge5/sounds/chat/mention.mp3', elem);
        }
    }


    // Update thread list if needed
    if (data.related !== null && data.related.length > 0) {
        for (var i = 0; i < data.related.length; i++) {
            var t = data.related[i];
            var selector = 'a[data-thread-id='+t.thread_id.toString()+']';
            var test = $(selector, $(threadlist)).length;
            if (test == 0) {
                $.tmpl( "thread", t ).appendTo($(threadlist)).slideDown();
            }
        }
    }

    $(elem).find('.linkcount').text($(linklist).find('li').length);
    $(elem).find('.threadcount').text($(threadlist).find('li').length);
}


function scrollBack(link) {

    if (tabChanging) {
        return;
    }

    var line = $(link).parents('.line');

    var chatArea = $(link).parents('.chat-area');
    var chatPanel = $(link).parents('.chatpanel');
    var chatTab = $(chatPanel).parent();
    var linklist = $(chatPanel).find('.linklist ul')[0];

    var thread_id = $(chatPanel).find('.threadId').val();
    var firstMessage = $(chatPanel).find('.message[data-message-id]:first');
    if (firstMessage == null || firstMessage.length == 0) {
        // Nothing to pull
        alert("Hmmm, there doesn't appear to be anything to scroll back to here. Sorry for the confusion!");
        $(line).fadeOut(function() {
            $.tmpl( "scrollback.none", {} ).insertAfter($(line)).fadeIn();
            $(line).remove();
        });
        return;
    }
    var before = $(firstMessage).attr('data-message-id');

    var url = '/gf/chat/?action=ChatHistory&thread_id='+thread_id+'&before='+before;

    $(link).effect("pulsate", {
        times:5
    }, 1000);

    xhr = $.ajax({
        async: false,
        type: "GET",
        url: url,
        dataType: "json",
        cache: false,
        timeout: 5000,
        success: function(data) {
            $(line).fadeOut(function() {
                if (data.messages !== null && data.messages.length > 0) {
                    var first = $(line).next();
                    for (var i = 0; i < data.messages.length; i++) {
                        var message = data.messages[i];
                        var msgclass = message.message_type;

                        var dateString = getLongDateString(new Date(message.date)) + ' ' + getTimeString(new Date(message.date), true);

                        var msg_data = {
                            message_id	:	message.message_id,
                            dateString	:	dateString,
                            unix_name	:	message.user.unix_name,
                            avatar		:	message.user.avatar,
                            msg_type	:	msgclass,
                            msg			:	emotify(targetLinks(message.body))
                        };

                        newItem = $.tmpl( "msg", msg_data );
                        $(newItem).insertBefore( first );

                        //Activates ellipsis
                        var arr_aux = $(obj_aux).children('.message');
                        var obj_aux = arr_aux[0];
                        var obj_ellipsis = $(obj_aux).find('.chat-autolink-description')[0];
                        $(obj_ellipsis).ellipsis(true, false, ' (read more)', ' (less)');

                        updateLinkList(newItem, linklist);
                    }
                } else {
                    $.tmpl( "scrollback.none", {} ).insertAfter(line).fadeIn();
                }

                $(line).remove();
                $(chatTab).find('.linkcount').text($(linklist).find('li').length);
                checkAddScrollback($(chatArea));
            });

        },
        error: function(jqXHR, textStatus, errorThrown) {
            var msg = "Could not retrieve chat data";
            if (textStatus == 'error') {
                msg += '. An error occurred';
            }
            if (textStatus == 'timeout') {
                msg += '. The request timed out';
            }
            if (textStatus == 'abort') {
                msg += '. The request was cancelled';
            }
            if (textStatus == 'parseerror') {
                msg += '. The response was not valid';
            }
            if ('' != errorThrown) {
                msg += ': ' + errorThrown;
            }
            if (log) console.log('scrollBack from '+url+' error, message is '+msg);
        },
        complete: function() {
            if (link) $(link).stop();
        }
    });
}

/* UI effects and management */
function adjustAllFooters() {
    if ($('.tab-container > .ui-tabs-nav > li').length == 0) {
        window.close();
        return true;
    }

    $('.chatpanel').each(function(index, elem) {
        adjustFooter(elem);
    });
}

function adjustFooter(elem) {
    if (log) console.log("adjustFooter");
    if (log) console.log($(elem));
    var containerHeight = $('.ui-tabs').first().innerHeight();
    if (log) console.log('ContainerHeight is '+containerHeight);
    var container = $(elem).parent();
    if (log) console.log($(container));
    var footerHeight = 12;
    if ($('.send-message-area', container).is(':visible')) {
        var footerHeight = $('.send-message-area', container).height();
    }

    var tabHeight = $(container).find('.ui-tabs-nav').height() * 1.2;
    var newHeight = (containerHeight - footerHeight - tabHeight - 12);  // fudge factor for tab headings, borders, etc.

    if (log) console.log("adjustFooter, setting new height to "+newHeight);

    $('.chat-area', elem).outerHeight(newHeight);
    $('.right-container', elem).outerHeight(newHeight + footerHeight - 6);
}

function scrollBottom(elem, forceScroll) {
    if (forceScroll == undefined) forceScroll = false;
    var scrollArea = $(elem).find(".chat-area:first")[0];

    var message_count = ($(elem).find(".chat-area .message, .chat-area .project")).length;
    if (message_count > 10) {
        //Get 10th message from bottom up
        var tenth_message = ($(elem).find(".chat-area .message, .chat-area .project"))[message_count-11];
    }

    if (!forceScroll) {
        if (scrollArea.scrollHeight > 0) {
            if ( (message_count > 10 && ( $(tenth_message).position('chat-area').top - $(scrollArea).innerHeight() ) <= 0) || message_count <= 10) {
                forceScroll = true;
            }
        }
    }

    if (forceScroll) {
        var scrollHeight = $(scrollArea).prop("scrollHeight");
        if (scrollHeight <= 0) {
            return;
        }
        $(scrollArea).animate({
            scrollTop: scrollHeight
        }, 2000);
        // Check for user scroll event, and if so, stop the animation.
        $(scrollArea).bind("scroll mousedown DOMMouseScroll mousewheel keyup", function(e) {
            if (e.which > 0 || e.type == "mousedown" || e.type == "mousewheel") {
                $(scrollArea).stop().unbind("scroll mousedown DOMMouseScroll mousewheel keyup");
            }
        });
    }
}

function updateLinkList(newItem, linklist) {
    $('.link-summary p.title a, .link-summary p.url a', newItem).each( function(index, elem) {
        var title = $(elem).text().replace(/^\s+|\s+$/g, '');
        if ($(elem).prop('href') != "" && title != "") {
            var tmpl_data = {
                url : $(elem).prop('href'),
                title : $(elem).text()
            };

            var selector = 'a[href="'+tmpl_data.url+'"]';

            var test = $(selector, linklist).length;

            if (test == 0) {
                appendWithEffects($.tmpl( "link", tmpl_data ), $(linklist));
            }
        }
    });
}

function appendWithEffects(elem, parent) {
    $(elem).appendTo( $(parent) ).slideDown(
        function() {
            $(this).effect("highlight", {
                color: '#5073B0'
            }, 1000);
        }
    );
    return $(elem);
}
/* End UI effects and management */

/* Users and presence functions */
function updateUserList(users, elem, thread_id) {
    var userlist = $(elem).find('.userlist ul');

    if (users == undefined) {
        return;
    }

    $(userlist).find('li').not('.head').addClass('check-user');

    for (var i = 0; i < users.length; i++) {
        var user = users[i];
        user['thread_id'] = thread_id;
        var userclass = 'user-'+user.user_id.toString();
        if ($(userlist).find('.'+userclass).length == 0) {
            appendWithEffects($.tmpl( "user", user ), $(userlist));
            if ($('.whoami', elem).val() != user.unix_name) {
                playChatSound('/themes/gforge5/sounds/chat/userenter.mp3', elem);
            }
        } else {
            $(userlist).find('.'+userclass).removeClass('check-user');
        }
    }

    // Check for departed friends
    var playLeave = false;
    $(userlist).find('.check-user').each(function(index, userElem) {
        $(userElem).slideUp();
        $(userElem).remove();
        playLeave = true;
    });
    if (playLeave) {
        playChatSound('/themes/gforge5/sounds/chat/userleave.mp3', elem);
    }

    //Sorts user list
    $('.userlist-'+thread_id, elem).sortElements(function(a, b){
        return $(a).attr('data-bind-id') > $(b).attr('data-bind-id') ? 1 : -1;
    });

    //Adds online user count
    $(".usercount", elem).text( users.length );
}

function checkDisappearedThreads(data) {
    $(document).find('.threadId').each( function(index, elem) {
        var threadId = $(elem).val();
        var found = false;
        for(var i = 0, len = data.threads.length; i < len; ++i) {
            var thread = data.threads[i];
            if (thread.thread_id == threadId) {
                found = true;
            }
        }
        if (! found) {
            // Didn't find it, oh noes
            var tabs = $('.tab-container').tabs();

            var link = getTabLink(threadId);
            if (link) {
                var name = $(link).text();
                var msg = "Whoa!  It looks like " + name + " has gone away. The tab for this chat will close.  You can go back to the discussion link and try again.";
                alert(msg);

                var index = $( "li", tabs ).index( $( link ).parent() );
                $(tabs).tabs( "remove", index );
                if ($(tabs).tabs("length") < 1) {
                    window.close();
                } else {
                    adjustAllFooters();
                }
            }
        }
    });
}

function submitChat(elem, showTempMsg) {
    if (showTempMsg == undefined) showTempMsg = true;

    var baseUrl = $(elem).find('.baseUrl').val();
    var threadId = $(elem).find('.threadId').val();

    var body = $(elem).find('.sendie').val();
    body = body.replace(/\n/g, '<br>');
    if (body == '<br>' || body == '') return;

    var data = {
        body: body
    };
    $(elem).find('.sendie').val('');
    var tempElem = null;

    if (showTempMsg) {
        var msg_data = {
            dateString	:	getTimeString(new Date(), true),
            unix_name	:	$(elem).find('.whoami').val(),
            avatar		:	'/images/stock_avatars/default.png',
            msg_type	:	'message temp-message',
            msg			:	body
        };
        tempElem = $.tmpl( "msg", msg_data );
        $(tempElem).appendTo( $(elem).find('.chat-area') );
        scrollBottom(elem, false);
    }

    var url = baseUrl.concat(
        "?action=ChatMessage&thread_id=",
        threadId
    );

    $.ajax({
        type: "POST",
        url: url,
        data: data,
        dataType: "json",
        cache: false,
        statusCode: {
            301: function() { showSubmitError("The content is not available."); },
            302: function() { showSubmitError("The content has been moved."); },
            401: function() { showSubmitError("Your session has expired."); },
            403: function() {
                showSubmitError("The Discussion is moderated, and Live postings are not allowed.");
                $(tempElem).slideUp();
            },
            404: function() { showSubmitError("The message could not be found."); },
            500: function() { showSubmitError("An error occurred."); }
        }
    });
}

function cancelExistingRequest(requestor) {
    // Prevent long-poll from re-opening the same tab we're trying to close
    // Cancel an existing poll if it's waiting for response
    if(xhr && xhr.readystate != 4) {
        if (log) console.log(requestor+' killing active xhr');
        xhr.abort();
        xhr = null;
    }
    return true;
}

function leaveChat(link) {

    var tabs = $('.tab-container');
    var tab = $(link).parent();
    var panelId = $(tab).attr('aria-controls');
    var panel = $("#" + panelId);

    var threadId = $(panel).find('.threadId').val();
    if (threadId == null) {
        if (log) console.log($('.tab-container .ui-tabs-nav > li'));
        $(tab).remove();
        $(panel).remove();
        $(tabs).tabs('refresh');
        adjustAllFooters();
        return true;
    }

    var url = "?action=ChatLeave&thread_id=" + threadId.toString();

    tabChanging = true;
    cancelExistingRequest('leaveChat');

    $.ajax({
        async: false,
        type: "GET",
        url: url,
        dataType: "json",
        cache: false,
        complete: function() {

            if (log) console.log($('.tab-container .ui-tabs-nav > li'));
            $(tab).remove();
            $(panel).remove();
            $(tabs).tabs('refresh');
            adjustAllFooters();

            if (log) console.log("leaveChat complete handler");
            tabChanging = false;
            updateChats();
        }
    });
    return true;
}

/* Date-related functions */
function minutesAgo(dateString) {
    var diff = Math.abs(new Date() - new Date(dateString));
    var minutes = Math.floor((diff/1000)/60);
    return minutes;
}

function checkDateDelimiter(parentElem, d) {
    var lastDateElem = $(parentElem).find(".lastMessageDate");
    if (! (d instanceof Date)) {
        d = new Date(d);
    }
    var dateString = getDateDelimiterString(lastDateElem.val(), d.toUTCString());
    if (dateString !== null && dateString != "0:00 AM") {
        $.tmpl( "dateline", {
            dateString : dateString
        } ).appendTo( $(parentElem).find('.chat-area') );
    }
    $(lastDateElem).val(d.toUTCString());
}

function getDateDelimiterString(lastDateString, newDateString) {

    var newDate = new Date(newDateString);
    var newDateFormatted = getLongDateString(newDate);

    if (lastDateString == '') {
        // just starting to display, draw a line with the "day" format
        if (newDateString != '') {
            return newDateFormatted;
        }
    } else {
        var lastDate = new Date(lastDateString);
        var lastDateFormatted = getLongDateString(lastDate);

        if (lastDate.getFullYear() != newDate.getFullYear() ||
            lastDate.getMonth() != newDate.getMonth() ||
            lastDate.getDate() != newDate.getDate()) {
            // Last message date was not today - show a date delimiter
            if (newDate.getFullYear() >= 1970) {
                return newDateFormatted;
            }
        } else if (lastDate.getHours() != newDate.getHours()) {
            // Last message was earlier today - show an hourly delimiter
            var ret = getTimeString(newDate, false);
            return ret;
        }
    }
    return null;
}

function getLongDateString(d) {
    if ($.datepicker !== null) {
        return $.datepicker.formatDate("D, M d yy", d);
    } else {
        return d.toLocaleDateString();
    }
}

function getTimeString(d, includeMinutes) {
    // Last message was earlier today - show an hourly delimiter
    var hour = d.getHours();
    var mins = ("0" + d.getMinutes()).slice(-2);
    if (includeMinutes != true) {
        mins = "00";
    }
    var ap = " AM";
    if (hour > 11) {
        ap = " PM";
        if (hour > 12) {
            hour -= 12;
        }
    }
    return hour.toString() + ":" + mins + ap;
}
/* End date-related functions */

/* Emoticons */
function addEmoticon(link, elem) {

    if (link) {
        var text = $(link).attr('data-emoticon');
        if (elem) {
            var field = $(elem).find('.sendie')[0];
            if (field) $(field).insertAtCaretPos(" " + text + " ");
        }
    }
}
/* End Emoticons */

/* Sound-related functions */
function cancelChatSound() {
    $("#chatsound_").remove();
}
function playChatSound(url, elem) {
    cancelChatSound();
    var sound_toggle = $(elem).find('.sound-toggle')[0];
    try {
        var preferenceValue = $(sound_toggle).attr('data-sound-setting');
        if (preferenceValue == 'Y') {
            $.tmpl('play.sound', { url: url }).appendTo('body');
        }
    } catch(err) {
        if (log) console.log(err);
    }
}

function sendPlaySoundItem(link) {

    var title = $(link).attr('title');
    var url = $(link).attr('data-sound-path');

    var panel = $(link).parents('.chatpanel')[0];
    var content = $.tmpl( "action.sound", {
        title: title,
        url: url
    } ).html();

    $(panel).find('.sendie').val(content);

    submitChat(panel, false);
}

function toggleSoundSetting(el) {
    var threadId = $(el).attr('data-thread-id');
    var preferenceName = 'chat-' + threadId + '-sound';
    var preferenceValue = $(el).attr('data-sound-setting');
    if (preferenceValue == 'Y') {
        preferenceValue = 'N';
        cancelChatSound();
    } else {
        preferenceValue = 'Y';
    }

    var url = '/gf/ajax/?action=UserSetPreferenceAjax&preference_name=' + preferenceName + '&preference_value=' + preferenceValue;

    $.ajax({
        type: "GET",
        url: url,
        dataType: "html",
        cache: false,
        success: function(data) {
            $(el).attr('data-sound-setting', preferenceValue);
            var img = $('img', el);
            var src_image = $(img).attr('src');
            if (preferenceValue == 'N') {
                src_image = src_image.replace("sound_on.png", "sound_off.png");
            } else {
                src_image = src_image.replace("sound_off.png", "sound_on.png");
            }
            $(img).attr('src', src_image);
        },
        complete: function() {
            toggleBlockButtons(false);
        }
    });
}


/**
 * jQuery.fn.sortElements
 * --------------
 * @param Function comparator:
 *   Exactly the same behaviour as [1,2,3].sort(comparator)
 *
 * @param Function getSortable
 *   A function that should return the element that is
 *   to be sorted. The comparator will run on the
 *   current collection, but you may want the actual
 *   resulting sort to occur on a parent or another
 *   associated element.
 *
 *   E.g. $('td').sortElements(comparator, function(){
 *      return this.parentNode; 
 *   })
 *
 *   The <td>'s parent (<tr>) will be sorted instead
 *   of the <td> itself.
 */
jQuery.fn.sortElements = (function(){

    var sort = [].sort;

    return function(comparator, getSortable) {

        getSortable = getSortable || function(){
            return this;
        };

        var placements = this.map(function(){

            var sortElement = getSortable.call(this),
                parentNode = sortElement.parentNode,

            // Since the element itself will change position, we have
            // to have some way of storing its original position in
            // the DOM. The easiest way is to have a 'flag' node:
                nextSibling = parentNode.insertBefore(
                    document.createTextNode(''),
                    sortElement.nextSibling
                );

            return function() {

                if (parentNode === this) {
                    throw new Error(
                        "You can't sort elements if any one is a descendant of another."
                    );
                }

                // Insert before flag:
                parentNode.insertBefore(this, nextSibling);
                // Remove flag:
                parentNode.removeChild(nextSibling);

            };

        });

        return sort.call(this, comparator).each(function(i){
            placements[i].call(getSortable.call(this));
        });

    };

})();
/* End Sound-related functions */

/* Project activity functions */
function toggleActivitySetting(el) {
    var panel = $(el).parents('.chatpanel')[0];
    var threadId = $(panel).find('.threadId').val();
    var preferenceName = 'chat-' + threadId + '-proj';
    var preferenceValue = $(el).attr('data-activity-setting');
    if (preferenceValue == 'Y') {
        preferenceValue = 'N';
    } else {
        preferenceValue = 'Y';
    }

    tabChanging = true;
    cancelExistingRequest('Activity setting');

    var url = '/gf/ajax/?action=UserSetPreferenceAjax&preference_name=' + preferenceName + '&preference_value=' + preferenceValue;

    $.ajax({
        type: "GET",
        url: url,
        dataType: "html",
        cache: false,
        success: function(data) {
            $(el).attr('data-activity-setting', preferenceValue);
            var img = $('img', el);
            var src_image = $(img).attr('src');
            if (preferenceValue == 'N') {
                src_image = src_image.replace("project_activity_on.png", "project_activity_off.png");
            } else {
                src_image = src_image.replace("project_activity_off.png", "project_activity_on.png");
            }
            $(img).attr('src', src_image);
        },
        complete: function() {
            tabChanging = false;
            updateChats();
            toggleBlockButtons(false);
        }
    });
}
/* End project activity functions */



function getRoleSettings(thread_id) {
    var url = '/gf/ajax/?action=GetRoleSettings&ref_id=' + thread_id;

    $.ajax({
        url: url,
        async: false,
        success: function(data) {
            var role_settings = JSON.parse(data);

            var role_count = $('#roles-link-' + thread_id).find('.rolescount');
            $(role_count).html(role_settings.length);

            var rolelist = $('#roles-tab-'+thread_id).find('ul')[0];
            $(rolelist).empty();

            for(var i=0; i < role_settings.length; i++) {
                var role_setting = role_settings[i];

                var str_role = '<li class="hot-hover"><a href="' + role_setting.url + '" target="_blank"><span>' + role_setting.rolename + ': </span><span>' + role_setting.setting +'</span></a></li>';
                $(rolelist).append(str_role);
            }
        }
    });

    return true;
}

function updateContent(container, url) {
    $(container).fadeOut(function() {
        $.ajax({
            url: url,
            async: true,
            success: function(data) {
                $(container).html(data);
                $(container).fadeIn();
            },
            complete: function() {
                if (spinner) spinner.stop();
            }
        });
    });
}

function showHelp() {
    $( "#chat-tips" ).dialog({
        title: "Tips",
        resizable: false,
        height: 370,
        width: 260,
        modal: false,
        position: [525,40],
        open: function () {
            $('#chat-tips').scrollTop("0");
        }
    });


}

/**
 * Expects a piece of html as param, adds target='_blank' to all detected A
 * links and returns html.
 *
 * @param {string} txt An HTML chunk
 *
 * @return {string}    HTML with all A links targeted to _blank
 */
function targetLinks(txt) {
    var $node = $('<div>'+txt+'</div>');
    $('a', $node).each(function(index, elem) {
        $(elem).attr('target', '_blank');
    });
    return $node.html();
}

