/**
 * Provides a set or reusable dialogs
 * 
 */
$(function() {
	window.GFDialogs = {
		// Dialog DOM elements
		ui_dialog: null,
		dialog_content: null,
		dialog_buttons: null,
		// Spinner
		spinner: new Spinner({className:'spinner'}),
		spinner_container: null,

		// Initialize
		init: function() {
			// Create dialog DOM elements
			this.dialog_content = $('<span>').addClass('content');
			this.ui_dialog = $('<div>').attr('id', 'ui-dialog').append(
				$('<span>').addClass('ui-icon')
			).append(
				this.dialog_content
			);
			// Append to doc body
			$('body').append(this.ui_dialog);
		},

		//  Generic dialog to show info to the user
		showInfoDialog: function(params) {
			this.setDialogIcon({type: params.icon_type});
			this.spinner_container = $('<div class="spinner-container"></div>');
			this.dialog_content.html(params.text);
			if (params.spinner) {
				this.dialog_content.after(
					this.spinner_container.append(this.spinner.spin().el)
				);
			}
			this.ui_dialog.dialog({
				modal: true,
				resizable: false,
				draggable: false,
				buttons: {
					'OK': function() {
						$(this).dialog("close");
					}
				}
			});
			this.dialog_buttons = $('.ui-dialog-buttonpane');
			if (params.spinner) {
				this.dialog_buttons.hide();
			}
		},
		
		// Generic dialog to ask for user confirmation before proceeding to do something
		showConfirmDialog: function(params) {
			var buttons = {};
			this.setDialogIcon({type: 'info'});
			this.spinner_container = $('<div class="spinner-container"></div>');			
			this.dialog_content.html(params.confirmation_question);
			if (params.warning.length) {
				$(params.warning).each(function(i, v){
					GFDialogs.dialog_content.before(
						$('<div class="warning-container"></div>').text(v)
					);
				})
			}			
			buttons[params.cancel_button_label] = function() {
				$(this).dialog("close");
			};
			buttons[params.ok_button_label] = function() {
				GFDialogs.dialog_buttons.hide();
				GFDialogs.dialog_content.after(
					GFDialogs.spinner_container.append(GFDialogs.spinner.spin().el)
				);
				params.ok_handler();
			}
			GFDialogs.ui_dialog.dialog({
				modal: true,
				resizable: false,
				draggable: false,
				buttons: buttons,
				close: function() {
					$('.warning-container', GFDialogs.ui_dialog).remove();
				}
			});
			this.dialog_buttons = $('.ui-dialog-buttonpane');
		},

		// Set dialog icon
		setDialogIcon: function(params) {
			$('.ui-icon', this.ui_dialog).removeClass(function(i, css) {
				return (css.match(/ui-icon-\w+/g) || []).join(' ');
			}).addClass('ui-icon-' + params.type);
		},
		
		// On before ajax call starts
		ajax_start: function(text) {
			this.showInfoDialog({
				icon_type: 'info', 
				spinner: true,
				text: text				
			});
		},

		// On ajax call success
		ajax_success: function(response, custom_success_handler) {
			if (response.success) {
				custom_success_handler(response);
				GFDialogs.dialog_content.html(response.message);
			} else {
				GFDialogs.dialog_content.append('<p>Error: ' + response.error + '</p>');
			}
		},

		// On ajax call error
		ajax_error: function(jqXHR, textStatus, errorThrown) {
			GFDialogs.dialog_content.append('<p>Error: ' + textStatus + '</p>');
		},

		// On ajax call completed
		ajax_complete: function() {
			GFDialogs.spinner.stop();
			GFDialogs.spinner_container.remove();
			GFDialogs.dialog_buttons.slideDown();
		},

		/*
		 * Generic handler for issuing ajax calls, using the dialog to provide feedback to the user.
		 * params object(
		 *  start_text: text to show in the dialog while waiting for server side response.
		 *	action_url: server side action to handle the request.
		 *				The json response from the server must contain at least the following values:
		 *					'success': to indicate wether the attempt to perform the action succeded
		 *					'message': message to display in case the attempt succeds
		 *					'error': error message to show in case the attempt  fails
		 *	action_data: data to pass to the server side action
		 *	success_handler: function with some action to perform before the success message is shown
		 * )
		 */
		do_ajax_call: function(params) {
		   GFDialogs.ajax_start(params.start_text);
		   $.ajax({
			   url: params.action_url,
			   type: "POST",
			   data: params.action_data,
			   dataType: 'json',
			   success: function(response){GFDialogs.ajax_success(response, params.success_handler)},
			   error: GFDialogs.ajax_error,
			   complete: GFDialogs.ajax_complete
		   });
		   return false;
		},

	};
	GFDialogs.init();
});