/**
 * Trove mass update form client side library
 * 
 * @requires GFDialogs (www/js/gf-dialogs.js)
 * 
 */
$(function(){
        
        var clasesObject = $('.trove-enabled-toggle').attr("id");
        for (var objectId in clasesObject){
            var enableDisableText = $('#toggle-trove-' + objectId).text();
            if (enableDisableText === '[Enable]'){
                $('#toggle-trove-' + objectId).parent().parent().parent().css("color", "#a5a5a5");
            }
        }
        
	window.GFTroveMassUpdate = {
		// DOM elements
		access_level: $('.trove-mass-update-form select[name="access_level"]'),
		show_in_search: $('.trove-mass-update-form select[name="show_in_search"]'),
		move: $('.trove-mass-update-form select[name="move"]'),

		// Aux
		text: null,
		server_side_actions: null,

		// I18N Strings coming from the server
		setText: function(text) {
			this.text = text;
		},

		// Urls for actions referenced within this library
		setServerSideActions: function(urls) {
			this.server_side_actions = urls;
		},

		// Get parent's tree node checkbox
		getParent: function(checkbox) {
			return checkbox.parents('li ul li').eq(1).find('>div>input[type="checkbox"][name="trove_ids[]"]');
		},

		// Get descendant tree nodes checkboxes
		getDescendants: function(checkbox) {
			return checkbox.parent().siblings('ul').find('input[name="trove_ids[]"]');
		},

		// Is the checkbox in a leaf node ?
		isLeaf: function(checkbox) {
			return !checkbox.parent().siblings('ul').length;
		},

		// Is the checkbox in a node for a fixed category ?
		isFixed: function(checkbox) {
			return $('a.trove-enabled-toggle', checkbox.siblings('span')).length;
		},

		// Climb up the tree trying to find a checked ancestor checkbox
		isAncestorChecked: function(checkbox) {
			var parent = this.getParent(checkbox);
			if (parent.length > 0) {
				return parent.is(':checked') || this.isAncestorChecked(parent);
			} else {
				return false;
			}
		},

		// Handle enable / disable trove
		enableDisableHandler: function() {
			var id = $(this).attr('id'),
				trove_id = id.slice(id.lastIndexOf('-') + 1),
				start_text = $(this).text().replace(/[\[\]']+/g, '') + ' ' + $(this).data('categoryName'),
				action_url = GFTroveMassUpdate.server_side_actions.toggle_enabled,
				action_data = { id: trove_id },
				params = {
					start_text: start_text,
					action_url: action_url,
					action_data: action_data,
					success_handler: function(response) {
						$('#toggle-trove-' + response.id).text('[' + response.next_action + ']');
                                                if(response.next_action=='Enable'){
                                                    $('#toggle-trove-' + response.id).parent().parent().parent().css("color", "#a5a5a5");
                                                }else{
                                                    $('#toggle-trove-' + response.id).parent().parent().parent().css("color", "#666");
                                                }
					}
				};
			GFDialogs.do_ajax_call(params);
		},
		
		/**
		* Enable or disable move options depending on whether the associated checkbox is checked
		*/
		updateMoveSelectOption: function(checkbox) {
		   var move_select = $('select[name="move"]');
		   var move_option = $('option[value="' + checkbox.val() +'"]', move_select);
		   if (checkbox.is(':checked')) {
			   if (move_option.is(':selected')) {
				   move_select.val(-1);
			   }
			   move_option.attr('disabled', 'disabled');
		   } else {
			   if (!this.isAncestorChecked(checkbox)) {
				   move_option.removeAttr('disabled');
			   }
		   }
		   move_select.trigger("chosen:updated");
		},
		
		/**
		 * Applies updateMoveSelectOption to checkbox clicked and its descendants
		 */
		subtreeUpdateMoveSelectOptions: function(){
			var _self = $(this);
			GFTroveMassUpdate.updateMoveSelectOption(_self);
			GFTroveMassUpdate.getDescendants($(this)).each(function(){
				$(this).attr('checked', _self.is(':checked'));
				GFTroveMassUpdate.updateMoveSelectOption($(this));
			});
		},
		
		// Show dialog for confirming or cancelling mass update
		confirmMassUpdateSubmit: function() {
			var warnings = {
					wont_move: [],
					wont_merge: [],
					merge_delete: []
				},
				warning_messages = [],
				warning = '',
				merging,
				merge_delete_ids = [];
			// If we're moving / merging
			if ($('select[name="move"]').val() != -1) {
				// Are we merging?
				merging = $('input[name="merge"]').is(':checked');
				// For each selected category
				$('input[type="checkbox"][name="trove_ids[]"]:checked').each(function(){
					if (merging) {
						if (GFTroveMassUpdate.isFixed($(this)) || GFTroveMassUpdate.isLeaf($(this)) && !GFTroveMassUpdate.isAncestorChecked($(this))) {
							// Merge has no effect for fixed categories or leafs where no ancestor is checked
							warnings.wont_merge.push($(this));
						} else if (!GFTroveMassUpdate.isAncestorChecked($(this))) {
							// Warn that the category will be deleted as a consequence of the merge
							warnings.merge_delete.push($(this));
						}
					} else {
						if (GFTroveMassUpdate.isFixed($(this))) {
							// Move has no effect for fix categories
							warnings.wont_move.push($(this));
						}
					}
				});
				// if there is any category to warn about
				if (warnings.wont_move.length || warnings.wont_merge.length || warnings.merge_delete.length) {
					// Build the warning for no move
					if (warnings.wont_move.length) {
						warning = this.text.move_warning;
						$(warnings.wont_move).each(function(i,e) {
							warning += $(e).data('categoryName') + ', ';
						});
						warning_messages.push(warning);
					}
					// Build the warning for no merge
					if (warnings.wont_merge.length) {
						warning = this.text.merge_warning;
						$(warnings.wont_merge).each(function(i,e) {
							warning += $(e).data('categoryName') + ', ';
						});
						warning_messages.push(warning);
					}
					// Build the warning for deleted by merge
					if (warnings.merge_delete.length) {
						warning = this.text.merge_delete_warning;
						$(warnings.merge_delete).each(function(i,e) {
							merge_delete_ids.push($(e).val());
							warning += $(e).data('categoryName') + ', ';
						});
						warning_messages.push(warning);
						$.ajax({
							url: this.server_side_actions.get_delete_warnings,
							type: "POST",
							async: false,
							data: {ids: merge_delete_ids},
							dataType: 'json',
							success: function(response) {
								if (response.warnings) {
									$(response.warnings).each(function(i, msg) {
										warning_messages.push(msg);
									});
								}
							},
							error: function(jqXHR, textStatus, errorThrown) {
								console.log('Error getting link association delete warnings: ' + textStatus);
							}
						});
					}
				}
			}
			GFDialogs.showConfirmDialog({
				confirmation_question: GFTroveMassUpdate.text.mass_update_confirm,
				warning: warning_messages,
				cancel_button_label: GFTroveMassUpdate.text.cancel,
				ok_button_label: GFTroveMassUpdate.text.yes,
				ok_handler: function() {$('.trove-mass-update-form').submit();},
			});
		},
		
		// Client side validation for mass update action
		validMassUpdateForm: function() {
			var selected_checkboxes = $('input[type="checkbox"][name="trove_ids[]"]:checked'),
				error,
				valid = true;
			if (!selected_checkboxes.length) {
				error = this.text.no_items_selected;
				valid = false;
			}
			if (
					valid
					&& this.access_level.val() == -1
					&& this.show_in_search.val() == -1
					&& this.move.val() == -1
			) {
				error = this.text.no_changes_required;
				valid = false;
			}
			if (!valid) {
				GFDialogs.showInfoDialog({
					icon_type: 'alert',
					spinner: false,
					text: error,
				});
			}
			return valid;
		},
		
		// Client side validation for mass delete action
		validMassDeleteForm: function() {
			var selected_checkboxes = $('input[type="checkbox"][name="trove_ids[]"]:checked'),
				error,
				valid = true;
			if (!selected_checkboxes.length) {
				error = this.text.no_items_selected;
				valid = false;
			}
			if (!valid) {
				GFDialogs.showInfoDialog({
					icon_type: 'alert',
					spinner: false,
					text: error,
				});
			}
			return valid;
		},
		
		// Mass update form submit handler
		submitMassUpdateForm: function() {
			if (GFTroveMassUpdate.validMassUpdateForm()) {
				GFTroveMassUpdate.confirmMassUpdateSubmit();
			}
			return false;
		},
		
		// Mass delete button click handler
		submitMassDelete: function() {
			return GFTroveMassUpdate.validMassDeleteForm();
		},
		
		// Initialize
		init: function() {
			// Mktree for expanding / collapsing trove tree nodes
			if (!window.nodeOpenClass) window.nodeOpenClass = 'liOpen';
			expandTree('trove-map-tree');

			// Chosen select init
			this.access_level.attr('data-placeholder', this.text.no_change).chosen({disable_search_threshold: 10, width: "160px", allow_single_deselect: true});
			this.show_in_search.attr('data-placeholder', this.text.no_change).chosen({disable_search_threshold: 10, width: "150px", allow_single_deselect: true});
			this.move.attr('data-placeholder', this.text.no_change).chosen({allow_single_deselect: true});

			// Click handler for enable/disable trove category links
			$('.trove-enabled-toggle').click(this.enableDisableHandler);

			/**
			* Prevent moves where dest category is contained in sources categories
			* or where dest category is a descendant of any category in sources categories
			*/
			$('.trove-mass-update-form input[type="checkbox"]').click(this.subtreeUpdateMoveSelectOptions);

			// Update submit button handler
			$('#mass-update-submit').click(this.submitMassUpdateForm);

			// Delete submit button handler
			$('#mass-delete-submit').click(this.submitMassDelete);
		}
	};
});