<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('session/Session.php');
require_once('exceptions/NotLoggedInException.php');
require_once('exceptions/MissingDataException.php');
require_once('lib/core/Monitor.php');
require_once('Validator.php');

$session = Session::getSession();
if (!$session->isLoggedIn()) {
	throw new NotLoggedInException();
}

$lang = $session->getLanguage();



if (!isset($_GET['section']) || !isset($_GET['ref_id'])) {
	throw new MissingDataException();
}

$section = Validator::sanitizeString($_GET['section']);
$ref_id = Validator::sanitizeInt($_GET['ref_id']);
// By default, monitor the item
$notify = (!isset($_GET['notify']) || $_GET['notify'] == 1) ? true : false;
$user = $session->getUser();
$user_id = $user->getUserId();

$c = new Criteria();
$c->add(MonitorPeer::SECTION, $section);
$c->add(MonitorPeer::REF_ID, $ref_id);
$c->add(MonitorPeer::USER_ID, $user_id);
$mon = MonitorPeer::doSelectOne($c);

$feedback = '';
if ($notify && is_null($mon)) {			// new monitor
	$mon = new Monitor();
	$mon->setSection($section);
	$mon->setRefId($ref_id);
	$mon->setUserId($user_id);
	
	if ($section == 'site' && $ref_id == 0) { //Only for Site Notification
		$user->setSiteMonitoring(true);
		$feedback = $lang->getText('Common.SuccessMonitor');
	} else { 
		$mon->save();
		$feedback = $lang->getText('Common.SuccessMonitor');
	}
} elseif (!$notify && !is_null($mon)) {
	$mon->delete();
	$feedback = $lang->getText('Common.SuccessStopMonitor');
}

if (isset($_GET['redirect_to'])) {
	$redirect = $_GET['redirect_to'];
		if ($feedback) {
		// attach message to redirect URL
		if (strpos($redirect, '?') === false) {
			$redirect = $redirect.'?feedback='.urlencode($feedback);
		} else {
			$redirect = $redirect.'&feedback='.urlencode($feedback);
		}

	//Hackish and ugly, but it will work until we start using session vars
	//in the new codebase
	if(preg_match('/UsersMonitoringBrowse/', $redirect)){
		$redirect .= "&section=$section&ref_id=$ref_id&user_id$user_id";
	}

	}


	
	if (!Validator::validateRedirect($redirect)) {
		throw new Exception($lang->getText('Exception.InvalidRedirect'));
	}

	GFUrl::getUrl()->redirect($redirect);
}
?>
