<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */


require_once('session/LoginFactory.php');
require_once('Validator.php');
require_once('plugin/ActionManager.php');
require_once('StringUtils.php');

$login = LoginFactory::getLogin();
$error = "";
$showTFAForm = false;

if (isset($_REQUEST['username']) || (GFConf::get('system.useSSO') && isset($_SERVER[GFConf::get('system.SSOServerVar')]))) {
	try {
		//sso begin
		if (GFConf::get('system.useSSO')) {
			//query the database to get the user account info to auto-login
			//based on the presented SSO ID
			//the remote address may need to be split and take only the first part
			$tmpsplitchar=GFConf::get('system.SSOServerVarSplitChar');
			if (!empty($tmpsplitchar)) {
				$tmpserverid=explode(GFConf::get('system.SSOServerVarSplitChar'), $_SERVER[GFConf::get('system.SSOServerVar')]);
				$_SERVER[GFConf::get('system.SSOServerVar')]=$tmpserverid[0];
			}

			$c = new Criteria(UserPeer::DATABASE_NAME);
			if (GFConf::get('system.SSOIDMatchesUnixName')) {
				$c->add(UserPeer::UNIX_NAME, strtolower($_SERVER[GFConf::get('system.SSOServerVar')]));
			} else {
				$c->add(UserPeer::EXTERNAL_ID, $_SERVER[GFConf::get('system.SSOServerVar')]);
			}
			$user = UserPeer::doSelectOne($c);

			if (is_null($user)) {
				// Redirect to the register page. The user name will be filled
				// with the SM user name.

				GFUrl::redirect(GFUrl::getUrl()->baseUrl('UserAdd', '', 'account'));
				exit;
			}
			
			if ($user->getStatus() != User::STATUS_ACTIVE) {
				throw new Exception(Language::getSessText('User.AccountDisabled'));
			}

			//Do not remove this. This check must be performed to avoid a propel exception in case
			//the session hash already exists in user_session table. Since SSOLogin doesn't have a proper
			//Login class (like DatabaseLogin and LdapLogin) we need to do it here.
			$c = new Criteria();
			$c->add(UserSessionPeer::SESSION_HASH, session_id());
			$userSession = UserSessionPeer::doSelectOne($c);
			if (is_null($userSession)) {
				$userSession = new UserSession();
			}

			$userSession->setUser($user);
			$_REQUEST['username']=$user->getUnixName();
			$_REQUEST['password']=$user->getPasswordMd5();
			redirectLoginSuccess($userSession);


		} else {
			//non-sso login			
			$username = $_REQUEST['username'];
			$password = $_REQUEST['password'];
						
			$userSession = $login->doLogin($username, $password);
		}
		/*
		if (GFConf::get('system.useTFA')) {
			// TFA active, show TFA form and validate SMS token
			$showTFAForm = true;
			$tfa = new TwoFactorAuthenticator();
			if (empty($_POST['evaltfa'])) {// send token, show TFA form
				$cell = $userSession->getUser()->getCell();
				if (!empty($cell)) {// user has cell defined
					$from = GFConf::get('system.tfaTwilioCellNumber');
					try {
						$tfa->sendToken($cell, 'login');
					} catch (Exception $e) {
						throw new LoginException($e->getMessage());
					}
					$feedback = Language::getSessText('User.TFA.LoginMessage');
					require_once('Login.php');
				} else {// TFA active but user has no cell, skip
					//flag user session, starting TFA catchup
					$_SESSION['catchup_tfa'] = '1';
					redirectLoginSuccess($userSession);
				}
			} else { // eval form
				$tokenError = $tfa->verifyToken($_POST['token'], 'login');
				if (!empty($tokenError)) {
					error_log($tokenError);
					throw new LoginException($tokenError);
				} else {
					redirectLoginSuccess($userSession);
				}
			}
		} else { // TFA inactive, end login successfully
			redirectLoginSuccess($userSession);
		}
		*/
		redirectLoginSuccess($userSession);
	} catch (LoginException $e) {
		$feedback = $e->getMessage();
		require_once('Login.php');
	} catch (PasswordExpiredException $e) {
		GFUrl::redirect(GFUrl::getUrl()->baseUrl('UserPasswordEdit', '', 'account', array('username'=>$username, 'password_expired'=>true)));
		exit;
	}
} else {
	$feedback = '* '.Language::getSessText('Common.UsernameRequired');
	include('Login.php');
}

function redirectLoginSuccess($userSession)
{
	$session = Session::getSession();
	$session->setUserSession($userSession, $_SERVER["REMOTE_ADDR"]);

	$params = array('username' => $_REQUEST['username'],
					'password' => $_REQUEST['password']);
	ActionManager::notify('USER_LOGIN', $params);

	if(isset($_REQUEST['redirect']) && !empty($_REQUEST['redirect']) && $_REQUEST['redirect']) {
		header("Location: ".StringUtils::stripHtmlTags($_REQUEST['redirect']));
	} else {
		header("Location: /gf/my/");
	}
}

?>


