<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code creates a new Project and saves it to the 
// database
//
require_once('lib/core/Project.php');
require_once('lib/core/ProjectPeer.php');
require_once('lib/exceptions/PermissionDeniedException.php');
require_once('lib/core/Language.php');
require_once('lib/SystemUtils.php');
require_once('lib/core/DeletedProjectPeer.php');

$Lang = Language::getSessLang();

$redirect = (isset($_REQUEST['redirect'])) ? Validator::sanitizeString($_REQUEST['redirect']) : '';
if ($redirect != 'approve-pending') {
	$redirect = 'ProjectBrowse';
}

$delete_type = (isset($_REQUEST['delete_type'])) ? $_REQUEST['delete_type'] : '';

if (array_key_exists('sure',$_POST)) {
	//NEW Batch Delete
	if (isset($_POST['id'])) {
		$feedback = deleteProject($_POST['id']);
		GFUrl::redirect($url->currentBaseURL($redirect, array('feedback' => $feedback['feedback'])));
	} elseif (isset($_REQUEST['ids'])) {
		//An array of projects to remove
	    $idsArray = explode('-', $_REQUEST['ids']);
	    //Arrays for feedback
	    $removedCount = 0;
	    $notRemovedCount = 0;
	    foreach ($idsArray as $proj_id) {
	    	$feedback = deleteProject($proj_id, $delete_type);
	    	//Combine the feedback
	    	if ($feedback['removed']) {
	    		$removedCount++;
	    	} else {
	    		$notRemovedCount++;
	    	}
	    }
	    $batchFeedbak = '';
	    if ($removedCount > 0) {
	    	$batchFeedback .= $Lang->getText('SiteAdmin.ProjectsRemoved', array($removedCount));
	    }
	    if ($notRemovedCount > 0) {
	    	$batchFeedback .= $Lang->getText('SiteAdmin.ProjectsNotRemoved', array($notRemovedCount));
	    }
	    GFUrl::redirect($url->currentBaseURL($redirect, array('feedback' => $batchFeedback)));
	}
} else {
	GFUrl::redirect($url->currentBaseURL($redirect), array('feedback' => $Lang->getText('SiteAdmin.ProjectEdit.NoChanges')));
}

function deleteProject($project_id, $delete_type='') {
	$removed = false;
	$Project = ProjectPeer::retrieveByPk($project_id);
	$unix_name = $Project->getUnixName();
	
	if (is_object($Project) && $Project->userCanDelete()) {
		if (array_key_exists('send_reject',$_POST)) {
			if (!empty($_POST['response_title']) && !empty($_POST['response_text'])) {
				$users_projects = $Project->getUserProjects();

				if ($Project->remove()) {
					// See if the project was rejected. If it was, erase it directly from the deleted_project table.
					if ($delete_type == 'rejection') {
						DeletedProjectPeer::doDelete($unix_name);
					}
					$removed = true;
					$feedback = Language::getSessText('SiteAdmin.ProjectApprove.Removed',array($Project->getName())) . '<br />';
					// Send the rejection email to the users in the project (at this
					// stage, it is probably only one user)
					foreach ($users_projects as $user_project) {
						$user = $user_project->getUser();
						SystemUtils::sendMail($_POST['response_title'],$_POST['response_text'],$user);
					}
					
					$feedback .= Language::getSessText('SiteAdmin.ProjectApprove.RejectMailSent') . '<br />';
				} else {
					$removed = false;
					$feedback = Language::getSessText('SiteAdmin.ProjectApprove.NotRemoved',array($Project->getName())) . '<br />';
				}
			} else {
				$feedback = Language::getSessText('SiteAdmin.ProjectApprove.EmptyMailInfo') . '<br />';
			}
		} elseif ($Project->remove()) {
			// See if the project was rejected. If it was, erase it directly from the deleted_project table.
			if ($delete_type == 'rejection') {
				DeletedProjectPeer::doDelete($unix_name);
			}
			$removed = true;
			$feedback = Language::getSessText('SiteAdmin.ProjectApprove.Removed',array($Project->getName())) . '<br />';
		} else {
			$removed = false;
			$feedback = Language::getSessText('SiteAdmin.ProjectApprove.NotRemoved',array($Project->getName())) . '<br />';
		}
		return array(
			'removed'      => $removed,
			'feedback'     => $feedback
		);
	} else {
		throw new PermissionDeniedException();
	}
}

?>
