<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

//
// This code creates a new Project and saves it to the 
// database
//
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('core/Project.php');
require_once('exceptions/NotLoggedInException.php');

$session = Session::getSession();
$Lang = $session->getLanguage();
if (!$session->isLoggedIn()) {
	throw new NotLoggedInException();
}

if ($_SERVER['REQUEST_METHOD'] != 'POST' || !$session->isSiteAdmin()) {
	throw new PermissionDeniedException();
}

// WHERE SHOULD WE DEFINE THIS?
define('TEMPLATE_VERSION', '1.0');

// Save Template to a temporary file
$temp_file = '/tmp/template_user-' . $session->getUser()->getUserId() . '.xml';

if (!isset($_FILES['template']['tmp_name']) || $_FILES['template']['tmp_name'] == '') {
	$feedback = '* '.$Lang->getText('ProjectTemplate.NoFileUploaded');
	include('ProjectTemplateImport.php');
	exit();
}

if (!move_uploaded_file($_FILES['template']['tmp_name'], $temp_file)) {
	$feedback = '* '.$Lang->getText('ProjectTemplate.UploadError');
	include('ProjectTemplateImport.php');
	exit();
}

// Load the XML file in a DOM object
$dom = new DOMDocument();
$dom->load($temp_file);

// Delete the temporary file
@unlink($temp_file);

// Get the Templates of the file should be only 1 anyway
$TemplateXML = $dom->getElementsByTagName('ProjectTemplate')->item(0);

// Let's see if we found a project template in the XML
if ($TemplateXML == null || !is_object($TemplateXML)) {
	$feedback = '* '.$Lang->getText('ProjectTemplate.XMLError');
	include('ProjectTemplateImport.php');
} else {
	$version = $TemplateXML->getAttribute('version');

	// If it's the same version, let's continue
	// this could be improved using version_compare()
	if ($version != TEMPLATE_VERSION) {
		$feedback = '* '.$Lang->getText('ProjectTemplate.InvalidVersion');
		include('ProjectTemplateImport.php');
	}
	else {
		try {
			$project_name = $TemplateXML->getElementsByTagName('Name')->item(0)->nodeValue;
			$unix_name = $TemplateXML->getElementsByTagName('UnixName')->item(0)->nodeValue;
			$description = $TemplateXML->getElementsByTagName('Description')->item(0)->nodeValue;
	
			$Project = new Project();
			$Project->setProjectName(Validator::sanitizeString($project_name) );
			$Project->setUnixName(Validator::sanitizeString($unix_name) );
			$Project->setDescription( Validator::sanitizeString($description) );
			$Project->setStatus(Project::STATUS_ACTIVE);
			$Project->setIsTemplate(true);
			$Project->setIsPublic(true);
			$Project->setHomepageUrl('');
			$Project->setRegisterPurpose('');
			$Project->setRegisterLicenseOther('');
			$Project->setTemplateProjectId(0);
			$Project->save();
			
			// Add Roles
			addRoles($Project, $TemplateXML);
				
			// Get Plugins
			$PluginsNode = $TemplateXML->getElementsByTagName('Plugins')->item(0);
			$Plugins = $PluginsNode->childNodes;
			foreach ($Plugins as $Plugin) {
				
				switch ($Plugin->tagName) {
					case 'Trackers':
						$Project->enablePlugin(TrackerPlugin::SECTION);
					    addTrackers($Project, $Plugin);
					    break;
					case 'Mailman':
						$Project->enablePlugin(MailmanPlugin::SECTION);
					    addMailman($Project, $Plugin);
					    break;
					case 'Frs':
						$Project->enablePlugin(FrsPlugin::SECTION);
					    addFrs($Project, $Plugin);
					    break;
					case 'Forums':
						$Project->enablePlugin(ForumPlugin::SECTION);
					    addForums($Project, $Plugin);
					    break;
					case 'Docman':
						$Project->enablePlugin(DocmanPlugin::SECTION);
					    addDocman($Project, $Plugin);
					    break;
					case 'SCM':
						//TODO: Enable Plugin
						addSCM($Project, $Plugin);
						break;			
				}
			}
	
		} catch (InvalidDataException $ex) {
			$errors = $ex->getErrors();
			$feedback = '* '.implode('<br />* ', $errors);
			include('ProjectTemplateImport.php');
			exit();
		} catch (Exception $ex) {
			$errors[] = $ex->getMessage();
			$feedback = '* '.implode('<br />* ', $errors);
			include('ProjectTemplateImport.php');
			exit();
		}
		GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('adminindex'), array('feedback' => $Lang->getText('ProjectTemplate.ImportSucessful')));
	}
}

function addTrackers($Project, $Plugin) {
	addRoleSettings($Project, $Plugin, 'tracker', 0);
	$domTrackers = $Plugin->getElementsByTagName('Tracker');
	foreach ($domTrackers as $domTracker) {
		$description = $domTracker->getElementsByTagName('Description')->item(0)->nodeValue;
		$Tracker = new Tracker();
		$Tracker->setProject($Project);
		$Tracker->setTrackerName(Validator::sanitizeString($domTracker->getAttribute('name')));
		$Tracker->setDatatype(Validator::sanitizeInt($domTracker->getAttribute('type')));
		$Tracker->setDescription(Validator::sanitizeString($description));
		$Tracker->setIsPublic($domTracker->getAttribute('is_public') == 'true' ? true : false);
		$Tracker->setRestrictBrowse($domTracker->getAttribute('restrict_browse') == 'true' ? true : false);
		$Tracker->setEmailAllUpdates($domTracker->getAttribute('email_updates') == 'true' ? true : false);
		$duedays = ($domTracker->getAttribute('due_period') != '') ? Validator::sanitizeInt($domTracker->getAttribute('due_period')) : 15;
		$Tracker->setDuePeriod($duedays*60*60*24);
		$Tracker->setEmailAddress('');
		$Tracker->setSubmitInstructions('');
		$Tracker->setBrowseInstructions('');
		$Tracker->save();
		addRoleSettings($Project, $domTracker, 'tracker', $Tracker->getTrackerId());
		
		$domExtraFields = $domTracker->getElementsByTagName('ExtraField');
		$ef_count = 0;
		foreach ($domExtraFields as $domExtraField) {
			$domProperties = $domExtraField->getElementsByTagName('Properties')->item(0);
			$TrackerExtraFields[$ef_count] = new TrackerExtraField();
			$TrackerExtraFields[$ef_count]->setTracker($Tracker);
			$TrackerExtraFields[$ef_count]->setFieldName(Validator::sanitizeString($domExtraField->getAttribute('name')));
			$TrackerExtraFields[$ef_count]->setFieldType(Validator::sanitizeInt($domExtraField->getAttribute('type')));
			$TrackerExtraFields[$ef_count]->setAttribute1(Validator::sanitizeString($domExtraField->getAttribute('attr1')));
			$TrackerExtraFields[$ef_count]->setAttribute2(Validator::sanitizeString($domExtraField->getAttribute('attr2')));
			$TrackerExtraFields[$ef_count]->setIsRequired($domProperties->getAttribute('required') == 'true' ? true : false);
			$TrackerExtraFields[$ef_count]->setFieldOrder(Validator::sanitizeInt($domProperties->getAttribute('order')));
			$TrackerExtraFields[$ef_count]->setUseInQuickBrowse($domProperties->getAttribute('quick_browse') == 'true' ? true : false);
			$TrackerExtraFields[$ef_count]->setShowOnSubmit($domProperties->getAttribute('on_submit') == 'true' ? true : false);
			$TrackerExtraFields[$ef_count]->setUseWorkFlow($domProperties->getAttribute('workflow') == 'true' ? true : false);
			$TrackerExtraFields[$ef_count]->setUseCommitRule($domProperties->getAttribute('commit_rule') == 'true' ? true : false);
			$TrackerExtraFields[$ef_count]->save();
			
			$TrackerExtraFieldsElements[$ef_count]=array();
			
			$domFieldElements = $domExtraField->getElementsByTagName('ExtraFieldElement');
			$TrackerExtraFieldsElementsCount[$ef_count] = 0;
			
			foreach ($domFieldElements as $domFieldElement) {
				$TrackerExtraFieldsElements[$ef_count][$TrackerExtraFieldsElementsCount[$ef_count]] = new TrackerExtraFieldElement();
				$TrackerExtraFieldsElements[$ef_count][$TrackerExtraFieldsElementsCount[$ef_count]]->setTrackerExtraField($TrackerExtraFields[$ef_count]);
				$TrackerExtraFieldsElements[$ef_count][$TrackerExtraFieldsElementsCount[$ef_count]]->setElementName(Validator::sanitizeString($domFieldElement->getAttribute('name')));
				$TrackerExtraFieldsElements[$ef_count][$TrackerExtraFieldsElementsCount[$ef_count]]->setStatusId(Validator::sanitizeInt($domFieldElement->getAttribute('status')));
				$TrackerExtraFieldsElements[$ef_count][$TrackerExtraFieldsElementsCount[$ef_count]]->setDefaultSelected($domFieldElement->getAttribute('default') == 'true' ? true : false);
				$TrackerExtraFieldsElements[$ef_count][$TrackerExtraFieldsElementsCount[$ef_count]]->setCanCommit($domFieldElement->getAttribute('can_commit') == 'true' ? true : false);
				$TrackerExtraFieldsElements[$ef_count][$TrackerExtraFieldsElementsCount[$ef_count]]->setSortOrder(Validator::sanitizeInt($domFieldElement->getAttribute('order')));
				$TrackerExtraFieldsElements[$ef_count][$TrackerExtraFieldsElementsCount[$ef_count]]->save();
				$TrackerExtraFieldsElementsCount[$ef_count]++;
			}
			
			$ef_count++;
		}
			
		//Load Workflow Transition Rules
		//We load the transition rules after loading all the  TrackerExtraFields and TrackerExtraFieldElements, because
		//the rules cross reference this elements, and if it is not done this way then
		//its possible that a rule references a yet non existent element.
		
		for ($ef=0;$ef<$ef_count;$ef++) {
			$TrackerExtraField = $TrackerExtraFields[$ef];
			for($efe=0;$efe<$TrackerExtraFieldsElementsCount[$ef];$efe++) {
				$TrackerExtraFieldElement = $TrackerExtraFieldsElements[$ef][$efe];
				$domFieldElements = $domExtraFields->item($ef)->getElementsByTagName('ExtraFieldElement');
				
				$domTransitionRules = $domFieldElements->item($efe)->getElementsByTagName('ExtraFieldElementTransitionRule');
								
				foreach ($domTransitionRules as $domTransitionRule) {

					$domTransitionRuleRoles = $domTransitionRule->getElementsByTagName('ExtraFieldElementTransitionRuleRole');
					$domTransitionRuleNotifyRoles = $domTransitionRule->getElementsByTagName('ExtraFieldElementTransitionRuleNotifyRole');
					$domTransitionRuleAssignRoles = $domTransitionRule->getElementsByTagName('ExtraFieldElementTransitionRuleAssignRole');
					$domTransitionRuleFields = $domTransitionRule->getElementsByTagName('ExtraFieldElementTransitionRuleField');

					$TrackerExtraFieldElementTransition = new TrackerExtraFieldElementTransition();
					$TrackerExtraFieldElementTransition->setElementId($TrackerExtraFieldElement->getElementId());
					
					//Get "Change To" Element		
					$toElement = Validator::sanitizeString($domTransitionRule->getAttribute('toElement'));
					foreach($TrackerExtraFieldsElements[$ef] as $tefe) {
						if ($tefe->getElementName() == $toElement) {
							$TrackerExtraFieldElementTransition->setToElementId($tefe->getElementId());		
						}
					}
					
					// Notification message
					$TrackerExtraFieldElementTransition->setNotificationMessage(Validator::sanitizeString($domTransitionRule->getAttribute('notificationMessage')));
					
					// Roles allowed for the rule
					foreach($domTransitionRuleRoles as $domTransitionRuleRole) {
						$crit = new Criteria();
						$crit->add(RolePeer::ROLE_NAME, Validator::sanitizeString($domTransitionRuleRole->getAttribute('name')));
						$crit->add(RolePeer::PROJECT_ID, $Project->getId());
						$role = RolePeer::doSelectOne($crit);
						$tefetr=new TrackerExtraFieldElementTransitionRole();
						$tefetr->setRole($role);
						$TrackerExtraFieldElementTransition->addTrackerExtraFieldElementTransitionRole($tefetr);
						$tefetr->save();
					}
					
					// Roles to be notified of changes
					foreach($domTransitionRuleNotifyRoles as $domTransitionRuleRole) {
						$crit = new Criteria();
						$crit->add(RolePeer::ROLE_NAME, Validator::sanitizeString($domTransitionRuleRole->getAttribute('name')));
						$crit->add(RolePeer::PROJECT_ID, $Project->getId());
						$role = RolePeer::doSelectOne($crit);
						$tefetn=new TrackerExtraFieldElementTransitionNotify();
						$tefetn->setRole($role);
						$TrackerExtraFieldElementTransition->addTrackerExtraFieldElementTransitionNotify($tefetn);
						$tefetn->save();
					}
					
					// Roles to be reassigned to
					foreach($domTransitionRuleAssignRoles as $domTransitionRuleRole) {
						$crit = new Criteria();
						$crit->add(RolePeer::ROLE_NAME, Validator::sanitizeString($domTransitionRuleRole->getAttribute('name')));
						$crit->add(RolePeer::PROJECT_ID, $Project->getId());
						$role = RolePeer::doSelectOne($crit);
						$tefetr=new TrackerEfeTransitionReassign();
						$tefetr->setRole($role);
						$TrackerExtraFieldElementTransition->addTrackerEfeTransitionReassign($tefetr);
						$tefetr->save();
					}
					
					// Fields on which the rule depends.
					foreach($domTransitionRuleFields as $domTransitionRuleField) {
						$field_name = Validator::sanitizeString($domTransitionRuleField->getAttribute('name'));

						foreach ($TrackerExtraFields as $tef) {
							if ($tef->getFieldName() == $field_name) {
								$tefetf=new TrackerExtraFieldElementTransitionField();
								$tefetf->setTrackerExtraField($tef);
								$TrackerExtraFieldElementTransition->addTrackerExtraFieldElementTransitionField($tefetf);
								$tefetf->save();
								break;
							}
						}
					}
									
					$TrackerExtraFieldElementTransition->save();	
				}
				$TrackerExtraFieldElement->save();
			}
			$TrackerExtraField->save();
		}
	}
}

function addMailman($Project, $Plugin) {
	$domLists = $Plugin->getElementsByTagName('List');
	foreach ($domLists as $domList) {
		$description = $domList->getElementsByTagName('Description')->item(0)->nodeValue;
		$listPassword= substr(md5(rand(0,10000)), 0, 16);
		$Mailman = new Mailman();
		$Mailman->setProjectId($Project->getProjectId());
		$Mailman->setListName(Validator::sanitizeString($Project->getUnixName().'-'.$domList->getAttribute('name')));
		$Mailman->setIsPublic($domList->getAttribute('is_public') == 'true' ? true : false);
		$Mailman->setListPassword($listPassword);
		$Mailman->setCreatedBy(Session::getSession()->getUser()->getUserId());
		$Mailman->setStatus(Mailman::STATUS_REQUESTED);
		$Mailman->setListDescription(Validator::sanitizeString($description));
		$Mailman->save();
		addRoleSettings($Project, $domList, 'mailman', $Mailman->getMailmanId());
	}
}

function addFrs($Project, $Plugin) {
	addRoleSettings($Project, $Plugin, 'frs', 0);
	$domPackages = $Plugin->getElementsByTagName('Package');
	foreach ($domPackages as $domPackage) {
		$FrsPackage = new FrsPackage();
		$FrsPackage->setProject($Project);
		$FrsPackage->setPackageName(Validator::sanitizeString($domPackage->getAttribute('name')));
		$FrsPackage->setStatusId(Validator::sanitizeInt($domPackage->getAttribute('status')));
		$FrsPackage->setIsPublic($domPackage->getAttribute('is_public') == 'true' ? true : false);
		$FrsPackage->setRequireLogin($domPackage->getAttribute('req_login') == 'true' ? true : false);
		$FrsPackage->save();
		
	}
}

function addForums($Project, $Plugin) {
	$domForums = $Plugin->getElementsByTagName('Forum');
	addRoleSettings($Project, $Plugin, 'forum', 0);
	foreach ($domForums as $domForum) {
		$description = $domForum->getElementsByTagName('Description')->item(0)->nodeValue;
		$Forum = new Forum();
		$Forum->setSection('project');
		$Forum->setRefId($Project->getProjectId());
		$Forum->setForumName(Validator::sanitizeString($domForum->getAttribute('name')));
		$Forum->setDescription(Validator::sanitizeString($description));
		$Forum->setIsPublic($domForum->getAttribute('is_public') == 'true' ? true : false);
		$Forum->setModerationLevel(Validator::sanitizeInt($domForum->getAttribute('mod_level')));
		$Forum->save();
		addRoleSettings($Project, $domForum, 'forum', $Forum->getForumId());
	}
	
}

function addDocman($Project, $Plugin) {
	$domFolders = $Plugin->getElementsByTagName('Folder');
	foreach ($domFolders as $domFolder) {
		addFolders($Project, $domFolder, 0);
	}
	addRoleSettings($Project, $Plugin, 'docman', 0);
}

function addFolders($Project, $domFolder, $parent_id) {
	$DocmanFolder = new DocmanFolder();
	$DocmanFolder->setProject($Project);
	$DocmanFolder->setParentFolderId($parent_id);
	$DocmanFolder->setFolderName(Validator::sanitizeString($domFolder->getAttribute('name')));
	$DocmanFolder->setIsPublic($domFolder->getAttribute('is_public') == 'true' ? true : false);
	$DocmanFolder->save();

	$domChildren = $domFolder->childNodes;
	foreach ($domChildren as $domChild) {
		addFolders($Project, $domChild, $DocmanFolder->getDocmanFolderId());
	}
}

function addSCM($Project, $Plugin) {
	addRoleSettings($Project, $Plugin, 'scm', 0);
}

function addRoles($Project, $domNode) {
	$xmlRoles = $domNode->getElementsByTagName('Role');
	foreach ($xmlRoles as $xmlRole) {	
		if (strtolower(Validator::sanitizeString($xmlRole->getAttribute('name'))) == strtolower(Project::PROJECT_ROLE_ADMIN_NAME))
		{
			// This Role was created automatically when the project was saved, ignore it.
			continue;
		}
		else {
			// Non admin role, regular creation.
			$role = new Role();
			$role->setIsNewProject();
			$role->setRoleName($xmlRole->getAttribute('name'));
			$role->setProject($Project);
			$role->save();
			
			//Set projectadmin role setting
			$rolesetting = new RoleSetting();
			$rolesetting->setRefId(0);
			$rolesetting->setValue(Validator::sanitizeInt($xmlRole->getAttribute('projectadmin')));
			$rolesetting->setSection('projectadmin');
			$role->addRoleSetting($rolesetting);
			$rolesetting->save();
		}
	}
}

function addRoleSettings($Project, $domNode, $section, $ref_id) {
	
	//$xmlRoleSettings = $domNode->getElementsByTagName('RoleSetting');
	foreach ($domNode->childNodes as $xmlRoleSetting) {
		if($xmlRoleSetting->tagName=='RoleSetting') {
			$crit = new Criteria();
			$crit->add(RolePeer::PROJECT_ID, $Project->getProjectId());
			$crit->add(RolePeer::ROLE_NAME, Validator::sanitizeString($xmlRoleSetting->getAttribute('role')));
			$crit->addAscendingOrderByColumn(RolePeer::ROLE_NAME);
			$role = RolePeer::doSelectOne($crit);

			if(is_object($role)) {
				//Check if the RoleSetting does not yet exist. It can happen!!
				$c = new Criteria();
				$c->add(RoleSettingPeer::ROLE_ID,$role->getRoleID());
				$c->add(RoleSettingPeer::SECTION, $section);
				$c->add(RoleSettingPeer::REF_ID, $ref_id);
				$rolesetting = RoleSettingPeer::doSelectOne($c);

				if(!is_object($rolesetting)) {
					$rolesetting = new RoleSetting();
					$rolesetting->setRefId($ref_id);
					$rolesetting->setSection($section);
					$role->addRoleSetting($rolesetting);
				}

				$rolesetting->setValue(Validator::sanitizeInt($xmlRoleSetting->getAttribute('value')));
				$rolesetting->save();
			}
		}
	}
}
?>
