<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('core/SiteAdmin.php');
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');

if (!Session::getSession()->isSiteAdmin()) {
	throw new PermissionDeniedException();
}
if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}
$errors = array();
$Lang = Language::getSessLang();


if(isset($_POST['edit'])) {
	$TroveCategory = TroveCategoryPeer::retrieveByPK($_REQUEST['trove_cat_id']);
	
	if($TroveCategory->getRootTroveCategoryId() == 0) { //check if its a cat root
		$root_category_id = Validator::sanitizeInt($_REQUEST['parent_trove_cat_id']);
	} else {	
		$root_category_id = $TroveCategory->getRootTroveCategoryId();
	}

	//Check if alias already exist
	$c = new Criteria();
	$c->add(TroveCategoryPeer::ALIAS, Validator::sanitizeString($_POST['alias']), Criteria::EQUAL);
	$alias = TroveCategoryPeer::doSelect($c);


	if (isset($_POST['show_in_search'])) {
		if ($_POST['show_in_search'] == "1") {
			$show_in_search = true;
		} else {
			$show_in_search = false;
		}
	} else {
		$show_in_search = false;
	}

	try {
		$TroveCategory->setRootTroveCategoryId($root_category_id);
		$TroveCategory->setParentTroveCategoryId($_POST['parent_trove_cat_id']);
		$TroveCategory->setAlias(Validator::sanitizeString($_POST['alias']));
		$TroveCategory->setCategoryName(Validator::sanitizeString($_POST['fullname']));
		$TroveCategory->setDescription(Validator::sanitizeString($_POST['description']));
		$TroveCategory->setShowInSearch($show_in_search);
		$TroveCategory->setAccessLevel((int)$_POST['access_level']);
		$TroveCategory->save();

		GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('DisplayTroveMap'), array('feedback' => $language->getText('TroveEdit.EditFeedback')));
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());
		$feedback = '* '.implode('<br />* ', $errors);
		include('TroveEdit.php');
	}
}

if(isset($_POST['delete'])) {
	if (!array_key_exists('sure',$_POST) || !$_POST['sure']) {
		GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TroveDelete'), array('trove_cat_id' => $_REQUEST['trove_cat_id'], 'feedback' => $language->getText('TroveEdit.DeleteNotConfirm')));
	}

    $troveCatIds = explode(',', $_REQUEST['trove_cat_id']);
    $con = Propel::getConnection(TroveCategoryPeer::DATABASE_NAME);
    $con->begin();
    foreach ($troveCatIds as $trove_id) { 
        $TroveCategory = TroveCategoryPeer::retrieveByPk($trove_id);
        try {
            // Proceed only if category exists, and it is not fixed
            if (!$TroveCategory || !is_object($TroveCategory) || $TroveCategory->isFixed()) { 
                continue;
            }
            $TroveCategory->delete();
        } catch (InvalidDataException $ex) {
            $con->rollback();
            $errors = $ex->getErrors();
            $feedback = '* '.implode('<br />* ', $errors);
            GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('DisplayTroveMap'), array('feedback' => $feedback));
        }
    }
    $con->commit();
    GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('DisplayTroveMap'), array('feedback' => $language->getText('TroveEdit.DeleteFeedback')));
}

?>
