<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   
 * @author Daniel A. Perez	daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('exceptions/PermissionDeniedException.php');

// db conn
$con = Propel::getConnection(TroveCategoryPeer::DATABASE_NAME);

// field_name => no change value
$no_change = array(
    'access_level' => -1,
    'show_in_search' => -1,
    'move' => -1,
    'merge' => NULL,
    'massdelete' => NULL
);

/**
 * Determines whether the value of a posted mass update form field means an update is required
 * 
 * @global array $no_change
 * @param string $field_name
 * @return bool
 */
function requires_change($field_name) {
    global $no_change;
    return $_POST[$field_name] != $no_change[$field_name];
}

/**
 * Removes the id of any category such that the id for an ancestor category is also on the list
 * 
 * @global $con
 * @param array $trove_ids list of trove ids
 * @return array
 */
function removeDescendants($trove_ids) {
    global $con;
    foreach ($trove_ids as $trove_id) {
        $trove_id = Validator::sanitizeInt($trove_id);
        if ($trove_id > 0) {
            $stmt = $con->createStatement();
            $children = $stmt->executeQuery("SELECT trove_category_id FROM trove_children($trove_id, FALSE)", ResultSet::FETCHMODE_ASSOC);
            $children_ids = array();
            while ($children->next()) {
                $children_ids[] = $children->getInt('trove_category_id');
            }
            // Remove children
            $trove_ids = array_diff($trove_ids, $children_ids);
        }
    }
    return $trove_ids;
}

// Allow POST only
if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

// default value for required params
if (!isset($_POST['massdelete'])) $_POST['massdelete'] = NULL;
if (!isset($_POST['merge'])) $_POST['merge'] = NULL;

// Validate at least one trove category selected for mass update
if (!isset($_POST['trove_ids'])) {
	GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('DisplayTroveMap', array(
        'feedback' => $language->getText('TroveMassUpdate.NoItemsSelected')
    )));
}

// Validate at least one field value selected to be updated or the action is mass delete
if (!requires_change('access_level') && !requires_change('show_in_search') && !requires_change('massdelete') && !requires_change('move') && !requires_change('merge')) {
	GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('DisplayTroveMap', array(
        'feedback' => $language->getText('TroveMassUpdate.NoChangesRequired')
    )));
}

if (requires_change('massdelete')) {
    // Redirect to delete trove view
    GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TroveDelete'), array(
        'trove_cat_id' => implode(',', $_REQUEST['trove_ids'])
    ));
} else {
    if (requires_change('move') || requires_change('merge')) {
        // Get rid of descendants, will be moved when moving their ancestors
        $move_trove_ids = array_diff(removeDescendants($_POST['trove_ids']), TroveCategoryPeer::getFixedCategoriesIds());
    }
    $Troves = TroveCategoryPeer::retrieveByPKs($_POST['trove_ids']);
    // Process update    
    $con->begin();    
    foreach ($Troves as $Trove) { 
        try {
            if (requires_change('access_level')) {
                $Trove->setAccessLevel($_POST['access_level']);
            }
            if (requires_change('show_in_search')) {
                $Trove->setShowInSearch($_POST['show_in_search'] == 1);
            }
            // Proceed to merge only if it is in our list of "ok to move"
            if (requires_change('merge') && in_array($Trove->getTroveCategoryId(), $move_trove_ids)) {
                $Trove->merge($_POST['move']);
            }
            // Proceed to move only if we're not merging and it is in our list of "ok to move"
            if (!requires_change('merge') && requires_change('move') && in_array($Trove->getTroveCategoryId(), $move_trove_ids)) {
                $Trove->move($_POST['move']);
            }
            $Trove->save();
        } catch (InvalidDataException $ex) {
            $con->rollback();
            GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('DisplayTroveMap', array(
                'feedback' => implode('<br />', $ex->getErrors()),
            )));
        }
    }
    $con->commit();

    GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('DisplayTroveMap', array(
        'feedback' => $language->getText('TroveMassUpdate.Feedback', array(count($_POST['trove_ids'])))
    )));
}

?>