<?php
/**
 * Copyright (c) 2005-2012 GForge LLC
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('core/UserPeer.php');
require_once('plugins/forum/lib/ForumPlugin.php');

$session = Session::getSession();

if (!$session->isLoggedIn()) {
	error_log("GetRoleSettings, session not logged in, returning 401");
	header('HTTP/1.1 401');
	exit;
}

$user = $session->getUser();

if (!is_object($user)) {
	error_log("GetRoleSettings, session has no user, returning 401");
	header('HTTP/1.1 401');
	exit;
}
 
$plugin	= isset($_GET['section']) ? $_GET['section'] : '';
$ref_id	= isset($_GET['ref_id']) ? $_GET['ref_id'] : 0;

$thread = ForumThreadPeer::retrieveByPk($ref_id);
if (!is_object($thread)) {
	exit;
}

$forum = $thread->getForum();
$project = $forum->getProject();

if (!is_object($project) ||
    ! $project->getPluginByName(ForumPlugin::SECTION)) {
	error_log("GetRoleSettings, project does not exist or does not have forum plugin enabled, returning 404");
	header('HTTP/1.1 404');
	exit;
}

//Validate permissions
if (!$session->isSiteAdmin() && !$project->userIsAdmin()) {
	echo "[]";
	exit;
}

//Get the plugin settings
$forum_plugin = new ForumPlugin();
$params = array('project' => $project);
$plugin_result = $forum_plugin->getRoleChoices($params);
$roles = $plugin_result->getResult();

$arr_rolesettings = array();

$avail_rolesettings = $forum_plugin->getAvailableRoleSettings();
$roles = $project->getRoles();

foreach ($roles as $role) {
	// Ask the role settings
	$c = new Criteria();

	$c->add(RoleSettingPeer::ROLE_ID, $role->getPrimaryKey());
	$c->add(RoleSettingPeer::SECTION, 'forum');
	$c->add(RoleSettingPeer::REF_ID, $forum->getForumId());
	$role_setting = RoleSettingPeer::doSelectOne($c);

	if (is_object($role_setting)) {
		$checked = $role_setting->getValue();
		$setting = $avail_rolesettings[0][$checked+1];
	} else {
		$setting = $avail_rolesettings[0][ForumPlugin::READ];
	}
	
	$arr_rolesetting = array();
	$arr_rolesetting['role_id'] = $role->getRoleId();
	$arr_rolesetting['rolename'] = $role->getRoleName();
	$arr_rolesetting['setting'] = $setting;
	$arr_rolesetting['url'] = GFUrl::getUrl()->projectURL($project, 'RoleEdit','','admin',array('role_id'=>$role->getRoleId()));
	$arr_rolesettings[] = $arr_rolesetting;
	
}

echo json_encode($arr_rolesettings);
?>
