<?php

/**
 * Copyright (c) 2005-2012 GForge, LLC
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('html/SiteTemplate.php');
require_once('external/Browser.php');
require_once('lib/ChatManager.php');


$session = Session::getSession();
if ($session == null || $session->isLoggedIn() == false) {
    require_once('exceptions/NotLoggedInException.php');
    throw new NotLoggedInException();
}

if (isset($_GET['section']) || isset($_GET['refid']) || isset($_GET['thread_id'])) {
    if (isset($_GET['thread_id'])) {
        //Get forum by thread id
        $forum = ChatManager::getForumByThreadId($_GET['thread_id']);
        // Go get the discussion thread
        $thread = ForumThreadPeer::retrieveByPk($_GET['thread_id']);
    } else{
        if (isset($_GET['section']) && isset($_GET['refid'])){
            $project = ProjectPeer::retrieveByPk($_GET['refid']);      
            //Get forum by section and ref id
            $forum = ChatManager::getForumByRefIdSection($_GET['section'], $_GET['refid'], $project->getProjectName());
            // Go get the discussion thread
            $thread = ForumThreadPeer::getNamedThread($forum, $session->getUser(), $project->getProjectName());     
        }else{
            require_once('exceptions/InvalidDataException.php');
            throw new InvalidDataException();
        }
    }    
    // Did we get a forum?
    if (is_null($forum)) {
        header("HTTP/1.0 404 Not Found", true, '404');
        return;
    }
    // Can this user read the forum?        
    if (!$forum->userCanRead()) {
        header("HTTP/1.0 403 Not Allowed", true, '403');
        return;
    }  
    // Did we find the thread?  
    if (is_null($thread)) {
        header("HTTP/1.0 404 Not Found", true, '404');
        return;
    }
    //Add user to chat windows        
    $var =  ChatManager::getChatWindow($forum, $thread);
    echo $var;
}



$Lang = Session::getSession()->getLanguage();

// Extra head element here for IE border-radius compat in IE9+
$params = array(
	'whoami' => Session::getSession()->getUser()->getUnixName(),
    'title' => 'GForge Discussions',
	'extra_heads' => array('<meta http-equiv="X-UA-Compatible" content="IE=edge" />'),
	'extra_css' => array(
		"/themes/gforge5/gf-chat.css",
		"/themes/gforge5/jquery.dropdown.css"
	),
	'extra_js' => array(
		"/jquery.a-tools-1.5.2.min.js",
		"/jquery.asuggest.js",
		"/ba-emotify.min.js",
		"/js/gf-emoticons.js",
		"/jquery.dropdown.js",
		"/jquery.ellipsis.js",
		"/spin.js",
		"/js/gf-chat.js"
	),
	'recent.tab' => $Lang->getText('Discuss.Recent'),
	'scrollback.title' => $Lang->getText('Discuss.Scrollback'),
	'scrollback.none.title' => $Lang->getText('Discuss.Scrollback.None'),
	'enableUploads' => true
);

$browser = new Browser();
$browser_name = strtolower($browser->getBrowser());
$browser_version = floatval($browser->getVersion());
$can_drag_drop = false;

//Determine if it's an HTML5 compliant browser and allow drag'n drop
if (($browser_name == 'firefox' && $browser_version >= 4) || ($browser_name == 'chrome')) {
	$can_drag_drop = true;
}

//If the safe browsing key does not exist, this feature is disabled
$safe_browsing = false;
try {
	$apiKey = GFConf::get('system.safeBrowsingKey');
} catch(Exception $e) {
	$apiKey = '';
}
if ($apiKey != '') {
	$safe_browsing = true;
}

$theme = new SiteTemplate();
$theme->headerStart($params);
include 'forms/Chat-tpl.php';
$theme->footerEnd($params);