<?php

/**
 * Copyright (c) 2005-2012 GForge, LLC
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('exceptions/PermissionDeniedException.php');
require_once('plugins/forum/lib/ForumThreadPeer.php');
require_once('plugins/forum/lib/ForumPeer.php');
require_once('html/SiteTemplate.php');

$kept_variables = array('action', 'thread_id');

$session = Session::getSession();
$theme = new SiteTemplate();

if ($session == null || $session->getUser() == null) {
	header("HTTP/1.0 403 Not Allowed", true, '403');
	return;
}

$Lang = $session->getLanguage();

$thread = null;
$forum = null;
if (isset($_GET['thread_id'])) {
	$thread_id = Validator::sanitizeInt($_GET['thread_id']);
	$thread = ForumThreadPeer::retrieveByPk($thread_id);
	if (!is_null($thread)) {
		$forum = $thread->getForum();
	}
}

if (is_null($thread) || is_null($forum)) {
	header("HTTP/1.0 404 Not Found", true, '404');
	return;
}

//Get user preference
$user = $session->getUser();
$sound_setting = 'Y';
$sound_icon = 'sound_on.png';

$activity_setting = 'Y';
$activity_icon = 'project_activity_on.png';

if (is_object($user)) {
	$sound_preference = $user->getPreference('chat-' . $thread_id . '-sound');
	if ($sound_preference != 'Y') {
		$sound_setting = 'N';
		$sound_icon = 'sound_off.png';
	}

	$activity_preference = $user->getPreference('chat-' . $thread_id . '-proj');
	if ($activity_preference != 'Y') {
		$activity_setting = 'N';
		$activity_icon = 'project_activity_off.png';
	}
}

$forum = $thread->getForum();
$arr_title = ForumPeer::getRelatedItemTitle($forum);
if (is_array($arr_title) && count($arr_title) > 0) {
	$title = $arr_title['title'];
	$title_full = $arr_title['title_full'];
} else {
	$title = 'unknown';
	$title_full = 'Could not find the related item for this chat';
}
$params = array(
	'thread_id' => $thread_id,
	'whoami' => $session->getUser()->getUnixName(),
	'tab_url' => "/gf/chat/?action=ChatTab&thread_id=$thread_id",
	'base_url' => "/gf/chat/",
	'sounds' => getSoundList($theme, $thread),
	'sound_setting' => $sound_setting,
	'sound_icon' => $sound_icon,
	'activity_setting' => $activity_setting,
	'activity_icon' => $activity_icon,
	'title' => $title,
	'title_full' => $title_full,
	'writable' => $forum->userCanPost() ? 'Y' : 'N'
);

include 'forms/ChatTab-tpl.php';

function getSoundList($theme, $thread) {
//Get default sounds path
	$stock_sounds_path = '/themes/' . $theme->getName() . '/sounds/chat/user_sounds/';

//Array of standard available sounds
	$arr_sounds = array(
		'Audience Applause' => $stock_sounds_path . 'applause.mp3',
		'Audience Booing' => $stock_sounds_path . 'booing.mp3',
		'Cricket' => $stock_sounds_path . 'cricket.mp3',
		'Laugh with Applause' => $stock_sounds_path . 'laugh_with_applause.mp3',
		'Magic Gong' => $stock_sounds_path . 'magic_gong.mp3',
	);


//Get sounds from "Sounds" folder
	$forum = $thread->getForum();
	if (is_object($forum)) {
		$project = ProjectPeer::retrieveByPk($forum->getRefId());

		if (is_object($project)) {
			//Get "Sounds" folder in this project
			$c = new Criteria();
			$c->add(DocmanFolderPeer::PROJECT_ID, $project->getProjectId());
			$c->add(DocmanFolderPeer::FOLDER_NAME, 'Sounds');
			$docman_folder = DocmanFolderPeer::doSelectOne($c);

			if (is_object($docman_folder)) {
				//Now, get the files associated to this folder
				$c = new Criteria();
				$c->add(DocmanFileFolderPeer::DOCMAN_FOLDER_ID, $docman_folder->getDocmanFolderId());
				$c->addJoin(DocmanFileFolderPeer::DOCMAN_FILE_ID, DocmanFilePeer::DOCMAN_FILE_ID);
				$docman_files = DocmanFilePeer::doSelect($c);

				foreach ($docman_files as $docman_file) {
					$latestfile = $docman_file->getLatestVersion()->getFile();

					if (preg_match('/(mp3|mpeg|mpg|wav)/', $latestfile->getFileType(), $arr)) {
						$url = GFUrl::getURL()->downloadURL(DocmanFileVersion::SECTION, $docman_file->getLatestVersion()->getDocmanFileVersionId(), $latestfile->getFilesystemId(), $latestfile->getFileNameSafe());
						$str_title = ucfirst($docman_file->getDescription());
						$str_title = preg_replace('/(\.mp3|\.mpeg|\.mpg|\.wav)/', '', $str_title);
						$arr_sounds[$str_title] = $url . '/open';
					}
				}
			}
		}
	}
	ksort($arr_sounds);
	return $arr_sounds;
}