<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('core/ProjectPeer.php');
require_once('session/Session.php');
require_once('core/RolePeer.php');
require_once('core/UserProject.php');
require_once('html/HTMLWidget.php');

require_once('html/MyTemplate.php');
$session = Session::getSession();
if (!$session->isLoggedIn()) {
	throw new NotLoggedInException();
}
$Language = $session->getLanguage();
$user = $session->getUser();
$template = new MyTemplate();
$params = array(
	'user'=>$user,
	'title' => Language::getSessText('MyActivity.Title'),
	'feedback' => isset($_REQUEST['feedback']) ? Validator::sanitizeString($_REQUEST['feedback']) : (isset($feedback) ? $feedback : ''),
	'action' => isset($action) ? $action : '',
	'selectedbottomtab' => 'activity',
	);

if (!isset($feedback)) {
	$feedback = '';
}

// get all projects for this user
$all_projects = $user->getAllProjects();

// Create an array with all the trackers (id=>name)
$projects = array();
$projects[0] = $Language->getText('MyActivity.AllProjects');
foreach($all_projects as $project) {
	$projects[$project->getProjectId()]=$project->getProjectName();
}
//an array of all project ids that the user is a member of
$allids=$user->getAllProjectIds();
if (count($allids) > 0) {
	$allids=implode(',',$user->getAllProjectIds());
} else {
	$allids=0;
}


// Save the project_id that came in the request, only if it's valid
$project_id = (isset($_REQUEST['project_id']) && 
			  $_REQUEST['project_id'] > 0 &&
			  array_key_exists($_REQUEST['project_id'], $projects) === true) ? intval($_REQUEST['project_id']) : 0;



$template->header($params);
$begin=((isset($_REQUEST['start_date'])) ? $_REQUEST['start_date'] : 0 );
$end=((isset($_REQUEST['end_date'])) ? $_REQUEST['end_date'] : 0 );

if ($begin=="0") {
	$begin = (time()-(30*86400));
} else {
	$begin = strtotime($begin);
}
if ($end=="0") {
	$end = time();
} else {
	$end=strtotime($end)+86399;
}
if ($begin > $end) {
	$endtmp=$end;
	$end=$begin;
	$begin=$endtmp;
}

// Check if we want to filter only the user's items
$showall = (isset($_REQUEST['showall']) && $_REQUEST['showall'] == 1) ? true : false;
$ungroup = (isset($_REQUEST['ungroup']) && $_REQUEST['ungroup'] == 1) ? true : false;

$ids = array();
$ids[] = 'scmcommit';
$ids[] = 'trackeritem';
$ids[] = 'forumthread';
$ids[] = 'wiki';
$ids[] = 'docman';
$ids[] = 'frspackage';
$ids[] = 'frsrelease';

$texts=array();
$texts['scmcommit']=$Language->getText('ProjectActivity.Commit');
$texts['trackeritem']=$Language->getText('ProjectActivity.TrackerItem');
$texts['forumthread']=$Language->getText('Discuss.WindowTitle');
$texts['wiki']=$Language->getText('ProjectActivity.Wiki');
$texts['docman']=$Language->getText('ProjectActivity.Docman');
$texts['frspackage']=$Language->getText('ProjectActivity.FrsPackage');
$texts['frsrelease']=$Language->getText('ProjectActivity.FrsRelease');

if (!isset($_REQUEST['show'])) {
	$show=$ids;
} else {
	$show=$_REQUEST['show'];
}

if(!is_array($show)) Throw new Exception('Invalid Data Passed to query');

foreach ($show as $showthis) {
	if (array_search($showthis,$ids) === false) {
		Throw new Exception('Invalid Data Passed to query');
	}
}

$multiselect=HTMLWidget::multiSelectFromAssoc('show[]',$texts,$show,'',false,null,false,null,5);

$con = Propel::getConnection(UserPeer::DATABASE_NAME);
$stmt = $con->createStatement();

if ($project_id > 0) {
	$sqlproject = "AND project_id='$project_id' ";
} else {
	$sqlproject = "AND project_id IN ($allids) ";
}

if (!$showall) {
	$sqlmine = "AND user_id='".$user->getUserId()."' ";
} else {
	$sqlmine = '';
}

//Copy show values (except forumthread) to a temp array to be used in sql queries
$sections = array();
foreach($show as $s) {
	if ($s != 'forumthread') {
		$sections[] = $s;
	} else if ($ungroup) {
		$sections[] = $s;
	}
}


/** This is a hack. The previous code was a hack too, but performance was ugly
 * We are using an union between activity_log and a grouping of discussions (this is needed by the paginator)
 *
 * the mapping between fields is as follows (activity_log => discussion query over activity_log)
 * user_id => 0
 * 0 => message_count
 * activity_date => activity_date::date (for proper groupping)
 * 0::int => posters_count
 * activity_type => activity_type
 * description => ''::text
 * project_id => project_id
 * section => section
 * ref_id => ref_id
 * url => ''::text url
 *
 */
$myTimezone = $user->getTimezone();
$myTimezone = (empty($myTimezone)) ? 'UTC' : $myTimezone;
//set all following db queries to user timezone
$rs = $stmt->executeQuery("SET TIMEZONE TO '$myTimezone'");
/*
NOTE ABOUT THE BETWEEN STATEMENT: the activity_date field has as type
 'timestamp with timezone'. to_timestamp() expects as parameter a unix epoc
 (a.e. unix timestamp) and also returns a 'timestamp with time zone' field.
 So, we set before any queries the user timezone with the query above to
 assure that all 3 fields in comparison work in the same timezone. (neat uh?)
 - Hector.
Source: http://www.postgresql.org/docs/8.1/static/functions-formatting.html
*/

// pulling 'activity_date' will return date and time
$base_sql_non_message = "SELECT user_id, 0 AS messages_count,
		activity_date,
		0::int AS posters_count, activity_type, project_id, description, section, ref_id, url
		FROM activity_log WHERE
		activity_date BETWEEN to_timestamp($begin) AND to_timestamp($end)
		$sqlmine
		AND section IN ('".implode("','",$sections)."')
		$sqlproject";

if (in_array('forumthread', $show) && !$ungroup) {
	// pulling activity_date::date returns YYYY-MM-DD, time is not included.
	// GROUP BY requires it this way to gather messages from the same date in
	// a single row. Someone else found out about this trick - Hector.
	$base_sql_forum_totals = "UNION SELECT 0 as user_id, count(ref_id::text || project_id::text) as messages_count,
		activity_date::date,
		count( DISTINCT user_id) as posters_count,activity_type, project_id, ''::text as description, section, ref_id, ''::text as url 
		FROM activity_log WHERE
		activity_date BETWEEN to_timestamp($begin) AND to_timestamp($end)
		$sqlmine 
		AND section = 'forumthread'
		$sqlproject 
		AND user_id <> 100
		AND activity_type='update'
		GROUP BY activity_date::date, activity_type,project_id,section,ref_id";
}

$count_sql="SELECT COUNT(*) FROM ( $base_sql_non_message $base_sql_forum_totals ) tmp";
$rs = $stmt->executeQuery($count_sql, ResultSet::FETCHMODE_NUM );
$rs->next();
$total = $rs->getInt(1);

$paginator = new Paginator('myactiv', -1, $total, 'activity_date', array('activity_date'=>''));

$display_sql = "$base_sql_non_message $base_sql_forum_totals {$paginator->getSQL()}";
$rs = $stmt->executeQuery($display_sql);
?>

<script language="javascript" type="text/javascript" src="/datetimepicker.js">
</script>
<br />
<form action="/gf/my/activity/" method="post">
<table border="0" cellspacing="0" cellpadding="3" bgcolor="#E0E0E0">
<tr>
	<td><strong><?php echo $Language->getText('MyActivity.FilterByProject') ?></strong></td>
	<td><strong><?php echo $Language->getText('ProjectActivity.HeaderActivity') ?></strong></td>
	<td><strong><?php echo $Language->getText('ProjectActivity.StartDate') ?></strong></td>
	<td><strong><?php echo $Language->getText('ProjectActivity.EndDate') ?></strong></td>
	<td rowspan="3" valign="middle"><?php echo HTMLWidget::button("submit", $Language->getText('Common.Submit'), "submit"); ?></td>
</tr>
<tr>
	<td valign="top"><?php echo HTMLWidget::selectBoxFromAssoc($projects,'project_id',$project_id, false, false) ?></td>
	<td rowspan="2" valign="top"><?php echo $multiselect; ?></td>
	<td valign="top"><input id="startdate" name="start_date" value="<?php echo date('M d Y',$begin); ?>" size="11" maxlength="11" /><a href="javascript:NewCal('startdate','MMMddyyyy')"><img src="/themes/gforge5/images/cal.gif" width="16" height="16" border="0" alt="Pick a date"/></a></td>
	<td valign="top"><input id="enddate" name="end_date" value="<?php echo date('M d Y',$end); ?>" size="11" maxlength="11" /><a href="javascript:NewCal('enddate','MMMddyyyy')"><img src="/themes/gforge5/images/cal.gif" width="16" height="16" border="0" alt="Pick a date"/></a></td>
</tr>
<tr>
	<td>&nbsp;</td>
	<td colspan="2" align="right"><strong><?php echo $Language->getText('MyActivity.IncludeAll') ?></strong>
	<input type="checkbox" name="showall" value="1" <?php if ($showall) echo 'checked' ?> /> <br />
	<strong><?php echo 'Don\'t group discussions' ?></strong>
		<input type="checkbox" name="ungroup" value="1" <?php if ($ungroup) echo 'checked' ?> />
	</td>
</tr>
</table>
</form>
<br />
	<?php

	$theader=array();
	$theader[]=$Language->getText('ProjectActivity.HeaderTime');
	$theader[]=$Language->getText('ProjectActivity.HeaderActivity');
	if ($project_id == 0) {
		if ($showall == true) {
			$theader[]=$Language->getText('MyActivity.HeaderProject');
		} else {
			$theader[] = $paginator->insertLimitOptions($Language->getText('MyActivity.HeaderProject'));
		}
	}
	if ($showall == true) {
		$theader[]=$paginator->insertLimitOptions($Language->getText('ProjectActivity.HeaderPerson'));
	}

	echo $template->tableTop($theader);

	$j=0;
	$last_day=0;

	while ($rs->next()) {
		$j++;
		$arr = $rs->getRow();
		$myDate = date('Y-M-d', strtotime($arr['activity_date']));
		if ($last_day != $myDate) {
			echo $template->tableFullRow($myDate);
			$last_day=$myDate;
		}
		$project = ProjectPeer::retrieveByPk($arr['project_id']);
		$user = UserPeer::retrieveByPk($arr['user_id']);
		$userlink='';
		if (is_object($user)) {
			$userlink=UserPeer::retrieveByPk($arr['user_id'])->getUserLink();
		}

		$projectlink='<a href="' . GFUrl::getUrl()->projectURL($project) . '" >' .  $project->getProjectName() . '</a>';

		switch ($arr['section']) {
			/*
			case 'projectchat': {
				$activity_desc=$Language->getText('ProjectActivity.ProjectChatDescription', array($arr['user_id'], $arr['total'], $arr['description2']));
				$forum_id = $arr['subref_id'];
				$url = '<a href="'.GFUrl::getURL()->projectURL($project, 'ForumBrowse', 'forum', '', array('forum_id' => $forum_id)).'">'.$activity_desc.'</a>';
				break;
			}
			case 'chat': {
				if ($arr['description1'] == 'projectchat') {
					$activity_desc=$Language->getText('ProjectActivity.ProjectChatDescription', array($arr['user_id'], $arr['total'], $arr['description2']));
					$forum_id = $arr['subref_id'];
					$url = '<a href="'.GFUrl::getURL()->projectURL($project, 'ForumBrowse', 'forum', '', array('forum_id' => $forum_id)).'">'.$activity_desc.'</a>';
				} else {
					$plugin_results = ActionManager::notify("CHAT_ACTIVITY_SUMMARY", $arr);
					foreach ($plugin_results as $result) {
						$url = $result->getResult();
						if ($url != '') {
							break;
						}
					}					
				}
				break;
			}*/
			case 'forumthread'; {
				if($ungroup) {
					$arr['ungroup'] = true;
				} else {
					$arr['ungroup'] = false;
				}
			}
			default: {
				$plugin_results = ActionManager::notify("PROJECT_EVENT_PULL", $arr);
				foreach ($plugin_results as $result) {
					$url = $result->getResult();
					if ($url != '') {
						break;
					}
				}
				break;
			}
		}
		$trow=array();
		if ($arr['section'] == 'forumthread') {
			$trow[]='';
		} else {
			$trow[]='&nbsp;&nbsp;&nbsp;'.date('H:i:s',strtotime($arr['activity_date']));
		}
		$trow[]=$url;
		if ($project_id == 0) {
			$trow[]=$projectlink;
		}
		if ($showall == true) {
			$trow[]=$userlink;
		}
		echo $template->tableRow($trow, array(), false);
		unset($trow);
	}
	if ($j==0) {
		echo $template->tableFullRow($Language->getText('ProjectActivity.NoneFound'));
	}
	else {
		//echo $template->listPaginatorRow($paginator);
	}
	echo $template->tableBottom();

	if(isset($paginator) && is_object($paginator)) {
		echo '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
	}


$template->footer();
?>
