<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code shows a form for editing News
//
require_once('lib/core/WallPost.php');
require_once('lib/core/WallPostPeer.php');
require_once('lib/core/WallPostComment.php');
require_once('lib/core/WallPostCommentPeer.php');
require_once('exceptions/PermissionDeniedException.php');

$session = Session::getSession();
$Lang = $session->getLanguage();
$user = $session->getUser();
$project = GFUrl::getUrl()->getProject();

$errors = array();

$id = isset($_REQUEST['id']) ? Validator::sanitizeInt($_REQUEST['id']) : 0;

$wallpost = WallPostPeer::retrieveByPk($id);

if (!is_object($wallpost) || !$wallpost->userCanDelete()) {
	echo "You don't have permission to delete this post";
	exit;
}

//Delete wall post comments
$c = new Criteria();
$c->add(WallPostCommentPeer::WALL_POST_ID, $id);
$wallpostcomments = WallPostCommentPeer::doSelect($c);

foreach($wallpostcomments as $wallpostcomment) {
	$wallpostcomment->delete();
}

try {
	$wallpost->delete();
} catch (InvalidDataException $ex) {
	$errors = array_merge($errors, $ex->getErrors());		
}


//if errors found, the wall post wasn't saved, so show the page again and display errors
if (sizeof($errors) > 0)  {
	$feedback = implode('<br />',$errors);
	echo $feedback;
} else {
	
	//Associates the attached file, if any
	$c = new Criteria();
	$c->add(FilesystemPeer::REF_ID, $id);
	$c->add(FilesystemPeer::SECTION, "messagewall");
	
	$attach = FilesystemPeer::doSelectOne($c);
	
	if (is_object($attach)) {
		try {
			$attach->delete();
		} catch (InvalidDataException $ex) {
			$errors = array_merge($errors, $ex->getErrors());		
		}
	}
	
	//Delete target user associations
	$c = new Criteria();
	$c->add(AssociationPeer::REF_ID, $id);
	$c->add(AssociationPeer::SECTION, "messagewall");
	
	$associations = AssociationPeer::doSelect($c);
	
	foreach($associations as $association) {
		try {
			$association->delete();
		} catch (InvalidDataException $ex) {
			$errors = array_merge($errors, $ex->getErrors());		
		}
	}

}

if (sizeof($errors) > 0)  {
	$feedback = implode('<br />',$errors);
	echo $feedback;
} else {
	echo "OK";
}

?>