<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

//
// This code edits a Role and saves it to the 
// database
//
require_once('core/Role.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');
require_once('exceptions/MissingDataException.php');

$project = GFUrl::getUrl()->getProject();
if (!$project->userIsAdmin()) {
	// Only project admins can change member information
	throw new PermissionDeniedException();
}

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

if (!array_key_exists('data', $_POST)) {
	throw new MissingDataException('Missing role data');
}

$Lang = Language::getSessLang();

try {
	$data = $_POST['data'];
	
	// Save project admin data
	if (isset($data['projectstatus'])) {
		if ($data['projectstatus'][0] == 1) {
			$project->setIsPublic(true);
		} else {
			$project->setIsPublic(false);
		}
		$project->save();
	}
	
	// Now inform the plugins
	$params = array('project' => $project, 'data' => $data);
	ActionManager::notify('UPDATE_OBSERVER_SETTINGS', $params);

		
	
	// Do the audit trail
	$audit = new AuditTrail();
	$audit->setUser(Session::getSession()->getUser());
	$audit->setSection(Project::SECTION);
	$audit->setRefId($project->getProjectId());
	$audit->setFieldName('observersetting');
	$audit->setNewValue('');
	$audit->setOldValue('');
	$audit->save();

	if(GFConf::get('system.useDaemon')){
		GFDaemon::notify('DAEMON_UPDATE_OBSERVER', $project->getUnixName());
	}
	$url = GFUrl::getUrl();
	GFUrl::redirect($url->currentBaseURL('index'), array('feedback' => $Lang->getText('ProjectAdmin.Role.Updated')));
} catch (InvalidDataException $ex) {
	$errors = $ex->getErrors();
	$feedback = implode('<br />', $errors);
	include('ObserverEdit.php');
}

?>
