<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Federico Tarantini federico@gforgegroup.com
 * @date 2006-12-01
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('core/ProjectPeer.php');
require_once('html/templates/ProjectAdminTemplate.php');
require_once('html/Paginator.php');
require_once('html/HTMLWidget.php');

$session = Session::getSession();
$user = $session->getUser();
$url = GFUrl::getUrl();
$project = $url->getProject();
$parent_project = $project->getParentProject();

if (!$project->userIsAdmin()) {
        throw new PermissionDeniedException();
}

$template = new ProjectAdminTemplate();
$params = array(
                'selectedbottomtab'     => 'project_admin',
                'project' => $project,
                'title' => Language::getSessText('ProjectAdmin.ParentProjectTitle'),
                'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : ''
        );
$template->header($params);

echo '<p><div class="infobox">'.Language::getSessText('ProjectAdmin.ParentProjectSelectTitle').'</div></p>';

if($parent_project != null) {
	$headers = array(Language::getSessText('ProjectAdmin.ParentProject'),
			 Language::getSessText('ProjectAdmin.ParentProjectRemove')
			);
	echo $template->tableTop($headers);
	$trow=array();
	$trow[] = $parent_project->getProjectName();
	$trow[] = '<a href="'.$url->currentBaseUrl('ParentProjectAction', array('do' => 'remove')).'">'.Language::getSessText('ProjectAdmin.ParentProjectRemove').'</a>';
	echo $template->tableRow($trow);
} else {
	//$headers = array(Language::getSessText('ProjectAdmin.ParentProject'));
	
	$columns = array(ProjectPeer::PROJECT_NAME => Language::getSessText('ProjectAdmin.ParentProject'));
	
	$crit = new Criteria();
	$crit->add(ProjectPeer::PROJECT_ID, $project->getProjectId(), Criteria::NOT_EQUAL);
	$crit->addAnd(ProjectPeer::PROJECT_ID, array_values($project->getAllSubProjectIds()), Criteria::NOT_IN);	
	if (!$user->isSiteAdmin()) {
		$crit->addAnd(ProjectPeer::PROJECT_ID, array_values($user->getAllProjectIds()), Criteria::IN);
	}	
	$count = ProjectPeer::doCount($crit);
	$paginator = new Paginator('prj_admin', -1, $count, ProjectPeer::PROJECT_NAME, $columns);
	$paginator->addPageClauses($crit);
	$projects = ProjectPeer::doSelect($crit);
	$headers = $paginator->getSortableTableHeaders($template);
	//echo $paginator->showLimitOptions();
	echo $template->tableTop($headers);
	$count = 0;
	foreach ($projects as $parent_proj) {
		if ($parent_proj->userIsAdmin()) {
			$trow=array();
			$trow[] = '<a href="'.$url->currentBaseUrl('ParentProjectAction', array('id' => $parent_proj->getProjectId())).'">'.$parent_proj->getName().'</a>';
			echo $template->tableRow($trow);
			$count++;
		}
	}
	if ($count == 0) {
		echo $template->tableFullRow( Language::getSessText('ProjectAdmin.ParentProjectsNone') );
	} else {
		//echo $theme->listPaginatorRow($paginator);
	}
		
	//echo $theme->tableBottom($this->getForumButtons($forum));
}
echo $template->tableBottom();

if(isset($paginator) && is_object($paginator)) {
	echo '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
}


$template->footer();
?>
