<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version $Id$
 * @author Daniel A. Pérez	daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('core/UserProject.php');
require_once('core/UserProjectRole.php');
require_once('core/ProjectJoinRequestPeer.php');
require_once('core/ProjectJoinRequest.php');

$project = GFUrl::getUrl()->getProject();
$project_id = $project->getProjectId();
$Lang = $session->getLanguage();

if (!$project->userIsAdmin()) {
	// Only project admins can change member information
	throw new PermissionDeniedException();
}

if (isset($_POST['unix_name']) && strtolower($_POST['unix_name']) == 'nobody'){
	$_POST = array();
	$feedback = $Lang->getText('ProjectAdmin.UserNotExists');
}

if (array_key_exists('adduser',$_POST)) {
	if (!array_key_exists('unix_name', $_POST) || strlen($_POST['unix_name']) == 0) {
		$feedback = $Lang->getText('ProjectAdmin.SpecifyUnixName');
	} elseif (!array_key_exists('role_id', $_POST) || count($_POST['role_id']) == 0) {
		$feedback = $Lang->getText('ProjectAdmin.SpecifyRole');
	} else {
		// add the user to the project
		$unix_name = strtolower($_POST['unix_name']);
		$roles = $_POST['role_id'];
		$user = UserPeer::retrieveByUnixName($unix_name);
		if (is_object($user)){
			if($user->getStatus() == User::STATUS_ACTIVE){			
				//check if there is a join request
				$crit = new Criteria();
				$crit->add(ProjectJoinRequestPeer::PROJECT_ID, $project->getProjectId());
				$crit->add(ProjectJoinRequestPeer::USER_ID, $user->getUserId());
				$join_request = ProjectJoinRequestPeer::doSelectOne($crit);

				//if there's a join request placed
				$send_accepted_message = false;
				if ( is_object($join_request) ) {
					$send_accepted_message = true;
					$join_request->delete();				
				}

				// first check if the user already exists
				$c = new Criteria();
				$c->add(UserProjectPeer::USER_ID,$user->getUserId());
				$c->add(UserProjectPeer::PROJECT_ID,$project->getProjectId());
				$already_exists = UserProjectPeer::doSelectOne($c);
				if ($already_exists instanceof UserProject) {
					$feedback = $Lang->getText('ProjectAdmin.UserAlreadyExists');
				} else {
					$user_project = new UserProject();
					$user_project->setUser($user);
					$user_project->setProject($project);
					$role_warnings = array();
					foreach ($roles AS $role) {
						$upr=new UserProjectRole();
						$upr->setRoleId($role);
						$user_project->addUserProjectRole($upr);
						$role = $upr->getRole();
						if (!$role->matchesMemberFieldRules($user)) {
							/** @var TrackerExtraField $extraField */
							$extraField = $role->getRelatedMemberField();
							$params = array($role->getName(), $extraField->getName(),
								$extraField->getTracker()->getTrackerName(), $user->getUnixName());
							if ($user->getIsGroup()) {
								//User is a group, but field only allows regular users

								$role_warnings[] =$Lang->getText('ProjectAdmin.Role.OnlyUsers', $params);
							} else {
								//User is not a group, but field only allows groups
								$role_warnings[] = $Lang->getText('ProjectAdmin.Role.OnlyGroups', $params);
							}

						}
					}
					
					$maxmembers = GFConf::varExists('system.licenseMaxMembers') ? GFConf::get('system.licenseMaxMembers') : 0;
					
					//if the user we are adding is a usergrup, and the license limits the 
					//number of members, verify if the available licenses are enough
					if ($user->getIsGroup() == true && maxmembers > 0) {
						$stmt = Propel::getConnection()->createStatement();
						
						//Count users in usergroup, that are not already in another usergroup associated to some project
						//and that are not individually associated to any other project
						$sql = 'SELECT count(*) AS count FROM usergroup_user WHERE usergroup_id = ' . $user->getUserId() . '
										AND member_user_id NOT IN
											(SELECT user_id FROM user_project)
										AND member_user_id NOT IN
											(SELECT member_user_id FROM user_project, usergroup_user
											 WHERE
											 user_project.user_id = usergroup_user.usergroup_id)
										';

						$rs = $stmt->executeQuery($sql);
						$rs->next();
						$users_in_usergroup = $rs->getInt('count');  //users in the usergroup that will consume a license when
																												 // assgning the usergroup to the project

						//Get number of licenses already used
						$sql = 'SELECT count(*) AS count FROM "user" WHERE user_id IN (
							SELECT user_id FROM user_project NATURAL JOIN "user" WHERE "user".is_group=false
							UNION
							SELECT member_user_id AS user_id FROM usergroup_user WHERE usergroup_id IN (
							SELECT user_id FROM user_project NATURAL JOIN "user" WHERE "user".is_group=true
							)
						)';

						$rs = $stmt->executeQuery($sql);
						$rs->next();
						$licenses_used = $rs->getInt('count');	//number of licenses already used
						
						//if the number of licenses already used plus the number of licensed that adding the usergroup
						//will consume is greater than the max number of licenses allowed, throw an exception
						if (($licenses_used + $users_in_usergroup) > $maxmembers) {
							throw new InvalidLicenseException('Maximum licensed members exceeded. Contact GForge Group for help.');
						}
						
					}

					if ($send_accepted_message == true) {
						$join_request->sendAcceptMessage();
					}

					$user_project->save();
					$feedback = $Lang->getText('ProjectAdmin.UserAdded');

					foreach ($role_warnings as $warning) {
						$feedback .= '<br>'.$warning.'<br />';
					}

				}
			} else {
				$feedback = $Lang->getText('ProjectAdmin.UserNotActive');
			}
		} else {
			$feedback = $Lang->getText('ProjectAdmin.UserNotExists');
		}
	}

	if(GFConf::get('system.useDaemon')){
		GFDaemon::notify('DAEMON_UP_SAVE', $project->getUnixName());
		$projectChilds = $project->getChildProjects();
		if (count($projectChilds)) {
			foreach ($projectChilds as $project) {
				GFDaemon::notify('DAEMON_UP_SAVE', $project->getUnixName());
			}
		}
	}
}

if (array_key_exists('rmuser',$_POST)) {
	// remove the user from the project
	$user_id = $_POST['user_id'];
	$c = new Criteria();
	$c->add(UserProjectPeer::USER_ID,$user_id);
	$c->add(UserProjectPeer::PROJECT_ID,$project_id);
	$user_project = UserProjectPeer::doSelectOne($c);
	if (is_object($user_project)) {
		$user_project->delete();
		$feedback = $Lang->getText('ProjectAdmin.UserDeleted');
		if(GFConf::get('system.useDaemon')){
			GFDaemon::notify('DAEMON_UP_DELETE', $project->getUnixName());
			$projectChilds = $project->getChildProjects();
			if (count($projectChilds)) {
				foreach ($projectChilds as $project) {
					GFDaemon::notify('DAEMON_UP_SAVE', $project->getUnixName());
				}
			}
		}
	} else {
		$feedback = $Lang->getText('ProjectAdmin.UserNotExists');
	}
}

if (array_key_exists('updateuser',$_POST)) {
	// update a user
	$user_id = $_POST['user_id'];
	$user = UserPeer::retrieveByPk($user_id);
	$roles = $_POST['role_id'];
	$c = new Criteria();
	$c->add(UserProjectPeer::USER_ID,$user_id);
	$c->add(UserProjectPeer::PROJECT_ID,$project_id);
	$user_project = UserProjectPeer::doSelectOne($c);
	if (is_object($user_project)) {
		$existingrolesids = array();
		$existingroles = $user_project->getUserProjectRoles();
		foreach ($existingroles as $exrole) {
			$existingrolesids[] = $exrole->getRoleId();
		}
		
		// We delete and insert the roles separately so the audit trail
		// has the correct values
		
		$deleted_roles_ids = array_diff($existingrolesids, $roles);
		foreach ($deleted_roles_ids as $deleted_role_id) {
			$c = new Criteria();
			$c->add(UserProjectRolePeer::USER_PROJECT_ID, $user_project->getUserProjectId());
			$c->add(UserProjectRolePeer::ROLE_ID, $deleted_role_id);
			$user_project_role = UserProjectRolePeer::doSelectOne($c);
			if (is_object($user_project_role)) {
				$user_project_role->delete();
			}
		}
		
		$new_roles_ids = array_diff($roles, $existingrolesids);
		foreach ($new_roles_ids as $new_role_id) {
			$user_project_role = new UserProjectRole();
			$user_project_role->setUserProjectId($user_project->getUserProjectId());
			$user_project_role->setRoleId($new_role_id);
			$user_project_role->save();
			$role = $user_project_role->getRole();
			if (!$role->matchesMemberFieldRules($user)) {
				/** @var TrackerExtraField $extraField */
				$extraField = $role->getRelatedMemberField();
				$params = array($role->getName(), $extraField->getName(),
					$extraField->getTracker()->getTrackerName(), $user->getUnixName());
				if ($user->getIsGroup()) {
					//User is a group, but field only allows regular users

					$role_warnings[] =$Lang->getText('ProjectAdmin.Role.OnlyUsers', $params);
				} else {
					//User is not a group, but field only allows groups
					$role_warnings[] = $Lang->getText('ProjectAdmin.Role.OnlyGroups', $params);
				}

			}

		}

//		$user_project->save();
		$warnings = '';
		foreach($role_warnings as $warning) {
			$warnings .= $warning.'<br />';
		}
		$feedback = $Lang->getText('ProjectAdmin.UserUpdated') . '<br />'.$warnings;

		$project->purgePermissions();
		if (!$project->userIsAdmin()) { // special case where we are removing ourselves from admin, go to summary page
			GFUrl::redirect(GFUrl::getUrl()->projectURL($project,'','','',array('feedback'=>$feedback)));			
		}
	} else {
		$feedback = $Lang->getText('ProjectAdmin.UserNotExists');
	}


	if(GFConf::get('system.useDaemon')){
		GFDaemon::notify('DAEMON_UP_SAVE', $project->getUnixName());
		$projectChilds = $project->getChildProjects();
		if (count($projectChilds)) {
			foreach ($projectChilds as $project) {
				GFDaemon::notify('DAEMON_UP_SAVE', $project->getUnixName());
			}
		}
	}
}

GFUrl::redirect(GFUrl::getUrl()->currentBaseURL('UserRole',array('feedback'=>$feedback)));


?>
