<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code creates a new Association and saves it to the 
// database
//
require_once('core/Association.php');
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');
require_once('lib/html/association.php');
require_once('plugins/docman/lib/FastJSON.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$Association = new Association();

$Lang = Language::getSessLang();

$errors = array();

try {
	$Association->setSection( Validator::sanitizeString($_POST['from_section']) );
	$Association->setRefId( Validator::sanitizeInt($_POST['from_ref_id']) );
	$Association->setToSection( Validator::sanitizeString($_POST['to_section']) );
	$Association->setToRefId( Validator::sanitizeInt($_POST['to_ref_id']) );
	$Association->setComment( Validator::sanitizeString($_POST['comment']) );
	$ajax_call = isset($_POST['ajax']) ? $_POST['ajax'] : 0;
	$return_to = $_POST['return_to'];

	$Association->save();
	
	if ($ajax_call) {
		header('HTTP/1.1 200');
		$from_element_name = isset($_POST['from_element_name']) ? Validator::sanitizeString($_POST['from_element_name']) : '';
		$theme = Session::getSession()->getTheme();
		$project = GFUrl::getUrl()->getProject();
		$html = renderAssociationList($Association->getSection(),$Association->getRefId(),true,$theme,$project,true,  "docmanFile.addAssociation(" . Validator::sanitizeInt($_POST['from_ref_id']) . "  , '" . $from_element_name . "')");
		$arr_feedback = array();
		$arr_feedback["html"] = $html;
		echo FastJSON::encode($arr_feedback);
		exit;
	}
	
	if (strpos($return_to,'?') === false) {
		$q='?';
	} else {
		$q='';
	}
	header("Location: ".$return_to.$q.'&feedback='. urlencode($Lang->getText('Association.AddFeedback')));
} catch (InvalidDataException $ex) {
	$errors = array_merge($errors, $ex->getErrors());		// attach object errors to validation errors
	$feedback = '* '.implode('<br />* ', $errors);
	if ($ajax_call) {
		header('HTTP/1.1 401');
		$arr_feedback = array();
		$arr_feedback["error_message"] = $feedback;
		echo FastJSON::encode($arr_feedback);
		exit;
	} else {
		include('AssociationAdd.php');
	}

}
?>
