<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Julian Yerfino	jyerfino@gforgegroup.com	
 * @date 2008-01-04
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

include ("lib/external/jpgraph/jpgraph.php");
include ("lib/external/jpgraph/jpgraph_line.php");
require_once('lib/core/DiskUsage.php');
require_once('lib/core/DiskUsagePeer.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

$Project = GFUrl::getUrl()->getProject();
$project_id = $Project->getProjectId();

// Check if we want to filter by package
if (array_key_exists('start_date', $_REQUEST)) {
	$begin = Validator::sanitizeDate($_REQUEST['start_date']);
}
else {
	$begin = date('Y-m-d', 0);
}

if (array_key_exists('end_date', $_REQUEST)) {
	$end = Validator::sanitizeDate($_REQUEST['end_date']);
}
else {
	$end = date('Y-m-d', time());
}

$subprojects=((isset($_REQUEST['subprojects']) && $_REQUEST['subprojects']==true) ? true : false );

$c = new Criteria();
$c->addOr(DiskUsagePeer::PROJECT_ID, $project_id);
$c->add(DiskUsagePeer::CHECK_DATE, DiskUsagePeer::CHECK_DATE . " >= to_date('$begin','YYYY-MM-DD') AND " . DiskUsagePeer::CHECK_DATE .  " <= to_date('$end','YYYY-MM-DD')", Criteria::CUSTOM);
$c->addAscendingOrderByColumn(DiskUsagePeer::CHECK_DATE);
$c->addAsColumn('bytes', 'SUM(' . DiskUsagePeer::BYTES_USED . ')');
$c->addSelectColumn(DiskUsagePeer::CHECK_DATE);
$c->addGroupByColumn(DiskUsagePeer::CHECK_DATE);

if ($subprojects) {
	$project = ProjectPeer::retrieveByPk($project_id);
	$sub_ids = $project->getAllSubprojectIds();
	foreach ($sub_ids as $id) {
		$c->addOr(DiskUsagePeer::PROJECT_ID, $id);
	}
}

$rs = DiskUsagePeer::doSelectRS($c);

$total_results = $rs->getRecordCount();
$num_x_entries = 18;	// Number of entries on the x axis
$usage = array();
$data = array();
$item_count = array();
$item_countf = 0.0;

for ($i=0; $i<$num_x_entries; $i++) {
	$usage[$i] = 0;
	$item_count[$i] = intval($total_results/$num_x_entries);
	$item_count[$i] += (($total_results-$item_count[$i]*$num_x_entries) > $i) ? 1:0;
}

// Make $num_x_entries groups, each group will have a mean value
$rs->next();
$rsdata = $rs->getRow();
for ($g=0; $g<$num_x_entries; $g++) {
	if ($item_count[$g]>0) {
		$label = $rsdata[0];
	
		for ($i=0; $i<$item_count[$g]; $i++) {
			$usage[$g] += $rsdata[1];
			$rs->next();
			$rsdata = $rs->getRow();
		}			
		$data[$label] = $usage[$g] / ($item_count[$g]*1024*1024);	// Average in MB
	}
}


if (count($data) == 0 ) { 
	// Fill 18 months values
	$months=array();
	$startdate=(time()-1.5*365*60*60*24);
	$addmonth=intval(date('m',$startdate));
	$startyear=intval(date('Y',$startdate));
	$addyear=0;
	$months["".($startyear+$addyear).'-'.($addmonth).""]=0;
	for ($i=0; $i<18; $i++) {
		$addmonth++;
		if ($addmonth > 12) {
			$addmonth=1;
			$addyear++;
		}
		$months[]="".($startyear+$addyear).'-'.($addmonth)."";
		$data[$months[$i]] = 0;
	}
}
else if (count($data) == 1 ) { 
	$prev_date = array_keys($data);
	$new_time = strptime($prev_date[0],"%Y-%m-%d");
	$new_time = mktime(0,0,0,$new_time['tm_mon']+1,$new_time['tm_mday'],$new_time['tm_year']+1900)-60*60*24;
	$new_time = strftime("%Y-%m-%d",$new_time);
	$new_data = array($new_time => 0, $prev_date[0] => $data[$prev_date[0]]);
	$data = $new_data;
	
}

// Create the graph. These two calls are always required
$graph = new Graph(400, 250);
$graph->title->SetFont(FF_DEFAULT,FS_BOLD);
$graph->title->Set($Lang->getText('ProjectReporting.DiskUsageByDate'));
$graph->SetScale('textlin');
$graph->setBox(false);
$graph->setFrame(false);
$graph->img->SetMargin(60,20,30,80);
$graph->yaxis->scale->SetGrace(0);
$graph->yaxis->SetLabelMargin(1);
$graph->yaxis->SetTitle($Lang->getText('ProjectReporting.UsageInMB'), 'middle');
$graph->yaxis->title->SetFont(FF_DEFAULT,FS_NORMAL, 7);
$graph->yaxis->SetTitleMargin(50);
$graph->xaxis->SetTickLabels(array_keys($data));
$graph->xaxis->SetLabelAngle(90);
$lineplot = new LinePlot(array_values($data));
$lineplot->SetFillColor('#ADD8E6');
$graph->Add($lineplot);

if (ob_get_level()) ob_end_clean();
$graph->Stroke();
 
