<?php
/**
 * Gforge 5.0 Project Summary.
 *
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: gf 386 2006-02-01 19:42:23Z marcelo $
 * @author
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('html/ProjectTemplate.php');
require_once('lib/core/ProjectPeer.php');
require_once('lib/core/ProjectPluginPeer.php');
require_once('lib/core/RolePeer.php');
require_once('lib/core/Role.php');
require_once('lib/core/UserPeer.php');
require_once('lib/core/TroveLinkPeer.php');
require_once('lib/core/TroveCategoryPeer.php');
require_once('plugins/tracker/lib/TrackerQueryPeer.php');
require_once('reporting/BurndownWidget.php');

$project = GFUrl::getUrl()->getProject();
if (!is_object($project)) {
	throw new Exception('<h2>' . Language::getSessText('Exception.InvalidProjectNoName') . '</h2>');
}

$project_name = $project->getUnixName();

function render_project_tree($projarr,$parent_id,$indent) {
	$indent++;
	if (isset($projarr[$parent_id])) {
		foreach ($projarr[$parent_id] as $prj) {
			echo str_repeat('&nbsp;',$indent*6).$prj->getProjectLink().'<br />';
			render_project_tree($projarr,$prj->getProjectId(),$indent);
		}
	}
}

$template = new ProjectTemplate();
$params = array(
		'selectedbottomtab'	=> 'summary',
		'project' => $project,
		'title'=>'Home',
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : '')
	);
$template->header($params);
$theme = Session::getSession()->getTheme(); // get the theme

// Get the home page contents for this project
$contents = $project->getHomepage()->getFileData(false);

// Process any wiki words that might be in there
$wiki = new Wiki('project', $project->getProjectId(), $theme, $project);
$contents = $wiki->processExternalContent($contents, '');

$str = '';
if (stripos($contents, '%%CLASSIC%%') !== false) {
	$prs = ActionManager::notify("BUILD_PROJECT_SUMMARY",array('project'=>$project,'theme'=>$template));
	$class=0;
	$split=false;
	if (count($prs) > 0) {
		$str .= $theme->infoTableTop(Language::getSessText('Project.GeneralSections'));
		$str .= '<table border="0" width="100%"><tr><td valign="top" width="50%">';
		foreach ($prs as $result) {
			if ($class >= ($prs->count()/2)) {
				if (!$split) {
					$str .= '</td><td valign="top" width="50%">';
				}
				$split=true;
			}
			$str .= $result->getResult();
			$class++;
		}
		$str .= '</td></tr></table>';
		$str .= $theme->infoTableBottom();
		$contents = str_replace('%%CLASSIC%%', $str, $contents);
	}
}
if (stripos($contents, '%%NEWS%%') !== false) {
	$news = '';
	$news_prs = ActionManager::notify("BUILD_PROJECT_NEWS_SUMMARY",array('project'=>$project,'theme'=>$template));
	foreach ($news_prs as $result) {
		$news .= $result->getResult();
	}
	
	$contents = str_replace('%%NEWS%%', $news, $contents);
}
if (stripos($contents, '%%ACTIVITY%%') !== false) {
	require_once('html/StatsRenderer.php');

	// Get the latest activity items
	$con = Propel::getConnection(UserPeer::DATABASE_NAME);
	$stmt = $con->createStatement();

	$sql = "SELECT * FROM (SELECT user_id, 0 AS messages_count, activity_date,0::int AS posters_count, activity_type,
		project_id, description, section, ref_id, url
		FROM activity_log WHERE section IN ('trackeritem','scmcommit', 'trackersprint')
		UNION
		SELECT 0 as user_id, count(ref_id::text || project_id::text) as messages_count, activity_date::date,
		count( DISTINCT user_id) as posters_count, activity_type, project_id, ''::text as description, section, ref_id,
		''::text as url FROM activity_log WHERE section='forumthread' AND activity_type='update' AND user_id <>  100
		GROUP BY activity_date::date, activity_type,project_id,section,ref_id) tmp
		 WHERE project_id =" . $project->getProjectId() . " ORDER BY activity_date DESC LIMIT 15";
	$rs = $stmt->executeQuery($sql);
	$renderer = new StatsRenderer($project, $rs);
	$table = $renderer->getStatsTable();

	$contents = str_replace('%%ACTIVITY%%', $table, $contents);
}
if (stripos($contents, '%%WALL%%') !== false) {
	ob_start();
	require_once("messagewall/MessageWallPostBrowse.php");
	$wall = ob_get_contents();
	ob_end_clean();
	
	$contents = str_replace('%%WALL%%', $wall, $contents);
}
$bd_index = 0;
$widget_pos = stripos($contents, '%%BURNDOWN%%');
while ($widget_pos !== false) {
    $bd_index++;

    // Get the settings for this indexed burndown chart
    $bd_query = SettingPeer::GetSetting(Project::SECTION, $project->getProjectId(), "burndown-{$bd_index}-query", 'query_id=0');

    $bd_content = burndownWidget($project, $bd_index, $bd_query);

    $contents = preg_replace('/%%BURNDOWN%%/', $bd_content, $contents, 1);
    $widget_pos = stripos($contents, '%%BURNDOWN%%');
}

?>
<table width="100%" border="0">
	<tr>
		<td width="70%" valign="top">
			<?php 
				echo '<div>'.$contents.'</div>'; 
				if ($project->userIsAdmin()) {
					echo '<br />&nbsp;';
					echo HTMLWidget::button("button", Language::getSessText('Project.EditHomepage'),"","","window.location='". GFUrl::getUrl()->projectUrl($project, 'EditHomepage', '', 'admin')."';");
				}
			?>
		</td>
		<td>&nbsp;</td>
		<td nowrap="nowrap" valign="top" width="30%" rowspan="2">
			<?php
			echo $theme->infoTableTop(Language::getSessText('Project.Activity'));
			?>
			<center>
			<img src="<?php echo GFUrl::getURL()->projectURL($project, 'Activity', 'graphs');?>" width="230" height="150" border="0" alt="Activity"/>
			<?php
 			// Show the "Request to join" button only if the user doesn't belong to the project
 			$user = Session::getSession()->getUser();
 			if (GFConf::get('system.allowProjectJoinRequests')) {
 				echo "<br />";
	 			if (!is_null($user) && !$user->isProjectMember($project)) {
	 				echo HTMLWidget::button("button",Language::getSessText('ProjectJoinRequest.AddNew'),"","","window.location='". GFUrl::getUrl()->projectUrl($project, 'ProjectJoinRequestAdd', '')."';");
	 			} else {
	 				echo HTMLWidget::button("button",Language::getSessText('ProjectJoinRequest.AddNew'),"","","", false);
	 			}
	 		}
			echo '</center>';
			echo $theme->infoTableMiddle(Language::getSessText('Project.DescriptionName'));
			echo nl2br(wordwrap($project->getDescription(),40));

			//START Plugin panels
			$params = array('project' => $project);
			$prs = ActionManager::notify('PROJECT_INFO_PANELS', $params);
			foreach ($prs as $pr) {
				$panels = $pr->getResult();

				foreach ($panels as $panel) {
					echo $theme->infoTableMiddle($panel['title']);
					echo $panel['contents'];
				}
			}
			//END Plugin panels

			if(GFConf::get('system.showDevelopersOnProjectSummary')){
				echo $theme->infoTableMiddle(Language::getSessText('Project.DeveloperInfo'));
        echo "<div class='scrollable-info'>";
				$devs = $project->getMembers();
				if (count($devs) < 1) {
					echo 'No Developers in this Project';
				}
				foreach ($devs as $dev) {
					if (GFConf::get('system.displayAvatar')) {
						echo '<span style="padding-right:5px">' . $dev->getAvatarHTML('width:50px;height:40px').'</span>'  .
							'<span style="text-align:left">' . $dev->getUserLink().'</span><br />';
					} else {
						echo $dev->getUserLink() .'<br />';
					}
				}
        echo "</div>";
			}

			if ($project->getRootPath() != '') {
				echo $theme->infoTableMiddle(Language::getSessText('Project.TreeView'));
				$projs=$project->getAllProjectsInTree();
				$projarr=array();
				foreach ($projs as $proj) {
					$projarr[$proj->getParentProjectId()][]=$proj;
				}
				echo render_project_tree($projarr,0,-1);

			}

		if (GFConf::get('system.troveEnabled')) {
			echo $theme->infoTableMiddle(Language::getSessText('ProjectAdmin.TroveCategorization'));
			
			//
			// Trove categorization
			//
			$trove = TroveLinkPeer::getSelectedCategoriesTree(Project::SECTION, $project->getPrimaryKey());
			if (count($trove) > 0) {
				echo '<ul>';
				foreach ($trove as $trove_root => $data) {
					if (count($data['children']) > 0) {
						echo '<li>'.$data['name'].': ';
						$children_names = array();
						foreach ($data['children'] as $children) {
							$children_names[] = $children->getCategoryName();
						}

						echo implode(', <br /> ', $children_names);
						echo '</li>'."\n";
					}
				}
				echo '</ul>';
			} else {
				echo Language::getSessText('Project.NotTroveCat');
			}
			if ($project->userIsAdmin()) {
				echo '<span style="text-align:center; display:block; padding:5px 0">';
				echo '<a href="' . GFUrl::getUrl()->projectURL($project, 'TroveEdit', '', 'admin') . '">';
				echo Language::getSessText('Project.TroveCategorizationEdit');
				echo '</a>';
				echo '</span>';
			}
		}	
			echo $template->infoTableBottom();

			?>
		</td>
	</tr>
</table>
<?php
$template->footer();
?>
