<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('core/ProjectPeer.php');
require_once('session/Session.php');
require_once('core/Plugin.php');
require_once('plugin/GFPlugin.php');
require_once('core/RolePeer.php');
require_once('core/UserProject.php');
require_once('html/HTMLWidget.php');
require_once('html/StatsRenderer.php');

$session = Session::getSession();
$project = GFUrl::getUrl()->getProject();
$project_id = $project->getProjectId();
$Language = $session->getLanguage();

if (!isset($feedback)) {
	$feedback = '';
}

require_once('html/templates/ProjectReportingTemplate.php');
$template = new ProjectReportingTemplate();
$params = array(
		'project' => $project,
		'feedback' => $feedback,
		'title' => Language::getSessText('ProjectActivity.title')
	);
$template->header($params);
$begin=((isset($_REQUEST['start_date']) && strlen($_REQUEST['start_date']) > 5) ? Validator::sanitizeDate($_REQUEST['start_date']) : 0 );
$end=((isset($_REQUEST['end_date']) && strlen($_REQUEST['end_date']) > 5) ? Validator::sanitizeDate($_REQUEST['end_date']) : 0 );
$member_id = isset($_REQUEST['members']) && $_REQUEST['members'] != 0 ? Validator::sanitizeInt($_REQUEST['members']) : User::UNDEFINED_USER_ID;

if ($begin=="0") {
	$begin = (time()-(30*86400));
} else {
	$begin = strtotime($begin);
}
if ($end=="0") {
	$end = time();
} else {
	$end=strtotime($end)+86399;
}
if ($begin > $end) {
	$endtmp=$end;
	$end=$begin;
	$begin=$endtmp;
}
$ids = array();
$ids[] = 'scmcommit';
$ids[] = 'trackeritem';
$ids[] = 'forumthread';
$ids[] = 'wiki';
$ids[] = 'docman';
$ids[] = 'frspackage';
$ids[] = 'frsrelease';

$texts=array();
$texts['scmcommit']=$Language->getText('ProjectActivity.Commit');
$texts['trackeritem']=$Language->getText('ProjectActivity.TrackerItem');
$texts['forumthread']=$Language->getText('Discuss.WindowTitle');
$texts['wiki']=$Language->getText('ProjectActivity.Wiki');
$texts['docman']=$Language->getText('ProjectActivity.Docman');
$texts['frspackage']=$Language->getText('ProjectActivity.FrsPackage');
$texts['frsrelease']=$Language->getText('ProjectActivity.FrsRelease');

if (!isset($_REQUEST['show'])) {
	$show=$ids;
} else {
	$show=$_REQUEST['show'];
}

if(!is_array($show)) Throw new Exception('Invalid Data Passed to query');

foreach ($show as $showthis) {
	if (array_search($showthis,$ids) === false) {
		Throw new Exception('Invalid Data Passed to query');
	}
}

$multiselect=HTMLWidget::multiSelectFromAssoc('show[]',$texts,$show,'',false,null,false,null,5);

//Copy show values (except forumthread) to a temp array to be used in sql queries
$sections = array();
$discussion = false;
foreach($show as $s) {
	if ($s == 'forumthread') {
		$discussion = true;
	} else {
		$sections[] = $s;
	}
}


$con = Propel::getConnection(UserPeer::DATABASE_NAME);
$stmt = $con->createStatement();

$restrict_user = '';
if ($member_id != User::UNDEFINED_USER_ID) {
	$restrict_user .= ' AND user_id = ' . $member_id;
}

$sql="SELECT COUNT(*) FROM ( SELECT user_id, 0 AS messages_count, activity_date,
		0::int AS posters_count, activity_type, project_id, description, section, ref_id, url
		FROM activity_log WHERE activity_date
		BETWEEN '".date('Y-m-d',$begin)." 00:00:00' AND '".date('Y-m-d',$end)." 23:59:59'
		AND section IN ('".implode("','",$sections)."') AND project_id={$project->getProjectId()}
		$restrict_user
		";

if ($discussion) {
	$sql .= "
		UNION
		SELECT 0 as user_id, count(ref_id::text || project_id::text) as messages_count, activity_date::date,
		count( DISTINCT user_id) as posters_count,activity_type, project_id, ''::text as description, section, ref_id,
		''::text as url FROM activity_log WHERE section='forumthread' AND activity_date BETWEEN '".date('Y-m-d',$begin).
		" 00:00:00'
		AND '".date('Y-m-d',$end)." 23:59:59'  AND project_id={$project->getProjectId()} AND user_id <> 100
		AND activity_type='update'
		$restrict_user
		GROUP BY activity_date::date, activity_type,project_id,section,ref_id";
}

$sql .= ") tmp";

$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM );
$rs->next();
$total = $rs->getInt(1);
$paginator = new Paginator('prjactivity', -1, $total, 'activity_date', array('activity_date'=>''));

$sql="SELECT user_id, 0 AS messages_count, activity_date,
		0::int AS posters_count, activity_type, project_id, description, section, ref_id, url
		FROM activity_log WHERE activity_date
		BETWEEN '".date('Y-m-d',$begin)." 00:00:00' AND '".date('Y-m-d',$end)." 23:59:59'
		AND section IN ('".implode("','",$sections)."') AND project_id={$project->getProjectId()}
		$restrict_user
		";

if ($discussion) {
	$sql .= "
		UNION
		SELECT 0 as user_id, count(ref_id::text || project_id::text) as messages_count, activity_date::date,
		count( DISTINCT user_id) as posters_count,activity_type, project_id, ''::text as description, section, ref_id,
		''::text as url FROM activity_log WHERE section='forumthread' AND activity_date BETWEEN '".date('Y-m-d',$begin).
		" 00:00:00'
		AND '".date('Y-m-d',$end)." 23:59:59'  AND project_id={$project->getProjectId()} AND user_id <> 100
		AND activity_type='update'
		$restrict_user
		GROUP BY activity_date::date, activity_type,project_id,section,ref_id";
}

$sql .= ' ' . $paginator->getSQL();

$rs = $stmt->executeQuery($sql);

$members = $project->getMembers(true);
$arr_members = array();
foreach($members as $member) {
	$arr_members[$member->getUserId()] = $member->getFullname();
}
?>
<script language="javascript" type="text/javascript" src="/datetimepicker.js">
</script>
<br />
<table border="0" cellspacing="0" cellpadding="3" bgcolor="#E0E0E0">
<form action="<?php echo GFUrl::getUrl()->currentURL() ?>" method="get">
<input type="hidden" name="action" value="Activity">
<tr>
	<td><strong><?php echo $Language->getText('ProjectActivity.HeaderActivity') ?></strong></td>
	<td><strong><?php echo $Language->getText('ProjectActivity.StartDate') ?></strong></td>
	<td><strong><?php echo $Language->getText('ProjectActivity.EndDate') ?></strong></td>
	<td><strong><?php echo $Language->getText('ProjectActivity.Member') ?></strong></td>
</tr>
<tr>
	<td><?php echo $multiselect; ?></td>
	<td valign="top"><input id="startdate" name="start_date" value="<?php echo date('M d Y',$begin); ?>" size="11" maxlength="11" /><a href="javascript:NewCal('startdate','MMMddyyyy')"><img src="/themes/gforge5/images/cal.gif" width="16" height="16" border="0" alt="Pick a date"/></a></td>
	<td valign="top"><input id="enddate" name="end_date" value="<?php echo date('M d Y',$end); ?>" size="11" maxlength="11" /><a href="javascript:NewCal('enddate','MMMddyyyy')"><img src="/themes/gforge5/images/cal.gif" width="16" height="16" border="0" alt="Pick a date"/></a></td>
	<td valign="top"><?php echo HTMLWidget::selectBoxFromAssoc($arr_members, 'members', $member_id, false, false, null, true,  null); ?></td>
	<td valign="top"><?php echo HTMLWidget::button("submit", $Language->getText('Common.Submit'), "submit"); ?></td>
</tr>
</form>
</table>
<br />
<?php
	
	$renderer = new StatsRenderer($project, $rs, $paginator);
	echo $renderer->getStatsTable();
	echo '<div class="paginator" align="center">'.$paginator->getPaginator().'</div><br />';
	$template->footer();
?>
