<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version $Id$
 * @author Daniel A. P�rez	daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */


require_once('wwwlib/search/commonsearch.php');
require_once('html/SiteTemplate.php');

$template = new SiteTemplate();
$lang = Session::getSession()->getLanguage();
$search_params = array();

if (isset($_REQUEST['do_search'])) {
	$search=(isset($_REQUEST['search']) ? $_REQUEST['search'] : array());
	$words =(isset($_REQUEST['words']) ? StringUtils::getPlainUTF8FromHtml($_REQUEST['words']) : '');
	$scope = (isset($_REQUEST['search_scope']) ? $_REQUEST['search_scope'] : 'all');
	$search_params['search_scope'] = $scope;
	$search_params['template'] = $template;
	$search_params['search']= $search;
	$search_params['words'] = $words;
	$search_params['search_by'] = isset($_REQUEST['search_by']) && $_REQUEST['search_by'] == 'tags' ? 'tags' : 'keywords';
	if (isset($_REQUEST['startdate']) && isset($_REQUEST['enddate'])) {
		$search_params['startdate'] = Validator::sanitizeDate($_REQUEST['startdate']);
		$search_params['enddate'] = Validator::sanitizeDate($_REQUEST['enddate']);
	} else {
		if (isset($_REQUEST['startdate']) || isset($_REQUEST['enddate'])) {
			$feedback = 'Date Range Must Include Both Dates';
		}
		$search_params['startdate'] = null;
		$search_params['enddate'] = null;
	}
	$search_params['search_attach'] = isset($_REQUEST['search_attach']) ? Validator::sanitizeBool($_REQUEST['search_attach']) : false;

	if(Session::getSession()->isLoggedIn()) {
		$user = Session::getSession()->getUser();
		UserPreferencePeer::setUserPref($user->getUserId(),'site_search', setSearchParams($search_params));
	} else {
		$_SESSION['site_search'] = setSearchParams($search_params);
	}
} else {
	$words = '';
	$search = array();
}

$params = array(
	'selectedtoptab'=>'search',
	'title'=>'Search',
	'feedback'=>isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : '')
);	
$template->header($params);

include('SiteSearchForm.php');

echo '<br /><br />';

try {
	if (array_key_exists('do_search', $_REQUEST)) {
		if (strlen($_REQUEST['words']) > 0) {
			$searchByTags = (isset($_REQUEST['search_by']) && $_REQUEST['search_by'] == 'tags');
			if ($scope == 'my') {
				$myProjectsOnly = true;
				$user = Session::getSession()->getUser();
				if ($user) {
					$search_params['project_ids'] = $user->getAllProjectIds();
					if (!count($params['project_ids'])) {
						//We set an non-existent project_id so plugins returns the proper
						//(empty) result set when the user required to search in his projects
						//but he is not member of any
						$search_params['project_ids'][]='-1';
					}
				}
			} else {
				$search_params['project_ids'] = null;
				$myProjectsOnly = false;

			}
			if ($searchByTags) {
				if (!count($search_params['search'])) {
					echo "<p><h3>".Language::getSessText('Tag.EmptySearch')."</h3></p>";
				} else {
					echo searchTags($search_params);
				}

			} else {
				$searchAction  = 'DO_SEARCH';
				echo '<h3>'.$language->getSessText('    Search.SearchResultsTitle').'</h3>';
				$results = ActionManager::notify($searchAction, $search_params);

				$params['template'] = $template;
				$params['words']=$_GET['words'];

				$search = new GFSearch($search_params);


				$projectSearchContent = searchProjects($search, false, $myProjectsOnly);


				echo '<div class="tabber">';
				echo '<div class="tabbertab" title="'.'Projects'.'">';
				echo $projectSearchContent;
				echo '</div>';
				foreach ($results as $result) {
					$result = $result->getResult();
					if (!is_array($result)) {
						continue;
					}

					$title = $result['title'];
					$content = $result['content'];

					echo '<div class="tabbertab" title="'.$title.'">';
					echo $content;
					echo '</div>';
				}
				echo '</div>';
			}

		} else {
			echo "<p><h3>Empty Search</h3></p>";
		}
	}
} catch (SQLException $e) {
	//echo $e->getMessage();
	// SQL exception here means that there was a syntax error on the search terms
	echo '<span class="feedback">'.$lang->getText('Search.InvalidSearch').'</span>';
}

$template->footer();


?>
